# This file was automatically generated by SWIG (http://www.swig.org).
# Version 4.0.1
#
# Do not make changes to this file unless you know what you are doing--modify
# the SWIG interface file instead.

from sys import version_info as _swig_python_version_info
if _swig_python_version_info < (2, 7, 0):
    raise RuntimeError("Python 2.7 or later required")

# Import the low-level C/C++ module
if __package__ or "." in __name__:
    from . import _thosttraderapi
else:
    import _thosttraderapi

try:
    import builtins as __builtin__
except ImportError:
    print('ImportError')
    # import __builtin__

def _swig_repr(self):
    try:
        strthis = "proxy of " + self.this.__repr__()
    except __builtin__.Exception:
        strthis = ""
    return "<%s.%s; %s >" % (self.__class__.__module__, self.__class__.__name__, strthis,)


def _swig_setattr_nondynamic_instance_variable(set):
    def set_instance_attr(self, name, value):
        if name == "thisown":
            self.this.own(value)
        elif name == "this":
            set(self, name, value)
        elif hasattr(self, name) and isinstance(getattr(type(self), name), property):
            set(self, name, value)
        else:
            raise AttributeError("You cannot add instance attributes to %s" % self)
    return set_instance_attr


def _swig_setattr_nondynamic_class_variable(set):
    def set_class_attr(cls, name, value):
        if hasattr(cls, name) and not isinstance(getattr(cls, name), property):
            set(cls, name, value)
        else:
            raise AttributeError("You cannot add class attributes to %s" % cls)
    return set_class_attr


def _swig_add_metaclass(metaclass):
    """Class decorator for adding a metaclass to a SWIG wrapped class - a slimmed down version of six.add_metaclass"""
    def wrapper(cls):
        return metaclass(cls.__name__, cls.__bases__, cls.__dict__.copy())
    return wrapper


class _SwigNonDynamicMeta(type):
    """Meta class to enforce nondynamic attributes (no new attributes) for a class"""
    __setattr__ = _swig_setattr_nondynamic_class_variable(type.__setattr__)


import weakref

THOST_TERT_RESTART = _thosttraderapi.THOST_TERT_RESTART
THOST_TERT_RESUME = _thosttraderapi.THOST_TERT_RESUME
THOST_TERT_QUICK = _thosttraderapi.THOST_TERT_QUICK
THOST_FTDC_EXP_Normal = _thosttraderapi.THOST_FTDC_EXP_Normal
THOST_FTDC_EXP_GenOrderByTrade = _thosttraderapi.THOST_FTDC_EXP_GenOrderByTrade
THOST_FTDC_ICT_EID = _thosttraderapi.THOST_FTDC_ICT_EID
THOST_FTDC_ICT_IDCard = _thosttraderapi.THOST_FTDC_ICT_IDCard
THOST_FTDC_ICT_OfficerIDCard = _thosttraderapi.THOST_FTDC_ICT_OfficerIDCard
THOST_FTDC_ICT_PoliceIDCard = _thosttraderapi.THOST_FTDC_ICT_PoliceIDCard
THOST_FTDC_ICT_SoldierIDCard = _thosttraderapi.THOST_FTDC_ICT_SoldierIDCard
THOST_FTDC_ICT_HouseholdRegister = _thosttraderapi.THOST_FTDC_ICT_HouseholdRegister
THOST_FTDC_ICT_Passport = _thosttraderapi.THOST_FTDC_ICT_Passport
THOST_FTDC_ICT_TaiwanCompatriotIDCard = _thosttraderapi.THOST_FTDC_ICT_TaiwanCompatriotIDCard
THOST_FTDC_ICT_HomeComingCard = _thosttraderapi.THOST_FTDC_ICT_HomeComingCard
THOST_FTDC_ICT_LicenseNo = _thosttraderapi.THOST_FTDC_ICT_LicenseNo
THOST_FTDC_ICT_TaxNo = _thosttraderapi.THOST_FTDC_ICT_TaxNo
THOST_FTDC_ICT_HMMainlandTravelPermit = _thosttraderapi.THOST_FTDC_ICT_HMMainlandTravelPermit
THOST_FTDC_ICT_TwMainlandTravelPermit = _thosttraderapi.THOST_FTDC_ICT_TwMainlandTravelPermit
THOST_FTDC_ICT_DrivingLicense = _thosttraderapi.THOST_FTDC_ICT_DrivingLicense
THOST_FTDC_ICT_SocialID = _thosttraderapi.THOST_FTDC_ICT_SocialID
THOST_FTDC_ICT_LocalID = _thosttraderapi.THOST_FTDC_ICT_LocalID
THOST_FTDC_ICT_BusinessRegistration = _thosttraderapi.THOST_FTDC_ICT_BusinessRegistration
THOST_FTDC_ICT_HKMCIDCard = _thosttraderapi.THOST_FTDC_ICT_HKMCIDCard
THOST_FTDC_ICT_AccountsPermits = _thosttraderapi.THOST_FTDC_ICT_AccountsPermits
THOST_FTDC_ICT_FrgPrmtRdCard = _thosttraderapi.THOST_FTDC_ICT_FrgPrmtRdCard
THOST_FTDC_ICT_CptMngPrdLetter = _thosttraderapi.THOST_FTDC_ICT_CptMngPrdLetter
THOST_FTDC_ICT_OtherCard = _thosttraderapi.THOST_FTDC_ICT_OtherCard
THOST_FTDC_IR_All = _thosttraderapi.THOST_FTDC_IR_All
THOST_FTDC_IR_Group = _thosttraderapi.THOST_FTDC_IR_Group
THOST_FTDC_IR_Single = _thosttraderapi.THOST_FTDC_IR_Single
THOST_FTDC_DR_All = _thosttraderapi.THOST_FTDC_DR_All
THOST_FTDC_DR_Group = _thosttraderapi.THOST_FTDC_DR_Group
THOST_FTDC_DR_Single = _thosttraderapi.THOST_FTDC_DR_Single
THOST_FTDC_DS_Asynchronous = _thosttraderapi.THOST_FTDC_DS_Asynchronous
THOST_FTDC_DS_Synchronizing = _thosttraderapi.THOST_FTDC_DS_Synchronizing
THOST_FTDC_DS_Synchronized = _thosttraderapi.THOST_FTDC_DS_Synchronized
THOST_FTDC_BDS_Synchronized = _thosttraderapi.THOST_FTDC_BDS_Synchronized
THOST_FTDC_BDS_Synchronizing = _thosttraderapi.THOST_FTDC_BDS_Synchronizing
THOST_FTDC_ECS_NoConnection = _thosttraderapi.THOST_FTDC_ECS_NoConnection
THOST_FTDC_ECS_QryInstrumentSent = _thosttraderapi.THOST_FTDC_ECS_QryInstrumentSent
THOST_FTDC_ECS_GotInformation = _thosttraderapi.THOST_FTDC_ECS_GotInformation
THOST_FTDC_TCS_NotConnected = _thosttraderapi.THOST_FTDC_TCS_NotConnected
THOST_FTDC_TCS_Connected = _thosttraderapi.THOST_FTDC_TCS_Connected
THOST_FTDC_TCS_QryInstrumentSent = _thosttraderapi.THOST_FTDC_TCS_QryInstrumentSent
THOST_FTDC_TCS_SubPrivateFlow = _thosttraderapi.THOST_FTDC_TCS_SubPrivateFlow
THOST_FTDC_FC_DataAsync = _thosttraderapi.THOST_FTDC_FC_DataAsync
THOST_FTDC_FC_ForceUserLogout = _thosttraderapi.THOST_FTDC_FC_ForceUserLogout
THOST_FTDC_FC_UserPasswordUpdate = _thosttraderapi.THOST_FTDC_FC_UserPasswordUpdate
THOST_FTDC_FC_BrokerPasswordUpdate = _thosttraderapi.THOST_FTDC_FC_BrokerPasswordUpdate
THOST_FTDC_FC_InvestorPasswordUpdate = _thosttraderapi.THOST_FTDC_FC_InvestorPasswordUpdate
THOST_FTDC_FC_OrderInsert = _thosttraderapi.THOST_FTDC_FC_OrderInsert
THOST_FTDC_FC_OrderAction = _thosttraderapi.THOST_FTDC_FC_OrderAction
THOST_FTDC_FC_SyncSystemData = _thosttraderapi.THOST_FTDC_FC_SyncSystemData
THOST_FTDC_FC_SyncBrokerData = _thosttraderapi.THOST_FTDC_FC_SyncBrokerData
THOST_FTDC_FC_BachSyncBrokerData = _thosttraderapi.THOST_FTDC_FC_BachSyncBrokerData
THOST_FTDC_FC_SuperQuery = _thosttraderapi.THOST_FTDC_FC_SuperQuery
THOST_FTDC_FC_ParkedOrderInsert = _thosttraderapi.THOST_FTDC_FC_ParkedOrderInsert
THOST_FTDC_FC_ParkedOrderAction = _thosttraderapi.THOST_FTDC_FC_ParkedOrderAction
THOST_FTDC_FC_SyncOTP = _thosttraderapi.THOST_FTDC_FC_SyncOTP
THOST_FTDC_FC_DeleteOrder = _thosttraderapi.THOST_FTDC_FC_DeleteOrder
THOST_FTDC_BFC_ForceUserLogout = _thosttraderapi.THOST_FTDC_BFC_ForceUserLogout
THOST_FTDC_BFC_UserPasswordUpdate = _thosttraderapi.THOST_FTDC_BFC_UserPasswordUpdate
THOST_FTDC_BFC_SyncBrokerData = _thosttraderapi.THOST_FTDC_BFC_SyncBrokerData
THOST_FTDC_BFC_BachSyncBrokerData = _thosttraderapi.THOST_FTDC_BFC_BachSyncBrokerData
THOST_FTDC_BFC_OrderInsert = _thosttraderapi.THOST_FTDC_BFC_OrderInsert
THOST_FTDC_BFC_OrderAction = _thosttraderapi.THOST_FTDC_BFC_OrderAction
THOST_FTDC_BFC_AllQuery = _thosttraderapi.THOST_FTDC_BFC_AllQuery
THOST_FTDC_BFC_log = _thosttraderapi.THOST_FTDC_BFC_log
THOST_FTDC_BFC_BaseQry = _thosttraderapi.THOST_FTDC_BFC_BaseQry
THOST_FTDC_BFC_TradeQry = _thosttraderapi.THOST_FTDC_BFC_TradeQry
THOST_FTDC_BFC_Trade = _thosttraderapi.THOST_FTDC_BFC_Trade
THOST_FTDC_BFC_Virement = _thosttraderapi.THOST_FTDC_BFC_Virement
THOST_FTDC_BFC_Risk = _thosttraderapi.THOST_FTDC_BFC_Risk
THOST_FTDC_BFC_Session = _thosttraderapi.THOST_FTDC_BFC_Session
THOST_FTDC_BFC_RiskNoticeCtl = _thosttraderapi.THOST_FTDC_BFC_RiskNoticeCtl
THOST_FTDC_BFC_RiskNotice = _thosttraderapi.THOST_FTDC_BFC_RiskNotice
THOST_FTDC_BFC_BrokerDeposit = _thosttraderapi.THOST_FTDC_BFC_BrokerDeposit
THOST_FTDC_BFC_QueryFund = _thosttraderapi.THOST_FTDC_BFC_QueryFund
THOST_FTDC_BFC_QueryOrder = _thosttraderapi.THOST_FTDC_BFC_QueryOrder
THOST_FTDC_BFC_QueryTrade = _thosttraderapi.THOST_FTDC_BFC_QueryTrade
THOST_FTDC_BFC_QueryPosition = _thosttraderapi.THOST_FTDC_BFC_QueryPosition
THOST_FTDC_BFC_QueryMarketData = _thosttraderapi.THOST_FTDC_BFC_QueryMarketData
THOST_FTDC_BFC_QueryUserEvent = _thosttraderapi.THOST_FTDC_BFC_QueryUserEvent
THOST_FTDC_BFC_QueryRiskNotify = _thosttraderapi.THOST_FTDC_BFC_QueryRiskNotify
THOST_FTDC_BFC_QueryFundChange = _thosttraderapi.THOST_FTDC_BFC_QueryFundChange
THOST_FTDC_BFC_QueryInvestor = _thosttraderapi.THOST_FTDC_BFC_QueryInvestor
THOST_FTDC_BFC_QueryTradingCode = _thosttraderapi.THOST_FTDC_BFC_QueryTradingCode
THOST_FTDC_BFC_ForceClose = _thosttraderapi.THOST_FTDC_BFC_ForceClose
THOST_FTDC_BFC_PressTest = _thosttraderapi.THOST_FTDC_BFC_PressTest
THOST_FTDC_BFC_RemainCalc = _thosttraderapi.THOST_FTDC_BFC_RemainCalc
THOST_FTDC_BFC_NetPositionInd = _thosttraderapi.THOST_FTDC_BFC_NetPositionInd
THOST_FTDC_BFC_RiskPredict = _thosttraderapi.THOST_FTDC_BFC_RiskPredict
THOST_FTDC_BFC_DataExport = _thosttraderapi.THOST_FTDC_BFC_DataExport
THOST_FTDC_BFC_RiskTargetSetup = _thosttraderapi.THOST_FTDC_BFC_RiskTargetSetup
THOST_FTDC_BFC_MarketDataWarn = _thosttraderapi.THOST_FTDC_BFC_MarketDataWarn
THOST_FTDC_BFC_QryBizNotice = _thosttraderapi.THOST_FTDC_BFC_QryBizNotice
THOST_FTDC_BFC_CfgBizNotice = _thosttraderapi.THOST_FTDC_BFC_CfgBizNotice
THOST_FTDC_BFC_SyncOTP = _thosttraderapi.THOST_FTDC_BFC_SyncOTP
THOST_FTDC_BFC_SendBizNotice = _thosttraderapi.THOST_FTDC_BFC_SendBizNotice
THOST_FTDC_BFC_CfgRiskLevelStd = _thosttraderapi.THOST_FTDC_BFC_CfgRiskLevelStd
THOST_FTDC_BFC_TbCommand = _thosttraderapi.THOST_FTDC_BFC_TbCommand
THOST_FTDC_BFC_DeleteOrder = _thosttraderapi.THOST_FTDC_BFC_DeleteOrder
THOST_FTDC_BFC_ParkedOrderInsert = _thosttraderapi.THOST_FTDC_BFC_ParkedOrderInsert
THOST_FTDC_BFC_ParkedOrderAction = _thosttraderapi.THOST_FTDC_BFC_ParkedOrderAction
THOST_FTDC_BFC_ExecOrderNoCheck = _thosttraderapi.THOST_FTDC_BFC_ExecOrderNoCheck
THOST_FTDC_BFC_Designate = _thosttraderapi.THOST_FTDC_BFC_Designate
THOST_FTDC_BFC_StockDisposal = _thosttraderapi.THOST_FTDC_BFC_StockDisposal
THOST_FTDC_BFC_BrokerDepositWarn = _thosttraderapi.THOST_FTDC_BFC_BrokerDepositWarn
THOST_FTDC_BFC_CoverWarn = _thosttraderapi.THOST_FTDC_BFC_CoverWarn
THOST_FTDC_BFC_PreExecOrder = _thosttraderapi.THOST_FTDC_BFC_PreExecOrder
THOST_FTDC_BFC_ExecOrderRisk = _thosttraderapi.THOST_FTDC_BFC_ExecOrderRisk
THOST_FTDC_BFC_PosiLimitWarn = _thosttraderapi.THOST_FTDC_BFC_PosiLimitWarn
THOST_FTDC_BFC_QryPosiLimit = _thosttraderapi.THOST_FTDC_BFC_QryPosiLimit
THOST_FTDC_BFC_FBSign = _thosttraderapi.THOST_FTDC_BFC_FBSign
THOST_FTDC_BFC_FBAccount = _thosttraderapi.THOST_FTDC_BFC_FBAccount
THOST_FTDC_OAS_Submitted = _thosttraderapi.THOST_FTDC_OAS_Submitted
THOST_FTDC_OAS_Accepted = _thosttraderapi.THOST_FTDC_OAS_Accepted
THOST_FTDC_OAS_Rejected = _thosttraderapi.THOST_FTDC_OAS_Rejected
THOST_FTDC_OST_AllTraded = _thosttraderapi.THOST_FTDC_OST_AllTraded
THOST_FTDC_OST_PartTradedQueueing = _thosttraderapi.THOST_FTDC_OST_PartTradedQueueing
THOST_FTDC_OST_PartTradedNotQueueing = _thosttraderapi.THOST_FTDC_OST_PartTradedNotQueueing
THOST_FTDC_OST_NoTradeQueueing = _thosttraderapi.THOST_FTDC_OST_NoTradeQueueing
THOST_FTDC_OST_NoTradeNotQueueing = _thosttraderapi.THOST_FTDC_OST_NoTradeNotQueueing
THOST_FTDC_OST_Canceled = _thosttraderapi.THOST_FTDC_OST_Canceled
THOST_FTDC_OST_Unknown = _thosttraderapi.THOST_FTDC_OST_Unknown
THOST_FTDC_OST_NotTouched = _thosttraderapi.THOST_FTDC_OST_NotTouched
THOST_FTDC_OST_Touched = _thosttraderapi.THOST_FTDC_OST_Touched
THOST_FTDC_OSS_InsertSubmitted = _thosttraderapi.THOST_FTDC_OSS_InsertSubmitted
THOST_FTDC_OSS_CancelSubmitted = _thosttraderapi.THOST_FTDC_OSS_CancelSubmitted
THOST_FTDC_OSS_ModifySubmitted = _thosttraderapi.THOST_FTDC_OSS_ModifySubmitted
THOST_FTDC_OSS_Accepted = _thosttraderapi.THOST_FTDC_OSS_Accepted
THOST_FTDC_OSS_InsertRejected = _thosttraderapi.THOST_FTDC_OSS_InsertRejected
THOST_FTDC_OSS_CancelRejected = _thosttraderapi.THOST_FTDC_OSS_CancelRejected
THOST_FTDC_OSS_ModifyRejected = _thosttraderapi.THOST_FTDC_OSS_ModifyRejected
THOST_FTDC_PSD_Today = _thosttraderapi.THOST_FTDC_PSD_Today
THOST_FTDC_PSD_History = _thosttraderapi.THOST_FTDC_PSD_History
THOST_FTDC_PDT_UseHistory = _thosttraderapi.THOST_FTDC_PDT_UseHistory
THOST_FTDC_PDT_NoUseHistory = _thosttraderapi.THOST_FTDC_PDT_NoUseHistory
THOST_FTDC_ER_Broker = _thosttraderapi.THOST_FTDC_ER_Broker
THOST_FTDC_ER_Host = _thosttraderapi.THOST_FTDC_ER_Host
THOST_FTDC_ER_Maker = _thosttraderapi.THOST_FTDC_ER_Maker
THOST_FTDC_PC_Futures = _thosttraderapi.THOST_FTDC_PC_Futures
THOST_FTDC_PC_Options = _thosttraderapi.THOST_FTDC_PC_Options
THOST_FTDC_PC_Combination = _thosttraderapi.THOST_FTDC_PC_Combination
THOST_FTDC_PC_Spot = _thosttraderapi.THOST_FTDC_PC_Spot
THOST_FTDC_PC_EFP = _thosttraderapi.THOST_FTDC_PC_EFP
THOST_FTDC_PC_SpotOption = _thosttraderapi.THOST_FTDC_PC_SpotOption
THOST_FTDC_IP_NotStart = _thosttraderapi.THOST_FTDC_IP_NotStart
THOST_FTDC_IP_Started = _thosttraderapi.THOST_FTDC_IP_Started
THOST_FTDC_IP_Pause = _thosttraderapi.THOST_FTDC_IP_Pause
THOST_FTDC_IP_Expired = _thosttraderapi.THOST_FTDC_IP_Expired
THOST_FTDC_D_Buy = _thosttraderapi.THOST_FTDC_D_Buy
THOST_FTDC_D_Sell = _thosttraderapi.THOST_FTDC_D_Sell
THOST_FTDC_PT_Net = _thosttraderapi.THOST_FTDC_PT_Net
THOST_FTDC_PT_Gross = _thosttraderapi.THOST_FTDC_PT_Gross
THOST_FTDC_PD_Net = _thosttraderapi.THOST_FTDC_PD_Net
THOST_FTDC_PD_Long = _thosttraderapi.THOST_FTDC_PD_Long
THOST_FTDC_PD_Short = _thosttraderapi.THOST_FTDC_PD_Short
THOST_FTDC_SS_NonActive = _thosttraderapi.THOST_FTDC_SS_NonActive
THOST_FTDC_SS_Startup = _thosttraderapi.THOST_FTDC_SS_Startup
THOST_FTDC_SS_Operating = _thosttraderapi.THOST_FTDC_SS_Operating
THOST_FTDC_SS_Settlement = _thosttraderapi.THOST_FTDC_SS_Settlement
THOST_FTDC_SS_SettlementFinished = _thosttraderapi.THOST_FTDC_SS_SettlementFinished
THOST_FTDC_RA_Trade = _thosttraderapi.THOST_FTDC_RA_Trade
THOST_FTDC_RA_Settlement = _thosttraderapi.THOST_FTDC_RA_Settlement
THOST_FTDC_HF_Speculation = _thosttraderapi.THOST_FTDC_HF_Speculation
THOST_FTDC_HF_Arbitrage = _thosttraderapi.THOST_FTDC_HF_Arbitrage
THOST_FTDC_HF_Hedge = _thosttraderapi.THOST_FTDC_HF_Hedge
THOST_FTDC_HF_MarketMaker = _thosttraderapi.THOST_FTDC_HF_MarketMaker
THOST_FTDC_HF_SpecHedge = _thosttraderapi.THOST_FTDC_HF_SpecHedge
THOST_FTDC_HF_HedgeSpec = _thosttraderapi.THOST_FTDC_HF_HedgeSpec
THOST_FTDC_BHF_Speculation = _thosttraderapi.THOST_FTDC_BHF_Speculation
THOST_FTDC_BHF_Arbitrage = _thosttraderapi.THOST_FTDC_BHF_Arbitrage
THOST_FTDC_BHF_Hedge = _thosttraderapi.THOST_FTDC_BHF_Hedge
THOST_FTDC_CIDT_Speculation = _thosttraderapi.THOST_FTDC_CIDT_Speculation
THOST_FTDC_CIDT_Arbitrage = _thosttraderapi.THOST_FTDC_CIDT_Arbitrage
THOST_FTDC_CIDT_Hedge = _thosttraderapi.THOST_FTDC_CIDT_Hedge
THOST_FTDC_CIDT_MarketMaker = _thosttraderapi.THOST_FTDC_CIDT_MarketMaker
THOST_FTDC_OPT_AnyPrice = _thosttraderapi.THOST_FTDC_OPT_AnyPrice
THOST_FTDC_OPT_LimitPrice = _thosttraderapi.THOST_FTDC_OPT_LimitPrice
THOST_FTDC_OPT_BestPrice = _thosttraderapi.THOST_FTDC_OPT_BestPrice
THOST_FTDC_OPT_LastPrice = _thosttraderapi.THOST_FTDC_OPT_LastPrice
THOST_FTDC_OPT_LastPricePlusOneTicks = _thosttraderapi.THOST_FTDC_OPT_LastPricePlusOneTicks
THOST_FTDC_OPT_LastPricePlusTwoTicks = _thosttraderapi.THOST_FTDC_OPT_LastPricePlusTwoTicks
THOST_FTDC_OPT_LastPricePlusThreeTicks = _thosttraderapi.THOST_FTDC_OPT_LastPricePlusThreeTicks
THOST_FTDC_OPT_AskPrice1 = _thosttraderapi.THOST_FTDC_OPT_AskPrice1
THOST_FTDC_OPT_AskPrice1PlusOneTicks = _thosttraderapi.THOST_FTDC_OPT_AskPrice1PlusOneTicks
THOST_FTDC_OPT_AskPrice1PlusTwoTicks = _thosttraderapi.THOST_FTDC_OPT_AskPrice1PlusTwoTicks
THOST_FTDC_OPT_AskPrice1PlusThreeTicks = _thosttraderapi.THOST_FTDC_OPT_AskPrice1PlusThreeTicks
THOST_FTDC_OPT_BidPrice1 = _thosttraderapi.THOST_FTDC_OPT_BidPrice1
THOST_FTDC_OPT_BidPrice1PlusOneTicks = _thosttraderapi.THOST_FTDC_OPT_BidPrice1PlusOneTicks
THOST_FTDC_OPT_BidPrice1PlusTwoTicks = _thosttraderapi.THOST_FTDC_OPT_BidPrice1PlusTwoTicks
THOST_FTDC_OPT_BidPrice1PlusThreeTicks = _thosttraderapi.THOST_FTDC_OPT_BidPrice1PlusThreeTicks
THOST_FTDC_OPT_FiveLevelPrice = _thosttraderapi.THOST_FTDC_OPT_FiveLevelPrice
THOST_FTDC_OF_Open = _thosttraderapi.THOST_FTDC_OF_Open
THOST_FTDC_OF_Close = _thosttraderapi.THOST_FTDC_OF_Close
THOST_FTDC_OF_ForceClose = _thosttraderapi.THOST_FTDC_OF_ForceClose
THOST_FTDC_OF_CloseToday = _thosttraderapi.THOST_FTDC_OF_CloseToday
THOST_FTDC_OF_CloseYesterday = _thosttraderapi.THOST_FTDC_OF_CloseYesterday
THOST_FTDC_OF_ForceOff = _thosttraderapi.THOST_FTDC_OF_ForceOff
THOST_FTDC_OF_LocalForceClose = _thosttraderapi.THOST_FTDC_OF_LocalForceClose
THOST_FTDC_FCC_NotForceClose = _thosttraderapi.THOST_FTDC_FCC_NotForceClose
THOST_FTDC_FCC_LackDeposit = _thosttraderapi.THOST_FTDC_FCC_LackDeposit
THOST_FTDC_FCC_ClientOverPositionLimit = _thosttraderapi.THOST_FTDC_FCC_ClientOverPositionLimit
THOST_FTDC_FCC_MemberOverPositionLimit = _thosttraderapi.THOST_FTDC_FCC_MemberOverPositionLimit
THOST_FTDC_FCC_NotMultiple = _thosttraderapi.THOST_FTDC_FCC_NotMultiple
THOST_FTDC_FCC_Violation = _thosttraderapi.THOST_FTDC_FCC_Violation
THOST_FTDC_FCC_Other = _thosttraderapi.THOST_FTDC_FCC_Other
THOST_FTDC_FCC_PersonDeliv = _thosttraderapi.THOST_FTDC_FCC_PersonDeliv
THOST_FTDC_ORDT_Normal = _thosttraderapi.THOST_FTDC_ORDT_Normal
THOST_FTDC_ORDT_DeriveFromQuote = _thosttraderapi.THOST_FTDC_ORDT_DeriveFromQuote
THOST_FTDC_ORDT_DeriveFromCombination = _thosttraderapi.THOST_FTDC_ORDT_DeriveFromCombination
THOST_FTDC_ORDT_Combination = _thosttraderapi.THOST_FTDC_ORDT_Combination
THOST_FTDC_ORDT_ConditionalOrder = _thosttraderapi.THOST_FTDC_ORDT_ConditionalOrder
THOST_FTDC_ORDT_Swap = _thosttraderapi.THOST_FTDC_ORDT_Swap
THOST_FTDC_ORDT_DeriveFromBlockTrade = _thosttraderapi.THOST_FTDC_ORDT_DeriveFromBlockTrade
THOST_FTDC_ORDT_DeriveFromEFPTrade = _thosttraderapi.THOST_FTDC_ORDT_DeriveFromEFPTrade
THOST_FTDC_TC_IOC = _thosttraderapi.THOST_FTDC_TC_IOC
THOST_FTDC_TC_GFS = _thosttraderapi.THOST_FTDC_TC_GFS
THOST_FTDC_TC_GFD = _thosttraderapi.THOST_FTDC_TC_GFD
THOST_FTDC_TC_GTD = _thosttraderapi.THOST_FTDC_TC_GTD
THOST_FTDC_TC_GTC = _thosttraderapi.THOST_FTDC_TC_GTC
THOST_FTDC_TC_GFA = _thosttraderapi.THOST_FTDC_TC_GFA
THOST_FTDC_VC_AV = _thosttraderapi.THOST_FTDC_VC_AV
THOST_FTDC_VC_MV = _thosttraderapi.THOST_FTDC_VC_MV
THOST_FTDC_VC_CV = _thosttraderapi.THOST_FTDC_VC_CV
THOST_FTDC_CC_Immediately = _thosttraderapi.THOST_FTDC_CC_Immediately
THOST_FTDC_CC_Touch = _thosttraderapi.THOST_FTDC_CC_Touch
THOST_FTDC_CC_TouchProfit = _thosttraderapi.THOST_FTDC_CC_TouchProfit
THOST_FTDC_CC_ParkedOrder = _thosttraderapi.THOST_FTDC_CC_ParkedOrder
THOST_FTDC_CC_LastPriceGreaterThanStopPrice = _thosttraderapi.THOST_FTDC_CC_LastPriceGreaterThanStopPrice
THOST_FTDC_CC_LastPriceGreaterEqualStopPrice = _thosttraderapi.THOST_FTDC_CC_LastPriceGreaterEqualStopPrice
THOST_FTDC_CC_LastPriceLesserThanStopPrice = _thosttraderapi.THOST_FTDC_CC_LastPriceLesserThanStopPrice
THOST_FTDC_CC_LastPriceLesserEqualStopPrice = _thosttraderapi.THOST_FTDC_CC_LastPriceLesserEqualStopPrice
THOST_FTDC_CC_AskPriceGreaterThanStopPrice = _thosttraderapi.THOST_FTDC_CC_AskPriceGreaterThanStopPrice
THOST_FTDC_CC_AskPriceGreaterEqualStopPrice = _thosttraderapi.THOST_FTDC_CC_AskPriceGreaterEqualStopPrice
THOST_FTDC_CC_AskPriceLesserThanStopPrice = _thosttraderapi.THOST_FTDC_CC_AskPriceLesserThanStopPrice
THOST_FTDC_CC_AskPriceLesserEqualStopPrice = _thosttraderapi.THOST_FTDC_CC_AskPriceLesserEqualStopPrice
THOST_FTDC_CC_BidPriceGreaterThanStopPrice = _thosttraderapi.THOST_FTDC_CC_BidPriceGreaterThanStopPrice
THOST_FTDC_CC_BidPriceGreaterEqualStopPrice = _thosttraderapi.THOST_FTDC_CC_BidPriceGreaterEqualStopPrice
THOST_FTDC_CC_BidPriceLesserThanStopPrice = _thosttraderapi.THOST_FTDC_CC_BidPriceLesserThanStopPrice
THOST_FTDC_CC_BidPriceLesserEqualStopPrice = _thosttraderapi.THOST_FTDC_CC_BidPriceLesserEqualStopPrice
THOST_FTDC_AF_Delete = _thosttraderapi.THOST_FTDC_AF_Delete
THOST_FTDC_AF_Modify = _thosttraderapi.THOST_FTDC_AF_Modify
THOST_FTDC_TR_Allow = _thosttraderapi.THOST_FTDC_TR_Allow
THOST_FTDC_TR_CloseOnly = _thosttraderapi.THOST_FTDC_TR_CloseOnly
THOST_FTDC_TR_Forbidden = _thosttraderapi.THOST_FTDC_TR_Forbidden
THOST_FTDC_OSRC_Participant = _thosttraderapi.THOST_FTDC_OSRC_Participant
THOST_FTDC_OSRC_Administrator = _thosttraderapi.THOST_FTDC_OSRC_Administrator
THOST_FTDC_TRDT_SplitCombination = _thosttraderapi.THOST_FTDC_TRDT_SplitCombination
THOST_FTDC_TRDT_Common = _thosttraderapi.THOST_FTDC_TRDT_Common
THOST_FTDC_TRDT_OptionsExecution = _thosttraderapi.THOST_FTDC_TRDT_OptionsExecution
THOST_FTDC_TRDT_OTC = _thosttraderapi.THOST_FTDC_TRDT_OTC
THOST_FTDC_TRDT_EFPDerived = _thosttraderapi.THOST_FTDC_TRDT_EFPDerived
THOST_FTDC_TRDT_CombinationDerived = _thosttraderapi.THOST_FTDC_TRDT_CombinationDerived
THOST_FTDC_TRDT_BlockTrade = _thosttraderapi.THOST_FTDC_TRDT_BlockTrade
THOST_FTDC_PSRC_LastPrice = _thosttraderapi.THOST_FTDC_PSRC_LastPrice
THOST_FTDC_PSRC_Buy = _thosttraderapi.THOST_FTDC_PSRC_Buy
THOST_FTDC_PSRC_Sell = _thosttraderapi.THOST_FTDC_PSRC_Sell
THOST_FTDC_PSRC_OTC = _thosttraderapi.THOST_FTDC_PSRC_OTC
THOST_FTDC_IS_BeforeTrading = _thosttraderapi.THOST_FTDC_IS_BeforeTrading
THOST_FTDC_IS_NoTrading = _thosttraderapi.THOST_FTDC_IS_NoTrading
THOST_FTDC_IS_Continous = _thosttraderapi.THOST_FTDC_IS_Continous
THOST_FTDC_IS_AuctionOrdering = _thosttraderapi.THOST_FTDC_IS_AuctionOrdering
THOST_FTDC_IS_AuctionBalance = _thosttraderapi.THOST_FTDC_IS_AuctionBalance
THOST_FTDC_IS_AuctionMatch = _thosttraderapi.THOST_FTDC_IS_AuctionMatch
THOST_FTDC_IS_Closed = _thosttraderapi.THOST_FTDC_IS_Closed
THOST_FTDC_IER_Automatic = _thosttraderapi.THOST_FTDC_IER_Automatic
THOST_FTDC_IER_Manual = _thosttraderapi.THOST_FTDC_IER_Manual
THOST_FTDC_IER_Fuse = _thosttraderapi.THOST_FTDC_IER_Fuse
THOST_FTDC_BS_NoUpload = _thosttraderapi.THOST_FTDC_BS_NoUpload
THOST_FTDC_BS_Uploaded = _thosttraderapi.THOST_FTDC_BS_Uploaded
THOST_FTDC_BS_Failed = _thosttraderapi.THOST_FTDC_BS_Failed
THOST_FTDC_RS_All = _thosttraderapi.THOST_FTDC_RS_All
THOST_FTDC_RS_ByProduct = _thosttraderapi.THOST_FTDC_RS_ByProduct
THOST_FTDC_RP_ByVolume = _thosttraderapi.THOST_FTDC_RP_ByVolume
THOST_FTDC_RP_ByFeeOnHand = _thosttraderapi.THOST_FTDC_RP_ByFeeOnHand
THOST_FTDC_RL_Level1 = _thosttraderapi.THOST_FTDC_RL_Level1
THOST_FTDC_RL_Level2 = _thosttraderapi.THOST_FTDC_RL_Level2
THOST_FTDC_RL_Level3 = _thosttraderapi.THOST_FTDC_RL_Level3
THOST_FTDC_RL_Level4 = _thosttraderapi.THOST_FTDC_RL_Level4
THOST_FTDC_RL_Level5 = _thosttraderapi.THOST_FTDC_RL_Level5
THOST_FTDC_RL_Level6 = _thosttraderapi.THOST_FTDC_RL_Level6
THOST_FTDC_RL_Level7 = _thosttraderapi.THOST_FTDC_RL_Level7
THOST_FTDC_RL_Level8 = _thosttraderapi.THOST_FTDC_RL_Level8
THOST_FTDC_RL_Level9 = _thosttraderapi.THOST_FTDC_RL_Level9
THOST_FTDC_RSD_ByPeriod = _thosttraderapi.THOST_FTDC_RSD_ByPeriod
THOST_FTDC_RSD_ByStandard = _thosttraderapi.THOST_FTDC_RSD_ByStandard
THOST_FTDC_MT_Out = _thosttraderapi.THOST_FTDC_MT_Out
THOST_FTDC_MT_In = _thosttraderapi.THOST_FTDC_MT_In
THOST_FTDC_ISPI_MortgageRatio = _thosttraderapi.THOST_FTDC_ISPI_MortgageRatio
THOST_FTDC_ISPI_MarginWay = _thosttraderapi.THOST_FTDC_ISPI_MarginWay
THOST_FTDC_ISPI_BillDeposit = _thosttraderapi.THOST_FTDC_ISPI_BillDeposit
THOST_FTDC_ESPI_MortgageRatio = _thosttraderapi.THOST_FTDC_ESPI_MortgageRatio
THOST_FTDC_ESPI_OtherFundItem = _thosttraderapi.THOST_FTDC_ESPI_OtherFundItem
THOST_FTDC_ESPI_OtherFundImport = _thosttraderapi.THOST_FTDC_ESPI_OtherFundImport
THOST_FTDC_ESPI_CFFEXMinPrepa = _thosttraderapi.THOST_FTDC_ESPI_CFFEXMinPrepa
THOST_FTDC_ESPI_CZCESettlementType = _thosttraderapi.THOST_FTDC_ESPI_CZCESettlementType
THOST_FTDC_ESPI_ExchDelivFeeMode = _thosttraderapi.THOST_FTDC_ESPI_ExchDelivFeeMode
THOST_FTDC_ESPI_DelivFeeMode = _thosttraderapi.THOST_FTDC_ESPI_DelivFeeMode
THOST_FTDC_ESPI_CZCEComMarginType = _thosttraderapi.THOST_FTDC_ESPI_CZCEComMarginType
THOST_FTDC_ESPI_DceComMarginType = _thosttraderapi.THOST_FTDC_ESPI_DceComMarginType
THOST_FTDC_ESPI_OptOutDisCountRate = _thosttraderapi.THOST_FTDC_ESPI_OptOutDisCountRate
THOST_FTDC_ESPI_OptMiniGuarantee = _thosttraderapi.THOST_FTDC_ESPI_OptMiniGuarantee
THOST_FTDC_SPI_InvestorIDMinLength = _thosttraderapi.THOST_FTDC_SPI_InvestorIDMinLength
THOST_FTDC_SPI_AccountIDMinLength = _thosttraderapi.THOST_FTDC_SPI_AccountIDMinLength
THOST_FTDC_SPI_UserRightLogon = _thosttraderapi.THOST_FTDC_SPI_UserRightLogon
THOST_FTDC_SPI_SettlementBillTrade = _thosttraderapi.THOST_FTDC_SPI_SettlementBillTrade
THOST_FTDC_SPI_TradingCode = _thosttraderapi.THOST_FTDC_SPI_TradingCode
THOST_FTDC_SPI_CheckFund = _thosttraderapi.THOST_FTDC_SPI_CheckFund
THOST_FTDC_SPI_CommModelRight = _thosttraderapi.THOST_FTDC_SPI_CommModelRight
THOST_FTDC_SPI_MarginModelRight = _thosttraderapi.THOST_FTDC_SPI_MarginModelRight
THOST_FTDC_SPI_IsStandardActive = _thosttraderapi.THOST_FTDC_SPI_IsStandardActive
THOST_FTDC_SPI_UploadSettlementFile = _thosttraderapi.THOST_FTDC_SPI_UploadSettlementFile
THOST_FTDC_SPI_DownloadCSRCFile = _thosttraderapi.THOST_FTDC_SPI_DownloadCSRCFile
THOST_FTDC_SPI_SettlementBillFile = _thosttraderapi.THOST_FTDC_SPI_SettlementBillFile
THOST_FTDC_SPI_CSRCOthersFile = _thosttraderapi.THOST_FTDC_SPI_CSRCOthersFile
THOST_FTDC_SPI_InvestorPhoto = _thosttraderapi.THOST_FTDC_SPI_InvestorPhoto
THOST_FTDC_SPI_CSRCData = _thosttraderapi.THOST_FTDC_SPI_CSRCData
THOST_FTDC_SPI_InvestorPwdModel = _thosttraderapi.THOST_FTDC_SPI_InvestorPwdModel
THOST_FTDC_SPI_CFFEXInvestorSettleFile = _thosttraderapi.THOST_FTDC_SPI_CFFEXInvestorSettleFile
THOST_FTDC_SPI_InvestorIDType = _thosttraderapi.THOST_FTDC_SPI_InvestorIDType
THOST_FTDC_SPI_FreezeMaxReMain = _thosttraderapi.THOST_FTDC_SPI_FreezeMaxReMain
THOST_FTDC_SPI_IsSync = _thosttraderapi.THOST_FTDC_SPI_IsSync
THOST_FTDC_SPI_RelieveOpenLimit = _thosttraderapi.THOST_FTDC_SPI_RelieveOpenLimit
THOST_FTDC_SPI_IsStandardFreeze = _thosttraderapi.THOST_FTDC_SPI_IsStandardFreeze
THOST_FTDC_SPI_CZCENormalProductHedge = _thosttraderapi.THOST_FTDC_SPI_CZCENormalProductHedge
THOST_FTDC_TPID_EncryptionStandard = _thosttraderapi.THOST_FTDC_TPID_EncryptionStandard
THOST_FTDC_TPID_RiskMode = _thosttraderapi.THOST_FTDC_TPID_RiskMode
THOST_FTDC_TPID_RiskModeGlobal = _thosttraderapi.THOST_FTDC_TPID_RiskModeGlobal
THOST_FTDC_TPID_modeEncode = _thosttraderapi.THOST_FTDC_TPID_modeEncode
THOST_FTDC_TPID_tickMode = _thosttraderapi.THOST_FTDC_TPID_tickMode
THOST_FTDC_TPID_SingleUserSessionMaxNum = _thosttraderapi.THOST_FTDC_TPID_SingleUserSessionMaxNum
THOST_FTDC_TPID_LoginFailMaxNum = _thosttraderapi.THOST_FTDC_TPID_LoginFailMaxNum
THOST_FTDC_TPID_IsAuthForce = _thosttraderapi.THOST_FTDC_TPID_IsAuthForce
THOST_FTDC_TPID_IsPosiFreeze = _thosttraderapi.THOST_FTDC_TPID_IsPosiFreeze
THOST_FTDC_TPID_IsPosiLimit = _thosttraderapi.THOST_FTDC_TPID_IsPosiLimit
THOST_FTDC_TPID_ForQuoteTimeInterval = _thosttraderapi.THOST_FTDC_TPID_ForQuoteTimeInterval
THOST_FTDC_TPID_IsFuturePosiLimit = _thosttraderapi.THOST_FTDC_TPID_IsFuturePosiLimit
THOST_FTDC_TPID_IsFutureOrderFreq = _thosttraderapi.THOST_FTDC_TPID_IsFutureOrderFreq
THOST_FTDC_TPID_IsExecOrderProfit = _thosttraderapi.THOST_FTDC_TPID_IsExecOrderProfit
THOST_FTDC_TPID_IsCheckBankAcc = _thosttraderapi.THOST_FTDC_TPID_IsCheckBankAcc
THOST_FTDC_TPID_PasswordDeadLine = _thosttraderapi.THOST_FTDC_TPID_PasswordDeadLine
THOST_FTDC_TPID_IsStrongPassword = _thosttraderapi.THOST_FTDC_TPID_IsStrongPassword
THOST_FTDC_TPID_BalanceMorgage = _thosttraderapi.THOST_FTDC_TPID_BalanceMorgage
THOST_FTDC_TPID_MinPwdLen = _thosttraderapi.THOST_FTDC_TPID_MinPwdLen
THOST_FTDC_TPID_LoginFailMaxNumForIP = _thosttraderapi.THOST_FTDC_TPID_LoginFailMaxNumForIP
THOST_FTDC_TPID_PasswordPeriod = _thosttraderapi.THOST_FTDC_TPID_PasswordPeriod
THOST_FTDC_FI_SettlementFund = _thosttraderapi.THOST_FTDC_FI_SettlementFund
THOST_FTDC_FI_Trade = _thosttraderapi.THOST_FTDC_FI_Trade
THOST_FTDC_FI_InvestorPosition = _thosttraderapi.THOST_FTDC_FI_InvestorPosition
THOST_FTDC_FI_SubEntryFund = _thosttraderapi.THOST_FTDC_FI_SubEntryFund
THOST_FTDC_FI_CZCECombinationPos = _thosttraderapi.THOST_FTDC_FI_CZCECombinationPos
THOST_FTDC_FI_CSRCData = _thosttraderapi.THOST_FTDC_FI_CSRCData
THOST_FTDC_FI_CZCEClose = _thosttraderapi.THOST_FTDC_FI_CZCEClose
THOST_FTDC_FI_CZCENoClose = _thosttraderapi.THOST_FTDC_FI_CZCENoClose
THOST_FTDC_FI_PositionDtl = _thosttraderapi.THOST_FTDC_FI_PositionDtl
THOST_FTDC_FI_OptionStrike = _thosttraderapi.THOST_FTDC_FI_OptionStrike
THOST_FTDC_FI_SettlementPriceComparison = _thosttraderapi.THOST_FTDC_FI_SettlementPriceComparison
THOST_FTDC_FI_NonTradePosChange = _thosttraderapi.THOST_FTDC_FI_NonTradePosChange
THOST_FTDC_FUT_Settlement = _thosttraderapi.THOST_FTDC_FUT_Settlement
THOST_FTDC_FUT_Check = _thosttraderapi.THOST_FTDC_FUT_Check
THOST_FTDC_FFT_Txt = _thosttraderapi.THOST_FTDC_FFT_Txt
THOST_FTDC_FFT_Zip = _thosttraderapi.THOST_FTDC_FFT_Zip
THOST_FTDC_FFT_DBF = _thosttraderapi.THOST_FTDC_FFT_DBF
THOST_FTDC_FUS_SucceedUpload = _thosttraderapi.THOST_FTDC_FUS_SucceedUpload
THOST_FTDC_FUS_FailedUpload = _thosttraderapi.THOST_FTDC_FUS_FailedUpload
THOST_FTDC_FUS_SucceedLoad = _thosttraderapi.THOST_FTDC_FUS_SucceedLoad
THOST_FTDC_FUS_PartSucceedLoad = _thosttraderapi.THOST_FTDC_FUS_PartSucceedLoad
THOST_FTDC_FUS_FailedLoad = _thosttraderapi.THOST_FTDC_FUS_FailedLoad
THOST_FTDC_TD_Out = _thosttraderapi.THOST_FTDC_TD_Out
THOST_FTDC_TD_In = _thosttraderapi.THOST_FTDC_TD_In
THOST_FTDC_SC_NoSpecialRule = _thosttraderapi.THOST_FTDC_SC_NoSpecialRule
THOST_FTDC_SC_NoSpringFestival = _thosttraderapi.THOST_FTDC_SC_NoSpringFestival
THOST_FTDC_IPT_LastSettlement = _thosttraderapi.THOST_FTDC_IPT_LastSettlement
THOST_FTDC_IPT_LaseClose = _thosttraderapi.THOST_FTDC_IPT_LaseClose
THOST_FTDC_PLP_Active = _thosttraderapi.THOST_FTDC_PLP_Active
THOST_FTDC_PLP_NonActive = _thosttraderapi.THOST_FTDC_PLP_NonActive
THOST_FTDC_PLP_Canceled = _thosttraderapi.THOST_FTDC_PLP_Canceled
THOST_FTDC_DM_CashDeliv = _thosttraderapi.THOST_FTDC_DM_CashDeliv
THOST_FTDC_DM_CommodityDeliv = _thosttraderapi.THOST_FTDC_DM_CommodityDeliv
THOST_FTDC_FIOT_FundIO = _thosttraderapi.THOST_FTDC_FIOT_FundIO
THOST_FTDC_FIOT_Transfer = _thosttraderapi.THOST_FTDC_FIOT_Transfer
THOST_FTDC_FIOT_SwapCurrency = _thosttraderapi.THOST_FTDC_FIOT_SwapCurrency
THOST_FTDC_FT_Deposite = _thosttraderapi.THOST_FTDC_FT_Deposite
THOST_FTDC_FT_ItemFund = _thosttraderapi.THOST_FTDC_FT_ItemFund
THOST_FTDC_FT_Company = _thosttraderapi.THOST_FTDC_FT_Company
THOST_FTDC_FT_InnerTransfer = _thosttraderapi.THOST_FTDC_FT_InnerTransfer
THOST_FTDC_FD_In = _thosttraderapi.THOST_FTDC_FD_In
THOST_FTDC_FD_Out = _thosttraderapi.THOST_FTDC_FD_Out
THOST_FTDC_FS_Record = _thosttraderapi.THOST_FTDC_FS_Record
THOST_FTDC_FS_Check = _thosttraderapi.THOST_FTDC_FS_Check
THOST_FTDC_FS_Charge = _thosttraderapi.THOST_FTDC_FS_Charge
THOST_FTDC_PS_None = _thosttraderapi.THOST_FTDC_PS_None
THOST_FTDC_PS_Publishing = _thosttraderapi.THOST_FTDC_PS_Publishing
THOST_FTDC_PS_Published = _thosttraderapi.THOST_FTDC_PS_Published
THOST_FTDC_ES_NonActive = _thosttraderapi.THOST_FTDC_ES_NonActive
THOST_FTDC_ES_Startup = _thosttraderapi.THOST_FTDC_ES_Startup
THOST_FTDC_ES_Initialize = _thosttraderapi.THOST_FTDC_ES_Initialize
THOST_FTDC_ES_Initialized = _thosttraderapi.THOST_FTDC_ES_Initialized
THOST_FTDC_ES_Close = _thosttraderapi.THOST_FTDC_ES_Close
THOST_FTDC_ES_Closed = _thosttraderapi.THOST_FTDC_ES_Closed
THOST_FTDC_ES_Settlement = _thosttraderapi.THOST_FTDC_ES_Settlement
THOST_FTDC_STS_Initialize = _thosttraderapi.THOST_FTDC_STS_Initialize
THOST_FTDC_STS_Settlementing = _thosttraderapi.THOST_FTDC_STS_Settlementing
THOST_FTDC_STS_Settlemented = _thosttraderapi.THOST_FTDC_STS_Settlemented
THOST_FTDC_STS_Finished = _thosttraderapi.THOST_FTDC_STS_Finished
THOST_FTDC_CT_Person = _thosttraderapi.THOST_FTDC_CT_Person
THOST_FTDC_CT_Company = _thosttraderapi.THOST_FTDC_CT_Company
THOST_FTDC_CT_Fund = _thosttraderapi.THOST_FTDC_CT_Fund
THOST_FTDC_CT_SpecialOrgan = _thosttraderapi.THOST_FTDC_CT_SpecialOrgan
THOST_FTDC_CT_Asset = _thosttraderapi.THOST_FTDC_CT_Asset
THOST_FTDC_BT_Trade = _thosttraderapi.THOST_FTDC_BT_Trade
THOST_FTDC_BT_TradeSettle = _thosttraderapi.THOST_FTDC_BT_TradeSettle
THOST_FTDC_FAS_Low = _thosttraderapi.THOST_FTDC_FAS_Low
THOST_FTDC_FAS_Normal = _thosttraderapi.THOST_FTDC_FAS_Normal
THOST_FTDC_FAS_Focus = _thosttraderapi.THOST_FTDC_FAS_Focus
THOST_FTDC_FAS_Risk = _thosttraderapi.THOST_FTDC_FAS_Risk
THOST_FTDC_FAS_ByTrade = _thosttraderapi.THOST_FTDC_FAS_ByTrade
THOST_FTDC_FAS_ByDeliv = _thosttraderapi.THOST_FTDC_FAS_ByDeliv
THOST_FTDC_FAS_None = _thosttraderapi.THOST_FTDC_FAS_None
THOST_FTDC_FAS_FixFee = _thosttraderapi.THOST_FTDC_FAS_FixFee
THOST_FTDC_PWDT_Trade = _thosttraderapi.THOST_FTDC_PWDT_Trade
THOST_FTDC_PWDT_Account = _thosttraderapi.THOST_FTDC_PWDT_Account
THOST_FTDC_AG_All = _thosttraderapi.THOST_FTDC_AG_All
THOST_FTDC_AG_OnlyLost = _thosttraderapi.THOST_FTDC_AG_OnlyLost
THOST_FTDC_AG_OnlyGain = _thosttraderapi.THOST_FTDC_AG_OnlyGain
THOST_FTDC_AG_None = _thosttraderapi.THOST_FTDC_AG_None
THOST_FTDC_ICP_Include = _thosttraderapi.THOST_FTDC_ICP_Include
THOST_FTDC_ICP_NotInclude = _thosttraderapi.THOST_FTDC_ICP_NotInclude
THOST_FTDC_AWT_Enable = _thosttraderapi.THOST_FTDC_AWT_Enable
THOST_FTDC_AWT_Disable = _thosttraderapi.THOST_FTDC_AWT_Disable
THOST_FTDC_AWT_NoHoldEnable = _thosttraderapi.THOST_FTDC_AWT_NoHoldEnable
THOST_FTDC_FPWD_UnCheck = _thosttraderapi.THOST_FTDC_FPWD_UnCheck
THOST_FTDC_FPWD_Check = _thosttraderapi.THOST_FTDC_FPWD_Check
THOST_FTDC_TT_BankToFuture = _thosttraderapi.THOST_FTDC_TT_BankToFuture
THOST_FTDC_TT_FutureToBank = _thosttraderapi.THOST_FTDC_TT_FutureToBank
THOST_FTDC_TVF_Invalid = _thosttraderapi.THOST_FTDC_TVF_Invalid
THOST_FTDC_TVF_Valid = _thosttraderapi.THOST_FTDC_TVF_Valid
THOST_FTDC_TVF_Reverse = _thosttraderapi.THOST_FTDC_TVF_Reverse
THOST_FTDC_RN_CD = _thosttraderapi.THOST_FTDC_RN_CD
THOST_FTDC_RN_ZT = _thosttraderapi.THOST_FTDC_RN_ZT
THOST_FTDC_RN_QT = _thosttraderapi.THOST_FTDC_RN_QT
THOST_FTDC_SEX_None = _thosttraderapi.THOST_FTDC_SEX_None
THOST_FTDC_SEX_Man = _thosttraderapi.THOST_FTDC_SEX_Man
THOST_FTDC_SEX_Woman = _thosttraderapi.THOST_FTDC_SEX_Woman
THOST_FTDC_UT_Investor = _thosttraderapi.THOST_FTDC_UT_Investor
THOST_FTDC_UT_Operator = _thosttraderapi.THOST_FTDC_UT_Operator
THOST_FTDC_UT_SuperUser = _thosttraderapi.THOST_FTDC_UT_SuperUser
THOST_FTDC_RATETYPE_MarginRate = _thosttraderapi.THOST_FTDC_RATETYPE_MarginRate
THOST_FTDC_NOTETYPE_TradeSettleBill = _thosttraderapi.THOST_FTDC_NOTETYPE_TradeSettleBill
THOST_FTDC_NOTETYPE_TradeSettleMonth = _thosttraderapi.THOST_FTDC_NOTETYPE_TradeSettleMonth
THOST_FTDC_NOTETYPE_CallMarginNotes = _thosttraderapi.THOST_FTDC_NOTETYPE_CallMarginNotes
THOST_FTDC_NOTETYPE_ForceCloseNotes = _thosttraderapi.THOST_FTDC_NOTETYPE_ForceCloseNotes
THOST_FTDC_NOTETYPE_TradeNotes = _thosttraderapi.THOST_FTDC_NOTETYPE_TradeNotes
THOST_FTDC_NOTETYPE_DelivNotes = _thosttraderapi.THOST_FTDC_NOTETYPE_DelivNotes
THOST_FTDC_SBS_Day = _thosttraderapi.THOST_FTDC_SBS_Day
THOST_FTDC_SBS_Volume = _thosttraderapi.THOST_FTDC_SBS_Volume
THOST_FTDC_ST_Day = _thosttraderapi.THOST_FTDC_ST_Day
THOST_FTDC_ST_Month = _thosttraderapi.THOST_FTDC_ST_Month
THOST_FTDC_URT_Logon = _thosttraderapi.THOST_FTDC_URT_Logon
THOST_FTDC_URT_Transfer = _thosttraderapi.THOST_FTDC_URT_Transfer
THOST_FTDC_URT_EMail = _thosttraderapi.THOST_FTDC_URT_EMail
THOST_FTDC_URT_Fax = _thosttraderapi.THOST_FTDC_URT_Fax
THOST_FTDC_URT_ConditionOrder = _thosttraderapi.THOST_FTDC_URT_ConditionOrder
THOST_FTDC_MPT_PreSettlementPrice = _thosttraderapi.THOST_FTDC_MPT_PreSettlementPrice
THOST_FTDC_MPT_SettlementPrice = _thosttraderapi.THOST_FTDC_MPT_SettlementPrice
THOST_FTDC_MPT_AveragePrice = _thosttraderapi.THOST_FTDC_MPT_AveragePrice
THOST_FTDC_MPT_OpenPrice = _thosttraderapi.THOST_FTDC_MPT_OpenPrice
THOST_FTDC_BGS_None = _thosttraderapi.THOST_FTDC_BGS_None
THOST_FTDC_BGS_NoGenerated = _thosttraderapi.THOST_FTDC_BGS_NoGenerated
THOST_FTDC_BGS_Generated = _thosttraderapi.THOST_FTDC_BGS_Generated
THOST_FTDC_AT_HandlePositionAlgo = _thosttraderapi.THOST_FTDC_AT_HandlePositionAlgo
THOST_FTDC_AT_FindMarginRateAlgo = _thosttraderapi.THOST_FTDC_AT_FindMarginRateAlgo
THOST_FTDC_HPA_Base = _thosttraderapi.THOST_FTDC_HPA_Base
THOST_FTDC_HPA_DCE = _thosttraderapi.THOST_FTDC_HPA_DCE
THOST_FTDC_HPA_CZCE = _thosttraderapi.THOST_FTDC_HPA_CZCE
THOST_FTDC_FMRA_Base = _thosttraderapi.THOST_FTDC_FMRA_Base
THOST_FTDC_FMRA_DCE = _thosttraderapi.THOST_FTDC_FMRA_DCE
THOST_FTDC_FMRA_CZCE = _thosttraderapi.THOST_FTDC_FMRA_CZCE
THOST_FTDC_HTAA_Base = _thosttraderapi.THOST_FTDC_HTAA_Base
THOST_FTDC_HTAA_DCE = _thosttraderapi.THOST_FTDC_HTAA_DCE
THOST_FTDC_HTAA_CZCE = _thosttraderapi.THOST_FTDC_HTAA_CZCE
THOST_FTDC_PST_Order = _thosttraderapi.THOST_FTDC_PST_Order
THOST_FTDC_PST_Open = _thosttraderapi.THOST_FTDC_PST_Open
THOST_FTDC_PST_Fund = _thosttraderapi.THOST_FTDC_PST_Fund
THOST_FTDC_PST_Settlement = _thosttraderapi.THOST_FTDC_PST_Settlement
THOST_FTDC_PST_Company = _thosttraderapi.THOST_FTDC_PST_Company
THOST_FTDC_PST_Corporation = _thosttraderapi.THOST_FTDC_PST_Corporation
THOST_FTDC_PST_LinkMan = _thosttraderapi.THOST_FTDC_PST_LinkMan
THOST_FTDC_PST_Ledger = _thosttraderapi.THOST_FTDC_PST_Ledger
THOST_FTDC_PST_Trustee = _thosttraderapi.THOST_FTDC_PST_Trustee
THOST_FTDC_PST_TrusteeCorporation = _thosttraderapi.THOST_FTDC_PST_TrusteeCorporation
THOST_FTDC_PST_TrusteeOpen = _thosttraderapi.THOST_FTDC_PST_TrusteeOpen
THOST_FTDC_PST_TrusteeContact = _thosttraderapi.THOST_FTDC_PST_TrusteeContact
THOST_FTDC_PST_ForeignerRefer = _thosttraderapi.THOST_FTDC_PST_ForeignerRefer
THOST_FTDC_PST_CorporationRefer = _thosttraderapi.THOST_FTDC_PST_CorporationRefer
THOST_FTDC_QIR_All = _thosttraderapi.THOST_FTDC_QIR_All
THOST_FTDC_QIR_Group = _thosttraderapi.THOST_FTDC_QIR_Group
THOST_FTDC_QIR_Single = _thosttraderapi.THOST_FTDC_QIR_Single
THOST_FTDC_IRS_Normal = _thosttraderapi.THOST_FTDC_IRS_Normal
THOST_FTDC_IRS_Warn = _thosttraderapi.THOST_FTDC_IRS_Warn
THOST_FTDC_IRS_Call = _thosttraderapi.THOST_FTDC_IRS_Call
THOST_FTDC_IRS_Force = _thosttraderapi.THOST_FTDC_IRS_Force
THOST_FTDC_IRS_Exception = _thosttraderapi.THOST_FTDC_IRS_Exception
THOST_FTDC_UET_Login = _thosttraderapi.THOST_FTDC_UET_Login
THOST_FTDC_UET_Logout = _thosttraderapi.THOST_FTDC_UET_Logout
THOST_FTDC_UET_Trading = _thosttraderapi.THOST_FTDC_UET_Trading
THOST_FTDC_UET_TradingError = _thosttraderapi.THOST_FTDC_UET_TradingError
THOST_FTDC_UET_UpdatePassword = _thosttraderapi.THOST_FTDC_UET_UpdatePassword
THOST_FTDC_UET_Authenticate = _thosttraderapi.THOST_FTDC_UET_Authenticate
THOST_FTDC_UET_Other = _thosttraderapi.THOST_FTDC_UET_Other
THOST_FTDC_ICS_Close = _thosttraderapi.THOST_FTDC_ICS_Close
THOST_FTDC_ICS_CloseToday = _thosttraderapi.THOST_FTDC_ICS_CloseToday
THOST_FTDC_SM_Non = _thosttraderapi.THOST_FTDC_SM_Non
THOST_FTDC_SM_Instrument = _thosttraderapi.THOST_FTDC_SM_Instrument
THOST_FTDC_SM_Product = _thosttraderapi.THOST_FTDC_SM_Product
THOST_FTDC_SM_Investor = _thosttraderapi.THOST_FTDC_SM_Investor
THOST_FTDC_PAOS_NotSend = _thosttraderapi.THOST_FTDC_PAOS_NotSend
THOST_FTDC_PAOS_Send = _thosttraderapi.THOST_FTDC_PAOS_Send
THOST_FTDC_PAOS_Deleted = _thosttraderapi.THOST_FTDC_PAOS_Deleted
THOST_FTDC_VDS_Dealing = _thosttraderapi.THOST_FTDC_VDS_Dealing
THOST_FTDC_VDS_DeaclSucceed = _thosttraderapi.THOST_FTDC_VDS_DeaclSucceed
THOST_FTDC_ORGS_Standard = _thosttraderapi.THOST_FTDC_ORGS_Standard
THOST_FTDC_ORGS_ESunny = _thosttraderapi.THOST_FTDC_ORGS_ESunny
THOST_FTDC_ORGS_KingStarV6 = _thosttraderapi.THOST_FTDC_ORGS_KingStarV6
THOST_FTDC_VTS_NaturalDeal = _thosttraderapi.THOST_FTDC_VTS_NaturalDeal
THOST_FTDC_VTS_SucceedEnd = _thosttraderapi.THOST_FTDC_VTS_SucceedEnd
THOST_FTDC_VTS_FailedEND = _thosttraderapi.THOST_FTDC_VTS_FailedEND
THOST_FTDC_VTS_Exception = _thosttraderapi.THOST_FTDC_VTS_Exception
THOST_FTDC_VTS_ManualDeal = _thosttraderapi.THOST_FTDC_VTS_ManualDeal
THOST_FTDC_VTS_MesException = _thosttraderapi.THOST_FTDC_VTS_MesException
THOST_FTDC_VTS_SysException = _thosttraderapi.THOST_FTDC_VTS_SysException
THOST_FTDC_VBAT_BankBook = _thosttraderapi.THOST_FTDC_VBAT_BankBook
THOST_FTDC_VBAT_BankCard = _thosttraderapi.THOST_FTDC_VBAT_BankCard
THOST_FTDC_VBAT_CreditCard = _thosttraderapi.THOST_FTDC_VBAT_CreditCard
THOST_FTDC_VMS_Natural = _thosttraderapi.THOST_FTDC_VMS_Natural
THOST_FTDC_VMS_Canceled = _thosttraderapi.THOST_FTDC_VMS_Canceled
THOST_FTDC_VAA_NoAvailAbility = _thosttraderapi.THOST_FTDC_VAA_NoAvailAbility
THOST_FTDC_VAA_AvailAbility = _thosttraderapi.THOST_FTDC_VAA_AvailAbility
THOST_FTDC_VAA_Repeal = _thosttraderapi.THOST_FTDC_VAA_Repeal
THOST_FTDC_GEN_Program = _thosttraderapi.THOST_FTDC_GEN_Program
THOST_FTDC_GEN_HandWork = _thosttraderapi.THOST_FTDC_GEN_HandWork
THOST_FTDC_CFMMCKK_REQUEST = _thosttraderapi.THOST_FTDC_CFMMCKK_REQUEST
THOST_FTDC_CFMMCKK_AUTO = _thosttraderapi.THOST_FTDC_CFMMCKK_AUTO
THOST_FTDC_CFMMCKK_MANUAL = _thosttraderapi.THOST_FTDC_CFMMCKK_MANUAL
THOST_FTDC_CFT_IDCard = _thosttraderapi.THOST_FTDC_CFT_IDCard
THOST_FTDC_CFT_Passport = _thosttraderapi.THOST_FTDC_CFT_Passport
THOST_FTDC_CFT_OfficerIDCard = _thosttraderapi.THOST_FTDC_CFT_OfficerIDCard
THOST_FTDC_CFT_SoldierIDCard = _thosttraderapi.THOST_FTDC_CFT_SoldierIDCard
THOST_FTDC_CFT_HomeComingCard = _thosttraderapi.THOST_FTDC_CFT_HomeComingCard
THOST_FTDC_CFT_HouseholdRegister = _thosttraderapi.THOST_FTDC_CFT_HouseholdRegister
THOST_FTDC_CFT_LicenseNo = _thosttraderapi.THOST_FTDC_CFT_LicenseNo
THOST_FTDC_CFT_InstitutionCodeCard = _thosttraderapi.THOST_FTDC_CFT_InstitutionCodeCard
THOST_FTDC_CFT_TempLicenseNo = _thosttraderapi.THOST_FTDC_CFT_TempLicenseNo
THOST_FTDC_CFT_NoEnterpriseLicenseNo = _thosttraderapi.THOST_FTDC_CFT_NoEnterpriseLicenseNo
THOST_FTDC_CFT_OtherCard = _thosttraderapi.THOST_FTDC_CFT_OtherCard
THOST_FTDC_CFT_SuperDepAgree = _thosttraderapi.THOST_FTDC_CFT_SuperDepAgree
THOST_FTDC_FBC_Others = _thosttraderapi.THOST_FTDC_FBC_Others
THOST_FTDC_FBC_TransferDetails = _thosttraderapi.THOST_FTDC_FBC_TransferDetails
THOST_FTDC_FBC_CustAccStatus = _thosttraderapi.THOST_FTDC_FBC_CustAccStatus
THOST_FTDC_FBC_AccountTradeDetails = _thosttraderapi.THOST_FTDC_FBC_AccountTradeDetails
THOST_FTDC_FBC_FutureAccountChangeInfoDetails = _thosttraderapi.THOST_FTDC_FBC_FutureAccountChangeInfoDetails
THOST_FTDC_FBC_CustMoneyDetail = _thosttraderapi.THOST_FTDC_FBC_CustMoneyDetail
THOST_FTDC_FBC_CustCancelAccountInfo = _thosttraderapi.THOST_FTDC_FBC_CustCancelAccountInfo
THOST_FTDC_FBC_CustMoneyResult = _thosttraderapi.THOST_FTDC_FBC_CustMoneyResult
THOST_FTDC_FBC_OthersExceptionResult = _thosttraderapi.THOST_FTDC_FBC_OthersExceptionResult
THOST_FTDC_FBC_CustInterestNetMoneyDetails = _thosttraderapi.THOST_FTDC_FBC_CustInterestNetMoneyDetails
THOST_FTDC_FBC_CustMoneySendAndReceiveDetails = _thosttraderapi.THOST_FTDC_FBC_CustMoneySendAndReceiveDetails
THOST_FTDC_FBC_CorporationMoneyTotal = _thosttraderapi.THOST_FTDC_FBC_CorporationMoneyTotal
THOST_FTDC_FBC_MainbodyMoneyTotal = _thosttraderapi.THOST_FTDC_FBC_MainbodyMoneyTotal
THOST_FTDC_FBC_MainPartMonitorData = _thosttraderapi.THOST_FTDC_FBC_MainPartMonitorData
THOST_FTDC_FBC_PreparationMoney = _thosttraderapi.THOST_FTDC_FBC_PreparationMoney
THOST_FTDC_FBC_BankMoneyMonitorData = _thosttraderapi.THOST_FTDC_FBC_BankMoneyMonitorData
THOST_FTDC_CEC_Exchange = _thosttraderapi.THOST_FTDC_CEC_Exchange
THOST_FTDC_CEC_Cash = _thosttraderapi.THOST_FTDC_CEC_Cash
THOST_FTDC_YNI_Yes = _thosttraderapi.THOST_FTDC_YNI_Yes
THOST_FTDC_YNI_No = _thosttraderapi.THOST_FTDC_YNI_No
THOST_FTDC_BLT_CurrentMoney = _thosttraderapi.THOST_FTDC_BLT_CurrentMoney
THOST_FTDC_BLT_UsableMoney = _thosttraderapi.THOST_FTDC_BLT_UsableMoney
THOST_FTDC_BLT_FetchableMoney = _thosttraderapi.THOST_FTDC_BLT_FetchableMoney
THOST_FTDC_BLT_FreezeMoney = _thosttraderapi.THOST_FTDC_BLT_FreezeMoney
THOST_FTDC_GD_Unknown = _thosttraderapi.THOST_FTDC_GD_Unknown
THOST_FTDC_GD_Male = _thosttraderapi.THOST_FTDC_GD_Male
THOST_FTDC_GD_Female = _thosttraderapi.THOST_FTDC_GD_Female
THOST_FTDC_FPF_BEN = _thosttraderapi.THOST_FTDC_FPF_BEN
THOST_FTDC_FPF_OUR = _thosttraderapi.THOST_FTDC_FPF_OUR
THOST_FTDC_FPF_SHA = _thosttraderapi.THOST_FTDC_FPF_SHA
THOST_FTDC_PWKT_ExchangeKey = _thosttraderapi.THOST_FTDC_PWKT_ExchangeKey
THOST_FTDC_PWKT_PassWordKey = _thosttraderapi.THOST_FTDC_PWKT_PassWordKey
THOST_FTDC_PWKT_MACKey = _thosttraderapi.THOST_FTDC_PWKT_MACKey
THOST_FTDC_PWKT_MessageKey = _thosttraderapi.THOST_FTDC_PWKT_MessageKey
THOST_FTDC_PWT_Query = _thosttraderapi.THOST_FTDC_PWT_Query
THOST_FTDC_PWT_Fetch = _thosttraderapi.THOST_FTDC_PWT_Fetch
THOST_FTDC_PWT_Transfer = _thosttraderapi.THOST_FTDC_PWT_Transfer
THOST_FTDC_PWT_Trade = _thosttraderapi.THOST_FTDC_PWT_Trade
THOST_FTDC_EM_NoEncry = _thosttraderapi.THOST_FTDC_EM_NoEncry
THOST_FTDC_EM_DES = _thosttraderapi.THOST_FTDC_EM_DES
THOST_FTDC_EM_3DES = _thosttraderapi.THOST_FTDC_EM_3DES
THOST_FTDC_BRF_BankNotNeedRepeal = _thosttraderapi.THOST_FTDC_BRF_BankNotNeedRepeal
THOST_FTDC_BRF_BankWaitingRepeal = _thosttraderapi.THOST_FTDC_BRF_BankWaitingRepeal
THOST_FTDC_BRF_BankBeenRepealed = _thosttraderapi.THOST_FTDC_BRF_BankBeenRepealed
THOST_FTDC_BRORF_BrokerNotNeedRepeal = _thosttraderapi.THOST_FTDC_BRORF_BrokerNotNeedRepeal
THOST_FTDC_BRORF_BrokerWaitingRepeal = _thosttraderapi.THOST_FTDC_BRORF_BrokerWaitingRepeal
THOST_FTDC_BRORF_BrokerBeenRepealed = _thosttraderapi.THOST_FTDC_BRORF_BrokerBeenRepealed
THOST_FTDC_TS_Bank = _thosttraderapi.THOST_FTDC_TS_Bank
THOST_FTDC_TS_Future = _thosttraderapi.THOST_FTDC_TS_Future
THOST_FTDC_TS_Store = _thosttraderapi.THOST_FTDC_TS_Store
THOST_FTDC_LF_Yes = _thosttraderapi.THOST_FTDC_LF_Yes
THOST_FTDC_LF_No = _thosttraderapi.THOST_FTDC_LF_No
THOST_FTDC_BAS_Normal = _thosttraderapi.THOST_FTDC_BAS_Normal
THOST_FTDC_BAS_Freeze = _thosttraderapi.THOST_FTDC_BAS_Freeze
THOST_FTDC_BAS_ReportLoss = _thosttraderapi.THOST_FTDC_BAS_ReportLoss
THOST_FTDC_MAS_Normal = _thosttraderapi.THOST_FTDC_MAS_Normal
THOST_FTDC_MAS_Cancel = _thosttraderapi.THOST_FTDC_MAS_Cancel
THOST_FTDC_MSS_Point = _thosttraderapi.THOST_FTDC_MSS_Point
THOST_FTDC_MSS_PrePoint = _thosttraderapi.THOST_FTDC_MSS_PrePoint
THOST_FTDC_MSS_CancelPoint = _thosttraderapi.THOST_FTDC_MSS_CancelPoint
THOST_FTDC_SYT_FutureBankTransfer = _thosttraderapi.THOST_FTDC_SYT_FutureBankTransfer
THOST_FTDC_SYT_StockBankTransfer = _thosttraderapi.THOST_FTDC_SYT_StockBankTransfer
THOST_FTDC_SYT_TheThirdPartStore = _thosttraderapi.THOST_FTDC_SYT_TheThirdPartStore
THOST_FTDC_TEF_NormalProcessing = _thosttraderapi.THOST_FTDC_TEF_NormalProcessing
THOST_FTDC_TEF_Success = _thosttraderapi.THOST_FTDC_TEF_Success
THOST_FTDC_TEF_Failed = _thosttraderapi.THOST_FTDC_TEF_Failed
THOST_FTDC_TEF_Abnormal = _thosttraderapi.THOST_FTDC_TEF_Abnormal
THOST_FTDC_TEF_ManualProcessedForException = _thosttraderapi.THOST_FTDC_TEF_ManualProcessedForException
THOST_FTDC_TEF_CommuFailedNeedManualProcess = _thosttraderapi.THOST_FTDC_TEF_CommuFailedNeedManualProcess
THOST_FTDC_TEF_SysErrorNeedManualProcess = _thosttraderapi.THOST_FTDC_TEF_SysErrorNeedManualProcess
THOST_FTDC_PSS_NotProcess = _thosttraderapi.THOST_FTDC_PSS_NotProcess
THOST_FTDC_PSS_StartProcess = _thosttraderapi.THOST_FTDC_PSS_StartProcess
THOST_FTDC_PSS_Finished = _thosttraderapi.THOST_FTDC_PSS_Finished
THOST_FTDC_CUSTT_Person = _thosttraderapi.THOST_FTDC_CUSTT_Person
THOST_FTDC_CUSTT_Institution = _thosttraderapi.THOST_FTDC_CUSTT_Institution
THOST_FTDC_FBTTD_FromBankToFuture = _thosttraderapi.THOST_FTDC_FBTTD_FromBankToFuture
THOST_FTDC_FBTTD_FromFutureToBank = _thosttraderapi.THOST_FTDC_FBTTD_FromFutureToBank
THOST_FTDC_OOD_Open = _thosttraderapi.THOST_FTDC_OOD_Open
THOST_FTDC_OOD_Destroy = _thosttraderapi.THOST_FTDC_OOD_Destroy
THOST_FTDC_AVAF_Invalid = _thosttraderapi.THOST_FTDC_AVAF_Invalid
THOST_FTDC_AVAF_Valid = _thosttraderapi.THOST_FTDC_AVAF_Valid
THOST_FTDC_AVAF_Repeal = _thosttraderapi.THOST_FTDC_AVAF_Repeal
THOST_FTDC_OT_Bank = _thosttraderapi.THOST_FTDC_OT_Bank
THOST_FTDC_OT_Future = _thosttraderapi.THOST_FTDC_OT_Future
THOST_FTDC_OT_PlateForm = _thosttraderapi.THOST_FTDC_OT_PlateForm
THOST_FTDC_OL_HeadQuarters = _thosttraderapi.THOST_FTDC_OL_HeadQuarters
THOST_FTDC_OL_Branch = _thosttraderapi.THOST_FTDC_OL_Branch
THOST_FTDC_PID_FutureProtocal = _thosttraderapi.THOST_FTDC_PID_FutureProtocal
THOST_FTDC_PID_ICBCProtocal = _thosttraderapi.THOST_FTDC_PID_ICBCProtocal
THOST_FTDC_PID_ABCProtocal = _thosttraderapi.THOST_FTDC_PID_ABCProtocal
THOST_FTDC_PID_CBCProtocal = _thosttraderapi.THOST_FTDC_PID_CBCProtocal
THOST_FTDC_PID_CCBProtocal = _thosttraderapi.THOST_FTDC_PID_CCBProtocal
THOST_FTDC_PID_BOCOMProtocal = _thosttraderapi.THOST_FTDC_PID_BOCOMProtocal
THOST_FTDC_PID_FBTPlateFormProtocal = _thosttraderapi.THOST_FTDC_PID_FBTPlateFormProtocal
THOST_FTDC_CM_ShortConnect = _thosttraderapi.THOST_FTDC_CM_ShortConnect
THOST_FTDC_CM_LongConnect = _thosttraderapi.THOST_FTDC_CM_LongConnect
THOST_FTDC_SRM_ASync = _thosttraderapi.THOST_FTDC_SRM_ASync
THOST_FTDC_SRM_Sync = _thosttraderapi.THOST_FTDC_SRM_Sync
THOST_FTDC_BAT_BankBook = _thosttraderapi.THOST_FTDC_BAT_BankBook
THOST_FTDC_BAT_SavingCard = _thosttraderapi.THOST_FTDC_BAT_SavingCard
THOST_FTDC_BAT_CreditCard = _thosttraderapi.THOST_FTDC_BAT_CreditCard
THOST_FTDC_FAT_BankBook = _thosttraderapi.THOST_FTDC_FAT_BankBook
THOST_FTDC_FAT_SavingCard = _thosttraderapi.THOST_FTDC_FAT_SavingCard
THOST_FTDC_FAT_CreditCard = _thosttraderapi.THOST_FTDC_FAT_CreditCard
THOST_FTDC_OS_Ready = _thosttraderapi.THOST_FTDC_OS_Ready
THOST_FTDC_OS_CheckIn = _thosttraderapi.THOST_FTDC_OS_CheckIn
THOST_FTDC_OS_CheckOut = _thosttraderapi.THOST_FTDC_OS_CheckOut
THOST_FTDC_OS_CheckFileArrived = _thosttraderapi.THOST_FTDC_OS_CheckFileArrived
THOST_FTDC_OS_CheckDetail = _thosttraderapi.THOST_FTDC_OS_CheckDetail
THOST_FTDC_OS_DayEndClean = _thosttraderapi.THOST_FTDC_OS_DayEndClean
THOST_FTDC_OS_Invalid = _thosttraderapi.THOST_FTDC_OS_Invalid
THOST_FTDC_CCBFM_ByAmount = _thosttraderapi.THOST_FTDC_CCBFM_ByAmount
THOST_FTDC_CCBFM_ByMonth = _thosttraderapi.THOST_FTDC_CCBFM_ByMonth
THOST_FTDC_CAPIT_Client = _thosttraderapi.THOST_FTDC_CAPIT_Client
THOST_FTDC_CAPIT_Server = _thosttraderapi.THOST_FTDC_CAPIT_Server
THOST_FTDC_CAPIT_UserApi = _thosttraderapi.THOST_FTDC_CAPIT_UserApi
THOST_FTDC_LS_Connected = _thosttraderapi.THOST_FTDC_LS_Connected
THOST_FTDC_LS_Disconnected = _thosttraderapi.THOST_FTDC_LS_Disconnected
THOST_FTDC_BPWDF_NoCheck = _thosttraderapi.THOST_FTDC_BPWDF_NoCheck
THOST_FTDC_BPWDF_BlankCheck = _thosttraderapi.THOST_FTDC_BPWDF_BlankCheck
THOST_FTDC_BPWDF_EncryptCheck = _thosttraderapi.THOST_FTDC_BPWDF_EncryptCheck
THOST_FTDC_SAT_AccountID = _thosttraderapi.THOST_FTDC_SAT_AccountID
THOST_FTDC_SAT_CardID = _thosttraderapi.THOST_FTDC_SAT_CardID
THOST_FTDC_SAT_SHStockholderID = _thosttraderapi.THOST_FTDC_SAT_SHStockholderID
THOST_FTDC_SAT_SZStockholderID = _thosttraderapi.THOST_FTDC_SAT_SZStockholderID
THOST_FTDC_TRFS_Normal = _thosttraderapi.THOST_FTDC_TRFS_Normal
THOST_FTDC_TRFS_Repealed = _thosttraderapi.THOST_FTDC_TRFS_Repealed
THOST_FTDC_SPTYPE_Broker = _thosttraderapi.THOST_FTDC_SPTYPE_Broker
THOST_FTDC_SPTYPE_Bank = _thosttraderapi.THOST_FTDC_SPTYPE_Bank
THOST_FTDC_REQRSP_Request = _thosttraderapi.THOST_FTDC_REQRSP_Request
THOST_FTDC_REQRSP_Response = _thosttraderapi.THOST_FTDC_REQRSP_Response
THOST_FTDC_FBTUET_SignIn = _thosttraderapi.THOST_FTDC_FBTUET_SignIn
THOST_FTDC_FBTUET_FromBankToFuture = _thosttraderapi.THOST_FTDC_FBTUET_FromBankToFuture
THOST_FTDC_FBTUET_FromFutureToBank = _thosttraderapi.THOST_FTDC_FBTUET_FromFutureToBank
THOST_FTDC_FBTUET_OpenAccount = _thosttraderapi.THOST_FTDC_FBTUET_OpenAccount
THOST_FTDC_FBTUET_CancelAccount = _thosttraderapi.THOST_FTDC_FBTUET_CancelAccount
THOST_FTDC_FBTUET_ChangeAccount = _thosttraderapi.THOST_FTDC_FBTUET_ChangeAccount
THOST_FTDC_FBTUET_RepealFromBankToFuture = _thosttraderapi.THOST_FTDC_FBTUET_RepealFromBankToFuture
THOST_FTDC_FBTUET_RepealFromFutureToBank = _thosttraderapi.THOST_FTDC_FBTUET_RepealFromFutureToBank
THOST_FTDC_FBTUET_QueryBankAccount = _thosttraderapi.THOST_FTDC_FBTUET_QueryBankAccount
THOST_FTDC_FBTUET_QueryFutureAccount = _thosttraderapi.THOST_FTDC_FBTUET_QueryFutureAccount
THOST_FTDC_FBTUET_SignOut = _thosttraderapi.THOST_FTDC_FBTUET_SignOut
THOST_FTDC_FBTUET_SyncKey = _thosttraderapi.THOST_FTDC_FBTUET_SyncKey
THOST_FTDC_FBTUET_ReserveOpenAccount = _thosttraderapi.THOST_FTDC_FBTUET_ReserveOpenAccount
THOST_FTDC_FBTUET_CancelReserveOpenAccount = _thosttraderapi.THOST_FTDC_FBTUET_CancelReserveOpenAccount
THOST_FTDC_FBTUET_ReserveOpenAccountConfirm = _thosttraderapi.THOST_FTDC_FBTUET_ReserveOpenAccountConfirm
THOST_FTDC_FBTUET_Other = _thosttraderapi.THOST_FTDC_FBTUET_Other
THOST_FTDC_DBOP_Insert = _thosttraderapi.THOST_FTDC_DBOP_Insert
THOST_FTDC_DBOP_Update = _thosttraderapi.THOST_FTDC_DBOP_Update
THOST_FTDC_DBOP_Delete = _thosttraderapi.THOST_FTDC_DBOP_Delete
THOST_FTDC_SYNF_Yes = _thosttraderapi.THOST_FTDC_SYNF_Yes
THOST_FTDC_SYNF_No = _thosttraderapi.THOST_FTDC_SYNF_No
THOST_FTDC_SYNT_OneOffSync = _thosttraderapi.THOST_FTDC_SYNT_OneOffSync
THOST_FTDC_SYNT_TimerSync = _thosttraderapi.THOST_FTDC_SYNT_TimerSync
THOST_FTDC_SYNT_TimerFullSync = _thosttraderapi.THOST_FTDC_SYNT_TimerFullSync
THOST_FTDC_FBEDIR_Settlement = _thosttraderapi.THOST_FTDC_FBEDIR_Settlement
THOST_FTDC_FBEDIR_Sale = _thosttraderapi.THOST_FTDC_FBEDIR_Sale
THOST_FTDC_FBERES_Success = _thosttraderapi.THOST_FTDC_FBERES_Success
THOST_FTDC_FBERES_InsufficientBalance = _thosttraderapi.THOST_FTDC_FBERES_InsufficientBalance
THOST_FTDC_FBERES_UnknownTrading = _thosttraderapi.THOST_FTDC_FBERES_UnknownTrading
THOST_FTDC_FBERES_Fail = _thosttraderapi.THOST_FTDC_FBERES_Fail
THOST_FTDC_FBEES_Normal = _thosttraderapi.THOST_FTDC_FBEES_Normal
THOST_FTDC_FBEES_ReExchange = _thosttraderapi.THOST_FTDC_FBEES_ReExchange
THOST_FTDC_FBEFG_DataPackage = _thosttraderapi.THOST_FTDC_FBEFG_DataPackage
THOST_FTDC_FBEFG_File = _thosttraderapi.THOST_FTDC_FBEFG_File
THOST_FTDC_FBEAT_NotTrade = _thosttraderapi.THOST_FTDC_FBEAT_NotTrade
THOST_FTDC_FBEAT_Trade = _thosttraderapi.THOST_FTDC_FBEAT_Trade
THOST_FTDC_FBEUET_SignIn = _thosttraderapi.THOST_FTDC_FBEUET_SignIn
THOST_FTDC_FBEUET_Exchange = _thosttraderapi.THOST_FTDC_FBEUET_Exchange
THOST_FTDC_FBEUET_ReExchange = _thosttraderapi.THOST_FTDC_FBEUET_ReExchange
THOST_FTDC_FBEUET_QueryBankAccount = _thosttraderapi.THOST_FTDC_FBEUET_QueryBankAccount
THOST_FTDC_FBEUET_QueryExchDetial = _thosttraderapi.THOST_FTDC_FBEUET_QueryExchDetial
THOST_FTDC_FBEUET_QueryExchSummary = _thosttraderapi.THOST_FTDC_FBEUET_QueryExchSummary
THOST_FTDC_FBEUET_QueryExchRate = _thosttraderapi.THOST_FTDC_FBEUET_QueryExchRate
THOST_FTDC_FBEUET_CheckBankAccount = _thosttraderapi.THOST_FTDC_FBEUET_CheckBankAccount
THOST_FTDC_FBEUET_SignOut = _thosttraderapi.THOST_FTDC_FBEUET_SignOut
THOST_FTDC_FBEUET_Other = _thosttraderapi.THOST_FTDC_FBEUET_Other
THOST_FTDC_FBERF_UnProcessed = _thosttraderapi.THOST_FTDC_FBERF_UnProcessed
THOST_FTDC_FBERF_WaitSend = _thosttraderapi.THOST_FTDC_FBERF_WaitSend
THOST_FTDC_FBERF_SendSuccess = _thosttraderapi.THOST_FTDC_FBERF_SendSuccess
THOST_FTDC_FBERF_SendFailed = _thosttraderapi.THOST_FTDC_FBERF_SendFailed
THOST_FTDC_FBERF_WaitReSend = _thosttraderapi.THOST_FTDC_FBERF_WaitReSend
THOST_FTDC_NC_NOERROR = _thosttraderapi.THOST_FTDC_NC_NOERROR
THOST_FTDC_NC_Warn = _thosttraderapi.THOST_FTDC_NC_Warn
THOST_FTDC_NC_Call = _thosttraderapi.THOST_FTDC_NC_Call
THOST_FTDC_NC_Force = _thosttraderapi.THOST_FTDC_NC_Force
THOST_FTDC_NC_CHUANCANG = _thosttraderapi.THOST_FTDC_NC_CHUANCANG
THOST_FTDC_NC_Exception = _thosttraderapi.THOST_FTDC_NC_Exception
THOST_FTDC_FCT_Manual = _thosttraderapi.THOST_FTDC_FCT_Manual
THOST_FTDC_FCT_Single = _thosttraderapi.THOST_FTDC_FCT_Single
THOST_FTDC_FCT_Group = _thosttraderapi.THOST_FTDC_FCT_Group
THOST_FTDC_RNM_System = _thosttraderapi.THOST_FTDC_RNM_System
THOST_FTDC_RNM_SMS = _thosttraderapi.THOST_FTDC_RNM_SMS
THOST_FTDC_RNM_EMail = _thosttraderapi.THOST_FTDC_RNM_EMail
THOST_FTDC_RNM_Manual = _thosttraderapi.THOST_FTDC_RNM_Manual
THOST_FTDC_RNS_NotGen = _thosttraderapi.THOST_FTDC_RNS_NotGen
THOST_FTDC_RNS_Generated = _thosttraderapi.THOST_FTDC_RNS_Generated
THOST_FTDC_RNS_SendError = _thosttraderapi.THOST_FTDC_RNS_SendError
THOST_FTDC_RNS_SendOk = _thosttraderapi.THOST_FTDC_RNS_SendOk
THOST_FTDC_RNS_Received = _thosttraderapi.THOST_FTDC_RNS_Received
THOST_FTDC_RNS_Confirmed = _thosttraderapi.THOST_FTDC_RNS_Confirmed
THOST_FTDC_RUE_ExportData = _thosttraderapi.THOST_FTDC_RUE_ExportData
THOST_FTDC_COST_LastPriceAsc = _thosttraderapi.THOST_FTDC_COST_LastPriceAsc
THOST_FTDC_COST_LastPriceDesc = _thosttraderapi.THOST_FTDC_COST_LastPriceDesc
THOST_FTDC_COST_AskPriceAsc = _thosttraderapi.THOST_FTDC_COST_AskPriceAsc
THOST_FTDC_COST_AskPriceDesc = _thosttraderapi.THOST_FTDC_COST_AskPriceDesc
THOST_FTDC_COST_BidPriceAsc = _thosttraderapi.THOST_FTDC_COST_BidPriceAsc
THOST_FTDC_COST_BidPriceDesc = _thosttraderapi.THOST_FTDC_COST_BidPriceDesc
THOST_FTDC_UOAST_NoSend = _thosttraderapi.THOST_FTDC_UOAST_NoSend
THOST_FTDC_UOAST_Sended = _thosttraderapi.THOST_FTDC_UOAST_Sended
THOST_FTDC_UOAST_Generated = _thosttraderapi.THOST_FTDC_UOAST_Generated
THOST_FTDC_UOAST_SendFail = _thosttraderapi.THOST_FTDC_UOAST_SendFail
THOST_FTDC_UOAST_Success = _thosttraderapi.THOST_FTDC_UOAST_Success
THOST_FTDC_UOAST_Fail = _thosttraderapi.THOST_FTDC_UOAST_Fail
THOST_FTDC_UOAST_Cancel = _thosttraderapi.THOST_FTDC_UOAST_Cancel
THOST_FTDC_UOACS_NoApply = _thosttraderapi.THOST_FTDC_UOACS_NoApply
THOST_FTDC_UOACS_Submited = _thosttraderapi.THOST_FTDC_UOACS_Submited
THOST_FTDC_UOACS_Sended = _thosttraderapi.THOST_FTDC_UOACS_Sended
THOST_FTDC_UOACS_Success = _thosttraderapi.THOST_FTDC_UOACS_Success
THOST_FTDC_UOACS_Refuse = _thosttraderapi.THOST_FTDC_UOACS_Refuse
THOST_FTDC_UOACS_Cancel = _thosttraderapi.THOST_FTDC_UOACS_Cancel
THOST_FTDC_QT_Radio = _thosttraderapi.THOST_FTDC_QT_Radio
THOST_FTDC_QT_Option = _thosttraderapi.THOST_FTDC_QT_Option
THOST_FTDC_QT_Blank = _thosttraderapi.THOST_FTDC_QT_Blank
THOST_FTDC_BT_Request = _thosttraderapi.THOST_FTDC_BT_Request
THOST_FTDC_BT_Response = _thosttraderapi.THOST_FTDC_BT_Response
THOST_FTDC_BT_Notice = _thosttraderapi.THOST_FTDC_BT_Notice
THOST_FTDC_CRC_Success = _thosttraderapi.THOST_FTDC_CRC_Success
THOST_FTDC_CRC_Working = _thosttraderapi.THOST_FTDC_CRC_Working
THOST_FTDC_CRC_InfoFail = _thosttraderapi.THOST_FTDC_CRC_InfoFail
THOST_FTDC_CRC_IDCardFail = _thosttraderapi.THOST_FTDC_CRC_IDCardFail
THOST_FTDC_CRC_OtherFail = _thosttraderapi.THOST_FTDC_CRC_OtherFail
THOST_FTDC_CfMMCCT_All = _thosttraderapi.THOST_FTDC_CfMMCCT_All
THOST_FTDC_CfMMCCT_Person = _thosttraderapi.THOST_FTDC_CfMMCCT_Person
THOST_FTDC_CfMMCCT_Company = _thosttraderapi.THOST_FTDC_CfMMCCT_Company
THOST_FTDC_CfMMCCT_Other = _thosttraderapi.THOST_FTDC_CfMMCCT_Other
THOST_FTDC_CfMMCCT_SpecialOrgan = _thosttraderapi.THOST_FTDC_CfMMCCT_SpecialOrgan
THOST_FTDC_CfMMCCT_Asset = _thosttraderapi.THOST_FTDC_CfMMCCT_Asset
THOST_FTDC_EIDT_SHFE = _thosttraderapi.THOST_FTDC_EIDT_SHFE
THOST_FTDC_EIDT_CZCE = _thosttraderapi.THOST_FTDC_EIDT_CZCE
THOST_FTDC_EIDT_DCE = _thosttraderapi.THOST_FTDC_EIDT_DCE
THOST_FTDC_EIDT_CFFEX = _thosttraderapi.THOST_FTDC_EIDT_CFFEX
THOST_FTDC_EIDT_INE = _thosttraderapi.THOST_FTDC_EIDT_INE
THOST_FTDC_ECIDT_Hedge = _thosttraderapi.THOST_FTDC_ECIDT_Hedge
THOST_FTDC_ECIDT_Arbitrage = _thosttraderapi.THOST_FTDC_ECIDT_Arbitrage
THOST_FTDC_ECIDT_Speculation = _thosttraderapi.THOST_FTDC_ECIDT_Speculation
THOST_FTDC_UF_NoUpdate = _thosttraderapi.THOST_FTDC_UF_NoUpdate
THOST_FTDC_UF_Success = _thosttraderapi.THOST_FTDC_UF_Success
THOST_FTDC_UF_Fail = _thosttraderapi.THOST_FTDC_UF_Fail
THOST_FTDC_UF_TCSuccess = _thosttraderapi.THOST_FTDC_UF_TCSuccess
THOST_FTDC_UF_TCFail = _thosttraderapi.THOST_FTDC_UF_TCFail
THOST_FTDC_UF_Cancel = _thosttraderapi.THOST_FTDC_UF_Cancel
THOST_FTDC_AOID_OpenInvestor = _thosttraderapi.THOST_FTDC_AOID_OpenInvestor
THOST_FTDC_AOID_ModifyIDCard = _thosttraderapi.THOST_FTDC_AOID_ModifyIDCard
THOST_FTDC_AOID_ModifyNoIDCard = _thosttraderapi.THOST_FTDC_AOID_ModifyNoIDCard
THOST_FTDC_AOID_ApplyTradingCode = _thosttraderapi.THOST_FTDC_AOID_ApplyTradingCode
THOST_FTDC_AOID_CancelTradingCode = _thosttraderapi.THOST_FTDC_AOID_CancelTradingCode
THOST_FTDC_AOID_CancelInvestor = _thosttraderapi.THOST_FTDC_AOID_CancelInvestor
THOST_FTDC_AOID_FreezeAccount = _thosttraderapi.THOST_FTDC_AOID_FreezeAccount
THOST_FTDC_AOID_ActiveFreezeAccount = _thosttraderapi.THOST_FTDC_AOID_ActiveFreezeAccount
THOST_FTDC_ASID_NoComplete = _thosttraderapi.THOST_FTDC_ASID_NoComplete
THOST_FTDC_ASID_Submited = _thosttraderapi.THOST_FTDC_ASID_Submited
THOST_FTDC_ASID_Checked = _thosttraderapi.THOST_FTDC_ASID_Checked
THOST_FTDC_ASID_Refused = _thosttraderapi.THOST_FTDC_ASID_Refused
THOST_FTDC_ASID_Deleted = _thosttraderapi.THOST_FTDC_ASID_Deleted
THOST_FTDC_UOASM_ByAPI = _thosttraderapi.THOST_FTDC_UOASM_ByAPI
THOST_FTDC_UOASM_ByFile = _thosttraderapi.THOST_FTDC_UOASM_ByFile
THOST_FTDC_EvM_ADD = _thosttraderapi.THOST_FTDC_EvM_ADD
THOST_FTDC_EvM_UPDATE = _thosttraderapi.THOST_FTDC_EvM_UPDATE
THOST_FTDC_EvM_DELETE = _thosttraderapi.THOST_FTDC_EvM_DELETE
THOST_FTDC_EvM_CHECK = _thosttraderapi.THOST_FTDC_EvM_CHECK
THOST_FTDC_EvM_COPY = _thosttraderapi.THOST_FTDC_EvM_COPY
THOST_FTDC_EvM_CANCEL = _thosttraderapi.THOST_FTDC_EvM_CANCEL
THOST_FTDC_EvM_Reverse = _thosttraderapi.THOST_FTDC_EvM_Reverse
THOST_FTDC_UOAA_ASR = _thosttraderapi.THOST_FTDC_UOAA_ASR
THOST_FTDC_UOAA_ASNR = _thosttraderapi.THOST_FTDC_UOAA_ASNR
THOST_FTDC_UOAA_NSAR = _thosttraderapi.THOST_FTDC_UOAA_NSAR
THOST_FTDC_UOAA_NSR = _thosttraderapi.THOST_FTDC_UOAA_NSR
THOST_FTDC_EvM_InvestorGroupFlow = _thosttraderapi.THOST_FTDC_EvM_InvestorGroupFlow
THOST_FTDC_EvM_InvestorRate = _thosttraderapi.THOST_FTDC_EvM_InvestorRate
THOST_FTDC_EvM_InvestorCommRateModel = _thosttraderapi.THOST_FTDC_EvM_InvestorCommRateModel
THOST_FTDC_CL_Zero = _thosttraderapi.THOST_FTDC_CL_Zero
THOST_FTDC_CL_One = _thosttraderapi.THOST_FTDC_CL_One
THOST_FTDC_CL_Two = _thosttraderapi.THOST_FTDC_CL_Two
THOST_FTDC_CHS_Init = _thosttraderapi.THOST_FTDC_CHS_Init
THOST_FTDC_CHS_Checking = _thosttraderapi.THOST_FTDC_CHS_Checking
THOST_FTDC_CHS_Checked = _thosttraderapi.THOST_FTDC_CHS_Checked
THOST_FTDC_CHS_Refuse = _thosttraderapi.THOST_FTDC_CHS_Refuse
THOST_FTDC_CHS_Cancel = _thosttraderapi.THOST_FTDC_CHS_Cancel
THOST_FTDC_CHU_Unused = _thosttraderapi.THOST_FTDC_CHU_Unused
THOST_FTDC_CHU_Used = _thosttraderapi.THOST_FTDC_CHU_Used
THOST_FTDC_CHU_Fail = _thosttraderapi.THOST_FTDC_CHU_Fail
THOST_FTDC_BAO_ByAccProperty = _thosttraderapi.THOST_FTDC_BAO_ByAccProperty
THOST_FTDC_BAO_ByFBTransfer = _thosttraderapi.THOST_FTDC_BAO_ByFBTransfer
THOST_FTDC_MBTS_ByInstrument = _thosttraderapi.THOST_FTDC_MBTS_ByInstrument
THOST_FTDC_MBTS_ByDayInsPrc = _thosttraderapi.THOST_FTDC_MBTS_ByDayInsPrc
THOST_FTDC_MBTS_ByDayIns = _thosttraderapi.THOST_FTDC_MBTS_ByDayIns
THOST_FTDC_OTP_NONE = _thosttraderapi.THOST_FTDC_OTP_NONE
THOST_FTDC_OTP_TOTP = _thosttraderapi.THOST_FTDC_OTP_TOTP
THOST_FTDC_OTPS_Unused = _thosttraderapi.THOST_FTDC_OTPS_Unused
THOST_FTDC_OTPS_Used = _thosttraderapi.THOST_FTDC_OTPS_Used
THOST_FTDC_OTPS_Disuse = _thosttraderapi.THOST_FTDC_OTPS_Disuse
THOST_FTDC_BUT_Investor = _thosttraderapi.THOST_FTDC_BUT_Investor
THOST_FTDC_BUT_BrokerUser = _thosttraderapi.THOST_FTDC_BUT_BrokerUser
THOST_FTDC_FUTT_Commodity = _thosttraderapi.THOST_FTDC_FUTT_Commodity
THOST_FTDC_FUTT_Financial = _thosttraderapi.THOST_FTDC_FUTT_Financial
THOST_FTDC_FET_Restriction = _thosttraderapi.THOST_FTDC_FET_Restriction
THOST_FTDC_FET_TodayRestriction = _thosttraderapi.THOST_FTDC_FET_TodayRestriction
THOST_FTDC_FET_Transfer = _thosttraderapi.THOST_FTDC_FET_Transfer
THOST_FTDC_FET_Credit = _thosttraderapi.THOST_FTDC_FET_Credit
THOST_FTDC_FET_InvestorWithdrawAlm = _thosttraderapi.THOST_FTDC_FET_InvestorWithdrawAlm
THOST_FTDC_FET_BankRestriction = _thosttraderapi.THOST_FTDC_FET_BankRestriction
THOST_FTDC_FET_Accountregister = _thosttraderapi.THOST_FTDC_FET_Accountregister
THOST_FTDC_FET_ExchangeFundIO = _thosttraderapi.THOST_FTDC_FET_ExchangeFundIO
THOST_FTDC_FET_InvestorFundIO = _thosttraderapi.THOST_FTDC_FET_InvestorFundIO
THOST_FTDC_AST_FBTransfer = _thosttraderapi.THOST_FTDC_AST_FBTransfer
THOST_FTDC_AST_ManualEntry = _thosttraderapi.THOST_FTDC_AST_ManualEntry
THOST_FTDC_CST_UnifyAccount = _thosttraderapi.THOST_FTDC_CST_UnifyAccount
THOST_FTDC_CST_ManualEntry = _thosttraderapi.THOST_FTDC_CST_ManualEntry
THOST_FTDC_UR_All = _thosttraderapi.THOST_FTDC_UR_All
THOST_FTDC_UR_Single = _thosttraderapi.THOST_FTDC_UR_Single
THOST_FTDC_BG_Investor = _thosttraderapi.THOST_FTDC_BG_Investor
THOST_FTDC_BG_Group = _thosttraderapi.THOST_FTDC_BG_Group
THOST_FTDC_TSSM_Instrument = _thosttraderapi.THOST_FTDC_TSSM_Instrument
THOST_FTDC_TSSM_Product = _thosttraderapi.THOST_FTDC_TSSM_Product
THOST_FTDC_TSSM_Exchange = _thosttraderapi.THOST_FTDC_TSSM_Exchange
THOST_FTDC_ESM_Relative = _thosttraderapi.THOST_FTDC_ESM_Relative
THOST_FTDC_ESM_Typical = _thosttraderapi.THOST_FTDC_ESM_Typical
THOST_FTDC_RIR_All = _thosttraderapi.THOST_FTDC_RIR_All
THOST_FTDC_RIR_Model = _thosttraderapi.THOST_FTDC_RIR_Model
THOST_FTDC_RIR_Single = _thosttraderapi.THOST_FTDC_RIR_Single
THOST_FTDC_SDS_Initialize = _thosttraderapi.THOST_FTDC_SDS_Initialize
THOST_FTDC_SDS_Settlementing = _thosttraderapi.THOST_FTDC_SDS_Settlementing
THOST_FTDC_SDS_Settlemented = _thosttraderapi.THOST_FTDC_SDS_Settlemented
THOST_FTDC_TSRC_NORMAL = _thosttraderapi.THOST_FTDC_TSRC_NORMAL
THOST_FTDC_TSRC_QUERY = _thosttraderapi.THOST_FTDC_TSRC_QUERY
THOST_FTDC_FSM_Product = _thosttraderapi.THOST_FTDC_FSM_Product
THOST_FTDC_FSM_Exchange = _thosttraderapi.THOST_FTDC_FSM_Exchange
THOST_FTDC_FSM_All = _thosttraderapi.THOST_FTDC_FSM_All
THOST_FTDC_BIR_Property = _thosttraderapi.THOST_FTDC_BIR_Property
THOST_FTDC_BIR_All = _thosttraderapi.THOST_FTDC_BIR_All
THOST_FTDC_PIR_All = _thosttraderapi.THOST_FTDC_PIR_All
THOST_FTDC_PIR_Property = _thosttraderapi.THOST_FTDC_PIR_Property
THOST_FTDC_PIR_Single = _thosttraderapi.THOST_FTDC_PIR_Single
THOST_FTDC_FIS_NoCreate = _thosttraderapi.THOST_FTDC_FIS_NoCreate
THOST_FTDC_FIS_Created = _thosttraderapi.THOST_FTDC_FIS_Created
THOST_FTDC_FIS_Failed = _thosttraderapi.THOST_FTDC_FIS_Failed
THOST_FTDC_FGS_FileTransmit = _thosttraderapi.THOST_FTDC_FGS_FileTransmit
THOST_FTDC_FGS_FileGen = _thosttraderapi.THOST_FTDC_FGS_FileGen
THOST_FTDC_SoM_Add = _thosttraderapi.THOST_FTDC_SoM_Add
THOST_FTDC_SoM_Update = _thosttraderapi.THOST_FTDC_SoM_Update
THOST_FTDC_SoM_Delete = _thosttraderapi.THOST_FTDC_SoM_Delete
THOST_FTDC_SoM_Copy = _thosttraderapi.THOST_FTDC_SoM_Copy
THOST_FTDC_SoM_AcTive = _thosttraderapi.THOST_FTDC_SoM_AcTive
THOST_FTDC_SoM_CanCel = _thosttraderapi.THOST_FTDC_SoM_CanCel
THOST_FTDC_SoM_ReSet = _thosttraderapi.THOST_FTDC_SoM_ReSet
THOST_FTDC_SoT_UpdatePassword = _thosttraderapi.THOST_FTDC_SoT_UpdatePassword
THOST_FTDC_SoT_UserDepartment = _thosttraderapi.THOST_FTDC_SoT_UserDepartment
THOST_FTDC_SoT_RoleManager = _thosttraderapi.THOST_FTDC_SoT_RoleManager
THOST_FTDC_SoT_RoleFunction = _thosttraderapi.THOST_FTDC_SoT_RoleFunction
THOST_FTDC_SoT_BaseParam = _thosttraderapi.THOST_FTDC_SoT_BaseParam
THOST_FTDC_SoT_SetUserID = _thosttraderapi.THOST_FTDC_SoT_SetUserID
THOST_FTDC_SoT_SetUserRole = _thosttraderapi.THOST_FTDC_SoT_SetUserRole
THOST_FTDC_SoT_UserIpRestriction = _thosttraderapi.THOST_FTDC_SoT_UserIpRestriction
THOST_FTDC_SoT_DepartmentManager = _thosttraderapi.THOST_FTDC_SoT_DepartmentManager
THOST_FTDC_SoT_DepartmentCopy = _thosttraderapi.THOST_FTDC_SoT_DepartmentCopy
THOST_FTDC_SoT_Tradingcode = _thosttraderapi.THOST_FTDC_SoT_Tradingcode
THOST_FTDC_SoT_InvestorStatus = _thosttraderapi.THOST_FTDC_SoT_InvestorStatus
THOST_FTDC_SoT_InvestorAuthority = _thosttraderapi.THOST_FTDC_SoT_InvestorAuthority
THOST_FTDC_SoT_PropertySet = _thosttraderapi.THOST_FTDC_SoT_PropertySet
THOST_FTDC_SoT_ReSetInvestorPasswd = _thosttraderapi.THOST_FTDC_SoT_ReSetInvestorPasswd
THOST_FTDC_SoT_InvestorPersonalityInfo = _thosttraderapi.THOST_FTDC_SoT_InvestorPersonalityInfo
THOST_FTDC_CSRCQ_Current = _thosttraderapi.THOST_FTDC_CSRCQ_Current
THOST_FTDC_CSRCQ_History = _thosttraderapi.THOST_FTDC_CSRCQ_History
THOST_FTDC_FRS_Normal = _thosttraderapi.THOST_FTDC_FRS_Normal
THOST_FTDC_FRS_Freeze = _thosttraderapi.THOST_FTDC_FRS_Freeze
THOST_FTDC_STST_Standard = _thosttraderapi.THOST_FTDC_STST_Standard
THOST_FTDC_STST_NonStandard = _thosttraderapi.THOST_FTDC_STST_NonStandard
THOST_FTDC_RPT_Freeze = _thosttraderapi.THOST_FTDC_RPT_Freeze
THOST_FTDC_RPT_FreezeActive = _thosttraderapi.THOST_FTDC_RPT_FreezeActive
THOST_FTDC_RPT_OpenLimit = _thosttraderapi.THOST_FTDC_RPT_OpenLimit
THOST_FTDC_RPT_RelieveOpenLimit = _thosttraderapi.THOST_FTDC_RPT_RelieveOpenLimit
THOST_FTDC_AMLDS_Normal = _thosttraderapi.THOST_FTDC_AMLDS_Normal
THOST_FTDC_AMLDS_Deleted = _thosttraderapi.THOST_FTDC_AMLDS_Deleted
THOST_FTDC_AMLCHS_Init = _thosttraderapi.THOST_FTDC_AMLCHS_Init
THOST_FTDC_AMLCHS_Checking = _thosttraderapi.THOST_FTDC_AMLCHS_Checking
THOST_FTDC_AMLCHS_Checked = _thosttraderapi.THOST_FTDC_AMLCHS_Checked
THOST_FTDC_AMLCHS_RefuseReport = _thosttraderapi.THOST_FTDC_AMLCHS_RefuseReport
THOST_FTDC_AMLDT_DrawDay = _thosttraderapi.THOST_FTDC_AMLDT_DrawDay
THOST_FTDC_AMLDT_TouchDay = _thosttraderapi.THOST_FTDC_AMLDT_TouchDay
THOST_FTDC_AMLCL_CheckLevel0 = _thosttraderapi.THOST_FTDC_AMLCL_CheckLevel0
THOST_FTDC_AMLCL_CheckLevel1 = _thosttraderapi.THOST_FTDC_AMLCL_CheckLevel1
THOST_FTDC_AMLCL_CheckLevel2 = _thosttraderapi.THOST_FTDC_AMLCL_CheckLevel2
THOST_FTDC_AMLCL_CheckLevel3 = _thosttraderapi.THOST_FTDC_AMLCL_CheckLevel3
THOST_FTDC_EFT_CSV = _thosttraderapi.THOST_FTDC_EFT_CSV
THOST_FTDC_EFT_EXCEL = _thosttraderapi.THOST_FTDC_EFT_EXCEL
THOST_FTDC_EFT_DBF = _thosttraderapi.THOST_FTDC_EFT_DBF
THOST_FTDC_SMT_Before = _thosttraderapi.THOST_FTDC_SMT_Before
THOST_FTDC_SMT_Settlement = _thosttraderapi.THOST_FTDC_SMT_Settlement
THOST_FTDC_SMT_After = _thosttraderapi.THOST_FTDC_SMT_After
THOST_FTDC_SMT_Settlemented = _thosttraderapi.THOST_FTDC_SMT_Settlemented
THOST_FTDC_SML_Must = _thosttraderapi.THOST_FTDC_SML_Must
THOST_FTDC_SML_Alarm = _thosttraderapi.THOST_FTDC_SML_Alarm
THOST_FTDC_SML_Prompt = _thosttraderapi.THOST_FTDC_SML_Prompt
THOST_FTDC_SML_Ignore = _thosttraderapi.THOST_FTDC_SML_Ignore
THOST_FTDC_SMG_Exhcange = _thosttraderapi.THOST_FTDC_SMG_Exhcange
THOST_FTDC_SMG_ASP = _thosttraderapi.THOST_FTDC_SMG_ASP
THOST_FTDC_SMG_CSRC = _thosttraderapi.THOST_FTDC_SMG_CSRC
THOST_FTDC_LUT_Repeatable = _thosttraderapi.THOST_FTDC_LUT_Repeatable
THOST_FTDC_LUT_Unrepeatable = _thosttraderapi.THOST_FTDC_LUT_Unrepeatable
THOST_FTDC_DAR_Settle = _thosttraderapi.THOST_FTDC_DAR_Settle
THOST_FTDC_DAR_Exchange = _thosttraderapi.THOST_FTDC_DAR_Exchange
THOST_FTDC_DAR_CSRC = _thosttraderapi.THOST_FTDC_DAR_CSRC
THOST_FTDC_MGT_ExchMarginRate = _thosttraderapi.THOST_FTDC_MGT_ExchMarginRate
THOST_FTDC_MGT_InstrMarginRate = _thosttraderapi.THOST_FTDC_MGT_InstrMarginRate
THOST_FTDC_MGT_InstrMarginRateTrade = _thosttraderapi.THOST_FTDC_MGT_InstrMarginRateTrade
THOST_FTDC_ACT_Intraday = _thosttraderapi.THOST_FTDC_ACT_Intraday
THOST_FTDC_ACT_Long = _thosttraderapi.THOST_FTDC_ACT_Long
THOST_FTDC_MRT_Exchange = _thosttraderapi.THOST_FTDC_MRT_Exchange
THOST_FTDC_MRT_Investor = _thosttraderapi.THOST_FTDC_MRT_Investor
THOST_FTDC_MRT_InvestorTrade = _thosttraderapi.THOST_FTDC_MRT_InvestorTrade
THOST_FTDC_BUS_UnBak = _thosttraderapi.THOST_FTDC_BUS_UnBak
THOST_FTDC_BUS_BakUp = _thosttraderapi.THOST_FTDC_BUS_BakUp
THOST_FTDC_BUS_BakUped = _thosttraderapi.THOST_FTDC_BUS_BakUped
THOST_FTDC_BUS_BakFail = _thosttraderapi.THOST_FTDC_BUS_BakFail
THOST_FTDC_SIS_UnInitialize = _thosttraderapi.THOST_FTDC_SIS_UnInitialize
THOST_FTDC_SIS_Initialize = _thosttraderapi.THOST_FTDC_SIS_Initialize
THOST_FTDC_SIS_Initialized = _thosttraderapi.THOST_FTDC_SIS_Initialized
THOST_FTDC_SRS_NoCreate = _thosttraderapi.THOST_FTDC_SRS_NoCreate
THOST_FTDC_SRS_Create = _thosttraderapi.THOST_FTDC_SRS_Create
THOST_FTDC_SRS_Created = _thosttraderapi.THOST_FTDC_SRS_Created
THOST_FTDC_SRS_CreateFail = _thosttraderapi.THOST_FTDC_SRS_CreateFail
THOST_FTDC_SSS_UnSaveData = _thosttraderapi.THOST_FTDC_SSS_UnSaveData
THOST_FTDC_SSS_SaveDatad = _thosttraderapi.THOST_FTDC_SSS_SaveDatad
THOST_FTDC_SAS_UnArchived = _thosttraderapi.THOST_FTDC_SAS_UnArchived
THOST_FTDC_SAS_Archiving = _thosttraderapi.THOST_FTDC_SAS_Archiving
THOST_FTDC_SAS_Archived = _thosttraderapi.THOST_FTDC_SAS_Archived
THOST_FTDC_SAS_ArchiveFail = _thosttraderapi.THOST_FTDC_SAS_ArchiveFail
THOST_FTDC_CTPT_Unkown = _thosttraderapi.THOST_FTDC_CTPT_Unkown
THOST_FTDC_CTPT_MainCenter = _thosttraderapi.THOST_FTDC_CTPT_MainCenter
THOST_FTDC_CTPT_BackUp = _thosttraderapi.THOST_FTDC_CTPT_BackUp
THOST_FTDC_CDT_Normal = _thosttraderapi.THOST_FTDC_CDT_Normal
THOST_FTDC_CDT_SpecFirst = _thosttraderapi.THOST_FTDC_CDT_SpecFirst
THOST_FTDC_MFUR_None = _thosttraderapi.THOST_FTDC_MFUR_None
THOST_FTDC_MFUR_Margin = _thosttraderapi.THOST_FTDC_MFUR_Margin
THOST_FTDC_MFUR_All = _thosttraderapi.THOST_FTDC_MFUR_All
THOST_FTDC_MFUR_CNY3 = _thosttraderapi.THOST_FTDC_MFUR_CNY3
THOST_FTDC_SPT_CzceHedge = _thosttraderapi.THOST_FTDC_SPT_CzceHedge
THOST_FTDC_SPT_IneForeignCurrency = _thosttraderapi.THOST_FTDC_SPT_IneForeignCurrency
THOST_FTDC_SPT_DceOpenClose = _thosttraderapi.THOST_FTDC_SPT_DceOpenClose
THOST_FTDC_FMT_Mortgage = _thosttraderapi.THOST_FTDC_FMT_Mortgage
THOST_FTDC_FMT_Redemption = _thosttraderapi.THOST_FTDC_FMT_Redemption
THOST_FTDC_ASPI_BaseMargin = _thosttraderapi.THOST_FTDC_ASPI_BaseMargin
THOST_FTDC_ASPI_LowestInterest = _thosttraderapi.THOST_FTDC_ASPI_LowestInterest
THOST_FTDC_FMD_In = _thosttraderapi.THOST_FTDC_FMD_In
THOST_FTDC_FMD_Out = _thosttraderapi.THOST_FTDC_FMD_Out
THOST_FTDC_BT_Profit = _thosttraderapi.THOST_FTDC_BT_Profit
THOST_FTDC_BT_Loss = _thosttraderapi.THOST_FTDC_BT_Loss
THOST_FTDC_BT_Other = _thosttraderapi.THOST_FTDC_BT_Other
THOST_FTDC_SST_Manual = _thosttraderapi.THOST_FTDC_SST_Manual
THOST_FTDC_SST_Automatic = _thosttraderapi.THOST_FTDC_SST_Automatic
THOST_FTDC_CED_Settlement = _thosttraderapi.THOST_FTDC_CED_Settlement
THOST_FTDC_CED_Sale = _thosttraderapi.THOST_FTDC_CED_Sale
THOST_FTDC_CSS_Entry = _thosttraderapi.THOST_FTDC_CSS_Entry
THOST_FTDC_CSS_Approve = _thosttraderapi.THOST_FTDC_CSS_Approve
THOST_FTDC_CSS_Refuse = _thosttraderapi.THOST_FTDC_CSS_Refuse
THOST_FTDC_CSS_Revoke = _thosttraderapi.THOST_FTDC_CSS_Revoke
THOST_FTDC_CSS_Send = _thosttraderapi.THOST_FTDC_CSS_Send
THOST_FTDC_CSS_Success = _thosttraderapi.THOST_FTDC_CSS_Success
THOST_FTDC_CSS_Failure = _thosttraderapi.THOST_FTDC_CSS_Failure
THOST_FTDC_REQF_NoSend = _thosttraderapi.THOST_FTDC_REQF_NoSend
THOST_FTDC_REQF_SendSuccess = _thosttraderapi.THOST_FTDC_REQF_SendSuccess
THOST_FTDC_REQF_SendFailed = _thosttraderapi.THOST_FTDC_REQF_SendFailed
THOST_FTDC_REQF_WaitReSend = _thosttraderapi.THOST_FTDC_REQF_WaitReSend
THOST_FTDC_RESF_Success = _thosttraderapi.THOST_FTDC_RESF_Success
THOST_FTDC_RESF_InsuffiCient = _thosttraderapi.THOST_FTDC_RESF_InsuffiCient
THOST_FTDC_RESF_UnKnown = _thosttraderapi.THOST_FTDC_RESF_UnKnown
THOST_FTDC_EXS_Before = _thosttraderapi.THOST_FTDC_EXS_Before
THOST_FTDC_EXS_After = _thosttraderapi.THOST_FTDC_EXS_After
THOST_FTDC_CR_Domestic = _thosttraderapi.THOST_FTDC_CR_Domestic
THOST_FTDC_CR_GMT = _thosttraderapi.THOST_FTDC_CR_GMT
THOST_FTDC_CR_Foreign = _thosttraderapi.THOST_FTDC_CR_Foreign
THOST_FTDC_HB_No = _thosttraderapi.THOST_FTDC_HB_No
THOST_FTDC_HB_Yes = _thosttraderapi.THOST_FTDC_HB_Yes
THOST_FTDC_SM_Normal = _thosttraderapi.THOST_FTDC_SM_Normal
THOST_FTDC_SM_Emerge = _thosttraderapi.THOST_FTDC_SM_Emerge
THOST_FTDC_SM_Restore = _thosttraderapi.THOST_FTDC_SM_Restore
THOST_FTDC_TPT_Full = _thosttraderapi.THOST_FTDC_TPT_Full
THOST_FTDC_TPT_Increment = _thosttraderapi.THOST_FTDC_TPT_Increment
THOST_FTDC_TPT_BackUp = _thosttraderapi.THOST_FTDC_TPT_BackUp
THOST_FTDC_LM_Trade = _thosttraderapi.THOST_FTDC_LM_Trade
THOST_FTDC_LM_Transfer = _thosttraderapi.THOST_FTDC_LM_Transfer
THOST_FTDC_CPT_Instrument = _thosttraderapi.THOST_FTDC_CPT_Instrument
THOST_FTDC_CPT_Margin = _thosttraderapi.THOST_FTDC_CPT_Margin
THOST_FTDC_HT_Yes = _thosttraderapi.THOST_FTDC_HT_Yes
THOST_FTDC_HT_No = _thosttraderapi.THOST_FTDC_HT_No
THOST_FTDC_AMT_Bank = _thosttraderapi.THOST_FTDC_AMT_Bank
THOST_FTDC_AMT_Securities = _thosttraderapi.THOST_FTDC_AMT_Securities
THOST_FTDC_AMT_Fund = _thosttraderapi.THOST_FTDC_AMT_Fund
THOST_FTDC_AMT_Insurance = _thosttraderapi.THOST_FTDC_AMT_Insurance
THOST_FTDC_AMT_Trust = _thosttraderapi.THOST_FTDC_AMT_Trust
THOST_FTDC_AMT_Other = _thosttraderapi.THOST_FTDC_AMT_Other
THOST_FTDC_CFIOT_FundIO = _thosttraderapi.THOST_FTDC_CFIOT_FundIO
THOST_FTDC_CFIOT_SwapCurrency = _thosttraderapi.THOST_FTDC_CFIOT_SwapCurrency
THOST_FTDC_CAT_Futures = _thosttraderapi.THOST_FTDC_CAT_Futures
THOST_FTDC_CAT_AssetmgrFuture = _thosttraderapi.THOST_FTDC_CAT_AssetmgrFuture
THOST_FTDC_CAT_AssetmgrTrustee = _thosttraderapi.THOST_FTDC_CAT_AssetmgrTrustee
THOST_FTDC_CAT_AssetmgrTransfer = _thosttraderapi.THOST_FTDC_CAT_AssetmgrTransfer
THOST_FTDC_LT_Chinese = _thosttraderapi.THOST_FTDC_LT_Chinese
THOST_FTDC_LT_English = _thosttraderapi.THOST_FTDC_LT_English
THOST_FTDC_AMCT_Person = _thosttraderapi.THOST_FTDC_AMCT_Person
THOST_FTDC_AMCT_Organ = _thosttraderapi.THOST_FTDC_AMCT_Organ
THOST_FTDC_AMCT_SpecialOrgan = _thosttraderapi.THOST_FTDC_AMCT_SpecialOrgan
THOST_FTDC_ASST_Futures = _thosttraderapi.THOST_FTDC_ASST_Futures
THOST_FTDC_ASST_SpecialOrgan = _thosttraderapi.THOST_FTDC_ASST_SpecialOrgan
THOST_FTDC_CIT_HasExch = _thosttraderapi.THOST_FTDC_CIT_HasExch
THOST_FTDC_CIT_HasATP = _thosttraderapi.THOST_FTDC_CIT_HasATP
THOST_FTDC_CIT_HasDiff = _thosttraderapi.THOST_FTDC_CIT_HasDiff
THOST_FTDC_DT_HandDeliv = _thosttraderapi.THOST_FTDC_DT_HandDeliv
THOST_FTDC_DT_PersonDeliv = _thosttraderapi.THOST_FTDC_DT_PersonDeliv
THOST_FTDC_MMSA_NO = _thosttraderapi.THOST_FTDC_MMSA_NO
THOST_FTDC_MMSA_YES = _thosttraderapi.THOST_FTDC_MMSA_YES
THOST_FTDC_CACT_Person = _thosttraderapi.THOST_FTDC_CACT_Person
THOST_FTDC_CACT_Company = _thosttraderapi.THOST_FTDC_CACT_Company
THOST_FTDC_CACT_Other = _thosttraderapi.THOST_FTDC_CACT_Other
THOST_FTDC_UOAAT_Futures = _thosttraderapi.THOST_FTDC_UOAAT_Futures
THOST_FTDC_UOAAT_SpecialOrgan = _thosttraderapi.THOST_FTDC_UOAAT_SpecialOrgan
THOST_FTDC_DEN_Buy = _thosttraderapi.THOST_FTDC_DEN_Buy
THOST_FTDC_DEN_Sell = _thosttraderapi.THOST_FTDC_DEN_Sell
THOST_FTDC_OFEN_Open = _thosttraderapi.THOST_FTDC_OFEN_Open
THOST_FTDC_OFEN_Close = _thosttraderapi.THOST_FTDC_OFEN_Close
THOST_FTDC_OFEN_ForceClose = _thosttraderapi.THOST_FTDC_OFEN_ForceClose
THOST_FTDC_OFEN_CloseToday = _thosttraderapi.THOST_FTDC_OFEN_CloseToday
THOST_FTDC_OFEN_CloseYesterday = _thosttraderapi.THOST_FTDC_OFEN_CloseYesterday
THOST_FTDC_OFEN_ForceOff = _thosttraderapi.THOST_FTDC_OFEN_ForceOff
THOST_FTDC_OFEN_LocalForceClose = _thosttraderapi.THOST_FTDC_OFEN_LocalForceClose
THOST_FTDC_HFEN_Speculation = _thosttraderapi.THOST_FTDC_HFEN_Speculation
THOST_FTDC_HFEN_Arbitrage = _thosttraderapi.THOST_FTDC_HFEN_Arbitrage
THOST_FTDC_HFEN_Hedge = _thosttraderapi.THOST_FTDC_HFEN_Hedge
THOST_FTDC_FIOTEN_FundIO = _thosttraderapi.THOST_FTDC_FIOTEN_FundIO
THOST_FTDC_FIOTEN_Transfer = _thosttraderapi.THOST_FTDC_FIOTEN_Transfer
THOST_FTDC_FIOTEN_SwapCurrency = _thosttraderapi.THOST_FTDC_FIOTEN_SwapCurrency
THOST_FTDC_FTEN_Deposite = _thosttraderapi.THOST_FTDC_FTEN_Deposite
THOST_FTDC_FTEN_ItemFund = _thosttraderapi.THOST_FTDC_FTEN_ItemFund
THOST_FTDC_FTEN_Company = _thosttraderapi.THOST_FTDC_FTEN_Company
THOST_FTDC_FTEN_InnerTransfer = _thosttraderapi.THOST_FTDC_FTEN_InnerTransfer
THOST_FTDC_FDEN_In = _thosttraderapi.THOST_FTDC_FDEN_In
THOST_FTDC_FDEN_Out = _thosttraderapi.THOST_FTDC_FDEN_Out
THOST_FTDC_FMDEN_In = _thosttraderapi.THOST_FTDC_FMDEN_In
THOST_FTDC_FMDEN_Out = _thosttraderapi.THOST_FTDC_FMDEN_Out
THOST_FTDC_CP_CallOptions = _thosttraderapi.THOST_FTDC_CP_CallOptions
THOST_FTDC_CP_PutOptions = _thosttraderapi.THOST_FTDC_CP_PutOptions
THOST_FTDC_STM_Continental = _thosttraderapi.THOST_FTDC_STM_Continental
THOST_FTDC_STM_American = _thosttraderapi.THOST_FTDC_STM_American
THOST_FTDC_STM_Bermuda = _thosttraderapi.THOST_FTDC_STM_Bermuda
THOST_FTDC_STT_Hedge = _thosttraderapi.THOST_FTDC_STT_Hedge
THOST_FTDC_STT_Match = _thosttraderapi.THOST_FTDC_STT_Match
THOST_FTDC_APPT_NotStrikeNum = _thosttraderapi.THOST_FTDC_APPT_NotStrikeNum
THOST_FTDC_GUDS_Gen = _thosttraderapi.THOST_FTDC_GUDS_Gen
THOST_FTDC_GUDS_Hand = _thosttraderapi.THOST_FTDC_GUDS_Hand
THOST_FTDC_OER_NoExec = _thosttraderapi.THOST_FTDC_OER_NoExec
THOST_FTDC_OER_Canceled = _thosttraderapi.THOST_FTDC_OER_Canceled
THOST_FTDC_OER_OK = _thosttraderapi.THOST_FTDC_OER_OK
THOST_FTDC_OER_NoPosition = _thosttraderapi.THOST_FTDC_OER_NoPosition
THOST_FTDC_OER_NoDeposit = _thosttraderapi.THOST_FTDC_OER_NoDeposit
THOST_FTDC_OER_NoParticipant = _thosttraderapi.THOST_FTDC_OER_NoParticipant
THOST_FTDC_OER_NoClient = _thosttraderapi.THOST_FTDC_OER_NoClient
THOST_FTDC_OER_NoInstrument = _thosttraderapi.THOST_FTDC_OER_NoInstrument
THOST_FTDC_OER_NoRight = _thosttraderapi.THOST_FTDC_OER_NoRight
THOST_FTDC_OER_InvalidVolume = _thosttraderapi.THOST_FTDC_OER_InvalidVolume
THOST_FTDC_OER_NoEnoughHistoryTrade = _thosttraderapi.THOST_FTDC_OER_NoEnoughHistoryTrade
THOST_FTDC_OER_Unknown = _thosttraderapi.THOST_FTDC_OER_Unknown
THOST_FTDC_COMBT_Future = _thosttraderapi.THOST_FTDC_COMBT_Future
THOST_FTDC_COMBT_BUL = _thosttraderapi.THOST_FTDC_COMBT_BUL
THOST_FTDC_COMBT_BER = _thosttraderapi.THOST_FTDC_COMBT_BER
THOST_FTDC_COMBT_STD = _thosttraderapi.THOST_FTDC_COMBT_STD
THOST_FTDC_COMBT_STG = _thosttraderapi.THOST_FTDC_COMBT_STG
THOST_FTDC_COMBT_PRT = _thosttraderapi.THOST_FTDC_COMBT_PRT
THOST_FTDC_COMBT_CLD = _thosttraderapi.THOST_FTDC_COMBT_CLD
THOST_FTDC_DCECOMBT_SPL = _thosttraderapi.THOST_FTDC_DCECOMBT_SPL
THOST_FTDC_DCECOMBT_OPL = _thosttraderapi.THOST_FTDC_DCECOMBT_OPL
THOST_FTDC_DCECOMBT_SP = _thosttraderapi.THOST_FTDC_DCECOMBT_SP
THOST_FTDC_DCECOMBT_SPC = _thosttraderapi.THOST_FTDC_DCECOMBT_SPC
THOST_FTDC_DCECOMBT_BLS = _thosttraderapi.THOST_FTDC_DCECOMBT_BLS
THOST_FTDC_DCECOMBT_BES = _thosttraderapi.THOST_FTDC_DCECOMBT_BES
THOST_FTDC_DCECOMBT_CAS = _thosttraderapi.THOST_FTDC_DCECOMBT_CAS
THOST_FTDC_DCECOMBT_STD = _thosttraderapi.THOST_FTDC_DCECOMBT_STD
THOST_FTDC_DCECOMBT_STG = _thosttraderapi.THOST_FTDC_DCECOMBT_STG
THOST_FTDC_DCECOMBT_BFO = _thosttraderapi.THOST_FTDC_DCECOMBT_BFO
THOST_FTDC_DCECOMBT_SFO = _thosttraderapi.THOST_FTDC_DCECOMBT_SFO
THOST_FTDC_ORPT_PreSettlementPrice = _thosttraderapi.THOST_FTDC_ORPT_PreSettlementPrice
THOST_FTDC_ORPT_OpenPrice = _thosttraderapi.THOST_FTDC_ORPT_OpenPrice
THOST_FTDC_ORPT_MaxPreSettlementPrice = _thosttraderapi.THOST_FTDC_ORPT_MaxPreSettlementPrice
THOST_FTDC_BLAG_Default = _thosttraderapi.THOST_FTDC_BLAG_Default
THOST_FTDC_BLAG_IncludeOptValLost = _thosttraderapi.THOST_FTDC_BLAG_IncludeOptValLost
THOST_FTDC_ACTP_Exec = _thosttraderapi.THOST_FTDC_ACTP_Exec
THOST_FTDC_ACTP_Abandon = _thosttraderapi.THOST_FTDC_ACTP_Abandon
THOST_FTDC_FQST_Submitted = _thosttraderapi.THOST_FTDC_FQST_Submitted
THOST_FTDC_FQST_Accepted = _thosttraderapi.THOST_FTDC_FQST_Accepted
THOST_FTDC_FQST_Rejected = _thosttraderapi.THOST_FTDC_FQST_Rejected
THOST_FTDC_VM_Absolute = _thosttraderapi.THOST_FTDC_VM_Absolute
THOST_FTDC_VM_Ratio = _thosttraderapi.THOST_FTDC_VM_Ratio
THOST_FTDC_EOPF_Reserve = _thosttraderapi.THOST_FTDC_EOPF_Reserve
THOST_FTDC_EOPF_UnReserve = _thosttraderapi.THOST_FTDC_EOPF_UnReserve
THOST_FTDC_EOCF_AutoClose = _thosttraderapi.THOST_FTDC_EOCF_AutoClose
THOST_FTDC_EOCF_NotToClose = _thosttraderapi.THOST_FTDC_EOCF_NotToClose
THOST_FTDC_PTE_Futures = _thosttraderapi.THOST_FTDC_PTE_Futures
THOST_FTDC_PTE_Options = _thosttraderapi.THOST_FTDC_PTE_Options
THOST_FTDC_CUFN_CUFN_O = _thosttraderapi.THOST_FTDC_CUFN_CUFN_O
THOST_FTDC_CUFN_CUFN_T = _thosttraderapi.THOST_FTDC_CUFN_CUFN_T
THOST_FTDC_CUFN_CUFN_P = _thosttraderapi.THOST_FTDC_CUFN_CUFN_P
THOST_FTDC_CUFN_CUFN_N = _thosttraderapi.THOST_FTDC_CUFN_CUFN_N
THOST_FTDC_CUFN_CUFN_L = _thosttraderapi.THOST_FTDC_CUFN_CUFN_L
THOST_FTDC_CUFN_CUFN_F = _thosttraderapi.THOST_FTDC_CUFN_CUFN_F
THOST_FTDC_CUFN_CUFN_C = _thosttraderapi.THOST_FTDC_CUFN_CUFN_C
THOST_FTDC_CUFN_CUFN_M = _thosttraderapi.THOST_FTDC_CUFN_CUFN_M
THOST_FTDC_DUFN_DUFN_O = _thosttraderapi.THOST_FTDC_DUFN_DUFN_O
THOST_FTDC_DUFN_DUFN_T = _thosttraderapi.THOST_FTDC_DUFN_DUFN_T
THOST_FTDC_DUFN_DUFN_P = _thosttraderapi.THOST_FTDC_DUFN_DUFN_P
THOST_FTDC_DUFN_DUFN_F = _thosttraderapi.THOST_FTDC_DUFN_DUFN_F
THOST_FTDC_DUFN_DUFN_C = _thosttraderapi.THOST_FTDC_DUFN_DUFN_C
THOST_FTDC_DUFN_DUFN_D = _thosttraderapi.THOST_FTDC_DUFN_DUFN_D
THOST_FTDC_DUFN_DUFN_M = _thosttraderapi.THOST_FTDC_DUFN_DUFN_M
THOST_FTDC_DUFN_DUFN_S = _thosttraderapi.THOST_FTDC_DUFN_DUFN_S
THOST_FTDC_SUFN_SUFN_O = _thosttraderapi.THOST_FTDC_SUFN_SUFN_O
THOST_FTDC_SUFN_SUFN_T = _thosttraderapi.THOST_FTDC_SUFN_SUFN_T
THOST_FTDC_SUFN_SUFN_P = _thosttraderapi.THOST_FTDC_SUFN_SUFN_P
THOST_FTDC_SUFN_SUFN_F = _thosttraderapi.THOST_FTDC_SUFN_SUFN_F
THOST_FTDC_CFUFN_SUFN_T = _thosttraderapi.THOST_FTDC_CFUFN_SUFN_T
THOST_FTDC_CFUFN_SUFN_P = _thosttraderapi.THOST_FTDC_CFUFN_SUFN_P
THOST_FTDC_CFUFN_SUFN_F = _thosttraderapi.THOST_FTDC_CFUFN_SUFN_F
THOST_FTDC_CFUFN_SUFN_S = _thosttraderapi.THOST_FTDC_CFUFN_SUFN_S
THOST_FTDC_CMDR_Comb = _thosttraderapi.THOST_FTDC_CMDR_Comb
THOST_FTDC_CMDR_UnComb = _thosttraderapi.THOST_FTDC_CMDR_UnComb
THOST_FTDC_STOV_RealValue = _thosttraderapi.THOST_FTDC_STOV_RealValue
THOST_FTDC_STOV_ProfitValue = _thosttraderapi.THOST_FTDC_STOV_ProfitValue
THOST_FTDC_STOV_RealRatio = _thosttraderapi.THOST_FTDC_STOV_RealRatio
THOST_FTDC_STOV_ProfitRatio = _thosttraderapi.THOST_FTDC_STOV_ProfitRatio
THOST_FTDC_ROAST_Processing = _thosttraderapi.THOST_FTDC_ROAST_Processing
THOST_FTDC_ROAST_Cancelled = _thosttraderapi.THOST_FTDC_ROAST_Cancelled
THOST_FTDC_ROAST_Opened = _thosttraderapi.THOST_FTDC_ROAST_Opened
THOST_FTDC_ROAST_Invalid = _thosttraderapi.THOST_FTDC_ROAST_Invalid
THOST_FTDC_WPSR_Lib = _thosttraderapi.THOST_FTDC_WPSR_Lib
THOST_FTDC_WPSR_Manual = _thosttraderapi.THOST_FTDC_WPSR_Manual
THOST_FTDC_OSCF_CloseSelfOptionPosition = _thosttraderapi.THOST_FTDC_OSCF_CloseSelfOptionPosition
THOST_FTDC_OSCF_ReserveOptionPosition = _thosttraderapi.THOST_FTDC_OSCF_ReserveOptionPosition
THOST_FTDC_OSCF_SellCloseSelfFuturePosition = _thosttraderapi.THOST_FTDC_OSCF_SellCloseSelfFuturePosition
THOST_FTDC_OSCF_ReserveFuturePosition = _thosttraderapi.THOST_FTDC_OSCF_ReserveFuturePosition
THOST_FTDC_BZTP_Future = _thosttraderapi.THOST_FTDC_BZTP_Future
THOST_FTDC_BZTP_Stock = _thosttraderapi.THOST_FTDC_BZTP_Stock
THOST_FTDC_APP_TYPE_Investor = _thosttraderapi.THOST_FTDC_APP_TYPE_Investor
THOST_FTDC_APP_TYPE_InvestorRelay = _thosttraderapi.THOST_FTDC_APP_TYPE_InvestorRelay
THOST_FTDC_APP_TYPE_OperatorRelay = _thosttraderapi.THOST_FTDC_APP_TYPE_OperatorRelay
THOST_FTDC_APP_TYPE_UnKnown = _thosttraderapi.THOST_FTDC_APP_TYPE_UnKnown
THOST_FTDC_RV_Right = _thosttraderapi.THOST_FTDC_RV_Right
THOST_FTDC_RV_Refuse = _thosttraderapi.THOST_FTDC_RV_Refuse
THOST_FTDC_OTC_TRDT_Block = _thosttraderapi.THOST_FTDC_OTC_TRDT_Block
THOST_FTDC_OTC_TRDT_EFP = _thosttraderapi.THOST_FTDC_OTC_TRDT_EFP
THOST_FTDC_OTC_MT_DV01 = _thosttraderapi.THOST_FTDC_OTC_MT_DV01
THOST_FTDC_OTC_MT_ParValue = _thosttraderapi.THOST_FTDC_OTC_MT_ParValue
class CThostFtdcDisseminationField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    SequenceSeries = property(_thosttraderapi.CThostFtdcDisseminationField_SequenceSeries_get, _thosttraderapi.CThostFtdcDisseminationField_SequenceSeries_set)
    SequenceNo = property(_thosttraderapi.CThostFtdcDisseminationField_SequenceNo_get, _thosttraderapi.CThostFtdcDisseminationField_SequenceNo_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcDisseminationField_swiginit(self, _thosttraderapi.new_CThostFtdcDisseminationField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcDisseminationField

# Register CThostFtdcDisseminationField in _thosttraderapi:
_thosttraderapi.CThostFtdcDisseminationField_swigregister(CThostFtdcDisseminationField)

class CThostFtdcReqUserLoginField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    TradingDay = property(_thosttraderapi.CThostFtdcReqUserLoginField_TradingDay_get, _thosttraderapi.CThostFtdcReqUserLoginField_TradingDay_set)
    BrokerID = property(_thosttraderapi.CThostFtdcReqUserLoginField_BrokerID_get, _thosttraderapi.CThostFtdcReqUserLoginField_BrokerID_set)
    UserID = property(_thosttraderapi.CThostFtdcReqUserLoginField_UserID_get, _thosttraderapi.CThostFtdcReqUserLoginField_UserID_set)
    Password = property(_thosttraderapi.CThostFtdcReqUserLoginField_Password_get, _thosttraderapi.CThostFtdcReqUserLoginField_Password_set)
    UserProductInfo = property(_thosttraderapi.CThostFtdcReqUserLoginField_UserProductInfo_get, _thosttraderapi.CThostFtdcReqUserLoginField_UserProductInfo_set)
    InterfaceProductInfo = property(_thosttraderapi.CThostFtdcReqUserLoginField_InterfaceProductInfo_get, _thosttraderapi.CThostFtdcReqUserLoginField_InterfaceProductInfo_set)
    ProtocolInfo = property(_thosttraderapi.CThostFtdcReqUserLoginField_ProtocolInfo_get, _thosttraderapi.CThostFtdcReqUserLoginField_ProtocolInfo_set)
    MacAddress = property(_thosttraderapi.CThostFtdcReqUserLoginField_MacAddress_get, _thosttraderapi.CThostFtdcReqUserLoginField_MacAddress_set)
    OneTimePassword = property(_thosttraderapi.CThostFtdcReqUserLoginField_OneTimePassword_get, _thosttraderapi.CThostFtdcReqUserLoginField_OneTimePassword_set)
    ClientIPAddress = property(_thosttraderapi.CThostFtdcReqUserLoginField_ClientIPAddress_get, _thosttraderapi.CThostFtdcReqUserLoginField_ClientIPAddress_set)
    LoginRemark = property(_thosttraderapi.CThostFtdcReqUserLoginField_LoginRemark_get, _thosttraderapi.CThostFtdcReqUserLoginField_LoginRemark_set)
    ClientIPPort = property(_thosttraderapi.CThostFtdcReqUserLoginField_ClientIPPort_get, _thosttraderapi.CThostFtdcReqUserLoginField_ClientIPPort_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcReqUserLoginField_swiginit(self, _thosttraderapi.new_CThostFtdcReqUserLoginField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcReqUserLoginField

# Register CThostFtdcReqUserLoginField in _thosttraderapi:
_thosttraderapi.CThostFtdcReqUserLoginField_swigregister(CThostFtdcReqUserLoginField)

class CThostFtdcRspUserLoginField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    TradingDay = property(_thosttraderapi.CThostFtdcRspUserLoginField_TradingDay_get, _thosttraderapi.CThostFtdcRspUserLoginField_TradingDay_set)
    LoginTime = property(_thosttraderapi.CThostFtdcRspUserLoginField_LoginTime_get, _thosttraderapi.CThostFtdcRspUserLoginField_LoginTime_set)
    BrokerID = property(_thosttraderapi.CThostFtdcRspUserLoginField_BrokerID_get, _thosttraderapi.CThostFtdcRspUserLoginField_BrokerID_set)
    UserID = property(_thosttraderapi.CThostFtdcRspUserLoginField_UserID_get, _thosttraderapi.CThostFtdcRspUserLoginField_UserID_set)
    SystemName = property(_thosttraderapi.CThostFtdcRspUserLoginField_SystemName_get, _thosttraderapi.CThostFtdcRspUserLoginField_SystemName_set)
    FrontID = property(_thosttraderapi.CThostFtdcRspUserLoginField_FrontID_get, _thosttraderapi.CThostFtdcRspUserLoginField_FrontID_set)
    SessionID = property(_thosttraderapi.CThostFtdcRspUserLoginField_SessionID_get, _thosttraderapi.CThostFtdcRspUserLoginField_SessionID_set)
    MaxOrderRef = property(_thosttraderapi.CThostFtdcRspUserLoginField_MaxOrderRef_get, _thosttraderapi.CThostFtdcRspUserLoginField_MaxOrderRef_set)
    SHFETime = property(_thosttraderapi.CThostFtdcRspUserLoginField_SHFETime_get, _thosttraderapi.CThostFtdcRspUserLoginField_SHFETime_set)
    DCETime = property(_thosttraderapi.CThostFtdcRspUserLoginField_DCETime_get, _thosttraderapi.CThostFtdcRspUserLoginField_DCETime_set)
    CZCETime = property(_thosttraderapi.CThostFtdcRspUserLoginField_CZCETime_get, _thosttraderapi.CThostFtdcRspUserLoginField_CZCETime_set)
    FFEXTime = property(_thosttraderapi.CThostFtdcRspUserLoginField_FFEXTime_get, _thosttraderapi.CThostFtdcRspUserLoginField_FFEXTime_set)
    INETime = property(_thosttraderapi.CThostFtdcRspUserLoginField_INETime_get, _thosttraderapi.CThostFtdcRspUserLoginField_INETime_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcRspUserLoginField_swiginit(self, _thosttraderapi.new_CThostFtdcRspUserLoginField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcRspUserLoginField

# Register CThostFtdcRspUserLoginField in _thosttraderapi:
_thosttraderapi.CThostFtdcRspUserLoginField_swigregister(CThostFtdcRspUserLoginField)

class CThostFtdcUserLogoutField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thosttraderapi.CThostFtdcUserLogoutField_BrokerID_get, _thosttraderapi.CThostFtdcUserLogoutField_BrokerID_set)
    UserID = property(_thosttraderapi.CThostFtdcUserLogoutField_UserID_get, _thosttraderapi.CThostFtdcUserLogoutField_UserID_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcUserLogoutField_swiginit(self, _thosttraderapi.new_CThostFtdcUserLogoutField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcUserLogoutField

# Register CThostFtdcUserLogoutField in _thosttraderapi:
_thosttraderapi.CThostFtdcUserLogoutField_swigregister(CThostFtdcUserLogoutField)

class CThostFtdcForceUserLogoutField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thosttraderapi.CThostFtdcForceUserLogoutField_BrokerID_get, _thosttraderapi.CThostFtdcForceUserLogoutField_BrokerID_set)
    UserID = property(_thosttraderapi.CThostFtdcForceUserLogoutField_UserID_get, _thosttraderapi.CThostFtdcForceUserLogoutField_UserID_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcForceUserLogoutField_swiginit(self, _thosttraderapi.new_CThostFtdcForceUserLogoutField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcForceUserLogoutField

# Register CThostFtdcForceUserLogoutField in _thosttraderapi:
_thosttraderapi.CThostFtdcForceUserLogoutField_swigregister(CThostFtdcForceUserLogoutField)

class CThostFtdcReqAuthenticateField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thosttraderapi.CThostFtdcReqAuthenticateField_BrokerID_get, _thosttraderapi.CThostFtdcReqAuthenticateField_BrokerID_set)
    UserID = property(_thosttraderapi.CThostFtdcReqAuthenticateField_UserID_get, _thosttraderapi.CThostFtdcReqAuthenticateField_UserID_set)
    UserProductInfo = property(_thosttraderapi.CThostFtdcReqAuthenticateField_UserProductInfo_get, _thosttraderapi.CThostFtdcReqAuthenticateField_UserProductInfo_set)
    AuthCode = property(_thosttraderapi.CThostFtdcReqAuthenticateField_AuthCode_get, _thosttraderapi.CThostFtdcReqAuthenticateField_AuthCode_set)
    AppID = property(_thosttraderapi.CThostFtdcReqAuthenticateField_AppID_get, _thosttraderapi.CThostFtdcReqAuthenticateField_AppID_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcReqAuthenticateField_swiginit(self, _thosttraderapi.new_CThostFtdcReqAuthenticateField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcReqAuthenticateField

# Register CThostFtdcReqAuthenticateField in _thosttraderapi:
_thosttraderapi.CThostFtdcReqAuthenticateField_swigregister(CThostFtdcReqAuthenticateField)

class CThostFtdcRspAuthenticateField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thosttraderapi.CThostFtdcRspAuthenticateField_BrokerID_get, _thosttraderapi.CThostFtdcRspAuthenticateField_BrokerID_set)
    UserID = property(_thosttraderapi.CThostFtdcRspAuthenticateField_UserID_get, _thosttraderapi.CThostFtdcRspAuthenticateField_UserID_set)
    UserProductInfo = property(_thosttraderapi.CThostFtdcRspAuthenticateField_UserProductInfo_get, _thosttraderapi.CThostFtdcRspAuthenticateField_UserProductInfo_set)
    AppID = property(_thosttraderapi.CThostFtdcRspAuthenticateField_AppID_get, _thosttraderapi.CThostFtdcRspAuthenticateField_AppID_set)
    AppType = property(_thosttraderapi.CThostFtdcRspAuthenticateField_AppType_get, _thosttraderapi.CThostFtdcRspAuthenticateField_AppType_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcRspAuthenticateField_swiginit(self, _thosttraderapi.new_CThostFtdcRspAuthenticateField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcRspAuthenticateField

# Register CThostFtdcRspAuthenticateField in _thosttraderapi:
_thosttraderapi.CThostFtdcRspAuthenticateField_swigregister(CThostFtdcRspAuthenticateField)

class CThostFtdcAuthenticationInfoField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thosttraderapi.CThostFtdcAuthenticationInfoField_BrokerID_get, _thosttraderapi.CThostFtdcAuthenticationInfoField_BrokerID_set)
    UserID = property(_thosttraderapi.CThostFtdcAuthenticationInfoField_UserID_get, _thosttraderapi.CThostFtdcAuthenticationInfoField_UserID_set)
    UserProductInfo = property(_thosttraderapi.CThostFtdcAuthenticationInfoField_UserProductInfo_get, _thosttraderapi.CThostFtdcAuthenticationInfoField_UserProductInfo_set)
    AuthInfo = property(_thosttraderapi.CThostFtdcAuthenticationInfoField_AuthInfo_get, _thosttraderapi.CThostFtdcAuthenticationInfoField_AuthInfo_set)
    IsResult = property(_thosttraderapi.CThostFtdcAuthenticationInfoField_IsResult_get, _thosttraderapi.CThostFtdcAuthenticationInfoField_IsResult_set)
    AppID = property(_thosttraderapi.CThostFtdcAuthenticationInfoField_AppID_get, _thosttraderapi.CThostFtdcAuthenticationInfoField_AppID_set)
    AppType = property(_thosttraderapi.CThostFtdcAuthenticationInfoField_AppType_get, _thosttraderapi.CThostFtdcAuthenticationInfoField_AppType_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcAuthenticationInfoField_swiginit(self, _thosttraderapi.new_CThostFtdcAuthenticationInfoField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcAuthenticationInfoField

# Register CThostFtdcAuthenticationInfoField in _thosttraderapi:
_thosttraderapi.CThostFtdcAuthenticationInfoField_swigregister(CThostFtdcAuthenticationInfoField)

class CThostFtdcRspUserLogin2Field(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    TradingDay = property(_thosttraderapi.CThostFtdcRspUserLogin2Field_TradingDay_get, _thosttraderapi.CThostFtdcRspUserLogin2Field_TradingDay_set)
    LoginTime = property(_thosttraderapi.CThostFtdcRspUserLogin2Field_LoginTime_get, _thosttraderapi.CThostFtdcRspUserLogin2Field_LoginTime_set)
    BrokerID = property(_thosttraderapi.CThostFtdcRspUserLogin2Field_BrokerID_get, _thosttraderapi.CThostFtdcRspUserLogin2Field_BrokerID_set)
    UserID = property(_thosttraderapi.CThostFtdcRspUserLogin2Field_UserID_get, _thosttraderapi.CThostFtdcRspUserLogin2Field_UserID_set)
    SystemName = property(_thosttraderapi.CThostFtdcRspUserLogin2Field_SystemName_get, _thosttraderapi.CThostFtdcRspUserLogin2Field_SystemName_set)
    FrontID = property(_thosttraderapi.CThostFtdcRspUserLogin2Field_FrontID_get, _thosttraderapi.CThostFtdcRspUserLogin2Field_FrontID_set)
    SessionID = property(_thosttraderapi.CThostFtdcRspUserLogin2Field_SessionID_get, _thosttraderapi.CThostFtdcRspUserLogin2Field_SessionID_set)
    MaxOrderRef = property(_thosttraderapi.CThostFtdcRspUserLogin2Field_MaxOrderRef_get, _thosttraderapi.CThostFtdcRspUserLogin2Field_MaxOrderRef_set)
    SHFETime = property(_thosttraderapi.CThostFtdcRspUserLogin2Field_SHFETime_get, _thosttraderapi.CThostFtdcRspUserLogin2Field_SHFETime_set)
    DCETime = property(_thosttraderapi.CThostFtdcRspUserLogin2Field_DCETime_get, _thosttraderapi.CThostFtdcRspUserLogin2Field_DCETime_set)
    CZCETime = property(_thosttraderapi.CThostFtdcRspUserLogin2Field_CZCETime_get, _thosttraderapi.CThostFtdcRspUserLogin2Field_CZCETime_set)
    FFEXTime = property(_thosttraderapi.CThostFtdcRspUserLogin2Field_FFEXTime_get, _thosttraderapi.CThostFtdcRspUserLogin2Field_FFEXTime_set)
    INETime = property(_thosttraderapi.CThostFtdcRspUserLogin2Field_INETime_get, _thosttraderapi.CThostFtdcRspUserLogin2Field_INETime_set)
    RandomString = property(_thosttraderapi.CThostFtdcRspUserLogin2Field_RandomString_get, _thosttraderapi.CThostFtdcRspUserLogin2Field_RandomString_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcRspUserLogin2Field_swiginit(self, _thosttraderapi.new_CThostFtdcRspUserLogin2Field())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcRspUserLogin2Field

# Register CThostFtdcRspUserLogin2Field in _thosttraderapi:
_thosttraderapi.CThostFtdcRspUserLogin2Field_swigregister(CThostFtdcRspUserLogin2Field)

class CThostFtdcTransferHeaderField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    Version = property(_thosttraderapi.CThostFtdcTransferHeaderField_Version_get, _thosttraderapi.CThostFtdcTransferHeaderField_Version_set)
    TradeCode = property(_thosttraderapi.CThostFtdcTransferHeaderField_TradeCode_get, _thosttraderapi.CThostFtdcTransferHeaderField_TradeCode_set)
    TradeDate = property(_thosttraderapi.CThostFtdcTransferHeaderField_TradeDate_get, _thosttraderapi.CThostFtdcTransferHeaderField_TradeDate_set)
    TradeTime = property(_thosttraderapi.CThostFtdcTransferHeaderField_TradeTime_get, _thosttraderapi.CThostFtdcTransferHeaderField_TradeTime_set)
    TradeSerial = property(_thosttraderapi.CThostFtdcTransferHeaderField_TradeSerial_get, _thosttraderapi.CThostFtdcTransferHeaderField_TradeSerial_set)
    FutureID = property(_thosttraderapi.CThostFtdcTransferHeaderField_FutureID_get, _thosttraderapi.CThostFtdcTransferHeaderField_FutureID_set)
    BankID = property(_thosttraderapi.CThostFtdcTransferHeaderField_BankID_get, _thosttraderapi.CThostFtdcTransferHeaderField_BankID_set)
    BankBrchID = property(_thosttraderapi.CThostFtdcTransferHeaderField_BankBrchID_get, _thosttraderapi.CThostFtdcTransferHeaderField_BankBrchID_set)
    OperNo = property(_thosttraderapi.CThostFtdcTransferHeaderField_OperNo_get, _thosttraderapi.CThostFtdcTransferHeaderField_OperNo_set)
    DeviceID = property(_thosttraderapi.CThostFtdcTransferHeaderField_DeviceID_get, _thosttraderapi.CThostFtdcTransferHeaderField_DeviceID_set)
    RecordNum = property(_thosttraderapi.CThostFtdcTransferHeaderField_RecordNum_get, _thosttraderapi.CThostFtdcTransferHeaderField_RecordNum_set)
    SessionID = property(_thosttraderapi.CThostFtdcTransferHeaderField_SessionID_get, _thosttraderapi.CThostFtdcTransferHeaderField_SessionID_set)
    RequestID = property(_thosttraderapi.CThostFtdcTransferHeaderField_RequestID_get, _thosttraderapi.CThostFtdcTransferHeaderField_RequestID_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcTransferHeaderField_swiginit(self, _thosttraderapi.new_CThostFtdcTransferHeaderField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcTransferHeaderField

# Register CThostFtdcTransferHeaderField in _thosttraderapi:
_thosttraderapi.CThostFtdcTransferHeaderField_swigregister(CThostFtdcTransferHeaderField)

class CThostFtdcTransferBankToFutureReqField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    FutureAccount = property(_thosttraderapi.CThostFtdcTransferBankToFutureReqField_FutureAccount_get, _thosttraderapi.CThostFtdcTransferBankToFutureReqField_FutureAccount_set)
    FuturePwdFlag = property(_thosttraderapi.CThostFtdcTransferBankToFutureReqField_FuturePwdFlag_get, _thosttraderapi.CThostFtdcTransferBankToFutureReqField_FuturePwdFlag_set)
    FutureAccPwd = property(_thosttraderapi.CThostFtdcTransferBankToFutureReqField_FutureAccPwd_get, _thosttraderapi.CThostFtdcTransferBankToFutureReqField_FutureAccPwd_set)
    TradeAmt = property(_thosttraderapi.CThostFtdcTransferBankToFutureReqField_TradeAmt_get, _thosttraderapi.CThostFtdcTransferBankToFutureReqField_TradeAmt_set)
    CustFee = property(_thosttraderapi.CThostFtdcTransferBankToFutureReqField_CustFee_get, _thosttraderapi.CThostFtdcTransferBankToFutureReqField_CustFee_set)
    CurrencyCode = property(_thosttraderapi.CThostFtdcTransferBankToFutureReqField_CurrencyCode_get, _thosttraderapi.CThostFtdcTransferBankToFutureReqField_CurrencyCode_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcTransferBankToFutureReqField_swiginit(self, _thosttraderapi.new_CThostFtdcTransferBankToFutureReqField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcTransferBankToFutureReqField

# Register CThostFtdcTransferBankToFutureReqField in _thosttraderapi:
_thosttraderapi.CThostFtdcTransferBankToFutureReqField_swigregister(CThostFtdcTransferBankToFutureReqField)

class CThostFtdcTransferBankToFutureRspField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    RetCode = property(_thosttraderapi.CThostFtdcTransferBankToFutureRspField_RetCode_get, _thosttraderapi.CThostFtdcTransferBankToFutureRspField_RetCode_set)
    RetInfo = property(_thosttraderapi.CThostFtdcTransferBankToFutureRspField_RetInfo_get, _thosttraderapi.CThostFtdcTransferBankToFutureRspField_RetInfo_set)
    FutureAccount = property(_thosttraderapi.CThostFtdcTransferBankToFutureRspField_FutureAccount_get, _thosttraderapi.CThostFtdcTransferBankToFutureRspField_FutureAccount_set)
    TradeAmt = property(_thosttraderapi.CThostFtdcTransferBankToFutureRspField_TradeAmt_get, _thosttraderapi.CThostFtdcTransferBankToFutureRspField_TradeAmt_set)
    CustFee = property(_thosttraderapi.CThostFtdcTransferBankToFutureRspField_CustFee_get, _thosttraderapi.CThostFtdcTransferBankToFutureRspField_CustFee_set)
    CurrencyCode = property(_thosttraderapi.CThostFtdcTransferBankToFutureRspField_CurrencyCode_get, _thosttraderapi.CThostFtdcTransferBankToFutureRspField_CurrencyCode_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcTransferBankToFutureRspField_swiginit(self, _thosttraderapi.new_CThostFtdcTransferBankToFutureRspField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcTransferBankToFutureRspField

# Register CThostFtdcTransferBankToFutureRspField in _thosttraderapi:
_thosttraderapi.CThostFtdcTransferBankToFutureRspField_swigregister(CThostFtdcTransferBankToFutureRspField)

class CThostFtdcTransferFutureToBankReqField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    FutureAccount = property(_thosttraderapi.CThostFtdcTransferFutureToBankReqField_FutureAccount_get, _thosttraderapi.CThostFtdcTransferFutureToBankReqField_FutureAccount_set)
    FuturePwdFlag = property(_thosttraderapi.CThostFtdcTransferFutureToBankReqField_FuturePwdFlag_get, _thosttraderapi.CThostFtdcTransferFutureToBankReqField_FuturePwdFlag_set)
    FutureAccPwd = property(_thosttraderapi.CThostFtdcTransferFutureToBankReqField_FutureAccPwd_get, _thosttraderapi.CThostFtdcTransferFutureToBankReqField_FutureAccPwd_set)
    TradeAmt = property(_thosttraderapi.CThostFtdcTransferFutureToBankReqField_TradeAmt_get, _thosttraderapi.CThostFtdcTransferFutureToBankReqField_TradeAmt_set)
    CustFee = property(_thosttraderapi.CThostFtdcTransferFutureToBankReqField_CustFee_get, _thosttraderapi.CThostFtdcTransferFutureToBankReqField_CustFee_set)
    CurrencyCode = property(_thosttraderapi.CThostFtdcTransferFutureToBankReqField_CurrencyCode_get, _thosttraderapi.CThostFtdcTransferFutureToBankReqField_CurrencyCode_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcTransferFutureToBankReqField_swiginit(self, _thosttraderapi.new_CThostFtdcTransferFutureToBankReqField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcTransferFutureToBankReqField

# Register CThostFtdcTransferFutureToBankReqField in _thosttraderapi:
_thosttraderapi.CThostFtdcTransferFutureToBankReqField_swigregister(CThostFtdcTransferFutureToBankReqField)

class CThostFtdcTransferFutureToBankRspField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    RetCode = property(_thosttraderapi.CThostFtdcTransferFutureToBankRspField_RetCode_get, _thosttraderapi.CThostFtdcTransferFutureToBankRspField_RetCode_set)
    RetInfo = property(_thosttraderapi.CThostFtdcTransferFutureToBankRspField_RetInfo_get, _thosttraderapi.CThostFtdcTransferFutureToBankRspField_RetInfo_set)
    FutureAccount = property(_thosttraderapi.CThostFtdcTransferFutureToBankRspField_FutureAccount_get, _thosttraderapi.CThostFtdcTransferFutureToBankRspField_FutureAccount_set)
    TradeAmt = property(_thosttraderapi.CThostFtdcTransferFutureToBankRspField_TradeAmt_get, _thosttraderapi.CThostFtdcTransferFutureToBankRspField_TradeAmt_set)
    CustFee = property(_thosttraderapi.CThostFtdcTransferFutureToBankRspField_CustFee_get, _thosttraderapi.CThostFtdcTransferFutureToBankRspField_CustFee_set)
    CurrencyCode = property(_thosttraderapi.CThostFtdcTransferFutureToBankRspField_CurrencyCode_get, _thosttraderapi.CThostFtdcTransferFutureToBankRspField_CurrencyCode_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcTransferFutureToBankRspField_swiginit(self, _thosttraderapi.new_CThostFtdcTransferFutureToBankRspField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcTransferFutureToBankRspField

# Register CThostFtdcTransferFutureToBankRspField in _thosttraderapi:
_thosttraderapi.CThostFtdcTransferFutureToBankRspField_swigregister(CThostFtdcTransferFutureToBankRspField)

class CThostFtdcTransferQryBankReqField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    FutureAccount = property(_thosttraderapi.CThostFtdcTransferQryBankReqField_FutureAccount_get, _thosttraderapi.CThostFtdcTransferQryBankReqField_FutureAccount_set)
    FuturePwdFlag = property(_thosttraderapi.CThostFtdcTransferQryBankReqField_FuturePwdFlag_get, _thosttraderapi.CThostFtdcTransferQryBankReqField_FuturePwdFlag_set)
    FutureAccPwd = property(_thosttraderapi.CThostFtdcTransferQryBankReqField_FutureAccPwd_get, _thosttraderapi.CThostFtdcTransferQryBankReqField_FutureAccPwd_set)
    CurrencyCode = property(_thosttraderapi.CThostFtdcTransferQryBankReqField_CurrencyCode_get, _thosttraderapi.CThostFtdcTransferQryBankReqField_CurrencyCode_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcTransferQryBankReqField_swiginit(self, _thosttraderapi.new_CThostFtdcTransferQryBankReqField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcTransferQryBankReqField

# Register CThostFtdcTransferQryBankReqField in _thosttraderapi:
_thosttraderapi.CThostFtdcTransferQryBankReqField_swigregister(CThostFtdcTransferQryBankReqField)

class CThostFtdcTransferQryBankRspField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    RetCode = property(_thosttraderapi.CThostFtdcTransferQryBankRspField_RetCode_get, _thosttraderapi.CThostFtdcTransferQryBankRspField_RetCode_set)
    RetInfo = property(_thosttraderapi.CThostFtdcTransferQryBankRspField_RetInfo_get, _thosttraderapi.CThostFtdcTransferQryBankRspField_RetInfo_set)
    FutureAccount = property(_thosttraderapi.CThostFtdcTransferQryBankRspField_FutureAccount_get, _thosttraderapi.CThostFtdcTransferQryBankRspField_FutureAccount_set)
    TradeAmt = property(_thosttraderapi.CThostFtdcTransferQryBankRspField_TradeAmt_get, _thosttraderapi.CThostFtdcTransferQryBankRspField_TradeAmt_set)
    UseAmt = property(_thosttraderapi.CThostFtdcTransferQryBankRspField_UseAmt_get, _thosttraderapi.CThostFtdcTransferQryBankRspField_UseAmt_set)
    FetchAmt = property(_thosttraderapi.CThostFtdcTransferQryBankRspField_FetchAmt_get, _thosttraderapi.CThostFtdcTransferQryBankRspField_FetchAmt_set)
    CurrencyCode = property(_thosttraderapi.CThostFtdcTransferQryBankRspField_CurrencyCode_get, _thosttraderapi.CThostFtdcTransferQryBankRspField_CurrencyCode_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcTransferQryBankRspField_swiginit(self, _thosttraderapi.new_CThostFtdcTransferQryBankRspField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcTransferQryBankRspField

# Register CThostFtdcTransferQryBankRspField in _thosttraderapi:
_thosttraderapi.CThostFtdcTransferQryBankRspField_swigregister(CThostFtdcTransferQryBankRspField)

class CThostFtdcTransferQryDetailReqField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    FutureAccount = property(_thosttraderapi.CThostFtdcTransferQryDetailReqField_FutureAccount_get, _thosttraderapi.CThostFtdcTransferQryDetailReqField_FutureAccount_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcTransferQryDetailReqField_swiginit(self, _thosttraderapi.new_CThostFtdcTransferQryDetailReqField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcTransferQryDetailReqField

# Register CThostFtdcTransferQryDetailReqField in _thosttraderapi:
_thosttraderapi.CThostFtdcTransferQryDetailReqField_swigregister(CThostFtdcTransferQryDetailReqField)

class CThostFtdcTransferQryDetailRspField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    TradeDate = property(_thosttraderapi.CThostFtdcTransferQryDetailRspField_TradeDate_get, _thosttraderapi.CThostFtdcTransferQryDetailRspField_TradeDate_set)
    TradeTime = property(_thosttraderapi.CThostFtdcTransferQryDetailRspField_TradeTime_get, _thosttraderapi.CThostFtdcTransferQryDetailRspField_TradeTime_set)
    TradeCode = property(_thosttraderapi.CThostFtdcTransferQryDetailRspField_TradeCode_get, _thosttraderapi.CThostFtdcTransferQryDetailRspField_TradeCode_set)
    FutureSerial = property(_thosttraderapi.CThostFtdcTransferQryDetailRspField_FutureSerial_get, _thosttraderapi.CThostFtdcTransferQryDetailRspField_FutureSerial_set)
    FutureID = property(_thosttraderapi.CThostFtdcTransferQryDetailRspField_FutureID_get, _thosttraderapi.CThostFtdcTransferQryDetailRspField_FutureID_set)
    FutureAccount = property(_thosttraderapi.CThostFtdcTransferQryDetailRspField_FutureAccount_get, _thosttraderapi.CThostFtdcTransferQryDetailRspField_FutureAccount_set)
    BankSerial = property(_thosttraderapi.CThostFtdcTransferQryDetailRspField_BankSerial_get, _thosttraderapi.CThostFtdcTransferQryDetailRspField_BankSerial_set)
    BankID = property(_thosttraderapi.CThostFtdcTransferQryDetailRspField_BankID_get, _thosttraderapi.CThostFtdcTransferQryDetailRspField_BankID_set)
    BankBrchID = property(_thosttraderapi.CThostFtdcTransferQryDetailRspField_BankBrchID_get, _thosttraderapi.CThostFtdcTransferQryDetailRspField_BankBrchID_set)
    BankAccount = property(_thosttraderapi.CThostFtdcTransferQryDetailRspField_BankAccount_get, _thosttraderapi.CThostFtdcTransferQryDetailRspField_BankAccount_set)
    CertCode = property(_thosttraderapi.CThostFtdcTransferQryDetailRspField_CertCode_get, _thosttraderapi.CThostFtdcTransferQryDetailRspField_CertCode_set)
    CurrencyCode = property(_thosttraderapi.CThostFtdcTransferQryDetailRspField_CurrencyCode_get, _thosttraderapi.CThostFtdcTransferQryDetailRspField_CurrencyCode_set)
    TxAmount = property(_thosttraderapi.CThostFtdcTransferQryDetailRspField_TxAmount_get, _thosttraderapi.CThostFtdcTransferQryDetailRspField_TxAmount_set)
    Flag = property(_thosttraderapi.CThostFtdcTransferQryDetailRspField_Flag_get, _thosttraderapi.CThostFtdcTransferQryDetailRspField_Flag_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcTransferQryDetailRspField_swiginit(self, _thosttraderapi.new_CThostFtdcTransferQryDetailRspField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcTransferQryDetailRspField

# Register CThostFtdcTransferQryDetailRspField in _thosttraderapi:
_thosttraderapi.CThostFtdcTransferQryDetailRspField_swigregister(CThostFtdcTransferQryDetailRspField)

class CThostFtdcRspInfoField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    ErrorID = property(_thosttraderapi.CThostFtdcRspInfoField_ErrorID_get, _thosttraderapi.CThostFtdcRspInfoField_ErrorID_set)
    ErrorMsg = property(_thosttraderapi.CThostFtdcRspInfoField_ErrorMsg_get, _thosttraderapi.CThostFtdcRspInfoField_ErrorMsg_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcRspInfoField_swiginit(self, _thosttraderapi.new_CThostFtdcRspInfoField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcRspInfoField

# Register CThostFtdcRspInfoField in _thosttraderapi:
_thosttraderapi.CThostFtdcRspInfoField_swigregister(CThostFtdcRspInfoField)

class CThostFtdcExchangeField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    ExchangeID = property(_thosttraderapi.CThostFtdcExchangeField_ExchangeID_get, _thosttraderapi.CThostFtdcExchangeField_ExchangeID_set)
    ExchangeName = property(_thosttraderapi.CThostFtdcExchangeField_ExchangeName_get, _thosttraderapi.CThostFtdcExchangeField_ExchangeName_set)
    ExchangeProperty = property(_thosttraderapi.CThostFtdcExchangeField_ExchangeProperty_get, _thosttraderapi.CThostFtdcExchangeField_ExchangeProperty_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcExchangeField_swiginit(self, _thosttraderapi.new_CThostFtdcExchangeField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcExchangeField

# Register CThostFtdcExchangeField in _thosttraderapi:
_thosttraderapi.CThostFtdcExchangeField_swigregister(CThostFtdcExchangeField)

class CThostFtdcProductField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    ProductID = property(_thosttraderapi.CThostFtdcProductField_ProductID_get, _thosttraderapi.CThostFtdcProductField_ProductID_set)
    ProductName = property(_thosttraderapi.CThostFtdcProductField_ProductName_get, _thosttraderapi.CThostFtdcProductField_ProductName_set)
    ExchangeID = property(_thosttraderapi.CThostFtdcProductField_ExchangeID_get, _thosttraderapi.CThostFtdcProductField_ExchangeID_set)
    ProductClass = property(_thosttraderapi.CThostFtdcProductField_ProductClass_get, _thosttraderapi.CThostFtdcProductField_ProductClass_set)
    VolumeMultiple = property(_thosttraderapi.CThostFtdcProductField_VolumeMultiple_get, _thosttraderapi.CThostFtdcProductField_VolumeMultiple_set)
    PriceTick = property(_thosttraderapi.CThostFtdcProductField_PriceTick_get, _thosttraderapi.CThostFtdcProductField_PriceTick_set)
    MaxMarketOrderVolume = property(_thosttraderapi.CThostFtdcProductField_MaxMarketOrderVolume_get, _thosttraderapi.CThostFtdcProductField_MaxMarketOrderVolume_set)
    MinMarketOrderVolume = property(_thosttraderapi.CThostFtdcProductField_MinMarketOrderVolume_get, _thosttraderapi.CThostFtdcProductField_MinMarketOrderVolume_set)
    MaxLimitOrderVolume = property(_thosttraderapi.CThostFtdcProductField_MaxLimitOrderVolume_get, _thosttraderapi.CThostFtdcProductField_MaxLimitOrderVolume_set)
    MinLimitOrderVolume = property(_thosttraderapi.CThostFtdcProductField_MinLimitOrderVolume_get, _thosttraderapi.CThostFtdcProductField_MinLimitOrderVolume_set)
    PositionType = property(_thosttraderapi.CThostFtdcProductField_PositionType_get, _thosttraderapi.CThostFtdcProductField_PositionType_set)
    PositionDateType = property(_thosttraderapi.CThostFtdcProductField_PositionDateType_get, _thosttraderapi.CThostFtdcProductField_PositionDateType_set)
    CloseDealType = property(_thosttraderapi.CThostFtdcProductField_CloseDealType_get, _thosttraderapi.CThostFtdcProductField_CloseDealType_set)
    TradeCurrencyID = property(_thosttraderapi.CThostFtdcProductField_TradeCurrencyID_get, _thosttraderapi.CThostFtdcProductField_TradeCurrencyID_set)
    MortgageFundUseRange = property(_thosttraderapi.CThostFtdcProductField_MortgageFundUseRange_get, _thosttraderapi.CThostFtdcProductField_MortgageFundUseRange_set)
    ExchangeProductID = property(_thosttraderapi.CThostFtdcProductField_ExchangeProductID_get, _thosttraderapi.CThostFtdcProductField_ExchangeProductID_set)
    UnderlyingMultiple = property(_thosttraderapi.CThostFtdcProductField_UnderlyingMultiple_get, _thosttraderapi.CThostFtdcProductField_UnderlyingMultiple_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcProductField_swiginit(self, _thosttraderapi.new_CThostFtdcProductField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcProductField

# Register CThostFtdcProductField in _thosttraderapi:
_thosttraderapi.CThostFtdcProductField_swigregister(CThostFtdcProductField)

class CThostFtdcInstrumentField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    InstrumentID = property(_thosttraderapi.CThostFtdcInstrumentField_InstrumentID_get, _thosttraderapi.CThostFtdcInstrumentField_InstrumentID_set)
    ExchangeID = property(_thosttraderapi.CThostFtdcInstrumentField_ExchangeID_get, _thosttraderapi.CThostFtdcInstrumentField_ExchangeID_set)
    InstrumentName = property(_thosttraderapi.CThostFtdcInstrumentField_InstrumentName_get, _thosttraderapi.CThostFtdcInstrumentField_InstrumentName_set)
    ExchangeInstID = property(_thosttraderapi.CThostFtdcInstrumentField_ExchangeInstID_get, _thosttraderapi.CThostFtdcInstrumentField_ExchangeInstID_set)
    ProductID = property(_thosttraderapi.CThostFtdcInstrumentField_ProductID_get, _thosttraderapi.CThostFtdcInstrumentField_ProductID_set)
    ProductClass = property(_thosttraderapi.CThostFtdcInstrumentField_ProductClass_get, _thosttraderapi.CThostFtdcInstrumentField_ProductClass_set)
    DeliveryYear = property(_thosttraderapi.CThostFtdcInstrumentField_DeliveryYear_get, _thosttraderapi.CThostFtdcInstrumentField_DeliveryYear_set)
    DeliveryMonth = property(_thosttraderapi.CThostFtdcInstrumentField_DeliveryMonth_get, _thosttraderapi.CThostFtdcInstrumentField_DeliveryMonth_set)
    MaxMarketOrderVolume = property(_thosttraderapi.CThostFtdcInstrumentField_MaxMarketOrderVolume_get, _thosttraderapi.CThostFtdcInstrumentField_MaxMarketOrderVolume_set)
    MinMarketOrderVolume = property(_thosttraderapi.CThostFtdcInstrumentField_MinMarketOrderVolume_get, _thosttraderapi.CThostFtdcInstrumentField_MinMarketOrderVolume_set)
    MaxLimitOrderVolume = property(_thosttraderapi.CThostFtdcInstrumentField_MaxLimitOrderVolume_get, _thosttraderapi.CThostFtdcInstrumentField_MaxLimitOrderVolume_set)
    MinLimitOrderVolume = property(_thosttraderapi.CThostFtdcInstrumentField_MinLimitOrderVolume_get, _thosttraderapi.CThostFtdcInstrumentField_MinLimitOrderVolume_set)
    VolumeMultiple = property(_thosttraderapi.CThostFtdcInstrumentField_VolumeMultiple_get, _thosttraderapi.CThostFtdcInstrumentField_VolumeMultiple_set)
    PriceTick = property(_thosttraderapi.CThostFtdcInstrumentField_PriceTick_get, _thosttraderapi.CThostFtdcInstrumentField_PriceTick_set)
    CreateDate = property(_thosttraderapi.CThostFtdcInstrumentField_CreateDate_get, _thosttraderapi.CThostFtdcInstrumentField_CreateDate_set)
    OpenDate = property(_thosttraderapi.CThostFtdcInstrumentField_OpenDate_get, _thosttraderapi.CThostFtdcInstrumentField_OpenDate_set)
    ExpireDate = property(_thosttraderapi.CThostFtdcInstrumentField_ExpireDate_get, _thosttraderapi.CThostFtdcInstrumentField_ExpireDate_set)
    StartDelivDate = property(_thosttraderapi.CThostFtdcInstrumentField_StartDelivDate_get, _thosttraderapi.CThostFtdcInstrumentField_StartDelivDate_set)
    EndDelivDate = property(_thosttraderapi.CThostFtdcInstrumentField_EndDelivDate_get, _thosttraderapi.CThostFtdcInstrumentField_EndDelivDate_set)
    InstLifePhase = property(_thosttraderapi.CThostFtdcInstrumentField_InstLifePhase_get, _thosttraderapi.CThostFtdcInstrumentField_InstLifePhase_set)
    IsTrading = property(_thosttraderapi.CThostFtdcInstrumentField_IsTrading_get, _thosttraderapi.CThostFtdcInstrumentField_IsTrading_set)
    PositionType = property(_thosttraderapi.CThostFtdcInstrumentField_PositionType_get, _thosttraderapi.CThostFtdcInstrumentField_PositionType_set)
    PositionDateType = property(_thosttraderapi.CThostFtdcInstrumentField_PositionDateType_get, _thosttraderapi.CThostFtdcInstrumentField_PositionDateType_set)
    LongMarginRatio = property(_thosttraderapi.CThostFtdcInstrumentField_LongMarginRatio_get, _thosttraderapi.CThostFtdcInstrumentField_LongMarginRatio_set)
    ShortMarginRatio = property(_thosttraderapi.CThostFtdcInstrumentField_ShortMarginRatio_get, _thosttraderapi.CThostFtdcInstrumentField_ShortMarginRatio_set)
    MaxMarginSideAlgorithm = property(_thosttraderapi.CThostFtdcInstrumentField_MaxMarginSideAlgorithm_get, _thosttraderapi.CThostFtdcInstrumentField_MaxMarginSideAlgorithm_set)
    UnderlyingInstrID = property(_thosttraderapi.CThostFtdcInstrumentField_UnderlyingInstrID_get, _thosttraderapi.CThostFtdcInstrumentField_UnderlyingInstrID_set)
    StrikePrice = property(_thosttraderapi.CThostFtdcInstrumentField_StrikePrice_get, _thosttraderapi.CThostFtdcInstrumentField_StrikePrice_set)
    OptionsType = property(_thosttraderapi.CThostFtdcInstrumentField_OptionsType_get, _thosttraderapi.CThostFtdcInstrumentField_OptionsType_set)
    UnderlyingMultiple = property(_thosttraderapi.CThostFtdcInstrumentField_UnderlyingMultiple_get, _thosttraderapi.CThostFtdcInstrumentField_UnderlyingMultiple_set)
    CombinationType = property(_thosttraderapi.CThostFtdcInstrumentField_CombinationType_get, _thosttraderapi.CThostFtdcInstrumentField_CombinationType_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcInstrumentField_swiginit(self, _thosttraderapi.new_CThostFtdcInstrumentField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcInstrumentField

# Register CThostFtdcInstrumentField in _thosttraderapi:
_thosttraderapi.CThostFtdcInstrumentField_swigregister(CThostFtdcInstrumentField)

class CThostFtdcBrokerField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thosttraderapi.CThostFtdcBrokerField_BrokerID_get, _thosttraderapi.CThostFtdcBrokerField_BrokerID_set)
    BrokerAbbr = property(_thosttraderapi.CThostFtdcBrokerField_BrokerAbbr_get, _thosttraderapi.CThostFtdcBrokerField_BrokerAbbr_set)
    BrokerName = property(_thosttraderapi.CThostFtdcBrokerField_BrokerName_get, _thosttraderapi.CThostFtdcBrokerField_BrokerName_set)
    IsActive = property(_thosttraderapi.CThostFtdcBrokerField_IsActive_get, _thosttraderapi.CThostFtdcBrokerField_IsActive_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcBrokerField_swiginit(self, _thosttraderapi.new_CThostFtdcBrokerField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcBrokerField

# Register CThostFtdcBrokerField in _thosttraderapi:
_thosttraderapi.CThostFtdcBrokerField_swigregister(CThostFtdcBrokerField)

class CThostFtdcTraderField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    ExchangeID = property(_thosttraderapi.CThostFtdcTraderField_ExchangeID_get, _thosttraderapi.CThostFtdcTraderField_ExchangeID_set)
    TraderID = property(_thosttraderapi.CThostFtdcTraderField_TraderID_get, _thosttraderapi.CThostFtdcTraderField_TraderID_set)
    ParticipantID = property(_thosttraderapi.CThostFtdcTraderField_ParticipantID_get, _thosttraderapi.CThostFtdcTraderField_ParticipantID_set)
    Password = property(_thosttraderapi.CThostFtdcTraderField_Password_get, _thosttraderapi.CThostFtdcTraderField_Password_set)
    InstallCount = property(_thosttraderapi.CThostFtdcTraderField_InstallCount_get, _thosttraderapi.CThostFtdcTraderField_InstallCount_set)
    BrokerID = property(_thosttraderapi.CThostFtdcTraderField_BrokerID_get, _thosttraderapi.CThostFtdcTraderField_BrokerID_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcTraderField_swiginit(self, _thosttraderapi.new_CThostFtdcTraderField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcTraderField

# Register CThostFtdcTraderField in _thosttraderapi:
_thosttraderapi.CThostFtdcTraderField_swigregister(CThostFtdcTraderField)

class CThostFtdcInvestorField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    InvestorID = property(_thosttraderapi.CThostFtdcInvestorField_InvestorID_get, _thosttraderapi.CThostFtdcInvestorField_InvestorID_set)
    BrokerID = property(_thosttraderapi.CThostFtdcInvestorField_BrokerID_get, _thosttraderapi.CThostFtdcInvestorField_BrokerID_set)
    InvestorGroupID = property(_thosttraderapi.CThostFtdcInvestorField_InvestorGroupID_get, _thosttraderapi.CThostFtdcInvestorField_InvestorGroupID_set)
    InvestorName = property(_thosttraderapi.CThostFtdcInvestorField_InvestorName_get, _thosttraderapi.CThostFtdcInvestorField_InvestorName_set)
    IdentifiedCardType = property(_thosttraderapi.CThostFtdcInvestorField_IdentifiedCardType_get, _thosttraderapi.CThostFtdcInvestorField_IdentifiedCardType_set)
    IdentifiedCardNo = property(_thosttraderapi.CThostFtdcInvestorField_IdentifiedCardNo_get, _thosttraderapi.CThostFtdcInvestorField_IdentifiedCardNo_set)
    IsActive = property(_thosttraderapi.CThostFtdcInvestorField_IsActive_get, _thosttraderapi.CThostFtdcInvestorField_IsActive_set)
    Telephone = property(_thosttraderapi.CThostFtdcInvestorField_Telephone_get, _thosttraderapi.CThostFtdcInvestorField_Telephone_set)
    Address = property(_thosttraderapi.CThostFtdcInvestorField_Address_get, _thosttraderapi.CThostFtdcInvestorField_Address_set)
    OpenDate = property(_thosttraderapi.CThostFtdcInvestorField_OpenDate_get, _thosttraderapi.CThostFtdcInvestorField_OpenDate_set)
    Mobile = property(_thosttraderapi.CThostFtdcInvestorField_Mobile_get, _thosttraderapi.CThostFtdcInvestorField_Mobile_set)
    CommModelID = property(_thosttraderapi.CThostFtdcInvestorField_CommModelID_get, _thosttraderapi.CThostFtdcInvestorField_CommModelID_set)
    MarginModelID = property(_thosttraderapi.CThostFtdcInvestorField_MarginModelID_get, _thosttraderapi.CThostFtdcInvestorField_MarginModelID_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcInvestorField_swiginit(self, _thosttraderapi.new_CThostFtdcInvestorField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcInvestorField

# Register CThostFtdcInvestorField in _thosttraderapi:
_thosttraderapi.CThostFtdcInvestorField_swigregister(CThostFtdcInvestorField)

class CThostFtdcTradingCodeField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    InvestorID = property(_thosttraderapi.CThostFtdcTradingCodeField_InvestorID_get, _thosttraderapi.CThostFtdcTradingCodeField_InvestorID_set)
    BrokerID = property(_thosttraderapi.CThostFtdcTradingCodeField_BrokerID_get, _thosttraderapi.CThostFtdcTradingCodeField_BrokerID_set)
    ExchangeID = property(_thosttraderapi.CThostFtdcTradingCodeField_ExchangeID_get, _thosttraderapi.CThostFtdcTradingCodeField_ExchangeID_set)
    ClientID = property(_thosttraderapi.CThostFtdcTradingCodeField_ClientID_get, _thosttraderapi.CThostFtdcTradingCodeField_ClientID_set)
    IsActive = property(_thosttraderapi.CThostFtdcTradingCodeField_IsActive_get, _thosttraderapi.CThostFtdcTradingCodeField_IsActive_set)
    ClientIDType = property(_thosttraderapi.CThostFtdcTradingCodeField_ClientIDType_get, _thosttraderapi.CThostFtdcTradingCodeField_ClientIDType_set)
    BranchID = property(_thosttraderapi.CThostFtdcTradingCodeField_BranchID_get, _thosttraderapi.CThostFtdcTradingCodeField_BranchID_set)
    BizType = property(_thosttraderapi.CThostFtdcTradingCodeField_BizType_get, _thosttraderapi.CThostFtdcTradingCodeField_BizType_set)
    InvestUnitID = property(_thosttraderapi.CThostFtdcTradingCodeField_InvestUnitID_get, _thosttraderapi.CThostFtdcTradingCodeField_InvestUnitID_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcTradingCodeField_swiginit(self, _thosttraderapi.new_CThostFtdcTradingCodeField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcTradingCodeField

# Register CThostFtdcTradingCodeField in _thosttraderapi:
_thosttraderapi.CThostFtdcTradingCodeField_swigregister(CThostFtdcTradingCodeField)

class CThostFtdcPartBrokerField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thosttraderapi.CThostFtdcPartBrokerField_BrokerID_get, _thosttraderapi.CThostFtdcPartBrokerField_BrokerID_set)
    ExchangeID = property(_thosttraderapi.CThostFtdcPartBrokerField_ExchangeID_get, _thosttraderapi.CThostFtdcPartBrokerField_ExchangeID_set)
    ParticipantID = property(_thosttraderapi.CThostFtdcPartBrokerField_ParticipantID_get, _thosttraderapi.CThostFtdcPartBrokerField_ParticipantID_set)
    IsActive = property(_thosttraderapi.CThostFtdcPartBrokerField_IsActive_get, _thosttraderapi.CThostFtdcPartBrokerField_IsActive_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcPartBrokerField_swiginit(self, _thosttraderapi.new_CThostFtdcPartBrokerField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcPartBrokerField

# Register CThostFtdcPartBrokerField in _thosttraderapi:
_thosttraderapi.CThostFtdcPartBrokerField_swigregister(CThostFtdcPartBrokerField)

class CThostFtdcSuperUserField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    UserID = property(_thosttraderapi.CThostFtdcSuperUserField_UserID_get, _thosttraderapi.CThostFtdcSuperUserField_UserID_set)
    UserName = property(_thosttraderapi.CThostFtdcSuperUserField_UserName_get, _thosttraderapi.CThostFtdcSuperUserField_UserName_set)
    Password = property(_thosttraderapi.CThostFtdcSuperUserField_Password_get, _thosttraderapi.CThostFtdcSuperUserField_Password_set)
    IsActive = property(_thosttraderapi.CThostFtdcSuperUserField_IsActive_get, _thosttraderapi.CThostFtdcSuperUserField_IsActive_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcSuperUserField_swiginit(self, _thosttraderapi.new_CThostFtdcSuperUserField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcSuperUserField

# Register CThostFtdcSuperUserField in _thosttraderapi:
_thosttraderapi.CThostFtdcSuperUserField_swigregister(CThostFtdcSuperUserField)

class CThostFtdcSuperUserFunctionField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    UserID = property(_thosttraderapi.CThostFtdcSuperUserFunctionField_UserID_get, _thosttraderapi.CThostFtdcSuperUserFunctionField_UserID_set)
    FunctionCode = property(_thosttraderapi.CThostFtdcSuperUserFunctionField_FunctionCode_get, _thosttraderapi.CThostFtdcSuperUserFunctionField_FunctionCode_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcSuperUserFunctionField_swiginit(self, _thosttraderapi.new_CThostFtdcSuperUserFunctionField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcSuperUserFunctionField

# Register CThostFtdcSuperUserFunctionField in _thosttraderapi:
_thosttraderapi.CThostFtdcSuperUserFunctionField_swigregister(CThostFtdcSuperUserFunctionField)

class CThostFtdcInvestorGroupField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thosttraderapi.CThostFtdcInvestorGroupField_BrokerID_get, _thosttraderapi.CThostFtdcInvestorGroupField_BrokerID_set)
    InvestorGroupID = property(_thosttraderapi.CThostFtdcInvestorGroupField_InvestorGroupID_get, _thosttraderapi.CThostFtdcInvestorGroupField_InvestorGroupID_set)
    InvestorGroupName = property(_thosttraderapi.CThostFtdcInvestorGroupField_InvestorGroupName_get, _thosttraderapi.CThostFtdcInvestorGroupField_InvestorGroupName_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcInvestorGroupField_swiginit(self, _thosttraderapi.new_CThostFtdcInvestorGroupField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcInvestorGroupField

# Register CThostFtdcInvestorGroupField in _thosttraderapi:
_thosttraderapi.CThostFtdcInvestorGroupField_swigregister(CThostFtdcInvestorGroupField)

class CThostFtdcTradingAccountField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thosttraderapi.CThostFtdcTradingAccountField_BrokerID_get, _thosttraderapi.CThostFtdcTradingAccountField_BrokerID_set)
    AccountID = property(_thosttraderapi.CThostFtdcTradingAccountField_AccountID_get, _thosttraderapi.CThostFtdcTradingAccountField_AccountID_set)
    PreMortgage = property(_thosttraderapi.CThostFtdcTradingAccountField_PreMortgage_get, _thosttraderapi.CThostFtdcTradingAccountField_PreMortgage_set)
    PreCredit = property(_thosttraderapi.CThostFtdcTradingAccountField_PreCredit_get, _thosttraderapi.CThostFtdcTradingAccountField_PreCredit_set)
    PreDeposit = property(_thosttraderapi.CThostFtdcTradingAccountField_PreDeposit_get, _thosttraderapi.CThostFtdcTradingAccountField_PreDeposit_set)
    PreBalance = property(_thosttraderapi.CThostFtdcTradingAccountField_PreBalance_get, _thosttraderapi.CThostFtdcTradingAccountField_PreBalance_set)
    PreMargin = property(_thosttraderapi.CThostFtdcTradingAccountField_PreMargin_get, _thosttraderapi.CThostFtdcTradingAccountField_PreMargin_set)
    InterestBase = property(_thosttraderapi.CThostFtdcTradingAccountField_InterestBase_get, _thosttraderapi.CThostFtdcTradingAccountField_InterestBase_set)
    Interest = property(_thosttraderapi.CThostFtdcTradingAccountField_Interest_get, _thosttraderapi.CThostFtdcTradingAccountField_Interest_set)
    Deposit = property(_thosttraderapi.CThostFtdcTradingAccountField_Deposit_get, _thosttraderapi.CThostFtdcTradingAccountField_Deposit_set)
    Withdraw = property(_thosttraderapi.CThostFtdcTradingAccountField_Withdraw_get, _thosttraderapi.CThostFtdcTradingAccountField_Withdraw_set)
    FrozenMargin = property(_thosttraderapi.CThostFtdcTradingAccountField_FrozenMargin_get, _thosttraderapi.CThostFtdcTradingAccountField_FrozenMargin_set)
    FrozenCash = property(_thosttraderapi.CThostFtdcTradingAccountField_FrozenCash_get, _thosttraderapi.CThostFtdcTradingAccountField_FrozenCash_set)
    FrozenCommission = property(_thosttraderapi.CThostFtdcTradingAccountField_FrozenCommission_get, _thosttraderapi.CThostFtdcTradingAccountField_FrozenCommission_set)
    CurrMargin = property(_thosttraderapi.CThostFtdcTradingAccountField_CurrMargin_get, _thosttraderapi.CThostFtdcTradingAccountField_CurrMargin_set)
    CashIn = property(_thosttraderapi.CThostFtdcTradingAccountField_CashIn_get, _thosttraderapi.CThostFtdcTradingAccountField_CashIn_set)
    Commission = property(_thosttraderapi.CThostFtdcTradingAccountField_Commission_get, _thosttraderapi.CThostFtdcTradingAccountField_Commission_set)
    CloseProfit = property(_thosttraderapi.CThostFtdcTradingAccountField_CloseProfit_get, _thosttraderapi.CThostFtdcTradingAccountField_CloseProfit_set)
    PositionProfit = property(_thosttraderapi.CThostFtdcTradingAccountField_PositionProfit_get, _thosttraderapi.CThostFtdcTradingAccountField_PositionProfit_set)
    Balance = property(_thosttraderapi.CThostFtdcTradingAccountField_Balance_get, _thosttraderapi.CThostFtdcTradingAccountField_Balance_set)
    Available = property(_thosttraderapi.CThostFtdcTradingAccountField_Available_get, _thosttraderapi.CThostFtdcTradingAccountField_Available_set)
    WithdrawQuota = property(_thosttraderapi.CThostFtdcTradingAccountField_WithdrawQuota_get, _thosttraderapi.CThostFtdcTradingAccountField_WithdrawQuota_set)
    Reserve = property(_thosttraderapi.CThostFtdcTradingAccountField_Reserve_get, _thosttraderapi.CThostFtdcTradingAccountField_Reserve_set)
    TradingDay = property(_thosttraderapi.CThostFtdcTradingAccountField_TradingDay_get, _thosttraderapi.CThostFtdcTradingAccountField_TradingDay_set)
    SettlementID = property(_thosttraderapi.CThostFtdcTradingAccountField_SettlementID_get, _thosttraderapi.CThostFtdcTradingAccountField_SettlementID_set)
    Credit = property(_thosttraderapi.CThostFtdcTradingAccountField_Credit_get, _thosttraderapi.CThostFtdcTradingAccountField_Credit_set)
    Mortgage = property(_thosttraderapi.CThostFtdcTradingAccountField_Mortgage_get, _thosttraderapi.CThostFtdcTradingAccountField_Mortgage_set)
    ExchangeMargin = property(_thosttraderapi.CThostFtdcTradingAccountField_ExchangeMargin_get, _thosttraderapi.CThostFtdcTradingAccountField_ExchangeMargin_set)
    DeliveryMargin = property(_thosttraderapi.CThostFtdcTradingAccountField_DeliveryMargin_get, _thosttraderapi.CThostFtdcTradingAccountField_DeliveryMargin_set)
    ExchangeDeliveryMargin = property(_thosttraderapi.CThostFtdcTradingAccountField_ExchangeDeliveryMargin_get, _thosttraderapi.CThostFtdcTradingAccountField_ExchangeDeliveryMargin_set)
    ReserveBalance = property(_thosttraderapi.CThostFtdcTradingAccountField_ReserveBalance_get, _thosttraderapi.CThostFtdcTradingAccountField_ReserveBalance_set)
    CurrencyID = property(_thosttraderapi.CThostFtdcTradingAccountField_CurrencyID_get, _thosttraderapi.CThostFtdcTradingAccountField_CurrencyID_set)
    PreFundMortgageIn = property(_thosttraderapi.CThostFtdcTradingAccountField_PreFundMortgageIn_get, _thosttraderapi.CThostFtdcTradingAccountField_PreFundMortgageIn_set)
    PreFundMortgageOut = property(_thosttraderapi.CThostFtdcTradingAccountField_PreFundMortgageOut_get, _thosttraderapi.CThostFtdcTradingAccountField_PreFundMortgageOut_set)
    FundMortgageIn = property(_thosttraderapi.CThostFtdcTradingAccountField_FundMortgageIn_get, _thosttraderapi.CThostFtdcTradingAccountField_FundMortgageIn_set)
    FundMortgageOut = property(_thosttraderapi.CThostFtdcTradingAccountField_FundMortgageOut_get, _thosttraderapi.CThostFtdcTradingAccountField_FundMortgageOut_set)
    FundMortgageAvailable = property(_thosttraderapi.CThostFtdcTradingAccountField_FundMortgageAvailable_get, _thosttraderapi.CThostFtdcTradingAccountField_FundMortgageAvailable_set)
    MortgageableFund = property(_thosttraderapi.CThostFtdcTradingAccountField_MortgageableFund_get, _thosttraderapi.CThostFtdcTradingAccountField_MortgageableFund_set)
    SpecProductMargin = property(_thosttraderapi.CThostFtdcTradingAccountField_SpecProductMargin_get, _thosttraderapi.CThostFtdcTradingAccountField_SpecProductMargin_set)
    SpecProductFrozenMargin = property(_thosttraderapi.CThostFtdcTradingAccountField_SpecProductFrozenMargin_get, _thosttraderapi.CThostFtdcTradingAccountField_SpecProductFrozenMargin_set)
    SpecProductCommission = property(_thosttraderapi.CThostFtdcTradingAccountField_SpecProductCommission_get, _thosttraderapi.CThostFtdcTradingAccountField_SpecProductCommission_set)
    SpecProductFrozenCommission = property(_thosttraderapi.CThostFtdcTradingAccountField_SpecProductFrozenCommission_get, _thosttraderapi.CThostFtdcTradingAccountField_SpecProductFrozenCommission_set)
    SpecProductPositionProfit = property(_thosttraderapi.CThostFtdcTradingAccountField_SpecProductPositionProfit_get, _thosttraderapi.CThostFtdcTradingAccountField_SpecProductPositionProfit_set)
    SpecProductCloseProfit = property(_thosttraderapi.CThostFtdcTradingAccountField_SpecProductCloseProfit_get, _thosttraderapi.CThostFtdcTradingAccountField_SpecProductCloseProfit_set)
    SpecProductPositionProfitByAlg = property(_thosttraderapi.CThostFtdcTradingAccountField_SpecProductPositionProfitByAlg_get, _thosttraderapi.CThostFtdcTradingAccountField_SpecProductPositionProfitByAlg_set)
    SpecProductExchangeMargin = property(_thosttraderapi.CThostFtdcTradingAccountField_SpecProductExchangeMargin_get, _thosttraderapi.CThostFtdcTradingAccountField_SpecProductExchangeMargin_set)
    BizType = property(_thosttraderapi.CThostFtdcTradingAccountField_BizType_get, _thosttraderapi.CThostFtdcTradingAccountField_BizType_set)
    FrozenSwap = property(_thosttraderapi.CThostFtdcTradingAccountField_FrozenSwap_get, _thosttraderapi.CThostFtdcTradingAccountField_FrozenSwap_set)
    RemainSwap = property(_thosttraderapi.CThostFtdcTradingAccountField_RemainSwap_get, _thosttraderapi.CThostFtdcTradingAccountField_RemainSwap_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcTradingAccountField_swiginit(self, _thosttraderapi.new_CThostFtdcTradingAccountField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcTradingAccountField

# Register CThostFtdcTradingAccountField in _thosttraderapi:
_thosttraderapi.CThostFtdcTradingAccountField_swigregister(CThostFtdcTradingAccountField)

class CThostFtdcInvestorPositionField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    InstrumentID = property(_thosttraderapi.CThostFtdcInvestorPositionField_InstrumentID_get, _thosttraderapi.CThostFtdcInvestorPositionField_InstrumentID_set)
    BrokerID = property(_thosttraderapi.CThostFtdcInvestorPositionField_BrokerID_get, _thosttraderapi.CThostFtdcInvestorPositionField_BrokerID_set)
    InvestorID = property(_thosttraderapi.CThostFtdcInvestorPositionField_InvestorID_get, _thosttraderapi.CThostFtdcInvestorPositionField_InvestorID_set)
    PosiDirection = property(_thosttraderapi.CThostFtdcInvestorPositionField_PosiDirection_get, _thosttraderapi.CThostFtdcInvestorPositionField_PosiDirection_set)
    HedgeFlag = property(_thosttraderapi.CThostFtdcInvestorPositionField_HedgeFlag_get, _thosttraderapi.CThostFtdcInvestorPositionField_HedgeFlag_set)
    PositionDate = property(_thosttraderapi.CThostFtdcInvestorPositionField_PositionDate_get, _thosttraderapi.CThostFtdcInvestorPositionField_PositionDate_set)
    YdPosition = property(_thosttraderapi.CThostFtdcInvestorPositionField_YdPosition_get, _thosttraderapi.CThostFtdcInvestorPositionField_YdPosition_set)
    Position = property(_thosttraderapi.CThostFtdcInvestorPositionField_Position_get, _thosttraderapi.CThostFtdcInvestorPositionField_Position_set)
    LongFrozen = property(_thosttraderapi.CThostFtdcInvestorPositionField_LongFrozen_get, _thosttraderapi.CThostFtdcInvestorPositionField_LongFrozen_set)
    ShortFrozen = property(_thosttraderapi.CThostFtdcInvestorPositionField_ShortFrozen_get, _thosttraderapi.CThostFtdcInvestorPositionField_ShortFrozen_set)
    LongFrozenAmount = property(_thosttraderapi.CThostFtdcInvestorPositionField_LongFrozenAmount_get, _thosttraderapi.CThostFtdcInvestorPositionField_LongFrozenAmount_set)
    ShortFrozenAmount = property(_thosttraderapi.CThostFtdcInvestorPositionField_ShortFrozenAmount_get, _thosttraderapi.CThostFtdcInvestorPositionField_ShortFrozenAmount_set)
    OpenVolume = property(_thosttraderapi.CThostFtdcInvestorPositionField_OpenVolume_get, _thosttraderapi.CThostFtdcInvestorPositionField_OpenVolume_set)
    CloseVolume = property(_thosttraderapi.CThostFtdcInvestorPositionField_CloseVolume_get, _thosttraderapi.CThostFtdcInvestorPositionField_CloseVolume_set)
    OpenAmount = property(_thosttraderapi.CThostFtdcInvestorPositionField_OpenAmount_get, _thosttraderapi.CThostFtdcInvestorPositionField_OpenAmount_set)
    CloseAmount = property(_thosttraderapi.CThostFtdcInvestorPositionField_CloseAmount_get, _thosttraderapi.CThostFtdcInvestorPositionField_CloseAmount_set)
    PositionCost = property(_thosttraderapi.CThostFtdcInvestorPositionField_PositionCost_get, _thosttraderapi.CThostFtdcInvestorPositionField_PositionCost_set)
    PreMargin = property(_thosttraderapi.CThostFtdcInvestorPositionField_PreMargin_get, _thosttraderapi.CThostFtdcInvestorPositionField_PreMargin_set)
    UseMargin = property(_thosttraderapi.CThostFtdcInvestorPositionField_UseMargin_get, _thosttraderapi.CThostFtdcInvestorPositionField_UseMargin_set)
    FrozenMargin = property(_thosttraderapi.CThostFtdcInvestorPositionField_FrozenMargin_get, _thosttraderapi.CThostFtdcInvestorPositionField_FrozenMargin_set)
    FrozenCash = property(_thosttraderapi.CThostFtdcInvestorPositionField_FrozenCash_get, _thosttraderapi.CThostFtdcInvestorPositionField_FrozenCash_set)
    FrozenCommission = property(_thosttraderapi.CThostFtdcInvestorPositionField_FrozenCommission_get, _thosttraderapi.CThostFtdcInvestorPositionField_FrozenCommission_set)
    CashIn = property(_thosttraderapi.CThostFtdcInvestorPositionField_CashIn_get, _thosttraderapi.CThostFtdcInvestorPositionField_CashIn_set)
    Commission = property(_thosttraderapi.CThostFtdcInvestorPositionField_Commission_get, _thosttraderapi.CThostFtdcInvestorPositionField_Commission_set)
    CloseProfit = property(_thosttraderapi.CThostFtdcInvestorPositionField_CloseProfit_get, _thosttraderapi.CThostFtdcInvestorPositionField_CloseProfit_set)
    PositionProfit = property(_thosttraderapi.CThostFtdcInvestorPositionField_PositionProfit_get, _thosttraderapi.CThostFtdcInvestorPositionField_PositionProfit_set)
    PreSettlementPrice = property(_thosttraderapi.CThostFtdcInvestorPositionField_PreSettlementPrice_get, _thosttraderapi.CThostFtdcInvestorPositionField_PreSettlementPrice_set)
    SettlementPrice = property(_thosttraderapi.CThostFtdcInvestorPositionField_SettlementPrice_get, _thosttraderapi.CThostFtdcInvestorPositionField_SettlementPrice_set)
    TradingDay = property(_thosttraderapi.CThostFtdcInvestorPositionField_TradingDay_get, _thosttraderapi.CThostFtdcInvestorPositionField_TradingDay_set)
    SettlementID = property(_thosttraderapi.CThostFtdcInvestorPositionField_SettlementID_get, _thosttraderapi.CThostFtdcInvestorPositionField_SettlementID_set)
    OpenCost = property(_thosttraderapi.CThostFtdcInvestorPositionField_OpenCost_get, _thosttraderapi.CThostFtdcInvestorPositionField_OpenCost_set)
    ExchangeMargin = property(_thosttraderapi.CThostFtdcInvestorPositionField_ExchangeMargin_get, _thosttraderapi.CThostFtdcInvestorPositionField_ExchangeMargin_set)
    CombPosition = property(_thosttraderapi.CThostFtdcInvestorPositionField_CombPosition_get, _thosttraderapi.CThostFtdcInvestorPositionField_CombPosition_set)
    CombLongFrozen = property(_thosttraderapi.CThostFtdcInvestorPositionField_CombLongFrozen_get, _thosttraderapi.CThostFtdcInvestorPositionField_CombLongFrozen_set)
    CombShortFrozen = property(_thosttraderapi.CThostFtdcInvestorPositionField_CombShortFrozen_get, _thosttraderapi.CThostFtdcInvestorPositionField_CombShortFrozen_set)
    CloseProfitByDate = property(_thosttraderapi.CThostFtdcInvestorPositionField_CloseProfitByDate_get, _thosttraderapi.CThostFtdcInvestorPositionField_CloseProfitByDate_set)
    CloseProfitByTrade = property(_thosttraderapi.CThostFtdcInvestorPositionField_CloseProfitByTrade_get, _thosttraderapi.CThostFtdcInvestorPositionField_CloseProfitByTrade_set)
    TodayPosition = property(_thosttraderapi.CThostFtdcInvestorPositionField_TodayPosition_get, _thosttraderapi.CThostFtdcInvestorPositionField_TodayPosition_set)
    MarginRateByMoney = property(_thosttraderapi.CThostFtdcInvestorPositionField_MarginRateByMoney_get, _thosttraderapi.CThostFtdcInvestorPositionField_MarginRateByMoney_set)
    MarginRateByVolume = property(_thosttraderapi.CThostFtdcInvestorPositionField_MarginRateByVolume_get, _thosttraderapi.CThostFtdcInvestorPositionField_MarginRateByVolume_set)
    StrikeFrozen = property(_thosttraderapi.CThostFtdcInvestorPositionField_StrikeFrozen_get, _thosttraderapi.CThostFtdcInvestorPositionField_StrikeFrozen_set)
    StrikeFrozenAmount = property(_thosttraderapi.CThostFtdcInvestorPositionField_StrikeFrozenAmount_get, _thosttraderapi.CThostFtdcInvestorPositionField_StrikeFrozenAmount_set)
    AbandonFrozen = property(_thosttraderapi.CThostFtdcInvestorPositionField_AbandonFrozen_get, _thosttraderapi.CThostFtdcInvestorPositionField_AbandonFrozen_set)
    ExchangeID = property(_thosttraderapi.CThostFtdcInvestorPositionField_ExchangeID_get, _thosttraderapi.CThostFtdcInvestorPositionField_ExchangeID_set)
    YdStrikeFrozen = property(_thosttraderapi.CThostFtdcInvestorPositionField_YdStrikeFrozen_get, _thosttraderapi.CThostFtdcInvestorPositionField_YdStrikeFrozen_set)
    InvestUnitID = property(_thosttraderapi.CThostFtdcInvestorPositionField_InvestUnitID_get, _thosttraderapi.CThostFtdcInvestorPositionField_InvestUnitID_set)
    PositionCostOffset = property(_thosttraderapi.CThostFtdcInvestorPositionField_PositionCostOffset_get, _thosttraderapi.CThostFtdcInvestorPositionField_PositionCostOffset_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcInvestorPositionField_swiginit(self, _thosttraderapi.new_CThostFtdcInvestorPositionField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcInvestorPositionField

# Register CThostFtdcInvestorPositionField in _thosttraderapi:
_thosttraderapi.CThostFtdcInvestorPositionField_swigregister(CThostFtdcInvestorPositionField)

class CThostFtdcInstrumentMarginRateField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    InstrumentID = property(_thosttraderapi.CThostFtdcInstrumentMarginRateField_InstrumentID_get, _thosttraderapi.CThostFtdcInstrumentMarginRateField_InstrumentID_set)
    InvestorRange = property(_thosttraderapi.CThostFtdcInstrumentMarginRateField_InvestorRange_get, _thosttraderapi.CThostFtdcInstrumentMarginRateField_InvestorRange_set)
    BrokerID = property(_thosttraderapi.CThostFtdcInstrumentMarginRateField_BrokerID_get, _thosttraderapi.CThostFtdcInstrumentMarginRateField_BrokerID_set)
    InvestorID = property(_thosttraderapi.CThostFtdcInstrumentMarginRateField_InvestorID_get, _thosttraderapi.CThostFtdcInstrumentMarginRateField_InvestorID_set)
    HedgeFlag = property(_thosttraderapi.CThostFtdcInstrumentMarginRateField_HedgeFlag_get, _thosttraderapi.CThostFtdcInstrumentMarginRateField_HedgeFlag_set)
    LongMarginRatioByMoney = property(_thosttraderapi.CThostFtdcInstrumentMarginRateField_LongMarginRatioByMoney_get, _thosttraderapi.CThostFtdcInstrumentMarginRateField_LongMarginRatioByMoney_set)
    LongMarginRatioByVolume = property(_thosttraderapi.CThostFtdcInstrumentMarginRateField_LongMarginRatioByVolume_get, _thosttraderapi.CThostFtdcInstrumentMarginRateField_LongMarginRatioByVolume_set)
    ShortMarginRatioByMoney = property(_thosttraderapi.CThostFtdcInstrumentMarginRateField_ShortMarginRatioByMoney_get, _thosttraderapi.CThostFtdcInstrumentMarginRateField_ShortMarginRatioByMoney_set)
    ShortMarginRatioByVolume = property(_thosttraderapi.CThostFtdcInstrumentMarginRateField_ShortMarginRatioByVolume_get, _thosttraderapi.CThostFtdcInstrumentMarginRateField_ShortMarginRatioByVolume_set)
    IsRelative = property(_thosttraderapi.CThostFtdcInstrumentMarginRateField_IsRelative_get, _thosttraderapi.CThostFtdcInstrumentMarginRateField_IsRelative_set)
    ExchangeID = property(_thosttraderapi.CThostFtdcInstrumentMarginRateField_ExchangeID_get, _thosttraderapi.CThostFtdcInstrumentMarginRateField_ExchangeID_set)
    InvestUnitID = property(_thosttraderapi.CThostFtdcInstrumentMarginRateField_InvestUnitID_get, _thosttraderapi.CThostFtdcInstrumentMarginRateField_InvestUnitID_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcInstrumentMarginRateField_swiginit(self, _thosttraderapi.new_CThostFtdcInstrumentMarginRateField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcInstrumentMarginRateField

# Register CThostFtdcInstrumentMarginRateField in _thosttraderapi:
_thosttraderapi.CThostFtdcInstrumentMarginRateField_swigregister(CThostFtdcInstrumentMarginRateField)

class CThostFtdcInstrumentCommissionRateField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    InstrumentID = property(_thosttraderapi.CThostFtdcInstrumentCommissionRateField_InstrumentID_get, _thosttraderapi.CThostFtdcInstrumentCommissionRateField_InstrumentID_set)
    InvestorRange = property(_thosttraderapi.CThostFtdcInstrumentCommissionRateField_InvestorRange_get, _thosttraderapi.CThostFtdcInstrumentCommissionRateField_InvestorRange_set)
    BrokerID = property(_thosttraderapi.CThostFtdcInstrumentCommissionRateField_BrokerID_get, _thosttraderapi.CThostFtdcInstrumentCommissionRateField_BrokerID_set)
    InvestorID = property(_thosttraderapi.CThostFtdcInstrumentCommissionRateField_InvestorID_get, _thosttraderapi.CThostFtdcInstrumentCommissionRateField_InvestorID_set)
    OpenRatioByMoney = property(_thosttraderapi.CThostFtdcInstrumentCommissionRateField_OpenRatioByMoney_get, _thosttraderapi.CThostFtdcInstrumentCommissionRateField_OpenRatioByMoney_set)
    OpenRatioByVolume = property(_thosttraderapi.CThostFtdcInstrumentCommissionRateField_OpenRatioByVolume_get, _thosttraderapi.CThostFtdcInstrumentCommissionRateField_OpenRatioByVolume_set)
    CloseRatioByMoney = property(_thosttraderapi.CThostFtdcInstrumentCommissionRateField_CloseRatioByMoney_get, _thosttraderapi.CThostFtdcInstrumentCommissionRateField_CloseRatioByMoney_set)
    CloseRatioByVolume = property(_thosttraderapi.CThostFtdcInstrumentCommissionRateField_CloseRatioByVolume_get, _thosttraderapi.CThostFtdcInstrumentCommissionRateField_CloseRatioByVolume_set)
    CloseTodayRatioByMoney = property(_thosttraderapi.CThostFtdcInstrumentCommissionRateField_CloseTodayRatioByMoney_get, _thosttraderapi.CThostFtdcInstrumentCommissionRateField_CloseTodayRatioByMoney_set)
    CloseTodayRatioByVolume = property(_thosttraderapi.CThostFtdcInstrumentCommissionRateField_CloseTodayRatioByVolume_get, _thosttraderapi.CThostFtdcInstrumentCommissionRateField_CloseTodayRatioByVolume_set)
    ExchangeID = property(_thosttraderapi.CThostFtdcInstrumentCommissionRateField_ExchangeID_get, _thosttraderapi.CThostFtdcInstrumentCommissionRateField_ExchangeID_set)
    BizType = property(_thosttraderapi.CThostFtdcInstrumentCommissionRateField_BizType_get, _thosttraderapi.CThostFtdcInstrumentCommissionRateField_BizType_set)
    InvestUnitID = property(_thosttraderapi.CThostFtdcInstrumentCommissionRateField_InvestUnitID_get, _thosttraderapi.CThostFtdcInstrumentCommissionRateField_InvestUnitID_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcInstrumentCommissionRateField_swiginit(self, _thosttraderapi.new_CThostFtdcInstrumentCommissionRateField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcInstrumentCommissionRateField

# Register CThostFtdcInstrumentCommissionRateField in _thosttraderapi:
_thosttraderapi.CThostFtdcInstrumentCommissionRateField_swigregister(CThostFtdcInstrumentCommissionRateField)

class CThostFtdcDepthMarketDataField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    TradingDay = property(_thosttraderapi.CThostFtdcDepthMarketDataField_TradingDay_get, _thosttraderapi.CThostFtdcDepthMarketDataField_TradingDay_set)
    InstrumentID = property(_thosttraderapi.CThostFtdcDepthMarketDataField_InstrumentID_get, _thosttraderapi.CThostFtdcDepthMarketDataField_InstrumentID_set)
    ExchangeID = property(_thosttraderapi.CThostFtdcDepthMarketDataField_ExchangeID_get, _thosttraderapi.CThostFtdcDepthMarketDataField_ExchangeID_set)
    ExchangeInstID = property(_thosttraderapi.CThostFtdcDepthMarketDataField_ExchangeInstID_get, _thosttraderapi.CThostFtdcDepthMarketDataField_ExchangeInstID_set)
    LastPrice = property(_thosttraderapi.CThostFtdcDepthMarketDataField_LastPrice_get, _thosttraderapi.CThostFtdcDepthMarketDataField_LastPrice_set)
    PreSettlementPrice = property(_thosttraderapi.CThostFtdcDepthMarketDataField_PreSettlementPrice_get, _thosttraderapi.CThostFtdcDepthMarketDataField_PreSettlementPrice_set)
    PreClosePrice = property(_thosttraderapi.CThostFtdcDepthMarketDataField_PreClosePrice_get, _thosttraderapi.CThostFtdcDepthMarketDataField_PreClosePrice_set)
    PreOpenInterest = property(_thosttraderapi.CThostFtdcDepthMarketDataField_PreOpenInterest_get, _thosttraderapi.CThostFtdcDepthMarketDataField_PreOpenInterest_set)
    OpenPrice = property(_thosttraderapi.CThostFtdcDepthMarketDataField_OpenPrice_get, _thosttraderapi.CThostFtdcDepthMarketDataField_OpenPrice_set)
    HighestPrice = property(_thosttraderapi.CThostFtdcDepthMarketDataField_HighestPrice_get, _thosttraderapi.CThostFtdcDepthMarketDataField_HighestPrice_set)
    LowestPrice = property(_thosttraderapi.CThostFtdcDepthMarketDataField_LowestPrice_get, _thosttraderapi.CThostFtdcDepthMarketDataField_LowestPrice_set)
    Volume = property(_thosttraderapi.CThostFtdcDepthMarketDataField_Volume_get, _thosttraderapi.CThostFtdcDepthMarketDataField_Volume_set)
    Turnover = property(_thosttraderapi.CThostFtdcDepthMarketDataField_Turnover_get, _thosttraderapi.CThostFtdcDepthMarketDataField_Turnover_set)
    OpenInterest = property(_thosttraderapi.CThostFtdcDepthMarketDataField_OpenInterest_get, _thosttraderapi.CThostFtdcDepthMarketDataField_OpenInterest_set)
    ClosePrice = property(_thosttraderapi.CThostFtdcDepthMarketDataField_ClosePrice_get, _thosttraderapi.CThostFtdcDepthMarketDataField_ClosePrice_set)
    SettlementPrice = property(_thosttraderapi.CThostFtdcDepthMarketDataField_SettlementPrice_get, _thosttraderapi.CThostFtdcDepthMarketDataField_SettlementPrice_set)
    UpperLimitPrice = property(_thosttraderapi.CThostFtdcDepthMarketDataField_UpperLimitPrice_get, _thosttraderapi.CThostFtdcDepthMarketDataField_UpperLimitPrice_set)
    LowerLimitPrice = property(_thosttraderapi.CThostFtdcDepthMarketDataField_LowerLimitPrice_get, _thosttraderapi.CThostFtdcDepthMarketDataField_LowerLimitPrice_set)
    PreDelta = property(_thosttraderapi.CThostFtdcDepthMarketDataField_PreDelta_get, _thosttraderapi.CThostFtdcDepthMarketDataField_PreDelta_set)
    CurrDelta = property(_thosttraderapi.CThostFtdcDepthMarketDataField_CurrDelta_get, _thosttraderapi.CThostFtdcDepthMarketDataField_CurrDelta_set)
    UpdateTime = property(_thosttraderapi.CThostFtdcDepthMarketDataField_UpdateTime_get, _thosttraderapi.CThostFtdcDepthMarketDataField_UpdateTime_set)
    UpdateMillisec = property(_thosttraderapi.CThostFtdcDepthMarketDataField_UpdateMillisec_get, _thosttraderapi.CThostFtdcDepthMarketDataField_UpdateMillisec_set)
    BidPrice1 = property(_thosttraderapi.CThostFtdcDepthMarketDataField_BidPrice1_get, _thosttraderapi.CThostFtdcDepthMarketDataField_BidPrice1_set)
    BidVolume1 = property(_thosttraderapi.CThostFtdcDepthMarketDataField_BidVolume1_get, _thosttraderapi.CThostFtdcDepthMarketDataField_BidVolume1_set)
    AskPrice1 = property(_thosttraderapi.CThostFtdcDepthMarketDataField_AskPrice1_get, _thosttraderapi.CThostFtdcDepthMarketDataField_AskPrice1_set)
    AskVolume1 = property(_thosttraderapi.CThostFtdcDepthMarketDataField_AskVolume1_get, _thosttraderapi.CThostFtdcDepthMarketDataField_AskVolume1_set)
    BidPrice2 = property(_thosttraderapi.CThostFtdcDepthMarketDataField_BidPrice2_get, _thosttraderapi.CThostFtdcDepthMarketDataField_BidPrice2_set)
    BidVolume2 = property(_thosttraderapi.CThostFtdcDepthMarketDataField_BidVolume2_get, _thosttraderapi.CThostFtdcDepthMarketDataField_BidVolume2_set)
    AskPrice2 = property(_thosttraderapi.CThostFtdcDepthMarketDataField_AskPrice2_get, _thosttraderapi.CThostFtdcDepthMarketDataField_AskPrice2_set)
    AskVolume2 = property(_thosttraderapi.CThostFtdcDepthMarketDataField_AskVolume2_get, _thosttraderapi.CThostFtdcDepthMarketDataField_AskVolume2_set)
    BidPrice3 = property(_thosttraderapi.CThostFtdcDepthMarketDataField_BidPrice3_get, _thosttraderapi.CThostFtdcDepthMarketDataField_BidPrice3_set)
    BidVolume3 = property(_thosttraderapi.CThostFtdcDepthMarketDataField_BidVolume3_get, _thosttraderapi.CThostFtdcDepthMarketDataField_BidVolume3_set)
    AskPrice3 = property(_thosttraderapi.CThostFtdcDepthMarketDataField_AskPrice3_get, _thosttraderapi.CThostFtdcDepthMarketDataField_AskPrice3_set)
    AskVolume3 = property(_thosttraderapi.CThostFtdcDepthMarketDataField_AskVolume3_get, _thosttraderapi.CThostFtdcDepthMarketDataField_AskVolume3_set)
    BidPrice4 = property(_thosttraderapi.CThostFtdcDepthMarketDataField_BidPrice4_get, _thosttraderapi.CThostFtdcDepthMarketDataField_BidPrice4_set)
    BidVolume4 = property(_thosttraderapi.CThostFtdcDepthMarketDataField_BidVolume4_get, _thosttraderapi.CThostFtdcDepthMarketDataField_BidVolume4_set)
    AskPrice4 = property(_thosttraderapi.CThostFtdcDepthMarketDataField_AskPrice4_get, _thosttraderapi.CThostFtdcDepthMarketDataField_AskPrice4_set)
    AskVolume4 = property(_thosttraderapi.CThostFtdcDepthMarketDataField_AskVolume4_get, _thosttraderapi.CThostFtdcDepthMarketDataField_AskVolume4_set)
    BidPrice5 = property(_thosttraderapi.CThostFtdcDepthMarketDataField_BidPrice5_get, _thosttraderapi.CThostFtdcDepthMarketDataField_BidPrice5_set)
    BidVolume5 = property(_thosttraderapi.CThostFtdcDepthMarketDataField_BidVolume5_get, _thosttraderapi.CThostFtdcDepthMarketDataField_BidVolume5_set)
    AskPrice5 = property(_thosttraderapi.CThostFtdcDepthMarketDataField_AskPrice5_get, _thosttraderapi.CThostFtdcDepthMarketDataField_AskPrice5_set)
    AskVolume5 = property(_thosttraderapi.CThostFtdcDepthMarketDataField_AskVolume5_get, _thosttraderapi.CThostFtdcDepthMarketDataField_AskVolume5_set)
    AveragePrice = property(_thosttraderapi.CThostFtdcDepthMarketDataField_AveragePrice_get, _thosttraderapi.CThostFtdcDepthMarketDataField_AveragePrice_set)
    ActionDay = property(_thosttraderapi.CThostFtdcDepthMarketDataField_ActionDay_get, _thosttraderapi.CThostFtdcDepthMarketDataField_ActionDay_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcDepthMarketDataField_swiginit(self, _thosttraderapi.new_CThostFtdcDepthMarketDataField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcDepthMarketDataField

# Register CThostFtdcDepthMarketDataField in _thosttraderapi:
_thosttraderapi.CThostFtdcDepthMarketDataField_swigregister(CThostFtdcDepthMarketDataField)

class CThostFtdcInstrumentTradingRightField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    InstrumentID = property(_thosttraderapi.CThostFtdcInstrumentTradingRightField_InstrumentID_get, _thosttraderapi.CThostFtdcInstrumentTradingRightField_InstrumentID_set)
    InvestorRange = property(_thosttraderapi.CThostFtdcInstrumentTradingRightField_InvestorRange_get, _thosttraderapi.CThostFtdcInstrumentTradingRightField_InvestorRange_set)
    BrokerID = property(_thosttraderapi.CThostFtdcInstrumentTradingRightField_BrokerID_get, _thosttraderapi.CThostFtdcInstrumentTradingRightField_BrokerID_set)
    InvestorID = property(_thosttraderapi.CThostFtdcInstrumentTradingRightField_InvestorID_get, _thosttraderapi.CThostFtdcInstrumentTradingRightField_InvestorID_set)
    TradingRight = property(_thosttraderapi.CThostFtdcInstrumentTradingRightField_TradingRight_get, _thosttraderapi.CThostFtdcInstrumentTradingRightField_TradingRight_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcInstrumentTradingRightField_swiginit(self, _thosttraderapi.new_CThostFtdcInstrumentTradingRightField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcInstrumentTradingRightField

# Register CThostFtdcInstrumentTradingRightField in _thosttraderapi:
_thosttraderapi.CThostFtdcInstrumentTradingRightField_swigregister(CThostFtdcInstrumentTradingRightField)

class CThostFtdcBrokerUserField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thosttraderapi.CThostFtdcBrokerUserField_BrokerID_get, _thosttraderapi.CThostFtdcBrokerUserField_BrokerID_set)
    UserID = property(_thosttraderapi.CThostFtdcBrokerUserField_UserID_get, _thosttraderapi.CThostFtdcBrokerUserField_UserID_set)
    UserName = property(_thosttraderapi.CThostFtdcBrokerUserField_UserName_get, _thosttraderapi.CThostFtdcBrokerUserField_UserName_set)
    UserType = property(_thosttraderapi.CThostFtdcBrokerUserField_UserType_get, _thosttraderapi.CThostFtdcBrokerUserField_UserType_set)
    IsActive = property(_thosttraderapi.CThostFtdcBrokerUserField_IsActive_get, _thosttraderapi.CThostFtdcBrokerUserField_IsActive_set)
    IsUsingOTP = property(_thosttraderapi.CThostFtdcBrokerUserField_IsUsingOTP_get, _thosttraderapi.CThostFtdcBrokerUserField_IsUsingOTP_set)
    IsAuthForce = property(_thosttraderapi.CThostFtdcBrokerUserField_IsAuthForce_get, _thosttraderapi.CThostFtdcBrokerUserField_IsAuthForce_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcBrokerUserField_swiginit(self, _thosttraderapi.new_CThostFtdcBrokerUserField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcBrokerUserField

# Register CThostFtdcBrokerUserField in _thosttraderapi:
_thosttraderapi.CThostFtdcBrokerUserField_swigregister(CThostFtdcBrokerUserField)

class CThostFtdcBrokerUserPasswordField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thosttraderapi.CThostFtdcBrokerUserPasswordField_BrokerID_get, _thosttraderapi.CThostFtdcBrokerUserPasswordField_BrokerID_set)
    UserID = property(_thosttraderapi.CThostFtdcBrokerUserPasswordField_UserID_get, _thosttraderapi.CThostFtdcBrokerUserPasswordField_UserID_set)
    Password = property(_thosttraderapi.CThostFtdcBrokerUserPasswordField_Password_get, _thosttraderapi.CThostFtdcBrokerUserPasswordField_Password_set)
    LastUpdateTime = property(_thosttraderapi.CThostFtdcBrokerUserPasswordField_LastUpdateTime_get, _thosttraderapi.CThostFtdcBrokerUserPasswordField_LastUpdateTime_set)
    LastLoginTime = property(_thosttraderapi.CThostFtdcBrokerUserPasswordField_LastLoginTime_get, _thosttraderapi.CThostFtdcBrokerUserPasswordField_LastLoginTime_set)
    ExpireDate = property(_thosttraderapi.CThostFtdcBrokerUserPasswordField_ExpireDate_get, _thosttraderapi.CThostFtdcBrokerUserPasswordField_ExpireDate_set)
    WeakExpireDate = property(_thosttraderapi.CThostFtdcBrokerUserPasswordField_WeakExpireDate_get, _thosttraderapi.CThostFtdcBrokerUserPasswordField_WeakExpireDate_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcBrokerUserPasswordField_swiginit(self, _thosttraderapi.new_CThostFtdcBrokerUserPasswordField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcBrokerUserPasswordField

# Register CThostFtdcBrokerUserPasswordField in _thosttraderapi:
_thosttraderapi.CThostFtdcBrokerUserPasswordField_swigregister(CThostFtdcBrokerUserPasswordField)

class CThostFtdcBrokerUserFunctionField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thosttraderapi.CThostFtdcBrokerUserFunctionField_BrokerID_get, _thosttraderapi.CThostFtdcBrokerUserFunctionField_BrokerID_set)
    UserID = property(_thosttraderapi.CThostFtdcBrokerUserFunctionField_UserID_get, _thosttraderapi.CThostFtdcBrokerUserFunctionField_UserID_set)
    BrokerFunctionCode = property(_thosttraderapi.CThostFtdcBrokerUserFunctionField_BrokerFunctionCode_get, _thosttraderapi.CThostFtdcBrokerUserFunctionField_BrokerFunctionCode_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcBrokerUserFunctionField_swiginit(self, _thosttraderapi.new_CThostFtdcBrokerUserFunctionField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcBrokerUserFunctionField

# Register CThostFtdcBrokerUserFunctionField in _thosttraderapi:
_thosttraderapi.CThostFtdcBrokerUserFunctionField_swigregister(CThostFtdcBrokerUserFunctionField)

class CThostFtdcTraderOfferField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    ExchangeID = property(_thosttraderapi.CThostFtdcTraderOfferField_ExchangeID_get, _thosttraderapi.CThostFtdcTraderOfferField_ExchangeID_set)
    TraderID = property(_thosttraderapi.CThostFtdcTraderOfferField_TraderID_get, _thosttraderapi.CThostFtdcTraderOfferField_TraderID_set)
    ParticipantID = property(_thosttraderapi.CThostFtdcTraderOfferField_ParticipantID_get, _thosttraderapi.CThostFtdcTraderOfferField_ParticipantID_set)
    Password = property(_thosttraderapi.CThostFtdcTraderOfferField_Password_get, _thosttraderapi.CThostFtdcTraderOfferField_Password_set)
    InstallID = property(_thosttraderapi.CThostFtdcTraderOfferField_InstallID_get, _thosttraderapi.CThostFtdcTraderOfferField_InstallID_set)
    OrderLocalID = property(_thosttraderapi.CThostFtdcTraderOfferField_OrderLocalID_get, _thosttraderapi.CThostFtdcTraderOfferField_OrderLocalID_set)
    TraderConnectStatus = property(_thosttraderapi.CThostFtdcTraderOfferField_TraderConnectStatus_get, _thosttraderapi.CThostFtdcTraderOfferField_TraderConnectStatus_set)
    ConnectRequestDate = property(_thosttraderapi.CThostFtdcTraderOfferField_ConnectRequestDate_get, _thosttraderapi.CThostFtdcTraderOfferField_ConnectRequestDate_set)
    ConnectRequestTime = property(_thosttraderapi.CThostFtdcTraderOfferField_ConnectRequestTime_get, _thosttraderapi.CThostFtdcTraderOfferField_ConnectRequestTime_set)
    LastReportDate = property(_thosttraderapi.CThostFtdcTraderOfferField_LastReportDate_get, _thosttraderapi.CThostFtdcTraderOfferField_LastReportDate_set)
    LastReportTime = property(_thosttraderapi.CThostFtdcTraderOfferField_LastReportTime_get, _thosttraderapi.CThostFtdcTraderOfferField_LastReportTime_set)
    ConnectDate = property(_thosttraderapi.CThostFtdcTraderOfferField_ConnectDate_get, _thosttraderapi.CThostFtdcTraderOfferField_ConnectDate_set)
    ConnectTime = property(_thosttraderapi.CThostFtdcTraderOfferField_ConnectTime_get, _thosttraderapi.CThostFtdcTraderOfferField_ConnectTime_set)
    StartDate = property(_thosttraderapi.CThostFtdcTraderOfferField_StartDate_get, _thosttraderapi.CThostFtdcTraderOfferField_StartDate_set)
    StartTime = property(_thosttraderapi.CThostFtdcTraderOfferField_StartTime_get, _thosttraderapi.CThostFtdcTraderOfferField_StartTime_set)
    TradingDay = property(_thosttraderapi.CThostFtdcTraderOfferField_TradingDay_get, _thosttraderapi.CThostFtdcTraderOfferField_TradingDay_set)
    BrokerID = property(_thosttraderapi.CThostFtdcTraderOfferField_BrokerID_get, _thosttraderapi.CThostFtdcTraderOfferField_BrokerID_set)
    MaxTradeID = property(_thosttraderapi.CThostFtdcTraderOfferField_MaxTradeID_get, _thosttraderapi.CThostFtdcTraderOfferField_MaxTradeID_set)
    MaxOrderMessageReference = property(_thosttraderapi.CThostFtdcTraderOfferField_MaxOrderMessageReference_get, _thosttraderapi.CThostFtdcTraderOfferField_MaxOrderMessageReference_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcTraderOfferField_swiginit(self, _thosttraderapi.new_CThostFtdcTraderOfferField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcTraderOfferField

# Register CThostFtdcTraderOfferField in _thosttraderapi:
_thosttraderapi.CThostFtdcTraderOfferField_swigregister(CThostFtdcTraderOfferField)

class CThostFtdcSettlementInfoField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    TradingDay = property(_thosttraderapi.CThostFtdcSettlementInfoField_TradingDay_get, _thosttraderapi.CThostFtdcSettlementInfoField_TradingDay_set)
    SettlementID = property(_thosttraderapi.CThostFtdcSettlementInfoField_SettlementID_get, _thosttraderapi.CThostFtdcSettlementInfoField_SettlementID_set)
    BrokerID = property(_thosttraderapi.CThostFtdcSettlementInfoField_BrokerID_get, _thosttraderapi.CThostFtdcSettlementInfoField_BrokerID_set)
    InvestorID = property(_thosttraderapi.CThostFtdcSettlementInfoField_InvestorID_get, _thosttraderapi.CThostFtdcSettlementInfoField_InvestorID_set)
    SequenceNo = property(_thosttraderapi.CThostFtdcSettlementInfoField_SequenceNo_get, _thosttraderapi.CThostFtdcSettlementInfoField_SequenceNo_set)
    Content = property(_thosttraderapi.CThostFtdcSettlementInfoField_Content_get, _thosttraderapi.CThostFtdcSettlementInfoField_Content_set)
    AccountID = property(_thosttraderapi.CThostFtdcSettlementInfoField_AccountID_get, _thosttraderapi.CThostFtdcSettlementInfoField_AccountID_set)
    CurrencyID = property(_thosttraderapi.CThostFtdcSettlementInfoField_CurrencyID_get, _thosttraderapi.CThostFtdcSettlementInfoField_CurrencyID_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcSettlementInfoField_swiginit(self, _thosttraderapi.new_CThostFtdcSettlementInfoField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcSettlementInfoField

# Register CThostFtdcSettlementInfoField in _thosttraderapi:
_thosttraderapi.CThostFtdcSettlementInfoField_swigregister(CThostFtdcSettlementInfoField)

class CThostFtdcInstrumentMarginRateAdjustField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    InstrumentID = property(_thosttraderapi.CThostFtdcInstrumentMarginRateAdjustField_InstrumentID_get, _thosttraderapi.CThostFtdcInstrumentMarginRateAdjustField_InstrumentID_set)
    InvestorRange = property(_thosttraderapi.CThostFtdcInstrumentMarginRateAdjustField_InvestorRange_get, _thosttraderapi.CThostFtdcInstrumentMarginRateAdjustField_InvestorRange_set)
    BrokerID = property(_thosttraderapi.CThostFtdcInstrumentMarginRateAdjustField_BrokerID_get, _thosttraderapi.CThostFtdcInstrumentMarginRateAdjustField_BrokerID_set)
    InvestorID = property(_thosttraderapi.CThostFtdcInstrumentMarginRateAdjustField_InvestorID_get, _thosttraderapi.CThostFtdcInstrumentMarginRateAdjustField_InvestorID_set)
    HedgeFlag = property(_thosttraderapi.CThostFtdcInstrumentMarginRateAdjustField_HedgeFlag_get, _thosttraderapi.CThostFtdcInstrumentMarginRateAdjustField_HedgeFlag_set)
    LongMarginRatioByMoney = property(_thosttraderapi.CThostFtdcInstrumentMarginRateAdjustField_LongMarginRatioByMoney_get, _thosttraderapi.CThostFtdcInstrumentMarginRateAdjustField_LongMarginRatioByMoney_set)
    LongMarginRatioByVolume = property(_thosttraderapi.CThostFtdcInstrumentMarginRateAdjustField_LongMarginRatioByVolume_get, _thosttraderapi.CThostFtdcInstrumentMarginRateAdjustField_LongMarginRatioByVolume_set)
    ShortMarginRatioByMoney = property(_thosttraderapi.CThostFtdcInstrumentMarginRateAdjustField_ShortMarginRatioByMoney_get, _thosttraderapi.CThostFtdcInstrumentMarginRateAdjustField_ShortMarginRatioByMoney_set)
    ShortMarginRatioByVolume = property(_thosttraderapi.CThostFtdcInstrumentMarginRateAdjustField_ShortMarginRatioByVolume_get, _thosttraderapi.CThostFtdcInstrumentMarginRateAdjustField_ShortMarginRatioByVolume_set)
    IsRelative = property(_thosttraderapi.CThostFtdcInstrumentMarginRateAdjustField_IsRelative_get, _thosttraderapi.CThostFtdcInstrumentMarginRateAdjustField_IsRelative_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcInstrumentMarginRateAdjustField_swiginit(self, _thosttraderapi.new_CThostFtdcInstrumentMarginRateAdjustField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcInstrumentMarginRateAdjustField

# Register CThostFtdcInstrumentMarginRateAdjustField in _thosttraderapi:
_thosttraderapi.CThostFtdcInstrumentMarginRateAdjustField_swigregister(CThostFtdcInstrumentMarginRateAdjustField)

class CThostFtdcExchangeMarginRateField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thosttraderapi.CThostFtdcExchangeMarginRateField_BrokerID_get, _thosttraderapi.CThostFtdcExchangeMarginRateField_BrokerID_set)
    InstrumentID = property(_thosttraderapi.CThostFtdcExchangeMarginRateField_InstrumentID_get, _thosttraderapi.CThostFtdcExchangeMarginRateField_InstrumentID_set)
    HedgeFlag = property(_thosttraderapi.CThostFtdcExchangeMarginRateField_HedgeFlag_get, _thosttraderapi.CThostFtdcExchangeMarginRateField_HedgeFlag_set)
    LongMarginRatioByMoney = property(_thosttraderapi.CThostFtdcExchangeMarginRateField_LongMarginRatioByMoney_get, _thosttraderapi.CThostFtdcExchangeMarginRateField_LongMarginRatioByMoney_set)
    LongMarginRatioByVolume = property(_thosttraderapi.CThostFtdcExchangeMarginRateField_LongMarginRatioByVolume_get, _thosttraderapi.CThostFtdcExchangeMarginRateField_LongMarginRatioByVolume_set)
    ShortMarginRatioByMoney = property(_thosttraderapi.CThostFtdcExchangeMarginRateField_ShortMarginRatioByMoney_get, _thosttraderapi.CThostFtdcExchangeMarginRateField_ShortMarginRatioByMoney_set)
    ShortMarginRatioByVolume = property(_thosttraderapi.CThostFtdcExchangeMarginRateField_ShortMarginRatioByVolume_get, _thosttraderapi.CThostFtdcExchangeMarginRateField_ShortMarginRatioByVolume_set)
    ExchangeID = property(_thosttraderapi.CThostFtdcExchangeMarginRateField_ExchangeID_get, _thosttraderapi.CThostFtdcExchangeMarginRateField_ExchangeID_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcExchangeMarginRateField_swiginit(self, _thosttraderapi.new_CThostFtdcExchangeMarginRateField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcExchangeMarginRateField

# Register CThostFtdcExchangeMarginRateField in _thosttraderapi:
_thosttraderapi.CThostFtdcExchangeMarginRateField_swigregister(CThostFtdcExchangeMarginRateField)

class CThostFtdcExchangeMarginRateAdjustField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thosttraderapi.CThostFtdcExchangeMarginRateAdjustField_BrokerID_get, _thosttraderapi.CThostFtdcExchangeMarginRateAdjustField_BrokerID_set)
    InstrumentID = property(_thosttraderapi.CThostFtdcExchangeMarginRateAdjustField_InstrumentID_get, _thosttraderapi.CThostFtdcExchangeMarginRateAdjustField_InstrumentID_set)
    HedgeFlag = property(_thosttraderapi.CThostFtdcExchangeMarginRateAdjustField_HedgeFlag_get, _thosttraderapi.CThostFtdcExchangeMarginRateAdjustField_HedgeFlag_set)
    LongMarginRatioByMoney = property(_thosttraderapi.CThostFtdcExchangeMarginRateAdjustField_LongMarginRatioByMoney_get, _thosttraderapi.CThostFtdcExchangeMarginRateAdjustField_LongMarginRatioByMoney_set)
    LongMarginRatioByVolume = property(_thosttraderapi.CThostFtdcExchangeMarginRateAdjustField_LongMarginRatioByVolume_get, _thosttraderapi.CThostFtdcExchangeMarginRateAdjustField_LongMarginRatioByVolume_set)
    ShortMarginRatioByMoney = property(_thosttraderapi.CThostFtdcExchangeMarginRateAdjustField_ShortMarginRatioByMoney_get, _thosttraderapi.CThostFtdcExchangeMarginRateAdjustField_ShortMarginRatioByMoney_set)
    ShortMarginRatioByVolume = property(_thosttraderapi.CThostFtdcExchangeMarginRateAdjustField_ShortMarginRatioByVolume_get, _thosttraderapi.CThostFtdcExchangeMarginRateAdjustField_ShortMarginRatioByVolume_set)
    ExchLongMarginRatioByMoney = property(_thosttraderapi.CThostFtdcExchangeMarginRateAdjustField_ExchLongMarginRatioByMoney_get, _thosttraderapi.CThostFtdcExchangeMarginRateAdjustField_ExchLongMarginRatioByMoney_set)
    ExchLongMarginRatioByVolume = property(_thosttraderapi.CThostFtdcExchangeMarginRateAdjustField_ExchLongMarginRatioByVolume_get, _thosttraderapi.CThostFtdcExchangeMarginRateAdjustField_ExchLongMarginRatioByVolume_set)
    ExchShortMarginRatioByMoney = property(_thosttraderapi.CThostFtdcExchangeMarginRateAdjustField_ExchShortMarginRatioByMoney_get, _thosttraderapi.CThostFtdcExchangeMarginRateAdjustField_ExchShortMarginRatioByMoney_set)
    ExchShortMarginRatioByVolume = property(_thosttraderapi.CThostFtdcExchangeMarginRateAdjustField_ExchShortMarginRatioByVolume_get, _thosttraderapi.CThostFtdcExchangeMarginRateAdjustField_ExchShortMarginRatioByVolume_set)
    NoLongMarginRatioByMoney = property(_thosttraderapi.CThostFtdcExchangeMarginRateAdjustField_NoLongMarginRatioByMoney_get, _thosttraderapi.CThostFtdcExchangeMarginRateAdjustField_NoLongMarginRatioByMoney_set)
    NoLongMarginRatioByVolume = property(_thosttraderapi.CThostFtdcExchangeMarginRateAdjustField_NoLongMarginRatioByVolume_get, _thosttraderapi.CThostFtdcExchangeMarginRateAdjustField_NoLongMarginRatioByVolume_set)
    NoShortMarginRatioByMoney = property(_thosttraderapi.CThostFtdcExchangeMarginRateAdjustField_NoShortMarginRatioByMoney_get, _thosttraderapi.CThostFtdcExchangeMarginRateAdjustField_NoShortMarginRatioByMoney_set)
    NoShortMarginRatioByVolume = property(_thosttraderapi.CThostFtdcExchangeMarginRateAdjustField_NoShortMarginRatioByVolume_get, _thosttraderapi.CThostFtdcExchangeMarginRateAdjustField_NoShortMarginRatioByVolume_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcExchangeMarginRateAdjustField_swiginit(self, _thosttraderapi.new_CThostFtdcExchangeMarginRateAdjustField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcExchangeMarginRateAdjustField

# Register CThostFtdcExchangeMarginRateAdjustField in _thosttraderapi:
_thosttraderapi.CThostFtdcExchangeMarginRateAdjustField_swigregister(CThostFtdcExchangeMarginRateAdjustField)

class CThostFtdcExchangeRateField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thosttraderapi.CThostFtdcExchangeRateField_BrokerID_get, _thosttraderapi.CThostFtdcExchangeRateField_BrokerID_set)
    FromCurrencyID = property(_thosttraderapi.CThostFtdcExchangeRateField_FromCurrencyID_get, _thosttraderapi.CThostFtdcExchangeRateField_FromCurrencyID_set)
    FromCurrencyUnit = property(_thosttraderapi.CThostFtdcExchangeRateField_FromCurrencyUnit_get, _thosttraderapi.CThostFtdcExchangeRateField_FromCurrencyUnit_set)
    ToCurrencyID = property(_thosttraderapi.CThostFtdcExchangeRateField_ToCurrencyID_get, _thosttraderapi.CThostFtdcExchangeRateField_ToCurrencyID_set)
    ExchangeRate = property(_thosttraderapi.CThostFtdcExchangeRateField_ExchangeRate_get, _thosttraderapi.CThostFtdcExchangeRateField_ExchangeRate_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcExchangeRateField_swiginit(self, _thosttraderapi.new_CThostFtdcExchangeRateField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcExchangeRateField

# Register CThostFtdcExchangeRateField in _thosttraderapi:
_thosttraderapi.CThostFtdcExchangeRateField_swigregister(CThostFtdcExchangeRateField)

class CThostFtdcSettlementRefField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    TradingDay = property(_thosttraderapi.CThostFtdcSettlementRefField_TradingDay_get, _thosttraderapi.CThostFtdcSettlementRefField_TradingDay_set)
    SettlementID = property(_thosttraderapi.CThostFtdcSettlementRefField_SettlementID_get, _thosttraderapi.CThostFtdcSettlementRefField_SettlementID_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcSettlementRefField_swiginit(self, _thosttraderapi.new_CThostFtdcSettlementRefField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcSettlementRefField

# Register CThostFtdcSettlementRefField in _thosttraderapi:
_thosttraderapi.CThostFtdcSettlementRefField_swigregister(CThostFtdcSettlementRefField)

class CThostFtdcCurrentTimeField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    CurrDate = property(_thosttraderapi.CThostFtdcCurrentTimeField_CurrDate_get, _thosttraderapi.CThostFtdcCurrentTimeField_CurrDate_set)
    CurrTime = property(_thosttraderapi.CThostFtdcCurrentTimeField_CurrTime_get, _thosttraderapi.CThostFtdcCurrentTimeField_CurrTime_set)
    CurrMillisec = property(_thosttraderapi.CThostFtdcCurrentTimeField_CurrMillisec_get, _thosttraderapi.CThostFtdcCurrentTimeField_CurrMillisec_set)
    ActionDay = property(_thosttraderapi.CThostFtdcCurrentTimeField_ActionDay_get, _thosttraderapi.CThostFtdcCurrentTimeField_ActionDay_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcCurrentTimeField_swiginit(self, _thosttraderapi.new_CThostFtdcCurrentTimeField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcCurrentTimeField

# Register CThostFtdcCurrentTimeField in _thosttraderapi:
_thosttraderapi.CThostFtdcCurrentTimeField_swigregister(CThostFtdcCurrentTimeField)

class CThostFtdcCommPhaseField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    TradingDay = property(_thosttraderapi.CThostFtdcCommPhaseField_TradingDay_get, _thosttraderapi.CThostFtdcCommPhaseField_TradingDay_set)
    CommPhaseNo = property(_thosttraderapi.CThostFtdcCommPhaseField_CommPhaseNo_get, _thosttraderapi.CThostFtdcCommPhaseField_CommPhaseNo_set)
    SystemID = property(_thosttraderapi.CThostFtdcCommPhaseField_SystemID_get, _thosttraderapi.CThostFtdcCommPhaseField_SystemID_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcCommPhaseField_swiginit(self, _thosttraderapi.new_CThostFtdcCommPhaseField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcCommPhaseField

# Register CThostFtdcCommPhaseField in _thosttraderapi:
_thosttraderapi.CThostFtdcCommPhaseField_swigregister(CThostFtdcCommPhaseField)

class CThostFtdcLoginInfoField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    FrontID = property(_thosttraderapi.CThostFtdcLoginInfoField_FrontID_get, _thosttraderapi.CThostFtdcLoginInfoField_FrontID_set)
    SessionID = property(_thosttraderapi.CThostFtdcLoginInfoField_SessionID_get, _thosttraderapi.CThostFtdcLoginInfoField_SessionID_set)
    BrokerID = property(_thosttraderapi.CThostFtdcLoginInfoField_BrokerID_get, _thosttraderapi.CThostFtdcLoginInfoField_BrokerID_set)
    UserID = property(_thosttraderapi.CThostFtdcLoginInfoField_UserID_get, _thosttraderapi.CThostFtdcLoginInfoField_UserID_set)
    LoginDate = property(_thosttraderapi.CThostFtdcLoginInfoField_LoginDate_get, _thosttraderapi.CThostFtdcLoginInfoField_LoginDate_set)
    LoginTime = property(_thosttraderapi.CThostFtdcLoginInfoField_LoginTime_get, _thosttraderapi.CThostFtdcLoginInfoField_LoginTime_set)
    IPAddress = property(_thosttraderapi.CThostFtdcLoginInfoField_IPAddress_get, _thosttraderapi.CThostFtdcLoginInfoField_IPAddress_set)
    UserProductInfo = property(_thosttraderapi.CThostFtdcLoginInfoField_UserProductInfo_get, _thosttraderapi.CThostFtdcLoginInfoField_UserProductInfo_set)
    InterfaceProductInfo = property(_thosttraderapi.CThostFtdcLoginInfoField_InterfaceProductInfo_get, _thosttraderapi.CThostFtdcLoginInfoField_InterfaceProductInfo_set)
    ProtocolInfo = property(_thosttraderapi.CThostFtdcLoginInfoField_ProtocolInfo_get, _thosttraderapi.CThostFtdcLoginInfoField_ProtocolInfo_set)
    SystemName = property(_thosttraderapi.CThostFtdcLoginInfoField_SystemName_get, _thosttraderapi.CThostFtdcLoginInfoField_SystemName_set)
    PasswordDeprecated = property(_thosttraderapi.CThostFtdcLoginInfoField_PasswordDeprecated_get, _thosttraderapi.CThostFtdcLoginInfoField_PasswordDeprecated_set)
    MaxOrderRef = property(_thosttraderapi.CThostFtdcLoginInfoField_MaxOrderRef_get, _thosttraderapi.CThostFtdcLoginInfoField_MaxOrderRef_set)
    SHFETime = property(_thosttraderapi.CThostFtdcLoginInfoField_SHFETime_get, _thosttraderapi.CThostFtdcLoginInfoField_SHFETime_set)
    DCETime = property(_thosttraderapi.CThostFtdcLoginInfoField_DCETime_get, _thosttraderapi.CThostFtdcLoginInfoField_DCETime_set)
    CZCETime = property(_thosttraderapi.CThostFtdcLoginInfoField_CZCETime_get, _thosttraderapi.CThostFtdcLoginInfoField_CZCETime_set)
    FFEXTime = property(_thosttraderapi.CThostFtdcLoginInfoField_FFEXTime_get, _thosttraderapi.CThostFtdcLoginInfoField_FFEXTime_set)
    MacAddress = property(_thosttraderapi.CThostFtdcLoginInfoField_MacAddress_get, _thosttraderapi.CThostFtdcLoginInfoField_MacAddress_set)
    OneTimePassword = property(_thosttraderapi.CThostFtdcLoginInfoField_OneTimePassword_get, _thosttraderapi.CThostFtdcLoginInfoField_OneTimePassword_set)
    INETime = property(_thosttraderapi.CThostFtdcLoginInfoField_INETime_get, _thosttraderapi.CThostFtdcLoginInfoField_INETime_set)
    IsQryControl = property(_thosttraderapi.CThostFtdcLoginInfoField_IsQryControl_get, _thosttraderapi.CThostFtdcLoginInfoField_IsQryControl_set)
    LoginRemark = property(_thosttraderapi.CThostFtdcLoginInfoField_LoginRemark_get, _thosttraderapi.CThostFtdcLoginInfoField_LoginRemark_set)
    Password = property(_thosttraderapi.CThostFtdcLoginInfoField_Password_get, _thosttraderapi.CThostFtdcLoginInfoField_Password_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcLoginInfoField_swiginit(self, _thosttraderapi.new_CThostFtdcLoginInfoField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcLoginInfoField

# Register CThostFtdcLoginInfoField in _thosttraderapi:
_thosttraderapi.CThostFtdcLoginInfoField_swigregister(CThostFtdcLoginInfoField)

class CThostFtdcLogoutAllField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    FrontID = property(_thosttraderapi.CThostFtdcLogoutAllField_FrontID_get, _thosttraderapi.CThostFtdcLogoutAllField_FrontID_set)
    SessionID = property(_thosttraderapi.CThostFtdcLogoutAllField_SessionID_get, _thosttraderapi.CThostFtdcLogoutAllField_SessionID_set)
    SystemName = property(_thosttraderapi.CThostFtdcLogoutAllField_SystemName_get, _thosttraderapi.CThostFtdcLogoutAllField_SystemName_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcLogoutAllField_swiginit(self, _thosttraderapi.new_CThostFtdcLogoutAllField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcLogoutAllField

# Register CThostFtdcLogoutAllField in _thosttraderapi:
_thosttraderapi.CThostFtdcLogoutAllField_swigregister(CThostFtdcLogoutAllField)

class CThostFtdcFrontStatusField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    FrontID = property(_thosttraderapi.CThostFtdcFrontStatusField_FrontID_get, _thosttraderapi.CThostFtdcFrontStatusField_FrontID_set)
    LastReportDate = property(_thosttraderapi.CThostFtdcFrontStatusField_LastReportDate_get, _thosttraderapi.CThostFtdcFrontStatusField_LastReportDate_set)
    LastReportTime = property(_thosttraderapi.CThostFtdcFrontStatusField_LastReportTime_get, _thosttraderapi.CThostFtdcFrontStatusField_LastReportTime_set)
    IsActive = property(_thosttraderapi.CThostFtdcFrontStatusField_IsActive_get, _thosttraderapi.CThostFtdcFrontStatusField_IsActive_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcFrontStatusField_swiginit(self, _thosttraderapi.new_CThostFtdcFrontStatusField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcFrontStatusField

# Register CThostFtdcFrontStatusField in _thosttraderapi:
_thosttraderapi.CThostFtdcFrontStatusField_swigregister(CThostFtdcFrontStatusField)

class CThostFtdcUserPasswordUpdateField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thosttraderapi.CThostFtdcUserPasswordUpdateField_BrokerID_get, _thosttraderapi.CThostFtdcUserPasswordUpdateField_BrokerID_set)
    UserID = property(_thosttraderapi.CThostFtdcUserPasswordUpdateField_UserID_get, _thosttraderapi.CThostFtdcUserPasswordUpdateField_UserID_set)
    OldPassword = property(_thosttraderapi.CThostFtdcUserPasswordUpdateField_OldPassword_get, _thosttraderapi.CThostFtdcUserPasswordUpdateField_OldPassword_set)
    NewPassword = property(_thosttraderapi.CThostFtdcUserPasswordUpdateField_NewPassword_get, _thosttraderapi.CThostFtdcUserPasswordUpdateField_NewPassword_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcUserPasswordUpdateField_swiginit(self, _thosttraderapi.new_CThostFtdcUserPasswordUpdateField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcUserPasswordUpdateField

# Register CThostFtdcUserPasswordUpdateField in _thosttraderapi:
_thosttraderapi.CThostFtdcUserPasswordUpdateField_swigregister(CThostFtdcUserPasswordUpdateField)

class CThostFtdcInputOrderField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thosttraderapi.CThostFtdcInputOrderField_BrokerID_get, _thosttraderapi.CThostFtdcInputOrderField_BrokerID_set)
    InvestorID = property(_thosttraderapi.CThostFtdcInputOrderField_InvestorID_get, _thosttraderapi.CThostFtdcInputOrderField_InvestorID_set)
    InstrumentID = property(_thosttraderapi.CThostFtdcInputOrderField_InstrumentID_get, _thosttraderapi.CThostFtdcInputOrderField_InstrumentID_set)
    OrderRef = property(_thosttraderapi.CThostFtdcInputOrderField_OrderRef_get, _thosttraderapi.CThostFtdcInputOrderField_OrderRef_set)
    UserID = property(_thosttraderapi.CThostFtdcInputOrderField_UserID_get, _thosttraderapi.CThostFtdcInputOrderField_UserID_set)
    OrderPriceType = property(_thosttraderapi.CThostFtdcInputOrderField_OrderPriceType_get, _thosttraderapi.CThostFtdcInputOrderField_OrderPriceType_set)
    Direction = property(_thosttraderapi.CThostFtdcInputOrderField_Direction_get, _thosttraderapi.CThostFtdcInputOrderField_Direction_set)
    CombOffsetFlag = property(_thosttraderapi.CThostFtdcInputOrderField_CombOffsetFlag_get, _thosttraderapi.CThostFtdcInputOrderField_CombOffsetFlag_set)
    CombHedgeFlag = property(_thosttraderapi.CThostFtdcInputOrderField_CombHedgeFlag_get, _thosttraderapi.CThostFtdcInputOrderField_CombHedgeFlag_set)
    LimitPrice = property(_thosttraderapi.CThostFtdcInputOrderField_LimitPrice_get, _thosttraderapi.CThostFtdcInputOrderField_LimitPrice_set)
    VolumeTotalOriginal = property(_thosttraderapi.CThostFtdcInputOrderField_VolumeTotalOriginal_get, _thosttraderapi.CThostFtdcInputOrderField_VolumeTotalOriginal_set)
    TimeCondition = property(_thosttraderapi.CThostFtdcInputOrderField_TimeCondition_get, _thosttraderapi.CThostFtdcInputOrderField_TimeCondition_set)
    GTDDate = property(_thosttraderapi.CThostFtdcInputOrderField_GTDDate_get, _thosttraderapi.CThostFtdcInputOrderField_GTDDate_set)
    VolumeCondition = property(_thosttraderapi.CThostFtdcInputOrderField_VolumeCondition_get, _thosttraderapi.CThostFtdcInputOrderField_VolumeCondition_set)
    MinVolume = property(_thosttraderapi.CThostFtdcInputOrderField_MinVolume_get, _thosttraderapi.CThostFtdcInputOrderField_MinVolume_set)
    ContingentCondition = property(_thosttraderapi.CThostFtdcInputOrderField_ContingentCondition_get, _thosttraderapi.CThostFtdcInputOrderField_ContingentCondition_set)
    StopPrice = property(_thosttraderapi.CThostFtdcInputOrderField_StopPrice_get, _thosttraderapi.CThostFtdcInputOrderField_StopPrice_set)
    ForceCloseReason = property(_thosttraderapi.CThostFtdcInputOrderField_ForceCloseReason_get, _thosttraderapi.CThostFtdcInputOrderField_ForceCloseReason_set)
    IsAutoSuspend = property(_thosttraderapi.CThostFtdcInputOrderField_IsAutoSuspend_get, _thosttraderapi.CThostFtdcInputOrderField_IsAutoSuspend_set)
    BusinessUnit = property(_thosttraderapi.CThostFtdcInputOrderField_BusinessUnit_get, _thosttraderapi.CThostFtdcInputOrderField_BusinessUnit_set)
    RequestID = property(_thosttraderapi.CThostFtdcInputOrderField_RequestID_get, _thosttraderapi.CThostFtdcInputOrderField_RequestID_set)
    UserForceClose = property(_thosttraderapi.CThostFtdcInputOrderField_UserForceClose_get, _thosttraderapi.CThostFtdcInputOrderField_UserForceClose_set)
    IsSwapOrder = property(_thosttraderapi.CThostFtdcInputOrderField_IsSwapOrder_get, _thosttraderapi.CThostFtdcInputOrderField_IsSwapOrder_set)
    ExchangeID = property(_thosttraderapi.CThostFtdcInputOrderField_ExchangeID_get, _thosttraderapi.CThostFtdcInputOrderField_ExchangeID_set)
    InvestUnitID = property(_thosttraderapi.CThostFtdcInputOrderField_InvestUnitID_get, _thosttraderapi.CThostFtdcInputOrderField_InvestUnitID_set)
    AccountID = property(_thosttraderapi.CThostFtdcInputOrderField_AccountID_get, _thosttraderapi.CThostFtdcInputOrderField_AccountID_set)
    CurrencyID = property(_thosttraderapi.CThostFtdcInputOrderField_CurrencyID_get, _thosttraderapi.CThostFtdcInputOrderField_CurrencyID_set)
    ClientID = property(_thosttraderapi.CThostFtdcInputOrderField_ClientID_get, _thosttraderapi.CThostFtdcInputOrderField_ClientID_set)
    IPAddress = property(_thosttraderapi.CThostFtdcInputOrderField_IPAddress_get, _thosttraderapi.CThostFtdcInputOrderField_IPAddress_set)
    MacAddress = property(_thosttraderapi.CThostFtdcInputOrderField_MacAddress_get, _thosttraderapi.CThostFtdcInputOrderField_MacAddress_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcInputOrderField_swiginit(self, _thosttraderapi.new_CThostFtdcInputOrderField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcInputOrderField

# Register CThostFtdcInputOrderField in _thosttraderapi:
_thosttraderapi.CThostFtdcInputOrderField_swigregister(CThostFtdcInputOrderField)

class CThostFtdcOrderField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thosttraderapi.CThostFtdcOrderField_BrokerID_get, _thosttraderapi.CThostFtdcOrderField_BrokerID_set)
    InvestorID = property(_thosttraderapi.CThostFtdcOrderField_InvestorID_get, _thosttraderapi.CThostFtdcOrderField_InvestorID_set)
    InstrumentID = property(_thosttraderapi.CThostFtdcOrderField_InstrumentID_get, _thosttraderapi.CThostFtdcOrderField_InstrumentID_set)
    OrderRef = property(_thosttraderapi.CThostFtdcOrderField_OrderRef_get, _thosttraderapi.CThostFtdcOrderField_OrderRef_set)
    UserID = property(_thosttraderapi.CThostFtdcOrderField_UserID_get, _thosttraderapi.CThostFtdcOrderField_UserID_set)
    OrderPriceType = property(_thosttraderapi.CThostFtdcOrderField_OrderPriceType_get, _thosttraderapi.CThostFtdcOrderField_OrderPriceType_set)
    Direction = property(_thosttraderapi.CThostFtdcOrderField_Direction_get, _thosttraderapi.CThostFtdcOrderField_Direction_set)
    CombOffsetFlag = property(_thosttraderapi.CThostFtdcOrderField_CombOffsetFlag_get, _thosttraderapi.CThostFtdcOrderField_CombOffsetFlag_set)
    CombHedgeFlag = property(_thosttraderapi.CThostFtdcOrderField_CombHedgeFlag_get, _thosttraderapi.CThostFtdcOrderField_CombHedgeFlag_set)
    LimitPrice = property(_thosttraderapi.CThostFtdcOrderField_LimitPrice_get, _thosttraderapi.CThostFtdcOrderField_LimitPrice_set)
    VolumeTotalOriginal = property(_thosttraderapi.CThostFtdcOrderField_VolumeTotalOriginal_get, _thosttraderapi.CThostFtdcOrderField_VolumeTotalOriginal_set)
    TimeCondition = property(_thosttraderapi.CThostFtdcOrderField_TimeCondition_get, _thosttraderapi.CThostFtdcOrderField_TimeCondition_set)
    GTDDate = property(_thosttraderapi.CThostFtdcOrderField_GTDDate_get, _thosttraderapi.CThostFtdcOrderField_GTDDate_set)
    VolumeCondition = property(_thosttraderapi.CThostFtdcOrderField_VolumeCondition_get, _thosttraderapi.CThostFtdcOrderField_VolumeCondition_set)
    MinVolume = property(_thosttraderapi.CThostFtdcOrderField_MinVolume_get, _thosttraderapi.CThostFtdcOrderField_MinVolume_set)
    ContingentCondition = property(_thosttraderapi.CThostFtdcOrderField_ContingentCondition_get, _thosttraderapi.CThostFtdcOrderField_ContingentCondition_set)
    StopPrice = property(_thosttraderapi.CThostFtdcOrderField_StopPrice_get, _thosttraderapi.CThostFtdcOrderField_StopPrice_set)
    ForceCloseReason = property(_thosttraderapi.CThostFtdcOrderField_ForceCloseReason_get, _thosttraderapi.CThostFtdcOrderField_ForceCloseReason_set)
    IsAutoSuspend = property(_thosttraderapi.CThostFtdcOrderField_IsAutoSuspend_get, _thosttraderapi.CThostFtdcOrderField_IsAutoSuspend_set)
    BusinessUnit = property(_thosttraderapi.CThostFtdcOrderField_BusinessUnit_get, _thosttraderapi.CThostFtdcOrderField_BusinessUnit_set)
    RequestID = property(_thosttraderapi.CThostFtdcOrderField_RequestID_get, _thosttraderapi.CThostFtdcOrderField_RequestID_set)
    OrderLocalID = property(_thosttraderapi.CThostFtdcOrderField_OrderLocalID_get, _thosttraderapi.CThostFtdcOrderField_OrderLocalID_set)
    ExchangeID = property(_thosttraderapi.CThostFtdcOrderField_ExchangeID_get, _thosttraderapi.CThostFtdcOrderField_ExchangeID_set)
    ParticipantID = property(_thosttraderapi.CThostFtdcOrderField_ParticipantID_get, _thosttraderapi.CThostFtdcOrderField_ParticipantID_set)
    ClientID = property(_thosttraderapi.CThostFtdcOrderField_ClientID_get, _thosttraderapi.CThostFtdcOrderField_ClientID_set)
    ExchangeInstID = property(_thosttraderapi.CThostFtdcOrderField_ExchangeInstID_get, _thosttraderapi.CThostFtdcOrderField_ExchangeInstID_set)
    TraderID = property(_thosttraderapi.CThostFtdcOrderField_TraderID_get, _thosttraderapi.CThostFtdcOrderField_TraderID_set)
    InstallID = property(_thosttraderapi.CThostFtdcOrderField_InstallID_get, _thosttraderapi.CThostFtdcOrderField_InstallID_set)
    OrderSubmitStatus = property(_thosttraderapi.CThostFtdcOrderField_OrderSubmitStatus_get, _thosttraderapi.CThostFtdcOrderField_OrderSubmitStatus_set)
    NotifySequence = property(_thosttraderapi.CThostFtdcOrderField_NotifySequence_get, _thosttraderapi.CThostFtdcOrderField_NotifySequence_set)
    TradingDay = property(_thosttraderapi.CThostFtdcOrderField_TradingDay_get, _thosttraderapi.CThostFtdcOrderField_TradingDay_set)
    SettlementID = property(_thosttraderapi.CThostFtdcOrderField_SettlementID_get, _thosttraderapi.CThostFtdcOrderField_SettlementID_set)
    OrderSysID = property(_thosttraderapi.CThostFtdcOrderField_OrderSysID_get, _thosttraderapi.CThostFtdcOrderField_OrderSysID_set)
    OrderSource = property(_thosttraderapi.CThostFtdcOrderField_OrderSource_get, _thosttraderapi.CThostFtdcOrderField_OrderSource_set)
    OrderStatus = property(_thosttraderapi.CThostFtdcOrderField_OrderStatus_get, _thosttraderapi.CThostFtdcOrderField_OrderStatus_set)
    OrderType = property(_thosttraderapi.CThostFtdcOrderField_OrderType_get, _thosttraderapi.CThostFtdcOrderField_OrderType_set)
    VolumeTraded = property(_thosttraderapi.CThostFtdcOrderField_VolumeTraded_get, _thosttraderapi.CThostFtdcOrderField_VolumeTraded_set)
    VolumeTotal = property(_thosttraderapi.CThostFtdcOrderField_VolumeTotal_get, _thosttraderapi.CThostFtdcOrderField_VolumeTotal_set)
    InsertDate = property(_thosttraderapi.CThostFtdcOrderField_InsertDate_get, _thosttraderapi.CThostFtdcOrderField_InsertDate_set)
    InsertTime = property(_thosttraderapi.CThostFtdcOrderField_InsertTime_get, _thosttraderapi.CThostFtdcOrderField_InsertTime_set)
    ActiveTime = property(_thosttraderapi.CThostFtdcOrderField_ActiveTime_get, _thosttraderapi.CThostFtdcOrderField_ActiveTime_set)
    SuspendTime = property(_thosttraderapi.CThostFtdcOrderField_SuspendTime_get, _thosttraderapi.CThostFtdcOrderField_SuspendTime_set)
    UpdateTime = property(_thosttraderapi.CThostFtdcOrderField_UpdateTime_get, _thosttraderapi.CThostFtdcOrderField_UpdateTime_set)
    CancelTime = property(_thosttraderapi.CThostFtdcOrderField_CancelTime_get, _thosttraderapi.CThostFtdcOrderField_CancelTime_set)
    ActiveTraderID = property(_thosttraderapi.CThostFtdcOrderField_ActiveTraderID_get, _thosttraderapi.CThostFtdcOrderField_ActiveTraderID_set)
    ClearingPartID = property(_thosttraderapi.CThostFtdcOrderField_ClearingPartID_get, _thosttraderapi.CThostFtdcOrderField_ClearingPartID_set)
    SequenceNo = property(_thosttraderapi.CThostFtdcOrderField_SequenceNo_get, _thosttraderapi.CThostFtdcOrderField_SequenceNo_set)
    FrontID = property(_thosttraderapi.CThostFtdcOrderField_FrontID_get, _thosttraderapi.CThostFtdcOrderField_FrontID_set)
    SessionID = property(_thosttraderapi.CThostFtdcOrderField_SessionID_get, _thosttraderapi.CThostFtdcOrderField_SessionID_set)
    UserProductInfo = property(_thosttraderapi.CThostFtdcOrderField_UserProductInfo_get, _thosttraderapi.CThostFtdcOrderField_UserProductInfo_set)
    StatusMsg = property(_thosttraderapi.CThostFtdcOrderField_StatusMsg_get, _thosttraderapi.CThostFtdcOrderField_StatusMsg_set)
    UserForceClose = property(_thosttraderapi.CThostFtdcOrderField_UserForceClose_get, _thosttraderapi.CThostFtdcOrderField_UserForceClose_set)
    ActiveUserID = property(_thosttraderapi.CThostFtdcOrderField_ActiveUserID_get, _thosttraderapi.CThostFtdcOrderField_ActiveUserID_set)
    BrokerOrderSeq = property(_thosttraderapi.CThostFtdcOrderField_BrokerOrderSeq_get, _thosttraderapi.CThostFtdcOrderField_BrokerOrderSeq_set)
    RelativeOrderSysID = property(_thosttraderapi.CThostFtdcOrderField_RelativeOrderSysID_get, _thosttraderapi.CThostFtdcOrderField_RelativeOrderSysID_set)
    ZCETotalTradedVolume = property(_thosttraderapi.CThostFtdcOrderField_ZCETotalTradedVolume_get, _thosttraderapi.CThostFtdcOrderField_ZCETotalTradedVolume_set)
    IsSwapOrder = property(_thosttraderapi.CThostFtdcOrderField_IsSwapOrder_get, _thosttraderapi.CThostFtdcOrderField_IsSwapOrder_set)
    BranchID = property(_thosttraderapi.CThostFtdcOrderField_BranchID_get, _thosttraderapi.CThostFtdcOrderField_BranchID_set)
    InvestUnitID = property(_thosttraderapi.CThostFtdcOrderField_InvestUnitID_get, _thosttraderapi.CThostFtdcOrderField_InvestUnitID_set)
    AccountID = property(_thosttraderapi.CThostFtdcOrderField_AccountID_get, _thosttraderapi.CThostFtdcOrderField_AccountID_set)
    CurrencyID = property(_thosttraderapi.CThostFtdcOrderField_CurrencyID_get, _thosttraderapi.CThostFtdcOrderField_CurrencyID_set)
    IPAddress = property(_thosttraderapi.CThostFtdcOrderField_IPAddress_get, _thosttraderapi.CThostFtdcOrderField_IPAddress_set)
    MacAddress = property(_thosttraderapi.CThostFtdcOrderField_MacAddress_get, _thosttraderapi.CThostFtdcOrderField_MacAddress_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcOrderField_swiginit(self, _thosttraderapi.new_CThostFtdcOrderField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcOrderField

# Register CThostFtdcOrderField in _thosttraderapi:
_thosttraderapi.CThostFtdcOrderField_swigregister(CThostFtdcOrderField)

class CThostFtdcExchangeOrderField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    OrderPriceType = property(_thosttraderapi.CThostFtdcExchangeOrderField_OrderPriceType_get, _thosttraderapi.CThostFtdcExchangeOrderField_OrderPriceType_set)
    Direction = property(_thosttraderapi.CThostFtdcExchangeOrderField_Direction_get, _thosttraderapi.CThostFtdcExchangeOrderField_Direction_set)
    CombOffsetFlag = property(_thosttraderapi.CThostFtdcExchangeOrderField_CombOffsetFlag_get, _thosttraderapi.CThostFtdcExchangeOrderField_CombOffsetFlag_set)
    CombHedgeFlag = property(_thosttraderapi.CThostFtdcExchangeOrderField_CombHedgeFlag_get, _thosttraderapi.CThostFtdcExchangeOrderField_CombHedgeFlag_set)
    LimitPrice = property(_thosttraderapi.CThostFtdcExchangeOrderField_LimitPrice_get, _thosttraderapi.CThostFtdcExchangeOrderField_LimitPrice_set)
    VolumeTotalOriginal = property(_thosttraderapi.CThostFtdcExchangeOrderField_VolumeTotalOriginal_get, _thosttraderapi.CThostFtdcExchangeOrderField_VolumeTotalOriginal_set)
    TimeCondition = property(_thosttraderapi.CThostFtdcExchangeOrderField_TimeCondition_get, _thosttraderapi.CThostFtdcExchangeOrderField_TimeCondition_set)
    GTDDate = property(_thosttraderapi.CThostFtdcExchangeOrderField_GTDDate_get, _thosttraderapi.CThostFtdcExchangeOrderField_GTDDate_set)
    VolumeCondition = property(_thosttraderapi.CThostFtdcExchangeOrderField_VolumeCondition_get, _thosttraderapi.CThostFtdcExchangeOrderField_VolumeCondition_set)
    MinVolume = property(_thosttraderapi.CThostFtdcExchangeOrderField_MinVolume_get, _thosttraderapi.CThostFtdcExchangeOrderField_MinVolume_set)
    ContingentCondition = property(_thosttraderapi.CThostFtdcExchangeOrderField_ContingentCondition_get, _thosttraderapi.CThostFtdcExchangeOrderField_ContingentCondition_set)
    StopPrice = property(_thosttraderapi.CThostFtdcExchangeOrderField_StopPrice_get, _thosttraderapi.CThostFtdcExchangeOrderField_StopPrice_set)
    ForceCloseReason = property(_thosttraderapi.CThostFtdcExchangeOrderField_ForceCloseReason_get, _thosttraderapi.CThostFtdcExchangeOrderField_ForceCloseReason_set)
    IsAutoSuspend = property(_thosttraderapi.CThostFtdcExchangeOrderField_IsAutoSuspend_get, _thosttraderapi.CThostFtdcExchangeOrderField_IsAutoSuspend_set)
    BusinessUnit = property(_thosttraderapi.CThostFtdcExchangeOrderField_BusinessUnit_get, _thosttraderapi.CThostFtdcExchangeOrderField_BusinessUnit_set)
    RequestID = property(_thosttraderapi.CThostFtdcExchangeOrderField_RequestID_get, _thosttraderapi.CThostFtdcExchangeOrderField_RequestID_set)
    OrderLocalID = property(_thosttraderapi.CThostFtdcExchangeOrderField_OrderLocalID_get, _thosttraderapi.CThostFtdcExchangeOrderField_OrderLocalID_set)
    ExchangeID = property(_thosttraderapi.CThostFtdcExchangeOrderField_ExchangeID_get, _thosttraderapi.CThostFtdcExchangeOrderField_ExchangeID_set)
    ParticipantID = property(_thosttraderapi.CThostFtdcExchangeOrderField_ParticipantID_get, _thosttraderapi.CThostFtdcExchangeOrderField_ParticipantID_set)
    ClientID = property(_thosttraderapi.CThostFtdcExchangeOrderField_ClientID_get, _thosttraderapi.CThostFtdcExchangeOrderField_ClientID_set)
    ExchangeInstID = property(_thosttraderapi.CThostFtdcExchangeOrderField_ExchangeInstID_get, _thosttraderapi.CThostFtdcExchangeOrderField_ExchangeInstID_set)
    TraderID = property(_thosttraderapi.CThostFtdcExchangeOrderField_TraderID_get, _thosttraderapi.CThostFtdcExchangeOrderField_TraderID_set)
    InstallID = property(_thosttraderapi.CThostFtdcExchangeOrderField_InstallID_get, _thosttraderapi.CThostFtdcExchangeOrderField_InstallID_set)
    OrderSubmitStatus = property(_thosttraderapi.CThostFtdcExchangeOrderField_OrderSubmitStatus_get, _thosttraderapi.CThostFtdcExchangeOrderField_OrderSubmitStatus_set)
    NotifySequence = property(_thosttraderapi.CThostFtdcExchangeOrderField_NotifySequence_get, _thosttraderapi.CThostFtdcExchangeOrderField_NotifySequence_set)
    TradingDay = property(_thosttraderapi.CThostFtdcExchangeOrderField_TradingDay_get, _thosttraderapi.CThostFtdcExchangeOrderField_TradingDay_set)
    SettlementID = property(_thosttraderapi.CThostFtdcExchangeOrderField_SettlementID_get, _thosttraderapi.CThostFtdcExchangeOrderField_SettlementID_set)
    OrderSysID = property(_thosttraderapi.CThostFtdcExchangeOrderField_OrderSysID_get, _thosttraderapi.CThostFtdcExchangeOrderField_OrderSysID_set)
    OrderSource = property(_thosttraderapi.CThostFtdcExchangeOrderField_OrderSource_get, _thosttraderapi.CThostFtdcExchangeOrderField_OrderSource_set)
    OrderStatus = property(_thosttraderapi.CThostFtdcExchangeOrderField_OrderStatus_get, _thosttraderapi.CThostFtdcExchangeOrderField_OrderStatus_set)
    OrderType = property(_thosttraderapi.CThostFtdcExchangeOrderField_OrderType_get, _thosttraderapi.CThostFtdcExchangeOrderField_OrderType_set)
    VolumeTraded = property(_thosttraderapi.CThostFtdcExchangeOrderField_VolumeTraded_get, _thosttraderapi.CThostFtdcExchangeOrderField_VolumeTraded_set)
    VolumeTotal = property(_thosttraderapi.CThostFtdcExchangeOrderField_VolumeTotal_get, _thosttraderapi.CThostFtdcExchangeOrderField_VolumeTotal_set)
    InsertDate = property(_thosttraderapi.CThostFtdcExchangeOrderField_InsertDate_get, _thosttraderapi.CThostFtdcExchangeOrderField_InsertDate_set)
    InsertTime = property(_thosttraderapi.CThostFtdcExchangeOrderField_InsertTime_get, _thosttraderapi.CThostFtdcExchangeOrderField_InsertTime_set)
    ActiveTime = property(_thosttraderapi.CThostFtdcExchangeOrderField_ActiveTime_get, _thosttraderapi.CThostFtdcExchangeOrderField_ActiveTime_set)
    SuspendTime = property(_thosttraderapi.CThostFtdcExchangeOrderField_SuspendTime_get, _thosttraderapi.CThostFtdcExchangeOrderField_SuspendTime_set)
    UpdateTime = property(_thosttraderapi.CThostFtdcExchangeOrderField_UpdateTime_get, _thosttraderapi.CThostFtdcExchangeOrderField_UpdateTime_set)
    CancelTime = property(_thosttraderapi.CThostFtdcExchangeOrderField_CancelTime_get, _thosttraderapi.CThostFtdcExchangeOrderField_CancelTime_set)
    ActiveTraderID = property(_thosttraderapi.CThostFtdcExchangeOrderField_ActiveTraderID_get, _thosttraderapi.CThostFtdcExchangeOrderField_ActiveTraderID_set)
    ClearingPartID = property(_thosttraderapi.CThostFtdcExchangeOrderField_ClearingPartID_get, _thosttraderapi.CThostFtdcExchangeOrderField_ClearingPartID_set)
    SequenceNo = property(_thosttraderapi.CThostFtdcExchangeOrderField_SequenceNo_get, _thosttraderapi.CThostFtdcExchangeOrderField_SequenceNo_set)
    BranchID = property(_thosttraderapi.CThostFtdcExchangeOrderField_BranchID_get, _thosttraderapi.CThostFtdcExchangeOrderField_BranchID_set)
    IPAddress = property(_thosttraderapi.CThostFtdcExchangeOrderField_IPAddress_get, _thosttraderapi.CThostFtdcExchangeOrderField_IPAddress_set)
    MacAddress = property(_thosttraderapi.CThostFtdcExchangeOrderField_MacAddress_get, _thosttraderapi.CThostFtdcExchangeOrderField_MacAddress_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcExchangeOrderField_swiginit(self, _thosttraderapi.new_CThostFtdcExchangeOrderField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcExchangeOrderField

# Register CThostFtdcExchangeOrderField in _thosttraderapi:
_thosttraderapi.CThostFtdcExchangeOrderField_swigregister(CThostFtdcExchangeOrderField)

class CThostFtdcExchangeOrderInsertErrorField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    ExchangeID = property(_thosttraderapi.CThostFtdcExchangeOrderInsertErrorField_ExchangeID_get, _thosttraderapi.CThostFtdcExchangeOrderInsertErrorField_ExchangeID_set)
    ParticipantID = property(_thosttraderapi.CThostFtdcExchangeOrderInsertErrorField_ParticipantID_get, _thosttraderapi.CThostFtdcExchangeOrderInsertErrorField_ParticipantID_set)
    TraderID = property(_thosttraderapi.CThostFtdcExchangeOrderInsertErrorField_TraderID_get, _thosttraderapi.CThostFtdcExchangeOrderInsertErrorField_TraderID_set)
    InstallID = property(_thosttraderapi.CThostFtdcExchangeOrderInsertErrorField_InstallID_get, _thosttraderapi.CThostFtdcExchangeOrderInsertErrorField_InstallID_set)
    OrderLocalID = property(_thosttraderapi.CThostFtdcExchangeOrderInsertErrorField_OrderLocalID_get, _thosttraderapi.CThostFtdcExchangeOrderInsertErrorField_OrderLocalID_set)
    ErrorID = property(_thosttraderapi.CThostFtdcExchangeOrderInsertErrorField_ErrorID_get, _thosttraderapi.CThostFtdcExchangeOrderInsertErrorField_ErrorID_set)
    ErrorMsg = property(_thosttraderapi.CThostFtdcExchangeOrderInsertErrorField_ErrorMsg_get, _thosttraderapi.CThostFtdcExchangeOrderInsertErrorField_ErrorMsg_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcExchangeOrderInsertErrorField_swiginit(self, _thosttraderapi.new_CThostFtdcExchangeOrderInsertErrorField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcExchangeOrderInsertErrorField

# Register CThostFtdcExchangeOrderInsertErrorField in _thosttraderapi:
_thosttraderapi.CThostFtdcExchangeOrderInsertErrorField_swigregister(CThostFtdcExchangeOrderInsertErrorField)

class CThostFtdcInputOrderActionField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thosttraderapi.CThostFtdcInputOrderActionField_BrokerID_get, _thosttraderapi.CThostFtdcInputOrderActionField_BrokerID_set)
    InvestorID = property(_thosttraderapi.CThostFtdcInputOrderActionField_InvestorID_get, _thosttraderapi.CThostFtdcInputOrderActionField_InvestorID_set)
    OrderActionRef = property(_thosttraderapi.CThostFtdcInputOrderActionField_OrderActionRef_get, _thosttraderapi.CThostFtdcInputOrderActionField_OrderActionRef_set)
    OrderRef = property(_thosttraderapi.CThostFtdcInputOrderActionField_OrderRef_get, _thosttraderapi.CThostFtdcInputOrderActionField_OrderRef_set)
    RequestID = property(_thosttraderapi.CThostFtdcInputOrderActionField_RequestID_get, _thosttraderapi.CThostFtdcInputOrderActionField_RequestID_set)
    FrontID = property(_thosttraderapi.CThostFtdcInputOrderActionField_FrontID_get, _thosttraderapi.CThostFtdcInputOrderActionField_FrontID_set)
    SessionID = property(_thosttraderapi.CThostFtdcInputOrderActionField_SessionID_get, _thosttraderapi.CThostFtdcInputOrderActionField_SessionID_set)
    ExchangeID = property(_thosttraderapi.CThostFtdcInputOrderActionField_ExchangeID_get, _thosttraderapi.CThostFtdcInputOrderActionField_ExchangeID_set)
    OrderSysID = property(_thosttraderapi.CThostFtdcInputOrderActionField_OrderSysID_get, _thosttraderapi.CThostFtdcInputOrderActionField_OrderSysID_set)
    ActionFlag = property(_thosttraderapi.CThostFtdcInputOrderActionField_ActionFlag_get, _thosttraderapi.CThostFtdcInputOrderActionField_ActionFlag_set)
    LimitPrice = property(_thosttraderapi.CThostFtdcInputOrderActionField_LimitPrice_get, _thosttraderapi.CThostFtdcInputOrderActionField_LimitPrice_set)
    VolumeChange = property(_thosttraderapi.CThostFtdcInputOrderActionField_VolumeChange_get, _thosttraderapi.CThostFtdcInputOrderActionField_VolumeChange_set)
    UserID = property(_thosttraderapi.CThostFtdcInputOrderActionField_UserID_get, _thosttraderapi.CThostFtdcInputOrderActionField_UserID_set)
    InstrumentID = property(_thosttraderapi.CThostFtdcInputOrderActionField_InstrumentID_get, _thosttraderapi.CThostFtdcInputOrderActionField_InstrumentID_set)
    InvestUnitID = property(_thosttraderapi.CThostFtdcInputOrderActionField_InvestUnitID_get, _thosttraderapi.CThostFtdcInputOrderActionField_InvestUnitID_set)
    IPAddress = property(_thosttraderapi.CThostFtdcInputOrderActionField_IPAddress_get, _thosttraderapi.CThostFtdcInputOrderActionField_IPAddress_set)
    MacAddress = property(_thosttraderapi.CThostFtdcInputOrderActionField_MacAddress_get, _thosttraderapi.CThostFtdcInputOrderActionField_MacAddress_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcInputOrderActionField_swiginit(self, _thosttraderapi.new_CThostFtdcInputOrderActionField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcInputOrderActionField

# Register CThostFtdcInputOrderActionField in _thosttraderapi:
_thosttraderapi.CThostFtdcInputOrderActionField_swigregister(CThostFtdcInputOrderActionField)

class CThostFtdcOrderActionField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thosttraderapi.CThostFtdcOrderActionField_BrokerID_get, _thosttraderapi.CThostFtdcOrderActionField_BrokerID_set)
    InvestorID = property(_thosttraderapi.CThostFtdcOrderActionField_InvestorID_get, _thosttraderapi.CThostFtdcOrderActionField_InvestorID_set)
    OrderActionRef = property(_thosttraderapi.CThostFtdcOrderActionField_OrderActionRef_get, _thosttraderapi.CThostFtdcOrderActionField_OrderActionRef_set)
    OrderRef = property(_thosttraderapi.CThostFtdcOrderActionField_OrderRef_get, _thosttraderapi.CThostFtdcOrderActionField_OrderRef_set)
    RequestID = property(_thosttraderapi.CThostFtdcOrderActionField_RequestID_get, _thosttraderapi.CThostFtdcOrderActionField_RequestID_set)
    FrontID = property(_thosttraderapi.CThostFtdcOrderActionField_FrontID_get, _thosttraderapi.CThostFtdcOrderActionField_FrontID_set)
    SessionID = property(_thosttraderapi.CThostFtdcOrderActionField_SessionID_get, _thosttraderapi.CThostFtdcOrderActionField_SessionID_set)
    ExchangeID = property(_thosttraderapi.CThostFtdcOrderActionField_ExchangeID_get, _thosttraderapi.CThostFtdcOrderActionField_ExchangeID_set)
    OrderSysID = property(_thosttraderapi.CThostFtdcOrderActionField_OrderSysID_get, _thosttraderapi.CThostFtdcOrderActionField_OrderSysID_set)
    ActionFlag = property(_thosttraderapi.CThostFtdcOrderActionField_ActionFlag_get, _thosttraderapi.CThostFtdcOrderActionField_ActionFlag_set)
    LimitPrice = property(_thosttraderapi.CThostFtdcOrderActionField_LimitPrice_get, _thosttraderapi.CThostFtdcOrderActionField_LimitPrice_set)
    VolumeChange = property(_thosttraderapi.CThostFtdcOrderActionField_VolumeChange_get, _thosttraderapi.CThostFtdcOrderActionField_VolumeChange_set)
    ActionDate = property(_thosttraderapi.CThostFtdcOrderActionField_ActionDate_get, _thosttraderapi.CThostFtdcOrderActionField_ActionDate_set)
    ActionTime = property(_thosttraderapi.CThostFtdcOrderActionField_ActionTime_get, _thosttraderapi.CThostFtdcOrderActionField_ActionTime_set)
    TraderID = property(_thosttraderapi.CThostFtdcOrderActionField_TraderID_get, _thosttraderapi.CThostFtdcOrderActionField_TraderID_set)
    InstallID = property(_thosttraderapi.CThostFtdcOrderActionField_InstallID_get, _thosttraderapi.CThostFtdcOrderActionField_InstallID_set)
    OrderLocalID = property(_thosttraderapi.CThostFtdcOrderActionField_OrderLocalID_get, _thosttraderapi.CThostFtdcOrderActionField_OrderLocalID_set)
    ActionLocalID = property(_thosttraderapi.CThostFtdcOrderActionField_ActionLocalID_get, _thosttraderapi.CThostFtdcOrderActionField_ActionLocalID_set)
    ParticipantID = property(_thosttraderapi.CThostFtdcOrderActionField_ParticipantID_get, _thosttraderapi.CThostFtdcOrderActionField_ParticipantID_set)
    ClientID = property(_thosttraderapi.CThostFtdcOrderActionField_ClientID_get, _thosttraderapi.CThostFtdcOrderActionField_ClientID_set)
    BusinessUnit = property(_thosttraderapi.CThostFtdcOrderActionField_BusinessUnit_get, _thosttraderapi.CThostFtdcOrderActionField_BusinessUnit_set)
    OrderActionStatus = property(_thosttraderapi.CThostFtdcOrderActionField_OrderActionStatus_get, _thosttraderapi.CThostFtdcOrderActionField_OrderActionStatus_set)
    UserID = property(_thosttraderapi.CThostFtdcOrderActionField_UserID_get, _thosttraderapi.CThostFtdcOrderActionField_UserID_set)
    StatusMsg = property(_thosttraderapi.CThostFtdcOrderActionField_StatusMsg_get, _thosttraderapi.CThostFtdcOrderActionField_StatusMsg_set)
    InstrumentID = property(_thosttraderapi.CThostFtdcOrderActionField_InstrumentID_get, _thosttraderapi.CThostFtdcOrderActionField_InstrumentID_set)
    BranchID = property(_thosttraderapi.CThostFtdcOrderActionField_BranchID_get, _thosttraderapi.CThostFtdcOrderActionField_BranchID_set)
    InvestUnitID = property(_thosttraderapi.CThostFtdcOrderActionField_InvestUnitID_get, _thosttraderapi.CThostFtdcOrderActionField_InvestUnitID_set)
    IPAddress = property(_thosttraderapi.CThostFtdcOrderActionField_IPAddress_get, _thosttraderapi.CThostFtdcOrderActionField_IPAddress_set)
    MacAddress = property(_thosttraderapi.CThostFtdcOrderActionField_MacAddress_get, _thosttraderapi.CThostFtdcOrderActionField_MacAddress_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcOrderActionField_swiginit(self, _thosttraderapi.new_CThostFtdcOrderActionField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcOrderActionField

# Register CThostFtdcOrderActionField in _thosttraderapi:
_thosttraderapi.CThostFtdcOrderActionField_swigregister(CThostFtdcOrderActionField)

class CThostFtdcExchangeOrderActionField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    ExchangeID = property(_thosttraderapi.CThostFtdcExchangeOrderActionField_ExchangeID_get, _thosttraderapi.CThostFtdcExchangeOrderActionField_ExchangeID_set)
    OrderSysID = property(_thosttraderapi.CThostFtdcExchangeOrderActionField_OrderSysID_get, _thosttraderapi.CThostFtdcExchangeOrderActionField_OrderSysID_set)
    ActionFlag = property(_thosttraderapi.CThostFtdcExchangeOrderActionField_ActionFlag_get, _thosttraderapi.CThostFtdcExchangeOrderActionField_ActionFlag_set)
    LimitPrice = property(_thosttraderapi.CThostFtdcExchangeOrderActionField_LimitPrice_get, _thosttraderapi.CThostFtdcExchangeOrderActionField_LimitPrice_set)
    VolumeChange = property(_thosttraderapi.CThostFtdcExchangeOrderActionField_VolumeChange_get, _thosttraderapi.CThostFtdcExchangeOrderActionField_VolumeChange_set)
    ActionDate = property(_thosttraderapi.CThostFtdcExchangeOrderActionField_ActionDate_get, _thosttraderapi.CThostFtdcExchangeOrderActionField_ActionDate_set)
    ActionTime = property(_thosttraderapi.CThostFtdcExchangeOrderActionField_ActionTime_get, _thosttraderapi.CThostFtdcExchangeOrderActionField_ActionTime_set)
    TraderID = property(_thosttraderapi.CThostFtdcExchangeOrderActionField_TraderID_get, _thosttraderapi.CThostFtdcExchangeOrderActionField_TraderID_set)
    InstallID = property(_thosttraderapi.CThostFtdcExchangeOrderActionField_InstallID_get, _thosttraderapi.CThostFtdcExchangeOrderActionField_InstallID_set)
    OrderLocalID = property(_thosttraderapi.CThostFtdcExchangeOrderActionField_OrderLocalID_get, _thosttraderapi.CThostFtdcExchangeOrderActionField_OrderLocalID_set)
    ActionLocalID = property(_thosttraderapi.CThostFtdcExchangeOrderActionField_ActionLocalID_get, _thosttraderapi.CThostFtdcExchangeOrderActionField_ActionLocalID_set)
    ParticipantID = property(_thosttraderapi.CThostFtdcExchangeOrderActionField_ParticipantID_get, _thosttraderapi.CThostFtdcExchangeOrderActionField_ParticipantID_set)
    ClientID = property(_thosttraderapi.CThostFtdcExchangeOrderActionField_ClientID_get, _thosttraderapi.CThostFtdcExchangeOrderActionField_ClientID_set)
    BusinessUnit = property(_thosttraderapi.CThostFtdcExchangeOrderActionField_BusinessUnit_get, _thosttraderapi.CThostFtdcExchangeOrderActionField_BusinessUnit_set)
    OrderActionStatus = property(_thosttraderapi.CThostFtdcExchangeOrderActionField_OrderActionStatus_get, _thosttraderapi.CThostFtdcExchangeOrderActionField_OrderActionStatus_set)
    UserID = property(_thosttraderapi.CThostFtdcExchangeOrderActionField_UserID_get, _thosttraderapi.CThostFtdcExchangeOrderActionField_UserID_set)
    BranchID = property(_thosttraderapi.CThostFtdcExchangeOrderActionField_BranchID_get, _thosttraderapi.CThostFtdcExchangeOrderActionField_BranchID_set)
    IPAddress = property(_thosttraderapi.CThostFtdcExchangeOrderActionField_IPAddress_get, _thosttraderapi.CThostFtdcExchangeOrderActionField_IPAddress_set)
    MacAddress = property(_thosttraderapi.CThostFtdcExchangeOrderActionField_MacAddress_get, _thosttraderapi.CThostFtdcExchangeOrderActionField_MacAddress_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcExchangeOrderActionField_swiginit(self, _thosttraderapi.new_CThostFtdcExchangeOrderActionField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcExchangeOrderActionField

# Register CThostFtdcExchangeOrderActionField in _thosttraderapi:
_thosttraderapi.CThostFtdcExchangeOrderActionField_swigregister(CThostFtdcExchangeOrderActionField)

class CThostFtdcExchangeOrderActionErrorField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    ExchangeID = property(_thosttraderapi.CThostFtdcExchangeOrderActionErrorField_ExchangeID_get, _thosttraderapi.CThostFtdcExchangeOrderActionErrorField_ExchangeID_set)
    OrderSysID = property(_thosttraderapi.CThostFtdcExchangeOrderActionErrorField_OrderSysID_get, _thosttraderapi.CThostFtdcExchangeOrderActionErrorField_OrderSysID_set)
    TraderID = property(_thosttraderapi.CThostFtdcExchangeOrderActionErrorField_TraderID_get, _thosttraderapi.CThostFtdcExchangeOrderActionErrorField_TraderID_set)
    InstallID = property(_thosttraderapi.CThostFtdcExchangeOrderActionErrorField_InstallID_get, _thosttraderapi.CThostFtdcExchangeOrderActionErrorField_InstallID_set)
    OrderLocalID = property(_thosttraderapi.CThostFtdcExchangeOrderActionErrorField_OrderLocalID_get, _thosttraderapi.CThostFtdcExchangeOrderActionErrorField_OrderLocalID_set)
    ActionLocalID = property(_thosttraderapi.CThostFtdcExchangeOrderActionErrorField_ActionLocalID_get, _thosttraderapi.CThostFtdcExchangeOrderActionErrorField_ActionLocalID_set)
    ErrorID = property(_thosttraderapi.CThostFtdcExchangeOrderActionErrorField_ErrorID_get, _thosttraderapi.CThostFtdcExchangeOrderActionErrorField_ErrorID_set)
    ErrorMsg = property(_thosttraderapi.CThostFtdcExchangeOrderActionErrorField_ErrorMsg_get, _thosttraderapi.CThostFtdcExchangeOrderActionErrorField_ErrorMsg_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcExchangeOrderActionErrorField_swiginit(self, _thosttraderapi.new_CThostFtdcExchangeOrderActionErrorField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcExchangeOrderActionErrorField

# Register CThostFtdcExchangeOrderActionErrorField in _thosttraderapi:
_thosttraderapi.CThostFtdcExchangeOrderActionErrorField_swigregister(CThostFtdcExchangeOrderActionErrorField)

class CThostFtdcExchangeTradeField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    ExchangeID = property(_thosttraderapi.CThostFtdcExchangeTradeField_ExchangeID_get, _thosttraderapi.CThostFtdcExchangeTradeField_ExchangeID_set)
    TradeID = property(_thosttraderapi.CThostFtdcExchangeTradeField_TradeID_get, _thosttraderapi.CThostFtdcExchangeTradeField_TradeID_set)
    Direction = property(_thosttraderapi.CThostFtdcExchangeTradeField_Direction_get, _thosttraderapi.CThostFtdcExchangeTradeField_Direction_set)
    OrderSysID = property(_thosttraderapi.CThostFtdcExchangeTradeField_OrderSysID_get, _thosttraderapi.CThostFtdcExchangeTradeField_OrderSysID_set)
    ParticipantID = property(_thosttraderapi.CThostFtdcExchangeTradeField_ParticipantID_get, _thosttraderapi.CThostFtdcExchangeTradeField_ParticipantID_set)
    ClientID = property(_thosttraderapi.CThostFtdcExchangeTradeField_ClientID_get, _thosttraderapi.CThostFtdcExchangeTradeField_ClientID_set)
    TradingRole = property(_thosttraderapi.CThostFtdcExchangeTradeField_TradingRole_get, _thosttraderapi.CThostFtdcExchangeTradeField_TradingRole_set)
    ExchangeInstID = property(_thosttraderapi.CThostFtdcExchangeTradeField_ExchangeInstID_get, _thosttraderapi.CThostFtdcExchangeTradeField_ExchangeInstID_set)
    OffsetFlag = property(_thosttraderapi.CThostFtdcExchangeTradeField_OffsetFlag_get, _thosttraderapi.CThostFtdcExchangeTradeField_OffsetFlag_set)
    HedgeFlag = property(_thosttraderapi.CThostFtdcExchangeTradeField_HedgeFlag_get, _thosttraderapi.CThostFtdcExchangeTradeField_HedgeFlag_set)
    Price = property(_thosttraderapi.CThostFtdcExchangeTradeField_Price_get, _thosttraderapi.CThostFtdcExchangeTradeField_Price_set)
    Volume = property(_thosttraderapi.CThostFtdcExchangeTradeField_Volume_get, _thosttraderapi.CThostFtdcExchangeTradeField_Volume_set)
    TradeDate = property(_thosttraderapi.CThostFtdcExchangeTradeField_TradeDate_get, _thosttraderapi.CThostFtdcExchangeTradeField_TradeDate_set)
    TradeTime = property(_thosttraderapi.CThostFtdcExchangeTradeField_TradeTime_get, _thosttraderapi.CThostFtdcExchangeTradeField_TradeTime_set)
    TradeType = property(_thosttraderapi.CThostFtdcExchangeTradeField_TradeType_get, _thosttraderapi.CThostFtdcExchangeTradeField_TradeType_set)
    PriceSource = property(_thosttraderapi.CThostFtdcExchangeTradeField_PriceSource_get, _thosttraderapi.CThostFtdcExchangeTradeField_PriceSource_set)
    TraderID = property(_thosttraderapi.CThostFtdcExchangeTradeField_TraderID_get, _thosttraderapi.CThostFtdcExchangeTradeField_TraderID_set)
    OrderLocalID = property(_thosttraderapi.CThostFtdcExchangeTradeField_OrderLocalID_get, _thosttraderapi.CThostFtdcExchangeTradeField_OrderLocalID_set)
    ClearingPartID = property(_thosttraderapi.CThostFtdcExchangeTradeField_ClearingPartID_get, _thosttraderapi.CThostFtdcExchangeTradeField_ClearingPartID_set)
    BusinessUnit = property(_thosttraderapi.CThostFtdcExchangeTradeField_BusinessUnit_get, _thosttraderapi.CThostFtdcExchangeTradeField_BusinessUnit_set)
    SequenceNo = property(_thosttraderapi.CThostFtdcExchangeTradeField_SequenceNo_get, _thosttraderapi.CThostFtdcExchangeTradeField_SequenceNo_set)
    TradeSource = property(_thosttraderapi.CThostFtdcExchangeTradeField_TradeSource_get, _thosttraderapi.CThostFtdcExchangeTradeField_TradeSource_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcExchangeTradeField_swiginit(self, _thosttraderapi.new_CThostFtdcExchangeTradeField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcExchangeTradeField

# Register CThostFtdcExchangeTradeField in _thosttraderapi:
_thosttraderapi.CThostFtdcExchangeTradeField_swigregister(CThostFtdcExchangeTradeField)

class CThostFtdcTradeField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thosttraderapi.CThostFtdcTradeField_BrokerID_get, _thosttraderapi.CThostFtdcTradeField_BrokerID_set)
    InvestorID = property(_thosttraderapi.CThostFtdcTradeField_InvestorID_get, _thosttraderapi.CThostFtdcTradeField_InvestorID_set)
    InstrumentID = property(_thosttraderapi.CThostFtdcTradeField_InstrumentID_get, _thosttraderapi.CThostFtdcTradeField_InstrumentID_set)
    OrderRef = property(_thosttraderapi.CThostFtdcTradeField_OrderRef_get, _thosttraderapi.CThostFtdcTradeField_OrderRef_set)
    UserID = property(_thosttraderapi.CThostFtdcTradeField_UserID_get, _thosttraderapi.CThostFtdcTradeField_UserID_set)
    ExchangeID = property(_thosttraderapi.CThostFtdcTradeField_ExchangeID_get, _thosttraderapi.CThostFtdcTradeField_ExchangeID_set)
    TradeID = property(_thosttraderapi.CThostFtdcTradeField_TradeID_get, _thosttraderapi.CThostFtdcTradeField_TradeID_set)
    Direction = property(_thosttraderapi.CThostFtdcTradeField_Direction_get, _thosttraderapi.CThostFtdcTradeField_Direction_set)
    OrderSysID = property(_thosttraderapi.CThostFtdcTradeField_OrderSysID_get, _thosttraderapi.CThostFtdcTradeField_OrderSysID_set)
    ParticipantID = property(_thosttraderapi.CThostFtdcTradeField_ParticipantID_get, _thosttraderapi.CThostFtdcTradeField_ParticipantID_set)
    ClientID = property(_thosttraderapi.CThostFtdcTradeField_ClientID_get, _thosttraderapi.CThostFtdcTradeField_ClientID_set)
    TradingRole = property(_thosttraderapi.CThostFtdcTradeField_TradingRole_get, _thosttraderapi.CThostFtdcTradeField_TradingRole_set)
    ExchangeInstID = property(_thosttraderapi.CThostFtdcTradeField_ExchangeInstID_get, _thosttraderapi.CThostFtdcTradeField_ExchangeInstID_set)
    OffsetFlag = property(_thosttraderapi.CThostFtdcTradeField_OffsetFlag_get, _thosttraderapi.CThostFtdcTradeField_OffsetFlag_set)
    HedgeFlag = property(_thosttraderapi.CThostFtdcTradeField_HedgeFlag_get, _thosttraderapi.CThostFtdcTradeField_HedgeFlag_set)
    Price = property(_thosttraderapi.CThostFtdcTradeField_Price_get, _thosttraderapi.CThostFtdcTradeField_Price_set)
    Volume = property(_thosttraderapi.CThostFtdcTradeField_Volume_get, _thosttraderapi.CThostFtdcTradeField_Volume_set)
    TradeDate = property(_thosttraderapi.CThostFtdcTradeField_TradeDate_get, _thosttraderapi.CThostFtdcTradeField_TradeDate_set)
    TradeTime = property(_thosttraderapi.CThostFtdcTradeField_TradeTime_get, _thosttraderapi.CThostFtdcTradeField_TradeTime_set)
    TradeType = property(_thosttraderapi.CThostFtdcTradeField_TradeType_get, _thosttraderapi.CThostFtdcTradeField_TradeType_set)
    PriceSource = property(_thosttraderapi.CThostFtdcTradeField_PriceSource_get, _thosttraderapi.CThostFtdcTradeField_PriceSource_set)
    TraderID = property(_thosttraderapi.CThostFtdcTradeField_TraderID_get, _thosttraderapi.CThostFtdcTradeField_TraderID_set)
    OrderLocalID = property(_thosttraderapi.CThostFtdcTradeField_OrderLocalID_get, _thosttraderapi.CThostFtdcTradeField_OrderLocalID_set)
    ClearingPartID = property(_thosttraderapi.CThostFtdcTradeField_ClearingPartID_get, _thosttraderapi.CThostFtdcTradeField_ClearingPartID_set)
    BusinessUnit = property(_thosttraderapi.CThostFtdcTradeField_BusinessUnit_get, _thosttraderapi.CThostFtdcTradeField_BusinessUnit_set)
    SequenceNo = property(_thosttraderapi.CThostFtdcTradeField_SequenceNo_get, _thosttraderapi.CThostFtdcTradeField_SequenceNo_set)
    TradingDay = property(_thosttraderapi.CThostFtdcTradeField_TradingDay_get, _thosttraderapi.CThostFtdcTradeField_TradingDay_set)
    SettlementID = property(_thosttraderapi.CThostFtdcTradeField_SettlementID_get, _thosttraderapi.CThostFtdcTradeField_SettlementID_set)
    BrokerOrderSeq = property(_thosttraderapi.CThostFtdcTradeField_BrokerOrderSeq_get, _thosttraderapi.CThostFtdcTradeField_BrokerOrderSeq_set)
    TradeSource = property(_thosttraderapi.CThostFtdcTradeField_TradeSource_get, _thosttraderapi.CThostFtdcTradeField_TradeSource_set)
    InvestUnitID = property(_thosttraderapi.CThostFtdcTradeField_InvestUnitID_get, _thosttraderapi.CThostFtdcTradeField_InvestUnitID_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcTradeField_swiginit(self, _thosttraderapi.new_CThostFtdcTradeField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcTradeField

# Register CThostFtdcTradeField in _thosttraderapi:
_thosttraderapi.CThostFtdcTradeField_swigregister(CThostFtdcTradeField)

class CThostFtdcUserSessionField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    FrontID = property(_thosttraderapi.CThostFtdcUserSessionField_FrontID_get, _thosttraderapi.CThostFtdcUserSessionField_FrontID_set)
    SessionID = property(_thosttraderapi.CThostFtdcUserSessionField_SessionID_get, _thosttraderapi.CThostFtdcUserSessionField_SessionID_set)
    BrokerID = property(_thosttraderapi.CThostFtdcUserSessionField_BrokerID_get, _thosttraderapi.CThostFtdcUserSessionField_BrokerID_set)
    UserID = property(_thosttraderapi.CThostFtdcUserSessionField_UserID_get, _thosttraderapi.CThostFtdcUserSessionField_UserID_set)
    LoginDate = property(_thosttraderapi.CThostFtdcUserSessionField_LoginDate_get, _thosttraderapi.CThostFtdcUserSessionField_LoginDate_set)
    LoginTime = property(_thosttraderapi.CThostFtdcUserSessionField_LoginTime_get, _thosttraderapi.CThostFtdcUserSessionField_LoginTime_set)
    IPAddress = property(_thosttraderapi.CThostFtdcUserSessionField_IPAddress_get, _thosttraderapi.CThostFtdcUserSessionField_IPAddress_set)
    UserProductInfo = property(_thosttraderapi.CThostFtdcUserSessionField_UserProductInfo_get, _thosttraderapi.CThostFtdcUserSessionField_UserProductInfo_set)
    InterfaceProductInfo = property(_thosttraderapi.CThostFtdcUserSessionField_InterfaceProductInfo_get, _thosttraderapi.CThostFtdcUserSessionField_InterfaceProductInfo_set)
    ProtocolInfo = property(_thosttraderapi.CThostFtdcUserSessionField_ProtocolInfo_get, _thosttraderapi.CThostFtdcUserSessionField_ProtocolInfo_set)
    MacAddress = property(_thosttraderapi.CThostFtdcUserSessionField_MacAddress_get, _thosttraderapi.CThostFtdcUserSessionField_MacAddress_set)
    LoginRemark = property(_thosttraderapi.CThostFtdcUserSessionField_LoginRemark_get, _thosttraderapi.CThostFtdcUserSessionField_LoginRemark_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcUserSessionField_swiginit(self, _thosttraderapi.new_CThostFtdcUserSessionField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcUserSessionField

# Register CThostFtdcUserSessionField in _thosttraderapi:
_thosttraderapi.CThostFtdcUserSessionField_swigregister(CThostFtdcUserSessionField)

class CThostFtdcQueryMaxOrderVolumeField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thosttraderapi.CThostFtdcQueryMaxOrderVolumeField_BrokerID_get, _thosttraderapi.CThostFtdcQueryMaxOrderVolumeField_BrokerID_set)
    InvestorID = property(_thosttraderapi.CThostFtdcQueryMaxOrderVolumeField_InvestorID_get, _thosttraderapi.CThostFtdcQueryMaxOrderVolumeField_InvestorID_set)
    InstrumentID = property(_thosttraderapi.CThostFtdcQueryMaxOrderVolumeField_InstrumentID_get, _thosttraderapi.CThostFtdcQueryMaxOrderVolumeField_InstrumentID_set)
    Direction = property(_thosttraderapi.CThostFtdcQueryMaxOrderVolumeField_Direction_get, _thosttraderapi.CThostFtdcQueryMaxOrderVolumeField_Direction_set)
    OffsetFlag = property(_thosttraderapi.CThostFtdcQueryMaxOrderVolumeField_OffsetFlag_get, _thosttraderapi.CThostFtdcQueryMaxOrderVolumeField_OffsetFlag_set)
    HedgeFlag = property(_thosttraderapi.CThostFtdcQueryMaxOrderVolumeField_HedgeFlag_get, _thosttraderapi.CThostFtdcQueryMaxOrderVolumeField_HedgeFlag_set)
    MaxVolume = property(_thosttraderapi.CThostFtdcQueryMaxOrderVolumeField_MaxVolume_get, _thosttraderapi.CThostFtdcQueryMaxOrderVolumeField_MaxVolume_set)
    ExchangeID = property(_thosttraderapi.CThostFtdcQueryMaxOrderVolumeField_ExchangeID_get, _thosttraderapi.CThostFtdcQueryMaxOrderVolumeField_ExchangeID_set)
    InvestUnitID = property(_thosttraderapi.CThostFtdcQueryMaxOrderVolumeField_InvestUnitID_get, _thosttraderapi.CThostFtdcQueryMaxOrderVolumeField_InvestUnitID_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcQueryMaxOrderVolumeField_swiginit(self, _thosttraderapi.new_CThostFtdcQueryMaxOrderVolumeField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcQueryMaxOrderVolumeField

# Register CThostFtdcQueryMaxOrderVolumeField in _thosttraderapi:
_thosttraderapi.CThostFtdcQueryMaxOrderVolumeField_swigregister(CThostFtdcQueryMaxOrderVolumeField)

class CThostFtdcSettlementInfoConfirmField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thosttraderapi.CThostFtdcSettlementInfoConfirmField_BrokerID_get, _thosttraderapi.CThostFtdcSettlementInfoConfirmField_BrokerID_set)
    InvestorID = property(_thosttraderapi.CThostFtdcSettlementInfoConfirmField_InvestorID_get, _thosttraderapi.CThostFtdcSettlementInfoConfirmField_InvestorID_set)
    ConfirmDate = property(_thosttraderapi.CThostFtdcSettlementInfoConfirmField_ConfirmDate_get, _thosttraderapi.CThostFtdcSettlementInfoConfirmField_ConfirmDate_set)
    ConfirmTime = property(_thosttraderapi.CThostFtdcSettlementInfoConfirmField_ConfirmTime_get, _thosttraderapi.CThostFtdcSettlementInfoConfirmField_ConfirmTime_set)
    SettlementID = property(_thosttraderapi.CThostFtdcSettlementInfoConfirmField_SettlementID_get, _thosttraderapi.CThostFtdcSettlementInfoConfirmField_SettlementID_set)
    AccountID = property(_thosttraderapi.CThostFtdcSettlementInfoConfirmField_AccountID_get, _thosttraderapi.CThostFtdcSettlementInfoConfirmField_AccountID_set)
    CurrencyID = property(_thosttraderapi.CThostFtdcSettlementInfoConfirmField_CurrencyID_get, _thosttraderapi.CThostFtdcSettlementInfoConfirmField_CurrencyID_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcSettlementInfoConfirmField_swiginit(self, _thosttraderapi.new_CThostFtdcSettlementInfoConfirmField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcSettlementInfoConfirmField

# Register CThostFtdcSettlementInfoConfirmField in _thosttraderapi:
_thosttraderapi.CThostFtdcSettlementInfoConfirmField_swigregister(CThostFtdcSettlementInfoConfirmField)

class CThostFtdcSyncDepositField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    DepositSeqNo = property(_thosttraderapi.CThostFtdcSyncDepositField_DepositSeqNo_get, _thosttraderapi.CThostFtdcSyncDepositField_DepositSeqNo_set)
    BrokerID = property(_thosttraderapi.CThostFtdcSyncDepositField_BrokerID_get, _thosttraderapi.CThostFtdcSyncDepositField_BrokerID_set)
    InvestorID = property(_thosttraderapi.CThostFtdcSyncDepositField_InvestorID_get, _thosttraderapi.CThostFtdcSyncDepositField_InvestorID_set)
    Deposit = property(_thosttraderapi.CThostFtdcSyncDepositField_Deposit_get, _thosttraderapi.CThostFtdcSyncDepositField_Deposit_set)
    IsForce = property(_thosttraderapi.CThostFtdcSyncDepositField_IsForce_get, _thosttraderapi.CThostFtdcSyncDepositField_IsForce_set)
    CurrencyID = property(_thosttraderapi.CThostFtdcSyncDepositField_CurrencyID_get, _thosttraderapi.CThostFtdcSyncDepositField_CurrencyID_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcSyncDepositField_swiginit(self, _thosttraderapi.new_CThostFtdcSyncDepositField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcSyncDepositField

# Register CThostFtdcSyncDepositField in _thosttraderapi:
_thosttraderapi.CThostFtdcSyncDepositField_swigregister(CThostFtdcSyncDepositField)

class CThostFtdcSyncFundMortgageField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    MortgageSeqNo = property(_thosttraderapi.CThostFtdcSyncFundMortgageField_MortgageSeqNo_get, _thosttraderapi.CThostFtdcSyncFundMortgageField_MortgageSeqNo_set)
    BrokerID = property(_thosttraderapi.CThostFtdcSyncFundMortgageField_BrokerID_get, _thosttraderapi.CThostFtdcSyncFundMortgageField_BrokerID_set)
    InvestorID = property(_thosttraderapi.CThostFtdcSyncFundMortgageField_InvestorID_get, _thosttraderapi.CThostFtdcSyncFundMortgageField_InvestorID_set)
    FromCurrencyID = property(_thosttraderapi.CThostFtdcSyncFundMortgageField_FromCurrencyID_get, _thosttraderapi.CThostFtdcSyncFundMortgageField_FromCurrencyID_set)
    MortgageAmount = property(_thosttraderapi.CThostFtdcSyncFundMortgageField_MortgageAmount_get, _thosttraderapi.CThostFtdcSyncFundMortgageField_MortgageAmount_set)
    ToCurrencyID = property(_thosttraderapi.CThostFtdcSyncFundMortgageField_ToCurrencyID_get, _thosttraderapi.CThostFtdcSyncFundMortgageField_ToCurrencyID_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcSyncFundMortgageField_swiginit(self, _thosttraderapi.new_CThostFtdcSyncFundMortgageField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcSyncFundMortgageField

# Register CThostFtdcSyncFundMortgageField in _thosttraderapi:
_thosttraderapi.CThostFtdcSyncFundMortgageField_swigregister(CThostFtdcSyncFundMortgageField)

class CThostFtdcBrokerSyncField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thosttraderapi.CThostFtdcBrokerSyncField_BrokerID_get, _thosttraderapi.CThostFtdcBrokerSyncField_BrokerID_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcBrokerSyncField_swiginit(self, _thosttraderapi.new_CThostFtdcBrokerSyncField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcBrokerSyncField

# Register CThostFtdcBrokerSyncField in _thosttraderapi:
_thosttraderapi.CThostFtdcBrokerSyncField_swigregister(CThostFtdcBrokerSyncField)

class CThostFtdcSyncingInvestorField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    InvestorID = property(_thosttraderapi.CThostFtdcSyncingInvestorField_InvestorID_get, _thosttraderapi.CThostFtdcSyncingInvestorField_InvestorID_set)
    BrokerID = property(_thosttraderapi.CThostFtdcSyncingInvestorField_BrokerID_get, _thosttraderapi.CThostFtdcSyncingInvestorField_BrokerID_set)
    InvestorGroupID = property(_thosttraderapi.CThostFtdcSyncingInvestorField_InvestorGroupID_get, _thosttraderapi.CThostFtdcSyncingInvestorField_InvestorGroupID_set)
    InvestorName = property(_thosttraderapi.CThostFtdcSyncingInvestorField_InvestorName_get, _thosttraderapi.CThostFtdcSyncingInvestorField_InvestorName_set)
    IdentifiedCardType = property(_thosttraderapi.CThostFtdcSyncingInvestorField_IdentifiedCardType_get, _thosttraderapi.CThostFtdcSyncingInvestorField_IdentifiedCardType_set)
    IdentifiedCardNo = property(_thosttraderapi.CThostFtdcSyncingInvestorField_IdentifiedCardNo_get, _thosttraderapi.CThostFtdcSyncingInvestorField_IdentifiedCardNo_set)
    IsActive = property(_thosttraderapi.CThostFtdcSyncingInvestorField_IsActive_get, _thosttraderapi.CThostFtdcSyncingInvestorField_IsActive_set)
    Telephone = property(_thosttraderapi.CThostFtdcSyncingInvestorField_Telephone_get, _thosttraderapi.CThostFtdcSyncingInvestorField_Telephone_set)
    Address = property(_thosttraderapi.CThostFtdcSyncingInvestorField_Address_get, _thosttraderapi.CThostFtdcSyncingInvestorField_Address_set)
    OpenDate = property(_thosttraderapi.CThostFtdcSyncingInvestorField_OpenDate_get, _thosttraderapi.CThostFtdcSyncingInvestorField_OpenDate_set)
    Mobile = property(_thosttraderapi.CThostFtdcSyncingInvestorField_Mobile_get, _thosttraderapi.CThostFtdcSyncingInvestorField_Mobile_set)
    CommModelID = property(_thosttraderapi.CThostFtdcSyncingInvestorField_CommModelID_get, _thosttraderapi.CThostFtdcSyncingInvestorField_CommModelID_set)
    MarginModelID = property(_thosttraderapi.CThostFtdcSyncingInvestorField_MarginModelID_get, _thosttraderapi.CThostFtdcSyncingInvestorField_MarginModelID_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcSyncingInvestorField_swiginit(self, _thosttraderapi.new_CThostFtdcSyncingInvestorField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcSyncingInvestorField

# Register CThostFtdcSyncingInvestorField in _thosttraderapi:
_thosttraderapi.CThostFtdcSyncingInvestorField_swigregister(CThostFtdcSyncingInvestorField)

class CThostFtdcSyncingTradingCodeField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    InvestorID = property(_thosttraderapi.CThostFtdcSyncingTradingCodeField_InvestorID_get, _thosttraderapi.CThostFtdcSyncingTradingCodeField_InvestorID_set)
    BrokerID = property(_thosttraderapi.CThostFtdcSyncingTradingCodeField_BrokerID_get, _thosttraderapi.CThostFtdcSyncingTradingCodeField_BrokerID_set)
    ExchangeID = property(_thosttraderapi.CThostFtdcSyncingTradingCodeField_ExchangeID_get, _thosttraderapi.CThostFtdcSyncingTradingCodeField_ExchangeID_set)
    ClientID = property(_thosttraderapi.CThostFtdcSyncingTradingCodeField_ClientID_get, _thosttraderapi.CThostFtdcSyncingTradingCodeField_ClientID_set)
    IsActive = property(_thosttraderapi.CThostFtdcSyncingTradingCodeField_IsActive_get, _thosttraderapi.CThostFtdcSyncingTradingCodeField_IsActive_set)
    ClientIDType = property(_thosttraderapi.CThostFtdcSyncingTradingCodeField_ClientIDType_get, _thosttraderapi.CThostFtdcSyncingTradingCodeField_ClientIDType_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcSyncingTradingCodeField_swiginit(self, _thosttraderapi.new_CThostFtdcSyncingTradingCodeField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcSyncingTradingCodeField

# Register CThostFtdcSyncingTradingCodeField in _thosttraderapi:
_thosttraderapi.CThostFtdcSyncingTradingCodeField_swigregister(CThostFtdcSyncingTradingCodeField)

class CThostFtdcSyncingInvestorGroupField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thosttraderapi.CThostFtdcSyncingInvestorGroupField_BrokerID_get, _thosttraderapi.CThostFtdcSyncingInvestorGroupField_BrokerID_set)
    InvestorGroupID = property(_thosttraderapi.CThostFtdcSyncingInvestorGroupField_InvestorGroupID_get, _thosttraderapi.CThostFtdcSyncingInvestorGroupField_InvestorGroupID_set)
    InvestorGroupName = property(_thosttraderapi.CThostFtdcSyncingInvestorGroupField_InvestorGroupName_get, _thosttraderapi.CThostFtdcSyncingInvestorGroupField_InvestorGroupName_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcSyncingInvestorGroupField_swiginit(self, _thosttraderapi.new_CThostFtdcSyncingInvestorGroupField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcSyncingInvestorGroupField

# Register CThostFtdcSyncingInvestorGroupField in _thosttraderapi:
_thosttraderapi.CThostFtdcSyncingInvestorGroupField_swigregister(CThostFtdcSyncingInvestorGroupField)

class CThostFtdcSyncingTradingAccountField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thosttraderapi.CThostFtdcSyncingTradingAccountField_BrokerID_get, _thosttraderapi.CThostFtdcSyncingTradingAccountField_BrokerID_set)
    AccountID = property(_thosttraderapi.CThostFtdcSyncingTradingAccountField_AccountID_get, _thosttraderapi.CThostFtdcSyncingTradingAccountField_AccountID_set)
    PreMortgage = property(_thosttraderapi.CThostFtdcSyncingTradingAccountField_PreMortgage_get, _thosttraderapi.CThostFtdcSyncingTradingAccountField_PreMortgage_set)
    PreCredit = property(_thosttraderapi.CThostFtdcSyncingTradingAccountField_PreCredit_get, _thosttraderapi.CThostFtdcSyncingTradingAccountField_PreCredit_set)
    PreDeposit = property(_thosttraderapi.CThostFtdcSyncingTradingAccountField_PreDeposit_get, _thosttraderapi.CThostFtdcSyncingTradingAccountField_PreDeposit_set)
    PreBalance = property(_thosttraderapi.CThostFtdcSyncingTradingAccountField_PreBalance_get, _thosttraderapi.CThostFtdcSyncingTradingAccountField_PreBalance_set)
    PreMargin = property(_thosttraderapi.CThostFtdcSyncingTradingAccountField_PreMargin_get, _thosttraderapi.CThostFtdcSyncingTradingAccountField_PreMargin_set)
    InterestBase = property(_thosttraderapi.CThostFtdcSyncingTradingAccountField_InterestBase_get, _thosttraderapi.CThostFtdcSyncingTradingAccountField_InterestBase_set)
    Interest = property(_thosttraderapi.CThostFtdcSyncingTradingAccountField_Interest_get, _thosttraderapi.CThostFtdcSyncingTradingAccountField_Interest_set)
    Deposit = property(_thosttraderapi.CThostFtdcSyncingTradingAccountField_Deposit_get, _thosttraderapi.CThostFtdcSyncingTradingAccountField_Deposit_set)
    Withdraw = property(_thosttraderapi.CThostFtdcSyncingTradingAccountField_Withdraw_get, _thosttraderapi.CThostFtdcSyncingTradingAccountField_Withdraw_set)
    FrozenMargin = property(_thosttraderapi.CThostFtdcSyncingTradingAccountField_FrozenMargin_get, _thosttraderapi.CThostFtdcSyncingTradingAccountField_FrozenMargin_set)
    FrozenCash = property(_thosttraderapi.CThostFtdcSyncingTradingAccountField_FrozenCash_get, _thosttraderapi.CThostFtdcSyncingTradingAccountField_FrozenCash_set)
    FrozenCommission = property(_thosttraderapi.CThostFtdcSyncingTradingAccountField_FrozenCommission_get, _thosttraderapi.CThostFtdcSyncingTradingAccountField_FrozenCommission_set)
    CurrMargin = property(_thosttraderapi.CThostFtdcSyncingTradingAccountField_CurrMargin_get, _thosttraderapi.CThostFtdcSyncingTradingAccountField_CurrMargin_set)
    CashIn = property(_thosttraderapi.CThostFtdcSyncingTradingAccountField_CashIn_get, _thosttraderapi.CThostFtdcSyncingTradingAccountField_CashIn_set)
    Commission = property(_thosttraderapi.CThostFtdcSyncingTradingAccountField_Commission_get, _thosttraderapi.CThostFtdcSyncingTradingAccountField_Commission_set)
    CloseProfit = property(_thosttraderapi.CThostFtdcSyncingTradingAccountField_CloseProfit_get, _thosttraderapi.CThostFtdcSyncingTradingAccountField_CloseProfit_set)
    PositionProfit = property(_thosttraderapi.CThostFtdcSyncingTradingAccountField_PositionProfit_get, _thosttraderapi.CThostFtdcSyncingTradingAccountField_PositionProfit_set)
    Balance = property(_thosttraderapi.CThostFtdcSyncingTradingAccountField_Balance_get, _thosttraderapi.CThostFtdcSyncingTradingAccountField_Balance_set)
    Available = property(_thosttraderapi.CThostFtdcSyncingTradingAccountField_Available_get, _thosttraderapi.CThostFtdcSyncingTradingAccountField_Available_set)
    WithdrawQuota = property(_thosttraderapi.CThostFtdcSyncingTradingAccountField_WithdrawQuota_get, _thosttraderapi.CThostFtdcSyncingTradingAccountField_WithdrawQuota_set)
    Reserve = property(_thosttraderapi.CThostFtdcSyncingTradingAccountField_Reserve_get, _thosttraderapi.CThostFtdcSyncingTradingAccountField_Reserve_set)
    TradingDay = property(_thosttraderapi.CThostFtdcSyncingTradingAccountField_TradingDay_get, _thosttraderapi.CThostFtdcSyncingTradingAccountField_TradingDay_set)
    SettlementID = property(_thosttraderapi.CThostFtdcSyncingTradingAccountField_SettlementID_get, _thosttraderapi.CThostFtdcSyncingTradingAccountField_SettlementID_set)
    Credit = property(_thosttraderapi.CThostFtdcSyncingTradingAccountField_Credit_get, _thosttraderapi.CThostFtdcSyncingTradingAccountField_Credit_set)
    Mortgage = property(_thosttraderapi.CThostFtdcSyncingTradingAccountField_Mortgage_get, _thosttraderapi.CThostFtdcSyncingTradingAccountField_Mortgage_set)
    ExchangeMargin = property(_thosttraderapi.CThostFtdcSyncingTradingAccountField_ExchangeMargin_get, _thosttraderapi.CThostFtdcSyncingTradingAccountField_ExchangeMargin_set)
    DeliveryMargin = property(_thosttraderapi.CThostFtdcSyncingTradingAccountField_DeliveryMargin_get, _thosttraderapi.CThostFtdcSyncingTradingAccountField_DeliveryMargin_set)
    ExchangeDeliveryMargin = property(_thosttraderapi.CThostFtdcSyncingTradingAccountField_ExchangeDeliveryMargin_get, _thosttraderapi.CThostFtdcSyncingTradingAccountField_ExchangeDeliveryMargin_set)
    ReserveBalance = property(_thosttraderapi.CThostFtdcSyncingTradingAccountField_ReserveBalance_get, _thosttraderapi.CThostFtdcSyncingTradingAccountField_ReserveBalance_set)
    CurrencyID = property(_thosttraderapi.CThostFtdcSyncingTradingAccountField_CurrencyID_get, _thosttraderapi.CThostFtdcSyncingTradingAccountField_CurrencyID_set)
    PreFundMortgageIn = property(_thosttraderapi.CThostFtdcSyncingTradingAccountField_PreFundMortgageIn_get, _thosttraderapi.CThostFtdcSyncingTradingAccountField_PreFundMortgageIn_set)
    PreFundMortgageOut = property(_thosttraderapi.CThostFtdcSyncingTradingAccountField_PreFundMortgageOut_get, _thosttraderapi.CThostFtdcSyncingTradingAccountField_PreFundMortgageOut_set)
    FundMortgageIn = property(_thosttraderapi.CThostFtdcSyncingTradingAccountField_FundMortgageIn_get, _thosttraderapi.CThostFtdcSyncingTradingAccountField_FundMortgageIn_set)
    FundMortgageOut = property(_thosttraderapi.CThostFtdcSyncingTradingAccountField_FundMortgageOut_get, _thosttraderapi.CThostFtdcSyncingTradingAccountField_FundMortgageOut_set)
    FundMortgageAvailable = property(_thosttraderapi.CThostFtdcSyncingTradingAccountField_FundMortgageAvailable_get, _thosttraderapi.CThostFtdcSyncingTradingAccountField_FundMortgageAvailable_set)
    MortgageableFund = property(_thosttraderapi.CThostFtdcSyncingTradingAccountField_MortgageableFund_get, _thosttraderapi.CThostFtdcSyncingTradingAccountField_MortgageableFund_set)
    SpecProductMargin = property(_thosttraderapi.CThostFtdcSyncingTradingAccountField_SpecProductMargin_get, _thosttraderapi.CThostFtdcSyncingTradingAccountField_SpecProductMargin_set)
    SpecProductFrozenMargin = property(_thosttraderapi.CThostFtdcSyncingTradingAccountField_SpecProductFrozenMargin_get, _thosttraderapi.CThostFtdcSyncingTradingAccountField_SpecProductFrozenMargin_set)
    SpecProductCommission = property(_thosttraderapi.CThostFtdcSyncingTradingAccountField_SpecProductCommission_get, _thosttraderapi.CThostFtdcSyncingTradingAccountField_SpecProductCommission_set)
    SpecProductFrozenCommission = property(_thosttraderapi.CThostFtdcSyncingTradingAccountField_SpecProductFrozenCommission_get, _thosttraderapi.CThostFtdcSyncingTradingAccountField_SpecProductFrozenCommission_set)
    SpecProductPositionProfit = property(_thosttraderapi.CThostFtdcSyncingTradingAccountField_SpecProductPositionProfit_get, _thosttraderapi.CThostFtdcSyncingTradingAccountField_SpecProductPositionProfit_set)
    SpecProductCloseProfit = property(_thosttraderapi.CThostFtdcSyncingTradingAccountField_SpecProductCloseProfit_get, _thosttraderapi.CThostFtdcSyncingTradingAccountField_SpecProductCloseProfit_set)
    SpecProductPositionProfitByAlg = property(_thosttraderapi.CThostFtdcSyncingTradingAccountField_SpecProductPositionProfitByAlg_get, _thosttraderapi.CThostFtdcSyncingTradingAccountField_SpecProductPositionProfitByAlg_set)
    SpecProductExchangeMargin = property(_thosttraderapi.CThostFtdcSyncingTradingAccountField_SpecProductExchangeMargin_get, _thosttraderapi.CThostFtdcSyncingTradingAccountField_SpecProductExchangeMargin_set)
    FrozenSwap = property(_thosttraderapi.CThostFtdcSyncingTradingAccountField_FrozenSwap_get, _thosttraderapi.CThostFtdcSyncingTradingAccountField_FrozenSwap_set)
    RemainSwap = property(_thosttraderapi.CThostFtdcSyncingTradingAccountField_RemainSwap_get, _thosttraderapi.CThostFtdcSyncingTradingAccountField_RemainSwap_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcSyncingTradingAccountField_swiginit(self, _thosttraderapi.new_CThostFtdcSyncingTradingAccountField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcSyncingTradingAccountField

# Register CThostFtdcSyncingTradingAccountField in _thosttraderapi:
_thosttraderapi.CThostFtdcSyncingTradingAccountField_swigregister(CThostFtdcSyncingTradingAccountField)

class CThostFtdcSyncingInvestorPositionField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    InstrumentID = property(_thosttraderapi.CThostFtdcSyncingInvestorPositionField_InstrumentID_get, _thosttraderapi.CThostFtdcSyncingInvestorPositionField_InstrumentID_set)
    BrokerID = property(_thosttraderapi.CThostFtdcSyncingInvestorPositionField_BrokerID_get, _thosttraderapi.CThostFtdcSyncingInvestorPositionField_BrokerID_set)
    InvestorID = property(_thosttraderapi.CThostFtdcSyncingInvestorPositionField_InvestorID_get, _thosttraderapi.CThostFtdcSyncingInvestorPositionField_InvestorID_set)
    PosiDirection = property(_thosttraderapi.CThostFtdcSyncingInvestorPositionField_PosiDirection_get, _thosttraderapi.CThostFtdcSyncingInvestorPositionField_PosiDirection_set)
    HedgeFlag = property(_thosttraderapi.CThostFtdcSyncingInvestorPositionField_HedgeFlag_get, _thosttraderapi.CThostFtdcSyncingInvestorPositionField_HedgeFlag_set)
    PositionDate = property(_thosttraderapi.CThostFtdcSyncingInvestorPositionField_PositionDate_get, _thosttraderapi.CThostFtdcSyncingInvestorPositionField_PositionDate_set)
    YdPosition = property(_thosttraderapi.CThostFtdcSyncingInvestorPositionField_YdPosition_get, _thosttraderapi.CThostFtdcSyncingInvestorPositionField_YdPosition_set)
    Position = property(_thosttraderapi.CThostFtdcSyncingInvestorPositionField_Position_get, _thosttraderapi.CThostFtdcSyncingInvestorPositionField_Position_set)
    LongFrozen = property(_thosttraderapi.CThostFtdcSyncingInvestorPositionField_LongFrozen_get, _thosttraderapi.CThostFtdcSyncingInvestorPositionField_LongFrozen_set)
    ShortFrozen = property(_thosttraderapi.CThostFtdcSyncingInvestorPositionField_ShortFrozen_get, _thosttraderapi.CThostFtdcSyncingInvestorPositionField_ShortFrozen_set)
    LongFrozenAmount = property(_thosttraderapi.CThostFtdcSyncingInvestorPositionField_LongFrozenAmount_get, _thosttraderapi.CThostFtdcSyncingInvestorPositionField_LongFrozenAmount_set)
    ShortFrozenAmount = property(_thosttraderapi.CThostFtdcSyncingInvestorPositionField_ShortFrozenAmount_get, _thosttraderapi.CThostFtdcSyncingInvestorPositionField_ShortFrozenAmount_set)
    OpenVolume = property(_thosttraderapi.CThostFtdcSyncingInvestorPositionField_OpenVolume_get, _thosttraderapi.CThostFtdcSyncingInvestorPositionField_OpenVolume_set)
    CloseVolume = property(_thosttraderapi.CThostFtdcSyncingInvestorPositionField_CloseVolume_get, _thosttraderapi.CThostFtdcSyncingInvestorPositionField_CloseVolume_set)
    OpenAmount = property(_thosttraderapi.CThostFtdcSyncingInvestorPositionField_OpenAmount_get, _thosttraderapi.CThostFtdcSyncingInvestorPositionField_OpenAmount_set)
    CloseAmount = property(_thosttraderapi.CThostFtdcSyncingInvestorPositionField_CloseAmount_get, _thosttraderapi.CThostFtdcSyncingInvestorPositionField_CloseAmount_set)
    PositionCost = property(_thosttraderapi.CThostFtdcSyncingInvestorPositionField_PositionCost_get, _thosttraderapi.CThostFtdcSyncingInvestorPositionField_PositionCost_set)
    PreMargin = property(_thosttraderapi.CThostFtdcSyncingInvestorPositionField_PreMargin_get, _thosttraderapi.CThostFtdcSyncingInvestorPositionField_PreMargin_set)
    UseMargin = property(_thosttraderapi.CThostFtdcSyncingInvestorPositionField_UseMargin_get, _thosttraderapi.CThostFtdcSyncingInvestorPositionField_UseMargin_set)
    FrozenMargin = property(_thosttraderapi.CThostFtdcSyncingInvestorPositionField_FrozenMargin_get, _thosttraderapi.CThostFtdcSyncingInvestorPositionField_FrozenMargin_set)
    FrozenCash = property(_thosttraderapi.CThostFtdcSyncingInvestorPositionField_FrozenCash_get, _thosttraderapi.CThostFtdcSyncingInvestorPositionField_FrozenCash_set)
    FrozenCommission = property(_thosttraderapi.CThostFtdcSyncingInvestorPositionField_FrozenCommission_get, _thosttraderapi.CThostFtdcSyncingInvestorPositionField_FrozenCommission_set)
    CashIn = property(_thosttraderapi.CThostFtdcSyncingInvestorPositionField_CashIn_get, _thosttraderapi.CThostFtdcSyncingInvestorPositionField_CashIn_set)
    Commission = property(_thosttraderapi.CThostFtdcSyncingInvestorPositionField_Commission_get, _thosttraderapi.CThostFtdcSyncingInvestorPositionField_Commission_set)
    CloseProfit = property(_thosttraderapi.CThostFtdcSyncingInvestorPositionField_CloseProfit_get, _thosttraderapi.CThostFtdcSyncingInvestorPositionField_CloseProfit_set)
    PositionProfit = property(_thosttraderapi.CThostFtdcSyncingInvestorPositionField_PositionProfit_get, _thosttraderapi.CThostFtdcSyncingInvestorPositionField_PositionProfit_set)
    PreSettlementPrice = property(_thosttraderapi.CThostFtdcSyncingInvestorPositionField_PreSettlementPrice_get, _thosttraderapi.CThostFtdcSyncingInvestorPositionField_PreSettlementPrice_set)
    SettlementPrice = property(_thosttraderapi.CThostFtdcSyncingInvestorPositionField_SettlementPrice_get, _thosttraderapi.CThostFtdcSyncingInvestorPositionField_SettlementPrice_set)
    TradingDay = property(_thosttraderapi.CThostFtdcSyncingInvestorPositionField_TradingDay_get, _thosttraderapi.CThostFtdcSyncingInvestorPositionField_TradingDay_set)
    SettlementID = property(_thosttraderapi.CThostFtdcSyncingInvestorPositionField_SettlementID_get, _thosttraderapi.CThostFtdcSyncingInvestorPositionField_SettlementID_set)
    OpenCost = property(_thosttraderapi.CThostFtdcSyncingInvestorPositionField_OpenCost_get, _thosttraderapi.CThostFtdcSyncingInvestorPositionField_OpenCost_set)
    ExchangeMargin = property(_thosttraderapi.CThostFtdcSyncingInvestorPositionField_ExchangeMargin_get, _thosttraderapi.CThostFtdcSyncingInvestorPositionField_ExchangeMargin_set)
    CombPosition = property(_thosttraderapi.CThostFtdcSyncingInvestorPositionField_CombPosition_get, _thosttraderapi.CThostFtdcSyncingInvestorPositionField_CombPosition_set)
    CombLongFrozen = property(_thosttraderapi.CThostFtdcSyncingInvestorPositionField_CombLongFrozen_get, _thosttraderapi.CThostFtdcSyncingInvestorPositionField_CombLongFrozen_set)
    CombShortFrozen = property(_thosttraderapi.CThostFtdcSyncingInvestorPositionField_CombShortFrozen_get, _thosttraderapi.CThostFtdcSyncingInvestorPositionField_CombShortFrozen_set)
    CloseProfitByDate = property(_thosttraderapi.CThostFtdcSyncingInvestorPositionField_CloseProfitByDate_get, _thosttraderapi.CThostFtdcSyncingInvestorPositionField_CloseProfitByDate_set)
    CloseProfitByTrade = property(_thosttraderapi.CThostFtdcSyncingInvestorPositionField_CloseProfitByTrade_get, _thosttraderapi.CThostFtdcSyncingInvestorPositionField_CloseProfitByTrade_set)
    TodayPosition = property(_thosttraderapi.CThostFtdcSyncingInvestorPositionField_TodayPosition_get, _thosttraderapi.CThostFtdcSyncingInvestorPositionField_TodayPosition_set)
    MarginRateByMoney = property(_thosttraderapi.CThostFtdcSyncingInvestorPositionField_MarginRateByMoney_get, _thosttraderapi.CThostFtdcSyncingInvestorPositionField_MarginRateByMoney_set)
    MarginRateByVolume = property(_thosttraderapi.CThostFtdcSyncingInvestorPositionField_MarginRateByVolume_get, _thosttraderapi.CThostFtdcSyncingInvestorPositionField_MarginRateByVolume_set)
    StrikeFrozen = property(_thosttraderapi.CThostFtdcSyncingInvestorPositionField_StrikeFrozen_get, _thosttraderapi.CThostFtdcSyncingInvestorPositionField_StrikeFrozen_set)
    StrikeFrozenAmount = property(_thosttraderapi.CThostFtdcSyncingInvestorPositionField_StrikeFrozenAmount_get, _thosttraderapi.CThostFtdcSyncingInvestorPositionField_StrikeFrozenAmount_set)
    AbandonFrozen = property(_thosttraderapi.CThostFtdcSyncingInvestorPositionField_AbandonFrozen_get, _thosttraderapi.CThostFtdcSyncingInvestorPositionField_AbandonFrozen_set)
    ExchangeID = property(_thosttraderapi.CThostFtdcSyncingInvestorPositionField_ExchangeID_get, _thosttraderapi.CThostFtdcSyncingInvestorPositionField_ExchangeID_set)
    YdStrikeFrozen = property(_thosttraderapi.CThostFtdcSyncingInvestorPositionField_YdStrikeFrozen_get, _thosttraderapi.CThostFtdcSyncingInvestorPositionField_YdStrikeFrozen_set)
    InvestUnitID = property(_thosttraderapi.CThostFtdcSyncingInvestorPositionField_InvestUnitID_get, _thosttraderapi.CThostFtdcSyncingInvestorPositionField_InvestUnitID_set)
    PositionCostOffset = property(_thosttraderapi.CThostFtdcSyncingInvestorPositionField_PositionCostOffset_get, _thosttraderapi.CThostFtdcSyncingInvestorPositionField_PositionCostOffset_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcSyncingInvestorPositionField_swiginit(self, _thosttraderapi.new_CThostFtdcSyncingInvestorPositionField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcSyncingInvestorPositionField

# Register CThostFtdcSyncingInvestorPositionField in _thosttraderapi:
_thosttraderapi.CThostFtdcSyncingInvestorPositionField_swigregister(CThostFtdcSyncingInvestorPositionField)

class CThostFtdcSyncingInstrumentMarginRateField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    InstrumentID = property(_thosttraderapi.CThostFtdcSyncingInstrumentMarginRateField_InstrumentID_get, _thosttraderapi.CThostFtdcSyncingInstrumentMarginRateField_InstrumentID_set)
    InvestorRange = property(_thosttraderapi.CThostFtdcSyncingInstrumentMarginRateField_InvestorRange_get, _thosttraderapi.CThostFtdcSyncingInstrumentMarginRateField_InvestorRange_set)
    BrokerID = property(_thosttraderapi.CThostFtdcSyncingInstrumentMarginRateField_BrokerID_get, _thosttraderapi.CThostFtdcSyncingInstrumentMarginRateField_BrokerID_set)
    InvestorID = property(_thosttraderapi.CThostFtdcSyncingInstrumentMarginRateField_InvestorID_get, _thosttraderapi.CThostFtdcSyncingInstrumentMarginRateField_InvestorID_set)
    HedgeFlag = property(_thosttraderapi.CThostFtdcSyncingInstrumentMarginRateField_HedgeFlag_get, _thosttraderapi.CThostFtdcSyncingInstrumentMarginRateField_HedgeFlag_set)
    LongMarginRatioByMoney = property(_thosttraderapi.CThostFtdcSyncingInstrumentMarginRateField_LongMarginRatioByMoney_get, _thosttraderapi.CThostFtdcSyncingInstrumentMarginRateField_LongMarginRatioByMoney_set)
    LongMarginRatioByVolume = property(_thosttraderapi.CThostFtdcSyncingInstrumentMarginRateField_LongMarginRatioByVolume_get, _thosttraderapi.CThostFtdcSyncingInstrumentMarginRateField_LongMarginRatioByVolume_set)
    ShortMarginRatioByMoney = property(_thosttraderapi.CThostFtdcSyncingInstrumentMarginRateField_ShortMarginRatioByMoney_get, _thosttraderapi.CThostFtdcSyncingInstrumentMarginRateField_ShortMarginRatioByMoney_set)
    ShortMarginRatioByVolume = property(_thosttraderapi.CThostFtdcSyncingInstrumentMarginRateField_ShortMarginRatioByVolume_get, _thosttraderapi.CThostFtdcSyncingInstrumentMarginRateField_ShortMarginRatioByVolume_set)
    IsRelative = property(_thosttraderapi.CThostFtdcSyncingInstrumentMarginRateField_IsRelative_get, _thosttraderapi.CThostFtdcSyncingInstrumentMarginRateField_IsRelative_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcSyncingInstrumentMarginRateField_swiginit(self, _thosttraderapi.new_CThostFtdcSyncingInstrumentMarginRateField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcSyncingInstrumentMarginRateField

# Register CThostFtdcSyncingInstrumentMarginRateField in _thosttraderapi:
_thosttraderapi.CThostFtdcSyncingInstrumentMarginRateField_swigregister(CThostFtdcSyncingInstrumentMarginRateField)

class CThostFtdcSyncingInstrumentCommissionRateField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    InstrumentID = property(_thosttraderapi.CThostFtdcSyncingInstrumentCommissionRateField_InstrumentID_get, _thosttraderapi.CThostFtdcSyncingInstrumentCommissionRateField_InstrumentID_set)
    InvestorRange = property(_thosttraderapi.CThostFtdcSyncingInstrumentCommissionRateField_InvestorRange_get, _thosttraderapi.CThostFtdcSyncingInstrumentCommissionRateField_InvestorRange_set)
    BrokerID = property(_thosttraderapi.CThostFtdcSyncingInstrumentCommissionRateField_BrokerID_get, _thosttraderapi.CThostFtdcSyncingInstrumentCommissionRateField_BrokerID_set)
    InvestorID = property(_thosttraderapi.CThostFtdcSyncingInstrumentCommissionRateField_InvestorID_get, _thosttraderapi.CThostFtdcSyncingInstrumentCommissionRateField_InvestorID_set)
    OpenRatioByMoney = property(_thosttraderapi.CThostFtdcSyncingInstrumentCommissionRateField_OpenRatioByMoney_get, _thosttraderapi.CThostFtdcSyncingInstrumentCommissionRateField_OpenRatioByMoney_set)
    OpenRatioByVolume = property(_thosttraderapi.CThostFtdcSyncingInstrumentCommissionRateField_OpenRatioByVolume_get, _thosttraderapi.CThostFtdcSyncingInstrumentCommissionRateField_OpenRatioByVolume_set)
    CloseRatioByMoney = property(_thosttraderapi.CThostFtdcSyncingInstrumentCommissionRateField_CloseRatioByMoney_get, _thosttraderapi.CThostFtdcSyncingInstrumentCommissionRateField_CloseRatioByMoney_set)
    CloseRatioByVolume = property(_thosttraderapi.CThostFtdcSyncingInstrumentCommissionRateField_CloseRatioByVolume_get, _thosttraderapi.CThostFtdcSyncingInstrumentCommissionRateField_CloseRatioByVolume_set)
    CloseTodayRatioByMoney = property(_thosttraderapi.CThostFtdcSyncingInstrumentCommissionRateField_CloseTodayRatioByMoney_get, _thosttraderapi.CThostFtdcSyncingInstrumentCommissionRateField_CloseTodayRatioByMoney_set)
    CloseTodayRatioByVolume = property(_thosttraderapi.CThostFtdcSyncingInstrumentCommissionRateField_CloseTodayRatioByVolume_get, _thosttraderapi.CThostFtdcSyncingInstrumentCommissionRateField_CloseTodayRatioByVolume_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcSyncingInstrumentCommissionRateField_swiginit(self, _thosttraderapi.new_CThostFtdcSyncingInstrumentCommissionRateField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcSyncingInstrumentCommissionRateField

# Register CThostFtdcSyncingInstrumentCommissionRateField in _thosttraderapi:
_thosttraderapi.CThostFtdcSyncingInstrumentCommissionRateField_swigregister(CThostFtdcSyncingInstrumentCommissionRateField)

class CThostFtdcSyncingInstrumentTradingRightField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    InstrumentID = property(_thosttraderapi.CThostFtdcSyncingInstrumentTradingRightField_InstrumentID_get, _thosttraderapi.CThostFtdcSyncingInstrumentTradingRightField_InstrumentID_set)
    InvestorRange = property(_thosttraderapi.CThostFtdcSyncingInstrumentTradingRightField_InvestorRange_get, _thosttraderapi.CThostFtdcSyncingInstrumentTradingRightField_InvestorRange_set)
    BrokerID = property(_thosttraderapi.CThostFtdcSyncingInstrumentTradingRightField_BrokerID_get, _thosttraderapi.CThostFtdcSyncingInstrumentTradingRightField_BrokerID_set)
    InvestorID = property(_thosttraderapi.CThostFtdcSyncingInstrumentTradingRightField_InvestorID_get, _thosttraderapi.CThostFtdcSyncingInstrumentTradingRightField_InvestorID_set)
    TradingRight = property(_thosttraderapi.CThostFtdcSyncingInstrumentTradingRightField_TradingRight_get, _thosttraderapi.CThostFtdcSyncingInstrumentTradingRightField_TradingRight_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcSyncingInstrumentTradingRightField_swiginit(self, _thosttraderapi.new_CThostFtdcSyncingInstrumentTradingRightField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcSyncingInstrumentTradingRightField

# Register CThostFtdcSyncingInstrumentTradingRightField in _thosttraderapi:
_thosttraderapi.CThostFtdcSyncingInstrumentTradingRightField_swigregister(CThostFtdcSyncingInstrumentTradingRightField)

class CThostFtdcQryOrderField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thosttraderapi.CThostFtdcQryOrderField_BrokerID_get, _thosttraderapi.CThostFtdcQryOrderField_BrokerID_set)
    InvestorID = property(_thosttraderapi.CThostFtdcQryOrderField_InvestorID_get, _thosttraderapi.CThostFtdcQryOrderField_InvestorID_set)
    InstrumentID = property(_thosttraderapi.CThostFtdcQryOrderField_InstrumentID_get, _thosttraderapi.CThostFtdcQryOrderField_InstrumentID_set)
    ExchangeID = property(_thosttraderapi.CThostFtdcQryOrderField_ExchangeID_get, _thosttraderapi.CThostFtdcQryOrderField_ExchangeID_set)
    OrderSysID = property(_thosttraderapi.CThostFtdcQryOrderField_OrderSysID_get, _thosttraderapi.CThostFtdcQryOrderField_OrderSysID_set)
    InsertTimeStart = property(_thosttraderapi.CThostFtdcQryOrderField_InsertTimeStart_get, _thosttraderapi.CThostFtdcQryOrderField_InsertTimeStart_set)
    InsertTimeEnd = property(_thosttraderapi.CThostFtdcQryOrderField_InsertTimeEnd_get, _thosttraderapi.CThostFtdcQryOrderField_InsertTimeEnd_set)
    InvestUnitID = property(_thosttraderapi.CThostFtdcQryOrderField_InvestUnitID_get, _thosttraderapi.CThostFtdcQryOrderField_InvestUnitID_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcQryOrderField_swiginit(self, _thosttraderapi.new_CThostFtdcQryOrderField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcQryOrderField

# Register CThostFtdcQryOrderField in _thosttraderapi:
_thosttraderapi.CThostFtdcQryOrderField_swigregister(CThostFtdcQryOrderField)

class CThostFtdcQryTradeField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thosttraderapi.CThostFtdcQryTradeField_BrokerID_get, _thosttraderapi.CThostFtdcQryTradeField_BrokerID_set)
    InvestorID = property(_thosttraderapi.CThostFtdcQryTradeField_InvestorID_get, _thosttraderapi.CThostFtdcQryTradeField_InvestorID_set)
    InstrumentID = property(_thosttraderapi.CThostFtdcQryTradeField_InstrumentID_get, _thosttraderapi.CThostFtdcQryTradeField_InstrumentID_set)
    ExchangeID = property(_thosttraderapi.CThostFtdcQryTradeField_ExchangeID_get, _thosttraderapi.CThostFtdcQryTradeField_ExchangeID_set)
    TradeID = property(_thosttraderapi.CThostFtdcQryTradeField_TradeID_get, _thosttraderapi.CThostFtdcQryTradeField_TradeID_set)
    TradeTimeStart = property(_thosttraderapi.CThostFtdcQryTradeField_TradeTimeStart_get, _thosttraderapi.CThostFtdcQryTradeField_TradeTimeStart_set)
    TradeTimeEnd = property(_thosttraderapi.CThostFtdcQryTradeField_TradeTimeEnd_get, _thosttraderapi.CThostFtdcQryTradeField_TradeTimeEnd_set)
    InvestUnitID = property(_thosttraderapi.CThostFtdcQryTradeField_InvestUnitID_get, _thosttraderapi.CThostFtdcQryTradeField_InvestUnitID_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcQryTradeField_swiginit(self, _thosttraderapi.new_CThostFtdcQryTradeField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcQryTradeField

# Register CThostFtdcQryTradeField in _thosttraderapi:
_thosttraderapi.CThostFtdcQryTradeField_swigregister(CThostFtdcQryTradeField)

class CThostFtdcQryInvestorPositionField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thosttraderapi.CThostFtdcQryInvestorPositionField_BrokerID_get, _thosttraderapi.CThostFtdcQryInvestorPositionField_BrokerID_set)
    InvestorID = property(_thosttraderapi.CThostFtdcQryInvestorPositionField_InvestorID_get, _thosttraderapi.CThostFtdcQryInvestorPositionField_InvestorID_set)
    InstrumentID = property(_thosttraderapi.CThostFtdcQryInvestorPositionField_InstrumentID_get, _thosttraderapi.CThostFtdcQryInvestorPositionField_InstrumentID_set)
    ExchangeID = property(_thosttraderapi.CThostFtdcQryInvestorPositionField_ExchangeID_get, _thosttraderapi.CThostFtdcQryInvestorPositionField_ExchangeID_set)
    InvestUnitID = property(_thosttraderapi.CThostFtdcQryInvestorPositionField_InvestUnitID_get, _thosttraderapi.CThostFtdcQryInvestorPositionField_InvestUnitID_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcQryInvestorPositionField_swiginit(self, _thosttraderapi.new_CThostFtdcQryInvestorPositionField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcQryInvestorPositionField

# Register CThostFtdcQryInvestorPositionField in _thosttraderapi:
_thosttraderapi.CThostFtdcQryInvestorPositionField_swigregister(CThostFtdcQryInvestorPositionField)

class CThostFtdcQryTradingAccountField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thosttraderapi.CThostFtdcQryTradingAccountField_BrokerID_get, _thosttraderapi.CThostFtdcQryTradingAccountField_BrokerID_set)
    InvestorID = property(_thosttraderapi.CThostFtdcQryTradingAccountField_InvestorID_get, _thosttraderapi.CThostFtdcQryTradingAccountField_InvestorID_set)
    CurrencyID = property(_thosttraderapi.CThostFtdcQryTradingAccountField_CurrencyID_get, _thosttraderapi.CThostFtdcQryTradingAccountField_CurrencyID_set)
    BizType = property(_thosttraderapi.CThostFtdcQryTradingAccountField_BizType_get, _thosttraderapi.CThostFtdcQryTradingAccountField_BizType_set)
    AccountID = property(_thosttraderapi.CThostFtdcQryTradingAccountField_AccountID_get, _thosttraderapi.CThostFtdcQryTradingAccountField_AccountID_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcQryTradingAccountField_swiginit(self, _thosttraderapi.new_CThostFtdcQryTradingAccountField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcQryTradingAccountField

# Register CThostFtdcQryTradingAccountField in _thosttraderapi:
_thosttraderapi.CThostFtdcQryTradingAccountField_swigregister(CThostFtdcQryTradingAccountField)

class CThostFtdcQryInvestorField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thosttraderapi.CThostFtdcQryInvestorField_BrokerID_get, _thosttraderapi.CThostFtdcQryInvestorField_BrokerID_set)
    InvestorID = property(_thosttraderapi.CThostFtdcQryInvestorField_InvestorID_get, _thosttraderapi.CThostFtdcQryInvestorField_InvestorID_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcQryInvestorField_swiginit(self, _thosttraderapi.new_CThostFtdcQryInvestorField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcQryInvestorField

# Register CThostFtdcQryInvestorField in _thosttraderapi:
_thosttraderapi.CThostFtdcQryInvestorField_swigregister(CThostFtdcQryInvestorField)

class CThostFtdcQryTradingCodeField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thosttraderapi.CThostFtdcQryTradingCodeField_BrokerID_get, _thosttraderapi.CThostFtdcQryTradingCodeField_BrokerID_set)
    InvestorID = property(_thosttraderapi.CThostFtdcQryTradingCodeField_InvestorID_get, _thosttraderapi.CThostFtdcQryTradingCodeField_InvestorID_set)
    ExchangeID = property(_thosttraderapi.CThostFtdcQryTradingCodeField_ExchangeID_get, _thosttraderapi.CThostFtdcQryTradingCodeField_ExchangeID_set)
    ClientID = property(_thosttraderapi.CThostFtdcQryTradingCodeField_ClientID_get, _thosttraderapi.CThostFtdcQryTradingCodeField_ClientID_set)
    ClientIDType = property(_thosttraderapi.CThostFtdcQryTradingCodeField_ClientIDType_get, _thosttraderapi.CThostFtdcQryTradingCodeField_ClientIDType_set)
    InvestUnitID = property(_thosttraderapi.CThostFtdcQryTradingCodeField_InvestUnitID_get, _thosttraderapi.CThostFtdcQryTradingCodeField_InvestUnitID_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcQryTradingCodeField_swiginit(self, _thosttraderapi.new_CThostFtdcQryTradingCodeField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcQryTradingCodeField

# Register CThostFtdcQryTradingCodeField in _thosttraderapi:
_thosttraderapi.CThostFtdcQryTradingCodeField_swigregister(CThostFtdcQryTradingCodeField)

class CThostFtdcQryInvestorGroupField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thosttraderapi.CThostFtdcQryInvestorGroupField_BrokerID_get, _thosttraderapi.CThostFtdcQryInvestorGroupField_BrokerID_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcQryInvestorGroupField_swiginit(self, _thosttraderapi.new_CThostFtdcQryInvestorGroupField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcQryInvestorGroupField

# Register CThostFtdcQryInvestorGroupField in _thosttraderapi:
_thosttraderapi.CThostFtdcQryInvestorGroupField_swigregister(CThostFtdcQryInvestorGroupField)

class CThostFtdcQryInstrumentMarginRateField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thosttraderapi.CThostFtdcQryInstrumentMarginRateField_BrokerID_get, _thosttraderapi.CThostFtdcQryInstrumentMarginRateField_BrokerID_set)
    InvestorID = property(_thosttraderapi.CThostFtdcQryInstrumentMarginRateField_InvestorID_get, _thosttraderapi.CThostFtdcQryInstrumentMarginRateField_InvestorID_set)
    InstrumentID = property(_thosttraderapi.CThostFtdcQryInstrumentMarginRateField_InstrumentID_get, _thosttraderapi.CThostFtdcQryInstrumentMarginRateField_InstrumentID_set)
    HedgeFlag = property(_thosttraderapi.CThostFtdcQryInstrumentMarginRateField_HedgeFlag_get, _thosttraderapi.CThostFtdcQryInstrumentMarginRateField_HedgeFlag_set)
    ExchangeID = property(_thosttraderapi.CThostFtdcQryInstrumentMarginRateField_ExchangeID_get, _thosttraderapi.CThostFtdcQryInstrumentMarginRateField_ExchangeID_set)
    InvestUnitID = property(_thosttraderapi.CThostFtdcQryInstrumentMarginRateField_InvestUnitID_get, _thosttraderapi.CThostFtdcQryInstrumentMarginRateField_InvestUnitID_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcQryInstrumentMarginRateField_swiginit(self, _thosttraderapi.new_CThostFtdcQryInstrumentMarginRateField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcQryInstrumentMarginRateField

# Register CThostFtdcQryInstrumentMarginRateField in _thosttraderapi:
_thosttraderapi.CThostFtdcQryInstrumentMarginRateField_swigregister(CThostFtdcQryInstrumentMarginRateField)

class CThostFtdcQryInstrumentCommissionRateField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thosttraderapi.CThostFtdcQryInstrumentCommissionRateField_BrokerID_get, _thosttraderapi.CThostFtdcQryInstrumentCommissionRateField_BrokerID_set)
    InvestorID = property(_thosttraderapi.CThostFtdcQryInstrumentCommissionRateField_InvestorID_get, _thosttraderapi.CThostFtdcQryInstrumentCommissionRateField_InvestorID_set)
    InstrumentID = property(_thosttraderapi.CThostFtdcQryInstrumentCommissionRateField_InstrumentID_get, _thosttraderapi.CThostFtdcQryInstrumentCommissionRateField_InstrumentID_set)
    ExchangeID = property(_thosttraderapi.CThostFtdcQryInstrumentCommissionRateField_ExchangeID_get, _thosttraderapi.CThostFtdcQryInstrumentCommissionRateField_ExchangeID_set)
    InvestUnitID = property(_thosttraderapi.CThostFtdcQryInstrumentCommissionRateField_InvestUnitID_get, _thosttraderapi.CThostFtdcQryInstrumentCommissionRateField_InvestUnitID_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcQryInstrumentCommissionRateField_swiginit(self, _thosttraderapi.new_CThostFtdcQryInstrumentCommissionRateField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcQryInstrumentCommissionRateField

# Register CThostFtdcQryInstrumentCommissionRateField in _thosttraderapi:
_thosttraderapi.CThostFtdcQryInstrumentCommissionRateField_swigregister(CThostFtdcQryInstrumentCommissionRateField)

class CThostFtdcQryInstrumentTradingRightField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thosttraderapi.CThostFtdcQryInstrumentTradingRightField_BrokerID_get, _thosttraderapi.CThostFtdcQryInstrumentTradingRightField_BrokerID_set)
    InvestorID = property(_thosttraderapi.CThostFtdcQryInstrumentTradingRightField_InvestorID_get, _thosttraderapi.CThostFtdcQryInstrumentTradingRightField_InvestorID_set)
    InstrumentID = property(_thosttraderapi.CThostFtdcQryInstrumentTradingRightField_InstrumentID_get, _thosttraderapi.CThostFtdcQryInstrumentTradingRightField_InstrumentID_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcQryInstrumentTradingRightField_swiginit(self, _thosttraderapi.new_CThostFtdcQryInstrumentTradingRightField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcQryInstrumentTradingRightField

# Register CThostFtdcQryInstrumentTradingRightField in _thosttraderapi:
_thosttraderapi.CThostFtdcQryInstrumentTradingRightField_swigregister(CThostFtdcQryInstrumentTradingRightField)

class CThostFtdcQryBrokerField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thosttraderapi.CThostFtdcQryBrokerField_BrokerID_get, _thosttraderapi.CThostFtdcQryBrokerField_BrokerID_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcQryBrokerField_swiginit(self, _thosttraderapi.new_CThostFtdcQryBrokerField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcQryBrokerField

# Register CThostFtdcQryBrokerField in _thosttraderapi:
_thosttraderapi.CThostFtdcQryBrokerField_swigregister(CThostFtdcQryBrokerField)

class CThostFtdcQryTraderField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    ExchangeID = property(_thosttraderapi.CThostFtdcQryTraderField_ExchangeID_get, _thosttraderapi.CThostFtdcQryTraderField_ExchangeID_set)
    ParticipantID = property(_thosttraderapi.CThostFtdcQryTraderField_ParticipantID_get, _thosttraderapi.CThostFtdcQryTraderField_ParticipantID_set)
    TraderID = property(_thosttraderapi.CThostFtdcQryTraderField_TraderID_get, _thosttraderapi.CThostFtdcQryTraderField_TraderID_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcQryTraderField_swiginit(self, _thosttraderapi.new_CThostFtdcQryTraderField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcQryTraderField

# Register CThostFtdcQryTraderField in _thosttraderapi:
_thosttraderapi.CThostFtdcQryTraderField_swigregister(CThostFtdcQryTraderField)

class CThostFtdcQrySuperUserFunctionField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    UserID = property(_thosttraderapi.CThostFtdcQrySuperUserFunctionField_UserID_get, _thosttraderapi.CThostFtdcQrySuperUserFunctionField_UserID_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcQrySuperUserFunctionField_swiginit(self, _thosttraderapi.new_CThostFtdcQrySuperUserFunctionField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcQrySuperUserFunctionField

# Register CThostFtdcQrySuperUserFunctionField in _thosttraderapi:
_thosttraderapi.CThostFtdcQrySuperUserFunctionField_swigregister(CThostFtdcQrySuperUserFunctionField)

class CThostFtdcQryUserSessionField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    FrontID = property(_thosttraderapi.CThostFtdcQryUserSessionField_FrontID_get, _thosttraderapi.CThostFtdcQryUserSessionField_FrontID_set)
    SessionID = property(_thosttraderapi.CThostFtdcQryUserSessionField_SessionID_get, _thosttraderapi.CThostFtdcQryUserSessionField_SessionID_set)
    BrokerID = property(_thosttraderapi.CThostFtdcQryUserSessionField_BrokerID_get, _thosttraderapi.CThostFtdcQryUserSessionField_BrokerID_set)
    UserID = property(_thosttraderapi.CThostFtdcQryUserSessionField_UserID_get, _thosttraderapi.CThostFtdcQryUserSessionField_UserID_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcQryUserSessionField_swiginit(self, _thosttraderapi.new_CThostFtdcQryUserSessionField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcQryUserSessionField

# Register CThostFtdcQryUserSessionField in _thosttraderapi:
_thosttraderapi.CThostFtdcQryUserSessionField_swigregister(CThostFtdcQryUserSessionField)

class CThostFtdcQryPartBrokerField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    ExchangeID = property(_thosttraderapi.CThostFtdcQryPartBrokerField_ExchangeID_get, _thosttraderapi.CThostFtdcQryPartBrokerField_ExchangeID_set)
    BrokerID = property(_thosttraderapi.CThostFtdcQryPartBrokerField_BrokerID_get, _thosttraderapi.CThostFtdcQryPartBrokerField_BrokerID_set)
    ParticipantID = property(_thosttraderapi.CThostFtdcQryPartBrokerField_ParticipantID_get, _thosttraderapi.CThostFtdcQryPartBrokerField_ParticipantID_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcQryPartBrokerField_swiginit(self, _thosttraderapi.new_CThostFtdcQryPartBrokerField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcQryPartBrokerField

# Register CThostFtdcQryPartBrokerField in _thosttraderapi:
_thosttraderapi.CThostFtdcQryPartBrokerField_swigregister(CThostFtdcQryPartBrokerField)

class CThostFtdcQryFrontStatusField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    FrontID = property(_thosttraderapi.CThostFtdcQryFrontStatusField_FrontID_get, _thosttraderapi.CThostFtdcQryFrontStatusField_FrontID_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcQryFrontStatusField_swiginit(self, _thosttraderapi.new_CThostFtdcQryFrontStatusField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcQryFrontStatusField

# Register CThostFtdcQryFrontStatusField in _thosttraderapi:
_thosttraderapi.CThostFtdcQryFrontStatusField_swigregister(CThostFtdcQryFrontStatusField)

class CThostFtdcQryExchangeOrderField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    ParticipantID = property(_thosttraderapi.CThostFtdcQryExchangeOrderField_ParticipantID_get, _thosttraderapi.CThostFtdcQryExchangeOrderField_ParticipantID_set)
    ClientID = property(_thosttraderapi.CThostFtdcQryExchangeOrderField_ClientID_get, _thosttraderapi.CThostFtdcQryExchangeOrderField_ClientID_set)
    ExchangeInstID = property(_thosttraderapi.CThostFtdcQryExchangeOrderField_ExchangeInstID_get, _thosttraderapi.CThostFtdcQryExchangeOrderField_ExchangeInstID_set)
    ExchangeID = property(_thosttraderapi.CThostFtdcQryExchangeOrderField_ExchangeID_get, _thosttraderapi.CThostFtdcQryExchangeOrderField_ExchangeID_set)
    TraderID = property(_thosttraderapi.CThostFtdcQryExchangeOrderField_TraderID_get, _thosttraderapi.CThostFtdcQryExchangeOrderField_TraderID_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcQryExchangeOrderField_swiginit(self, _thosttraderapi.new_CThostFtdcQryExchangeOrderField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcQryExchangeOrderField

# Register CThostFtdcQryExchangeOrderField in _thosttraderapi:
_thosttraderapi.CThostFtdcQryExchangeOrderField_swigregister(CThostFtdcQryExchangeOrderField)

class CThostFtdcQryOrderActionField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thosttraderapi.CThostFtdcQryOrderActionField_BrokerID_get, _thosttraderapi.CThostFtdcQryOrderActionField_BrokerID_set)
    InvestorID = property(_thosttraderapi.CThostFtdcQryOrderActionField_InvestorID_get, _thosttraderapi.CThostFtdcQryOrderActionField_InvestorID_set)
    ExchangeID = property(_thosttraderapi.CThostFtdcQryOrderActionField_ExchangeID_get, _thosttraderapi.CThostFtdcQryOrderActionField_ExchangeID_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcQryOrderActionField_swiginit(self, _thosttraderapi.new_CThostFtdcQryOrderActionField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcQryOrderActionField

# Register CThostFtdcQryOrderActionField in _thosttraderapi:
_thosttraderapi.CThostFtdcQryOrderActionField_swigregister(CThostFtdcQryOrderActionField)

class CThostFtdcQryExchangeOrderActionField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    ParticipantID = property(_thosttraderapi.CThostFtdcQryExchangeOrderActionField_ParticipantID_get, _thosttraderapi.CThostFtdcQryExchangeOrderActionField_ParticipantID_set)
    ClientID = property(_thosttraderapi.CThostFtdcQryExchangeOrderActionField_ClientID_get, _thosttraderapi.CThostFtdcQryExchangeOrderActionField_ClientID_set)
    ExchangeID = property(_thosttraderapi.CThostFtdcQryExchangeOrderActionField_ExchangeID_get, _thosttraderapi.CThostFtdcQryExchangeOrderActionField_ExchangeID_set)
    TraderID = property(_thosttraderapi.CThostFtdcQryExchangeOrderActionField_TraderID_get, _thosttraderapi.CThostFtdcQryExchangeOrderActionField_TraderID_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcQryExchangeOrderActionField_swiginit(self, _thosttraderapi.new_CThostFtdcQryExchangeOrderActionField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcQryExchangeOrderActionField

# Register CThostFtdcQryExchangeOrderActionField in _thosttraderapi:
_thosttraderapi.CThostFtdcQryExchangeOrderActionField_swigregister(CThostFtdcQryExchangeOrderActionField)

class CThostFtdcQrySuperUserField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    UserID = property(_thosttraderapi.CThostFtdcQrySuperUserField_UserID_get, _thosttraderapi.CThostFtdcQrySuperUserField_UserID_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcQrySuperUserField_swiginit(self, _thosttraderapi.new_CThostFtdcQrySuperUserField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcQrySuperUserField

# Register CThostFtdcQrySuperUserField in _thosttraderapi:
_thosttraderapi.CThostFtdcQrySuperUserField_swigregister(CThostFtdcQrySuperUserField)

class CThostFtdcQryExchangeField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    ExchangeID = property(_thosttraderapi.CThostFtdcQryExchangeField_ExchangeID_get, _thosttraderapi.CThostFtdcQryExchangeField_ExchangeID_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcQryExchangeField_swiginit(self, _thosttraderapi.new_CThostFtdcQryExchangeField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcQryExchangeField

# Register CThostFtdcQryExchangeField in _thosttraderapi:
_thosttraderapi.CThostFtdcQryExchangeField_swigregister(CThostFtdcQryExchangeField)

class CThostFtdcQryProductField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    ProductID = property(_thosttraderapi.CThostFtdcQryProductField_ProductID_get, _thosttraderapi.CThostFtdcQryProductField_ProductID_set)
    ProductClass = property(_thosttraderapi.CThostFtdcQryProductField_ProductClass_get, _thosttraderapi.CThostFtdcQryProductField_ProductClass_set)
    ExchangeID = property(_thosttraderapi.CThostFtdcQryProductField_ExchangeID_get, _thosttraderapi.CThostFtdcQryProductField_ExchangeID_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcQryProductField_swiginit(self, _thosttraderapi.new_CThostFtdcQryProductField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcQryProductField

# Register CThostFtdcQryProductField in _thosttraderapi:
_thosttraderapi.CThostFtdcQryProductField_swigregister(CThostFtdcQryProductField)

class CThostFtdcQryInstrumentField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    InstrumentID = property(_thosttraderapi.CThostFtdcQryInstrumentField_InstrumentID_get, _thosttraderapi.CThostFtdcQryInstrumentField_InstrumentID_set)
    ExchangeID = property(_thosttraderapi.CThostFtdcQryInstrumentField_ExchangeID_get, _thosttraderapi.CThostFtdcQryInstrumentField_ExchangeID_set)
    ExchangeInstID = property(_thosttraderapi.CThostFtdcQryInstrumentField_ExchangeInstID_get, _thosttraderapi.CThostFtdcQryInstrumentField_ExchangeInstID_set)
    ProductID = property(_thosttraderapi.CThostFtdcQryInstrumentField_ProductID_get, _thosttraderapi.CThostFtdcQryInstrumentField_ProductID_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcQryInstrumentField_swiginit(self, _thosttraderapi.new_CThostFtdcQryInstrumentField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcQryInstrumentField

# Register CThostFtdcQryInstrumentField in _thosttraderapi:
_thosttraderapi.CThostFtdcQryInstrumentField_swigregister(CThostFtdcQryInstrumentField)

class CThostFtdcQryDepthMarketDataField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    InstrumentID = property(_thosttraderapi.CThostFtdcQryDepthMarketDataField_InstrumentID_get, _thosttraderapi.CThostFtdcQryDepthMarketDataField_InstrumentID_set)
    ExchangeID = property(_thosttraderapi.CThostFtdcQryDepthMarketDataField_ExchangeID_get, _thosttraderapi.CThostFtdcQryDepthMarketDataField_ExchangeID_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcQryDepthMarketDataField_swiginit(self, _thosttraderapi.new_CThostFtdcQryDepthMarketDataField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcQryDepthMarketDataField

# Register CThostFtdcQryDepthMarketDataField in _thosttraderapi:
_thosttraderapi.CThostFtdcQryDepthMarketDataField_swigregister(CThostFtdcQryDepthMarketDataField)

class CThostFtdcQryBrokerUserField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thosttraderapi.CThostFtdcQryBrokerUserField_BrokerID_get, _thosttraderapi.CThostFtdcQryBrokerUserField_BrokerID_set)
    UserID = property(_thosttraderapi.CThostFtdcQryBrokerUserField_UserID_get, _thosttraderapi.CThostFtdcQryBrokerUserField_UserID_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcQryBrokerUserField_swiginit(self, _thosttraderapi.new_CThostFtdcQryBrokerUserField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcQryBrokerUserField

# Register CThostFtdcQryBrokerUserField in _thosttraderapi:
_thosttraderapi.CThostFtdcQryBrokerUserField_swigregister(CThostFtdcQryBrokerUserField)

class CThostFtdcQryBrokerUserFunctionField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thosttraderapi.CThostFtdcQryBrokerUserFunctionField_BrokerID_get, _thosttraderapi.CThostFtdcQryBrokerUserFunctionField_BrokerID_set)
    UserID = property(_thosttraderapi.CThostFtdcQryBrokerUserFunctionField_UserID_get, _thosttraderapi.CThostFtdcQryBrokerUserFunctionField_UserID_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcQryBrokerUserFunctionField_swiginit(self, _thosttraderapi.new_CThostFtdcQryBrokerUserFunctionField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcQryBrokerUserFunctionField

# Register CThostFtdcQryBrokerUserFunctionField in _thosttraderapi:
_thosttraderapi.CThostFtdcQryBrokerUserFunctionField_swigregister(CThostFtdcQryBrokerUserFunctionField)

class CThostFtdcQryTraderOfferField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    ExchangeID = property(_thosttraderapi.CThostFtdcQryTraderOfferField_ExchangeID_get, _thosttraderapi.CThostFtdcQryTraderOfferField_ExchangeID_set)
    ParticipantID = property(_thosttraderapi.CThostFtdcQryTraderOfferField_ParticipantID_get, _thosttraderapi.CThostFtdcQryTraderOfferField_ParticipantID_set)
    TraderID = property(_thosttraderapi.CThostFtdcQryTraderOfferField_TraderID_get, _thosttraderapi.CThostFtdcQryTraderOfferField_TraderID_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcQryTraderOfferField_swiginit(self, _thosttraderapi.new_CThostFtdcQryTraderOfferField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcQryTraderOfferField

# Register CThostFtdcQryTraderOfferField in _thosttraderapi:
_thosttraderapi.CThostFtdcQryTraderOfferField_swigregister(CThostFtdcQryTraderOfferField)

class CThostFtdcQrySyncDepositField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thosttraderapi.CThostFtdcQrySyncDepositField_BrokerID_get, _thosttraderapi.CThostFtdcQrySyncDepositField_BrokerID_set)
    DepositSeqNo = property(_thosttraderapi.CThostFtdcQrySyncDepositField_DepositSeqNo_get, _thosttraderapi.CThostFtdcQrySyncDepositField_DepositSeqNo_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcQrySyncDepositField_swiginit(self, _thosttraderapi.new_CThostFtdcQrySyncDepositField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcQrySyncDepositField

# Register CThostFtdcQrySyncDepositField in _thosttraderapi:
_thosttraderapi.CThostFtdcQrySyncDepositField_swigregister(CThostFtdcQrySyncDepositField)

class CThostFtdcQrySettlementInfoField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thosttraderapi.CThostFtdcQrySettlementInfoField_BrokerID_get, _thosttraderapi.CThostFtdcQrySettlementInfoField_BrokerID_set)
    InvestorID = property(_thosttraderapi.CThostFtdcQrySettlementInfoField_InvestorID_get, _thosttraderapi.CThostFtdcQrySettlementInfoField_InvestorID_set)
    TradingDay = property(_thosttraderapi.CThostFtdcQrySettlementInfoField_TradingDay_get, _thosttraderapi.CThostFtdcQrySettlementInfoField_TradingDay_set)
    AccountID = property(_thosttraderapi.CThostFtdcQrySettlementInfoField_AccountID_get, _thosttraderapi.CThostFtdcQrySettlementInfoField_AccountID_set)
    CurrencyID = property(_thosttraderapi.CThostFtdcQrySettlementInfoField_CurrencyID_get, _thosttraderapi.CThostFtdcQrySettlementInfoField_CurrencyID_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcQrySettlementInfoField_swiginit(self, _thosttraderapi.new_CThostFtdcQrySettlementInfoField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcQrySettlementInfoField

# Register CThostFtdcQrySettlementInfoField in _thosttraderapi:
_thosttraderapi.CThostFtdcQrySettlementInfoField_swigregister(CThostFtdcQrySettlementInfoField)

class CThostFtdcQryExchangeMarginRateField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thosttraderapi.CThostFtdcQryExchangeMarginRateField_BrokerID_get, _thosttraderapi.CThostFtdcQryExchangeMarginRateField_BrokerID_set)
    InstrumentID = property(_thosttraderapi.CThostFtdcQryExchangeMarginRateField_InstrumentID_get, _thosttraderapi.CThostFtdcQryExchangeMarginRateField_InstrumentID_set)
    HedgeFlag = property(_thosttraderapi.CThostFtdcQryExchangeMarginRateField_HedgeFlag_get, _thosttraderapi.CThostFtdcQryExchangeMarginRateField_HedgeFlag_set)
    ExchangeID = property(_thosttraderapi.CThostFtdcQryExchangeMarginRateField_ExchangeID_get, _thosttraderapi.CThostFtdcQryExchangeMarginRateField_ExchangeID_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcQryExchangeMarginRateField_swiginit(self, _thosttraderapi.new_CThostFtdcQryExchangeMarginRateField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcQryExchangeMarginRateField

# Register CThostFtdcQryExchangeMarginRateField in _thosttraderapi:
_thosttraderapi.CThostFtdcQryExchangeMarginRateField_swigregister(CThostFtdcQryExchangeMarginRateField)

class CThostFtdcQryExchangeMarginRateAdjustField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thosttraderapi.CThostFtdcQryExchangeMarginRateAdjustField_BrokerID_get, _thosttraderapi.CThostFtdcQryExchangeMarginRateAdjustField_BrokerID_set)
    InstrumentID = property(_thosttraderapi.CThostFtdcQryExchangeMarginRateAdjustField_InstrumentID_get, _thosttraderapi.CThostFtdcQryExchangeMarginRateAdjustField_InstrumentID_set)
    HedgeFlag = property(_thosttraderapi.CThostFtdcQryExchangeMarginRateAdjustField_HedgeFlag_get, _thosttraderapi.CThostFtdcQryExchangeMarginRateAdjustField_HedgeFlag_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcQryExchangeMarginRateAdjustField_swiginit(self, _thosttraderapi.new_CThostFtdcQryExchangeMarginRateAdjustField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcQryExchangeMarginRateAdjustField

# Register CThostFtdcQryExchangeMarginRateAdjustField in _thosttraderapi:
_thosttraderapi.CThostFtdcQryExchangeMarginRateAdjustField_swigregister(CThostFtdcQryExchangeMarginRateAdjustField)

class CThostFtdcQryExchangeRateField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thosttraderapi.CThostFtdcQryExchangeRateField_BrokerID_get, _thosttraderapi.CThostFtdcQryExchangeRateField_BrokerID_set)
    FromCurrencyID = property(_thosttraderapi.CThostFtdcQryExchangeRateField_FromCurrencyID_get, _thosttraderapi.CThostFtdcQryExchangeRateField_FromCurrencyID_set)
    ToCurrencyID = property(_thosttraderapi.CThostFtdcQryExchangeRateField_ToCurrencyID_get, _thosttraderapi.CThostFtdcQryExchangeRateField_ToCurrencyID_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcQryExchangeRateField_swiginit(self, _thosttraderapi.new_CThostFtdcQryExchangeRateField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcQryExchangeRateField

# Register CThostFtdcQryExchangeRateField in _thosttraderapi:
_thosttraderapi.CThostFtdcQryExchangeRateField_swigregister(CThostFtdcQryExchangeRateField)

class CThostFtdcQrySyncFundMortgageField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thosttraderapi.CThostFtdcQrySyncFundMortgageField_BrokerID_get, _thosttraderapi.CThostFtdcQrySyncFundMortgageField_BrokerID_set)
    MortgageSeqNo = property(_thosttraderapi.CThostFtdcQrySyncFundMortgageField_MortgageSeqNo_get, _thosttraderapi.CThostFtdcQrySyncFundMortgageField_MortgageSeqNo_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcQrySyncFundMortgageField_swiginit(self, _thosttraderapi.new_CThostFtdcQrySyncFundMortgageField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcQrySyncFundMortgageField

# Register CThostFtdcQrySyncFundMortgageField in _thosttraderapi:
_thosttraderapi.CThostFtdcQrySyncFundMortgageField_swigregister(CThostFtdcQrySyncFundMortgageField)

class CThostFtdcQryHisOrderField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thosttraderapi.CThostFtdcQryHisOrderField_BrokerID_get, _thosttraderapi.CThostFtdcQryHisOrderField_BrokerID_set)
    InvestorID = property(_thosttraderapi.CThostFtdcQryHisOrderField_InvestorID_get, _thosttraderapi.CThostFtdcQryHisOrderField_InvestorID_set)
    InstrumentID = property(_thosttraderapi.CThostFtdcQryHisOrderField_InstrumentID_get, _thosttraderapi.CThostFtdcQryHisOrderField_InstrumentID_set)
    ExchangeID = property(_thosttraderapi.CThostFtdcQryHisOrderField_ExchangeID_get, _thosttraderapi.CThostFtdcQryHisOrderField_ExchangeID_set)
    OrderSysID = property(_thosttraderapi.CThostFtdcQryHisOrderField_OrderSysID_get, _thosttraderapi.CThostFtdcQryHisOrderField_OrderSysID_set)
    InsertTimeStart = property(_thosttraderapi.CThostFtdcQryHisOrderField_InsertTimeStart_get, _thosttraderapi.CThostFtdcQryHisOrderField_InsertTimeStart_set)
    InsertTimeEnd = property(_thosttraderapi.CThostFtdcQryHisOrderField_InsertTimeEnd_get, _thosttraderapi.CThostFtdcQryHisOrderField_InsertTimeEnd_set)
    TradingDay = property(_thosttraderapi.CThostFtdcQryHisOrderField_TradingDay_get, _thosttraderapi.CThostFtdcQryHisOrderField_TradingDay_set)
    SettlementID = property(_thosttraderapi.CThostFtdcQryHisOrderField_SettlementID_get, _thosttraderapi.CThostFtdcQryHisOrderField_SettlementID_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcQryHisOrderField_swiginit(self, _thosttraderapi.new_CThostFtdcQryHisOrderField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcQryHisOrderField

# Register CThostFtdcQryHisOrderField in _thosttraderapi:
_thosttraderapi.CThostFtdcQryHisOrderField_swigregister(CThostFtdcQryHisOrderField)

class CThostFtdcOptionInstrMiniMarginField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    InstrumentID = property(_thosttraderapi.CThostFtdcOptionInstrMiniMarginField_InstrumentID_get, _thosttraderapi.CThostFtdcOptionInstrMiniMarginField_InstrumentID_set)
    InvestorRange = property(_thosttraderapi.CThostFtdcOptionInstrMiniMarginField_InvestorRange_get, _thosttraderapi.CThostFtdcOptionInstrMiniMarginField_InvestorRange_set)
    BrokerID = property(_thosttraderapi.CThostFtdcOptionInstrMiniMarginField_BrokerID_get, _thosttraderapi.CThostFtdcOptionInstrMiniMarginField_BrokerID_set)
    InvestorID = property(_thosttraderapi.CThostFtdcOptionInstrMiniMarginField_InvestorID_get, _thosttraderapi.CThostFtdcOptionInstrMiniMarginField_InvestorID_set)
    MinMargin = property(_thosttraderapi.CThostFtdcOptionInstrMiniMarginField_MinMargin_get, _thosttraderapi.CThostFtdcOptionInstrMiniMarginField_MinMargin_set)
    ValueMethod = property(_thosttraderapi.CThostFtdcOptionInstrMiniMarginField_ValueMethod_get, _thosttraderapi.CThostFtdcOptionInstrMiniMarginField_ValueMethod_set)
    IsRelative = property(_thosttraderapi.CThostFtdcOptionInstrMiniMarginField_IsRelative_get, _thosttraderapi.CThostFtdcOptionInstrMiniMarginField_IsRelative_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcOptionInstrMiniMarginField_swiginit(self, _thosttraderapi.new_CThostFtdcOptionInstrMiniMarginField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcOptionInstrMiniMarginField

# Register CThostFtdcOptionInstrMiniMarginField in _thosttraderapi:
_thosttraderapi.CThostFtdcOptionInstrMiniMarginField_swigregister(CThostFtdcOptionInstrMiniMarginField)

class CThostFtdcOptionInstrMarginAdjustField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    InstrumentID = property(_thosttraderapi.CThostFtdcOptionInstrMarginAdjustField_InstrumentID_get, _thosttraderapi.CThostFtdcOptionInstrMarginAdjustField_InstrumentID_set)
    InvestorRange = property(_thosttraderapi.CThostFtdcOptionInstrMarginAdjustField_InvestorRange_get, _thosttraderapi.CThostFtdcOptionInstrMarginAdjustField_InvestorRange_set)
    BrokerID = property(_thosttraderapi.CThostFtdcOptionInstrMarginAdjustField_BrokerID_get, _thosttraderapi.CThostFtdcOptionInstrMarginAdjustField_BrokerID_set)
    InvestorID = property(_thosttraderapi.CThostFtdcOptionInstrMarginAdjustField_InvestorID_get, _thosttraderapi.CThostFtdcOptionInstrMarginAdjustField_InvestorID_set)
    SShortMarginRatioByMoney = property(_thosttraderapi.CThostFtdcOptionInstrMarginAdjustField_SShortMarginRatioByMoney_get, _thosttraderapi.CThostFtdcOptionInstrMarginAdjustField_SShortMarginRatioByMoney_set)
    SShortMarginRatioByVolume = property(_thosttraderapi.CThostFtdcOptionInstrMarginAdjustField_SShortMarginRatioByVolume_get, _thosttraderapi.CThostFtdcOptionInstrMarginAdjustField_SShortMarginRatioByVolume_set)
    HShortMarginRatioByMoney = property(_thosttraderapi.CThostFtdcOptionInstrMarginAdjustField_HShortMarginRatioByMoney_get, _thosttraderapi.CThostFtdcOptionInstrMarginAdjustField_HShortMarginRatioByMoney_set)
    HShortMarginRatioByVolume = property(_thosttraderapi.CThostFtdcOptionInstrMarginAdjustField_HShortMarginRatioByVolume_get, _thosttraderapi.CThostFtdcOptionInstrMarginAdjustField_HShortMarginRatioByVolume_set)
    AShortMarginRatioByMoney = property(_thosttraderapi.CThostFtdcOptionInstrMarginAdjustField_AShortMarginRatioByMoney_get, _thosttraderapi.CThostFtdcOptionInstrMarginAdjustField_AShortMarginRatioByMoney_set)
    AShortMarginRatioByVolume = property(_thosttraderapi.CThostFtdcOptionInstrMarginAdjustField_AShortMarginRatioByVolume_get, _thosttraderapi.CThostFtdcOptionInstrMarginAdjustField_AShortMarginRatioByVolume_set)
    IsRelative = property(_thosttraderapi.CThostFtdcOptionInstrMarginAdjustField_IsRelative_get, _thosttraderapi.CThostFtdcOptionInstrMarginAdjustField_IsRelative_set)
    MShortMarginRatioByMoney = property(_thosttraderapi.CThostFtdcOptionInstrMarginAdjustField_MShortMarginRatioByMoney_get, _thosttraderapi.CThostFtdcOptionInstrMarginAdjustField_MShortMarginRatioByMoney_set)
    MShortMarginRatioByVolume = property(_thosttraderapi.CThostFtdcOptionInstrMarginAdjustField_MShortMarginRatioByVolume_get, _thosttraderapi.CThostFtdcOptionInstrMarginAdjustField_MShortMarginRatioByVolume_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcOptionInstrMarginAdjustField_swiginit(self, _thosttraderapi.new_CThostFtdcOptionInstrMarginAdjustField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcOptionInstrMarginAdjustField

# Register CThostFtdcOptionInstrMarginAdjustField in _thosttraderapi:
_thosttraderapi.CThostFtdcOptionInstrMarginAdjustField_swigregister(CThostFtdcOptionInstrMarginAdjustField)

class CThostFtdcOptionInstrCommRateField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    InstrumentID = property(_thosttraderapi.CThostFtdcOptionInstrCommRateField_InstrumentID_get, _thosttraderapi.CThostFtdcOptionInstrCommRateField_InstrumentID_set)
    InvestorRange = property(_thosttraderapi.CThostFtdcOptionInstrCommRateField_InvestorRange_get, _thosttraderapi.CThostFtdcOptionInstrCommRateField_InvestorRange_set)
    BrokerID = property(_thosttraderapi.CThostFtdcOptionInstrCommRateField_BrokerID_get, _thosttraderapi.CThostFtdcOptionInstrCommRateField_BrokerID_set)
    InvestorID = property(_thosttraderapi.CThostFtdcOptionInstrCommRateField_InvestorID_get, _thosttraderapi.CThostFtdcOptionInstrCommRateField_InvestorID_set)
    OpenRatioByMoney = property(_thosttraderapi.CThostFtdcOptionInstrCommRateField_OpenRatioByMoney_get, _thosttraderapi.CThostFtdcOptionInstrCommRateField_OpenRatioByMoney_set)
    OpenRatioByVolume = property(_thosttraderapi.CThostFtdcOptionInstrCommRateField_OpenRatioByVolume_get, _thosttraderapi.CThostFtdcOptionInstrCommRateField_OpenRatioByVolume_set)
    CloseRatioByMoney = property(_thosttraderapi.CThostFtdcOptionInstrCommRateField_CloseRatioByMoney_get, _thosttraderapi.CThostFtdcOptionInstrCommRateField_CloseRatioByMoney_set)
    CloseRatioByVolume = property(_thosttraderapi.CThostFtdcOptionInstrCommRateField_CloseRatioByVolume_get, _thosttraderapi.CThostFtdcOptionInstrCommRateField_CloseRatioByVolume_set)
    CloseTodayRatioByMoney = property(_thosttraderapi.CThostFtdcOptionInstrCommRateField_CloseTodayRatioByMoney_get, _thosttraderapi.CThostFtdcOptionInstrCommRateField_CloseTodayRatioByMoney_set)
    CloseTodayRatioByVolume = property(_thosttraderapi.CThostFtdcOptionInstrCommRateField_CloseTodayRatioByVolume_get, _thosttraderapi.CThostFtdcOptionInstrCommRateField_CloseTodayRatioByVolume_set)
    StrikeRatioByMoney = property(_thosttraderapi.CThostFtdcOptionInstrCommRateField_StrikeRatioByMoney_get, _thosttraderapi.CThostFtdcOptionInstrCommRateField_StrikeRatioByMoney_set)
    StrikeRatioByVolume = property(_thosttraderapi.CThostFtdcOptionInstrCommRateField_StrikeRatioByVolume_get, _thosttraderapi.CThostFtdcOptionInstrCommRateField_StrikeRatioByVolume_set)
    ExchangeID = property(_thosttraderapi.CThostFtdcOptionInstrCommRateField_ExchangeID_get, _thosttraderapi.CThostFtdcOptionInstrCommRateField_ExchangeID_set)
    InvestUnitID = property(_thosttraderapi.CThostFtdcOptionInstrCommRateField_InvestUnitID_get, _thosttraderapi.CThostFtdcOptionInstrCommRateField_InvestUnitID_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcOptionInstrCommRateField_swiginit(self, _thosttraderapi.new_CThostFtdcOptionInstrCommRateField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcOptionInstrCommRateField

# Register CThostFtdcOptionInstrCommRateField in _thosttraderapi:
_thosttraderapi.CThostFtdcOptionInstrCommRateField_swigregister(CThostFtdcOptionInstrCommRateField)

class CThostFtdcOptionInstrTradeCostField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thosttraderapi.CThostFtdcOptionInstrTradeCostField_BrokerID_get, _thosttraderapi.CThostFtdcOptionInstrTradeCostField_BrokerID_set)
    InvestorID = property(_thosttraderapi.CThostFtdcOptionInstrTradeCostField_InvestorID_get, _thosttraderapi.CThostFtdcOptionInstrTradeCostField_InvestorID_set)
    InstrumentID = property(_thosttraderapi.CThostFtdcOptionInstrTradeCostField_InstrumentID_get, _thosttraderapi.CThostFtdcOptionInstrTradeCostField_InstrumentID_set)
    HedgeFlag = property(_thosttraderapi.CThostFtdcOptionInstrTradeCostField_HedgeFlag_get, _thosttraderapi.CThostFtdcOptionInstrTradeCostField_HedgeFlag_set)
    FixedMargin = property(_thosttraderapi.CThostFtdcOptionInstrTradeCostField_FixedMargin_get, _thosttraderapi.CThostFtdcOptionInstrTradeCostField_FixedMargin_set)
    MiniMargin = property(_thosttraderapi.CThostFtdcOptionInstrTradeCostField_MiniMargin_get, _thosttraderapi.CThostFtdcOptionInstrTradeCostField_MiniMargin_set)
    Royalty = property(_thosttraderapi.CThostFtdcOptionInstrTradeCostField_Royalty_get, _thosttraderapi.CThostFtdcOptionInstrTradeCostField_Royalty_set)
    ExchFixedMargin = property(_thosttraderapi.CThostFtdcOptionInstrTradeCostField_ExchFixedMargin_get, _thosttraderapi.CThostFtdcOptionInstrTradeCostField_ExchFixedMargin_set)
    ExchMiniMargin = property(_thosttraderapi.CThostFtdcOptionInstrTradeCostField_ExchMiniMargin_get, _thosttraderapi.CThostFtdcOptionInstrTradeCostField_ExchMiniMargin_set)
    ExchangeID = property(_thosttraderapi.CThostFtdcOptionInstrTradeCostField_ExchangeID_get, _thosttraderapi.CThostFtdcOptionInstrTradeCostField_ExchangeID_set)
    InvestUnitID = property(_thosttraderapi.CThostFtdcOptionInstrTradeCostField_InvestUnitID_get, _thosttraderapi.CThostFtdcOptionInstrTradeCostField_InvestUnitID_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcOptionInstrTradeCostField_swiginit(self, _thosttraderapi.new_CThostFtdcOptionInstrTradeCostField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcOptionInstrTradeCostField

# Register CThostFtdcOptionInstrTradeCostField in _thosttraderapi:
_thosttraderapi.CThostFtdcOptionInstrTradeCostField_swigregister(CThostFtdcOptionInstrTradeCostField)

class CThostFtdcQryOptionInstrTradeCostField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thosttraderapi.CThostFtdcQryOptionInstrTradeCostField_BrokerID_get, _thosttraderapi.CThostFtdcQryOptionInstrTradeCostField_BrokerID_set)
    InvestorID = property(_thosttraderapi.CThostFtdcQryOptionInstrTradeCostField_InvestorID_get, _thosttraderapi.CThostFtdcQryOptionInstrTradeCostField_InvestorID_set)
    InstrumentID = property(_thosttraderapi.CThostFtdcQryOptionInstrTradeCostField_InstrumentID_get, _thosttraderapi.CThostFtdcQryOptionInstrTradeCostField_InstrumentID_set)
    HedgeFlag = property(_thosttraderapi.CThostFtdcQryOptionInstrTradeCostField_HedgeFlag_get, _thosttraderapi.CThostFtdcQryOptionInstrTradeCostField_HedgeFlag_set)
    InputPrice = property(_thosttraderapi.CThostFtdcQryOptionInstrTradeCostField_InputPrice_get, _thosttraderapi.CThostFtdcQryOptionInstrTradeCostField_InputPrice_set)
    UnderlyingPrice = property(_thosttraderapi.CThostFtdcQryOptionInstrTradeCostField_UnderlyingPrice_get, _thosttraderapi.CThostFtdcQryOptionInstrTradeCostField_UnderlyingPrice_set)
    ExchangeID = property(_thosttraderapi.CThostFtdcQryOptionInstrTradeCostField_ExchangeID_get, _thosttraderapi.CThostFtdcQryOptionInstrTradeCostField_ExchangeID_set)
    InvestUnitID = property(_thosttraderapi.CThostFtdcQryOptionInstrTradeCostField_InvestUnitID_get, _thosttraderapi.CThostFtdcQryOptionInstrTradeCostField_InvestUnitID_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcQryOptionInstrTradeCostField_swiginit(self, _thosttraderapi.new_CThostFtdcQryOptionInstrTradeCostField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcQryOptionInstrTradeCostField

# Register CThostFtdcQryOptionInstrTradeCostField in _thosttraderapi:
_thosttraderapi.CThostFtdcQryOptionInstrTradeCostField_swigregister(CThostFtdcQryOptionInstrTradeCostField)

class CThostFtdcQryOptionInstrCommRateField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thosttraderapi.CThostFtdcQryOptionInstrCommRateField_BrokerID_get, _thosttraderapi.CThostFtdcQryOptionInstrCommRateField_BrokerID_set)
    InvestorID = property(_thosttraderapi.CThostFtdcQryOptionInstrCommRateField_InvestorID_get, _thosttraderapi.CThostFtdcQryOptionInstrCommRateField_InvestorID_set)
    InstrumentID = property(_thosttraderapi.CThostFtdcQryOptionInstrCommRateField_InstrumentID_get, _thosttraderapi.CThostFtdcQryOptionInstrCommRateField_InstrumentID_set)
    ExchangeID = property(_thosttraderapi.CThostFtdcQryOptionInstrCommRateField_ExchangeID_get, _thosttraderapi.CThostFtdcQryOptionInstrCommRateField_ExchangeID_set)
    InvestUnitID = property(_thosttraderapi.CThostFtdcQryOptionInstrCommRateField_InvestUnitID_get, _thosttraderapi.CThostFtdcQryOptionInstrCommRateField_InvestUnitID_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcQryOptionInstrCommRateField_swiginit(self, _thosttraderapi.new_CThostFtdcQryOptionInstrCommRateField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcQryOptionInstrCommRateField

# Register CThostFtdcQryOptionInstrCommRateField in _thosttraderapi:
_thosttraderapi.CThostFtdcQryOptionInstrCommRateField_swigregister(CThostFtdcQryOptionInstrCommRateField)

class CThostFtdcIndexPriceField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thosttraderapi.CThostFtdcIndexPriceField_BrokerID_get, _thosttraderapi.CThostFtdcIndexPriceField_BrokerID_set)
    InstrumentID = property(_thosttraderapi.CThostFtdcIndexPriceField_InstrumentID_get, _thosttraderapi.CThostFtdcIndexPriceField_InstrumentID_set)
    ClosePrice = property(_thosttraderapi.CThostFtdcIndexPriceField_ClosePrice_get, _thosttraderapi.CThostFtdcIndexPriceField_ClosePrice_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcIndexPriceField_swiginit(self, _thosttraderapi.new_CThostFtdcIndexPriceField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcIndexPriceField

# Register CThostFtdcIndexPriceField in _thosttraderapi:
_thosttraderapi.CThostFtdcIndexPriceField_swigregister(CThostFtdcIndexPriceField)

class CThostFtdcInputExecOrderField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thosttraderapi.CThostFtdcInputExecOrderField_BrokerID_get, _thosttraderapi.CThostFtdcInputExecOrderField_BrokerID_set)
    InvestorID = property(_thosttraderapi.CThostFtdcInputExecOrderField_InvestorID_get, _thosttraderapi.CThostFtdcInputExecOrderField_InvestorID_set)
    InstrumentID = property(_thosttraderapi.CThostFtdcInputExecOrderField_InstrumentID_get, _thosttraderapi.CThostFtdcInputExecOrderField_InstrumentID_set)
    ExecOrderRef = property(_thosttraderapi.CThostFtdcInputExecOrderField_ExecOrderRef_get, _thosttraderapi.CThostFtdcInputExecOrderField_ExecOrderRef_set)
    UserID = property(_thosttraderapi.CThostFtdcInputExecOrderField_UserID_get, _thosttraderapi.CThostFtdcInputExecOrderField_UserID_set)
    Volume = property(_thosttraderapi.CThostFtdcInputExecOrderField_Volume_get, _thosttraderapi.CThostFtdcInputExecOrderField_Volume_set)
    RequestID = property(_thosttraderapi.CThostFtdcInputExecOrderField_RequestID_get, _thosttraderapi.CThostFtdcInputExecOrderField_RequestID_set)
    BusinessUnit = property(_thosttraderapi.CThostFtdcInputExecOrderField_BusinessUnit_get, _thosttraderapi.CThostFtdcInputExecOrderField_BusinessUnit_set)
    OffsetFlag = property(_thosttraderapi.CThostFtdcInputExecOrderField_OffsetFlag_get, _thosttraderapi.CThostFtdcInputExecOrderField_OffsetFlag_set)
    HedgeFlag = property(_thosttraderapi.CThostFtdcInputExecOrderField_HedgeFlag_get, _thosttraderapi.CThostFtdcInputExecOrderField_HedgeFlag_set)
    ActionType = property(_thosttraderapi.CThostFtdcInputExecOrderField_ActionType_get, _thosttraderapi.CThostFtdcInputExecOrderField_ActionType_set)
    PosiDirection = property(_thosttraderapi.CThostFtdcInputExecOrderField_PosiDirection_get, _thosttraderapi.CThostFtdcInputExecOrderField_PosiDirection_set)
    ReservePositionFlag = property(_thosttraderapi.CThostFtdcInputExecOrderField_ReservePositionFlag_get, _thosttraderapi.CThostFtdcInputExecOrderField_ReservePositionFlag_set)
    CloseFlag = property(_thosttraderapi.CThostFtdcInputExecOrderField_CloseFlag_get, _thosttraderapi.CThostFtdcInputExecOrderField_CloseFlag_set)
    ExchangeID = property(_thosttraderapi.CThostFtdcInputExecOrderField_ExchangeID_get, _thosttraderapi.CThostFtdcInputExecOrderField_ExchangeID_set)
    InvestUnitID = property(_thosttraderapi.CThostFtdcInputExecOrderField_InvestUnitID_get, _thosttraderapi.CThostFtdcInputExecOrderField_InvestUnitID_set)
    AccountID = property(_thosttraderapi.CThostFtdcInputExecOrderField_AccountID_get, _thosttraderapi.CThostFtdcInputExecOrderField_AccountID_set)
    CurrencyID = property(_thosttraderapi.CThostFtdcInputExecOrderField_CurrencyID_get, _thosttraderapi.CThostFtdcInputExecOrderField_CurrencyID_set)
    ClientID = property(_thosttraderapi.CThostFtdcInputExecOrderField_ClientID_get, _thosttraderapi.CThostFtdcInputExecOrderField_ClientID_set)
    IPAddress = property(_thosttraderapi.CThostFtdcInputExecOrderField_IPAddress_get, _thosttraderapi.CThostFtdcInputExecOrderField_IPAddress_set)
    MacAddress = property(_thosttraderapi.CThostFtdcInputExecOrderField_MacAddress_get, _thosttraderapi.CThostFtdcInputExecOrderField_MacAddress_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcInputExecOrderField_swiginit(self, _thosttraderapi.new_CThostFtdcInputExecOrderField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcInputExecOrderField

# Register CThostFtdcInputExecOrderField in _thosttraderapi:
_thosttraderapi.CThostFtdcInputExecOrderField_swigregister(CThostFtdcInputExecOrderField)

class CThostFtdcInputExecOrderActionField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thosttraderapi.CThostFtdcInputExecOrderActionField_BrokerID_get, _thosttraderapi.CThostFtdcInputExecOrderActionField_BrokerID_set)
    InvestorID = property(_thosttraderapi.CThostFtdcInputExecOrderActionField_InvestorID_get, _thosttraderapi.CThostFtdcInputExecOrderActionField_InvestorID_set)
    ExecOrderActionRef = property(_thosttraderapi.CThostFtdcInputExecOrderActionField_ExecOrderActionRef_get, _thosttraderapi.CThostFtdcInputExecOrderActionField_ExecOrderActionRef_set)
    ExecOrderRef = property(_thosttraderapi.CThostFtdcInputExecOrderActionField_ExecOrderRef_get, _thosttraderapi.CThostFtdcInputExecOrderActionField_ExecOrderRef_set)
    RequestID = property(_thosttraderapi.CThostFtdcInputExecOrderActionField_RequestID_get, _thosttraderapi.CThostFtdcInputExecOrderActionField_RequestID_set)
    FrontID = property(_thosttraderapi.CThostFtdcInputExecOrderActionField_FrontID_get, _thosttraderapi.CThostFtdcInputExecOrderActionField_FrontID_set)
    SessionID = property(_thosttraderapi.CThostFtdcInputExecOrderActionField_SessionID_get, _thosttraderapi.CThostFtdcInputExecOrderActionField_SessionID_set)
    ExchangeID = property(_thosttraderapi.CThostFtdcInputExecOrderActionField_ExchangeID_get, _thosttraderapi.CThostFtdcInputExecOrderActionField_ExchangeID_set)
    ExecOrderSysID = property(_thosttraderapi.CThostFtdcInputExecOrderActionField_ExecOrderSysID_get, _thosttraderapi.CThostFtdcInputExecOrderActionField_ExecOrderSysID_set)
    ActionFlag = property(_thosttraderapi.CThostFtdcInputExecOrderActionField_ActionFlag_get, _thosttraderapi.CThostFtdcInputExecOrderActionField_ActionFlag_set)
    UserID = property(_thosttraderapi.CThostFtdcInputExecOrderActionField_UserID_get, _thosttraderapi.CThostFtdcInputExecOrderActionField_UserID_set)
    InstrumentID = property(_thosttraderapi.CThostFtdcInputExecOrderActionField_InstrumentID_get, _thosttraderapi.CThostFtdcInputExecOrderActionField_InstrumentID_set)
    InvestUnitID = property(_thosttraderapi.CThostFtdcInputExecOrderActionField_InvestUnitID_get, _thosttraderapi.CThostFtdcInputExecOrderActionField_InvestUnitID_set)
    IPAddress = property(_thosttraderapi.CThostFtdcInputExecOrderActionField_IPAddress_get, _thosttraderapi.CThostFtdcInputExecOrderActionField_IPAddress_set)
    MacAddress = property(_thosttraderapi.CThostFtdcInputExecOrderActionField_MacAddress_get, _thosttraderapi.CThostFtdcInputExecOrderActionField_MacAddress_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcInputExecOrderActionField_swiginit(self, _thosttraderapi.new_CThostFtdcInputExecOrderActionField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcInputExecOrderActionField

# Register CThostFtdcInputExecOrderActionField in _thosttraderapi:
_thosttraderapi.CThostFtdcInputExecOrderActionField_swigregister(CThostFtdcInputExecOrderActionField)

class CThostFtdcExecOrderField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thosttraderapi.CThostFtdcExecOrderField_BrokerID_get, _thosttraderapi.CThostFtdcExecOrderField_BrokerID_set)
    InvestorID = property(_thosttraderapi.CThostFtdcExecOrderField_InvestorID_get, _thosttraderapi.CThostFtdcExecOrderField_InvestorID_set)
    InstrumentID = property(_thosttraderapi.CThostFtdcExecOrderField_InstrumentID_get, _thosttraderapi.CThostFtdcExecOrderField_InstrumentID_set)
    ExecOrderRef = property(_thosttraderapi.CThostFtdcExecOrderField_ExecOrderRef_get, _thosttraderapi.CThostFtdcExecOrderField_ExecOrderRef_set)
    UserID = property(_thosttraderapi.CThostFtdcExecOrderField_UserID_get, _thosttraderapi.CThostFtdcExecOrderField_UserID_set)
    Volume = property(_thosttraderapi.CThostFtdcExecOrderField_Volume_get, _thosttraderapi.CThostFtdcExecOrderField_Volume_set)
    RequestID = property(_thosttraderapi.CThostFtdcExecOrderField_RequestID_get, _thosttraderapi.CThostFtdcExecOrderField_RequestID_set)
    BusinessUnit = property(_thosttraderapi.CThostFtdcExecOrderField_BusinessUnit_get, _thosttraderapi.CThostFtdcExecOrderField_BusinessUnit_set)
    OffsetFlag = property(_thosttraderapi.CThostFtdcExecOrderField_OffsetFlag_get, _thosttraderapi.CThostFtdcExecOrderField_OffsetFlag_set)
    HedgeFlag = property(_thosttraderapi.CThostFtdcExecOrderField_HedgeFlag_get, _thosttraderapi.CThostFtdcExecOrderField_HedgeFlag_set)
    ActionType = property(_thosttraderapi.CThostFtdcExecOrderField_ActionType_get, _thosttraderapi.CThostFtdcExecOrderField_ActionType_set)
    PosiDirection = property(_thosttraderapi.CThostFtdcExecOrderField_PosiDirection_get, _thosttraderapi.CThostFtdcExecOrderField_PosiDirection_set)
    ReservePositionFlag = property(_thosttraderapi.CThostFtdcExecOrderField_ReservePositionFlag_get, _thosttraderapi.CThostFtdcExecOrderField_ReservePositionFlag_set)
    CloseFlag = property(_thosttraderapi.CThostFtdcExecOrderField_CloseFlag_get, _thosttraderapi.CThostFtdcExecOrderField_CloseFlag_set)
    ExecOrderLocalID = property(_thosttraderapi.CThostFtdcExecOrderField_ExecOrderLocalID_get, _thosttraderapi.CThostFtdcExecOrderField_ExecOrderLocalID_set)
    ExchangeID = property(_thosttraderapi.CThostFtdcExecOrderField_ExchangeID_get, _thosttraderapi.CThostFtdcExecOrderField_ExchangeID_set)
    ParticipantID = property(_thosttraderapi.CThostFtdcExecOrderField_ParticipantID_get, _thosttraderapi.CThostFtdcExecOrderField_ParticipantID_set)
    ClientID = property(_thosttraderapi.CThostFtdcExecOrderField_ClientID_get, _thosttraderapi.CThostFtdcExecOrderField_ClientID_set)
    ExchangeInstID = property(_thosttraderapi.CThostFtdcExecOrderField_ExchangeInstID_get, _thosttraderapi.CThostFtdcExecOrderField_ExchangeInstID_set)
    TraderID = property(_thosttraderapi.CThostFtdcExecOrderField_TraderID_get, _thosttraderapi.CThostFtdcExecOrderField_TraderID_set)
    InstallID = property(_thosttraderapi.CThostFtdcExecOrderField_InstallID_get, _thosttraderapi.CThostFtdcExecOrderField_InstallID_set)
    OrderSubmitStatus = property(_thosttraderapi.CThostFtdcExecOrderField_OrderSubmitStatus_get, _thosttraderapi.CThostFtdcExecOrderField_OrderSubmitStatus_set)
    NotifySequence = property(_thosttraderapi.CThostFtdcExecOrderField_NotifySequence_get, _thosttraderapi.CThostFtdcExecOrderField_NotifySequence_set)
    TradingDay = property(_thosttraderapi.CThostFtdcExecOrderField_TradingDay_get, _thosttraderapi.CThostFtdcExecOrderField_TradingDay_set)
    SettlementID = property(_thosttraderapi.CThostFtdcExecOrderField_SettlementID_get, _thosttraderapi.CThostFtdcExecOrderField_SettlementID_set)
    ExecOrderSysID = property(_thosttraderapi.CThostFtdcExecOrderField_ExecOrderSysID_get, _thosttraderapi.CThostFtdcExecOrderField_ExecOrderSysID_set)
    InsertDate = property(_thosttraderapi.CThostFtdcExecOrderField_InsertDate_get, _thosttraderapi.CThostFtdcExecOrderField_InsertDate_set)
    InsertTime = property(_thosttraderapi.CThostFtdcExecOrderField_InsertTime_get, _thosttraderapi.CThostFtdcExecOrderField_InsertTime_set)
    CancelTime = property(_thosttraderapi.CThostFtdcExecOrderField_CancelTime_get, _thosttraderapi.CThostFtdcExecOrderField_CancelTime_set)
    ExecResult = property(_thosttraderapi.CThostFtdcExecOrderField_ExecResult_get, _thosttraderapi.CThostFtdcExecOrderField_ExecResult_set)
    ClearingPartID = property(_thosttraderapi.CThostFtdcExecOrderField_ClearingPartID_get, _thosttraderapi.CThostFtdcExecOrderField_ClearingPartID_set)
    SequenceNo = property(_thosttraderapi.CThostFtdcExecOrderField_SequenceNo_get, _thosttraderapi.CThostFtdcExecOrderField_SequenceNo_set)
    FrontID = property(_thosttraderapi.CThostFtdcExecOrderField_FrontID_get, _thosttraderapi.CThostFtdcExecOrderField_FrontID_set)
    SessionID = property(_thosttraderapi.CThostFtdcExecOrderField_SessionID_get, _thosttraderapi.CThostFtdcExecOrderField_SessionID_set)
    UserProductInfo = property(_thosttraderapi.CThostFtdcExecOrderField_UserProductInfo_get, _thosttraderapi.CThostFtdcExecOrderField_UserProductInfo_set)
    StatusMsg = property(_thosttraderapi.CThostFtdcExecOrderField_StatusMsg_get, _thosttraderapi.CThostFtdcExecOrderField_StatusMsg_set)
    ActiveUserID = property(_thosttraderapi.CThostFtdcExecOrderField_ActiveUserID_get, _thosttraderapi.CThostFtdcExecOrderField_ActiveUserID_set)
    BrokerExecOrderSeq = property(_thosttraderapi.CThostFtdcExecOrderField_BrokerExecOrderSeq_get, _thosttraderapi.CThostFtdcExecOrderField_BrokerExecOrderSeq_set)
    BranchID = property(_thosttraderapi.CThostFtdcExecOrderField_BranchID_get, _thosttraderapi.CThostFtdcExecOrderField_BranchID_set)
    InvestUnitID = property(_thosttraderapi.CThostFtdcExecOrderField_InvestUnitID_get, _thosttraderapi.CThostFtdcExecOrderField_InvestUnitID_set)
    AccountID = property(_thosttraderapi.CThostFtdcExecOrderField_AccountID_get, _thosttraderapi.CThostFtdcExecOrderField_AccountID_set)
    CurrencyID = property(_thosttraderapi.CThostFtdcExecOrderField_CurrencyID_get, _thosttraderapi.CThostFtdcExecOrderField_CurrencyID_set)
    IPAddress = property(_thosttraderapi.CThostFtdcExecOrderField_IPAddress_get, _thosttraderapi.CThostFtdcExecOrderField_IPAddress_set)
    MacAddress = property(_thosttraderapi.CThostFtdcExecOrderField_MacAddress_get, _thosttraderapi.CThostFtdcExecOrderField_MacAddress_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcExecOrderField_swiginit(self, _thosttraderapi.new_CThostFtdcExecOrderField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcExecOrderField

# Register CThostFtdcExecOrderField in _thosttraderapi:
_thosttraderapi.CThostFtdcExecOrderField_swigregister(CThostFtdcExecOrderField)

class CThostFtdcExecOrderActionField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thosttraderapi.CThostFtdcExecOrderActionField_BrokerID_get, _thosttraderapi.CThostFtdcExecOrderActionField_BrokerID_set)
    InvestorID = property(_thosttraderapi.CThostFtdcExecOrderActionField_InvestorID_get, _thosttraderapi.CThostFtdcExecOrderActionField_InvestorID_set)
    ExecOrderActionRef = property(_thosttraderapi.CThostFtdcExecOrderActionField_ExecOrderActionRef_get, _thosttraderapi.CThostFtdcExecOrderActionField_ExecOrderActionRef_set)
    ExecOrderRef = property(_thosttraderapi.CThostFtdcExecOrderActionField_ExecOrderRef_get, _thosttraderapi.CThostFtdcExecOrderActionField_ExecOrderRef_set)
    RequestID = property(_thosttraderapi.CThostFtdcExecOrderActionField_RequestID_get, _thosttraderapi.CThostFtdcExecOrderActionField_RequestID_set)
    FrontID = property(_thosttraderapi.CThostFtdcExecOrderActionField_FrontID_get, _thosttraderapi.CThostFtdcExecOrderActionField_FrontID_set)
    SessionID = property(_thosttraderapi.CThostFtdcExecOrderActionField_SessionID_get, _thosttraderapi.CThostFtdcExecOrderActionField_SessionID_set)
    ExchangeID = property(_thosttraderapi.CThostFtdcExecOrderActionField_ExchangeID_get, _thosttraderapi.CThostFtdcExecOrderActionField_ExchangeID_set)
    ExecOrderSysID = property(_thosttraderapi.CThostFtdcExecOrderActionField_ExecOrderSysID_get, _thosttraderapi.CThostFtdcExecOrderActionField_ExecOrderSysID_set)
    ActionFlag = property(_thosttraderapi.CThostFtdcExecOrderActionField_ActionFlag_get, _thosttraderapi.CThostFtdcExecOrderActionField_ActionFlag_set)
    ActionDate = property(_thosttraderapi.CThostFtdcExecOrderActionField_ActionDate_get, _thosttraderapi.CThostFtdcExecOrderActionField_ActionDate_set)
    ActionTime = property(_thosttraderapi.CThostFtdcExecOrderActionField_ActionTime_get, _thosttraderapi.CThostFtdcExecOrderActionField_ActionTime_set)
    TraderID = property(_thosttraderapi.CThostFtdcExecOrderActionField_TraderID_get, _thosttraderapi.CThostFtdcExecOrderActionField_TraderID_set)
    InstallID = property(_thosttraderapi.CThostFtdcExecOrderActionField_InstallID_get, _thosttraderapi.CThostFtdcExecOrderActionField_InstallID_set)
    ExecOrderLocalID = property(_thosttraderapi.CThostFtdcExecOrderActionField_ExecOrderLocalID_get, _thosttraderapi.CThostFtdcExecOrderActionField_ExecOrderLocalID_set)
    ActionLocalID = property(_thosttraderapi.CThostFtdcExecOrderActionField_ActionLocalID_get, _thosttraderapi.CThostFtdcExecOrderActionField_ActionLocalID_set)
    ParticipantID = property(_thosttraderapi.CThostFtdcExecOrderActionField_ParticipantID_get, _thosttraderapi.CThostFtdcExecOrderActionField_ParticipantID_set)
    ClientID = property(_thosttraderapi.CThostFtdcExecOrderActionField_ClientID_get, _thosttraderapi.CThostFtdcExecOrderActionField_ClientID_set)
    BusinessUnit = property(_thosttraderapi.CThostFtdcExecOrderActionField_BusinessUnit_get, _thosttraderapi.CThostFtdcExecOrderActionField_BusinessUnit_set)
    OrderActionStatus = property(_thosttraderapi.CThostFtdcExecOrderActionField_OrderActionStatus_get, _thosttraderapi.CThostFtdcExecOrderActionField_OrderActionStatus_set)
    UserID = property(_thosttraderapi.CThostFtdcExecOrderActionField_UserID_get, _thosttraderapi.CThostFtdcExecOrderActionField_UserID_set)
    ActionType = property(_thosttraderapi.CThostFtdcExecOrderActionField_ActionType_get, _thosttraderapi.CThostFtdcExecOrderActionField_ActionType_set)
    StatusMsg = property(_thosttraderapi.CThostFtdcExecOrderActionField_StatusMsg_get, _thosttraderapi.CThostFtdcExecOrderActionField_StatusMsg_set)
    InstrumentID = property(_thosttraderapi.CThostFtdcExecOrderActionField_InstrumentID_get, _thosttraderapi.CThostFtdcExecOrderActionField_InstrumentID_set)
    BranchID = property(_thosttraderapi.CThostFtdcExecOrderActionField_BranchID_get, _thosttraderapi.CThostFtdcExecOrderActionField_BranchID_set)
    InvestUnitID = property(_thosttraderapi.CThostFtdcExecOrderActionField_InvestUnitID_get, _thosttraderapi.CThostFtdcExecOrderActionField_InvestUnitID_set)
    IPAddress = property(_thosttraderapi.CThostFtdcExecOrderActionField_IPAddress_get, _thosttraderapi.CThostFtdcExecOrderActionField_IPAddress_set)
    MacAddress = property(_thosttraderapi.CThostFtdcExecOrderActionField_MacAddress_get, _thosttraderapi.CThostFtdcExecOrderActionField_MacAddress_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcExecOrderActionField_swiginit(self, _thosttraderapi.new_CThostFtdcExecOrderActionField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcExecOrderActionField

# Register CThostFtdcExecOrderActionField in _thosttraderapi:
_thosttraderapi.CThostFtdcExecOrderActionField_swigregister(CThostFtdcExecOrderActionField)

class CThostFtdcQryExecOrderField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thosttraderapi.CThostFtdcQryExecOrderField_BrokerID_get, _thosttraderapi.CThostFtdcQryExecOrderField_BrokerID_set)
    InvestorID = property(_thosttraderapi.CThostFtdcQryExecOrderField_InvestorID_get, _thosttraderapi.CThostFtdcQryExecOrderField_InvestorID_set)
    InstrumentID = property(_thosttraderapi.CThostFtdcQryExecOrderField_InstrumentID_get, _thosttraderapi.CThostFtdcQryExecOrderField_InstrumentID_set)
    ExchangeID = property(_thosttraderapi.CThostFtdcQryExecOrderField_ExchangeID_get, _thosttraderapi.CThostFtdcQryExecOrderField_ExchangeID_set)
    ExecOrderSysID = property(_thosttraderapi.CThostFtdcQryExecOrderField_ExecOrderSysID_get, _thosttraderapi.CThostFtdcQryExecOrderField_ExecOrderSysID_set)
    InsertTimeStart = property(_thosttraderapi.CThostFtdcQryExecOrderField_InsertTimeStart_get, _thosttraderapi.CThostFtdcQryExecOrderField_InsertTimeStart_set)
    InsertTimeEnd = property(_thosttraderapi.CThostFtdcQryExecOrderField_InsertTimeEnd_get, _thosttraderapi.CThostFtdcQryExecOrderField_InsertTimeEnd_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcQryExecOrderField_swiginit(self, _thosttraderapi.new_CThostFtdcQryExecOrderField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcQryExecOrderField

# Register CThostFtdcQryExecOrderField in _thosttraderapi:
_thosttraderapi.CThostFtdcQryExecOrderField_swigregister(CThostFtdcQryExecOrderField)

class CThostFtdcExchangeExecOrderField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    Volume = property(_thosttraderapi.CThostFtdcExchangeExecOrderField_Volume_get, _thosttraderapi.CThostFtdcExchangeExecOrderField_Volume_set)
    RequestID = property(_thosttraderapi.CThostFtdcExchangeExecOrderField_RequestID_get, _thosttraderapi.CThostFtdcExchangeExecOrderField_RequestID_set)
    BusinessUnit = property(_thosttraderapi.CThostFtdcExchangeExecOrderField_BusinessUnit_get, _thosttraderapi.CThostFtdcExchangeExecOrderField_BusinessUnit_set)
    OffsetFlag = property(_thosttraderapi.CThostFtdcExchangeExecOrderField_OffsetFlag_get, _thosttraderapi.CThostFtdcExchangeExecOrderField_OffsetFlag_set)
    HedgeFlag = property(_thosttraderapi.CThostFtdcExchangeExecOrderField_HedgeFlag_get, _thosttraderapi.CThostFtdcExchangeExecOrderField_HedgeFlag_set)
    ActionType = property(_thosttraderapi.CThostFtdcExchangeExecOrderField_ActionType_get, _thosttraderapi.CThostFtdcExchangeExecOrderField_ActionType_set)
    PosiDirection = property(_thosttraderapi.CThostFtdcExchangeExecOrderField_PosiDirection_get, _thosttraderapi.CThostFtdcExchangeExecOrderField_PosiDirection_set)
    ReservePositionFlag = property(_thosttraderapi.CThostFtdcExchangeExecOrderField_ReservePositionFlag_get, _thosttraderapi.CThostFtdcExchangeExecOrderField_ReservePositionFlag_set)
    CloseFlag = property(_thosttraderapi.CThostFtdcExchangeExecOrderField_CloseFlag_get, _thosttraderapi.CThostFtdcExchangeExecOrderField_CloseFlag_set)
    ExecOrderLocalID = property(_thosttraderapi.CThostFtdcExchangeExecOrderField_ExecOrderLocalID_get, _thosttraderapi.CThostFtdcExchangeExecOrderField_ExecOrderLocalID_set)
    ExchangeID = property(_thosttraderapi.CThostFtdcExchangeExecOrderField_ExchangeID_get, _thosttraderapi.CThostFtdcExchangeExecOrderField_ExchangeID_set)
    ParticipantID = property(_thosttraderapi.CThostFtdcExchangeExecOrderField_ParticipantID_get, _thosttraderapi.CThostFtdcExchangeExecOrderField_ParticipantID_set)
    ClientID = property(_thosttraderapi.CThostFtdcExchangeExecOrderField_ClientID_get, _thosttraderapi.CThostFtdcExchangeExecOrderField_ClientID_set)
    ExchangeInstID = property(_thosttraderapi.CThostFtdcExchangeExecOrderField_ExchangeInstID_get, _thosttraderapi.CThostFtdcExchangeExecOrderField_ExchangeInstID_set)
    TraderID = property(_thosttraderapi.CThostFtdcExchangeExecOrderField_TraderID_get, _thosttraderapi.CThostFtdcExchangeExecOrderField_TraderID_set)
    InstallID = property(_thosttraderapi.CThostFtdcExchangeExecOrderField_InstallID_get, _thosttraderapi.CThostFtdcExchangeExecOrderField_InstallID_set)
    OrderSubmitStatus = property(_thosttraderapi.CThostFtdcExchangeExecOrderField_OrderSubmitStatus_get, _thosttraderapi.CThostFtdcExchangeExecOrderField_OrderSubmitStatus_set)
    NotifySequence = property(_thosttraderapi.CThostFtdcExchangeExecOrderField_NotifySequence_get, _thosttraderapi.CThostFtdcExchangeExecOrderField_NotifySequence_set)
    TradingDay = property(_thosttraderapi.CThostFtdcExchangeExecOrderField_TradingDay_get, _thosttraderapi.CThostFtdcExchangeExecOrderField_TradingDay_set)
    SettlementID = property(_thosttraderapi.CThostFtdcExchangeExecOrderField_SettlementID_get, _thosttraderapi.CThostFtdcExchangeExecOrderField_SettlementID_set)
    ExecOrderSysID = property(_thosttraderapi.CThostFtdcExchangeExecOrderField_ExecOrderSysID_get, _thosttraderapi.CThostFtdcExchangeExecOrderField_ExecOrderSysID_set)
    InsertDate = property(_thosttraderapi.CThostFtdcExchangeExecOrderField_InsertDate_get, _thosttraderapi.CThostFtdcExchangeExecOrderField_InsertDate_set)
    InsertTime = property(_thosttraderapi.CThostFtdcExchangeExecOrderField_InsertTime_get, _thosttraderapi.CThostFtdcExchangeExecOrderField_InsertTime_set)
    CancelTime = property(_thosttraderapi.CThostFtdcExchangeExecOrderField_CancelTime_get, _thosttraderapi.CThostFtdcExchangeExecOrderField_CancelTime_set)
    ExecResult = property(_thosttraderapi.CThostFtdcExchangeExecOrderField_ExecResult_get, _thosttraderapi.CThostFtdcExchangeExecOrderField_ExecResult_set)
    ClearingPartID = property(_thosttraderapi.CThostFtdcExchangeExecOrderField_ClearingPartID_get, _thosttraderapi.CThostFtdcExchangeExecOrderField_ClearingPartID_set)
    SequenceNo = property(_thosttraderapi.CThostFtdcExchangeExecOrderField_SequenceNo_get, _thosttraderapi.CThostFtdcExchangeExecOrderField_SequenceNo_set)
    BranchID = property(_thosttraderapi.CThostFtdcExchangeExecOrderField_BranchID_get, _thosttraderapi.CThostFtdcExchangeExecOrderField_BranchID_set)
    IPAddress = property(_thosttraderapi.CThostFtdcExchangeExecOrderField_IPAddress_get, _thosttraderapi.CThostFtdcExchangeExecOrderField_IPAddress_set)
    MacAddress = property(_thosttraderapi.CThostFtdcExchangeExecOrderField_MacAddress_get, _thosttraderapi.CThostFtdcExchangeExecOrderField_MacAddress_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcExchangeExecOrderField_swiginit(self, _thosttraderapi.new_CThostFtdcExchangeExecOrderField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcExchangeExecOrderField

# Register CThostFtdcExchangeExecOrderField in _thosttraderapi:
_thosttraderapi.CThostFtdcExchangeExecOrderField_swigregister(CThostFtdcExchangeExecOrderField)

class CThostFtdcQryExchangeExecOrderField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    ParticipantID = property(_thosttraderapi.CThostFtdcQryExchangeExecOrderField_ParticipantID_get, _thosttraderapi.CThostFtdcQryExchangeExecOrderField_ParticipantID_set)
    ClientID = property(_thosttraderapi.CThostFtdcQryExchangeExecOrderField_ClientID_get, _thosttraderapi.CThostFtdcQryExchangeExecOrderField_ClientID_set)
    ExchangeInstID = property(_thosttraderapi.CThostFtdcQryExchangeExecOrderField_ExchangeInstID_get, _thosttraderapi.CThostFtdcQryExchangeExecOrderField_ExchangeInstID_set)
    ExchangeID = property(_thosttraderapi.CThostFtdcQryExchangeExecOrderField_ExchangeID_get, _thosttraderapi.CThostFtdcQryExchangeExecOrderField_ExchangeID_set)
    TraderID = property(_thosttraderapi.CThostFtdcQryExchangeExecOrderField_TraderID_get, _thosttraderapi.CThostFtdcQryExchangeExecOrderField_TraderID_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcQryExchangeExecOrderField_swiginit(self, _thosttraderapi.new_CThostFtdcQryExchangeExecOrderField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcQryExchangeExecOrderField

# Register CThostFtdcQryExchangeExecOrderField in _thosttraderapi:
_thosttraderapi.CThostFtdcQryExchangeExecOrderField_swigregister(CThostFtdcQryExchangeExecOrderField)

class CThostFtdcQryExecOrderActionField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thosttraderapi.CThostFtdcQryExecOrderActionField_BrokerID_get, _thosttraderapi.CThostFtdcQryExecOrderActionField_BrokerID_set)
    InvestorID = property(_thosttraderapi.CThostFtdcQryExecOrderActionField_InvestorID_get, _thosttraderapi.CThostFtdcQryExecOrderActionField_InvestorID_set)
    ExchangeID = property(_thosttraderapi.CThostFtdcQryExecOrderActionField_ExchangeID_get, _thosttraderapi.CThostFtdcQryExecOrderActionField_ExchangeID_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcQryExecOrderActionField_swiginit(self, _thosttraderapi.new_CThostFtdcQryExecOrderActionField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcQryExecOrderActionField

# Register CThostFtdcQryExecOrderActionField in _thosttraderapi:
_thosttraderapi.CThostFtdcQryExecOrderActionField_swigregister(CThostFtdcQryExecOrderActionField)

class CThostFtdcExchangeExecOrderActionField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    ExchangeID = property(_thosttraderapi.CThostFtdcExchangeExecOrderActionField_ExchangeID_get, _thosttraderapi.CThostFtdcExchangeExecOrderActionField_ExchangeID_set)
    ExecOrderSysID = property(_thosttraderapi.CThostFtdcExchangeExecOrderActionField_ExecOrderSysID_get, _thosttraderapi.CThostFtdcExchangeExecOrderActionField_ExecOrderSysID_set)
    ActionFlag = property(_thosttraderapi.CThostFtdcExchangeExecOrderActionField_ActionFlag_get, _thosttraderapi.CThostFtdcExchangeExecOrderActionField_ActionFlag_set)
    ActionDate = property(_thosttraderapi.CThostFtdcExchangeExecOrderActionField_ActionDate_get, _thosttraderapi.CThostFtdcExchangeExecOrderActionField_ActionDate_set)
    ActionTime = property(_thosttraderapi.CThostFtdcExchangeExecOrderActionField_ActionTime_get, _thosttraderapi.CThostFtdcExchangeExecOrderActionField_ActionTime_set)
    TraderID = property(_thosttraderapi.CThostFtdcExchangeExecOrderActionField_TraderID_get, _thosttraderapi.CThostFtdcExchangeExecOrderActionField_TraderID_set)
    InstallID = property(_thosttraderapi.CThostFtdcExchangeExecOrderActionField_InstallID_get, _thosttraderapi.CThostFtdcExchangeExecOrderActionField_InstallID_set)
    ExecOrderLocalID = property(_thosttraderapi.CThostFtdcExchangeExecOrderActionField_ExecOrderLocalID_get, _thosttraderapi.CThostFtdcExchangeExecOrderActionField_ExecOrderLocalID_set)
    ActionLocalID = property(_thosttraderapi.CThostFtdcExchangeExecOrderActionField_ActionLocalID_get, _thosttraderapi.CThostFtdcExchangeExecOrderActionField_ActionLocalID_set)
    ParticipantID = property(_thosttraderapi.CThostFtdcExchangeExecOrderActionField_ParticipantID_get, _thosttraderapi.CThostFtdcExchangeExecOrderActionField_ParticipantID_set)
    ClientID = property(_thosttraderapi.CThostFtdcExchangeExecOrderActionField_ClientID_get, _thosttraderapi.CThostFtdcExchangeExecOrderActionField_ClientID_set)
    BusinessUnit = property(_thosttraderapi.CThostFtdcExchangeExecOrderActionField_BusinessUnit_get, _thosttraderapi.CThostFtdcExchangeExecOrderActionField_BusinessUnit_set)
    OrderActionStatus = property(_thosttraderapi.CThostFtdcExchangeExecOrderActionField_OrderActionStatus_get, _thosttraderapi.CThostFtdcExchangeExecOrderActionField_OrderActionStatus_set)
    UserID = property(_thosttraderapi.CThostFtdcExchangeExecOrderActionField_UserID_get, _thosttraderapi.CThostFtdcExchangeExecOrderActionField_UserID_set)
    ActionType = property(_thosttraderapi.CThostFtdcExchangeExecOrderActionField_ActionType_get, _thosttraderapi.CThostFtdcExchangeExecOrderActionField_ActionType_set)
    BranchID = property(_thosttraderapi.CThostFtdcExchangeExecOrderActionField_BranchID_get, _thosttraderapi.CThostFtdcExchangeExecOrderActionField_BranchID_set)
    IPAddress = property(_thosttraderapi.CThostFtdcExchangeExecOrderActionField_IPAddress_get, _thosttraderapi.CThostFtdcExchangeExecOrderActionField_IPAddress_set)
    MacAddress = property(_thosttraderapi.CThostFtdcExchangeExecOrderActionField_MacAddress_get, _thosttraderapi.CThostFtdcExchangeExecOrderActionField_MacAddress_set)
    ExchangeInstID = property(_thosttraderapi.CThostFtdcExchangeExecOrderActionField_ExchangeInstID_get, _thosttraderapi.CThostFtdcExchangeExecOrderActionField_ExchangeInstID_set)
    Volume = property(_thosttraderapi.CThostFtdcExchangeExecOrderActionField_Volume_get, _thosttraderapi.CThostFtdcExchangeExecOrderActionField_Volume_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcExchangeExecOrderActionField_swiginit(self, _thosttraderapi.new_CThostFtdcExchangeExecOrderActionField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcExchangeExecOrderActionField

# Register CThostFtdcExchangeExecOrderActionField in _thosttraderapi:
_thosttraderapi.CThostFtdcExchangeExecOrderActionField_swigregister(CThostFtdcExchangeExecOrderActionField)

class CThostFtdcQryExchangeExecOrderActionField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    ParticipantID = property(_thosttraderapi.CThostFtdcQryExchangeExecOrderActionField_ParticipantID_get, _thosttraderapi.CThostFtdcQryExchangeExecOrderActionField_ParticipantID_set)
    ClientID = property(_thosttraderapi.CThostFtdcQryExchangeExecOrderActionField_ClientID_get, _thosttraderapi.CThostFtdcQryExchangeExecOrderActionField_ClientID_set)
    ExchangeID = property(_thosttraderapi.CThostFtdcQryExchangeExecOrderActionField_ExchangeID_get, _thosttraderapi.CThostFtdcQryExchangeExecOrderActionField_ExchangeID_set)
    TraderID = property(_thosttraderapi.CThostFtdcQryExchangeExecOrderActionField_TraderID_get, _thosttraderapi.CThostFtdcQryExchangeExecOrderActionField_TraderID_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcQryExchangeExecOrderActionField_swiginit(self, _thosttraderapi.new_CThostFtdcQryExchangeExecOrderActionField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcQryExchangeExecOrderActionField

# Register CThostFtdcQryExchangeExecOrderActionField in _thosttraderapi:
_thosttraderapi.CThostFtdcQryExchangeExecOrderActionField_swigregister(CThostFtdcQryExchangeExecOrderActionField)

class CThostFtdcErrExecOrderField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thosttraderapi.CThostFtdcErrExecOrderField_BrokerID_get, _thosttraderapi.CThostFtdcErrExecOrderField_BrokerID_set)
    InvestorID = property(_thosttraderapi.CThostFtdcErrExecOrderField_InvestorID_get, _thosttraderapi.CThostFtdcErrExecOrderField_InvestorID_set)
    InstrumentID = property(_thosttraderapi.CThostFtdcErrExecOrderField_InstrumentID_get, _thosttraderapi.CThostFtdcErrExecOrderField_InstrumentID_set)
    ExecOrderRef = property(_thosttraderapi.CThostFtdcErrExecOrderField_ExecOrderRef_get, _thosttraderapi.CThostFtdcErrExecOrderField_ExecOrderRef_set)
    UserID = property(_thosttraderapi.CThostFtdcErrExecOrderField_UserID_get, _thosttraderapi.CThostFtdcErrExecOrderField_UserID_set)
    Volume = property(_thosttraderapi.CThostFtdcErrExecOrderField_Volume_get, _thosttraderapi.CThostFtdcErrExecOrderField_Volume_set)
    RequestID = property(_thosttraderapi.CThostFtdcErrExecOrderField_RequestID_get, _thosttraderapi.CThostFtdcErrExecOrderField_RequestID_set)
    BusinessUnit = property(_thosttraderapi.CThostFtdcErrExecOrderField_BusinessUnit_get, _thosttraderapi.CThostFtdcErrExecOrderField_BusinessUnit_set)
    OffsetFlag = property(_thosttraderapi.CThostFtdcErrExecOrderField_OffsetFlag_get, _thosttraderapi.CThostFtdcErrExecOrderField_OffsetFlag_set)
    HedgeFlag = property(_thosttraderapi.CThostFtdcErrExecOrderField_HedgeFlag_get, _thosttraderapi.CThostFtdcErrExecOrderField_HedgeFlag_set)
    ActionType = property(_thosttraderapi.CThostFtdcErrExecOrderField_ActionType_get, _thosttraderapi.CThostFtdcErrExecOrderField_ActionType_set)
    PosiDirection = property(_thosttraderapi.CThostFtdcErrExecOrderField_PosiDirection_get, _thosttraderapi.CThostFtdcErrExecOrderField_PosiDirection_set)
    ReservePositionFlag = property(_thosttraderapi.CThostFtdcErrExecOrderField_ReservePositionFlag_get, _thosttraderapi.CThostFtdcErrExecOrderField_ReservePositionFlag_set)
    CloseFlag = property(_thosttraderapi.CThostFtdcErrExecOrderField_CloseFlag_get, _thosttraderapi.CThostFtdcErrExecOrderField_CloseFlag_set)
    ExchangeID = property(_thosttraderapi.CThostFtdcErrExecOrderField_ExchangeID_get, _thosttraderapi.CThostFtdcErrExecOrderField_ExchangeID_set)
    InvestUnitID = property(_thosttraderapi.CThostFtdcErrExecOrderField_InvestUnitID_get, _thosttraderapi.CThostFtdcErrExecOrderField_InvestUnitID_set)
    AccountID = property(_thosttraderapi.CThostFtdcErrExecOrderField_AccountID_get, _thosttraderapi.CThostFtdcErrExecOrderField_AccountID_set)
    CurrencyID = property(_thosttraderapi.CThostFtdcErrExecOrderField_CurrencyID_get, _thosttraderapi.CThostFtdcErrExecOrderField_CurrencyID_set)
    ClientID = property(_thosttraderapi.CThostFtdcErrExecOrderField_ClientID_get, _thosttraderapi.CThostFtdcErrExecOrderField_ClientID_set)
    IPAddress = property(_thosttraderapi.CThostFtdcErrExecOrderField_IPAddress_get, _thosttraderapi.CThostFtdcErrExecOrderField_IPAddress_set)
    MacAddress = property(_thosttraderapi.CThostFtdcErrExecOrderField_MacAddress_get, _thosttraderapi.CThostFtdcErrExecOrderField_MacAddress_set)
    ErrorID = property(_thosttraderapi.CThostFtdcErrExecOrderField_ErrorID_get, _thosttraderapi.CThostFtdcErrExecOrderField_ErrorID_set)
    ErrorMsg = property(_thosttraderapi.CThostFtdcErrExecOrderField_ErrorMsg_get, _thosttraderapi.CThostFtdcErrExecOrderField_ErrorMsg_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcErrExecOrderField_swiginit(self, _thosttraderapi.new_CThostFtdcErrExecOrderField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcErrExecOrderField

# Register CThostFtdcErrExecOrderField in _thosttraderapi:
_thosttraderapi.CThostFtdcErrExecOrderField_swigregister(CThostFtdcErrExecOrderField)

class CThostFtdcQryErrExecOrderField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thosttraderapi.CThostFtdcQryErrExecOrderField_BrokerID_get, _thosttraderapi.CThostFtdcQryErrExecOrderField_BrokerID_set)
    InvestorID = property(_thosttraderapi.CThostFtdcQryErrExecOrderField_InvestorID_get, _thosttraderapi.CThostFtdcQryErrExecOrderField_InvestorID_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcQryErrExecOrderField_swiginit(self, _thosttraderapi.new_CThostFtdcQryErrExecOrderField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcQryErrExecOrderField

# Register CThostFtdcQryErrExecOrderField in _thosttraderapi:
_thosttraderapi.CThostFtdcQryErrExecOrderField_swigregister(CThostFtdcQryErrExecOrderField)

class CThostFtdcErrExecOrderActionField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thosttraderapi.CThostFtdcErrExecOrderActionField_BrokerID_get, _thosttraderapi.CThostFtdcErrExecOrderActionField_BrokerID_set)
    InvestorID = property(_thosttraderapi.CThostFtdcErrExecOrderActionField_InvestorID_get, _thosttraderapi.CThostFtdcErrExecOrderActionField_InvestorID_set)
    ExecOrderActionRef = property(_thosttraderapi.CThostFtdcErrExecOrderActionField_ExecOrderActionRef_get, _thosttraderapi.CThostFtdcErrExecOrderActionField_ExecOrderActionRef_set)
    ExecOrderRef = property(_thosttraderapi.CThostFtdcErrExecOrderActionField_ExecOrderRef_get, _thosttraderapi.CThostFtdcErrExecOrderActionField_ExecOrderRef_set)
    RequestID = property(_thosttraderapi.CThostFtdcErrExecOrderActionField_RequestID_get, _thosttraderapi.CThostFtdcErrExecOrderActionField_RequestID_set)
    FrontID = property(_thosttraderapi.CThostFtdcErrExecOrderActionField_FrontID_get, _thosttraderapi.CThostFtdcErrExecOrderActionField_FrontID_set)
    SessionID = property(_thosttraderapi.CThostFtdcErrExecOrderActionField_SessionID_get, _thosttraderapi.CThostFtdcErrExecOrderActionField_SessionID_set)
    ExchangeID = property(_thosttraderapi.CThostFtdcErrExecOrderActionField_ExchangeID_get, _thosttraderapi.CThostFtdcErrExecOrderActionField_ExchangeID_set)
    ExecOrderSysID = property(_thosttraderapi.CThostFtdcErrExecOrderActionField_ExecOrderSysID_get, _thosttraderapi.CThostFtdcErrExecOrderActionField_ExecOrderSysID_set)
    ActionFlag = property(_thosttraderapi.CThostFtdcErrExecOrderActionField_ActionFlag_get, _thosttraderapi.CThostFtdcErrExecOrderActionField_ActionFlag_set)
    UserID = property(_thosttraderapi.CThostFtdcErrExecOrderActionField_UserID_get, _thosttraderapi.CThostFtdcErrExecOrderActionField_UserID_set)
    InstrumentID = property(_thosttraderapi.CThostFtdcErrExecOrderActionField_InstrumentID_get, _thosttraderapi.CThostFtdcErrExecOrderActionField_InstrumentID_set)
    InvestUnitID = property(_thosttraderapi.CThostFtdcErrExecOrderActionField_InvestUnitID_get, _thosttraderapi.CThostFtdcErrExecOrderActionField_InvestUnitID_set)
    IPAddress = property(_thosttraderapi.CThostFtdcErrExecOrderActionField_IPAddress_get, _thosttraderapi.CThostFtdcErrExecOrderActionField_IPAddress_set)
    MacAddress = property(_thosttraderapi.CThostFtdcErrExecOrderActionField_MacAddress_get, _thosttraderapi.CThostFtdcErrExecOrderActionField_MacAddress_set)
    ErrorID = property(_thosttraderapi.CThostFtdcErrExecOrderActionField_ErrorID_get, _thosttraderapi.CThostFtdcErrExecOrderActionField_ErrorID_set)
    ErrorMsg = property(_thosttraderapi.CThostFtdcErrExecOrderActionField_ErrorMsg_get, _thosttraderapi.CThostFtdcErrExecOrderActionField_ErrorMsg_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcErrExecOrderActionField_swiginit(self, _thosttraderapi.new_CThostFtdcErrExecOrderActionField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcErrExecOrderActionField

# Register CThostFtdcErrExecOrderActionField in _thosttraderapi:
_thosttraderapi.CThostFtdcErrExecOrderActionField_swigregister(CThostFtdcErrExecOrderActionField)

class CThostFtdcQryErrExecOrderActionField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thosttraderapi.CThostFtdcQryErrExecOrderActionField_BrokerID_get, _thosttraderapi.CThostFtdcQryErrExecOrderActionField_BrokerID_set)
    InvestorID = property(_thosttraderapi.CThostFtdcQryErrExecOrderActionField_InvestorID_get, _thosttraderapi.CThostFtdcQryErrExecOrderActionField_InvestorID_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcQryErrExecOrderActionField_swiginit(self, _thosttraderapi.new_CThostFtdcQryErrExecOrderActionField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcQryErrExecOrderActionField

# Register CThostFtdcQryErrExecOrderActionField in _thosttraderapi:
_thosttraderapi.CThostFtdcQryErrExecOrderActionField_swigregister(CThostFtdcQryErrExecOrderActionField)

class CThostFtdcOptionInstrTradingRightField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    InstrumentID = property(_thosttraderapi.CThostFtdcOptionInstrTradingRightField_InstrumentID_get, _thosttraderapi.CThostFtdcOptionInstrTradingRightField_InstrumentID_set)
    InvestorRange = property(_thosttraderapi.CThostFtdcOptionInstrTradingRightField_InvestorRange_get, _thosttraderapi.CThostFtdcOptionInstrTradingRightField_InvestorRange_set)
    BrokerID = property(_thosttraderapi.CThostFtdcOptionInstrTradingRightField_BrokerID_get, _thosttraderapi.CThostFtdcOptionInstrTradingRightField_BrokerID_set)
    InvestorID = property(_thosttraderapi.CThostFtdcOptionInstrTradingRightField_InvestorID_get, _thosttraderapi.CThostFtdcOptionInstrTradingRightField_InvestorID_set)
    Direction = property(_thosttraderapi.CThostFtdcOptionInstrTradingRightField_Direction_get, _thosttraderapi.CThostFtdcOptionInstrTradingRightField_Direction_set)
    TradingRight = property(_thosttraderapi.CThostFtdcOptionInstrTradingRightField_TradingRight_get, _thosttraderapi.CThostFtdcOptionInstrTradingRightField_TradingRight_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcOptionInstrTradingRightField_swiginit(self, _thosttraderapi.new_CThostFtdcOptionInstrTradingRightField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcOptionInstrTradingRightField

# Register CThostFtdcOptionInstrTradingRightField in _thosttraderapi:
_thosttraderapi.CThostFtdcOptionInstrTradingRightField_swigregister(CThostFtdcOptionInstrTradingRightField)

class CThostFtdcQryOptionInstrTradingRightField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thosttraderapi.CThostFtdcQryOptionInstrTradingRightField_BrokerID_get, _thosttraderapi.CThostFtdcQryOptionInstrTradingRightField_BrokerID_set)
    InvestorID = property(_thosttraderapi.CThostFtdcQryOptionInstrTradingRightField_InvestorID_get, _thosttraderapi.CThostFtdcQryOptionInstrTradingRightField_InvestorID_set)
    InstrumentID = property(_thosttraderapi.CThostFtdcQryOptionInstrTradingRightField_InstrumentID_get, _thosttraderapi.CThostFtdcQryOptionInstrTradingRightField_InstrumentID_set)
    Direction = property(_thosttraderapi.CThostFtdcQryOptionInstrTradingRightField_Direction_get, _thosttraderapi.CThostFtdcQryOptionInstrTradingRightField_Direction_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcQryOptionInstrTradingRightField_swiginit(self, _thosttraderapi.new_CThostFtdcQryOptionInstrTradingRightField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcQryOptionInstrTradingRightField

# Register CThostFtdcQryOptionInstrTradingRightField in _thosttraderapi:
_thosttraderapi.CThostFtdcQryOptionInstrTradingRightField_swigregister(CThostFtdcQryOptionInstrTradingRightField)

class CThostFtdcInputForQuoteField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thosttraderapi.CThostFtdcInputForQuoteField_BrokerID_get, _thosttraderapi.CThostFtdcInputForQuoteField_BrokerID_set)
    InvestorID = property(_thosttraderapi.CThostFtdcInputForQuoteField_InvestorID_get, _thosttraderapi.CThostFtdcInputForQuoteField_InvestorID_set)
    InstrumentID = property(_thosttraderapi.CThostFtdcInputForQuoteField_InstrumentID_get, _thosttraderapi.CThostFtdcInputForQuoteField_InstrumentID_set)
    ForQuoteRef = property(_thosttraderapi.CThostFtdcInputForQuoteField_ForQuoteRef_get, _thosttraderapi.CThostFtdcInputForQuoteField_ForQuoteRef_set)
    UserID = property(_thosttraderapi.CThostFtdcInputForQuoteField_UserID_get, _thosttraderapi.CThostFtdcInputForQuoteField_UserID_set)
    ExchangeID = property(_thosttraderapi.CThostFtdcInputForQuoteField_ExchangeID_get, _thosttraderapi.CThostFtdcInputForQuoteField_ExchangeID_set)
    InvestUnitID = property(_thosttraderapi.CThostFtdcInputForQuoteField_InvestUnitID_get, _thosttraderapi.CThostFtdcInputForQuoteField_InvestUnitID_set)
    IPAddress = property(_thosttraderapi.CThostFtdcInputForQuoteField_IPAddress_get, _thosttraderapi.CThostFtdcInputForQuoteField_IPAddress_set)
    MacAddress = property(_thosttraderapi.CThostFtdcInputForQuoteField_MacAddress_get, _thosttraderapi.CThostFtdcInputForQuoteField_MacAddress_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcInputForQuoteField_swiginit(self, _thosttraderapi.new_CThostFtdcInputForQuoteField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcInputForQuoteField

# Register CThostFtdcInputForQuoteField in _thosttraderapi:
_thosttraderapi.CThostFtdcInputForQuoteField_swigregister(CThostFtdcInputForQuoteField)

class CThostFtdcForQuoteField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thosttraderapi.CThostFtdcForQuoteField_BrokerID_get, _thosttraderapi.CThostFtdcForQuoteField_BrokerID_set)
    InvestorID = property(_thosttraderapi.CThostFtdcForQuoteField_InvestorID_get, _thosttraderapi.CThostFtdcForQuoteField_InvestorID_set)
    InstrumentID = property(_thosttraderapi.CThostFtdcForQuoteField_InstrumentID_get, _thosttraderapi.CThostFtdcForQuoteField_InstrumentID_set)
    ForQuoteRef = property(_thosttraderapi.CThostFtdcForQuoteField_ForQuoteRef_get, _thosttraderapi.CThostFtdcForQuoteField_ForQuoteRef_set)
    UserID = property(_thosttraderapi.CThostFtdcForQuoteField_UserID_get, _thosttraderapi.CThostFtdcForQuoteField_UserID_set)
    ForQuoteLocalID = property(_thosttraderapi.CThostFtdcForQuoteField_ForQuoteLocalID_get, _thosttraderapi.CThostFtdcForQuoteField_ForQuoteLocalID_set)
    ExchangeID = property(_thosttraderapi.CThostFtdcForQuoteField_ExchangeID_get, _thosttraderapi.CThostFtdcForQuoteField_ExchangeID_set)
    ParticipantID = property(_thosttraderapi.CThostFtdcForQuoteField_ParticipantID_get, _thosttraderapi.CThostFtdcForQuoteField_ParticipantID_set)
    ClientID = property(_thosttraderapi.CThostFtdcForQuoteField_ClientID_get, _thosttraderapi.CThostFtdcForQuoteField_ClientID_set)
    ExchangeInstID = property(_thosttraderapi.CThostFtdcForQuoteField_ExchangeInstID_get, _thosttraderapi.CThostFtdcForQuoteField_ExchangeInstID_set)
    TraderID = property(_thosttraderapi.CThostFtdcForQuoteField_TraderID_get, _thosttraderapi.CThostFtdcForQuoteField_TraderID_set)
    InstallID = property(_thosttraderapi.CThostFtdcForQuoteField_InstallID_get, _thosttraderapi.CThostFtdcForQuoteField_InstallID_set)
    InsertDate = property(_thosttraderapi.CThostFtdcForQuoteField_InsertDate_get, _thosttraderapi.CThostFtdcForQuoteField_InsertDate_set)
    InsertTime = property(_thosttraderapi.CThostFtdcForQuoteField_InsertTime_get, _thosttraderapi.CThostFtdcForQuoteField_InsertTime_set)
    ForQuoteStatus = property(_thosttraderapi.CThostFtdcForQuoteField_ForQuoteStatus_get, _thosttraderapi.CThostFtdcForQuoteField_ForQuoteStatus_set)
    FrontID = property(_thosttraderapi.CThostFtdcForQuoteField_FrontID_get, _thosttraderapi.CThostFtdcForQuoteField_FrontID_set)
    SessionID = property(_thosttraderapi.CThostFtdcForQuoteField_SessionID_get, _thosttraderapi.CThostFtdcForQuoteField_SessionID_set)
    StatusMsg = property(_thosttraderapi.CThostFtdcForQuoteField_StatusMsg_get, _thosttraderapi.CThostFtdcForQuoteField_StatusMsg_set)
    ActiveUserID = property(_thosttraderapi.CThostFtdcForQuoteField_ActiveUserID_get, _thosttraderapi.CThostFtdcForQuoteField_ActiveUserID_set)
    BrokerForQutoSeq = property(_thosttraderapi.CThostFtdcForQuoteField_BrokerForQutoSeq_get, _thosttraderapi.CThostFtdcForQuoteField_BrokerForQutoSeq_set)
    InvestUnitID = property(_thosttraderapi.CThostFtdcForQuoteField_InvestUnitID_get, _thosttraderapi.CThostFtdcForQuoteField_InvestUnitID_set)
    IPAddress = property(_thosttraderapi.CThostFtdcForQuoteField_IPAddress_get, _thosttraderapi.CThostFtdcForQuoteField_IPAddress_set)
    MacAddress = property(_thosttraderapi.CThostFtdcForQuoteField_MacAddress_get, _thosttraderapi.CThostFtdcForQuoteField_MacAddress_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcForQuoteField_swiginit(self, _thosttraderapi.new_CThostFtdcForQuoteField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcForQuoteField

# Register CThostFtdcForQuoteField in _thosttraderapi:
_thosttraderapi.CThostFtdcForQuoteField_swigregister(CThostFtdcForQuoteField)

class CThostFtdcQryForQuoteField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thosttraderapi.CThostFtdcQryForQuoteField_BrokerID_get, _thosttraderapi.CThostFtdcQryForQuoteField_BrokerID_set)
    InvestorID = property(_thosttraderapi.CThostFtdcQryForQuoteField_InvestorID_get, _thosttraderapi.CThostFtdcQryForQuoteField_InvestorID_set)
    InstrumentID = property(_thosttraderapi.CThostFtdcQryForQuoteField_InstrumentID_get, _thosttraderapi.CThostFtdcQryForQuoteField_InstrumentID_set)
    ExchangeID = property(_thosttraderapi.CThostFtdcQryForQuoteField_ExchangeID_get, _thosttraderapi.CThostFtdcQryForQuoteField_ExchangeID_set)
    InsertTimeStart = property(_thosttraderapi.CThostFtdcQryForQuoteField_InsertTimeStart_get, _thosttraderapi.CThostFtdcQryForQuoteField_InsertTimeStart_set)
    InsertTimeEnd = property(_thosttraderapi.CThostFtdcQryForQuoteField_InsertTimeEnd_get, _thosttraderapi.CThostFtdcQryForQuoteField_InsertTimeEnd_set)
    InvestUnitID = property(_thosttraderapi.CThostFtdcQryForQuoteField_InvestUnitID_get, _thosttraderapi.CThostFtdcQryForQuoteField_InvestUnitID_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcQryForQuoteField_swiginit(self, _thosttraderapi.new_CThostFtdcQryForQuoteField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcQryForQuoteField

# Register CThostFtdcQryForQuoteField in _thosttraderapi:
_thosttraderapi.CThostFtdcQryForQuoteField_swigregister(CThostFtdcQryForQuoteField)

class CThostFtdcExchangeForQuoteField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    ForQuoteLocalID = property(_thosttraderapi.CThostFtdcExchangeForQuoteField_ForQuoteLocalID_get, _thosttraderapi.CThostFtdcExchangeForQuoteField_ForQuoteLocalID_set)
    ExchangeID = property(_thosttraderapi.CThostFtdcExchangeForQuoteField_ExchangeID_get, _thosttraderapi.CThostFtdcExchangeForQuoteField_ExchangeID_set)
    ParticipantID = property(_thosttraderapi.CThostFtdcExchangeForQuoteField_ParticipantID_get, _thosttraderapi.CThostFtdcExchangeForQuoteField_ParticipantID_set)
    ClientID = property(_thosttraderapi.CThostFtdcExchangeForQuoteField_ClientID_get, _thosttraderapi.CThostFtdcExchangeForQuoteField_ClientID_set)
    ExchangeInstID = property(_thosttraderapi.CThostFtdcExchangeForQuoteField_ExchangeInstID_get, _thosttraderapi.CThostFtdcExchangeForQuoteField_ExchangeInstID_set)
    TraderID = property(_thosttraderapi.CThostFtdcExchangeForQuoteField_TraderID_get, _thosttraderapi.CThostFtdcExchangeForQuoteField_TraderID_set)
    InstallID = property(_thosttraderapi.CThostFtdcExchangeForQuoteField_InstallID_get, _thosttraderapi.CThostFtdcExchangeForQuoteField_InstallID_set)
    InsertDate = property(_thosttraderapi.CThostFtdcExchangeForQuoteField_InsertDate_get, _thosttraderapi.CThostFtdcExchangeForQuoteField_InsertDate_set)
    InsertTime = property(_thosttraderapi.CThostFtdcExchangeForQuoteField_InsertTime_get, _thosttraderapi.CThostFtdcExchangeForQuoteField_InsertTime_set)
    ForQuoteStatus = property(_thosttraderapi.CThostFtdcExchangeForQuoteField_ForQuoteStatus_get, _thosttraderapi.CThostFtdcExchangeForQuoteField_ForQuoteStatus_set)
    IPAddress = property(_thosttraderapi.CThostFtdcExchangeForQuoteField_IPAddress_get, _thosttraderapi.CThostFtdcExchangeForQuoteField_IPAddress_set)
    MacAddress = property(_thosttraderapi.CThostFtdcExchangeForQuoteField_MacAddress_get, _thosttraderapi.CThostFtdcExchangeForQuoteField_MacAddress_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcExchangeForQuoteField_swiginit(self, _thosttraderapi.new_CThostFtdcExchangeForQuoteField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcExchangeForQuoteField

# Register CThostFtdcExchangeForQuoteField in _thosttraderapi:
_thosttraderapi.CThostFtdcExchangeForQuoteField_swigregister(CThostFtdcExchangeForQuoteField)

class CThostFtdcQryExchangeForQuoteField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    ParticipantID = property(_thosttraderapi.CThostFtdcQryExchangeForQuoteField_ParticipantID_get, _thosttraderapi.CThostFtdcQryExchangeForQuoteField_ParticipantID_set)
    ClientID = property(_thosttraderapi.CThostFtdcQryExchangeForQuoteField_ClientID_get, _thosttraderapi.CThostFtdcQryExchangeForQuoteField_ClientID_set)
    ExchangeInstID = property(_thosttraderapi.CThostFtdcQryExchangeForQuoteField_ExchangeInstID_get, _thosttraderapi.CThostFtdcQryExchangeForQuoteField_ExchangeInstID_set)
    ExchangeID = property(_thosttraderapi.CThostFtdcQryExchangeForQuoteField_ExchangeID_get, _thosttraderapi.CThostFtdcQryExchangeForQuoteField_ExchangeID_set)
    TraderID = property(_thosttraderapi.CThostFtdcQryExchangeForQuoteField_TraderID_get, _thosttraderapi.CThostFtdcQryExchangeForQuoteField_TraderID_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcQryExchangeForQuoteField_swiginit(self, _thosttraderapi.new_CThostFtdcQryExchangeForQuoteField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcQryExchangeForQuoteField

# Register CThostFtdcQryExchangeForQuoteField in _thosttraderapi:
_thosttraderapi.CThostFtdcQryExchangeForQuoteField_swigregister(CThostFtdcQryExchangeForQuoteField)

class CThostFtdcInputQuoteField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thosttraderapi.CThostFtdcInputQuoteField_BrokerID_get, _thosttraderapi.CThostFtdcInputQuoteField_BrokerID_set)
    InvestorID = property(_thosttraderapi.CThostFtdcInputQuoteField_InvestorID_get, _thosttraderapi.CThostFtdcInputQuoteField_InvestorID_set)
    InstrumentID = property(_thosttraderapi.CThostFtdcInputQuoteField_InstrumentID_get, _thosttraderapi.CThostFtdcInputQuoteField_InstrumentID_set)
    QuoteRef = property(_thosttraderapi.CThostFtdcInputQuoteField_QuoteRef_get, _thosttraderapi.CThostFtdcInputQuoteField_QuoteRef_set)
    UserID = property(_thosttraderapi.CThostFtdcInputQuoteField_UserID_get, _thosttraderapi.CThostFtdcInputQuoteField_UserID_set)
    AskPrice = property(_thosttraderapi.CThostFtdcInputQuoteField_AskPrice_get, _thosttraderapi.CThostFtdcInputQuoteField_AskPrice_set)
    BidPrice = property(_thosttraderapi.CThostFtdcInputQuoteField_BidPrice_get, _thosttraderapi.CThostFtdcInputQuoteField_BidPrice_set)
    AskVolume = property(_thosttraderapi.CThostFtdcInputQuoteField_AskVolume_get, _thosttraderapi.CThostFtdcInputQuoteField_AskVolume_set)
    BidVolume = property(_thosttraderapi.CThostFtdcInputQuoteField_BidVolume_get, _thosttraderapi.CThostFtdcInputQuoteField_BidVolume_set)
    RequestID = property(_thosttraderapi.CThostFtdcInputQuoteField_RequestID_get, _thosttraderapi.CThostFtdcInputQuoteField_RequestID_set)
    BusinessUnit = property(_thosttraderapi.CThostFtdcInputQuoteField_BusinessUnit_get, _thosttraderapi.CThostFtdcInputQuoteField_BusinessUnit_set)
    AskOffsetFlag = property(_thosttraderapi.CThostFtdcInputQuoteField_AskOffsetFlag_get, _thosttraderapi.CThostFtdcInputQuoteField_AskOffsetFlag_set)
    BidOffsetFlag = property(_thosttraderapi.CThostFtdcInputQuoteField_BidOffsetFlag_get, _thosttraderapi.CThostFtdcInputQuoteField_BidOffsetFlag_set)
    AskHedgeFlag = property(_thosttraderapi.CThostFtdcInputQuoteField_AskHedgeFlag_get, _thosttraderapi.CThostFtdcInputQuoteField_AskHedgeFlag_set)
    BidHedgeFlag = property(_thosttraderapi.CThostFtdcInputQuoteField_BidHedgeFlag_get, _thosttraderapi.CThostFtdcInputQuoteField_BidHedgeFlag_set)
    AskOrderRef = property(_thosttraderapi.CThostFtdcInputQuoteField_AskOrderRef_get, _thosttraderapi.CThostFtdcInputQuoteField_AskOrderRef_set)
    BidOrderRef = property(_thosttraderapi.CThostFtdcInputQuoteField_BidOrderRef_get, _thosttraderapi.CThostFtdcInputQuoteField_BidOrderRef_set)
    ForQuoteSysID = property(_thosttraderapi.CThostFtdcInputQuoteField_ForQuoteSysID_get, _thosttraderapi.CThostFtdcInputQuoteField_ForQuoteSysID_set)
    ExchangeID = property(_thosttraderapi.CThostFtdcInputQuoteField_ExchangeID_get, _thosttraderapi.CThostFtdcInputQuoteField_ExchangeID_set)
    InvestUnitID = property(_thosttraderapi.CThostFtdcInputQuoteField_InvestUnitID_get, _thosttraderapi.CThostFtdcInputQuoteField_InvestUnitID_set)
    ClientID = property(_thosttraderapi.CThostFtdcInputQuoteField_ClientID_get, _thosttraderapi.CThostFtdcInputQuoteField_ClientID_set)
    IPAddress = property(_thosttraderapi.CThostFtdcInputQuoteField_IPAddress_get, _thosttraderapi.CThostFtdcInputQuoteField_IPAddress_set)
    MacAddress = property(_thosttraderapi.CThostFtdcInputQuoteField_MacAddress_get, _thosttraderapi.CThostFtdcInputQuoteField_MacAddress_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcInputQuoteField_swiginit(self, _thosttraderapi.new_CThostFtdcInputQuoteField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcInputQuoteField

# Register CThostFtdcInputQuoteField in _thosttraderapi:
_thosttraderapi.CThostFtdcInputQuoteField_swigregister(CThostFtdcInputQuoteField)

class CThostFtdcInputQuoteActionField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thosttraderapi.CThostFtdcInputQuoteActionField_BrokerID_get, _thosttraderapi.CThostFtdcInputQuoteActionField_BrokerID_set)
    InvestorID = property(_thosttraderapi.CThostFtdcInputQuoteActionField_InvestorID_get, _thosttraderapi.CThostFtdcInputQuoteActionField_InvestorID_set)
    QuoteActionRef = property(_thosttraderapi.CThostFtdcInputQuoteActionField_QuoteActionRef_get, _thosttraderapi.CThostFtdcInputQuoteActionField_QuoteActionRef_set)
    QuoteRef = property(_thosttraderapi.CThostFtdcInputQuoteActionField_QuoteRef_get, _thosttraderapi.CThostFtdcInputQuoteActionField_QuoteRef_set)
    RequestID = property(_thosttraderapi.CThostFtdcInputQuoteActionField_RequestID_get, _thosttraderapi.CThostFtdcInputQuoteActionField_RequestID_set)
    FrontID = property(_thosttraderapi.CThostFtdcInputQuoteActionField_FrontID_get, _thosttraderapi.CThostFtdcInputQuoteActionField_FrontID_set)
    SessionID = property(_thosttraderapi.CThostFtdcInputQuoteActionField_SessionID_get, _thosttraderapi.CThostFtdcInputQuoteActionField_SessionID_set)
    ExchangeID = property(_thosttraderapi.CThostFtdcInputQuoteActionField_ExchangeID_get, _thosttraderapi.CThostFtdcInputQuoteActionField_ExchangeID_set)
    QuoteSysID = property(_thosttraderapi.CThostFtdcInputQuoteActionField_QuoteSysID_get, _thosttraderapi.CThostFtdcInputQuoteActionField_QuoteSysID_set)
    ActionFlag = property(_thosttraderapi.CThostFtdcInputQuoteActionField_ActionFlag_get, _thosttraderapi.CThostFtdcInputQuoteActionField_ActionFlag_set)
    UserID = property(_thosttraderapi.CThostFtdcInputQuoteActionField_UserID_get, _thosttraderapi.CThostFtdcInputQuoteActionField_UserID_set)
    InstrumentID = property(_thosttraderapi.CThostFtdcInputQuoteActionField_InstrumentID_get, _thosttraderapi.CThostFtdcInputQuoteActionField_InstrumentID_set)
    InvestUnitID = property(_thosttraderapi.CThostFtdcInputQuoteActionField_InvestUnitID_get, _thosttraderapi.CThostFtdcInputQuoteActionField_InvestUnitID_set)
    ClientID = property(_thosttraderapi.CThostFtdcInputQuoteActionField_ClientID_get, _thosttraderapi.CThostFtdcInputQuoteActionField_ClientID_set)
    IPAddress = property(_thosttraderapi.CThostFtdcInputQuoteActionField_IPAddress_get, _thosttraderapi.CThostFtdcInputQuoteActionField_IPAddress_set)
    MacAddress = property(_thosttraderapi.CThostFtdcInputQuoteActionField_MacAddress_get, _thosttraderapi.CThostFtdcInputQuoteActionField_MacAddress_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcInputQuoteActionField_swiginit(self, _thosttraderapi.new_CThostFtdcInputQuoteActionField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcInputQuoteActionField

# Register CThostFtdcInputQuoteActionField in _thosttraderapi:
_thosttraderapi.CThostFtdcInputQuoteActionField_swigregister(CThostFtdcInputQuoteActionField)

class CThostFtdcQuoteField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thosttraderapi.CThostFtdcQuoteField_BrokerID_get, _thosttraderapi.CThostFtdcQuoteField_BrokerID_set)
    InvestorID = property(_thosttraderapi.CThostFtdcQuoteField_InvestorID_get, _thosttraderapi.CThostFtdcQuoteField_InvestorID_set)
    InstrumentID = property(_thosttraderapi.CThostFtdcQuoteField_InstrumentID_get, _thosttraderapi.CThostFtdcQuoteField_InstrumentID_set)
    QuoteRef = property(_thosttraderapi.CThostFtdcQuoteField_QuoteRef_get, _thosttraderapi.CThostFtdcQuoteField_QuoteRef_set)
    UserID = property(_thosttraderapi.CThostFtdcQuoteField_UserID_get, _thosttraderapi.CThostFtdcQuoteField_UserID_set)
    AskPrice = property(_thosttraderapi.CThostFtdcQuoteField_AskPrice_get, _thosttraderapi.CThostFtdcQuoteField_AskPrice_set)
    BidPrice = property(_thosttraderapi.CThostFtdcQuoteField_BidPrice_get, _thosttraderapi.CThostFtdcQuoteField_BidPrice_set)
    AskVolume = property(_thosttraderapi.CThostFtdcQuoteField_AskVolume_get, _thosttraderapi.CThostFtdcQuoteField_AskVolume_set)
    BidVolume = property(_thosttraderapi.CThostFtdcQuoteField_BidVolume_get, _thosttraderapi.CThostFtdcQuoteField_BidVolume_set)
    RequestID = property(_thosttraderapi.CThostFtdcQuoteField_RequestID_get, _thosttraderapi.CThostFtdcQuoteField_RequestID_set)
    BusinessUnit = property(_thosttraderapi.CThostFtdcQuoteField_BusinessUnit_get, _thosttraderapi.CThostFtdcQuoteField_BusinessUnit_set)
    AskOffsetFlag = property(_thosttraderapi.CThostFtdcQuoteField_AskOffsetFlag_get, _thosttraderapi.CThostFtdcQuoteField_AskOffsetFlag_set)
    BidOffsetFlag = property(_thosttraderapi.CThostFtdcQuoteField_BidOffsetFlag_get, _thosttraderapi.CThostFtdcQuoteField_BidOffsetFlag_set)
    AskHedgeFlag = property(_thosttraderapi.CThostFtdcQuoteField_AskHedgeFlag_get, _thosttraderapi.CThostFtdcQuoteField_AskHedgeFlag_set)
    BidHedgeFlag = property(_thosttraderapi.CThostFtdcQuoteField_BidHedgeFlag_get, _thosttraderapi.CThostFtdcQuoteField_BidHedgeFlag_set)
    QuoteLocalID = property(_thosttraderapi.CThostFtdcQuoteField_QuoteLocalID_get, _thosttraderapi.CThostFtdcQuoteField_QuoteLocalID_set)
    ExchangeID = property(_thosttraderapi.CThostFtdcQuoteField_ExchangeID_get, _thosttraderapi.CThostFtdcQuoteField_ExchangeID_set)
    ParticipantID = property(_thosttraderapi.CThostFtdcQuoteField_ParticipantID_get, _thosttraderapi.CThostFtdcQuoteField_ParticipantID_set)
    ClientID = property(_thosttraderapi.CThostFtdcQuoteField_ClientID_get, _thosttraderapi.CThostFtdcQuoteField_ClientID_set)
    ExchangeInstID = property(_thosttraderapi.CThostFtdcQuoteField_ExchangeInstID_get, _thosttraderapi.CThostFtdcQuoteField_ExchangeInstID_set)
    TraderID = property(_thosttraderapi.CThostFtdcQuoteField_TraderID_get, _thosttraderapi.CThostFtdcQuoteField_TraderID_set)
    InstallID = property(_thosttraderapi.CThostFtdcQuoteField_InstallID_get, _thosttraderapi.CThostFtdcQuoteField_InstallID_set)
    NotifySequence = property(_thosttraderapi.CThostFtdcQuoteField_NotifySequence_get, _thosttraderapi.CThostFtdcQuoteField_NotifySequence_set)
    OrderSubmitStatus = property(_thosttraderapi.CThostFtdcQuoteField_OrderSubmitStatus_get, _thosttraderapi.CThostFtdcQuoteField_OrderSubmitStatus_set)
    TradingDay = property(_thosttraderapi.CThostFtdcQuoteField_TradingDay_get, _thosttraderapi.CThostFtdcQuoteField_TradingDay_set)
    SettlementID = property(_thosttraderapi.CThostFtdcQuoteField_SettlementID_get, _thosttraderapi.CThostFtdcQuoteField_SettlementID_set)
    QuoteSysID = property(_thosttraderapi.CThostFtdcQuoteField_QuoteSysID_get, _thosttraderapi.CThostFtdcQuoteField_QuoteSysID_set)
    InsertDate = property(_thosttraderapi.CThostFtdcQuoteField_InsertDate_get, _thosttraderapi.CThostFtdcQuoteField_InsertDate_set)
    InsertTime = property(_thosttraderapi.CThostFtdcQuoteField_InsertTime_get, _thosttraderapi.CThostFtdcQuoteField_InsertTime_set)
    CancelTime = property(_thosttraderapi.CThostFtdcQuoteField_CancelTime_get, _thosttraderapi.CThostFtdcQuoteField_CancelTime_set)
    QuoteStatus = property(_thosttraderapi.CThostFtdcQuoteField_QuoteStatus_get, _thosttraderapi.CThostFtdcQuoteField_QuoteStatus_set)
    ClearingPartID = property(_thosttraderapi.CThostFtdcQuoteField_ClearingPartID_get, _thosttraderapi.CThostFtdcQuoteField_ClearingPartID_set)
    SequenceNo = property(_thosttraderapi.CThostFtdcQuoteField_SequenceNo_get, _thosttraderapi.CThostFtdcQuoteField_SequenceNo_set)
    AskOrderSysID = property(_thosttraderapi.CThostFtdcQuoteField_AskOrderSysID_get, _thosttraderapi.CThostFtdcQuoteField_AskOrderSysID_set)
    BidOrderSysID = property(_thosttraderapi.CThostFtdcQuoteField_BidOrderSysID_get, _thosttraderapi.CThostFtdcQuoteField_BidOrderSysID_set)
    FrontID = property(_thosttraderapi.CThostFtdcQuoteField_FrontID_get, _thosttraderapi.CThostFtdcQuoteField_FrontID_set)
    SessionID = property(_thosttraderapi.CThostFtdcQuoteField_SessionID_get, _thosttraderapi.CThostFtdcQuoteField_SessionID_set)
    UserProductInfo = property(_thosttraderapi.CThostFtdcQuoteField_UserProductInfo_get, _thosttraderapi.CThostFtdcQuoteField_UserProductInfo_set)
    StatusMsg = property(_thosttraderapi.CThostFtdcQuoteField_StatusMsg_get, _thosttraderapi.CThostFtdcQuoteField_StatusMsg_set)
    ActiveUserID = property(_thosttraderapi.CThostFtdcQuoteField_ActiveUserID_get, _thosttraderapi.CThostFtdcQuoteField_ActiveUserID_set)
    BrokerQuoteSeq = property(_thosttraderapi.CThostFtdcQuoteField_BrokerQuoteSeq_get, _thosttraderapi.CThostFtdcQuoteField_BrokerQuoteSeq_set)
    AskOrderRef = property(_thosttraderapi.CThostFtdcQuoteField_AskOrderRef_get, _thosttraderapi.CThostFtdcQuoteField_AskOrderRef_set)
    BidOrderRef = property(_thosttraderapi.CThostFtdcQuoteField_BidOrderRef_get, _thosttraderapi.CThostFtdcQuoteField_BidOrderRef_set)
    ForQuoteSysID = property(_thosttraderapi.CThostFtdcQuoteField_ForQuoteSysID_get, _thosttraderapi.CThostFtdcQuoteField_ForQuoteSysID_set)
    BranchID = property(_thosttraderapi.CThostFtdcQuoteField_BranchID_get, _thosttraderapi.CThostFtdcQuoteField_BranchID_set)
    InvestUnitID = property(_thosttraderapi.CThostFtdcQuoteField_InvestUnitID_get, _thosttraderapi.CThostFtdcQuoteField_InvestUnitID_set)
    AccountID = property(_thosttraderapi.CThostFtdcQuoteField_AccountID_get, _thosttraderapi.CThostFtdcQuoteField_AccountID_set)
    CurrencyID = property(_thosttraderapi.CThostFtdcQuoteField_CurrencyID_get, _thosttraderapi.CThostFtdcQuoteField_CurrencyID_set)
    IPAddress = property(_thosttraderapi.CThostFtdcQuoteField_IPAddress_get, _thosttraderapi.CThostFtdcQuoteField_IPAddress_set)
    MacAddress = property(_thosttraderapi.CThostFtdcQuoteField_MacAddress_get, _thosttraderapi.CThostFtdcQuoteField_MacAddress_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcQuoteField_swiginit(self, _thosttraderapi.new_CThostFtdcQuoteField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcQuoteField

# Register CThostFtdcQuoteField in _thosttraderapi:
_thosttraderapi.CThostFtdcQuoteField_swigregister(CThostFtdcQuoteField)

class CThostFtdcQuoteActionField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thosttraderapi.CThostFtdcQuoteActionField_BrokerID_get, _thosttraderapi.CThostFtdcQuoteActionField_BrokerID_set)
    InvestorID = property(_thosttraderapi.CThostFtdcQuoteActionField_InvestorID_get, _thosttraderapi.CThostFtdcQuoteActionField_InvestorID_set)
    QuoteActionRef = property(_thosttraderapi.CThostFtdcQuoteActionField_QuoteActionRef_get, _thosttraderapi.CThostFtdcQuoteActionField_QuoteActionRef_set)
    QuoteRef = property(_thosttraderapi.CThostFtdcQuoteActionField_QuoteRef_get, _thosttraderapi.CThostFtdcQuoteActionField_QuoteRef_set)
    RequestID = property(_thosttraderapi.CThostFtdcQuoteActionField_RequestID_get, _thosttraderapi.CThostFtdcQuoteActionField_RequestID_set)
    FrontID = property(_thosttraderapi.CThostFtdcQuoteActionField_FrontID_get, _thosttraderapi.CThostFtdcQuoteActionField_FrontID_set)
    SessionID = property(_thosttraderapi.CThostFtdcQuoteActionField_SessionID_get, _thosttraderapi.CThostFtdcQuoteActionField_SessionID_set)
    ExchangeID = property(_thosttraderapi.CThostFtdcQuoteActionField_ExchangeID_get, _thosttraderapi.CThostFtdcQuoteActionField_ExchangeID_set)
    QuoteSysID = property(_thosttraderapi.CThostFtdcQuoteActionField_QuoteSysID_get, _thosttraderapi.CThostFtdcQuoteActionField_QuoteSysID_set)
    ActionFlag = property(_thosttraderapi.CThostFtdcQuoteActionField_ActionFlag_get, _thosttraderapi.CThostFtdcQuoteActionField_ActionFlag_set)
    ActionDate = property(_thosttraderapi.CThostFtdcQuoteActionField_ActionDate_get, _thosttraderapi.CThostFtdcQuoteActionField_ActionDate_set)
    ActionTime = property(_thosttraderapi.CThostFtdcQuoteActionField_ActionTime_get, _thosttraderapi.CThostFtdcQuoteActionField_ActionTime_set)
    TraderID = property(_thosttraderapi.CThostFtdcQuoteActionField_TraderID_get, _thosttraderapi.CThostFtdcQuoteActionField_TraderID_set)
    InstallID = property(_thosttraderapi.CThostFtdcQuoteActionField_InstallID_get, _thosttraderapi.CThostFtdcQuoteActionField_InstallID_set)
    QuoteLocalID = property(_thosttraderapi.CThostFtdcQuoteActionField_QuoteLocalID_get, _thosttraderapi.CThostFtdcQuoteActionField_QuoteLocalID_set)
    ActionLocalID = property(_thosttraderapi.CThostFtdcQuoteActionField_ActionLocalID_get, _thosttraderapi.CThostFtdcQuoteActionField_ActionLocalID_set)
    ParticipantID = property(_thosttraderapi.CThostFtdcQuoteActionField_ParticipantID_get, _thosttraderapi.CThostFtdcQuoteActionField_ParticipantID_set)
    ClientID = property(_thosttraderapi.CThostFtdcQuoteActionField_ClientID_get, _thosttraderapi.CThostFtdcQuoteActionField_ClientID_set)
    BusinessUnit = property(_thosttraderapi.CThostFtdcQuoteActionField_BusinessUnit_get, _thosttraderapi.CThostFtdcQuoteActionField_BusinessUnit_set)
    OrderActionStatus = property(_thosttraderapi.CThostFtdcQuoteActionField_OrderActionStatus_get, _thosttraderapi.CThostFtdcQuoteActionField_OrderActionStatus_set)
    UserID = property(_thosttraderapi.CThostFtdcQuoteActionField_UserID_get, _thosttraderapi.CThostFtdcQuoteActionField_UserID_set)
    StatusMsg = property(_thosttraderapi.CThostFtdcQuoteActionField_StatusMsg_get, _thosttraderapi.CThostFtdcQuoteActionField_StatusMsg_set)
    InstrumentID = property(_thosttraderapi.CThostFtdcQuoteActionField_InstrumentID_get, _thosttraderapi.CThostFtdcQuoteActionField_InstrumentID_set)
    BranchID = property(_thosttraderapi.CThostFtdcQuoteActionField_BranchID_get, _thosttraderapi.CThostFtdcQuoteActionField_BranchID_set)
    InvestUnitID = property(_thosttraderapi.CThostFtdcQuoteActionField_InvestUnitID_get, _thosttraderapi.CThostFtdcQuoteActionField_InvestUnitID_set)
    IPAddress = property(_thosttraderapi.CThostFtdcQuoteActionField_IPAddress_get, _thosttraderapi.CThostFtdcQuoteActionField_IPAddress_set)
    MacAddress = property(_thosttraderapi.CThostFtdcQuoteActionField_MacAddress_get, _thosttraderapi.CThostFtdcQuoteActionField_MacAddress_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcQuoteActionField_swiginit(self, _thosttraderapi.new_CThostFtdcQuoteActionField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcQuoteActionField

# Register CThostFtdcQuoteActionField in _thosttraderapi:
_thosttraderapi.CThostFtdcQuoteActionField_swigregister(CThostFtdcQuoteActionField)

class CThostFtdcQryQuoteField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thosttraderapi.CThostFtdcQryQuoteField_BrokerID_get, _thosttraderapi.CThostFtdcQryQuoteField_BrokerID_set)
    InvestorID = property(_thosttraderapi.CThostFtdcQryQuoteField_InvestorID_get, _thosttraderapi.CThostFtdcQryQuoteField_InvestorID_set)
    InstrumentID = property(_thosttraderapi.CThostFtdcQryQuoteField_InstrumentID_get, _thosttraderapi.CThostFtdcQryQuoteField_InstrumentID_set)
    ExchangeID = property(_thosttraderapi.CThostFtdcQryQuoteField_ExchangeID_get, _thosttraderapi.CThostFtdcQryQuoteField_ExchangeID_set)
    QuoteSysID = property(_thosttraderapi.CThostFtdcQryQuoteField_QuoteSysID_get, _thosttraderapi.CThostFtdcQryQuoteField_QuoteSysID_set)
    InsertTimeStart = property(_thosttraderapi.CThostFtdcQryQuoteField_InsertTimeStart_get, _thosttraderapi.CThostFtdcQryQuoteField_InsertTimeStart_set)
    InsertTimeEnd = property(_thosttraderapi.CThostFtdcQryQuoteField_InsertTimeEnd_get, _thosttraderapi.CThostFtdcQryQuoteField_InsertTimeEnd_set)
    InvestUnitID = property(_thosttraderapi.CThostFtdcQryQuoteField_InvestUnitID_get, _thosttraderapi.CThostFtdcQryQuoteField_InvestUnitID_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcQryQuoteField_swiginit(self, _thosttraderapi.new_CThostFtdcQryQuoteField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcQryQuoteField

# Register CThostFtdcQryQuoteField in _thosttraderapi:
_thosttraderapi.CThostFtdcQryQuoteField_swigregister(CThostFtdcQryQuoteField)

class CThostFtdcExchangeQuoteField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    AskPrice = property(_thosttraderapi.CThostFtdcExchangeQuoteField_AskPrice_get, _thosttraderapi.CThostFtdcExchangeQuoteField_AskPrice_set)
    BidPrice = property(_thosttraderapi.CThostFtdcExchangeQuoteField_BidPrice_get, _thosttraderapi.CThostFtdcExchangeQuoteField_BidPrice_set)
    AskVolume = property(_thosttraderapi.CThostFtdcExchangeQuoteField_AskVolume_get, _thosttraderapi.CThostFtdcExchangeQuoteField_AskVolume_set)
    BidVolume = property(_thosttraderapi.CThostFtdcExchangeQuoteField_BidVolume_get, _thosttraderapi.CThostFtdcExchangeQuoteField_BidVolume_set)
    RequestID = property(_thosttraderapi.CThostFtdcExchangeQuoteField_RequestID_get, _thosttraderapi.CThostFtdcExchangeQuoteField_RequestID_set)
    BusinessUnit = property(_thosttraderapi.CThostFtdcExchangeQuoteField_BusinessUnit_get, _thosttraderapi.CThostFtdcExchangeQuoteField_BusinessUnit_set)
    AskOffsetFlag = property(_thosttraderapi.CThostFtdcExchangeQuoteField_AskOffsetFlag_get, _thosttraderapi.CThostFtdcExchangeQuoteField_AskOffsetFlag_set)
    BidOffsetFlag = property(_thosttraderapi.CThostFtdcExchangeQuoteField_BidOffsetFlag_get, _thosttraderapi.CThostFtdcExchangeQuoteField_BidOffsetFlag_set)
    AskHedgeFlag = property(_thosttraderapi.CThostFtdcExchangeQuoteField_AskHedgeFlag_get, _thosttraderapi.CThostFtdcExchangeQuoteField_AskHedgeFlag_set)
    BidHedgeFlag = property(_thosttraderapi.CThostFtdcExchangeQuoteField_BidHedgeFlag_get, _thosttraderapi.CThostFtdcExchangeQuoteField_BidHedgeFlag_set)
    QuoteLocalID = property(_thosttraderapi.CThostFtdcExchangeQuoteField_QuoteLocalID_get, _thosttraderapi.CThostFtdcExchangeQuoteField_QuoteLocalID_set)
    ExchangeID = property(_thosttraderapi.CThostFtdcExchangeQuoteField_ExchangeID_get, _thosttraderapi.CThostFtdcExchangeQuoteField_ExchangeID_set)
    ParticipantID = property(_thosttraderapi.CThostFtdcExchangeQuoteField_ParticipantID_get, _thosttraderapi.CThostFtdcExchangeQuoteField_ParticipantID_set)
    ClientID = property(_thosttraderapi.CThostFtdcExchangeQuoteField_ClientID_get, _thosttraderapi.CThostFtdcExchangeQuoteField_ClientID_set)
    ExchangeInstID = property(_thosttraderapi.CThostFtdcExchangeQuoteField_ExchangeInstID_get, _thosttraderapi.CThostFtdcExchangeQuoteField_ExchangeInstID_set)
    TraderID = property(_thosttraderapi.CThostFtdcExchangeQuoteField_TraderID_get, _thosttraderapi.CThostFtdcExchangeQuoteField_TraderID_set)
    InstallID = property(_thosttraderapi.CThostFtdcExchangeQuoteField_InstallID_get, _thosttraderapi.CThostFtdcExchangeQuoteField_InstallID_set)
    NotifySequence = property(_thosttraderapi.CThostFtdcExchangeQuoteField_NotifySequence_get, _thosttraderapi.CThostFtdcExchangeQuoteField_NotifySequence_set)
    OrderSubmitStatus = property(_thosttraderapi.CThostFtdcExchangeQuoteField_OrderSubmitStatus_get, _thosttraderapi.CThostFtdcExchangeQuoteField_OrderSubmitStatus_set)
    TradingDay = property(_thosttraderapi.CThostFtdcExchangeQuoteField_TradingDay_get, _thosttraderapi.CThostFtdcExchangeQuoteField_TradingDay_set)
    SettlementID = property(_thosttraderapi.CThostFtdcExchangeQuoteField_SettlementID_get, _thosttraderapi.CThostFtdcExchangeQuoteField_SettlementID_set)
    QuoteSysID = property(_thosttraderapi.CThostFtdcExchangeQuoteField_QuoteSysID_get, _thosttraderapi.CThostFtdcExchangeQuoteField_QuoteSysID_set)
    InsertDate = property(_thosttraderapi.CThostFtdcExchangeQuoteField_InsertDate_get, _thosttraderapi.CThostFtdcExchangeQuoteField_InsertDate_set)
    InsertTime = property(_thosttraderapi.CThostFtdcExchangeQuoteField_InsertTime_get, _thosttraderapi.CThostFtdcExchangeQuoteField_InsertTime_set)
    CancelTime = property(_thosttraderapi.CThostFtdcExchangeQuoteField_CancelTime_get, _thosttraderapi.CThostFtdcExchangeQuoteField_CancelTime_set)
    QuoteStatus = property(_thosttraderapi.CThostFtdcExchangeQuoteField_QuoteStatus_get, _thosttraderapi.CThostFtdcExchangeQuoteField_QuoteStatus_set)
    ClearingPartID = property(_thosttraderapi.CThostFtdcExchangeQuoteField_ClearingPartID_get, _thosttraderapi.CThostFtdcExchangeQuoteField_ClearingPartID_set)
    SequenceNo = property(_thosttraderapi.CThostFtdcExchangeQuoteField_SequenceNo_get, _thosttraderapi.CThostFtdcExchangeQuoteField_SequenceNo_set)
    AskOrderSysID = property(_thosttraderapi.CThostFtdcExchangeQuoteField_AskOrderSysID_get, _thosttraderapi.CThostFtdcExchangeQuoteField_AskOrderSysID_set)
    BidOrderSysID = property(_thosttraderapi.CThostFtdcExchangeQuoteField_BidOrderSysID_get, _thosttraderapi.CThostFtdcExchangeQuoteField_BidOrderSysID_set)
    ForQuoteSysID = property(_thosttraderapi.CThostFtdcExchangeQuoteField_ForQuoteSysID_get, _thosttraderapi.CThostFtdcExchangeQuoteField_ForQuoteSysID_set)
    BranchID = property(_thosttraderapi.CThostFtdcExchangeQuoteField_BranchID_get, _thosttraderapi.CThostFtdcExchangeQuoteField_BranchID_set)
    IPAddress = property(_thosttraderapi.CThostFtdcExchangeQuoteField_IPAddress_get, _thosttraderapi.CThostFtdcExchangeQuoteField_IPAddress_set)
    MacAddress = property(_thosttraderapi.CThostFtdcExchangeQuoteField_MacAddress_get, _thosttraderapi.CThostFtdcExchangeQuoteField_MacAddress_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcExchangeQuoteField_swiginit(self, _thosttraderapi.new_CThostFtdcExchangeQuoteField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcExchangeQuoteField

# Register CThostFtdcExchangeQuoteField in _thosttraderapi:
_thosttraderapi.CThostFtdcExchangeQuoteField_swigregister(CThostFtdcExchangeQuoteField)

class CThostFtdcQryExchangeQuoteField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    ParticipantID = property(_thosttraderapi.CThostFtdcQryExchangeQuoteField_ParticipantID_get, _thosttraderapi.CThostFtdcQryExchangeQuoteField_ParticipantID_set)
    ClientID = property(_thosttraderapi.CThostFtdcQryExchangeQuoteField_ClientID_get, _thosttraderapi.CThostFtdcQryExchangeQuoteField_ClientID_set)
    ExchangeInstID = property(_thosttraderapi.CThostFtdcQryExchangeQuoteField_ExchangeInstID_get, _thosttraderapi.CThostFtdcQryExchangeQuoteField_ExchangeInstID_set)
    ExchangeID = property(_thosttraderapi.CThostFtdcQryExchangeQuoteField_ExchangeID_get, _thosttraderapi.CThostFtdcQryExchangeQuoteField_ExchangeID_set)
    TraderID = property(_thosttraderapi.CThostFtdcQryExchangeQuoteField_TraderID_get, _thosttraderapi.CThostFtdcQryExchangeQuoteField_TraderID_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcQryExchangeQuoteField_swiginit(self, _thosttraderapi.new_CThostFtdcQryExchangeQuoteField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcQryExchangeQuoteField

# Register CThostFtdcQryExchangeQuoteField in _thosttraderapi:
_thosttraderapi.CThostFtdcQryExchangeQuoteField_swigregister(CThostFtdcQryExchangeQuoteField)

class CThostFtdcQryQuoteActionField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thosttraderapi.CThostFtdcQryQuoteActionField_BrokerID_get, _thosttraderapi.CThostFtdcQryQuoteActionField_BrokerID_set)
    InvestorID = property(_thosttraderapi.CThostFtdcQryQuoteActionField_InvestorID_get, _thosttraderapi.CThostFtdcQryQuoteActionField_InvestorID_set)
    ExchangeID = property(_thosttraderapi.CThostFtdcQryQuoteActionField_ExchangeID_get, _thosttraderapi.CThostFtdcQryQuoteActionField_ExchangeID_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcQryQuoteActionField_swiginit(self, _thosttraderapi.new_CThostFtdcQryQuoteActionField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcQryQuoteActionField

# Register CThostFtdcQryQuoteActionField in _thosttraderapi:
_thosttraderapi.CThostFtdcQryQuoteActionField_swigregister(CThostFtdcQryQuoteActionField)

class CThostFtdcExchangeQuoteActionField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    ExchangeID = property(_thosttraderapi.CThostFtdcExchangeQuoteActionField_ExchangeID_get, _thosttraderapi.CThostFtdcExchangeQuoteActionField_ExchangeID_set)
    QuoteSysID = property(_thosttraderapi.CThostFtdcExchangeQuoteActionField_QuoteSysID_get, _thosttraderapi.CThostFtdcExchangeQuoteActionField_QuoteSysID_set)
    ActionFlag = property(_thosttraderapi.CThostFtdcExchangeQuoteActionField_ActionFlag_get, _thosttraderapi.CThostFtdcExchangeQuoteActionField_ActionFlag_set)
    ActionDate = property(_thosttraderapi.CThostFtdcExchangeQuoteActionField_ActionDate_get, _thosttraderapi.CThostFtdcExchangeQuoteActionField_ActionDate_set)
    ActionTime = property(_thosttraderapi.CThostFtdcExchangeQuoteActionField_ActionTime_get, _thosttraderapi.CThostFtdcExchangeQuoteActionField_ActionTime_set)
    TraderID = property(_thosttraderapi.CThostFtdcExchangeQuoteActionField_TraderID_get, _thosttraderapi.CThostFtdcExchangeQuoteActionField_TraderID_set)
    InstallID = property(_thosttraderapi.CThostFtdcExchangeQuoteActionField_InstallID_get, _thosttraderapi.CThostFtdcExchangeQuoteActionField_InstallID_set)
    QuoteLocalID = property(_thosttraderapi.CThostFtdcExchangeQuoteActionField_QuoteLocalID_get, _thosttraderapi.CThostFtdcExchangeQuoteActionField_QuoteLocalID_set)
    ActionLocalID = property(_thosttraderapi.CThostFtdcExchangeQuoteActionField_ActionLocalID_get, _thosttraderapi.CThostFtdcExchangeQuoteActionField_ActionLocalID_set)
    ParticipantID = property(_thosttraderapi.CThostFtdcExchangeQuoteActionField_ParticipantID_get, _thosttraderapi.CThostFtdcExchangeQuoteActionField_ParticipantID_set)
    ClientID = property(_thosttraderapi.CThostFtdcExchangeQuoteActionField_ClientID_get, _thosttraderapi.CThostFtdcExchangeQuoteActionField_ClientID_set)
    BusinessUnit = property(_thosttraderapi.CThostFtdcExchangeQuoteActionField_BusinessUnit_get, _thosttraderapi.CThostFtdcExchangeQuoteActionField_BusinessUnit_set)
    OrderActionStatus = property(_thosttraderapi.CThostFtdcExchangeQuoteActionField_OrderActionStatus_get, _thosttraderapi.CThostFtdcExchangeQuoteActionField_OrderActionStatus_set)
    UserID = property(_thosttraderapi.CThostFtdcExchangeQuoteActionField_UserID_get, _thosttraderapi.CThostFtdcExchangeQuoteActionField_UserID_set)
    IPAddress = property(_thosttraderapi.CThostFtdcExchangeQuoteActionField_IPAddress_get, _thosttraderapi.CThostFtdcExchangeQuoteActionField_IPAddress_set)
    MacAddress = property(_thosttraderapi.CThostFtdcExchangeQuoteActionField_MacAddress_get, _thosttraderapi.CThostFtdcExchangeQuoteActionField_MacAddress_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcExchangeQuoteActionField_swiginit(self, _thosttraderapi.new_CThostFtdcExchangeQuoteActionField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcExchangeQuoteActionField

# Register CThostFtdcExchangeQuoteActionField in _thosttraderapi:
_thosttraderapi.CThostFtdcExchangeQuoteActionField_swigregister(CThostFtdcExchangeQuoteActionField)

class CThostFtdcQryExchangeQuoteActionField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    ParticipantID = property(_thosttraderapi.CThostFtdcQryExchangeQuoteActionField_ParticipantID_get, _thosttraderapi.CThostFtdcQryExchangeQuoteActionField_ParticipantID_set)
    ClientID = property(_thosttraderapi.CThostFtdcQryExchangeQuoteActionField_ClientID_get, _thosttraderapi.CThostFtdcQryExchangeQuoteActionField_ClientID_set)
    ExchangeID = property(_thosttraderapi.CThostFtdcQryExchangeQuoteActionField_ExchangeID_get, _thosttraderapi.CThostFtdcQryExchangeQuoteActionField_ExchangeID_set)
    TraderID = property(_thosttraderapi.CThostFtdcQryExchangeQuoteActionField_TraderID_get, _thosttraderapi.CThostFtdcQryExchangeQuoteActionField_TraderID_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcQryExchangeQuoteActionField_swiginit(self, _thosttraderapi.new_CThostFtdcQryExchangeQuoteActionField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcQryExchangeQuoteActionField

# Register CThostFtdcQryExchangeQuoteActionField in _thosttraderapi:
_thosttraderapi.CThostFtdcQryExchangeQuoteActionField_swigregister(CThostFtdcQryExchangeQuoteActionField)

class CThostFtdcOptionInstrDeltaField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    InstrumentID = property(_thosttraderapi.CThostFtdcOptionInstrDeltaField_InstrumentID_get, _thosttraderapi.CThostFtdcOptionInstrDeltaField_InstrumentID_set)
    InvestorRange = property(_thosttraderapi.CThostFtdcOptionInstrDeltaField_InvestorRange_get, _thosttraderapi.CThostFtdcOptionInstrDeltaField_InvestorRange_set)
    BrokerID = property(_thosttraderapi.CThostFtdcOptionInstrDeltaField_BrokerID_get, _thosttraderapi.CThostFtdcOptionInstrDeltaField_BrokerID_set)
    InvestorID = property(_thosttraderapi.CThostFtdcOptionInstrDeltaField_InvestorID_get, _thosttraderapi.CThostFtdcOptionInstrDeltaField_InvestorID_set)
    Delta = property(_thosttraderapi.CThostFtdcOptionInstrDeltaField_Delta_get, _thosttraderapi.CThostFtdcOptionInstrDeltaField_Delta_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcOptionInstrDeltaField_swiginit(self, _thosttraderapi.new_CThostFtdcOptionInstrDeltaField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcOptionInstrDeltaField

# Register CThostFtdcOptionInstrDeltaField in _thosttraderapi:
_thosttraderapi.CThostFtdcOptionInstrDeltaField_swigregister(CThostFtdcOptionInstrDeltaField)

class CThostFtdcForQuoteRspField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    TradingDay = property(_thosttraderapi.CThostFtdcForQuoteRspField_TradingDay_get, _thosttraderapi.CThostFtdcForQuoteRspField_TradingDay_set)
    InstrumentID = property(_thosttraderapi.CThostFtdcForQuoteRspField_InstrumentID_get, _thosttraderapi.CThostFtdcForQuoteRspField_InstrumentID_set)
    ForQuoteSysID = property(_thosttraderapi.CThostFtdcForQuoteRspField_ForQuoteSysID_get, _thosttraderapi.CThostFtdcForQuoteRspField_ForQuoteSysID_set)
    ForQuoteTime = property(_thosttraderapi.CThostFtdcForQuoteRspField_ForQuoteTime_get, _thosttraderapi.CThostFtdcForQuoteRspField_ForQuoteTime_set)
    ActionDay = property(_thosttraderapi.CThostFtdcForQuoteRspField_ActionDay_get, _thosttraderapi.CThostFtdcForQuoteRspField_ActionDay_set)
    ExchangeID = property(_thosttraderapi.CThostFtdcForQuoteRspField_ExchangeID_get, _thosttraderapi.CThostFtdcForQuoteRspField_ExchangeID_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcForQuoteRspField_swiginit(self, _thosttraderapi.new_CThostFtdcForQuoteRspField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcForQuoteRspField

# Register CThostFtdcForQuoteRspField in _thosttraderapi:
_thosttraderapi.CThostFtdcForQuoteRspField_swigregister(CThostFtdcForQuoteRspField)

class CThostFtdcStrikeOffsetField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    InstrumentID = property(_thosttraderapi.CThostFtdcStrikeOffsetField_InstrumentID_get, _thosttraderapi.CThostFtdcStrikeOffsetField_InstrumentID_set)
    InvestorRange = property(_thosttraderapi.CThostFtdcStrikeOffsetField_InvestorRange_get, _thosttraderapi.CThostFtdcStrikeOffsetField_InvestorRange_set)
    BrokerID = property(_thosttraderapi.CThostFtdcStrikeOffsetField_BrokerID_get, _thosttraderapi.CThostFtdcStrikeOffsetField_BrokerID_set)
    InvestorID = property(_thosttraderapi.CThostFtdcStrikeOffsetField_InvestorID_get, _thosttraderapi.CThostFtdcStrikeOffsetField_InvestorID_set)
    Offset = property(_thosttraderapi.CThostFtdcStrikeOffsetField_Offset_get, _thosttraderapi.CThostFtdcStrikeOffsetField_Offset_set)
    OffsetType = property(_thosttraderapi.CThostFtdcStrikeOffsetField_OffsetType_get, _thosttraderapi.CThostFtdcStrikeOffsetField_OffsetType_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcStrikeOffsetField_swiginit(self, _thosttraderapi.new_CThostFtdcStrikeOffsetField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcStrikeOffsetField

# Register CThostFtdcStrikeOffsetField in _thosttraderapi:
_thosttraderapi.CThostFtdcStrikeOffsetField_swigregister(CThostFtdcStrikeOffsetField)

class CThostFtdcQryStrikeOffsetField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thosttraderapi.CThostFtdcQryStrikeOffsetField_BrokerID_get, _thosttraderapi.CThostFtdcQryStrikeOffsetField_BrokerID_set)
    InvestorID = property(_thosttraderapi.CThostFtdcQryStrikeOffsetField_InvestorID_get, _thosttraderapi.CThostFtdcQryStrikeOffsetField_InvestorID_set)
    InstrumentID = property(_thosttraderapi.CThostFtdcQryStrikeOffsetField_InstrumentID_get, _thosttraderapi.CThostFtdcQryStrikeOffsetField_InstrumentID_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcQryStrikeOffsetField_swiginit(self, _thosttraderapi.new_CThostFtdcQryStrikeOffsetField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcQryStrikeOffsetField

# Register CThostFtdcQryStrikeOffsetField in _thosttraderapi:
_thosttraderapi.CThostFtdcQryStrikeOffsetField_swigregister(CThostFtdcQryStrikeOffsetField)

class CThostFtdcInputBatchOrderActionField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thosttraderapi.CThostFtdcInputBatchOrderActionField_BrokerID_get, _thosttraderapi.CThostFtdcInputBatchOrderActionField_BrokerID_set)
    InvestorID = property(_thosttraderapi.CThostFtdcInputBatchOrderActionField_InvestorID_get, _thosttraderapi.CThostFtdcInputBatchOrderActionField_InvestorID_set)
    OrderActionRef = property(_thosttraderapi.CThostFtdcInputBatchOrderActionField_OrderActionRef_get, _thosttraderapi.CThostFtdcInputBatchOrderActionField_OrderActionRef_set)
    RequestID = property(_thosttraderapi.CThostFtdcInputBatchOrderActionField_RequestID_get, _thosttraderapi.CThostFtdcInputBatchOrderActionField_RequestID_set)
    FrontID = property(_thosttraderapi.CThostFtdcInputBatchOrderActionField_FrontID_get, _thosttraderapi.CThostFtdcInputBatchOrderActionField_FrontID_set)
    SessionID = property(_thosttraderapi.CThostFtdcInputBatchOrderActionField_SessionID_get, _thosttraderapi.CThostFtdcInputBatchOrderActionField_SessionID_set)
    ExchangeID = property(_thosttraderapi.CThostFtdcInputBatchOrderActionField_ExchangeID_get, _thosttraderapi.CThostFtdcInputBatchOrderActionField_ExchangeID_set)
    UserID = property(_thosttraderapi.CThostFtdcInputBatchOrderActionField_UserID_get, _thosttraderapi.CThostFtdcInputBatchOrderActionField_UserID_set)
    InvestUnitID = property(_thosttraderapi.CThostFtdcInputBatchOrderActionField_InvestUnitID_get, _thosttraderapi.CThostFtdcInputBatchOrderActionField_InvestUnitID_set)
    IPAddress = property(_thosttraderapi.CThostFtdcInputBatchOrderActionField_IPAddress_get, _thosttraderapi.CThostFtdcInputBatchOrderActionField_IPAddress_set)
    MacAddress = property(_thosttraderapi.CThostFtdcInputBatchOrderActionField_MacAddress_get, _thosttraderapi.CThostFtdcInputBatchOrderActionField_MacAddress_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcInputBatchOrderActionField_swiginit(self, _thosttraderapi.new_CThostFtdcInputBatchOrderActionField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcInputBatchOrderActionField

# Register CThostFtdcInputBatchOrderActionField in _thosttraderapi:
_thosttraderapi.CThostFtdcInputBatchOrderActionField_swigregister(CThostFtdcInputBatchOrderActionField)

class CThostFtdcBatchOrderActionField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thosttraderapi.CThostFtdcBatchOrderActionField_BrokerID_get, _thosttraderapi.CThostFtdcBatchOrderActionField_BrokerID_set)
    InvestorID = property(_thosttraderapi.CThostFtdcBatchOrderActionField_InvestorID_get, _thosttraderapi.CThostFtdcBatchOrderActionField_InvestorID_set)
    OrderActionRef = property(_thosttraderapi.CThostFtdcBatchOrderActionField_OrderActionRef_get, _thosttraderapi.CThostFtdcBatchOrderActionField_OrderActionRef_set)
    RequestID = property(_thosttraderapi.CThostFtdcBatchOrderActionField_RequestID_get, _thosttraderapi.CThostFtdcBatchOrderActionField_RequestID_set)
    FrontID = property(_thosttraderapi.CThostFtdcBatchOrderActionField_FrontID_get, _thosttraderapi.CThostFtdcBatchOrderActionField_FrontID_set)
    SessionID = property(_thosttraderapi.CThostFtdcBatchOrderActionField_SessionID_get, _thosttraderapi.CThostFtdcBatchOrderActionField_SessionID_set)
    ExchangeID = property(_thosttraderapi.CThostFtdcBatchOrderActionField_ExchangeID_get, _thosttraderapi.CThostFtdcBatchOrderActionField_ExchangeID_set)
    ActionDate = property(_thosttraderapi.CThostFtdcBatchOrderActionField_ActionDate_get, _thosttraderapi.CThostFtdcBatchOrderActionField_ActionDate_set)
    ActionTime = property(_thosttraderapi.CThostFtdcBatchOrderActionField_ActionTime_get, _thosttraderapi.CThostFtdcBatchOrderActionField_ActionTime_set)
    TraderID = property(_thosttraderapi.CThostFtdcBatchOrderActionField_TraderID_get, _thosttraderapi.CThostFtdcBatchOrderActionField_TraderID_set)
    InstallID = property(_thosttraderapi.CThostFtdcBatchOrderActionField_InstallID_get, _thosttraderapi.CThostFtdcBatchOrderActionField_InstallID_set)
    ActionLocalID = property(_thosttraderapi.CThostFtdcBatchOrderActionField_ActionLocalID_get, _thosttraderapi.CThostFtdcBatchOrderActionField_ActionLocalID_set)
    ParticipantID = property(_thosttraderapi.CThostFtdcBatchOrderActionField_ParticipantID_get, _thosttraderapi.CThostFtdcBatchOrderActionField_ParticipantID_set)
    ClientID = property(_thosttraderapi.CThostFtdcBatchOrderActionField_ClientID_get, _thosttraderapi.CThostFtdcBatchOrderActionField_ClientID_set)
    BusinessUnit = property(_thosttraderapi.CThostFtdcBatchOrderActionField_BusinessUnit_get, _thosttraderapi.CThostFtdcBatchOrderActionField_BusinessUnit_set)
    OrderActionStatus = property(_thosttraderapi.CThostFtdcBatchOrderActionField_OrderActionStatus_get, _thosttraderapi.CThostFtdcBatchOrderActionField_OrderActionStatus_set)
    UserID = property(_thosttraderapi.CThostFtdcBatchOrderActionField_UserID_get, _thosttraderapi.CThostFtdcBatchOrderActionField_UserID_set)
    StatusMsg = property(_thosttraderapi.CThostFtdcBatchOrderActionField_StatusMsg_get, _thosttraderapi.CThostFtdcBatchOrderActionField_StatusMsg_set)
    InvestUnitID = property(_thosttraderapi.CThostFtdcBatchOrderActionField_InvestUnitID_get, _thosttraderapi.CThostFtdcBatchOrderActionField_InvestUnitID_set)
    IPAddress = property(_thosttraderapi.CThostFtdcBatchOrderActionField_IPAddress_get, _thosttraderapi.CThostFtdcBatchOrderActionField_IPAddress_set)
    MacAddress = property(_thosttraderapi.CThostFtdcBatchOrderActionField_MacAddress_get, _thosttraderapi.CThostFtdcBatchOrderActionField_MacAddress_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcBatchOrderActionField_swiginit(self, _thosttraderapi.new_CThostFtdcBatchOrderActionField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcBatchOrderActionField

# Register CThostFtdcBatchOrderActionField in _thosttraderapi:
_thosttraderapi.CThostFtdcBatchOrderActionField_swigregister(CThostFtdcBatchOrderActionField)

class CThostFtdcExchangeBatchOrderActionField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    ExchangeID = property(_thosttraderapi.CThostFtdcExchangeBatchOrderActionField_ExchangeID_get, _thosttraderapi.CThostFtdcExchangeBatchOrderActionField_ExchangeID_set)
    ActionDate = property(_thosttraderapi.CThostFtdcExchangeBatchOrderActionField_ActionDate_get, _thosttraderapi.CThostFtdcExchangeBatchOrderActionField_ActionDate_set)
    ActionTime = property(_thosttraderapi.CThostFtdcExchangeBatchOrderActionField_ActionTime_get, _thosttraderapi.CThostFtdcExchangeBatchOrderActionField_ActionTime_set)
    TraderID = property(_thosttraderapi.CThostFtdcExchangeBatchOrderActionField_TraderID_get, _thosttraderapi.CThostFtdcExchangeBatchOrderActionField_TraderID_set)
    InstallID = property(_thosttraderapi.CThostFtdcExchangeBatchOrderActionField_InstallID_get, _thosttraderapi.CThostFtdcExchangeBatchOrderActionField_InstallID_set)
    ActionLocalID = property(_thosttraderapi.CThostFtdcExchangeBatchOrderActionField_ActionLocalID_get, _thosttraderapi.CThostFtdcExchangeBatchOrderActionField_ActionLocalID_set)
    ParticipantID = property(_thosttraderapi.CThostFtdcExchangeBatchOrderActionField_ParticipantID_get, _thosttraderapi.CThostFtdcExchangeBatchOrderActionField_ParticipantID_set)
    ClientID = property(_thosttraderapi.CThostFtdcExchangeBatchOrderActionField_ClientID_get, _thosttraderapi.CThostFtdcExchangeBatchOrderActionField_ClientID_set)
    BusinessUnit = property(_thosttraderapi.CThostFtdcExchangeBatchOrderActionField_BusinessUnit_get, _thosttraderapi.CThostFtdcExchangeBatchOrderActionField_BusinessUnit_set)
    OrderActionStatus = property(_thosttraderapi.CThostFtdcExchangeBatchOrderActionField_OrderActionStatus_get, _thosttraderapi.CThostFtdcExchangeBatchOrderActionField_OrderActionStatus_set)
    UserID = property(_thosttraderapi.CThostFtdcExchangeBatchOrderActionField_UserID_get, _thosttraderapi.CThostFtdcExchangeBatchOrderActionField_UserID_set)
    IPAddress = property(_thosttraderapi.CThostFtdcExchangeBatchOrderActionField_IPAddress_get, _thosttraderapi.CThostFtdcExchangeBatchOrderActionField_IPAddress_set)
    MacAddress = property(_thosttraderapi.CThostFtdcExchangeBatchOrderActionField_MacAddress_get, _thosttraderapi.CThostFtdcExchangeBatchOrderActionField_MacAddress_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcExchangeBatchOrderActionField_swiginit(self, _thosttraderapi.new_CThostFtdcExchangeBatchOrderActionField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcExchangeBatchOrderActionField

# Register CThostFtdcExchangeBatchOrderActionField in _thosttraderapi:
_thosttraderapi.CThostFtdcExchangeBatchOrderActionField_swigregister(CThostFtdcExchangeBatchOrderActionField)

class CThostFtdcQryBatchOrderActionField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thosttraderapi.CThostFtdcQryBatchOrderActionField_BrokerID_get, _thosttraderapi.CThostFtdcQryBatchOrderActionField_BrokerID_set)
    InvestorID = property(_thosttraderapi.CThostFtdcQryBatchOrderActionField_InvestorID_get, _thosttraderapi.CThostFtdcQryBatchOrderActionField_InvestorID_set)
    ExchangeID = property(_thosttraderapi.CThostFtdcQryBatchOrderActionField_ExchangeID_get, _thosttraderapi.CThostFtdcQryBatchOrderActionField_ExchangeID_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcQryBatchOrderActionField_swiginit(self, _thosttraderapi.new_CThostFtdcQryBatchOrderActionField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcQryBatchOrderActionField

# Register CThostFtdcQryBatchOrderActionField in _thosttraderapi:
_thosttraderapi.CThostFtdcQryBatchOrderActionField_swigregister(CThostFtdcQryBatchOrderActionField)

class CThostFtdcCombInstrumentGuardField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thosttraderapi.CThostFtdcCombInstrumentGuardField_BrokerID_get, _thosttraderapi.CThostFtdcCombInstrumentGuardField_BrokerID_set)
    InstrumentID = property(_thosttraderapi.CThostFtdcCombInstrumentGuardField_InstrumentID_get, _thosttraderapi.CThostFtdcCombInstrumentGuardField_InstrumentID_set)
    GuarantRatio = property(_thosttraderapi.CThostFtdcCombInstrumentGuardField_GuarantRatio_get, _thosttraderapi.CThostFtdcCombInstrumentGuardField_GuarantRatio_set)
    ExchangeID = property(_thosttraderapi.CThostFtdcCombInstrumentGuardField_ExchangeID_get, _thosttraderapi.CThostFtdcCombInstrumentGuardField_ExchangeID_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcCombInstrumentGuardField_swiginit(self, _thosttraderapi.new_CThostFtdcCombInstrumentGuardField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcCombInstrumentGuardField

# Register CThostFtdcCombInstrumentGuardField in _thosttraderapi:
_thosttraderapi.CThostFtdcCombInstrumentGuardField_swigregister(CThostFtdcCombInstrumentGuardField)

class CThostFtdcQryCombInstrumentGuardField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thosttraderapi.CThostFtdcQryCombInstrumentGuardField_BrokerID_get, _thosttraderapi.CThostFtdcQryCombInstrumentGuardField_BrokerID_set)
    InstrumentID = property(_thosttraderapi.CThostFtdcQryCombInstrumentGuardField_InstrumentID_get, _thosttraderapi.CThostFtdcQryCombInstrumentGuardField_InstrumentID_set)
    ExchangeID = property(_thosttraderapi.CThostFtdcQryCombInstrumentGuardField_ExchangeID_get, _thosttraderapi.CThostFtdcQryCombInstrumentGuardField_ExchangeID_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcQryCombInstrumentGuardField_swiginit(self, _thosttraderapi.new_CThostFtdcQryCombInstrumentGuardField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcQryCombInstrumentGuardField

# Register CThostFtdcQryCombInstrumentGuardField in _thosttraderapi:
_thosttraderapi.CThostFtdcQryCombInstrumentGuardField_swigregister(CThostFtdcQryCombInstrumentGuardField)

class CThostFtdcInputCombActionField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thosttraderapi.CThostFtdcInputCombActionField_BrokerID_get, _thosttraderapi.CThostFtdcInputCombActionField_BrokerID_set)
    InvestorID = property(_thosttraderapi.CThostFtdcInputCombActionField_InvestorID_get, _thosttraderapi.CThostFtdcInputCombActionField_InvestorID_set)
    InstrumentID = property(_thosttraderapi.CThostFtdcInputCombActionField_InstrumentID_get, _thosttraderapi.CThostFtdcInputCombActionField_InstrumentID_set)
    CombActionRef = property(_thosttraderapi.CThostFtdcInputCombActionField_CombActionRef_get, _thosttraderapi.CThostFtdcInputCombActionField_CombActionRef_set)
    UserID = property(_thosttraderapi.CThostFtdcInputCombActionField_UserID_get, _thosttraderapi.CThostFtdcInputCombActionField_UserID_set)
    Direction = property(_thosttraderapi.CThostFtdcInputCombActionField_Direction_get, _thosttraderapi.CThostFtdcInputCombActionField_Direction_set)
    Volume = property(_thosttraderapi.CThostFtdcInputCombActionField_Volume_get, _thosttraderapi.CThostFtdcInputCombActionField_Volume_set)
    CombDirection = property(_thosttraderapi.CThostFtdcInputCombActionField_CombDirection_get, _thosttraderapi.CThostFtdcInputCombActionField_CombDirection_set)
    HedgeFlag = property(_thosttraderapi.CThostFtdcInputCombActionField_HedgeFlag_get, _thosttraderapi.CThostFtdcInputCombActionField_HedgeFlag_set)
    ExchangeID = property(_thosttraderapi.CThostFtdcInputCombActionField_ExchangeID_get, _thosttraderapi.CThostFtdcInputCombActionField_ExchangeID_set)
    IPAddress = property(_thosttraderapi.CThostFtdcInputCombActionField_IPAddress_get, _thosttraderapi.CThostFtdcInputCombActionField_IPAddress_set)
    MacAddress = property(_thosttraderapi.CThostFtdcInputCombActionField_MacAddress_get, _thosttraderapi.CThostFtdcInputCombActionField_MacAddress_set)
    InvestUnitID = property(_thosttraderapi.CThostFtdcInputCombActionField_InvestUnitID_get, _thosttraderapi.CThostFtdcInputCombActionField_InvestUnitID_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcInputCombActionField_swiginit(self, _thosttraderapi.new_CThostFtdcInputCombActionField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcInputCombActionField

# Register CThostFtdcInputCombActionField in _thosttraderapi:
_thosttraderapi.CThostFtdcInputCombActionField_swigregister(CThostFtdcInputCombActionField)

class CThostFtdcCombActionField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thosttraderapi.CThostFtdcCombActionField_BrokerID_get, _thosttraderapi.CThostFtdcCombActionField_BrokerID_set)
    InvestorID = property(_thosttraderapi.CThostFtdcCombActionField_InvestorID_get, _thosttraderapi.CThostFtdcCombActionField_InvestorID_set)
    InstrumentID = property(_thosttraderapi.CThostFtdcCombActionField_InstrumentID_get, _thosttraderapi.CThostFtdcCombActionField_InstrumentID_set)
    CombActionRef = property(_thosttraderapi.CThostFtdcCombActionField_CombActionRef_get, _thosttraderapi.CThostFtdcCombActionField_CombActionRef_set)
    UserID = property(_thosttraderapi.CThostFtdcCombActionField_UserID_get, _thosttraderapi.CThostFtdcCombActionField_UserID_set)
    Direction = property(_thosttraderapi.CThostFtdcCombActionField_Direction_get, _thosttraderapi.CThostFtdcCombActionField_Direction_set)
    Volume = property(_thosttraderapi.CThostFtdcCombActionField_Volume_get, _thosttraderapi.CThostFtdcCombActionField_Volume_set)
    CombDirection = property(_thosttraderapi.CThostFtdcCombActionField_CombDirection_get, _thosttraderapi.CThostFtdcCombActionField_CombDirection_set)
    HedgeFlag = property(_thosttraderapi.CThostFtdcCombActionField_HedgeFlag_get, _thosttraderapi.CThostFtdcCombActionField_HedgeFlag_set)
    ActionLocalID = property(_thosttraderapi.CThostFtdcCombActionField_ActionLocalID_get, _thosttraderapi.CThostFtdcCombActionField_ActionLocalID_set)
    ExchangeID = property(_thosttraderapi.CThostFtdcCombActionField_ExchangeID_get, _thosttraderapi.CThostFtdcCombActionField_ExchangeID_set)
    ParticipantID = property(_thosttraderapi.CThostFtdcCombActionField_ParticipantID_get, _thosttraderapi.CThostFtdcCombActionField_ParticipantID_set)
    ClientID = property(_thosttraderapi.CThostFtdcCombActionField_ClientID_get, _thosttraderapi.CThostFtdcCombActionField_ClientID_set)
    ExchangeInstID = property(_thosttraderapi.CThostFtdcCombActionField_ExchangeInstID_get, _thosttraderapi.CThostFtdcCombActionField_ExchangeInstID_set)
    TraderID = property(_thosttraderapi.CThostFtdcCombActionField_TraderID_get, _thosttraderapi.CThostFtdcCombActionField_TraderID_set)
    InstallID = property(_thosttraderapi.CThostFtdcCombActionField_InstallID_get, _thosttraderapi.CThostFtdcCombActionField_InstallID_set)
    ActionStatus = property(_thosttraderapi.CThostFtdcCombActionField_ActionStatus_get, _thosttraderapi.CThostFtdcCombActionField_ActionStatus_set)
    NotifySequence = property(_thosttraderapi.CThostFtdcCombActionField_NotifySequence_get, _thosttraderapi.CThostFtdcCombActionField_NotifySequence_set)
    TradingDay = property(_thosttraderapi.CThostFtdcCombActionField_TradingDay_get, _thosttraderapi.CThostFtdcCombActionField_TradingDay_set)
    SettlementID = property(_thosttraderapi.CThostFtdcCombActionField_SettlementID_get, _thosttraderapi.CThostFtdcCombActionField_SettlementID_set)
    SequenceNo = property(_thosttraderapi.CThostFtdcCombActionField_SequenceNo_get, _thosttraderapi.CThostFtdcCombActionField_SequenceNo_set)
    FrontID = property(_thosttraderapi.CThostFtdcCombActionField_FrontID_get, _thosttraderapi.CThostFtdcCombActionField_FrontID_set)
    SessionID = property(_thosttraderapi.CThostFtdcCombActionField_SessionID_get, _thosttraderapi.CThostFtdcCombActionField_SessionID_set)
    UserProductInfo = property(_thosttraderapi.CThostFtdcCombActionField_UserProductInfo_get, _thosttraderapi.CThostFtdcCombActionField_UserProductInfo_set)
    StatusMsg = property(_thosttraderapi.CThostFtdcCombActionField_StatusMsg_get, _thosttraderapi.CThostFtdcCombActionField_StatusMsg_set)
    IPAddress = property(_thosttraderapi.CThostFtdcCombActionField_IPAddress_get, _thosttraderapi.CThostFtdcCombActionField_IPAddress_set)
    MacAddress = property(_thosttraderapi.CThostFtdcCombActionField_MacAddress_get, _thosttraderapi.CThostFtdcCombActionField_MacAddress_set)
    ComTradeID = property(_thosttraderapi.CThostFtdcCombActionField_ComTradeID_get, _thosttraderapi.CThostFtdcCombActionField_ComTradeID_set)
    BranchID = property(_thosttraderapi.CThostFtdcCombActionField_BranchID_get, _thosttraderapi.CThostFtdcCombActionField_BranchID_set)
    InvestUnitID = property(_thosttraderapi.CThostFtdcCombActionField_InvestUnitID_get, _thosttraderapi.CThostFtdcCombActionField_InvestUnitID_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcCombActionField_swiginit(self, _thosttraderapi.new_CThostFtdcCombActionField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcCombActionField

# Register CThostFtdcCombActionField in _thosttraderapi:
_thosttraderapi.CThostFtdcCombActionField_swigregister(CThostFtdcCombActionField)

class CThostFtdcQryCombActionField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thosttraderapi.CThostFtdcQryCombActionField_BrokerID_get, _thosttraderapi.CThostFtdcQryCombActionField_BrokerID_set)
    InvestorID = property(_thosttraderapi.CThostFtdcQryCombActionField_InvestorID_get, _thosttraderapi.CThostFtdcQryCombActionField_InvestorID_set)
    InstrumentID = property(_thosttraderapi.CThostFtdcQryCombActionField_InstrumentID_get, _thosttraderapi.CThostFtdcQryCombActionField_InstrumentID_set)
    ExchangeID = property(_thosttraderapi.CThostFtdcQryCombActionField_ExchangeID_get, _thosttraderapi.CThostFtdcQryCombActionField_ExchangeID_set)
    InvestUnitID = property(_thosttraderapi.CThostFtdcQryCombActionField_InvestUnitID_get, _thosttraderapi.CThostFtdcQryCombActionField_InvestUnitID_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcQryCombActionField_swiginit(self, _thosttraderapi.new_CThostFtdcQryCombActionField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcQryCombActionField

# Register CThostFtdcQryCombActionField in _thosttraderapi:
_thosttraderapi.CThostFtdcQryCombActionField_swigregister(CThostFtdcQryCombActionField)

class CThostFtdcExchangeCombActionField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    Direction = property(_thosttraderapi.CThostFtdcExchangeCombActionField_Direction_get, _thosttraderapi.CThostFtdcExchangeCombActionField_Direction_set)
    Volume = property(_thosttraderapi.CThostFtdcExchangeCombActionField_Volume_get, _thosttraderapi.CThostFtdcExchangeCombActionField_Volume_set)
    CombDirection = property(_thosttraderapi.CThostFtdcExchangeCombActionField_CombDirection_get, _thosttraderapi.CThostFtdcExchangeCombActionField_CombDirection_set)
    HedgeFlag = property(_thosttraderapi.CThostFtdcExchangeCombActionField_HedgeFlag_get, _thosttraderapi.CThostFtdcExchangeCombActionField_HedgeFlag_set)
    ActionLocalID = property(_thosttraderapi.CThostFtdcExchangeCombActionField_ActionLocalID_get, _thosttraderapi.CThostFtdcExchangeCombActionField_ActionLocalID_set)
    ExchangeID = property(_thosttraderapi.CThostFtdcExchangeCombActionField_ExchangeID_get, _thosttraderapi.CThostFtdcExchangeCombActionField_ExchangeID_set)
    ParticipantID = property(_thosttraderapi.CThostFtdcExchangeCombActionField_ParticipantID_get, _thosttraderapi.CThostFtdcExchangeCombActionField_ParticipantID_set)
    ClientID = property(_thosttraderapi.CThostFtdcExchangeCombActionField_ClientID_get, _thosttraderapi.CThostFtdcExchangeCombActionField_ClientID_set)
    ExchangeInstID = property(_thosttraderapi.CThostFtdcExchangeCombActionField_ExchangeInstID_get, _thosttraderapi.CThostFtdcExchangeCombActionField_ExchangeInstID_set)
    TraderID = property(_thosttraderapi.CThostFtdcExchangeCombActionField_TraderID_get, _thosttraderapi.CThostFtdcExchangeCombActionField_TraderID_set)
    InstallID = property(_thosttraderapi.CThostFtdcExchangeCombActionField_InstallID_get, _thosttraderapi.CThostFtdcExchangeCombActionField_InstallID_set)
    ActionStatus = property(_thosttraderapi.CThostFtdcExchangeCombActionField_ActionStatus_get, _thosttraderapi.CThostFtdcExchangeCombActionField_ActionStatus_set)
    NotifySequence = property(_thosttraderapi.CThostFtdcExchangeCombActionField_NotifySequence_get, _thosttraderapi.CThostFtdcExchangeCombActionField_NotifySequence_set)
    TradingDay = property(_thosttraderapi.CThostFtdcExchangeCombActionField_TradingDay_get, _thosttraderapi.CThostFtdcExchangeCombActionField_TradingDay_set)
    SettlementID = property(_thosttraderapi.CThostFtdcExchangeCombActionField_SettlementID_get, _thosttraderapi.CThostFtdcExchangeCombActionField_SettlementID_set)
    SequenceNo = property(_thosttraderapi.CThostFtdcExchangeCombActionField_SequenceNo_get, _thosttraderapi.CThostFtdcExchangeCombActionField_SequenceNo_set)
    IPAddress = property(_thosttraderapi.CThostFtdcExchangeCombActionField_IPAddress_get, _thosttraderapi.CThostFtdcExchangeCombActionField_IPAddress_set)
    MacAddress = property(_thosttraderapi.CThostFtdcExchangeCombActionField_MacAddress_get, _thosttraderapi.CThostFtdcExchangeCombActionField_MacAddress_set)
    ComTradeID = property(_thosttraderapi.CThostFtdcExchangeCombActionField_ComTradeID_get, _thosttraderapi.CThostFtdcExchangeCombActionField_ComTradeID_set)
    BranchID = property(_thosttraderapi.CThostFtdcExchangeCombActionField_BranchID_get, _thosttraderapi.CThostFtdcExchangeCombActionField_BranchID_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcExchangeCombActionField_swiginit(self, _thosttraderapi.new_CThostFtdcExchangeCombActionField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcExchangeCombActionField

# Register CThostFtdcExchangeCombActionField in _thosttraderapi:
_thosttraderapi.CThostFtdcExchangeCombActionField_swigregister(CThostFtdcExchangeCombActionField)

class CThostFtdcQryExchangeCombActionField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    ParticipantID = property(_thosttraderapi.CThostFtdcQryExchangeCombActionField_ParticipantID_get, _thosttraderapi.CThostFtdcQryExchangeCombActionField_ParticipantID_set)
    ClientID = property(_thosttraderapi.CThostFtdcQryExchangeCombActionField_ClientID_get, _thosttraderapi.CThostFtdcQryExchangeCombActionField_ClientID_set)
    ExchangeInstID = property(_thosttraderapi.CThostFtdcQryExchangeCombActionField_ExchangeInstID_get, _thosttraderapi.CThostFtdcQryExchangeCombActionField_ExchangeInstID_set)
    ExchangeID = property(_thosttraderapi.CThostFtdcQryExchangeCombActionField_ExchangeID_get, _thosttraderapi.CThostFtdcQryExchangeCombActionField_ExchangeID_set)
    TraderID = property(_thosttraderapi.CThostFtdcQryExchangeCombActionField_TraderID_get, _thosttraderapi.CThostFtdcQryExchangeCombActionField_TraderID_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcQryExchangeCombActionField_swiginit(self, _thosttraderapi.new_CThostFtdcQryExchangeCombActionField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcQryExchangeCombActionField

# Register CThostFtdcQryExchangeCombActionField in _thosttraderapi:
_thosttraderapi.CThostFtdcQryExchangeCombActionField_swigregister(CThostFtdcQryExchangeCombActionField)

class CThostFtdcProductExchRateField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    ProductID = property(_thosttraderapi.CThostFtdcProductExchRateField_ProductID_get, _thosttraderapi.CThostFtdcProductExchRateField_ProductID_set)
    QuoteCurrencyID = property(_thosttraderapi.CThostFtdcProductExchRateField_QuoteCurrencyID_get, _thosttraderapi.CThostFtdcProductExchRateField_QuoteCurrencyID_set)
    ExchangeRate = property(_thosttraderapi.CThostFtdcProductExchRateField_ExchangeRate_get, _thosttraderapi.CThostFtdcProductExchRateField_ExchangeRate_set)
    ExchangeID = property(_thosttraderapi.CThostFtdcProductExchRateField_ExchangeID_get, _thosttraderapi.CThostFtdcProductExchRateField_ExchangeID_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcProductExchRateField_swiginit(self, _thosttraderapi.new_CThostFtdcProductExchRateField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcProductExchRateField

# Register CThostFtdcProductExchRateField in _thosttraderapi:
_thosttraderapi.CThostFtdcProductExchRateField_swigregister(CThostFtdcProductExchRateField)

class CThostFtdcQryProductExchRateField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    ProductID = property(_thosttraderapi.CThostFtdcQryProductExchRateField_ProductID_get, _thosttraderapi.CThostFtdcQryProductExchRateField_ProductID_set)
    ExchangeID = property(_thosttraderapi.CThostFtdcQryProductExchRateField_ExchangeID_get, _thosttraderapi.CThostFtdcQryProductExchRateField_ExchangeID_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcQryProductExchRateField_swiginit(self, _thosttraderapi.new_CThostFtdcQryProductExchRateField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcQryProductExchRateField

# Register CThostFtdcQryProductExchRateField in _thosttraderapi:
_thosttraderapi.CThostFtdcQryProductExchRateField_swigregister(CThostFtdcQryProductExchRateField)

class CThostFtdcQryForQuoteParamField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thosttraderapi.CThostFtdcQryForQuoteParamField_BrokerID_get, _thosttraderapi.CThostFtdcQryForQuoteParamField_BrokerID_set)
    InstrumentID = property(_thosttraderapi.CThostFtdcQryForQuoteParamField_InstrumentID_get, _thosttraderapi.CThostFtdcQryForQuoteParamField_InstrumentID_set)
    ExchangeID = property(_thosttraderapi.CThostFtdcQryForQuoteParamField_ExchangeID_get, _thosttraderapi.CThostFtdcQryForQuoteParamField_ExchangeID_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcQryForQuoteParamField_swiginit(self, _thosttraderapi.new_CThostFtdcQryForQuoteParamField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcQryForQuoteParamField

# Register CThostFtdcQryForQuoteParamField in _thosttraderapi:
_thosttraderapi.CThostFtdcQryForQuoteParamField_swigregister(CThostFtdcQryForQuoteParamField)

class CThostFtdcForQuoteParamField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thosttraderapi.CThostFtdcForQuoteParamField_BrokerID_get, _thosttraderapi.CThostFtdcForQuoteParamField_BrokerID_set)
    InstrumentID = property(_thosttraderapi.CThostFtdcForQuoteParamField_InstrumentID_get, _thosttraderapi.CThostFtdcForQuoteParamField_InstrumentID_set)
    ExchangeID = property(_thosttraderapi.CThostFtdcForQuoteParamField_ExchangeID_get, _thosttraderapi.CThostFtdcForQuoteParamField_ExchangeID_set)
    LastPrice = property(_thosttraderapi.CThostFtdcForQuoteParamField_LastPrice_get, _thosttraderapi.CThostFtdcForQuoteParamField_LastPrice_set)
    PriceInterval = property(_thosttraderapi.CThostFtdcForQuoteParamField_PriceInterval_get, _thosttraderapi.CThostFtdcForQuoteParamField_PriceInterval_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcForQuoteParamField_swiginit(self, _thosttraderapi.new_CThostFtdcForQuoteParamField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcForQuoteParamField

# Register CThostFtdcForQuoteParamField in _thosttraderapi:
_thosttraderapi.CThostFtdcForQuoteParamField_swigregister(CThostFtdcForQuoteParamField)

class CThostFtdcMMOptionInstrCommRateField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    InstrumentID = property(_thosttraderapi.CThostFtdcMMOptionInstrCommRateField_InstrumentID_get, _thosttraderapi.CThostFtdcMMOptionInstrCommRateField_InstrumentID_set)
    InvestorRange = property(_thosttraderapi.CThostFtdcMMOptionInstrCommRateField_InvestorRange_get, _thosttraderapi.CThostFtdcMMOptionInstrCommRateField_InvestorRange_set)
    BrokerID = property(_thosttraderapi.CThostFtdcMMOptionInstrCommRateField_BrokerID_get, _thosttraderapi.CThostFtdcMMOptionInstrCommRateField_BrokerID_set)
    InvestorID = property(_thosttraderapi.CThostFtdcMMOptionInstrCommRateField_InvestorID_get, _thosttraderapi.CThostFtdcMMOptionInstrCommRateField_InvestorID_set)
    OpenRatioByMoney = property(_thosttraderapi.CThostFtdcMMOptionInstrCommRateField_OpenRatioByMoney_get, _thosttraderapi.CThostFtdcMMOptionInstrCommRateField_OpenRatioByMoney_set)
    OpenRatioByVolume = property(_thosttraderapi.CThostFtdcMMOptionInstrCommRateField_OpenRatioByVolume_get, _thosttraderapi.CThostFtdcMMOptionInstrCommRateField_OpenRatioByVolume_set)
    CloseRatioByMoney = property(_thosttraderapi.CThostFtdcMMOptionInstrCommRateField_CloseRatioByMoney_get, _thosttraderapi.CThostFtdcMMOptionInstrCommRateField_CloseRatioByMoney_set)
    CloseRatioByVolume = property(_thosttraderapi.CThostFtdcMMOptionInstrCommRateField_CloseRatioByVolume_get, _thosttraderapi.CThostFtdcMMOptionInstrCommRateField_CloseRatioByVolume_set)
    CloseTodayRatioByMoney = property(_thosttraderapi.CThostFtdcMMOptionInstrCommRateField_CloseTodayRatioByMoney_get, _thosttraderapi.CThostFtdcMMOptionInstrCommRateField_CloseTodayRatioByMoney_set)
    CloseTodayRatioByVolume = property(_thosttraderapi.CThostFtdcMMOptionInstrCommRateField_CloseTodayRatioByVolume_get, _thosttraderapi.CThostFtdcMMOptionInstrCommRateField_CloseTodayRatioByVolume_set)
    StrikeRatioByMoney = property(_thosttraderapi.CThostFtdcMMOptionInstrCommRateField_StrikeRatioByMoney_get, _thosttraderapi.CThostFtdcMMOptionInstrCommRateField_StrikeRatioByMoney_set)
    StrikeRatioByVolume = property(_thosttraderapi.CThostFtdcMMOptionInstrCommRateField_StrikeRatioByVolume_get, _thosttraderapi.CThostFtdcMMOptionInstrCommRateField_StrikeRatioByVolume_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcMMOptionInstrCommRateField_swiginit(self, _thosttraderapi.new_CThostFtdcMMOptionInstrCommRateField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcMMOptionInstrCommRateField

# Register CThostFtdcMMOptionInstrCommRateField in _thosttraderapi:
_thosttraderapi.CThostFtdcMMOptionInstrCommRateField_swigregister(CThostFtdcMMOptionInstrCommRateField)

class CThostFtdcQryMMOptionInstrCommRateField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thosttraderapi.CThostFtdcQryMMOptionInstrCommRateField_BrokerID_get, _thosttraderapi.CThostFtdcQryMMOptionInstrCommRateField_BrokerID_set)
    InvestorID = property(_thosttraderapi.CThostFtdcQryMMOptionInstrCommRateField_InvestorID_get, _thosttraderapi.CThostFtdcQryMMOptionInstrCommRateField_InvestorID_set)
    InstrumentID = property(_thosttraderapi.CThostFtdcQryMMOptionInstrCommRateField_InstrumentID_get, _thosttraderapi.CThostFtdcQryMMOptionInstrCommRateField_InstrumentID_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcQryMMOptionInstrCommRateField_swiginit(self, _thosttraderapi.new_CThostFtdcQryMMOptionInstrCommRateField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcQryMMOptionInstrCommRateField

# Register CThostFtdcQryMMOptionInstrCommRateField in _thosttraderapi:
_thosttraderapi.CThostFtdcQryMMOptionInstrCommRateField_swigregister(CThostFtdcQryMMOptionInstrCommRateField)

class CThostFtdcMMInstrumentCommissionRateField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    InstrumentID = property(_thosttraderapi.CThostFtdcMMInstrumentCommissionRateField_InstrumentID_get, _thosttraderapi.CThostFtdcMMInstrumentCommissionRateField_InstrumentID_set)
    InvestorRange = property(_thosttraderapi.CThostFtdcMMInstrumentCommissionRateField_InvestorRange_get, _thosttraderapi.CThostFtdcMMInstrumentCommissionRateField_InvestorRange_set)
    BrokerID = property(_thosttraderapi.CThostFtdcMMInstrumentCommissionRateField_BrokerID_get, _thosttraderapi.CThostFtdcMMInstrumentCommissionRateField_BrokerID_set)
    InvestorID = property(_thosttraderapi.CThostFtdcMMInstrumentCommissionRateField_InvestorID_get, _thosttraderapi.CThostFtdcMMInstrumentCommissionRateField_InvestorID_set)
    OpenRatioByMoney = property(_thosttraderapi.CThostFtdcMMInstrumentCommissionRateField_OpenRatioByMoney_get, _thosttraderapi.CThostFtdcMMInstrumentCommissionRateField_OpenRatioByMoney_set)
    OpenRatioByVolume = property(_thosttraderapi.CThostFtdcMMInstrumentCommissionRateField_OpenRatioByVolume_get, _thosttraderapi.CThostFtdcMMInstrumentCommissionRateField_OpenRatioByVolume_set)
    CloseRatioByMoney = property(_thosttraderapi.CThostFtdcMMInstrumentCommissionRateField_CloseRatioByMoney_get, _thosttraderapi.CThostFtdcMMInstrumentCommissionRateField_CloseRatioByMoney_set)
    CloseRatioByVolume = property(_thosttraderapi.CThostFtdcMMInstrumentCommissionRateField_CloseRatioByVolume_get, _thosttraderapi.CThostFtdcMMInstrumentCommissionRateField_CloseRatioByVolume_set)
    CloseTodayRatioByMoney = property(_thosttraderapi.CThostFtdcMMInstrumentCommissionRateField_CloseTodayRatioByMoney_get, _thosttraderapi.CThostFtdcMMInstrumentCommissionRateField_CloseTodayRatioByMoney_set)
    CloseTodayRatioByVolume = property(_thosttraderapi.CThostFtdcMMInstrumentCommissionRateField_CloseTodayRatioByVolume_get, _thosttraderapi.CThostFtdcMMInstrumentCommissionRateField_CloseTodayRatioByVolume_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcMMInstrumentCommissionRateField_swiginit(self, _thosttraderapi.new_CThostFtdcMMInstrumentCommissionRateField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcMMInstrumentCommissionRateField

# Register CThostFtdcMMInstrumentCommissionRateField in _thosttraderapi:
_thosttraderapi.CThostFtdcMMInstrumentCommissionRateField_swigregister(CThostFtdcMMInstrumentCommissionRateField)

class CThostFtdcQryMMInstrumentCommissionRateField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thosttraderapi.CThostFtdcQryMMInstrumentCommissionRateField_BrokerID_get, _thosttraderapi.CThostFtdcQryMMInstrumentCommissionRateField_BrokerID_set)
    InvestorID = property(_thosttraderapi.CThostFtdcQryMMInstrumentCommissionRateField_InvestorID_get, _thosttraderapi.CThostFtdcQryMMInstrumentCommissionRateField_InvestorID_set)
    InstrumentID = property(_thosttraderapi.CThostFtdcQryMMInstrumentCommissionRateField_InstrumentID_get, _thosttraderapi.CThostFtdcQryMMInstrumentCommissionRateField_InstrumentID_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcQryMMInstrumentCommissionRateField_swiginit(self, _thosttraderapi.new_CThostFtdcQryMMInstrumentCommissionRateField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcQryMMInstrumentCommissionRateField

# Register CThostFtdcQryMMInstrumentCommissionRateField in _thosttraderapi:
_thosttraderapi.CThostFtdcQryMMInstrumentCommissionRateField_swigregister(CThostFtdcQryMMInstrumentCommissionRateField)

class CThostFtdcInstrumentOrderCommRateField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    InstrumentID = property(_thosttraderapi.CThostFtdcInstrumentOrderCommRateField_InstrumentID_get, _thosttraderapi.CThostFtdcInstrumentOrderCommRateField_InstrumentID_set)
    InvestorRange = property(_thosttraderapi.CThostFtdcInstrumentOrderCommRateField_InvestorRange_get, _thosttraderapi.CThostFtdcInstrumentOrderCommRateField_InvestorRange_set)
    BrokerID = property(_thosttraderapi.CThostFtdcInstrumentOrderCommRateField_BrokerID_get, _thosttraderapi.CThostFtdcInstrumentOrderCommRateField_BrokerID_set)
    InvestorID = property(_thosttraderapi.CThostFtdcInstrumentOrderCommRateField_InvestorID_get, _thosttraderapi.CThostFtdcInstrumentOrderCommRateField_InvestorID_set)
    HedgeFlag = property(_thosttraderapi.CThostFtdcInstrumentOrderCommRateField_HedgeFlag_get, _thosttraderapi.CThostFtdcInstrumentOrderCommRateField_HedgeFlag_set)
    OrderCommByVolume = property(_thosttraderapi.CThostFtdcInstrumentOrderCommRateField_OrderCommByVolume_get, _thosttraderapi.CThostFtdcInstrumentOrderCommRateField_OrderCommByVolume_set)
    OrderActionCommByVolume = property(_thosttraderapi.CThostFtdcInstrumentOrderCommRateField_OrderActionCommByVolume_get, _thosttraderapi.CThostFtdcInstrumentOrderCommRateField_OrderActionCommByVolume_set)
    ExchangeID = property(_thosttraderapi.CThostFtdcInstrumentOrderCommRateField_ExchangeID_get, _thosttraderapi.CThostFtdcInstrumentOrderCommRateField_ExchangeID_set)
    InvestUnitID = property(_thosttraderapi.CThostFtdcInstrumentOrderCommRateField_InvestUnitID_get, _thosttraderapi.CThostFtdcInstrumentOrderCommRateField_InvestUnitID_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcInstrumentOrderCommRateField_swiginit(self, _thosttraderapi.new_CThostFtdcInstrumentOrderCommRateField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcInstrumentOrderCommRateField

# Register CThostFtdcInstrumentOrderCommRateField in _thosttraderapi:
_thosttraderapi.CThostFtdcInstrumentOrderCommRateField_swigregister(CThostFtdcInstrumentOrderCommRateField)

class CThostFtdcQryInstrumentOrderCommRateField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thosttraderapi.CThostFtdcQryInstrumentOrderCommRateField_BrokerID_get, _thosttraderapi.CThostFtdcQryInstrumentOrderCommRateField_BrokerID_set)
    InvestorID = property(_thosttraderapi.CThostFtdcQryInstrumentOrderCommRateField_InvestorID_get, _thosttraderapi.CThostFtdcQryInstrumentOrderCommRateField_InvestorID_set)
    InstrumentID = property(_thosttraderapi.CThostFtdcQryInstrumentOrderCommRateField_InstrumentID_get, _thosttraderapi.CThostFtdcQryInstrumentOrderCommRateField_InstrumentID_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcQryInstrumentOrderCommRateField_swiginit(self, _thosttraderapi.new_CThostFtdcQryInstrumentOrderCommRateField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcQryInstrumentOrderCommRateField

# Register CThostFtdcQryInstrumentOrderCommRateField in _thosttraderapi:
_thosttraderapi.CThostFtdcQryInstrumentOrderCommRateField_swigregister(CThostFtdcQryInstrumentOrderCommRateField)

class CThostFtdcTradeParamField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thosttraderapi.CThostFtdcTradeParamField_BrokerID_get, _thosttraderapi.CThostFtdcTradeParamField_BrokerID_set)
    TradeParamID = property(_thosttraderapi.CThostFtdcTradeParamField_TradeParamID_get, _thosttraderapi.CThostFtdcTradeParamField_TradeParamID_set)
    TradeParamValue = property(_thosttraderapi.CThostFtdcTradeParamField_TradeParamValue_get, _thosttraderapi.CThostFtdcTradeParamField_TradeParamValue_set)
    Memo = property(_thosttraderapi.CThostFtdcTradeParamField_Memo_get, _thosttraderapi.CThostFtdcTradeParamField_Memo_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcTradeParamField_swiginit(self, _thosttraderapi.new_CThostFtdcTradeParamField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcTradeParamField

# Register CThostFtdcTradeParamField in _thosttraderapi:
_thosttraderapi.CThostFtdcTradeParamField_swigregister(CThostFtdcTradeParamField)

class CThostFtdcInstrumentMarginRateULField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    InstrumentID = property(_thosttraderapi.CThostFtdcInstrumentMarginRateULField_InstrumentID_get, _thosttraderapi.CThostFtdcInstrumentMarginRateULField_InstrumentID_set)
    InvestorRange = property(_thosttraderapi.CThostFtdcInstrumentMarginRateULField_InvestorRange_get, _thosttraderapi.CThostFtdcInstrumentMarginRateULField_InvestorRange_set)
    BrokerID = property(_thosttraderapi.CThostFtdcInstrumentMarginRateULField_BrokerID_get, _thosttraderapi.CThostFtdcInstrumentMarginRateULField_BrokerID_set)
    InvestorID = property(_thosttraderapi.CThostFtdcInstrumentMarginRateULField_InvestorID_get, _thosttraderapi.CThostFtdcInstrumentMarginRateULField_InvestorID_set)
    HedgeFlag = property(_thosttraderapi.CThostFtdcInstrumentMarginRateULField_HedgeFlag_get, _thosttraderapi.CThostFtdcInstrumentMarginRateULField_HedgeFlag_set)
    LongMarginRatioByMoney = property(_thosttraderapi.CThostFtdcInstrumentMarginRateULField_LongMarginRatioByMoney_get, _thosttraderapi.CThostFtdcInstrumentMarginRateULField_LongMarginRatioByMoney_set)
    LongMarginRatioByVolume = property(_thosttraderapi.CThostFtdcInstrumentMarginRateULField_LongMarginRatioByVolume_get, _thosttraderapi.CThostFtdcInstrumentMarginRateULField_LongMarginRatioByVolume_set)
    ShortMarginRatioByMoney = property(_thosttraderapi.CThostFtdcInstrumentMarginRateULField_ShortMarginRatioByMoney_get, _thosttraderapi.CThostFtdcInstrumentMarginRateULField_ShortMarginRatioByMoney_set)
    ShortMarginRatioByVolume = property(_thosttraderapi.CThostFtdcInstrumentMarginRateULField_ShortMarginRatioByVolume_get, _thosttraderapi.CThostFtdcInstrumentMarginRateULField_ShortMarginRatioByVolume_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcInstrumentMarginRateULField_swiginit(self, _thosttraderapi.new_CThostFtdcInstrumentMarginRateULField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcInstrumentMarginRateULField

# Register CThostFtdcInstrumentMarginRateULField in _thosttraderapi:
_thosttraderapi.CThostFtdcInstrumentMarginRateULField_swigregister(CThostFtdcInstrumentMarginRateULField)

class CThostFtdcFutureLimitPosiParamField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    InvestorRange = property(_thosttraderapi.CThostFtdcFutureLimitPosiParamField_InvestorRange_get, _thosttraderapi.CThostFtdcFutureLimitPosiParamField_InvestorRange_set)
    BrokerID = property(_thosttraderapi.CThostFtdcFutureLimitPosiParamField_BrokerID_get, _thosttraderapi.CThostFtdcFutureLimitPosiParamField_BrokerID_set)
    InvestorID = property(_thosttraderapi.CThostFtdcFutureLimitPosiParamField_InvestorID_get, _thosttraderapi.CThostFtdcFutureLimitPosiParamField_InvestorID_set)
    ProductID = property(_thosttraderapi.CThostFtdcFutureLimitPosiParamField_ProductID_get, _thosttraderapi.CThostFtdcFutureLimitPosiParamField_ProductID_set)
    SpecOpenVolume = property(_thosttraderapi.CThostFtdcFutureLimitPosiParamField_SpecOpenVolume_get, _thosttraderapi.CThostFtdcFutureLimitPosiParamField_SpecOpenVolume_set)
    ArbiOpenVolume = property(_thosttraderapi.CThostFtdcFutureLimitPosiParamField_ArbiOpenVolume_get, _thosttraderapi.CThostFtdcFutureLimitPosiParamField_ArbiOpenVolume_set)
    OpenVolume = property(_thosttraderapi.CThostFtdcFutureLimitPosiParamField_OpenVolume_get, _thosttraderapi.CThostFtdcFutureLimitPosiParamField_OpenVolume_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcFutureLimitPosiParamField_swiginit(self, _thosttraderapi.new_CThostFtdcFutureLimitPosiParamField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcFutureLimitPosiParamField

# Register CThostFtdcFutureLimitPosiParamField in _thosttraderapi:
_thosttraderapi.CThostFtdcFutureLimitPosiParamField_swigregister(CThostFtdcFutureLimitPosiParamField)

class CThostFtdcLoginForbiddenIPField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    IPAddress = property(_thosttraderapi.CThostFtdcLoginForbiddenIPField_IPAddress_get, _thosttraderapi.CThostFtdcLoginForbiddenIPField_IPAddress_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcLoginForbiddenIPField_swiginit(self, _thosttraderapi.new_CThostFtdcLoginForbiddenIPField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcLoginForbiddenIPField

# Register CThostFtdcLoginForbiddenIPField in _thosttraderapi:
_thosttraderapi.CThostFtdcLoginForbiddenIPField_swigregister(CThostFtdcLoginForbiddenIPField)

class CThostFtdcIPListField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    IPAddress = property(_thosttraderapi.CThostFtdcIPListField_IPAddress_get, _thosttraderapi.CThostFtdcIPListField_IPAddress_set)
    IsWhite = property(_thosttraderapi.CThostFtdcIPListField_IsWhite_get, _thosttraderapi.CThostFtdcIPListField_IsWhite_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcIPListField_swiginit(self, _thosttraderapi.new_CThostFtdcIPListField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcIPListField

# Register CThostFtdcIPListField in _thosttraderapi:
_thosttraderapi.CThostFtdcIPListField_swigregister(CThostFtdcIPListField)

class CThostFtdcInputOptionSelfCloseField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thosttraderapi.CThostFtdcInputOptionSelfCloseField_BrokerID_get, _thosttraderapi.CThostFtdcInputOptionSelfCloseField_BrokerID_set)
    InvestorID = property(_thosttraderapi.CThostFtdcInputOptionSelfCloseField_InvestorID_get, _thosttraderapi.CThostFtdcInputOptionSelfCloseField_InvestorID_set)
    InstrumentID = property(_thosttraderapi.CThostFtdcInputOptionSelfCloseField_InstrumentID_get, _thosttraderapi.CThostFtdcInputOptionSelfCloseField_InstrumentID_set)
    OptionSelfCloseRef = property(_thosttraderapi.CThostFtdcInputOptionSelfCloseField_OptionSelfCloseRef_get, _thosttraderapi.CThostFtdcInputOptionSelfCloseField_OptionSelfCloseRef_set)
    UserID = property(_thosttraderapi.CThostFtdcInputOptionSelfCloseField_UserID_get, _thosttraderapi.CThostFtdcInputOptionSelfCloseField_UserID_set)
    Volume = property(_thosttraderapi.CThostFtdcInputOptionSelfCloseField_Volume_get, _thosttraderapi.CThostFtdcInputOptionSelfCloseField_Volume_set)
    RequestID = property(_thosttraderapi.CThostFtdcInputOptionSelfCloseField_RequestID_get, _thosttraderapi.CThostFtdcInputOptionSelfCloseField_RequestID_set)
    BusinessUnit = property(_thosttraderapi.CThostFtdcInputOptionSelfCloseField_BusinessUnit_get, _thosttraderapi.CThostFtdcInputOptionSelfCloseField_BusinessUnit_set)
    HedgeFlag = property(_thosttraderapi.CThostFtdcInputOptionSelfCloseField_HedgeFlag_get, _thosttraderapi.CThostFtdcInputOptionSelfCloseField_HedgeFlag_set)
    OptSelfCloseFlag = property(_thosttraderapi.CThostFtdcInputOptionSelfCloseField_OptSelfCloseFlag_get, _thosttraderapi.CThostFtdcInputOptionSelfCloseField_OptSelfCloseFlag_set)
    ExchangeID = property(_thosttraderapi.CThostFtdcInputOptionSelfCloseField_ExchangeID_get, _thosttraderapi.CThostFtdcInputOptionSelfCloseField_ExchangeID_set)
    InvestUnitID = property(_thosttraderapi.CThostFtdcInputOptionSelfCloseField_InvestUnitID_get, _thosttraderapi.CThostFtdcInputOptionSelfCloseField_InvestUnitID_set)
    AccountID = property(_thosttraderapi.CThostFtdcInputOptionSelfCloseField_AccountID_get, _thosttraderapi.CThostFtdcInputOptionSelfCloseField_AccountID_set)
    CurrencyID = property(_thosttraderapi.CThostFtdcInputOptionSelfCloseField_CurrencyID_get, _thosttraderapi.CThostFtdcInputOptionSelfCloseField_CurrencyID_set)
    ClientID = property(_thosttraderapi.CThostFtdcInputOptionSelfCloseField_ClientID_get, _thosttraderapi.CThostFtdcInputOptionSelfCloseField_ClientID_set)
    IPAddress = property(_thosttraderapi.CThostFtdcInputOptionSelfCloseField_IPAddress_get, _thosttraderapi.CThostFtdcInputOptionSelfCloseField_IPAddress_set)
    MacAddress = property(_thosttraderapi.CThostFtdcInputOptionSelfCloseField_MacAddress_get, _thosttraderapi.CThostFtdcInputOptionSelfCloseField_MacAddress_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcInputOptionSelfCloseField_swiginit(self, _thosttraderapi.new_CThostFtdcInputOptionSelfCloseField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcInputOptionSelfCloseField

# Register CThostFtdcInputOptionSelfCloseField in _thosttraderapi:
_thosttraderapi.CThostFtdcInputOptionSelfCloseField_swigregister(CThostFtdcInputOptionSelfCloseField)

class CThostFtdcInputOptionSelfCloseActionField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thosttraderapi.CThostFtdcInputOptionSelfCloseActionField_BrokerID_get, _thosttraderapi.CThostFtdcInputOptionSelfCloseActionField_BrokerID_set)
    InvestorID = property(_thosttraderapi.CThostFtdcInputOptionSelfCloseActionField_InvestorID_get, _thosttraderapi.CThostFtdcInputOptionSelfCloseActionField_InvestorID_set)
    OptionSelfCloseActionRef = property(_thosttraderapi.CThostFtdcInputOptionSelfCloseActionField_OptionSelfCloseActionRef_get, _thosttraderapi.CThostFtdcInputOptionSelfCloseActionField_OptionSelfCloseActionRef_set)
    OptionSelfCloseRef = property(_thosttraderapi.CThostFtdcInputOptionSelfCloseActionField_OptionSelfCloseRef_get, _thosttraderapi.CThostFtdcInputOptionSelfCloseActionField_OptionSelfCloseRef_set)
    RequestID = property(_thosttraderapi.CThostFtdcInputOptionSelfCloseActionField_RequestID_get, _thosttraderapi.CThostFtdcInputOptionSelfCloseActionField_RequestID_set)
    FrontID = property(_thosttraderapi.CThostFtdcInputOptionSelfCloseActionField_FrontID_get, _thosttraderapi.CThostFtdcInputOptionSelfCloseActionField_FrontID_set)
    SessionID = property(_thosttraderapi.CThostFtdcInputOptionSelfCloseActionField_SessionID_get, _thosttraderapi.CThostFtdcInputOptionSelfCloseActionField_SessionID_set)
    ExchangeID = property(_thosttraderapi.CThostFtdcInputOptionSelfCloseActionField_ExchangeID_get, _thosttraderapi.CThostFtdcInputOptionSelfCloseActionField_ExchangeID_set)
    OptionSelfCloseSysID = property(_thosttraderapi.CThostFtdcInputOptionSelfCloseActionField_OptionSelfCloseSysID_get, _thosttraderapi.CThostFtdcInputOptionSelfCloseActionField_OptionSelfCloseSysID_set)
    ActionFlag = property(_thosttraderapi.CThostFtdcInputOptionSelfCloseActionField_ActionFlag_get, _thosttraderapi.CThostFtdcInputOptionSelfCloseActionField_ActionFlag_set)
    UserID = property(_thosttraderapi.CThostFtdcInputOptionSelfCloseActionField_UserID_get, _thosttraderapi.CThostFtdcInputOptionSelfCloseActionField_UserID_set)
    InstrumentID = property(_thosttraderapi.CThostFtdcInputOptionSelfCloseActionField_InstrumentID_get, _thosttraderapi.CThostFtdcInputOptionSelfCloseActionField_InstrumentID_set)
    InvestUnitID = property(_thosttraderapi.CThostFtdcInputOptionSelfCloseActionField_InvestUnitID_get, _thosttraderapi.CThostFtdcInputOptionSelfCloseActionField_InvestUnitID_set)
    IPAddress = property(_thosttraderapi.CThostFtdcInputOptionSelfCloseActionField_IPAddress_get, _thosttraderapi.CThostFtdcInputOptionSelfCloseActionField_IPAddress_set)
    MacAddress = property(_thosttraderapi.CThostFtdcInputOptionSelfCloseActionField_MacAddress_get, _thosttraderapi.CThostFtdcInputOptionSelfCloseActionField_MacAddress_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcInputOptionSelfCloseActionField_swiginit(self, _thosttraderapi.new_CThostFtdcInputOptionSelfCloseActionField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcInputOptionSelfCloseActionField

# Register CThostFtdcInputOptionSelfCloseActionField in _thosttraderapi:
_thosttraderapi.CThostFtdcInputOptionSelfCloseActionField_swigregister(CThostFtdcInputOptionSelfCloseActionField)

class CThostFtdcOptionSelfCloseField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thosttraderapi.CThostFtdcOptionSelfCloseField_BrokerID_get, _thosttraderapi.CThostFtdcOptionSelfCloseField_BrokerID_set)
    InvestorID = property(_thosttraderapi.CThostFtdcOptionSelfCloseField_InvestorID_get, _thosttraderapi.CThostFtdcOptionSelfCloseField_InvestorID_set)
    InstrumentID = property(_thosttraderapi.CThostFtdcOptionSelfCloseField_InstrumentID_get, _thosttraderapi.CThostFtdcOptionSelfCloseField_InstrumentID_set)
    OptionSelfCloseRef = property(_thosttraderapi.CThostFtdcOptionSelfCloseField_OptionSelfCloseRef_get, _thosttraderapi.CThostFtdcOptionSelfCloseField_OptionSelfCloseRef_set)
    UserID = property(_thosttraderapi.CThostFtdcOptionSelfCloseField_UserID_get, _thosttraderapi.CThostFtdcOptionSelfCloseField_UserID_set)
    Volume = property(_thosttraderapi.CThostFtdcOptionSelfCloseField_Volume_get, _thosttraderapi.CThostFtdcOptionSelfCloseField_Volume_set)
    RequestID = property(_thosttraderapi.CThostFtdcOptionSelfCloseField_RequestID_get, _thosttraderapi.CThostFtdcOptionSelfCloseField_RequestID_set)
    BusinessUnit = property(_thosttraderapi.CThostFtdcOptionSelfCloseField_BusinessUnit_get, _thosttraderapi.CThostFtdcOptionSelfCloseField_BusinessUnit_set)
    HedgeFlag = property(_thosttraderapi.CThostFtdcOptionSelfCloseField_HedgeFlag_get, _thosttraderapi.CThostFtdcOptionSelfCloseField_HedgeFlag_set)
    OptSelfCloseFlag = property(_thosttraderapi.CThostFtdcOptionSelfCloseField_OptSelfCloseFlag_get, _thosttraderapi.CThostFtdcOptionSelfCloseField_OptSelfCloseFlag_set)
    OptionSelfCloseLocalID = property(_thosttraderapi.CThostFtdcOptionSelfCloseField_OptionSelfCloseLocalID_get, _thosttraderapi.CThostFtdcOptionSelfCloseField_OptionSelfCloseLocalID_set)
    ExchangeID = property(_thosttraderapi.CThostFtdcOptionSelfCloseField_ExchangeID_get, _thosttraderapi.CThostFtdcOptionSelfCloseField_ExchangeID_set)
    ParticipantID = property(_thosttraderapi.CThostFtdcOptionSelfCloseField_ParticipantID_get, _thosttraderapi.CThostFtdcOptionSelfCloseField_ParticipantID_set)
    ClientID = property(_thosttraderapi.CThostFtdcOptionSelfCloseField_ClientID_get, _thosttraderapi.CThostFtdcOptionSelfCloseField_ClientID_set)
    ExchangeInstID = property(_thosttraderapi.CThostFtdcOptionSelfCloseField_ExchangeInstID_get, _thosttraderapi.CThostFtdcOptionSelfCloseField_ExchangeInstID_set)
    TraderID = property(_thosttraderapi.CThostFtdcOptionSelfCloseField_TraderID_get, _thosttraderapi.CThostFtdcOptionSelfCloseField_TraderID_set)
    InstallID = property(_thosttraderapi.CThostFtdcOptionSelfCloseField_InstallID_get, _thosttraderapi.CThostFtdcOptionSelfCloseField_InstallID_set)
    OrderSubmitStatus = property(_thosttraderapi.CThostFtdcOptionSelfCloseField_OrderSubmitStatus_get, _thosttraderapi.CThostFtdcOptionSelfCloseField_OrderSubmitStatus_set)
    NotifySequence = property(_thosttraderapi.CThostFtdcOptionSelfCloseField_NotifySequence_get, _thosttraderapi.CThostFtdcOptionSelfCloseField_NotifySequence_set)
    TradingDay = property(_thosttraderapi.CThostFtdcOptionSelfCloseField_TradingDay_get, _thosttraderapi.CThostFtdcOptionSelfCloseField_TradingDay_set)
    SettlementID = property(_thosttraderapi.CThostFtdcOptionSelfCloseField_SettlementID_get, _thosttraderapi.CThostFtdcOptionSelfCloseField_SettlementID_set)
    OptionSelfCloseSysID = property(_thosttraderapi.CThostFtdcOptionSelfCloseField_OptionSelfCloseSysID_get, _thosttraderapi.CThostFtdcOptionSelfCloseField_OptionSelfCloseSysID_set)
    InsertDate = property(_thosttraderapi.CThostFtdcOptionSelfCloseField_InsertDate_get, _thosttraderapi.CThostFtdcOptionSelfCloseField_InsertDate_set)
    InsertTime = property(_thosttraderapi.CThostFtdcOptionSelfCloseField_InsertTime_get, _thosttraderapi.CThostFtdcOptionSelfCloseField_InsertTime_set)
    CancelTime = property(_thosttraderapi.CThostFtdcOptionSelfCloseField_CancelTime_get, _thosttraderapi.CThostFtdcOptionSelfCloseField_CancelTime_set)
    ExecResult = property(_thosttraderapi.CThostFtdcOptionSelfCloseField_ExecResult_get, _thosttraderapi.CThostFtdcOptionSelfCloseField_ExecResult_set)
    ClearingPartID = property(_thosttraderapi.CThostFtdcOptionSelfCloseField_ClearingPartID_get, _thosttraderapi.CThostFtdcOptionSelfCloseField_ClearingPartID_set)
    SequenceNo = property(_thosttraderapi.CThostFtdcOptionSelfCloseField_SequenceNo_get, _thosttraderapi.CThostFtdcOptionSelfCloseField_SequenceNo_set)
    FrontID = property(_thosttraderapi.CThostFtdcOptionSelfCloseField_FrontID_get, _thosttraderapi.CThostFtdcOptionSelfCloseField_FrontID_set)
    SessionID = property(_thosttraderapi.CThostFtdcOptionSelfCloseField_SessionID_get, _thosttraderapi.CThostFtdcOptionSelfCloseField_SessionID_set)
    UserProductInfo = property(_thosttraderapi.CThostFtdcOptionSelfCloseField_UserProductInfo_get, _thosttraderapi.CThostFtdcOptionSelfCloseField_UserProductInfo_set)
    StatusMsg = property(_thosttraderapi.CThostFtdcOptionSelfCloseField_StatusMsg_get, _thosttraderapi.CThostFtdcOptionSelfCloseField_StatusMsg_set)
    ActiveUserID = property(_thosttraderapi.CThostFtdcOptionSelfCloseField_ActiveUserID_get, _thosttraderapi.CThostFtdcOptionSelfCloseField_ActiveUserID_set)
    BrokerOptionSelfCloseSeq = property(_thosttraderapi.CThostFtdcOptionSelfCloseField_BrokerOptionSelfCloseSeq_get, _thosttraderapi.CThostFtdcOptionSelfCloseField_BrokerOptionSelfCloseSeq_set)
    BranchID = property(_thosttraderapi.CThostFtdcOptionSelfCloseField_BranchID_get, _thosttraderapi.CThostFtdcOptionSelfCloseField_BranchID_set)
    InvestUnitID = property(_thosttraderapi.CThostFtdcOptionSelfCloseField_InvestUnitID_get, _thosttraderapi.CThostFtdcOptionSelfCloseField_InvestUnitID_set)
    AccountID = property(_thosttraderapi.CThostFtdcOptionSelfCloseField_AccountID_get, _thosttraderapi.CThostFtdcOptionSelfCloseField_AccountID_set)
    CurrencyID = property(_thosttraderapi.CThostFtdcOptionSelfCloseField_CurrencyID_get, _thosttraderapi.CThostFtdcOptionSelfCloseField_CurrencyID_set)
    IPAddress = property(_thosttraderapi.CThostFtdcOptionSelfCloseField_IPAddress_get, _thosttraderapi.CThostFtdcOptionSelfCloseField_IPAddress_set)
    MacAddress = property(_thosttraderapi.CThostFtdcOptionSelfCloseField_MacAddress_get, _thosttraderapi.CThostFtdcOptionSelfCloseField_MacAddress_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcOptionSelfCloseField_swiginit(self, _thosttraderapi.new_CThostFtdcOptionSelfCloseField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcOptionSelfCloseField

# Register CThostFtdcOptionSelfCloseField in _thosttraderapi:
_thosttraderapi.CThostFtdcOptionSelfCloseField_swigregister(CThostFtdcOptionSelfCloseField)

class CThostFtdcOptionSelfCloseActionField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thosttraderapi.CThostFtdcOptionSelfCloseActionField_BrokerID_get, _thosttraderapi.CThostFtdcOptionSelfCloseActionField_BrokerID_set)
    InvestorID = property(_thosttraderapi.CThostFtdcOptionSelfCloseActionField_InvestorID_get, _thosttraderapi.CThostFtdcOptionSelfCloseActionField_InvestorID_set)
    OptionSelfCloseActionRef = property(_thosttraderapi.CThostFtdcOptionSelfCloseActionField_OptionSelfCloseActionRef_get, _thosttraderapi.CThostFtdcOptionSelfCloseActionField_OptionSelfCloseActionRef_set)
    OptionSelfCloseRef = property(_thosttraderapi.CThostFtdcOptionSelfCloseActionField_OptionSelfCloseRef_get, _thosttraderapi.CThostFtdcOptionSelfCloseActionField_OptionSelfCloseRef_set)
    RequestID = property(_thosttraderapi.CThostFtdcOptionSelfCloseActionField_RequestID_get, _thosttraderapi.CThostFtdcOptionSelfCloseActionField_RequestID_set)
    FrontID = property(_thosttraderapi.CThostFtdcOptionSelfCloseActionField_FrontID_get, _thosttraderapi.CThostFtdcOptionSelfCloseActionField_FrontID_set)
    SessionID = property(_thosttraderapi.CThostFtdcOptionSelfCloseActionField_SessionID_get, _thosttraderapi.CThostFtdcOptionSelfCloseActionField_SessionID_set)
    ExchangeID = property(_thosttraderapi.CThostFtdcOptionSelfCloseActionField_ExchangeID_get, _thosttraderapi.CThostFtdcOptionSelfCloseActionField_ExchangeID_set)
    OptionSelfCloseSysID = property(_thosttraderapi.CThostFtdcOptionSelfCloseActionField_OptionSelfCloseSysID_get, _thosttraderapi.CThostFtdcOptionSelfCloseActionField_OptionSelfCloseSysID_set)
    ActionFlag = property(_thosttraderapi.CThostFtdcOptionSelfCloseActionField_ActionFlag_get, _thosttraderapi.CThostFtdcOptionSelfCloseActionField_ActionFlag_set)
    ActionDate = property(_thosttraderapi.CThostFtdcOptionSelfCloseActionField_ActionDate_get, _thosttraderapi.CThostFtdcOptionSelfCloseActionField_ActionDate_set)
    ActionTime = property(_thosttraderapi.CThostFtdcOptionSelfCloseActionField_ActionTime_get, _thosttraderapi.CThostFtdcOptionSelfCloseActionField_ActionTime_set)
    TraderID = property(_thosttraderapi.CThostFtdcOptionSelfCloseActionField_TraderID_get, _thosttraderapi.CThostFtdcOptionSelfCloseActionField_TraderID_set)
    InstallID = property(_thosttraderapi.CThostFtdcOptionSelfCloseActionField_InstallID_get, _thosttraderapi.CThostFtdcOptionSelfCloseActionField_InstallID_set)
    OptionSelfCloseLocalID = property(_thosttraderapi.CThostFtdcOptionSelfCloseActionField_OptionSelfCloseLocalID_get, _thosttraderapi.CThostFtdcOptionSelfCloseActionField_OptionSelfCloseLocalID_set)
    ActionLocalID = property(_thosttraderapi.CThostFtdcOptionSelfCloseActionField_ActionLocalID_get, _thosttraderapi.CThostFtdcOptionSelfCloseActionField_ActionLocalID_set)
    ParticipantID = property(_thosttraderapi.CThostFtdcOptionSelfCloseActionField_ParticipantID_get, _thosttraderapi.CThostFtdcOptionSelfCloseActionField_ParticipantID_set)
    ClientID = property(_thosttraderapi.CThostFtdcOptionSelfCloseActionField_ClientID_get, _thosttraderapi.CThostFtdcOptionSelfCloseActionField_ClientID_set)
    BusinessUnit = property(_thosttraderapi.CThostFtdcOptionSelfCloseActionField_BusinessUnit_get, _thosttraderapi.CThostFtdcOptionSelfCloseActionField_BusinessUnit_set)
    OrderActionStatus = property(_thosttraderapi.CThostFtdcOptionSelfCloseActionField_OrderActionStatus_get, _thosttraderapi.CThostFtdcOptionSelfCloseActionField_OrderActionStatus_set)
    UserID = property(_thosttraderapi.CThostFtdcOptionSelfCloseActionField_UserID_get, _thosttraderapi.CThostFtdcOptionSelfCloseActionField_UserID_set)
    StatusMsg = property(_thosttraderapi.CThostFtdcOptionSelfCloseActionField_StatusMsg_get, _thosttraderapi.CThostFtdcOptionSelfCloseActionField_StatusMsg_set)
    InstrumentID = property(_thosttraderapi.CThostFtdcOptionSelfCloseActionField_InstrumentID_get, _thosttraderapi.CThostFtdcOptionSelfCloseActionField_InstrumentID_set)
    BranchID = property(_thosttraderapi.CThostFtdcOptionSelfCloseActionField_BranchID_get, _thosttraderapi.CThostFtdcOptionSelfCloseActionField_BranchID_set)
    InvestUnitID = property(_thosttraderapi.CThostFtdcOptionSelfCloseActionField_InvestUnitID_get, _thosttraderapi.CThostFtdcOptionSelfCloseActionField_InvestUnitID_set)
    IPAddress = property(_thosttraderapi.CThostFtdcOptionSelfCloseActionField_IPAddress_get, _thosttraderapi.CThostFtdcOptionSelfCloseActionField_IPAddress_set)
    MacAddress = property(_thosttraderapi.CThostFtdcOptionSelfCloseActionField_MacAddress_get, _thosttraderapi.CThostFtdcOptionSelfCloseActionField_MacAddress_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcOptionSelfCloseActionField_swiginit(self, _thosttraderapi.new_CThostFtdcOptionSelfCloseActionField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcOptionSelfCloseActionField

# Register CThostFtdcOptionSelfCloseActionField in _thosttraderapi:
_thosttraderapi.CThostFtdcOptionSelfCloseActionField_swigregister(CThostFtdcOptionSelfCloseActionField)

class CThostFtdcQryOptionSelfCloseField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thosttraderapi.CThostFtdcQryOptionSelfCloseField_BrokerID_get, _thosttraderapi.CThostFtdcQryOptionSelfCloseField_BrokerID_set)
    InvestorID = property(_thosttraderapi.CThostFtdcQryOptionSelfCloseField_InvestorID_get, _thosttraderapi.CThostFtdcQryOptionSelfCloseField_InvestorID_set)
    InstrumentID = property(_thosttraderapi.CThostFtdcQryOptionSelfCloseField_InstrumentID_get, _thosttraderapi.CThostFtdcQryOptionSelfCloseField_InstrumentID_set)
    ExchangeID = property(_thosttraderapi.CThostFtdcQryOptionSelfCloseField_ExchangeID_get, _thosttraderapi.CThostFtdcQryOptionSelfCloseField_ExchangeID_set)
    OptionSelfCloseSysID = property(_thosttraderapi.CThostFtdcQryOptionSelfCloseField_OptionSelfCloseSysID_get, _thosttraderapi.CThostFtdcQryOptionSelfCloseField_OptionSelfCloseSysID_set)
    InsertTimeStart = property(_thosttraderapi.CThostFtdcQryOptionSelfCloseField_InsertTimeStart_get, _thosttraderapi.CThostFtdcQryOptionSelfCloseField_InsertTimeStart_set)
    InsertTimeEnd = property(_thosttraderapi.CThostFtdcQryOptionSelfCloseField_InsertTimeEnd_get, _thosttraderapi.CThostFtdcQryOptionSelfCloseField_InsertTimeEnd_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcQryOptionSelfCloseField_swiginit(self, _thosttraderapi.new_CThostFtdcQryOptionSelfCloseField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcQryOptionSelfCloseField

# Register CThostFtdcQryOptionSelfCloseField in _thosttraderapi:
_thosttraderapi.CThostFtdcQryOptionSelfCloseField_swigregister(CThostFtdcQryOptionSelfCloseField)

class CThostFtdcExchangeOptionSelfCloseField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    Volume = property(_thosttraderapi.CThostFtdcExchangeOptionSelfCloseField_Volume_get, _thosttraderapi.CThostFtdcExchangeOptionSelfCloseField_Volume_set)
    RequestID = property(_thosttraderapi.CThostFtdcExchangeOptionSelfCloseField_RequestID_get, _thosttraderapi.CThostFtdcExchangeOptionSelfCloseField_RequestID_set)
    BusinessUnit = property(_thosttraderapi.CThostFtdcExchangeOptionSelfCloseField_BusinessUnit_get, _thosttraderapi.CThostFtdcExchangeOptionSelfCloseField_BusinessUnit_set)
    HedgeFlag = property(_thosttraderapi.CThostFtdcExchangeOptionSelfCloseField_HedgeFlag_get, _thosttraderapi.CThostFtdcExchangeOptionSelfCloseField_HedgeFlag_set)
    OptSelfCloseFlag = property(_thosttraderapi.CThostFtdcExchangeOptionSelfCloseField_OptSelfCloseFlag_get, _thosttraderapi.CThostFtdcExchangeOptionSelfCloseField_OptSelfCloseFlag_set)
    OptionSelfCloseLocalID = property(_thosttraderapi.CThostFtdcExchangeOptionSelfCloseField_OptionSelfCloseLocalID_get, _thosttraderapi.CThostFtdcExchangeOptionSelfCloseField_OptionSelfCloseLocalID_set)
    ExchangeID = property(_thosttraderapi.CThostFtdcExchangeOptionSelfCloseField_ExchangeID_get, _thosttraderapi.CThostFtdcExchangeOptionSelfCloseField_ExchangeID_set)
    ParticipantID = property(_thosttraderapi.CThostFtdcExchangeOptionSelfCloseField_ParticipantID_get, _thosttraderapi.CThostFtdcExchangeOptionSelfCloseField_ParticipantID_set)
    ClientID = property(_thosttraderapi.CThostFtdcExchangeOptionSelfCloseField_ClientID_get, _thosttraderapi.CThostFtdcExchangeOptionSelfCloseField_ClientID_set)
    ExchangeInstID = property(_thosttraderapi.CThostFtdcExchangeOptionSelfCloseField_ExchangeInstID_get, _thosttraderapi.CThostFtdcExchangeOptionSelfCloseField_ExchangeInstID_set)
    TraderID = property(_thosttraderapi.CThostFtdcExchangeOptionSelfCloseField_TraderID_get, _thosttraderapi.CThostFtdcExchangeOptionSelfCloseField_TraderID_set)
    InstallID = property(_thosttraderapi.CThostFtdcExchangeOptionSelfCloseField_InstallID_get, _thosttraderapi.CThostFtdcExchangeOptionSelfCloseField_InstallID_set)
    OrderSubmitStatus = property(_thosttraderapi.CThostFtdcExchangeOptionSelfCloseField_OrderSubmitStatus_get, _thosttraderapi.CThostFtdcExchangeOptionSelfCloseField_OrderSubmitStatus_set)
    NotifySequence = property(_thosttraderapi.CThostFtdcExchangeOptionSelfCloseField_NotifySequence_get, _thosttraderapi.CThostFtdcExchangeOptionSelfCloseField_NotifySequence_set)
    TradingDay = property(_thosttraderapi.CThostFtdcExchangeOptionSelfCloseField_TradingDay_get, _thosttraderapi.CThostFtdcExchangeOptionSelfCloseField_TradingDay_set)
    SettlementID = property(_thosttraderapi.CThostFtdcExchangeOptionSelfCloseField_SettlementID_get, _thosttraderapi.CThostFtdcExchangeOptionSelfCloseField_SettlementID_set)
    OptionSelfCloseSysID = property(_thosttraderapi.CThostFtdcExchangeOptionSelfCloseField_OptionSelfCloseSysID_get, _thosttraderapi.CThostFtdcExchangeOptionSelfCloseField_OptionSelfCloseSysID_set)
    InsertDate = property(_thosttraderapi.CThostFtdcExchangeOptionSelfCloseField_InsertDate_get, _thosttraderapi.CThostFtdcExchangeOptionSelfCloseField_InsertDate_set)
    InsertTime = property(_thosttraderapi.CThostFtdcExchangeOptionSelfCloseField_InsertTime_get, _thosttraderapi.CThostFtdcExchangeOptionSelfCloseField_InsertTime_set)
    CancelTime = property(_thosttraderapi.CThostFtdcExchangeOptionSelfCloseField_CancelTime_get, _thosttraderapi.CThostFtdcExchangeOptionSelfCloseField_CancelTime_set)
    ExecResult = property(_thosttraderapi.CThostFtdcExchangeOptionSelfCloseField_ExecResult_get, _thosttraderapi.CThostFtdcExchangeOptionSelfCloseField_ExecResult_set)
    ClearingPartID = property(_thosttraderapi.CThostFtdcExchangeOptionSelfCloseField_ClearingPartID_get, _thosttraderapi.CThostFtdcExchangeOptionSelfCloseField_ClearingPartID_set)
    SequenceNo = property(_thosttraderapi.CThostFtdcExchangeOptionSelfCloseField_SequenceNo_get, _thosttraderapi.CThostFtdcExchangeOptionSelfCloseField_SequenceNo_set)
    BranchID = property(_thosttraderapi.CThostFtdcExchangeOptionSelfCloseField_BranchID_get, _thosttraderapi.CThostFtdcExchangeOptionSelfCloseField_BranchID_set)
    IPAddress = property(_thosttraderapi.CThostFtdcExchangeOptionSelfCloseField_IPAddress_get, _thosttraderapi.CThostFtdcExchangeOptionSelfCloseField_IPAddress_set)
    MacAddress = property(_thosttraderapi.CThostFtdcExchangeOptionSelfCloseField_MacAddress_get, _thosttraderapi.CThostFtdcExchangeOptionSelfCloseField_MacAddress_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcExchangeOptionSelfCloseField_swiginit(self, _thosttraderapi.new_CThostFtdcExchangeOptionSelfCloseField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcExchangeOptionSelfCloseField

# Register CThostFtdcExchangeOptionSelfCloseField in _thosttraderapi:
_thosttraderapi.CThostFtdcExchangeOptionSelfCloseField_swigregister(CThostFtdcExchangeOptionSelfCloseField)

class CThostFtdcQryOptionSelfCloseActionField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thosttraderapi.CThostFtdcQryOptionSelfCloseActionField_BrokerID_get, _thosttraderapi.CThostFtdcQryOptionSelfCloseActionField_BrokerID_set)
    InvestorID = property(_thosttraderapi.CThostFtdcQryOptionSelfCloseActionField_InvestorID_get, _thosttraderapi.CThostFtdcQryOptionSelfCloseActionField_InvestorID_set)
    ExchangeID = property(_thosttraderapi.CThostFtdcQryOptionSelfCloseActionField_ExchangeID_get, _thosttraderapi.CThostFtdcQryOptionSelfCloseActionField_ExchangeID_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcQryOptionSelfCloseActionField_swiginit(self, _thosttraderapi.new_CThostFtdcQryOptionSelfCloseActionField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcQryOptionSelfCloseActionField

# Register CThostFtdcQryOptionSelfCloseActionField in _thosttraderapi:
_thosttraderapi.CThostFtdcQryOptionSelfCloseActionField_swigregister(CThostFtdcQryOptionSelfCloseActionField)

class CThostFtdcExchangeOptionSelfCloseActionField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    ExchangeID = property(_thosttraderapi.CThostFtdcExchangeOptionSelfCloseActionField_ExchangeID_get, _thosttraderapi.CThostFtdcExchangeOptionSelfCloseActionField_ExchangeID_set)
    OptionSelfCloseSysID = property(_thosttraderapi.CThostFtdcExchangeOptionSelfCloseActionField_OptionSelfCloseSysID_get, _thosttraderapi.CThostFtdcExchangeOptionSelfCloseActionField_OptionSelfCloseSysID_set)
    ActionFlag = property(_thosttraderapi.CThostFtdcExchangeOptionSelfCloseActionField_ActionFlag_get, _thosttraderapi.CThostFtdcExchangeOptionSelfCloseActionField_ActionFlag_set)
    ActionDate = property(_thosttraderapi.CThostFtdcExchangeOptionSelfCloseActionField_ActionDate_get, _thosttraderapi.CThostFtdcExchangeOptionSelfCloseActionField_ActionDate_set)
    ActionTime = property(_thosttraderapi.CThostFtdcExchangeOptionSelfCloseActionField_ActionTime_get, _thosttraderapi.CThostFtdcExchangeOptionSelfCloseActionField_ActionTime_set)
    TraderID = property(_thosttraderapi.CThostFtdcExchangeOptionSelfCloseActionField_TraderID_get, _thosttraderapi.CThostFtdcExchangeOptionSelfCloseActionField_TraderID_set)
    InstallID = property(_thosttraderapi.CThostFtdcExchangeOptionSelfCloseActionField_InstallID_get, _thosttraderapi.CThostFtdcExchangeOptionSelfCloseActionField_InstallID_set)
    OptionSelfCloseLocalID = property(_thosttraderapi.CThostFtdcExchangeOptionSelfCloseActionField_OptionSelfCloseLocalID_get, _thosttraderapi.CThostFtdcExchangeOptionSelfCloseActionField_OptionSelfCloseLocalID_set)
    ActionLocalID = property(_thosttraderapi.CThostFtdcExchangeOptionSelfCloseActionField_ActionLocalID_get, _thosttraderapi.CThostFtdcExchangeOptionSelfCloseActionField_ActionLocalID_set)
    ParticipantID = property(_thosttraderapi.CThostFtdcExchangeOptionSelfCloseActionField_ParticipantID_get, _thosttraderapi.CThostFtdcExchangeOptionSelfCloseActionField_ParticipantID_set)
    ClientID = property(_thosttraderapi.CThostFtdcExchangeOptionSelfCloseActionField_ClientID_get, _thosttraderapi.CThostFtdcExchangeOptionSelfCloseActionField_ClientID_set)
    BusinessUnit = property(_thosttraderapi.CThostFtdcExchangeOptionSelfCloseActionField_BusinessUnit_get, _thosttraderapi.CThostFtdcExchangeOptionSelfCloseActionField_BusinessUnit_set)
    OrderActionStatus = property(_thosttraderapi.CThostFtdcExchangeOptionSelfCloseActionField_OrderActionStatus_get, _thosttraderapi.CThostFtdcExchangeOptionSelfCloseActionField_OrderActionStatus_set)
    UserID = property(_thosttraderapi.CThostFtdcExchangeOptionSelfCloseActionField_UserID_get, _thosttraderapi.CThostFtdcExchangeOptionSelfCloseActionField_UserID_set)
    BranchID = property(_thosttraderapi.CThostFtdcExchangeOptionSelfCloseActionField_BranchID_get, _thosttraderapi.CThostFtdcExchangeOptionSelfCloseActionField_BranchID_set)
    IPAddress = property(_thosttraderapi.CThostFtdcExchangeOptionSelfCloseActionField_IPAddress_get, _thosttraderapi.CThostFtdcExchangeOptionSelfCloseActionField_IPAddress_set)
    MacAddress = property(_thosttraderapi.CThostFtdcExchangeOptionSelfCloseActionField_MacAddress_get, _thosttraderapi.CThostFtdcExchangeOptionSelfCloseActionField_MacAddress_set)
    ExchangeInstID = property(_thosttraderapi.CThostFtdcExchangeOptionSelfCloseActionField_ExchangeInstID_get, _thosttraderapi.CThostFtdcExchangeOptionSelfCloseActionField_ExchangeInstID_set)
    OptSelfCloseFlag = property(_thosttraderapi.CThostFtdcExchangeOptionSelfCloseActionField_OptSelfCloseFlag_get, _thosttraderapi.CThostFtdcExchangeOptionSelfCloseActionField_OptSelfCloseFlag_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcExchangeOptionSelfCloseActionField_swiginit(self, _thosttraderapi.new_CThostFtdcExchangeOptionSelfCloseActionField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcExchangeOptionSelfCloseActionField

# Register CThostFtdcExchangeOptionSelfCloseActionField in _thosttraderapi:
_thosttraderapi.CThostFtdcExchangeOptionSelfCloseActionField_swigregister(CThostFtdcExchangeOptionSelfCloseActionField)

class CThostFtdcSyncDelaySwapField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    DelaySwapSeqNo = property(_thosttraderapi.CThostFtdcSyncDelaySwapField_DelaySwapSeqNo_get, _thosttraderapi.CThostFtdcSyncDelaySwapField_DelaySwapSeqNo_set)
    BrokerID = property(_thosttraderapi.CThostFtdcSyncDelaySwapField_BrokerID_get, _thosttraderapi.CThostFtdcSyncDelaySwapField_BrokerID_set)
    InvestorID = property(_thosttraderapi.CThostFtdcSyncDelaySwapField_InvestorID_get, _thosttraderapi.CThostFtdcSyncDelaySwapField_InvestorID_set)
    FromCurrencyID = property(_thosttraderapi.CThostFtdcSyncDelaySwapField_FromCurrencyID_get, _thosttraderapi.CThostFtdcSyncDelaySwapField_FromCurrencyID_set)
    FromAmount = property(_thosttraderapi.CThostFtdcSyncDelaySwapField_FromAmount_get, _thosttraderapi.CThostFtdcSyncDelaySwapField_FromAmount_set)
    FromFrozenSwap = property(_thosttraderapi.CThostFtdcSyncDelaySwapField_FromFrozenSwap_get, _thosttraderapi.CThostFtdcSyncDelaySwapField_FromFrozenSwap_set)
    FromRemainSwap = property(_thosttraderapi.CThostFtdcSyncDelaySwapField_FromRemainSwap_get, _thosttraderapi.CThostFtdcSyncDelaySwapField_FromRemainSwap_set)
    ToCurrencyID = property(_thosttraderapi.CThostFtdcSyncDelaySwapField_ToCurrencyID_get, _thosttraderapi.CThostFtdcSyncDelaySwapField_ToCurrencyID_set)
    ToAmount = property(_thosttraderapi.CThostFtdcSyncDelaySwapField_ToAmount_get, _thosttraderapi.CThostFtdcSyncDelaySwapField_ToAmount_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcSyncDelaySwapField_swiginit(self, _thosttraderapi.new_CThostFtdcSyncDelaySwapField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcSyncDelaySwapField

# Register CThostFtdcSyncDelaySwapField in _thosttraderapi:
_thosttraderapi.CThostFtdcSyncDelaySwapField_swigregister(CThostFtdcSyncDelaySwapField)

class CThostFtdcQrySyncDelaySwapField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thosttraderapi.CThostFtdcQrySyncDelaySwapField_BrokerID_get, _thosttraderapi.CThostFtdcQrySyncDelaySwapField_BrokerID_set)
    DelaySwapSeqNo = property(_thosttraderapi.CThostFtdcQrySyncDelaySwapField_DelaySwapSeqNo_get, _thosttraderapi.CThostFtdcQrySyncDelaySwapField_DelaySwapSeqNo_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcQrySyncDelaySwapField_swiginit(self, _thosttraderapi.new_CThostFtdcQrySyncDelaySwapField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcQrySyncDelaySwapField

# Register CThostFtdcQrySyncDelaySwapField in _thosttraderapi:
_thosttraderapi.CThostFtdcQrySyncDelaySwapField_swigregister(CThostFtdcQrySyncDelaySwapField)

class CThostFtdcInvestUnitField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thosttraderapi.CThostFtdcInvestUnitField_BrokerID_get, _thosttraderapi.CThostFtdcInvestUnitField_BrokerID_set)
    InvestorID = property(_thosttraderapi.CThostFtdcInvestUnitField_InvestorID_get, _thosttraderapi.CThostFtdcInvestUnitField_InvestorID_set)
    InvestUnitID = property(_thosttraderapi.CThostFtdcInvestUnitField_InvestUnitID_get, _thosttraderapi.CThostFtdcInvestUnitField_InvestUnitID_set)
    InvestorUnitName = property(_thosttraderapi.CThostFtdcInvestUnitField_InvestorUnitName_get, _thosttraderapi.CThostFtdcInvestUnitField_InvestorUnitName_set)
    InvestorGroupID = property(_thosttraderapi.CThostFtdcInvestUnitField_InvestorGroupID_get, _thosttraderapi.CThostFtdcInvestUnitField_InvestorGroupID_set)
    CommModelID = property(_thosttraderapi.CThostFtdcInvestUnitField_CommModelID_get, _thosttraderapi.CThostFtdcInvestUnitField_CommModelID_set)
    MarginModelID = property(_thosttraderapi.CThostFtdcInvestUnitField_MarginModelID_get, _thosttraderapi.CThostFtdcInvestUnitField_MarginModelID_set)
    AccountID = property(_thosttraderapi.CThostFtdcInvestUnitField_AccountID_get, _thosttraderapi.CThostFtdcInvestUnitField_AccountID_set)
    CurrencyID = property(_thosttraderapi.CThostFtdcInvestUnitField_CurrencyID_get, _thosttraderapi.CThostFtdcInvestUnitField_CurrencyID_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcInvestUnitField_swiginit(self, _thosttraderapi.new_CThostFtdcInvestUnitField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcInvestUnitField

# Register CThostFtdcInvestUnitField in _thosttraderapi:
_thosttraderapi.CThostFtdcInvestUnitField_swigregister(CThostFtdcInvestUnitField)

class CThostFtdcQryInvestUnitField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thosttraderapi.CThostFtdcQryInvestUnitField_BrokerID_get, _thosttraderapi.CThostFtdcQryInvestUnitField_BrokerID_set)
    InvestorID = property(_thosttraderapi.CThostFtdcQryInvestUnitField_InvestorID_get, _thosttraderapi.CThostFtdcQryInvestUnitField_InvestorID_set)
    InvestUnitID = property(_thosttraderapi.CThostFtdcQryInvestUnitField_InvestUnitID_get, _thosttraderapi.CThostFtdcQryInvestUnitField_InvestUnitID_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcQryInvestUnitField_swiginit(self, _thosttraderapi.new_CThostFtdcQryInvestUnitField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcQryInvestUnitField

# Register CThostFtdcQryInvestUnitField in _thosttraderapi:
_thosttraderapi.CThostFtdcQryInvestUnitField_swigregister(CThostFtdcQryInvestUnitField)

class CThostFtdcSecAgentCheckModeField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    InvestorID = property(_thosttraderapi.CThostFtdcSecAgentCheckModeField_InvestorID_get, _thosttraderapi.CThostFtdcSecAgentCheckModeField_InvestorID_set)
    BrokerID = property(_thosttraderapi.CThostFtdcSecAgentCheckModeField_BrokerID_get, _thosttraderapi.CThostFtdcSecAgentCheckModeField_BrokerID_set)
    CurrencyID = property(_thosttraderapi.CThostFtdcSecAgentCheckModeField_CurrencyID_get, _thosttraderapi.CThostFtdcSecAgentCheckModeField_CurrencyID_set)
    BrokerSecAgentID = property(_thosttraderapi.CThostFtdcSecAgentCheckModeField_BrokerSecAgentID_get, _thosttraderapi.CThostFtdcSecAgentCheckModeField_BrokerSecAgentID_set)
    CheckSelfAccount = property(_thosttraderapi.CThostFtdcSecAgentCheckModeField_CheckSelfAccount_get, _thosttraderapi.CThostFtdcSecAgentCheckModeField_CheckSelfAccount_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcSecAgentCheckModeField_swiginit(self, _thosttraderapi.new_CThostFtdcSecAgentCheckModeField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcSecAgentCheckModeField

# Register CThostFtdcSecAgentCheckModeField in _thosttraderapi:
_thosttraderapi.CThostFtdcSecAgentCheckModeField_swigregister(CThostFtdcSecAgentCheckModeField)

class CThostFtdcSecAgentTradeInfoField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thosttraderapi.CThostFtdcSecAgentTradeInfoField_BrokerID_get, _thosttraderapi.CThostFtdcSecAgentTradeInfoField_BrokerID_set)
    BrokerSecAgentID = property(_thosttraderapi.CThostFtdcSecAgentTradeInfoField_BrokerSecAgentID_get, _thosttraderapi.CThostFtdcSecAgentTradeInfoField_BrokerSecAgentID_set)
    InvestorID = property(_thosttraderapi.CThostFtdcSecAgentTradeInfoField_InvestorID_get, _thosttraderapi.CThostFtdcSecAgentTradeInfoField_InvestorID_set)
    LongCustomerName = property(_thosttraderapi.CThostFtdcSecAgentTradeInfoField_LongCustomerName_get, _thosttraderapi.CThostFtdcSecAgentTradeInfoField_LongCustomerName_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcSecAgentTradeInfoField_swiginit(self, _thosttraderapi.new_CThostFtdcSecAgentTradeInfoField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcSecAgentTradeInfoField

# Register CThostFtdcSecAgentTradeInfoField in _thosttraderapi:
_thosttraderapi.CThostFtdcSecAgentTradeInfoField_swigregister(CThostFtdcSecAgentTradeInfoField)

class CThostFtdcMarketDataField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    TradingDay = property(_thosttraderapi.CThostFtdcMarketDataField_TradingDay_get, _thosttraderapi.CThostFtdcMarketDataField_TradingDay_set)
    InstrumentID = property(_thosttraderapi.CThostFtdcMarketDataField_InstrumentID_get, _thosttraderapi.CThostFtdcMarketDataField_InstrumentID_set)
    ExchangeID = property(_thosttraderapi.CThostFtdcMarketDataField_ExchangeID_get, _thosttraderapi.CThostFtdcMarketDataField_ExchangeID_set)
    ExchangeInstID = property(_thosttraderapi.CThostFtdcMarketDataField_ExchangeInstID_get, _thosttraderapi.CThostFtdcMarketDataField_ExchangeInstID_set)
    LastPrice = property(_thosttraderapi.CThostFtdcMarketDataField_LastPrice_get, _thosttraderapi.CThostFtdcMarketDataField_LastPrice_set)
    PreSettlementPrice = property(_thosttraderapi.CThostFtdcMarketDataField_PreSettlementPrice_get, _thosttraderapi.CThostFtdcMarketDataField_PreSettlementPrice_set)
    PreClosePrice = property(_thosttraderapi.CThostFtdcMarketDataField_PreClosePrice_get, _thosttraderapi.CThostFtdcMarketDataField_PreClosePrice_set)
    PreOpenInterest = property(_thosttraderapi.CThostFtdcMarketDataField_PreOpenInterest_get, _thosttraderapi.CThostFtdcMarketDataField_PreOpenInterest_set)
    OpenPrice = property(_thosttraderapi.CThostFtdcMarketDataField_OpenPrice_get, _thosttraderapi.CThostFtdcMarketDataField_OpenPrice_set)
    HighestPrice = property(_thosttraderapi.CThostFtdcMarketDataField_HighestPrice_get, _thosttraderapi.CThostFtdcMarketDataField_HighestPrice_set)
    LowestPrice = property(_thosttraderapi.CThostFtdcMarketDataField_LowestPrice_get, _thosttraderapi.CThostFtdcMarketDataField_LowestPrice_set)
    Volume = property(_thosttraderapi.CThostFtdcMarketDataField_Volume_get, _thosttraderapi.CThostFtdcMarketDataField_Volume_set)
    Turnover = property(_thosttraderapi.CThostFtdcMarketDataField_Turnover_get, _thosttraderapi.CThostFtdcMarketDataField_Turnover_set)
    OpenInterest = property(_thosttraderapi.CThostFtdcMarketDataField_OpenInterest_get, _thosttraderapi.CThostFtdcMarketDataField_OpenInterest_set)
    ClosePrice = property(_thosttraderapi.CThostFtdcMarketDataField_ClosePrice_get, _thosttraderapi.CThostFtdcMarketDataField_ClosePrice_set)
    SettlementPrice = property(_thosttraderapi.CThostFtdcMarketDataField_SettlementPrice_get, _thosttraderapi.CThostFtdcMarketDataField_SettlementPrice_set)
    UpperLimitPrice = property(_thosttraderapi.CThostFtdcMarketDataField_UpperLimitPrice_get, _thosttraderapi.CThostFtdcMarketDataField_UpperLimitPrice_set)
    LowerLimitPrice = property(_thosttraderapi.CThostFtdcMarketDataField_LowerLimitPrice_get, _thosttraderapi.CThostFtdcMarketDataField_LowerLimitPrice_set)
    PreDelta = property(_thosttraderapi.CThostFtdcMarketDataField_PreDelta_get, _thosttraderapi.CThostFtdcMarketDataField_PreDelta_set)
    CurrDelta = property(_thosttraderapi.CThostFtdcMarketDataField_CurrDelta_get, _thosttraderapi.CThostFtdcMarketDataField_CurrDelta_set)
    UpdateTime = property(_thosttraderapi.CThostFtdcMarketDataField_UpdateTime_get, _thosttraderapi.CThostFtdcMarketDataField_UpdateTime_set)
    UpdateMillisec = property(_thosttraderapi.CThostFtdcMarketDataField_UpdateMillisec_get, _thosttraderapi.CThostFtdcMarketDataField_UpdateMillisec_set)
    ActionDay = property(_thosttraderapi.CThostFtdcMarketDataField_ActionDay_get, _thosttraderapi.CThostFtdcMarketDataField_ActionDay_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcMarketDataField_swiginit(self, _thosttraderapi.new_CThostFtdcMarketDataField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcMarketDataField

# Register CThostFtdcMarketDataField in _thosttraderapi:
_thosttraderapi.CThostFtdcMarketDataField_swigregister(CThostFtdcMarketDataField)

class CThostFtdcMarketDataBaseField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    TradingDay = property(_thosttraderapi.CThostFtdcMarketDataBaseField_TradingDay_get, _thosttraderapi.CThostFtdcMarketDataBaseField_TradingDay_set)
    PreSettlementPrice = property(_thosttraderapi.CThostFtdcMarketDataBaseField_PreSettlementPrice_get, _thosttraderapi.CThostFtdcMarketDataBaseField_PreSettlementPrice_set)
    PreClosePrice = property(_thosttraderapi.CThostFtdcMarketDataBaseField_PreClosePrice_get, _thosttraderapi.CThostFtdcMarketDataBaseField_PreClosePrice_set)
    PreOpenInterest = property(_thosttraderapi.CThostFtdcMarketDataBaseField_PreOpenInterest_get, _thosttraderapi.CThostFtdcMarketDataBaseField_PreOpenInterest_set)
    PreDelta = property(_thosttraderapi.CThostFtdcMarketDataBaseField_PreDelta_get, _thosttraderapi.CThostFtdcMarketDataBaseField_PreDelta_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcMarketDataBaseField_swiginit(self, _thosttraderapi.new_CThostFtdcMarketDataBaseField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcMarketDataBaseField

# Register CThostFtdcMarketDataBaseField in _thosttraderapi:
_thosttraderapi.CThostFtdcMarketDataBaseField_swigregister(CThostFtdcMarketDataBaseField)

class CThostFtdcMarketDataStaticField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    OpenPrice = property(_thosttraderapi.CThostFtdcMarketDataStaticField_OpenPrice_get, _thosttraderapi.CThostFtdcMarketDataStaticField_OpenPrice_set)
    HighestPrice = property(_thosttraderapi.CThostFtdcMarketDataStaticField_HighestPrice_get, _thosttraderapi.CThostFtdcMarketDataStaticField_HighestPrice_set)
    LowestPrice = property(_thosttraderapi.CThostFtdcMarketDataStaticField_LowestPrice_get, _thosttraderapi.CThostFtdcMarketDataStaticField_LowestPrice_set)
    ClosePrice = property(_thosttraderapi.CThostFtdcMarketDataStaticField_ClosePrice_get, _thosttraderapi.CThostFtdcMarketDataStaticField_ClosePrice_set)
    UpperLimitPrice = property(_thosttraderapi.CThostFtdcMarketDataStaticField_UpperLimitPrice_get, _thosttraderapi.CThostFtdcMarketDataStaticField_UpperLimitPrice_set)
    LowerLimitPrice = property(_thosttraderapi.CThostFtdcMarketDataStaticField_LowerLimitPrice_get, _thosttraderapi.CThostFtdcMarketDataStaticField_LowerLimitPrice_set)
    SettlementPrice = property(_thosttraderapi.CThostFtdcMarketDataStaticField_SettlementPrice_get, _thosttraderapi.CThostFtdcMarketDataStaticField_SettlementPrice_set)
    CurrDelta = property(_thosttraderapi.CThostFtdcMarketDataStaticField_CurrDelta_get, _thosttraderapi.CThostFtdcMarketDataStaticField_CurrDelta_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcMarketDataStaticField_swiginit(self, _thosttraderapi.new_CThostFtdcMarketDataStaticField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcMarketDataStaticField

# Register CThostFtdcMarketDataStaticField in _thosttraderapi:
_thosttraderapi.CThostFtdcMarketDataStaticField_swigregister(CThostFtdcMarketDataStaticField)

class CThostFtdcMarketDataLastMatchField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    LastPrice = property(_thosttraderapi.CThostFtdcMarketDataLastMatchField_LastPrice_get, _thosttraderapi.CThostFtdcMarketDataLastMatchField_LastPrice_set)
    Volume = property(_thosttraderapi.CThostFtdcMarketDataLastMatchField_Volume_get, _thosttraderapi.CThostFtdcMarketDataLastMatchField_Volume_set)
    Turnover = property(_thosttraderapi.CThostFtdcMarketDataLastMatchField_Turnover_get, _thosttraderapi.CThostFtdcMarketDataLastMatchField_Turnover_set)
    OpenInterest = property(_thosttraderapi.CThostFtdcMarketDataLastMatchField_OpenInterest_get, _thosttraderapi.CThostFtdcMarketDataLastMatchField_OpenInterest_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcMarketDataLastMatchField_swiginit(self, _thosttraderapi.new_CThostFtdcMarketDataLastMatchField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcMarketDataLastMatchField

# Register CThostFtdcMarketDataLastMatchField in _thosttraderapi:
_thosttraderapi.CThostFtdcMarketDataLastMatchField_swigregister(CThostFtdcMarketDataLastMatchField)

class CThostFtdcMarketDataBestPriceField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BidPrice1 = property(_thosttraderapi.CThostFtdcMarketDataBestPriceField_BidPrice1_get, _thosttraderapi.CThostFtdcMarketDataBestPriceField_BidPrice1_set)
    BidVolume1 = property(_thosttraderapi.CThostFtdcMarketDataBestPriceField_BidVolume1_get, _thosttraderapi.CThostFtdcMarketDataBestPriceField_BidVolume1_set)
    AskPrice1 = property(_thosttraderapi.CThostFtdcMarketDataBestPriceField_AskPrice1_get, _thosttraderapi.CThostFtdcMarketDataBestPriceField_AskPrice1_set)
    AskVolume1 = property(_thosttraderapi.CThostFtdcMarketDataBestPriceField_AskVolume1_get, _thosttraderapi.CThostFtdcMarketDataBestPriceField_AskVolume1_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcMarketDataBestPriceField_swiginit(self, _thosttraderapi.new_CThostFtdcMarketDataBestPriceField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcMarketDataBestPriceField

# Register CThostFtdcMarketDataBestPriceField in _thosttraderapi:
_thosttraderapi.CThostFtdcMarketDataBestPriceField_swigregister(CThostFtdcMarketDataBestPriceField)

class CThostFtdcMarketDataBid23Field(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BidPrice2 = property(_thosttraderapi.CThostFtdcMarketDataBid23Field_BidPrice2_get, _thosttraderapi.CThostFtdcMarketDataBid23Field_BidPrice2_set)
    BidVolume2 = property(_thosttraderapi.CThostFtdcMarketDataBid23Field_BidVolume2_get, _thosttraderapi.CThostFtdcMarketDataBid23Field_BidVolume2_set)
    BidPrice3 = property(_thosttraderapi.CThostFtdcMarketDataBid23Field_BidPrice3_get, _thosttraderapi.CThostFtdcMarketDataBid23Field_BidPrice3_set)
    BidVolume3 = property(_thosttraderapi.CThostFtdcMarketDataBid23Field_BidVolume3_get, _thosttraderapi.CThostFtdcMarketDataBid23Field_BidVolume3_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcMarketDataBid23Field_swiginit(self, _thosttraderapi.new_CThostFtdcMarketDataBid23Field())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcMarketDataBid23Field

# Register CThostFtdcMarketDataBid23Field in _thosttraderapi:
_thosttraderapi.CThostFtdcMarketDataBid23Field_swigregister(CThostFtdcMarketDataBid23Field)

class CThostFtdcMarketDataAsk23Field(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    AskPrice2 = property(_thosttraderapi.CThostFtdcMarketDataAsk23Field_AskPrice2_get, _thosttraderapi.CThostFtdcMarketDataAsk23Field_AskPrice2_set)
    AskVolume2 = property(_thosttraderapi.CThostFtdcMarketDataAsk23Field_AskVolume2_get, _thosttraderapi.CThostFtdcMarketDataAsk23Field_AskVolume2_set)
    AskPrice3 = property(_thosttraderapi.CThostFtdcMarketDataAsk23Field_AskPrice3_get, _thosttraderapi.CThostFtdcMarketDataAsk23Field_AskPrice3_set)
    AskVolume3 = property(_thosttraderapi.CThostFtdcMarketDataAsk23Field_AskVolume3_get, _thosttraderapi.CThostFtdcMarketDataAsk23Field_AskVolume3_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcMarketDataAsk23Field_swiginit(self, _thosttraderapi.new_CThostFtdcMarketDataAsk23Field())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcMarketDataAsk23Field

# Register CThostFtdcMarketDataAsk23Field in _thosttraderapi:
_thosttraderapi.CThostFtdcMarketDataAsk23Field_swigregister(CThostFtdcMarketDataAsk23Field)

class CThostFtdcMarketDataBid45Field(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BidPrice4 = property(_thosttraderapi.CThostFtdcMarketDataBid45Field_BidPrice4_get, _thosttraderapi.CThostFtdcMarketDataBid45Field_BidPrice4_set)
    BidVolume4 = property(_thosttraderapi.CThostFtdcMarketDataBid45Field_BidVolume4_get, _thosttraderapi.CThostFtdcMarketDataBid45Field_BidVolume4_set)
    BidPrice5 = property(_thosttraderapi.CThostFtdcMarketDataBid45Field_BidPrice5_get, _thosttraderapi.CThostFtdcMarketDataBid45Field_BidPrice5_set)
    BidVolume5 = property(_thosttraderapi.CThostFtdcMarketDataBid45Field_BidVolume5_get, _thosttraderapi.CThostFtdcMarketDataBid45Field_BidVolume5_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcMarketDataBid45Field_swiginit(self, _thosttraderapi.new_CThostFtdcMarketDataBid45Field())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcMarketDataBid45Field

# Register CThostFtdcMarketDataBid45Field in _thosttraderapi:
_thosttraderapi.CThostFtdcMarketDataBid45Field_swigregister(CThostFtdcMarketDataBid45Field)

class CThostFtdcMarketDataAsk45Field(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    AskPrice4 = property(_thosttraderapi.CThostFtdcMarketDataAsk45Field_AskPrice4_get, _thosttraderapi.CThostFtdcMarketDataAsk45Field_AskPrice4_set)
    AskVolume4 = property(_thosttraderapi.CThostFtdcMarketDataAsk45Field_AskVolume4_get, _thosttraderapi.CThostFtdcMarketDataAsk45Field_AskVolume4_set)
    AskPrice5 = property(_thosttraderapi.CThostFtdcMarketDataAsk45Field_AskPrice5_get, _thosttraderapi.CThostFtdcMarketDataAsk45Field_AskPrice5_set)
    AskVolume5 = property(_thosttraderapi.CThostFtdcMarketDataAsk45Field_AskVolume5_get, _thosttraderapi.CThostFtdcMarketDataAsk45Field_AskVolume5_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcMarketDataAsk45Field_swiginit(self, _thosttraderapi.new_CThostFtdcMarketDataAsk45Field())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcMarketDataAsk45Field

# Register CThostFtdcMarketDataAsk45Field in _thosttraderapi:
_thosttraderapi.CThostFtdcMarketDataAsk45Field_swigregister(CThostFtdcMarketDataAsk45Field)

class CThostFtdcMarketDataUpdateTimeField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    InstrumentID = property(_thosttraderapi.CThostFtdcMarketDataUpdateTimeField_InstrumentID_get, _thosttraderapi.CThostFtdcMarketDataUpdateTimeField_InstrumentID_set)
    UpdateTime = property(_thosttraderapi.CThostFtdcMarketDataUpdateTimeField_UpdateTime_get, _thosttraderapi.CThostFtdcMarketDataUpdateTimeField_UpdateTime_set)
    UpdateMillisec = property(_thosttraderapi.CThostFtdcMarketDataUpdateTimeField_UpdateMillisec_get, _thosttraderapi.CThostFtdcMarketDataUpdateTimeField_UpdateMillisec_set)
    ActionDay = property(_thosttraderapi.CThostFtdcMarketDataUpdateTimeField_ActionDay_get, _thosttraderapi.CThostFtdcMarketDataUpdateTimeField_ActionDay_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcMarketDataUpdateTimeField_swiginit(self, _thosttraderapi.new_CThostFtdcMarketDataUpdateTimeField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcMarketDataUpdateTimeField

# Register CThostFtdcMarketDataUpdateTimeField in _thosttraderapi:
_thosttraderapi.CThostFtdcMarketDataUpdateTimeField_swigregister(CThostFtdcMarketDataUpdateTimeField)

class CThostFtdcMarketDataExchangeField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    ExchangeID = property(_thosttraderapi.CThostFtdcMarketDataExchangeField_ExchangeID_get, _thosttraderapi.CThostFtdcMarketDataExchangeField_ExchangeID_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcMarketDataExchangeField_swiginit(self, _thosttraderapi.new_CThostFtdcMarketDataExchangeField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcMarketDataExchangeField

# Register CThostFtdcMarketDataExchangeField in _thosttraderapi:
_thosttraderapi.CThostFtdcMarketDataExchangeField_swigregister(CThostFtdcMarketDataExchangeField)

class CThostFtdcSpecificInstrumentField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    InstrumentID = property(_thosttraderapi.CThostFtdcSpecificInstrumentField_InstrumentID_get, _thosttraderapi.CThostFtdcSpecificInstrumentField_InstrumentID_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcSpecificInstrumentField_swiginit(self, _thosttraderapi.new_CThostFtdcSpecificInstrumentField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcSpecificInstrumentField

# Register CThostFtdcSpecificInstrumentField in _thosttraderapi:
_thosttraderapi.CThostFtdcSpecificInstrumentField_swigregister(CThostFtdcSpecificInstrumentField)

class CThostFtdcInstrumentStatusField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    ExchangeID = property(_thosttraderapi.CThostFtdcInstrumentStatusField_ExchangeID_get, _thosttraderapi.CThostFtdcInstrumentStatusField_ExchangeID_set)
    ExchangeInstID = property(_thosttraderapi.CThostFtdcInstrumentStatusField_ExchangeInstID_get, _thosttraderapi.CThostFtdcInstrumentStatusField_ExchangeInstID_set)
    SettlementGroupID = property(_thosttraderapi.CThostFtdcInstrumentStatusField_SettlementGroupID_get, _thosttraderapi.CThostFtdcInstrumentStatusField_SettlementGroupID_set)
    InstrumentID = property(_thosttraderapi.CThostFtdcInstrumentStatusField_InstrumentID_get, _thosttraderapi.CThostFtdcInstrumentStatusField_InstrumentID_set)
    InstrumentStatus = property(_thosttraderapi.CThostFtdcInstrumentStatusField_InstrumentStatus_get, _thosttraderapi.CThostFtdcInstrumentStatusField_InstrumentStatus_set)
    TradingSegmentSN = property(_thosttraderapi.CThostFtdcInstrumentStatusField_TradingSegmentSN_get, _thosttraderapi.CThostFtdcInstrumentStatusField_TradingSegmentSN_set)
    EnterTime = property(_thosttraderapi.CThostFtdcInstrumentStatusField_EnterTime_get, _thosttraderapi.CThostFtdcInstrumentStatusField_EnterTime_set)
    EnterReason = property(_thosttraderapi.CThostFtdcInstrumentStatusField_EnterReason_get, _thosttraderapi.CThostFtdcInstrumentStatusField_EnterReason_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcInstrumentStatusField_swiginit(self, _thosttraderapi.new_CThostFtdcInstrumentStatusField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcInstrumentStatusField

# Register CThostFtdcInstrumentStatusField in _thosttraderapi:
_thosttraderapi.CThostFtdcInstrumentStatusField_swigregister(CThostFtdcInstrumentStatusField)

class CThostFtdcQryInstrumentStatusField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    ExchangeID = property(_thosttraderapi.CThostFtdcQryInstrumentStatusField_ExchangeID_get, _thosttraderapi.CThostFtdcQryInstrumentStatusField_ExchangeID_set)
    ExchangeInstID = property(_thosttraderapi.CThostFtdcQryInstrumentStatusField_ExchangeInstID_get, _thosttraderapi.CThostFtdcQryInstrumentStatusField_ExchangeInstID_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcQryInstrumentStatusField_swiginit(self, _thosttraderapi.new_CThostFtdcQryInstrumentStatusField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcQryInstrumentStatusField

# Register CThostFtdcQryInstrumentStatusField in _thosttraderapi:
_thosttraderapi.CThostFtdcQryInstrumentStatusField_swigregister(CThostFtdcQryInstrumentStatusField)

class CThostFtdcInvestorAccountField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thosttraderapi.CThostFtdcInvestorAccountField_BrokerID_get, _thosttraderapi.CThostFtdcInvestorAccountField_BrokerID_set)
    InvestorID = property(_thosttraderapi.CThostFtdcInvestorAccountField_InvestorID_get, _thosttraderapi.CThostFtdcInvestorAccountField_InvestorID_set)
    AccountID = property(_thosttraderapi.CThostFtdcInvestorAccountField_AccountID_get, _thosttraderapi.CThostFtdcInvestorAccountField_AccountID_set)
    CurrencyID = property(_thosttraderapi.CThostFtdcInvestorAccountField_CurrencyID_get, _thosttraderapi.CThostFtdcInvestorAccountField_CurrencyID_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcInvestorAccountField_swiginit(self, _thosttraderapi.new_CThostFtdcInvestorAccountField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcInvestorAccountField

# Register CThostFtdcInvestorAccountField in _thosttraderapi:
_thosttraderapi.CThostFtdcInvestorAccountField_swigregister(CThostFtdcInvestorAccountField)

class CThostFtdcPositionProfitAlgorithmField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thosttraderapi.CThostFtdcPositionProfitAlgorithmField_BrokerID_get, _thosttraderapi.CThostFtdcPositionProfitAlgorithmField_BrokerID_set)
    AccountID = property(_thosttraderapi.CThostFtdcPositionProfitAlgorithmField_AccountID_get, _thosttraderapi.CThostFtdcPositionProfitAlgorithmField_AccountID_set)
    Algorithm = property(_thosttraderapi.CThostFtdcPositionProfitAlgorithmField_Algorithm_get, _thosttraderapi.CThostFtdcPositionProfitAlgorithmField_Algorithm_set)
    Memo = property(_thosttraderapi.CThostFtdcPositionProfitAlgorithmField_Memo_get, _thosttraderapi.CThostFtdcPositionProfitAlgorithmField_Memo_set)
    CurrencyID = property(_thosttraderapi.CThostFtdcPositionProfitAlgorithmField_CurrencyID_get, _thosttraderapi.CThostFtdcPositionProfitAlgorithmField_CurrencyID_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcPositionProfitAlgorithmField_swiginit(self, _thosttraderapi.new_CThostFtdcPositionProfitAlgorithmField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcPositionProfitAlgorithmField

# Register CThostFtdcPositionProfitAlgorithmField in _thosttraderapi:
_thosttraderapi.CThostFtdcPositionProfitAlgorithmField_swigregister(CThostFtdcPositionProfitAlgorithmField)

class CThostFtdcDiscountField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thosttraderapi.CThostFtdcDiscountField_BrokerID_get, _thosttraderapi.CThostFtdcDiscountField_BrokerID_set)
    InvestorRange = property(_thosttraderapi.CThostFtdcDiscountField_InvestorRange_get, _thosttraderapi.CThostFtdcDiscountField_InvestorRange_set)
    InvestorID = property(_thosttraderapi.CThostFtdcDiscountField_InvestorID_get, _thosttraderapi.CThostFtdcDiscountField_InvestorID_set)
    Discount = property(_thosttraderapi.CThostFtdcDiscountField_Discount_get, _thosttraderapi.CThostFtdcDiscountField_Discount_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcDiscountField_swiginit(self, _thosttraderapi.new_CThostFtdcDiscountField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcDiscountField

# Register CThostFtdcDiscountField in _thosttraderapi:
_thosttraderapi.CThostFtdcDiscountField_swigregister(CThostFtdcDiscountField)

class CThostFtdcQryTransferBankField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BankID = property(_thosttraderapi.CThostFtdcQryTransferBankField_BankID_get, _thosttraderapi.CThostFtdcQryTransferBankField_BankID_set)
    BankBrchID = property(_thosttraderapi.CThostFtdcQryTransferBankField_BankBrchID_get, _thosttraderapi.CThostFtdcQryTransferBankField_BankBrchID_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcQryTransferBankField_swiginit(self, _thosttraderapi.new_CThostFtdcQryTransferBankField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcQryTransferBankField

# Register CThostFtdcQryTransferBankField in _thosttraderapi:
_thosttraderapi.CThostFtdcQryTransferBankField_swigregister(CThostFtdcQryTransferBankField)

class CThostFtdcTransferBankField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BankID = property(_thosttraderapi.CThostFtdcTransferBankField_BankID_get, _thosttraderapi.CThostFtdcTransferBankField_BankID_set)
    BankBrchID = property(_thosttraderapi.CThostFtdcTransferBankField_BankBrchID_get, _thosttraderapi.CThostFtdcTransferBankField_BankBrchID_set)
    BankName = property(_thosttraderapi.CThostFtdcTransferBankField_BankName_get, _thosttraderapi.CThostFtdcTransferBankField_BankName_set)
    IsActive = property(_thosttraderapi.CThostFtdcTransferBankField_IsActive_get, _thosttraderapi.CThostFtdcTransferBankField_IsActive_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcTransferBankField_swiginit(self, _thosttraderapi.new_CThostFtdcTransferBankField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcTransferBankField

# Register CThostFtdcTransferBankField in _thosttraderapi:
_thosttraderapi.CThostFtdcTransferBankField_swigregister(CThostFtdcTransferBankField)

class CThostFtdcQryInvestorPositionDetailField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thosttraderapi.CThostFtdcQryInvestorPositionDetailField_BrokerID_get, _thosttraderapi.CThostFtdcQryInvestorPositionDetailField_BrokerID_set)
    InvestorID = property(_thosttraderapi.CThostFtdcQryInvestorPositionDetailField_InvestorID_get, _thosttraderapi.CThostFtdcQryInvestorPositionDetailField_InvestorID_set)
    InstrumentID = property(_thosttraderapi.CThostFtdcQryInvestorPositionDetailField_InstrumentID_get, _thosttraderapi.CThostFtdcQryInvestorPositionDetailField_InstrumentID_set)
    ExchangeID = property(_thosttraderapi.CThostFtdcQryInvestorPositionDetailField_ExchangeID_get, _thosttraderapi.CThostFtdcQryInvestorPositionDetailField_ExchangeID_set)
    InvestUnitID = property(_thosttraderapi.CThostFtdcQryInvestorPositionDetailField_InvestUnitID_get, _thosttraderapi.CThostFtdcQryInvestorPositionDetailField_InvestUnitID_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcQryInvestorPositionDetailField_swiginit(self, _thosttraderapi.new_CThostFtdcQryInvestorPositionDetailField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcQryInvestorPositionDetailField

# Register CThostFtdcQryInvestorPositionDetailField in _thosttraderapi:
_thosttraderapi.CThostFtdcQryInvestorPositionDetailField_swigregister(CThostFtdcQryInvestorPositionDetailField)

class CThostFtdcInvestorPositionDetailField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    InstrumentID = property(_thosttraderapi.CThostFtdcInvestorPositionDetailField_InstrumentID_get, _thosttraderapi.CThostFtdcInvestorPositionDetailField_InstrumentID_set)
    BrokerID = property(_thosttraderapi.CThostFtdcInvestorPositionDetailField_BrokerID_get, _thosttraderapi.CThostFtdcInvestorPositionDetailField_BrokerID_set)
    InvestorID = property(_thosttraderapi.CThostFtdcInvestorPositionDetailField_InvestorID_get, _thosttraderapi.CThostFtdcInvestorPositionDetailField_InvestorID_set)
    HedgeFlag = property(_thosttraderapi.CThostFtdcInvestorPositionDetailField_HedgeFlag_get, _thosttraderapi.CThostFtdcInvestorPositionDetailField_HedgeFlag_set)
    Direction = property(_thosttraderapi.CThostFtdcInvestorPositionDetailField_Direction_get, _thosttraderapi.CThostFtdcInvestorPositionDetailField_Direction_set)
    OpenDate = property(_thosttraderapi.CThostFtdcInvestorPositionDetailField_OpenDate_get, _thosttraderapi.CThostFtdcInvestorPositionDetailField_OpenDate_set)
    TradeID = property(_thosttraderapi.CThostFtdcInvestorPositionDetailField_TradeID_get, _thosttraderapi.CThostFtdcInvestorPositionDetailField_TradeID_set)
    Volume = property(_thosttraderapi.CThostFtdcInvestorPositionDetailField_Volume_get, _thosttraderapi.CThostFtdcInvestorPositionDetailField_Volume_set)
    OpenPrice = property(_thosttraderapi.CThostFtdcInvestorPositionDetailField_OpenPrice_get, _thosttraderapi.CThostFtdcInvestorPositionDetailField_OpenPrice_set)
    TradingDay = property(_thosttraderapi.CThostFtdcInvestorPositionDetailField_TradingDay_get, _thosttraderapi.CThostFtdcInvestorPositionDetailField_TradingDay_set)
    SettlementID = property(_thosttraderapi.CThostFtdcInvestorPositionDetailField_SettlementID_get, _thosttraderapi.CThostFtdcInvestorPositionDetailField_SettlementID_set)
    TradeType = property(_thosttraderapi.CThostFtdcInvestorPositionDetailField_TradeType_get, _thosttraderapi.CThostFtdcInvestorPositionDetailField_TradeType_set)
    CombInstrumentID = property(_thosttraderapi.CThostFtdcInvestorPositionDetailField_CombInstrumentID_get, _thosttraderapi.CThostFtdcInvestorPositionDetailField_CombInstrumentID_set)
    ExchangeID = property(_thosttraderapi.CThostFtdcInvestorPositionDetailField_ExchangeID_get, _thosttraderapi.CThostFtdcInvestorPositionDetailField_ExchangeID_set)
    CloseProfitByDate = property(_thosttraderapi.CThostFtdcInvestorPositionDetailField_CloseProfitByDate_get, _thosttraderapi.CThostFtdcInvestorPositionDetailField_CloseProfitByDate_set)
    CloseProfitByTrade = property(_thosttraderapi.CThostFtdcInvestorPositionDetailField_CloseProfitByTrade_get, _thosttraderapi.CThostFtdcInvestorPositionDetailField_CloseProfitByTrade_set)
    PositionProfitByDate = property(_thosttraderapi.CThostFtdcInvestorPositionDetailField_PositionProfitByDate_get, _thosttraderapi.CThostFtdcInvestorPositionDetailField_PositionProfitByDate_set)
    PositionProfitByTrade = property(_thosttraderapi.CThostFtdcInvestorPositionDetailField_PositionProfitByTrade_get, _thosttraderapi.CThostFtdcInvestorPositionDetailField_PositionProfitByTrade_set)
    Margin = property(_thosttraderapi.CThostFtdcInvestorPositionDetailField_Margin_get, _thosttraderapi.CThostFtdcInvestorPositionDetailField_Margin_set)
    ExchMargin = property(_thosttraderapi.CThostFtdcInvestorPositionDetailField_ExchMargin_get, _thosttraderapi.CThostFtdcInvestorPositionDetailField_ExchMargin_set)
    MarginRateByMoney = property(_thosttraderapi.CThostFtdcInvestorPositionDetailField_MarginRateByMoney_get, _thosttraderapi.CThostFtdcInvestorPositionDetailField_MarginRateByMoney_set)
    MarginRateByVolume = property(_thosttraderapi.CThostFtdcInvestorPositionDetailField_MarginRateByVolume_get, _thosttraderapi.CThostFtdcInvestorPositionDetailField_MarginRateByVolume_set)
    LastSettlementPrice = property(_thosttraderapi.CThostFtdcInvestorPositionDetailField_LastSettlementPrice_get, _thosttraderapi.CThostFtdcInvestorPositionDetailField_LastSettlementPrice_set)
    SettlementPrice = property(_thosttraderapi.CThostFtdcInvestorPositionDetailField_SettlementPrice_get, _thosttraderapi.CThostFtdcInvestorPositionDetailField_SettlementPrice_set)
    CloseVolume = property(_thosttraderapi.CThostFtdcInvestorPositionDetailField_CloseVolume_get, _thosttraderapi.CThostFtdcInvestorPositionDetailField_CloseVolume_set)
    CloseAmount = property(_thosttraderapi.CThostFtdcInvestorPositionDetailField_CloseAmount_get, _thosttraderapi.CThostFtdcInvestorPositionDetailField_CloseAmount_set)
    TimeFirstVolume = property(_thosttraderapi.CThostFtdcInvestorPositionDetailField_TimeFirstVolume_get, _thosttraderapi.CThostFtdcInvestorPositionDetailField_TimeFirstVolume_set)
    InvestUnitID = property(_thosttraderapi.CThostFtdcInvestorPositionDetailField_InvestUnitID_get, _thosttraderapi.CThostFtdcInvestorPositionDetailField_InvestUnitID_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcInvestorPositionDetailField_swiginit(self, _thosttraderapi.new_CThostFtdcInvestorPositionDetailField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcInvestorPositionDetailField

# Register CThostFtdcInvestorPositionDetailField in _thosttraderapi:
_thosttraderapi.CThostFtdcInvestorPositionDetailField_swigregister(CThostFtdcInvestorPositionDetailField)

class CThostFtdcTradingAccountPasswordField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thosttraderapi.CThostFtdcTradingAccountPasswordField_BrokerID_get, _thosttraderapi.CThostFtdcTradingAccountPasswordField_BrokerID_set)
    AccountID = property(_thosttraderapi.CThostFtdcTradingAccountPasswordField_AccountID_get, _thosttraderapi.CThostFtdcTradingAccountPasswordField_AccountID_set)
    Password = property(_thosttraderapi.CThostFtdcTradingAccountPasswordField_Password_get, _thosttraderapi.CThostFtdcTradingAccountPasswordField_Password_set)
    CurrencyID = property(_thosttraderapi.CThostFtdcTradingAccountPasswordField_CurrencyID_get, _thosttraderapi.CThostFtdcTradingAccountPasswordField_CurrencyID_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcTradingAccountPasswordField_swiginit(self, _thosttraderapi.new_CThostFtdcTradingAccountPasswordField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcTradingAccountPasswordField

# Register CThostFtdcTradingAccountPasswordField in _thosttraderapi:
_thosttraderapi.CThostFtdcTradingAccountPasswordField_swigregister(CThostFtdcTradingAccountPasswordField)

class CThostFtdcMDTraderOfferField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    ExchangeID = property(_thosttraderapi.CThostFtdcMDTraderOfferField_ExchangeID_get, _thosttraderapi.CThostFtdcMDTraderOfferField_ExchangeID_set)
    TraderID = property(_thosttraderapi.CThostFtdcMDTraderOfferField_TraderID_get, _thosttraderapi.CThostFtdcMDTraderOfferField_TraderID_set)
    ParticipantID = property(_thosttraderapi.CThostFtdcMDTraderOfferField_ParticipantID_get, _thosttraderapi.CThostFtdcMDTraderOfferField_ParticipantID_set)
    Password = property(_thosttraderapi.CThostFtdcMDTraderOfferField_Password_get, _thosttraderapi.CThostFtdcMDTraderOfferField_Password_set)
    InstallID = property(_thosttraderapi.CThostFtdcMDTraderOfferField_InstallID_get, _thosttraderapi.CThostFtdcMDTraderOfferField_InstallID_set)
    OrderLocalID = property(_thosttraderapi.CThostFtdcMDTraderOfferField_OrderLocalID_get, _thosttraderapi.CThostFtdcMDTraderOfferField_OrderLocalID_set)
    TraderConnectStatus = property(_thosttraderapi.CThostFtdcMDTraderOfferField_TraderConnectStatus_get, _thosttraderapi.CThostFtdcMDTraderOfferField_TraderConnectStatus_set)
    ConnectRequestDate = property(_thosttraderapi.CThostFtdcMDTraderOfferField_ConnectRequestDate_get, _thosttraderapi.CThostFtdcMDTraderOfferField_ConnectRequestDate_set)
    ConnectRequestTime = property(_thosttraderapi.CThostFtdcMDTraderOfferField_ConnectRequestTime_get, _thosttraderapi.CThostFtdcMDTraderOfferField_ConnectRequestTime_set)
    LastReportDate = property(_thosttraderapi.CThostFtdcMDTraderOfferField_LastReportDate_get, _thosttraderapi.CThostFtdcMDTraderOfferField_LastReportDate_set)
    LastReportTime = property(_thosttraderapi.CThostFtdcMDTraderOfferField_LastReportTime_get, _thosttraderapi.CThostFtdcMDTraderOfferField_LastReportTime_set)
    ConnectDate = property(_thosttraderapi.CThostFtdcMDTraderOfferField_ConnectDate_get, _thosttraderapi.CThostFtdcMDTraderOfferField_ConnectDate_set)
    ConnectTime = property(_thosttraderapi.CThostFtdcMDTraderOfferField_ConnectTime_get, _thosttraderapi.CThostFtdcMDTraderOfferField_ConnectTime_set)
    StartDate = property(_thosttraderapi.CThostFtdcMDTraderOfferField_StartDate_get, _thosttraderapi.CThostFtdcMDTraderOfferField_StartDate_set)
    StartTime = property(_thosttraderapi.CThostFtdcMDTraderOfferField_StartTime_get, _thosttraderapi.CThostFtdcMDTraderOfferField_StartTime_set)
    TradingDay = property(_thosttraderapi.CThostFtdcMDTraderOfferField_TradingDay_get, _thosttraderapi.CThostFtdcMDTraderOfferField_TradingDay_set)
    BrokerID = property(_thosttraderapi.CThostFtdcMDTraderOfferField_BrokerID_get, _thosttraderapi.CThostFtdcMDTraderOfferField_BrokerID_set)
    MaxTradeID = property(_thosttraderapi.CThostFtdcMDTraderOfferField_MaxTradeID_get, _thosttraderapi.CThostFtdcMDTraderOfferField_MaxTradeID_set)
    MaxOrderMessageReference = property(_thosttraderapi.CThostFtdcMDTraderOfferField_MaxOrderMessageReference_get, _thosttraderapi.CThostFtdcMDTraderOfferField_MaxOrderMessageReference_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcMDTraderOfferField_swiginit(self, _thosttraderapi.new_CThostFtdcMDTraderOfferField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcMDTraderOfferField

# Register CThostFtdcMDTraderOfferField in _thosttraderapi:
_thosttraderapi.CThostFtdcMDTraderOfferField_swigregister(CThostFtdcMDTraderOfferField)

class CThostFtdcQryMDTraderOfferField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    ExchangeID = property(_thosttraderapi.CThostFtdcQryMDTraderOfferField_ExchangeID_get, _thosttraderapi.CThostFtdcQryMDTraderOfferField_ExchangeID_set)
    ParticipantID = property(_thosttraderapi.CThostFtdcQryMDTraderOfferField_ParticipantID_get, _thosttraderapi.CThostFtdcQryMDTraderOfferField_ParticipantID_set)
    TraderID = property(_thosttraderapi.CThostFtdcQryMDTraderOfferField_TraderID_get, _thosttraderapi.CThostFtdcQryMDTraderOfferField_TraderID_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcQryMDTraderOfferField_swiginit(self, _thosttraderapi.new_CThostFtdcQryMDTraderOfferField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcQryMDTraderOfferField

# Register CThostFtdcQryMDTraderOfferField in _thosttraderapi:
_thosttraderapi.CThostFtdcQryMDTraderOfferField_swigregister(CThostFtdcQryMDTraderOfferField)

class CThostFtdcQryNoticeField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thosttraderapi.CThostFtdcQryNoticeField_BrokerID_get, _thosttraderapi.CThostFtdcQryNoticeField_BrokerID_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcQryNoticeField_swiginit(self, _thosttraderapi.new_CThostFtdcQryNoticeField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcQryNoticeField

# Register CThostFtdcQryNoticeField in _thosttraderapi:
_thosttraderapi.CThostFtdcQryNoticeField_swigregister(CThostFtdcQryNoticeField)

class CThostFtdcNoticeField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thosttraderapi.CThostFtdcNoticeField_BrokerID_get, _thosttraderapi.CThostFtdcNoticeField_BrokerID_set)
    Content = property(_thosttraderapi.CThostFtdcNoticeField_Content_get, _thosttraderapi.CThostFtdcNoticeField_Content_set)
    SequenceLabel = property(_thosttraderapi.CThostFtdcNoticeField_SequenceLabel_get, _thosttraderapi.CThostFtdcNoticeField_SequenceLabel_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcNoticeField_swiginit(self, _thosttraderapi.new_CThostFtdcNoticeField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcNoticeField

# Register CThostFtdcNoticeField in _thosttraderapi:
_thosttraderapi.CThostFtdcNoticeField_swigregister(CThostFtdcNoticeField)

class CThostFtdcUserRightField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thosttraderapi.CThostFtdcUserRightField_BrokerID_get, _thosttraderapi.CThostFtdcUserRightField_BrokerID_set)
    UserID = property(_thosttraderapi.CThostFtdcUserRightField_UserID_get, _thosttraderapi.CThostFtdcUserRightField_UserID_set)
    UserRightType = property(_thosttraderapi.CThostFtdcUserRightField_UserRightType_get, _thosttraderapi.CThostFtdcUserRightField_UserRightType_set)
    IsForbidden = property(_thosttraderapi.CThostFtdcUserRightField_IsForbidden_get, _thosttraderapi.CThostFtdcUserRightField_IsForbidden_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcUserRightField_swiginit(self, _thosttraderapi.new_CThostFtdcUserRightField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcUserRightField

# Register CThostFtdcUserRightField in _thosttraderapi:
_thosttraderapi.CThostFtdcUserRightField_swigregister(CThostFtdcUserRightField)

class CThostFtdcQrySettlementInfoConfirmField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thosttraderapi.CThostFtdcQrySettlementInfoConfirmField_BrokerID_get, _thosttraderapi.CThostFtdcQrySettlementInfoConfirmField_BrokerID_set)
    InvestorID = property(_thosttraderapi.CThostFtdcQrySettlementInfoConfirmField_InvestorID_get, _thosttraderapi.CThostFtdcQrySettlementInfoConfirmField_InvestorID_set)
    AccountID = property(_thosttraderapi.CThostFtdcQrySettlementInfoConfirmField_AccountID_get, _thosttraderapi.CThostFtdcQrySettlementInfoConfirmField_AccountID_set)
    CurrencyID = property(_thosttraderapi.CThostFtdcQrySettlementInfoConfirmField_CurrencyID_get, _thosttraderapi.CThostFtdcQrySettlementInfoConfirmField_CurrencyID_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcQrySettlementInfoConfirmField_swiginit(self, _thosttraderapi.new_CThostFtdcQrySettlementInfoConfirmField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcQrySettlementInfoConfirmField

# Register CThostFtdcQrySettlementInfoConfirmField in _thosttraderapi:
_thosttraderapi.CThostFtdcQrySettlementInfoConfirmField_swigregister(CThostFtdcQrySettlementInfoConfirmField)

class CThostFtdcLoadSettlementInfoField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thosttraderapi.CThostFtdcLoadSettlementInfoField_BrokerID_get, _thosttraderapi.CThostFtdcLoadSettlementInfoField_BrokerID_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcLoadSettlementInfoField_swiginit(self, _thosttraderapi.new_CThostFtdcLoadSettlementInfoField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcLoadSettlementInfoField

# Register CThostFtdcLoadSettlementInfoField in _thosttraderapi:
_thosttraderapi.CThostFtdcLoadSettlementInfoField_swigregister(CThostFtdcLoadSettlementInfoField)

class CThostFtdcBrokerWithdrawAlgorithmField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thosttraderapi.CThostFtdcBrokerWithdrawAlgorithmField_BrokerID_get, _thosttraderapi.CThostFtdcBrokerWithdrawAlgorithmField_BrokerID_set)
    WithdrawAlgorithm = property(_thosttraderapi.CThostFtdcBrokerWithdrawAlgorithmField_WithdrawAlgorithm_get, _thosttraderapi.CThostFtdcBrokerWithdrawAlgorithmField_WithdrawAlgorithm_set)
    UsingRatio = property(_thosttraderapi.CThostFtdcBrokerWithdrawAlgorithmField_UsingRatio_get, _thosttraderapi.CThostFtdcBrokerWithdrawAlgorithmField_UsingRatio_set)
    IncludeCloseProfit = property(_thosttraderapi.CThostFtdcBrokerWithdrawAlgorithmField_IncludeCloseProfit_get, _thosttraderapi.CThostFtdcBrokerWithdrawAlgorithmField_IncludeCloseProfit_set)
    AllWithoutTrade = property(_thosttraderapi.CThostFtdcBrokerWithdrawAlgorithmField_AllWithoutTrade_get, _thosttraderapi.CThostFtdcBrokerWithdrawAlgorithmField_AllWithoutTrade_set)
    AvailIncludeCloseProfit = property(_thosttraderapi.CThostFtdcBrokerWithdrawAlgorithmField_AvailIncludeCloseProfit_get, _thosttraderapi.CThostFtdcBrokerWithdrawAlgorithmField_AvailIncludeCloseProfit_set)
    IsBrokerUserEvent = property(_thosttraderapi.CThostFtdcBrokerWithdrawAlgorithmField_IsBrokerUserEvent_get, _thosttraderapi.CThostFtdcBrokerWithdrawAlgorithmField_IsBrokerUserEvent_set)
    CurrencyID = property(_thosttraderapi.CThostFtdcBrokerWithdrawAlgorithmField_CurrencyID_get, _thosttraderapi.CThostFtdcBrokerWithdrawAlgorithmField_CurrencyID_set)
    FundMortgageRatio = property(_thosttraderapi.CThostFtdcBrokerWithdrawAlgorithmField_FundMortgageRatio_get, _thosttraderapi.CThostFtdcBrokerWithdrawAlgorithmField_FundMortgageRatio_set)
    BalanceAlgorithm = property(_thosttraderapi.CThostFtdcBrokerWithdrawAlgorithmField_BalanceAlgorithm_get, _thosttraderapi.CThostFtdcBrokerWithdrawAlgorithmField_BalanceAlgorithm_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcBrokerWithdrawAlgorithmField_swiginit(self, _thosttraderapi.new_CThostFtdcBrokerWithdrawAlgorithmField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcBrokerWithdrawAlgorithmField

# Register CThostFtdcBrokerWithdrawAlgorithmField in _thosttraderapi:
_thosttraderapi.CThostFtdcBrokerWithdrawAlgorithmField_swigregister(CThostFtdcBrokerWithdrawAlgorithmField)

class CThostFtdcTradingAccountPasswordUpdateV1Field(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thosttraderapi.CThostFtdcTradingAccountPasswordUpdateV1Field_BrokerID_get, _thosttraderapi.CThostFtdcTradingAccountPasswordUpdateV1Field_BrokerID_set)
    InvestorID = property(_thosttraderapi.CThostFtdcTradingAccountPasswordUpdateV1Field_InvestorID_get, _thosttraderapi.CThostFtdcTradingAccountPasswordUpdateV1Field_InvestorID_set)
    OldPassword = property(_thosttraderapi.CThostFtdcTradingAccountPasswordUpdateV1Field_OldPassword_get, _thosttraderapi.CThostFtdcTradingAccountPasswordUpdateV1Field_OldPassword_set)
    NewPassword = property(_thosttraderapi.CThostFtdcTradingAccountPasswordUpdateV1Field_NewPassword_get, _thosttraderapi.CThostFtdcTradingAccountPasswordUpdateV1Field_NewPassword_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcTradingAccountPasswordUpdateV1Field_swiginit(self, _thosttraderapi.new_CThostFtdcTradingAccountPasswordUpdateV1Field())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcTradingAccountPasswordUpdateV1Field

# Register CThostFtdcTradingAccountPasswordUpdateV1Field in _thosttraderapi:
_thosttraderapi.CThostFtdcTradingAccountPasswordUpdateV1Field_swigregister(CThostFtdcTradingAccountPasswordUpdateV1Field)

class CThostFtdcTradingAccountPasswordUpdateField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thosttraderapi.CThostFtdcTradingAccountPasswordUpdateField_BrokerID_get, _thosttraderapi.CThostFtdcTradingAccountPasswordUpdateField_BrokerID_set)
    AccountID = property(_thosttraderapi.CThostFtdcTradingAccountPasswordUpdateField_AccountID_get, _thosttraderapi.CThostFtdcTradingAccountPasswordUpdateField_AccountID_set)
    OldPassword = property(_thosttraderapi.CThostFtdcTradingAccountPasswordUpdateField_OldPassword_get, _thosttraderapi.CThostFtdcTradingAccountPasswordUpdateField_OldPassword_set)
    NewPassword = property(_thosttraderapi.CThostFtdcTradingAccountPasswordUpdateField_NewPassword_get, _thosttraderapi.CThostFtdcTradingAccountPasswordUpdateField_NewPassword_set)
    CurrencyID = property(_thosttraderapi.CThostFtdcTradingAccountPasswordUpdateField_CurrencyID_get, _thosttraderapi.CThostFtdcTradingAccountPasswordUpdateField_CurrencyID_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcTradingAccountPasswordUpdateField_swiginit(self, _thosttraderapi.new_CThostFtdcTradingAccountPasswordUpdateField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcTradingAccountPasswordUpdateField

# Register CThostFtdcTradingAccountPasswordUpdateField in _thosttraderapi:
_thosttraderapi.CThostFtdcTradingAccountPasswordUpdateField_swigregister(CThostFtdcTradingAccountPasswordUpdateField)

class CThostFtdcQryCombinationLegField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    CombInstrumentID = property(_thosttraderapi.CThostFtdcQryCombinationLegField_CombInstrumentID_get, _thosttraderapi.CThostFtdcQryCombinationLegField_CombInstrumentID_set)
    LegID = property(_thosttraderapi.CThostFtdcQryCombinationLegField_LegID_get, _thosttraderapi.CThostFtdcQryCombinationLegField_LegID_set)
    LegInstrumentID = property(_thosttraderapi.CThostFtdcQryCombinationLegField_LegInstrumentID_get, _thosttraderapi.CThostFtdcQryCombinationLegField_LegInstrumentID_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcQryCombinationLegField_swiginit(self, _thosttraderapi.new_CThostFtdcQryCombinationLegField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcQryCombinationLegField

# Register CThostFtdcQryCombinationLegField in _thosttraderapi:
_thosttraderapi.CThostFtdcQryCombinationLegField_swigregister(CThostFtdcQryCombinationLegField)

class CThostFtdcQrySyncStatusField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    TradingDay = property(_thosttraderapi.CThostFtdcQrySyncStatusField_TradingDay_get, _thosttraderapi.CThostFtdcQrySyncStatusField_TradingDay_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcQrySyncStatusField_swiginit(self, _thosttraderapi.new_CThostFtdcQrySyncStatusField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcQrySyncStatusField

# Register CThostFtdcQrySyncStatusField in _thosttraderapi:
_thosttraderapi.CThostFtdcQrySyncStatusField_swigregister(CThostFtdcQrySyncStatusField)

class CThostFtdcCombinationLegField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    CombInstrumentID = property(_thosttraderapi.CThostFtdcCombinationLegField_CombInstrumentID_get, _thosttraderapi.CThostFtdcCombinationLegField_CombInstrumentID_set)
    LegID = property(_thosttraderapi.CThostFtdcCombinationLegField_LegID_get, _thosttraderapi.CThostFtdcCombinationLegField_LegID_set)
    LegInstrumentID = property(_thosttraderapi.CThostFtdcCombinationLegField_LegInstrumentID_get, _thosttraderapi.CThostFtdcCombinationLegField_LegInstrumentID_set)
    Direction = property(_thosttraderapi.CThostFtdcCombinationLegField_Direction_get, _thosttraderapi.CThostFtdcCombinationLegField_Direction_set)
    LegMultiple = property(_thosttraderapi.CThostFtdcCombinationLegField_LegMultiple_get, _thosttraderapi.CThostFtdcCombinationLegField_LegMultiple_set)
    ImplyLevel = property(_thosttraderapi.CThostFtdcCombinationLegField_ImplyLevel_get, _thosttraderapi.CThostFtdcCombinationLegField_ImplyLevel_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcCombinationLegField_swiginit(self, _thosttraderapi.new_CThostFtdcCombinationLegField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcCombinationLegField

# Register CThostFtdcCombinationLegField in _thosttraderapi:
_thosttraderapi.CThostFtdcCombinationLegField_swigregister(CThostFtdcCombinationLegField)

class CThostFtdcSyncStatusField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    TradingDay = property(_thosttraderapi.CThostFtdcSyncStatusField_TradingDay_get, _thosttraderapi.CThostFtdcSyncStatusField_TradingDay_set)
    DataSyncStatus = property(_thosttraderapi.CThostFtdcSyncStatusField_DataSyncStatus_get, _thosttraderapi.CThostFtdcSyncStatusField_DataSyncStatus_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcSyncStatusField_swiginit(self, _thosttraderapi.new_CThostFtdcSyncStatusField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcSyncStatusField

# Register CThostFtdcSyncStatusField in _thosttraderapi:
_thosttraderapi.CThostFtdcSyncStatusField_swigregister(CThostFtdcSyncStatusField)

class CThostFtdcQryLinkManField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thosttraderapi.CThostFtdcQryLinkManField_BrokerID_get, _thosttraderapi.CThostFtdcQryLinkManField_BrokerID_set)
    InvestorID = property(_thosttraderapi.CThostFtdcQryLinkManField_InvestorID_get, _thosttraderapi.CThostFtdcQryLinkManField_InvestorID_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcQryLinkManField_swiginit(self, _thosttraderapi.new_CThostFtdcQryLinkManField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcQryLinkManField

# Register CThostFtdcQryLinkManField in _thosttraderapi:
_thosttraderapi.CThostFtdcQryLinkManField_swigregister(CThostFtdcQryLinkManField)

class CThostFtdcLinkManField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thosttraderapi.CThostFtdcLinkManField_BrokerID_get, _thosttraderapi.CThostFtdcLinkManField_BrokerID_set)
    InvestorID = property(_thosttraderapi.CThostFtdcLinkManField_InvestorID_get, _thosttraderapi.CThostFtdcLinkManField_InvestorID_set)
    PersonType = property(_thosttraderapi.CThostFtdcLinkManField_PersonType_get, _thosttraderapi.CThostFtdcLinkManField_PersonType_set)
    IdentifiedCardType = property(_thosttraderapi.CThostFtdcLinkManField_IdentifiedCardType_get, _thosttraderapi.CThostFtdcLinkManField_IdentifiedCardType_set)
    IdentifiedCardNo = property(_thosttraderapi.CThostFtdcLinkManField_IdentifiedCardNo_get, _thosttraderapi.CThostFtdcLinkManField_IdentifiedCardNo_set)
    PersonName = property(_thosttraderapi.CThostFtdcLinkManField_PersonName_get, _thosttraderapi.CThostFtdcLinkManField_PersonName_set)
    Telephone = property(_thosttraderapi.CThostFtdcLinkManField_Telephone_get, _thosttraderapi.CThostFtdcLinkManField_Telephone_set)
    Address = property(_thosttraderapi.CThostFtdcLinkManField_Address_get, _thosttraderapi.CThostFtdcLinkManField_Address_set)
    ZipCode = property(_thosttraderapi.CThostFtdcLinkManField_ZipCode_get, _thosttraderapi.CThostFtdcLinkManField_ZipCode_set)
    Priority = property(_thosttraderapi.CThostFtdcLinkManField_Priority_get, _thosttraderapi.CThostFtdcLinkManField_Priority_set)
    UOAZipCode = property(_thosttraderapi.CThostFtdcLinkManField_UOAZipCode_get, _thosttraderapi.CThostFtdcLinkManField_UOAZipCode_set)
    PersonFullName = property(_thosttraderapi.CThostFtdcLinkManField_PersonFullName_get, _thosttraderapi.CThostFtdcLinkManField_PersonFullName_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcLinkManField_swiginit(self, _thosttraderapi.new_CThostFtdcLinkManField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcLinkManField

# Register CThostFtdcLinkManField in _thosttraderapi:
_thosttraderapi.CThostFtdcLinkManField_swigregister(CThostFtdcLinkManField)

class CThostFtdcQryBrokerUserEventField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thosttraderapi.CThostFtdcQryBrokerUserEventField_BrokerID_get, _thosttraderapi.CThostFtdcQryBrokerUserEventField_BrokerID_set)
    UserID = property(_thosttraderapi.CThostFtdcQryBrokerUserEventField_UserID_get, _thosttraderapi.CThostFtdcQryBrokerUserEventField_UserID_set)
    UserEventType = property(_thosttraderapi.CThostFtdcQryBrokerUserEventField_UserEventType_get, _thosttraderapi.CThostFtdcQryBrokerUserEventField_UserEventType_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcQryBrokerUserEventField_swiginit(self, _thosttraderapi.new_CThostFtdcQryBrokerUserEventField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcQryBrokerUserEventField

# Register CThostFtdcQryBrokerUserEventField in _thosttraderapi:
_thosttraderapi.CThostFtdcQryBrokerUserEventField_swigregister(CThostFtdcQryBrokerUserEventField)

class CThostFtdcBrokerUserEventField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thosttraderapi.CThostFtdcBrokerUserEventField_BrokerID_get, _thosttraderapi.CThostFtdcBrokerUserEventField_BrokerID_set)
    UserID = property(_thosttraderapi.CThostFtdcBrokerUserEventField_UserID_get, _thosttraderapi.CThostFtdcBrokerUserEventField_UserID_set)
    UserEventType = property(_thosttraderapi.CThostFtdcBrokerUserEventField_UserEventType_get, _thosttraderapi.CThostFtdcBrokerUserEventField_UserEventType_set)
    EventSequenceNo = property(_thosttraderapi.CThostFtdcBrokerUserEventField_EventSequenceNo_get, _thosttraderapi.CThostFtdcBrokerUserEventField_EventSequenceNo_set)
    EventDate = property(_thosttraderapi.CThostFtdcBrokerUserEventField_EventDate_get, _thosttraderapi.CThostFtdcBrokerUserEventField_EventDate_set)
    EventTime = property(_thosttraderapi.CThostFtdcBrokerUserEventField_EventTime_get, _thosttraderapi.CThostFtdcBrokerUserEventField_EventTime_set)
    UserEventInfo = property(_thosttraderapi.CThostFtdcBrokerUserEventField_UserEventInfo_get, _thosttraderapi.CThostFtdcBrokerUserEventField_UserEventInfo_set)
    InvestorID = property(_thosttraderapi.CThostFtdcBrokerUserEventField_InvestorID_get, _thosttraderapi.CThostFtdcBrokerUserEventField_InvestorID_set)
    InstrumentID = property(_thosttraderapi.CThostFtdcBrokerUserEventField_InstrumentID_get, _thosttraderapi.CThostFtdcBrokerUserEventField_InstrumentID_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcBrokerUserEventField_swiginit(self, _thosttraderapi.new_CThostFtdcBrokerUserEventField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcBrokerUserEventField

# Register CThostFtdcBrokerUserEventField in _thosttraderapi:
_thosttraderapi.CThostFtdcBrokerUserEventField_swigregister(CThostFtdcBrokerUserEventField)

class CThostFtdcQryContractBankField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thosttraderapi.CThostFtdcQryContractBankField_BrokerID_get, _thosttraderapi.CThostFtdcQryContractBankField_BrokerID_set)
    BankID = property(_thosttraderapi.CThostFtdcQryContractBankField_BankID_get, _thosttraderapi.CThostFtdcQryContractBankField_BankID_set)
    BankBrchID = property(_thosttraderapi.CThostFtdcQryContractBankField_BankBrchID_get, _thosttraderapi.CThostFtdcQryContractBankField_BankBrchID_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcQryContractBankField_swiginit(self, _thosttraderapi.new_CThostFtdcQryContractBankField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcQryContractBankField

# Register CThostFtdcQryContractBankField in _thosttraderapi:
_thosttraderapi.CThostFtdcQryContractBankField_swigregister(CThostFtdcQryContractBankField)

class CThostFtdcContractBankField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thosttraderapi.CThostFtdcContractBankField_BrokerID_get, _thosttraderapi.CThostFtdcContractBankField_BrokerID_set)
    BankID = property(_thosttraderapi.CThostFtdcContractBankField_BankID_get, _thosttraderapi.CThostFtdcContractBankField_BankID_set)
    BankBrchID = property(_thosttraderapi.CThostFtdcContractBankField_BankBrchID_get, _thosttraderapi.CThostFtdcContractBankField_BankBrchID_set)
    BankName = property(_thosttraderapi.CThostFtdcContractBankField_BankName_get, _thosttraderapi.CThostFtdcContractBankField_BankName_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcContractBankField_swiginit(self, _thosttraderapi.new_CThostFtdcContractBankField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcContractBankField

# Register CThostFtdcContractBankField in _thosttraderapi:
_thosttraderapi.CThostFtdcContractBankField_swigregister(CThostFtdcContractBankField)

class CThostFtdcInvestorPositionCombineDetailField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    TradingDay = property(_thosttraderapi.CThostFtdcInvestorPositionCombineDetailField_TradingDay_get, _thosttraderapi.CThostFtdcInvestorPositionCombineDetailField_TradingDay_set)
    OpenDate = property(_thosttraderapi.CThostFtdcInvestorPositionCombineDetailField_OpenDate_get, _thosttraderapi.CThostFtdcInvestorPositionCombineDetailField_OpenDate_set)
    ExchangeID = property(_thosttraderapi.CThostFtdcInvestorPositionCombineDetailField_ExchangeID_get, _thosttraderapi.CThostFtdcInvestorPositionCombineDetailField_ExchangeID_set)
    SettlementID = property(_thosttraderapi.CThostFtdcInvestorPositionCombineDetailField_SettlementID_get, _thosttraderapi.CThostFtdcInvestorPositionCombineDetailField_SettlementID_set)
    BrokerID = property(_thosttraderapi.CThostFtdcInvestorPositionCombineDetailField_BrokerID_get, _thosttraderapi.CThostFtdcInvestorPositionCombineDetailField_BrokerID_set)
    InvestorID = property(_thosttraderapi.CThostFtdcInvestorPositionCombineDetailField_InvestorID_get, _thosttraderapi.CThostFtdcInvestorPositionCombineDetailField_InvestorID_set)
    ComTradeID = property(_thosttraderapi.CThostFtdcInvestorPositionCombineDetailField_ComTradeID_get, _thosttraderapi.CThostFtdcInvestorPositionCombineDetailField_ComTradeID_set)
    TradeID = property(_thosttraderapi.CThostFtdcInvestorPositionCombineDetailField_TradeID_get, _thosttraderapi.CThostFtdcInvestorPositionCombineDetailField_TradeID_set)
    InstrumentID = property(_thosttraderapi.CThostFtdcInvestorPositionCombineDetailField_InstrumentID_get, _thosttraderapi.CThostFtdcInvestorPositionCombineDetailField_InstrumentID_set)
    HedgeFlag = property(_thosttraderapi.CThostFtdcInvestorPositionCombineDetailField_HedgeFlag_get, _thosttraderapi.CThostFtdcInvestorPositionCombineDetailField_HedgeFlag_set)
    Direction = property(_thosttraderapi.CThostFtdcInvestorPositionCombineDetailField_Direction_get, _thosttraderapi.CThostFtdcInvestorPositionCombineDetailField_Direction_set)
    TotalAmt = property(_thosttraderapi.CThostFtdcInvestorPositionCombineDetailField_TotalAmt_get, _thosttraderapi.CThostFtdcInvestorPositionCombineDetailField_TotalAmt_set)
    Margin = property(_thosttraderapi.CThostFtdcInvestorPositionCombineDetailField_Margin_get, _thosttraderapi.CThostFtdcInvestorPositionCombineDetailField_Margin_set)
    ExchMargin = property(_thosttraderapi.CThostFtdcInvestorPositionCombineDetailField_ExchMargin_get, _thosttraderapi.CThostFtdcInvestorPositionCombineDetailField_ExchMargin_set)
    MarginRateByMoney = property(_thosttraderapi.CThostFtdcInvestorPositionCombineDetailField_MarginRateByMoney_get, _thosttraderapi.CThostFtdcInvestorPositionCombineDetailField_MarginRateByMoney_set)
    MarginRateByVolume = property(_thosttraderapi.CThostFtdcInvestorPositionCombineDetailField_MarginRateByVolume_get, _thosttraderapi.CThostFtdcInvestorPositionCombineDetailField_MarginRateByVolume_set)
    LegID = property(_thosttraderapi.CThostFtdcInvestorPositionCombineDetailField_LegID_get, _thosttraderapi.CThostFtdcInvestorPositionCombineDetailField_LegID_set)
    LegMultiple = property(_thosttraderapi.CThostFtdcInvestorPositionCombineDetailField_LegMultiple_get, _thosttraderapi.CThostFtdcInvestorPositionCombineDetailField_LegMultiple_set)
    CombInstrumentID = property(_thosttraderapi.CThostFtdcInvestorPositionCombineDetailField_CombInstrumentID_get, _thosttraderapi.CThostFtdcInvestorPositionCombineDetailField_CombInstrumentID_set)
    TradeGroupID = property(_thosttraderapi.CThostFtdcInvestorPositionCombineDetailField_TradeGroupID_get, _thosttraderapi.CThostFtdcInvestorPositionCombineDetailField_TradeGroupID_set)
    InvestUnitID = property(_thosttraderapi.CThostFtdcInvestorPositionCombineDetailField_InvestUnitID_get, _thosttraderapi.CThostFtdcInvestorPositionCombineDetailField_InvestUnitID_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcInvestorPositionCombineDetailField_swiginit(self, _thosttraderapi.new_CThostFtdcInvestorPositionCombineDetailField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcInvestorPositionCombineDetailField

# Register CThostFtdcInvestorPositionCombineDetailField in _thosttraderapi:
_thosttraderapi.CThostFtdcInvestorPositionCombineDetailField_swigregister(CThostFtdcInvestorPositionCombineDetailField)

class CThostFtdcParkedOrderField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thosttraderapi.CThostFtdcParkedOrderField_BrokerID_get, _thosttraderapi.CThostFtdcParkedOrderField_BrokerID_set)
    InvestorID = property(_thosttraderapi.CThostFtdcParkedOrderField_InvestorID_get, _thosttraderapi.CThostFtdcParkedOrderField_InvestorID_set)
    InstrumentID = property(_thosttraderapi.CThostFtdcParkedOrderField_InstrumentID_get, _thosttraderapi.CThostFtdcParkedOrderField_InstrumentID_set)
    OrderRef = property(_thosttraderapi.CThostFtdcParkedOrderField_OrderRef_get, _thosttraderapi.CThostFtdcParkedOrderField_OrderRef_set)
    UserID = property(_thosttraderapi.CThostFtdcParkedOrderField_UserID_get, _thosttraderapi.CThostFtdcParkedOrderField_UserID_set)
    OrderPriceType = property(_thosttraderapi.CThostFtdcParkedOrderField_OrderPriceType_get, _thosttraderapi.CThostFtdcParkedOrderField_OrderPriceType_set)
    Direction = property(_thosttraderapi.CThostFtdcParkedOrderField_Direction_get, _thosttraderapi.CThostFtdcParkedOrderField_Direction_set)
    CombOffsetFlag = property(_thosttraderapi.CThostFtdcParkedOrderField_CombOffsetFlag_get, _thosttraderapi.CThostFtdcParkedOrderField_CombOffsetFlag_set)
    CombHedgeFlag = property(_thosttraderapi.CThostFtdcParkedOrderField_CombHedgeFlag_get, _thosttraderapi.CThostFtdcParkedOrderField_CombHedgeFlag_set)
    LimitPrice = property(_thosttraderapi.CThostFtdcParkedOrderField_LimitPrice_get, _thosttraderapi.CThostFtdcParkedOrderField_LimitPrice_set)
    VolumeTotalOriginal = property(_thosttraderapi.CThostFtdcParkedOrderField_VolumeTotalOriginal_get, _thosttraderapi.CThostFtdcParkedOrderField_VolumeTotalOriginal_set)
    TimeCondition = property(_thosttraderapi.CThostFtdcParkedOrderField_TimeCondition_get, _thosttraderapi.CThostFtdcParkedOrderField_TimeCondition_set)
    GTDDate = property(_thosttraderapi.CThostFtdcParkedOrderField_GTDDate_get, _thosttraderapi.CThostFtdcParkedOrderField_GTDDate_set)
    VolumeCondition = property(_thosttraderapi.CThostFtdcParkedOrderField_VolumeCondition_get, _thosttraderapi.CThostFtdcParkedOrderField_VolumeCondition_set)
    MinVolume = property(_thosttraderapi.CThostFtdcParkedOrderField_MinVolume_get, _thosttraderapi.CThostFtdcParkedOrderField_MinVolume_set)
    ContingentCondition = property(_thosttraderapi.CThostFtdcParkedOrderField_ContingentCondition_get, _thosttraderapi.CThostFtdcParkedOrderField_ContingentCondition_set)
    StopPrice = property(_thosttraderapi.CThostFtdcParkedOrderField_StopPrice_get, _thosttraderapi.CThostFtdcParkedOrderField_StopPrice_set)
    ForceCloseReason = property(_thosttraderapi.CThostFtdcParkedOrderField_ForceCloseReason_get, _thosttraderapi.CThostFtdcParkedOrderField_ForceCloseReason_set)
    IsAutoSuspend = property(_thosttraderapi.CThostFtdcParkedOrderField_IsAutoSuspend_get, _thosttraderapi.CThostFtdcParkedOrderField_IsAutoSuspend_set)
    BusinessUnit = property(_thosttraderapi.CThostFtdcParkedOrderField_BusinessUnit_get, _thosttraderapi.CThostFtdcParkedOrderField_BusinessUnit_set)
    RequestID = property(_thosttraderapi.CThostFtdcParkedOrderField_RequestID_get, _thosttraderapi.CThostFtdcParkedOrderField_RequestID_set)
    UserForceClose = property(_thosttraderapi.CThostFtdcParkedOrderField_UserForceClose_get, _thosttraderapi.CThostFtdcParkedOrderField_UserForceClose_set)
    ExchangeID = property(_thosttraderapi.CThostFtdcParkedOrderField_ExchangeID_get, _thosttraderapi.CThostFtdcParkedOrderField_ExchangeID_set)
    ParkedOrderID = property(_thosttraderapi.CThostFtdcParkedOrderField_ParkedOrderID_get, _thosttraderapi.CThostFtdcParkedOrderField_ParkedOrderID_set)
    UserType = property(_thosttraderapi.CThostFtdcParkedOrderField_UserType_get, _thosttraderapi.CThostFtdcParkedOrderField_UserType_set)
    Status = property(_thosttraderapi.CThostFtdcParkedOrderField_Status_get, _thosttraderapi.CThostFtdcParkedOrderField_Status_set)
    ErrorID = property(_thosttraderapi.CThostFtdcParkedOrderField_ErrorID_get, _thosttraderapi.CThostFtdcParkedOrderField_ErrorID_set)
    ErrorMsg = property(_thosttraderapi.CThostFtdcParkedOrderField_ErrorMsg_get, _thosttraderapi.CThostFtdcParkedOrderField_ErrorMsg_set)
    IsSwapOrder = property(_thosttraderapi.CThostFtdcParkedOrderField_IsSwapOrder_get, _thosttraderapi.CThostFtdcParkedOrderField_IsSwapOrder_set)
    AccountID = property(_thosttraderapi.CThostFtdcParkedOrderField_AccountID_get, _thosttraderapi.CThostFtdcParkedOrderField_AccountID_set)
    CurrencyID = property(_thosttraderapi.CThostFtdcParkedOrderField_CurrencyID_get, _thosttraderapi.CThostFtdcParkedOrderField_CurrencyID_set)
    ClientID = property(_thosttraderapi.CThostFtdcParkedOrderField_ClientID_get, _thosttraderapi.CThostFtdcParkedOrderField_ClientID_set)
    InvestUnitID = property(_thosttraderapi.CThostFtdcParkedOrderField_InvestUnitID_get, _thosttraderapi.CThostFtdcParkedOrderField_InvestUnitID_set)
    IPAddress = property(_thosttraderapi.CThostFtdcParkedOrderField_IPAddress_get, _thosttraderapi.CThostFtdcParkedOrderField_IPAddress_set)
    MacAddress = property(_thosttraderapi.CThostFtdcParkedOrderField_MacAddress_get, _thosttraderapi.CThostFtdcParkedOrderField_MacAddress_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcParkedOrderField_swiginit(self, _thosttraderapi.new_CThostFtdcParkedOrderField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcParkedOrderField

# Register CThostFtdcParkedOrderField in _thosttraderapi:
_thosttraderapi.CThostFtdcParkedOrderField_swigregister(CThostFtdcParkedOrderField)

class CThostFtdcParkedOrderActionField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thosttraderapi.CThostFtdcParkedOrderActionField_BrokerID_get, _thosttraderapi.CThostFtdcParkedOrderActionField_BrokerID_set)
    InvestorID = property(_thosttraderapi.CThostFtdcParkedOrderActionField_InvestorID_get, _thosttraderapi.CThostFtdcParkedOrderActionField_InvestorID_set)
    OrderActionRef = property(_thosttraderapi.CThostFtdcParkedOrderActionField_OrderActionRef_get, _thosttraderapi.CThostFtdcParkedOrderActionField_OrderActionRef_set)
    OrderRef = property(_thosttraderapi.CThostFtdcParkedOrderActionField_OrderRef_get, _thosttraderapi.CThostFtdcParkedOrderActionField_OrderRef_set)
    RequestID = property(_thosttraderapi.CThostFtdcParkedOrderActionField_RequestID_get, _thosttraderapi.CThostFtdcParkedOrderActionField_RequestID_set)
    FrontID = property(_thosttraderapi.CThostFtdcParkedOrderActionField_FrontID_get, _thosttraderapi.CThostFtdcParkedOrderActionField_FrontID_set)
    SessionID = property(_thosttraderapi.CThostFtdcParkedOrderActionField_SessionID_get, _thosttraderapi.CThostFtdcParkedOrderActionField_SessionID_set)
    ExchangeID = property(_thosttraderapi.CThostFtdcParkedOrderActionField_ExchangeID_get, _thosttraderapi.CThostFtdcParkedOrderActionField_ExchangeID_set)
    OrderSysID = property(_thosttraderapi.CThostFtdcParkedOrderActionField_OrderSysID_get, _thosttraderapi.CThostFtdcParkedOrderActionField_OrderSysID_set)
    ActionFlag = property(_thosttraderapi.CThostFtdcParkedOrderActionField_ActionFlag_get, _thosttraderapi.CThostFtdcParkedOrderActionField_ActionFlag_set)
    LimitPrice = property(_thosttraderapi.CThostFtdcParkedOrderActionField_LimitPrice_get, _thosttraderapi.CThostFtdcParkedOrderActionField_LimitPrice_set)
    VolumeChange = property(_thosttraderapi.CThostFtdcParkedOrderActionField_VolumeChange_get, _thosttraderapi.CThostFtdcParkedOrderActionField_VolumeChange_set)
    UserID = property(_thosttraderapi.CThostFtdcParkedOrderActionField_UserID_get, _thosttraderapi.CThostFtdcParkedOrderActionField_UserID_set)
    InstrumentID = property(_thosttraderapi.CThostFtdcParkedOrderActionField_InstrumentID_get, _thosttraderapi.CThostFtdcParkedOrderActionField_InstrumentID_set)
    ParkedOrderActionID = property(_thosttraderapi.CThostFtdcParkedOrderActionField_ParkedOrderActionID_get, _thosttraderapi.CThostFtdcParkedOrderActionField_ParkedOrderActionID_set)
    UserType = property(_thosttraderapi.CThostFtdcParkedOrderActionField_UserType_get, _thosttraderapi.CThostFtdcParkedOrderActionField_UserType_set)
    Status = property(_thosttraderapi.CThostFtdcParkedOrderActionField_Status_get, _thosttraderapi.CThostFtdcParkedOrderActionField_Status_set)
    ErrorID = property(_thosttraderapi.CThostFtdcParkedOrderActionField_ErrorID_get, _thosttraderapi.CThostFtdcParkedOrderActionField_ErrorID_set)
    ErrorMsg = property(_thosttraderapi.CThostFtdcParkedOrderActionField_ErrorMsg_get, _thosttraderapi.CThostFtdcParkedOrderActionField_ErrorMsg_set)
    InvestUnitID = property(_thosttraderapi.CThostFtdcParkedOrderActionField_InvestUnitID_get, _thosttraderapi.CThostFtdcParkedOrderActionField_InvestUnitID_set)
    IPAddress = property(_thosttraderapi.CThostFtdcParkedOrderActionField_IPAddress_get, _thosttraderapi.CThostFtdcParkedOrderActionField_IPAddress_set)
    MacAddress = property(_thosttraderapi.CThostFtdcParkedOrderActionField_MacAddress_get, _thosttraderapi.CThostFtdcParkedOrderActionField_MacAddress_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcParkedOrderActionField_swiginit(self, _thosttraderapi.new_CThostFtdcParkedOrderActionField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcParkedOrderActionField

# Register CThostFtdcParkedOrderActionField in _thosttraderapi:
_thosttraderapi.CThostFtdcParkedOrderActionField_swigregister(CThostFtdcParkedOrderActionField)

class CThostFtdcQryParkedOrderField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thosttraderapi.CThostFtdcQryParkedOrderField_BrokerID_get, _thosttraderapi.CThostFtdcQryParkedOrderField_BrokerID_set)
    InvestorID = property(_thosttraderapi.CThostFtdcQryParkedOrderField_InvestorID_get, _thosttraderapi.CThostFtdcQryParkedOrderField_InvestorID_set)
    InstrumentID = property(_thosttraderapi.CThostFtdcQryParkedOrderField_InstrumentID_get, _thosttraderapi.CThostFtdcQryParkedOrderField_InstrumentID_set)
    ExchangeID = property(_thosttraderapi.CThostFtdcQryParkedOrderField_ExchangeID_get, _thosttraderapi.CThostFtdcQryParkedOrderField_ExchangeID_set)
    InvestUnitID = property(_thosttraderapi.CThostFtdcQryParkedOrderField_InvestUnitID_get, _thosttraderapi.CThostFtdcQryParkedOrderField_InvestUnitID_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcQryParkedOrderField_swiginit(self, _thosttraderapi.new_CThostFtdcQryParkedOrderField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcQryParkedOrderField

# Register CThostFtdcQryParkedOrderField in _thosttraderapi:
_thosttraderapi.CThostFtdcQryParkedOrderField_swigregister(CThostFtdcQryParkedOrderField)

class CThostFtdcQryParkedOrderActionField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thosttraderapi.CThostFtdcQryParkedOrderActionField_BrokerID_get, _thosttraderapi.CThostFtdcQryParkedOrderActionField_BrokerID_set)
    InvestorID = property(_thosttraderapi.CThostFtdcQryParkedOrderActionField_InvestorID_get, _thosttraderapi.CThostFtdcQryParkedOrderActionField_InvestorID_set)
    InstrumentID = property(_thosttraderapi.CThostFtdcQryParkedOrderActionField_InstrumentID_get, _thosttraderapi.CThostFtdcQryParkedOrderActionField_InstrumentID_set)
    ExchangeID = property(_thosttraderapi.CThostFtdcQryParkedOrderActionField_ExchangeID_get, _thosttraderapi.CThostFtdcQryParkedOrderActionField_ExchangeID_set)
    InvestUnitID = property(_thosttraderapi.CThostFtdcQryParkedOrderActionField_InvestUnitID_get, _thosttraderapi.CThostFtdcQryParkedOrderActionField_InvestUnitID_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcQryParkedOrderActionField_swiginit(self, _thosttraderapi.new_CThostFtdcQryParkedOrderActionField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcQryParkedOrderActionField

# Register CThostFtdcQryParkedOrderActionField in _thosttraderapi:
_thosttraderapi.CThostFtdcQryParkedOrderActionField_swigregister(CThostFtdcQryParkedOrderActionField)

class CThostFtdcRemoveParkedOrderField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thosttraderapi.CThostFtdcRemoveParkedOrderField_BrokerID_get, _thosttraderapi.CThostFtdcRemoveParkedOrderField_BrokerID_set)
    InvestorID = property(_thosttraderapi.CThostFtdcRemoveParkedOrderField_InvestorID_get, _thosttraderapi.CThostFtdcRemoveParkedOrderField_InvestorID_set)
    ParkedOrderID = property(_thosttraderapi.CThostFtdcRemoveParkedOrderField_ParkedOrderID_get, _thosttraderapi.CThostFtdcRemoveParkedOrderField_ParkedOrderID_set)
    InvestUnitID = property(_thosttraderapi.CThostFtdcRemoveParkedOrderField_InvestUnitID_get, _thosttraderapi.CThostFtdcRemoveParkedOrderField_InvestUnitID_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcRemoveParkedOrderField_swiginit(self, _thosttraderapi.new_CThostFtdcRemoveParkedOrderField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcRemoveParkedOrderField

# Register CThostFtdcRemoveParkedOrderField in _thosttraderapi:
_thosttraderapi.CThostFtdcRemoveParkedOrderField_swigregister(CThostFtdcRemoveParkedOrderField)

class CThostFtdcRemoveParkedOrderActionField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thosttraderapi.CThostFtdcRemoveParkedOrderActionField_BrokerID_get, _thosttraderapi.CThostFtdcRemoveParkedOrderActionField_BrokerID_set)
    InvestorID = property(_thosttraderapi.CThostFtdcRemoveParkedOrderActionField_InvestorID_get, _thosttraderapi.CThostFtdcRemoveParkedOrderActionField_InvestorID_set)
    ParkedOrderActionID = property(_thosttraderapi.CThostFtdcRemoveParkedOrderActionField_ParkedOrderActionID_get, _thosttraderapi.CThostFtdcRemoveParkedOrderActionField_ParkedOrderActionID_set)
    InvestUnitID = property(_thosttraderapi.CThostFtdcRemoveParkedOrderActionField_InvestUnitID_get, _thosttraderapi.CThostFtdcRemoveParkedOrderActionField_InvestUnitID_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcRemoveParkedOrderActionField_swiginit(self, _thosttraderapi.new_CThostFtdcRemoveParkedOrderActionField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcRemoveParkedOrderActionField

# Register CThostFtdcRemoveParkedOrderActionField in _thosttraderapi:
_thosttraderapi.CThostFtdcRemoveParkedOrderActionField_swigregister(CThostFtdcRemoveParkedOrderActionField)

class CThostFtdcInvestorWithdrawAlgorithmField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thosttraderapi.CThostFtdcInvestorWithdrawAlgorithmField_BrokerID_get, _thosttraderapi.CThostFtdcInvestorWithdrawAlgorithmField_BrokerID_set)
    InvestorRange = property(_thosttraderapi.CThostFtdcInvestorWithdrawAlgorithmField_InvestorRange_get, _thosttraderapi.CThostFtdcInvestorWithdrawAlgorithmField_InvestorRange_set)
    InvestorID = property(_thosttraderapi.CThostFtdcInvestorWithdrawAlgorithmField_InvestorID_get, _thosttraderapi.CThostFtdcInvestorWithdrawAlgorithmField_InvestorID_set)
    UsingRatio = property(_thosttraderapi.CThostFtdcInvestorWithdrawAlgorithmField_UsingRatio_get, _thosttraderapi.CThostFtdcInvestorWithdrawAlgorithmField_UsingRatio_set)
    CurrencyID = property(_thosttraderapi.CThostFtdcInvestorWithdrawAlgorithmField_CurrencyID_get, _thosttraderapi.CThostFtdcInvestorWithdrawAlgorithmField_CurrencyID_set)
    FundMortgageRatio = property(_thosttraderapi.CThostFtdcInvestorWithdrawAlgorithmField_FundMortgageRatio_get, _thosttraderapi.CThostFtdcInvestorWithdrawAlgorithmField_FundMortgageRatio_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcInvestorWithdrawAlgorithmField_swiginit(self, _thosttraderapi.new_CThostFtdcInvestorWithdrawAlgorithmField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcInvestorWithdrawAlgorithmField

# Register CThostFtdcInvestorWithdrawAlgorithmField in _thosttraderapi:
_thosttraderapi.CThostFtdcInvestorWithdrawAlgorithmField_swigregister(CThostFtdcInvestorWithdrawAlgorithmField)

class CThostFtdcQryInvestorPositionCombineDetailField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thosttraderapi.CThostFtdcQryInvestorPositionCombineDetailField_BrokerID_get, _thosttraderapi.CThostFtdcQryInvestorPositionCombineDetailField_BrokerID_set)
    InvestorID = property(_thosttraderapi.CThostFtdcQryInvestorPositionCombineDetailField_InvestorID_get, _thosttraderapi.CThostFtdcQryInvestorPositionCombineDetailField_InvestorID_set)
    CombInstrumentID = property(_thosttraderapi.CThostFtdcQryInvestorPositionCombineDetailField_CombInstrumentID_get, _thosttraderapi.CThostFtdcQryInvestorPositionCombineDetailField_CombInstrumentID_set)
    ExchangeID = property(_thosttraderapi.CThostFtdcQryInvestorPositionCombineDetailField_ExchangeID_get, _thosttraderapi.CThostFtdcQryInvestorPositionCombineDetailField_ExchangeID_set)
    InvestUnitID = property(_thosttraderapi.CThostFtdcQryInvestorPositionCombineDetailField_InvestUnitID_get, _thosttraderapi.CThostFtdcQryInvestorPositionCombineDetailField_InvestUnitID_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcQryInvestorPositionCombineDetailField_swiginit(self, _thosttraderapi.new_CThostFtdcQryInvestorPositionCombineDetailField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcQryInvestorPositionCombineDetailField

# Register CThostFtdcQryInvestorPositionCombineDetailField in _thosttraderapi:
_thosttraderapi.CThostFtdcQryInvestorPositionCombineDetailField_swigregister(CThostFtdcQryInvestorPositionCombineDetailField)

class CThostFtdcMarketDataAveragePriceField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    AveragePrice = property(_thosttraderapi.CThostFtdcMarketDataAveragePriceField_AveragePrice_get, _thosttraderapi.CThostFtdcMarketDataAveragePriceField_AveragePrice_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcMarketDataAveragePriceField_swiginit(self, _thosttraderapi.new_CThostFtdcMarketDataAveragePriceField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcMarketDataAveragePriceField

# Register CThostFtdcMarketDataAveragePriceField in _thosttraderapi:
_thosttraderapi.CThostFtdcMarketDataAveragePriceField_swigregister(CThostFtdcMarketDataAveragePriceField)

class CThostFtdcVerifyInvestorPasswordField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thosttraderapi.CThostFtdcVerifyInvestorPasswordField_BrokerID_get, _thosttraderapi.CThostFtdcVerifyInvestorPasswordField_BrokerID_set)
    InvestorID = property(_thosttraderapi.CThostFtdcVerifyInvestorPasswordField_InvestorID_get, _thosttraderapi.CThostFtdcVerifyInvestorPasswordField_InvestorID_set)
    Password = property(_thosttraderapi.CThostFtdcVerifyInvestorPasswordField_Password_get, _thosttraderapi.CThostFtdcVerifyInvestorPasswordField_Password_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcVerifyInvestorPasswordField_swiginit(self, _thosttraderapi.new_CThostFtdcVerifyInvestorPasswordField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcVerifyInvestorPasswordField

# Register CThostFtdcVerifyInvestorPasswordField in _thosttraderapi:
_thosttraderapi.CThostFtdcVerifyInvestorPasswordField_swigregister(CThostFtdcVerifyInvestorPasswordField)

class CThostFtdcUserIPField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thosttraderapi.CThostFtdcUserIPField_BrokerID_get, _thosttraderapi.CThostFtdcUserIPField_BrokerID_set)
    UserID = property(_thosttraderapi.CThostFtdcUserIPField_UserID_get, _thosttraderapi.CThostFtdcUserIPField_UserID_set)
    IPAddress = property(_thosttraderapi.CThostFtdcUserIPField_IPAddress_get, _thosttraderapi.CThostFtdcUserIPField_IPAddress_set)
    IPMask = property(_thosttraderapi.CThostFtdcUserIPField_IPMask_get, _thosttraderapi.CThostFtdcUserIPField_IPMask_set)
    MacAddress = property(_thosttraderapi.CThostFtdcUserIPField_MacAddress_get, _thosttraderapi.CThostFtdcUserIPField_MacAddress_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcUserIPField_swiginit(self, _thosttraderapi.new_CThostFtdcUserIPField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcUserIPField

# Register CThostFtdcUserIPField in _thosttraderapi:
_thosttraderapi.CThostFtdcUserIPField_swigregister(CThostFtdcUserIPField)

class CThostFtdcTradingNoticeInfoField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thosttraderapi.CThostFtdcTradingNoticeInfoField_BrokerID_get, _thosttraderapi.CThostFtdcTradingNoticeInfoField_BrokerID_set)
    InvestorID = property(_thosttraderapi.CThostFtdcTradingNoticeInfoField_InvestorID_get, _thosttraderapi.CThostFtdcTradingNoticeInfoField_InvestorID_set)
    SendTime = property(_thosttraderapi.CThostFtdcTradingNoticeInfoField_SendTime_get, _thosttraderapi.CThostFtdcTradingNoticeInfoField_SendTime_set)
    FieldContent = property(_thosttraderapi.CThostFtdcTradingNoticeInfoField_FieldContent_get, _thosttraderapi.CThostFtdcTradingNoticeInfoField_FieldContent_set)
    SequenceSeries = property(_thosttraderapi.CThostFtdcTradingNoticeInfoField_SequenceSeries_get, _thosttraderapi.CThostFtdcTradingNoticeInfoField_SequenceSeries_set)
    SequenceNo = property(_thosttraderapi.CThostFtdcTradingNoticeInfoField_SequenceNo_get, _thosttraderapi.CThostFtdcTradingNoticeInfoField_SequenceNo_set)
    InvestUnitID = property(_thosttraderapi.CThostFtdcTradingNoticeInfoField_InvestUnitID_get, _thosttraderapi.CThostFtdcTradingNoticeInfoField_InvestUnitID_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcTradingNoticeInfoField_swiginit(self, _thosttraderapi.new_CThostFtdcTradingNoticeInfoField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcTradingNoticeInfoField

# Register CThostFtdcTradingNoticeInfoField in _thosttraderapi:
_thosttraderapi.CThostFtdcTradingNoticeInfoField_swigregister(CThostFtdcTradingNoticeInfoField)

class CThostFtdcTradingNoticeField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thosttraderapi.CThostFtdcTradingNoticeField_BrokerID_get, _thosttraderapi.CThostFtdcTradingNoticeField_BrokerID_set)
    InvestorRange = property(_thosttraderapi.CThostFtdcTradingNoticeField_InvestorRange_get, _thosttraderapi.CThostFtdcTradingNoticeField_InvestorRange_set)
    InvestorID = property(_thosttraderapi.CThostFtdcTradingNoticeField_InvestorID_get, _thosttraderapi.CThostFtdcTradingNoticeField_InvestorID_set)
    SequenceSeries = property(_thosttraderapi.CThostFtdcTradingNoticeField_SequenceSeries_get, _thosttraderapi.CThostFtdcTradingNoticeField_SequenceSeries_set)
    UserID = property(_thosttraderapi.CThostFtdcTradingNoticeField_UserID_get, _thosttraderapi.CThostFtdcTradingNoticeField_UserID_set)
    SendTime = property(_thosttraderapi.CThostFtdcTradingNoticeField_SendTime_get, _thosttraderapi.CThostFtdcTradingNoticeField_SendTime_set)
    SequenceNo = property(_thosttraderapi.CThostFtdcTradingNoticeField_SequenceNo_get, _thosttraderapi.CThostFtdcTradingNoticeField_SequenceNo_set)
    FieldContent = property(_thosttraderapi.CThostFtdcTradingNoticeField_FieldContent_get, _thosttraderapi.CThostFtdcTradingNoticeField_FieldContent_set)
    InvestUnitID = property(_thosttraderapi.CThostFtdcTradingNoticeField_InvestUnitID_get, _thosttraderapi.CThostFtdcTradingNoticeField_InvestUnitID_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcTradingNoticeField_swiginit(self, _thosttraderapi.new_CThostFtdcTradingNoticeField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcTradingNoticeField

# Register CThostFtdcTradingNoticeField in _thosttraderapi:
_thosttraderapi.CThostFtdcTradingNoticeField_swigregister(CThostFtdcTradingNoticeField)

class CThostFtdcQryTradingNoticeField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thosttraderapi.CThostFtdcQryTradingNoticeField_BrokerID_get, _thosttraderapi.CThostFtdcQryTradingNoticeField_BrokerID_set)
    InvestorID = property(_thosttraderapi.CThostFtdcQryTradingNoticeField_InvestorID_get, _thosttraderapi.CThostFtdcQryTradingNoticeField_InvestorID_set)
    InvestUnitID = property(_thosttraderapi.CThostFtdcQryTradingNoticeField_InvestUnitID_get, _thosttraderapi.CThostFtdcQryTradingNoticeField_InvestUnitID_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcQryTradingNoticeField_swiginit(self, _thosttraderapi.new_CThostFtdcQryTradingNoticeField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcQryTradingNoticeField

# Register CThostFtdcQryTradingNoticeField in _thosttraderapi:
_thosttraderapi.CThostFtdcQryTradingNoticeField_swigregister(CThostFtdcQryTradingNoticeField)

class CThostFtdcQryErrOrderField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thosttraderapi.CThostFtdcQryErrOrderField_BrokerID_get, _thosttraderapi.CThostFtdcQryErrOrderField_BrokerID_set)
    InvestorID = property(_thosttraderapi.CThostFtdcQryErrOrderField_InvestorID_get, _thosttraderapi.CThostFtdcQryErrOrderField_InvestorID_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcQryErrOrderField_swiginit(self, _thosttraderapi.new_CThostFtdcQryErrOrderField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcQryErrOrderField

# Register CThostFtdcQryErrOrderField in _thosttraderapi:
_thosttraderapi.CThostFtdcQryErrOrderField_swigregister(CThostFtdcQryErrOrderField)

class CThostFtdcErrOrderField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thosttraderapi.CThostFtdcErrOrderField_BrokerID_get, _thosttraderapi.CThostFtdcErrOrderField_BrokerID_set)
    InvestorID = property(_thosttraderapi.CThostFtdcErrOrderField_InvestorID_get, _thosttraderapi.CThostFtdcErrOrderField_InvestorID_set)
    InstrumentID = property(_thosttraderapi.CThostFtdcErrOrderField_InstrumentID_get, _thosttraderapi.CThostFtdcErrOrderField_InstrumentID_set)
    OrderRef = property(_thosttraderapi.CThostFtdcErrOrderField_OrderRef_get, _thosttraderapi.CThostFtdcErrOrderField_OrderRef_set)
    UserID = property(_thosttraderapi.CThostFtdcErrOrderField_UserID_get, _thosttraderapi.CThostFtdcErrOrderField_UserID_set)
    OrderPriceType = property(_thosttraderapi.CThostFtdcErrOrderField_OrderPriceType_get, _thosttraderapi.CThostFtdcErrOrderField_OrderPriceType_set)
    Direction = property(_thosttraderapi.CThostFtdcErrOrderField_Direction_get, _thosttraderapi.CThostFtdcErrOrderField_Direction_set)
    CombOffsetFlag = property(_thosttraderapi.CThostFtdcErrOrderField_CombOffsetFlag_get, _thosttraderapi.CThostFtdcErrOrderField_CombOffsetFlag_set)
    CombHedgeFlag = property(_thosttraderapi.CThostFtdcErrOrderField_CombHedgeFlag_get, _thosttraderapi.CThostFtdcErrOrderField_CombHedgeFlag_set)
    LimitPrice = property(_thosttraderapi.CThostFtdcErrOrderField_LimitPrice_get, _thosttraderapi.CThostFtdcErrOrderField_LimitPrice_set)
    VolumeTotalOriginal = property(_thosttraderapi.CThostFtdcErrOrderField_VolumeTotalOriginal_get, _thosttraderapi.CThostFtdcErrOrderField_VolumeTotalOriginal_set)
    TimeCondition = property(_thosttraderapi.CThostFtdcErrOrderField_TimeCondition_get, _thosttraderapi.CThostFtdcErrOrderField_TimeCondition_set)
    GTDDate = property(_thosttraderapi.CThostFtdcErrOrderField_GTDDate_get, _thosttraderapi.CThostFtdcErrOrderField_GTDDate_set)
    VolumeCondition = property(_thosttraderapi.CThostFtdcErrOrderField_VolumeCondition_get, _thosttraderapi.CThostFtdcErrOrderField_VolumeCondition_set)
    MinVolume = property(_thosttraderapi.CThostFtdcErrOrderField_MinVolume_get, _thosttraderapi.CThostFtdcErrOrderField_MinVolume_set)
    ContingentCondition = property(_thosttraderapi.CThostFtdcErrOrderField_ContingentCondition_get, _thosttraderapi.CThostFtdcErrOrderField_ContingentCondition_set)
    StopPrice = property(_thosttraderapi.CThostFtdcErrOrderField_StopPrice_get, _thosttraderapi.CThostFtdcErrOrderField_StopPrice_set)
    ForceCloseReason = property(_thosttraderapi.CThostFtdcErrOrderField_ForceCloseReason_get, _thosttraderapi.CThostFtdcErrOrderField_ForceCloseReason_set)
    IsAutoSuspend = property(_thosttraderapi.CThostFtdcErrOrderField_IsAutoSuspend_get, _thosttraderapi.CThostFtdcErrOrderField_IsAutoSuspend_set)
    BusinessUnit = property(_thosttraderapi.CThostFtdcErrOrderField_BusinessUnit_get, _thosttraderapi.CThostFtdcErrOrderField_BusinessUnit_set)
    RequestID = property(_thosttraderapi.CThostFtdcErrOrderField_RequestID_get, _thosttraderapi.CThostFtdcErrOrderField_RequestID_set)
    UserForceClose = property(_thosttraderapi.CThostFtdcErrOrderField_UserForceClose_get, _thosttraderapi.CThostFtdcErrOrderField_UserForceClose_set)
    ErrorID = property(_thosttraderapi.CThostFtdcErrOrderField_ErrorID_get, _thosttraderapi.CThostFtdcErrOrderField_ErrorID_set)
    ErrorMsg = property(_thosttraderapi.CThostFtdcErrOrderField_ErrorMsg_get, _thosttraderapi.CThostFtdcErrOrderField_ErrorMsg_set)
    IsSwapOrder = property(_thosttraderapi.CThostFtdcErrOrderField_IsSwapOrder_get, _thosttraderapi.CThostFtdcErrOrderField_IsSwapOrder_set)
    ExchangeID = property(_thosttraderapi.CThostFtdcErrOrderField_ExchangeID_get, _thosttraderapi.CThostFtdcErrOrderField_ExchangeID_set)
    InvestUnitID = property(_thosttraderapi.CThostFtdcErrOrderField_InvestUnitID_get, _thosttraderapi.CThostFtdcErrOrderField_InvestUnitID_set)
    AccountID = property(_thosttraderapi.CThostFtdcErrOrderField_AccountID_get, _thosttraderapi.CThostFtdcErrOrderField_AccountID_set)
    CurrencyID = property(_thosttraderapi.CThostFtdcErrOrderField_CurrencyID_get, _thosttraderapi.CThostFtdcErrOrderField_CurrencyID_set)
    ClientID = property(_thosttraderapi.CThostFtdcErrOrderField_ClientID_get, _thosttraderapi.CThostFtdcErrOrderField_ClientID_set)
    IPAddress = property(_thosttraderapi.CThostFtdcErrOrderField_IPAddress_get, _thosttraderapi.CThostFtdcErrOrderField_IPAddress_set)
    MacAddress = property(_thosttraderapi.CThostFtdcErrOrderField_MacAddress_get, _thosttraderapi.CThostFtdcErrOrderField_MacAddress_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcErrOrderField_swiginit(self, _thosttraderapi.new_CThostFtdcErrOrderField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcErrOrderField

# Register CThostFtdcErrOrderField in _thosttraderapi:
_thosttraderapi.CThostFtdcErrOrderField_swigregister(CThostFtdcErrOrderField)

class CThostFtdcErrorConditionalOrderField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thosttraderapi.CThostFtdcErrorConditionalOrderField_BrokerID_get, _thosttraderapi.CThostFtdcErrorConditionalOrderField_BrokerID_set)
    InvestorID = property(_thosttraderapi.CThostFtdcErrorConditionalOrderField_InvestorID_get, _thosttraderapi.CThostFtdcErrorConditionalOrderField_InvestorID_set)
    InstrumentID = property(_thosttraderapi.CThostFtdcErrorConditionalOrderField_InstrumentID_get, _thosttraderapi.CThostFtdcErrorConditionalOrderField_InstrumentID_set)
    OrderRef = property(_thosttraderapi.CThostFtdcErrorConditionalOrderField_OrderRef_get, _thosttraderapi.CThostFtdcErrorConditionalOrderField_OrderRef_set)
    UserID = property(_thosttraderapi.CThostFtdcErrorConditionalOrderField_UserID_get, _thosttraderapi.CThostFtdcErrorConditionalOrderField_UserID_set)
    OrderPriceType = property(_thosttraderapi.CThostFtdcErrorConditionalOrderField_OrderPriceType_get, _thosttraderapi.CThostFtdcErrorConditionalOrderField_OrderPriceType_set)
    Direction = property(_thosttraderapi.CThostFtdcErrorConditionalOrderField_Direction_get, _thosttraderapi.CThostFtdcErrorConditionalOrderField_Direction_set)
    CombOffsetFlag = property(_thosttraderapi.CThostFtdcErrorConditionalOrderField_CombOffsetFlag_get, _thosttraderapi.CThostFtdcErrorConditionalOrderField_CombOffsetFlag_set)
    CombHedgeFlag = property(_thosttraderapi.CThostFtdcErrorConditionalOrderField_CombHedgeFlag_get, _thosttraderapi.CThostFtdcErrorConditionalOrderField_CombHedgeFlag_set)
    LimitPrice = property(_thosttraderapi.CThostFtdcErrorConditionalOrderField_LimitPrice_get, _thosttraderapi.CThostFtdcErrorConditionalOrderField_LimitPrice_set)
    VolumeTotalOriginal = property(_thosttraderapi.CThostFtdcErrorConditionalOrderField_VolumeTotalOriginal_get, _thosttraderapi.CThostFtdcErrorConditionalOrderField_VolumeTotalOriginal_set)
    TimeCondition = property(_thosttraderapi.CThostFtdcErrorConditionalOrderField_TimeCondition_get, _thosttraderapi.CThostFtdcErrorConditionalOrderField_TimeCondition_set)
    GTDDate = property(_thosttraderapi.CThostFtdcErrorConditionalOrderField_GTDDate_get, _thosttraderapi.CThostFtdcErrorConditionalOrderField_GTDDate_set)
    VolumeCondition = property(_thosttraderapi.CThostFtdcErrorConditionalOrderField_VolumeCondition_get, _thosttraderapi.CThostFtdcErrorConditionalOrderField_VolumeCondition_set)
    MinVolume = property(_thosttraderapi.CThostFtdcErrorConditionalOrderField_MinVolume_get, _thosttraderapi.CThostFtdcErrorConditionalOrderField_MinVolume_set)
    ContingentCondition = property(_thosttraderapi.CThostFtdcErrorConditionalOrderField_ContingentCondition_get, _thosttraderapi.CThostFtdcErrorConditionalOrderField_ContingentCondition_set)
    StopPrice = property(_thosttraderapi.CThostFtdcErrorConditionalOrderField_StopPrice_get, _thosttraderapi.CThostFtdcErrorConditionalOrderField_StopPrice_set)
    ForceCloseReason = property(_thosttraderapi.CThostFtdcErrorConditionalOrderField_ForceCloseReason_get, _thosttraderapi.CThostFtdcErrorConditionalOrderField_ForceCloseReason_set)
    IsAutoSuspend = property(_thosttraderapi.CThostFtdcErrorConditionalOrderField_IsAutoSuspend_get, _thosttraderapi.CThostFtdcErrorConditionalOrderField_IsAutoSuspend_set)
    BusinessUnit = property(_thosttraderapi.CThostFtdcErrorConditionalOrderField_BusinessUnit_get, _thosttraderapi.CThostFtdcErrorConditionalOrderField_BusinessUnit_set)
    RequestID = property(_thosttraderapi.CThostFtdcErrorConditionalOrderField_RequestID_get, _thosttraderapi.CThostFtdcErrorConditionalOrderField_RequestID_set)
    OrderLocalID = property(_thosttraderapi.CThostFtdcErrorConditionalOrderField_OrderLocalID_get, _thosttraderapi.CThostFtdcErrorConditionalOrderField_OrderLocalID_set)
    ExchangeID = property(_thosttraderapi.CThostFtdcErrorConditionalOrderField_ExchangeID_get, _thosttraderapi.CThostFtdcErrorConditionalOrderField_ExchangeID_set)
    ParticipantID = property(_thosttraderapi.CThostFtdcErrorConditionalOrderField_ParticipantID_get, _thosttraderapi.CThostFtdcErrorConditionalOrderField_ParticipantID_set)
    ClientID = property(_thosttraderapi.CThostFtdcErrorConditionalOrderField_ClientID_get, _thosttraderapi.CThostFtdcErrorConditionalOrderField_ClientID_set)
    ExchangeInstID = property(_thosttraderapi.CThostFtdcErrorConditionalOrderField_ExchangeInstID_get, _thosttraderapi.CThostFtdcErrorConditionalOrderField_ExchangeInstID_set)
    TraderID = property(_thosttraderapi.CThostFtdcErrorConditionalOrderField_TraderID_get, _thosttraderapi.CThostFtdcErrorConditionalOrderField_TraderID_set)
    InstallID = property(_thosttraderapi.CThostFtdcErrorConditionalOrderField_InstallID_get, _thosttraderapi.CThostFtdcErrorConditionalOrderField_InstallID_set)
    OrderSubmitStatus = property(_thosttraderapi.CThostFtdcErrorConditionalOrderField_OrderSubmitStatus_get, _thosttraderapi.CThostFtdcErrorConditionalOrderField_OrderSubmitStatus_set)
    NotifySequence = property(_thosttraderapi.CThostFtdcErrorConditionalOrderField_NotifySequence_get, _thosttraderapi.CThostFtdcErrorConditionalOrderField_NotifySequence_set)
    TradingDay = property(_thosttraderapi.CThostFtdcErrorConditionalOrderField_TradingDay_get, _thosttraderapi.CThostFtdcErrorConditionalOrderField_TradingDay_set)
    SettlementID = property(_thosttraderapi.CThostFtdcErrorConditionalOrderField_SettlementID_get, _thosttraderapi.CThostFtdcErrorConditionalOrderField_SettlementID_set)
    OrderSysID = property(_thosttraderapi.CThostFtdcErrorConditionalOrderField_OrderSysID_get, _thosttraderapi.CThostFtdcErrorConditionalOrderField_OrderSysID_set)
    OrderSource = property(_thosttraderapi.CThostFtdcErrorConditionalOrderField_OrderSource_get, _thosttraderapi.CThostFtdcErrorConditionalOrderField_OrderSource_set)
    OrderStatus = property(_thosttraderapi.CThostFtdcErrorConditionalOrderField_OrderStatus_get, _thosttraderapi.CThostFtdcErrorConditionalOrderField_OrderStatus_set)
    OrderType = property(_thosttraderapi.CThostFtdcErrorConditionalOrderField_OrderType_get, _thosttraderapi.CThostFtdcErrorConditionalOrderField_OrderType_set)
    VolumeTraded = property(_thosttraderapi.CThostFtdcErrorConditionalOrderField_VolumeTraded_get, _thosttraderapi.CThostFtdcErrorConditionalOrderField_VolumeTraded_set)
    VolumeTotal = property(_thosttraderapi.CThostFtdcErrorConditionalOrderField_VolumeTotal_get, _thosttraderapi.CThostFtdcErrorConditionalOrderField_VolumeTotal_set)
    InsertDate = property(_thosttraderapi.CThostFtdcErrorConditionalOrderField_InsertDate_get, _thosttraderapi.CThostFtdcErrorConditionalOrderField_InsertDate_set)
    InsertTime = property(_thosttraderapi.CThostFtdcErrorConditionalOrderField_InsertTime_get, _thosttraderapi.CThostFtdcErrorConditionalOrderField_InsertTime_set)
    ActiveTime = property(_thosttraderapi.CThostFtdcErrorConditionalOrderField_ActiveTime_get, _thosttraderapi.CThostFtdcErrorConditionalOrderField_ActiveTime_set)
    SuspendTime = property(_thosttraderapi.CThostFtdcErrorConditionalOrderField_SuspendTime_get, _thosttraderapi.CThostFtdcErrorConditionalOrderField_SuspendTime_set)
    UpdateTime = property(_thosttraderapi.CThostFtdcErrorConditionalOrderField_UpdateTime_get, _thosttraderapi.CThostFtdcErrorConditionalOrderField_UpdateTime_set)
    CancelTime = property(_thosttraderapi.CThostFtdcErrorConditionalOrderField_CancelTime_get, _thosttraderapi.CThostFtdcErrorConditionalOrderField_CancelTime_set)
    ActiveTraderID = property(_thosttraderapi.CThostFtdcErrorConditionalOrderField_ActiveTraderID_get, _thosttraderapi.CThostFtdcErrorConditionalOrderField_ActiveTraderID_set)
    ClearingPartID = property(_thosttraderapi.CThostFtdcErrorConditionalOrderField_ClearingPartID_get, _thosttraderapi.CThostFtdcErrorConditionalOrderField_ClearingPartID_set)
    SequenceNo = property(_thosttraderapi.CThostFtdcErrorConditionalOrderField_SequenceNo_get, _thosttraderapi.CThostFtdcErrorConditionalOrderField_SequenceNo_set)
    FrontID = property(_thosttraderapi.CThostFtdcErrorConditionalOrderField_FrontID_get, _thosttraderapi.CThostFtdcErrorConditionalOrderField_FrontID_set)
    SessionID = property(_thosttraderapi.CThostFtdcErrorConditionalOrderField_SessionID_get, _thosttraderapi.CThostFtdcErrorConditionalOrderField_SessionID_set)
    UserProductInfo = property(_thosttraderapi.CThostFtdcErrorConditionalOrderField_UserProductInfo_get, _thosttraderapi.CThostFtdcErrorConditionalOrderField_UserProductInfo_set)
    StatusMsg = property(_thosttraderapi.CThostFtdcErrorConditionalOrderField_StatusMsg_get, _thosttraderapi.CThostFtdcErrorConditionalOrderField_StatusMsg_set)
    UserForceClose = property(_thosttraderapi.CThostFtdcErrorConditionalOrderField_UserForceClose_get, _thosttraderapi.CThostFtdcErrorConditionalOrderField_UserForceClose_set)
    ActiveUserID = property(_thosttraderapi.CThostFtdcErrorConditionalOrderField_ActiveUserID_get, _thosttraderapi.CThostFtdcErrorConditionalOrderField_ActiveUserID_set)
    BrokerOrderSeq = property(_thosttraderapi.CThostFtdcErrorConditionalOrderField_BrokerOrderSeq_get, _thosttraderapi.CThostFtdcErrorConditionalOrderField_BrokerOrderSeq_set)
    RelativeOrderSysID = property(_thosttraderapi.CThostFtdcErrorConditionalOrderField_RelativeOrderSysID_get, _thosttraderapi.CThostFtdcErrorConditionalOrderField_RelativeOrderSysID_set)
    ZCETotalTradedVolume = property(_thosttraderapi.CThostFtdcErrorConditionalOrderField_ZCETotalTradedVolume_get, _thosttraderapi.CThostFtdcErrorConditionalOrderField_ZCETotalTradedVolume_set)
    ErrorID = property(_thosttraderapi.CThostFtdcErrorConditionalOrderField_ErrorID_get, _thosttraderapi.CThostFtdcErrorConditionalOrderField_ErrorID_set)
    ErrorMsg = property(_thosttraderapi.CThostFtdcErrorConditionalOrderField_ErrorMsg_get, _thosttraderapi.CThostFtdcErrorConditionalOrderField_ErrorMsg_set)
    IsSwapOrder = property(_thosttraderapi.CThostFtdcErrorConditionalOrderField_IsSwapOrder_get, _thosttraderapi.CThostFtdcErrorConditionalOrderField_IsSwapOrder_set)
    BranchID = property(_thosttraderapi.CThostFtdcErrorConditionalOrderField_BranchID_get, _thosttraderapi.CThostFtdcErrorConditionalOrderField_BranchID_set)
    InvestUnitID = property(_thosttraderapi.CThostFtdcErrorConditionalOrderField_InvestUnitID_get, _thosttraderapi.CThostFtdcErrorConditionalOrderField_InvestUnitID_set)
    AccountID = property(_thosttraderapi.CThostFtdcErrorConditionalOrderField_AccountID_get, _thosttraderapi.CThostFtdcErrorConditionalOrderField_AccountID_set)
    CurrencyID = property(_thosttraderapi.CThostFtdcErrorConditionalOrderField_CurrencyID_get, _thosttraderapi.CThostFtdcErrorConditionalOrderField_CurrencyID_set)
    IPAddress = property(_thosttraderapi.CThostFtdcErrorConditionalOrderField_IPAddress_get, _thosttraderapi.CThostFtdcErrorConditionalOrderField_IPAddress_set)
    MacAddress = property(_thosttraderapi.CThostFtdcErrorConditionalOrderField_MacAddress_get, _thosttraderapi.CThostFtdcErrorConditionalOrderField_MacAddress_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcErrorConditionalOrderField_swiginit(self, _thosttraderapi.new_CThostFtdcErrorConditionalOrderField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcErrorConditionalOrderField

# Register CThostFtdcErrorConditionalOrderField in _thosttraderapi:
_thosttraderapi.CThostFtdcErrorConditionalOrderField_swigregister(CThostFtdcErrorConditionalOrderField)

class CThostFtdcQryErrOrderActionField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thosttraderapi.CThostFtdcQryErrOrderActionField_BrokerID_get, _thosttraderapi.CThostFtdcQryErrOrderActionField_BrokerID_set)
    InvestorID = property(_thosttraderapi.CThostFtdcQryErrOrderActionField_InvestorID_get, _thosttraderapi.CThostFtdcQryErrOrderActionField_InvestorID_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcQryErrOrderActionField_swiginit(self, _thosttraderapi.new_CThostFtdcQryErrOrderActionField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcQryErrOrderActionField

# Register CThostFtdcQryErrOrderActionField in _thosttraderapi:
_thosttraderapi.CThostFtdcQryErrOrderActionField_swigregister(CThostFtdcQryErrOrderActionField)

class CThostFtdcErrOrderActionField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thosttraderapi.CThostFtdcErrOrderActionField_BrokerID_get, _thosttraderapi.CThostFtdcErrOrderActionField_BrokerID_set)
    InvestorID = property(_thosttraderapi.CThostFtdcErrOrderActionField_InvestorID_get, _thosttraderapi.CThostFtdcErrOrderActionField_InvestorID_set)
    OrderActionRef = property(_thosttraderapi.CThostFtdcErrOrderActionField_OrderActionRef_get, _thosttraderapi.CThostFtdcErrOrderActionField_OrderActionRef_set)
    OrderRef = property(_thosttraderapi.CThostFtdcErrOrderActionField_OrderRef_get, _thosttraderapi.CThostFtdcErrOrderActionField_OrderRef_set)
    RequestID = property(_thosttraderapi.CThostFtdcErrOrderActionField_RequestID_get, _thosttraderapi.CThostFtdcErrOrderActionField_RequestID_set)
    FrontID = property(_thosttraderapi.CThostFtdcErrOrderActionField_FrontID_get, _thosttraderapi.CThostFtdcErrOrderActionField_FrontID_set)
    SessionID = property(_thosttraderapi.CThostFtdcErrOrderActionField_SessionID_get, _thosttraderapi.CThostFtdcErrOrderActionField_SessionID_set)
    ExchangeID = property(_thosttraderapi.CThostFtdcErrOrderActionField_ExchangeID_get, _thosttraderapi.CThostFtdcErrOrderActionField_ExchangeID_set)
    OrderSysID = property(_thosttraderapi.CThostFtdcErrOrderActionField_OrderSysID_get, _thosttraderapi.CThostFtdcErrOrderActionField_OrderSysID_set)
    ActionFlag = property(_thosttraderapi.CThostFtdcErrOrderActionField_ActionFlag_get, _thosttraderapi.CThostFtdcErrOrderActionField_ActionFlag_set)
    LimitPrice = property(_thosttraderapi.CThostFtdcErrOrderActionField_LimitPrice_get, _thosttraderapi.CThostFtdcErrOrderActionField_LimitPrice_set)
    VolumeChange = property(_thosttraderapi.CThostFtdcErrOrderActionField_VolumeChange_get, _thosttraderapi.CThostFtdcErrOrderActionField_VolumeChange_set)
    ActionDate = property(_thosttraderapi.CThostFtdcErrOrderActionField_ActionDate_get, _thosttraderapi.CThostFtdcErrOrderActionField_ActionDate_set)
    ActionTime = property(_thosttraderapi.CThostFtdcErrOrderActionField_ActionTime_get, _thosttraderapi.CThostFtdcErrOrderActionField_ActionTime_set)
    TraderID = property(_thosttraderapi.CThostFtdcErrOrderActionField_TraderID_get, _thosttraderapi.CThostFtdcErrOrderActionField_TraderID_set)
    InstallID = property(_thosttraderapi.CThostFtdcErrOrderActionField_InstallID_get, _thosttraderapi.CThostFtdcErrOrderActionField_InstallID_set)
    OrderLocalID = property(_thosttraderapi.CThostFtdcErrOrderActionField_OrderLocalID_get, _thosttraderapi.CThostFtdcErrOrderActionField_OrderLocalID_set)
    ActionLocalID = property(_thosttraderapi.CThostFtdcErrOrderActionField_ActionLocalID_get, _thosttraderapi.CThostFtdcErrOrderActionField_ActionLocalID_set)
    ParticipantID = property(_thosttraderapi.CThostFtdcErrOrderActionField_ParticipantID_get, _thosttraderapi.CThostFtdcErrOrderActionField_ParticipantID_set)
    ClientID = property(_thosttraderapi.CThostFtdcErrOrderActionField_ClientID_get, _thosttraderapi.CThostFtdcErrOrderActionField_ClientID_set)
    BusinessUnit = property(_thosttraderapi.CThostFtdcErrOrderActionField_BusinessUnit_get, _thosttraderapi.CThostFtdcErrOrderActionField_BusinessUnit_set)
    OrderActionStatus = property(_thosttraderapi.CThostFtdcErrOrderActionField_OrderActionStatus_get, _thosttraderapi.CThostFtdcErrOrderActionField_OrderActionStatus_set)
    UserID = property(_thosttraderapi.CThostFtdcErrOrderActionField_UserID_get, _thosttraderapi.CThostFtdcErrOrderActionField_UserID_set)
    StatusMsg = property(_thosttraderapi.CThostFtdcErrOrderActionField_StatusMsg_get, _thosttraderapi.CThostFtdcErrOrderActionField_StatusMsg_set)
    InstrumentID = property(_thosttraderapi.CThostFtdcErrOrderActionField_InstrumentID_get, _thosttraderapi.CThostFtdcErrOrderActionField_InstrumentID_set)
    BranchID = property(_thosttraderapi.CThostFtdcErrOrderActionField_BranchID_get, _thosttraderapi.CThostFtdcErrOrderActionField_BranchID_set)
    InvestUnitID = property(_thosttraderapi.CThostFtdcErrOrderActionField_InvestUnitID_get, _thosttraderapi.CThostFtdcErrOrderActionField_InvestUnitID_set)
    IPAddress = property(_thosttraderapi.CThostFtdcErrOrderActionField_IPAddress_get, _thosttraderapi.CThostFtdcErrOrderActionField_IPAddress_set)
    MacAddress = property(_thosttraderapi.CThostFtdcErrOrderActionField_MacAddress_get, _thosttraderapi.CThostFtdcErrOrderActionField_MacAddress_set)
    ErrorID = property(_thosttraderapi.CThostFtdcErrOrderActionField_ErrorID_get, _thosttraderapi.CThostFtdcErrOrderActionField_ErrorID_set)
    ErrorMsg = property(_thosttraderapi.CThostFtdcErrOrderActionField_ErrorMsg_get, _thosttraderapi.CThostFtdcErrOrderActionField_ErrorMsg_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcErrOrderActionField_swiginit(self, _thosttraderapi.new_CThostFtdcErrOrderActionField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcErrOrderActionField

# Register CThostFtdcErrOrderActionField in _thosttraderapi:
_thosttraderapi.CThostFtdcErrOrderActionField_swigregister(CThostFtdcErrOrderActionField)

class CThostFtdcQryExchangeSequenceField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    ExchangeID = property(_thosttraderapi.CThostFtdcQryExchangeSequenceField_ExchangeID_get, _thosttraderapi.CThostFtdcQryExchangeSequenceField_ExchangeID_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcQryExchangeSequenceField_swiginit(self, _thosttraderapi.new_CThostFtdcQryExchangeSequenceField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcQryExchangeSequenceField

# Register CThostFtdcQryExchangeSequenceField in _thosttraderapi:
_thosttraderapi.CThostFtdcQryExchangeSequenceField_swigregister(CThostFtdcQryExchangeSequenceField)

class CThostFtdcExchangeSequenceField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    ExchangeID = property(_thosttraderapi.CThostFtdcExchangeSequenceField_ExchangeID_get, _thosttraderapi.CThostFtdcExchangeSequenceField_ExchangeID_set)
    SequenceNo = property(_thosttraderapi.CThostFtdcExchangeSequenceField_SequenceNo_get, _thosttraderapi.CThostFtdcExchangeSequenceField_SequenceNo_set)
    MarketStatus = property(_thosttraderapi.CThostFtdcExchangeSequenceField_MarketStatus_get, _thosttraderapi.CThostFtdcExchangeSequenceField_MarketStatus_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcExchangeSequenceField_swiginit(self, _thosttraderapi.new_CThostFtdcExchangeSequenceField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcExchangeSequenceField

# Register CThostFtdcExchangeSequenceField in _thosttraderapi:
_thosttraderapi.CThostFtdcExchangeSequenceField_swigregister(CThostFtdcExchangeSequenceField)

class CThostFtdcQueryMaxOrderVolumeWithPriceField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thosttraderapi.CThostFtdcQueryMaxOrderVolumeWithPriceField_BrokerID_get, _thosttraderapi.CThostFtdcQueryMaxOrderVolumeWithPriceField_BrokerID_set)
    InvestorID = property(_thosttraderapi.CThostFtdcQueryMaxOrderVolumeWithPriceField_InvestorID_get, _thosttraderapi.CThostFtdcQueryMaxOrderVolumeWithPriceField_InvestorID_set)
    InstrumentID = property(_thosttraderapi.CThostFtdcQueryMaxOrderVolumeWithPriceField_InstrumentID_get, _thosttraderapi.CThostFtdcQueryMaxOrderVolumeWithPriceField_InstrumentID_set)
    Direction = property(_thosttraderapi.CThostFtdcQueryMaxOrderVolumeWithPriceField_Direction_get, _thosttraderapi.CThostFtdcQueryMaxOrderVolumeWithPriceField_Direction_set)
    OffsetFlag = property(_thosttraderapi.CThostFtdcQueryMaxOrderVolumeWithPriceField_OffsetFlag_get, _thosttraderapi.CThostFtdcQueryMaxOrderVolumeWithPriceField_OffsetFlag_set)
    HedgeFlag = property(_thosttraderapi.CThostFtdcQueryMaxOrderVolumeWithPriceField_HedgeFlag_get, _thosttraderapi.CThostFtdcQueryMaxOrderVolumeWithPriceField_HedgeFlag_set)
    MaxVolume = property(_thosttraderapi.CThostFtdcQueryMaxOrderVolumeWithPriceField_MaxVolume_get, _thosttraderapi.CThostFtdcQueryMaxOrderVolumeWithPriceField_MaxVolume_set)
    Price = property(_thosttraderapi.CThostFtdcQueryMaxOrderVolumeWithPriceField_Price_get, _thosttraderapi.CThostFtdcQueryMaxOrderVolumeWithPriceField_Price_set)
    ExchangeID = property(_thosttraderapi.CThostFtdcQueryMaxOrderVolumeWithPriceField_ExchangeID_get, _thosttraderapi.CThostFtdcQueryMaxOrderVolumeWithPriceField_ExchangeID_set)
    InvestUnitID = property(_thosttraderapi.CThostFtdcQueryMaxOrderVolumeWithPriceField_InvestUnitID_get, _thosttraderapi.CThostFtdcQueryMaxOrderVolumeWithPriceField_InvestUnitID_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcQueryMaxOrderVolumeWithPriceField_swiginit(self, _thosttraderapi.new_CThostFtdcQueryMaxOrderVolumeWithPriceField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcQueryMaxOrderVolumeWithPriceField

# Register CThostFtdcQueryMaxOrderVolumeWithPriceField in _thosttraderapi:
_thosttraderapi.CThostFtdcQueryMaxOrderVolumeWithPriceField_swigregister(CThostFtdcQueryMaxOrderVolumeWithPriceField)

class CThostFtdcQryBrokerTradingParamsField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thosttraderapi.CThostFtdcQryBrokerTradingParamsField_BrokerID_get, _thosttraderapi.CThostFtdcQryBrokerTradingParamsField_BrokerID_set)
    InvestorID = property(_thosttraderapi.CThostFtdcQryBrokerTradingParamsField_InvestorID_get, _thosttraderapi.CThostFtdcQryBrokerTradingParamsField_InvestorID_set)
    CurrencyID = property(_thosttraderapi.CThostFtdcQryBrokerTradingParamsField_CurrencyID_get, _thosttraderapi.CThostFtdcQryBrokerTradingParamsField_CurrencyID_set)
    AccountID = property(_thosttraderapi.CThostFtdcQryBrokerTradingParamsField_AccountID_get, _thosttraderapi.CThostFtdcQryBrokerTradingParamsField_AccountID_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcQryBrokerTradingParamsField_swiginit(self, _thosttraderapi.new_CThostFtdcQryBrokerTradingParamsField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcQryBrokerTradingParamsField

# Register CThostFtdcQryBrokerTradingParamsField in _thosttraderapi:
_thosttraderapi.CThostFtdcQryBrokerTradingParamsField_swigregister(CThostFtdcQryBrokerTradingParamsField)

class CThostFtdcBrokerTradingParamsField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thosttraderapi.CThostFtdcBrokerTradingParamsField_BrokerID_get, _thosttraderapi.CThostFtdcBrokerTradingParamsField_BrokerID_set)
    InvestorID = property(_thosttraderapi.CThostFtdcBrokerTradingParamsField_InvestorID_get, _thosttraderapi.CThostFtdcBrokerTradingParamsField_InvestorID_set)
    MarginPriceType = property(_thosttraderapi.CThostFtdcBrokerTradingParamsField_MarginPriceType_get, _thosttraderapi.CThostFtdcBrokerTradingParamsField_MarginPriceType_set)
    Algorithm = property(_thosttraderapi.CThostFtdcBrokerTradingParamsField_Algorithm_get, _thosttraderapi.CThostFtdcBrokerTradingParamsField_Algorithm_set)
    AvailIncludeCloseProfit = property(_thosttraderapi.CThostFtdcBrokerTradingParamsField_AvailIncludeCloseProfit_get, _thosttraderapi.CThostFtdcBrokerTradingParamsField_AvailIncludeCloseProfit_set)
    CurrencyID = property(_thosttraderapi.CThostFtdcBrokerTradingParamsField_CurrencyID_get, _thosttraderapi.CThostFtdcBrokerTradingParamsField_CurrencyID_set)
    OptionRoyaltyPriceType = property(_thosttraderapi.CThostFtdcBrokerTradingParamsField_OptionRoyaltyPriceType_get, _thosttraderapi.CThostFtdcBrokerTradingParamsField_OptionRoyaltyPriceType_set)
    AccountID = property(_thosttraderapi.CThostFtdcBrokerTradingParamsField_AccountID_get, _thosttraderapi.CThostFtdcBrokerTradingParamsField_AccountID_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcBrokerTradingParamsField_swiginit(self, _thosttraderapi.new_CThostFtdcBrokerTradingParamsField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcBrokerTradingParamsField

# Register CThostFtdcBrokerTradingParamsField in _thosttraderapi:
_thosttraderapi.CThostFtdcBrokerTradingParamsField_swigregister(CThostFtdcBrokerTradingParamsField)

class CThostFtdcQryBrokerTradingAlgosField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thosttraderapi.CThostFtdcQryBrokerTradingAlgosField_BrokerID_get, _thosttraderapi.CThostFtdcQryBrokerTradingAlgosField_BrokerID_set)
    ExchangeID = property(_thosttraderapi.CThostFtdcQryBrokerTradingAlgosField_ExchangeID_get, _thosttraderapi.CThostFtdcQryBrokerTradingAlgosField_ExchangeID_set)
    InstrumentID = property(_thosttraderapi.CThostFtdcQryBrokerTradingAlgosField_InstrumentID_get, _thosttraderapi.CThostFtdcQryBrokerTradingAlgosField_InstrumentID_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcQryBrokerTradingAlgosField_swiginit(self, _thosttraderapi.new_CThostFtdcQryBrokerTradingAlgosField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcQryBrokerTradingAlgosField

# Register CThostFtdcQryBrokerTradingAlgosField in _thosttraderapi:
_thosttraderapi.CThostFtdcQryBrokerTradingAlgosField_swigregister(CThostFtdcQryBrokerTradingAlgosField)

class CThostFtdcBrokerTradingAlgosField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thosttraderapi.CThostFtdcBrokerTradingAlgosField_BrokerID_get, _thosttraderapi.CThostFtdcBrokerTradingAlgosField_BrokerID_set)
    ExchangeID = property(_thosttraderapi.CThostFtdcBrokerTradingAlgosField_ExchangeID_get, _thosttraderapi.CThostFtdcBrokerTradingAlgosField_ExchangeID_set)
    InstrumentID = property(_thosttraderapi.CThostFtdcBrokerTradingAlgosField_InstrumentID_get, _thosttraderapi.CThostFtdcBrokerTradingAlgosField_InstrumentID_set)
    HandlePositionAlgoID = property(_thosttraderapi.CThostFtdcBrokerTradingAlgosField_HandlePositionAlgoID_get, _thosttraderapi.CThostFtdcBrokerTradingAlgosField_HandlePositionAlgoID_set)
    FindMarginRateAlgoID = property(_thosttraderapi.CThostFtdcBrokerTradingAlgosField_FindMarginRateAlgoID_get, _thosttraderapi.CThostFtdcBrokerTradingAlgosField_FindMarginRateAlgoID_set)
    HandleTradingAccountAlgoID = property(_thosttraderapi.CThostFtdcBrokerTradingAlgosField_HandleTradingAccountAlgoID_get, _thosttraderapi.CThostFtdcBrokerTradingAlgosField_HandleTradingAccountAlgoID_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcBrokerTradingAlgosField_swiginit(self, _thosttraderapi.new_CThostFtdcBrokerTradingAlgosField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcBrokerTradingAlgosField

# Register CThostFtdcBrokerTradingAlgosField in _thosttraderapi:
_thosttraderapi.CThostFtdcBrokerTradingAlgosField_swigregister(CThostFtdcBrokerTradingAlgosField)

class CThostFtdcQueryBrokerDepositField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thosttraderapi.CThostFtdcQueryBrokerDepositField_BrokerID_get, _thosttraderapi.CThostFtdcQueryBrokerDepositField_BrokerID_set)
    ExchangeID = property(_thosttraderapi.CThostFtdcQueryBrokerDepositField_ExchangeID_get, _thosttraderapi.CThostFtdcQueryBrokerDepositField_ExchangeID_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcQueryBrokerDepositField_swiginit(self, _thosttraderapi.new_CThostFtdcQueryBrokerDepositField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcQueryBrokerDepositField

# Register CThostFtdcQueryBrokerDepositField in _thosttraderapi:
_thosttraderapi.CThostFtdcQueryBrokerDepositField_swigregister(CThostFtdcQueryBrokerDepositField)

class CThostFtdcBrokerDepositField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    TradingDay = property(_thosttraderapi.CThostFtdcBrokerDepositField_TradingDay_get, _thosttraderapi.CThostFtdcBrokerDepositField_TradingDay_set)
    BrokerID = property(_thosttraderapi.CThostFtdcBrokerDepositField_BrokerID_get, _thosttraderapi.CThostFtdcBrokerDepositField_BrokerID_set)
    ParticipantID = property(_thosttraderapi.CThostFtdcBrokerDepositField_ParticipantID_get, _thosttraderapi.CThostFtdcBrokerDepositField_ParticipantID_set)
    ExchangeID = property(_thosttraderapi.CThostFtdcBrokerDepositField_ExchangeID_get, _thosttraderapi.CThostFtdcBrokerDepositField_ExchangeID_set)
    PreBalance = property(_thosttraderapi.CThostFtdcBrokerDepositField_PreBalance_get, _thosttraderapi.CThostFtdcBrokerDepositField_PreBalance_set)
    CurrMargin = property(_thosttraderapi.CThostFtdcBrokerDepositField_CurrMargin_get, _thosttraderapi.CThostFtdcBrokerDepositField_CurrMargin_set)
    CloseProfit = property(_thosttraderapi.CThostFtdcBrokerDepositField_CloseProfit_get, _thosttraderapi.CThostFtdcBrokerDepositField_CloseProfit_set)
    Balance = property(_thosttraderapi.CThostFtdcBrokerDepositField_Balance_get, _thosttraderapi.CThostFtdcBrokerDepositField_Balance_set)
    Deposit = property(_thosttraderapi.CThostFtdcBrokerDepositField_Deposit_get, _thosttraderapi.CThostFtdcBrokerDepositField_Deposit_set)
    Withdraw = property(_thosttraderapi.CThostFtdcBrokerDepositField_Withdraw_get, _thosttraderapi.CThostFtdcBrokerDepositField_Withdraw_set)
    Available = property(_thosttraderapi.CThostFtdcBrokerDepositField_Available_get, _thosttraderapi.CThostFtdcBrokerDepositField_Available_set)
    Reserve = property(_thosttraderapi.CThostFtdcBrokerDepositField_Reserve_get, _thosttraderapi.CThostFtdcBrokerDepositField_Reserve_set)
    FrozenMargin = property(_thosttraderapi.CThostFtdcBrokerDepositField_FrozenMargin_get, _thosttraderapi.CThostFtdcBrokerDepositField_FrozenMargin_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcBrokerDepositField_swiginit(self, _thosttraderapi.new_CThostFtdcBrokerDepositField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcBrokerDepositField

# Register CThostFtdcBrokerDepositField in _thosttraderapi:
_thosttraderapi.CThostFtdcBrokerDepositField_swigregister(CThostFtdcBrokerDepositField)

class CThostFtdcQryCFMMCBrokerKeyField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thosttraderapi.CThostFtdcQryCFMMCBrokerKeyField_BrokerID_get, _thosttraderapi.CThostFtdcQryCFMMCBrokerKeyField_BrokerID_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcQryCFMMCBrokerKeyField_swiginit(self, _thosttraderapi.new_CThostFtdcQryCFMMCBrokerKeyField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcQryCFMMCBrokerKeyField

# Register CThostFtdcQryCFMMCBrokerKeyField in _thosttraderapi:
_thosttraderapi.CThostFtdcQryCFMMCBrokerKeyField_swigregister(CThostFtdcQryCFMMCBrokerKeyField)

class CThostFtdcCFMMCBrokerKeyField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thosttraderapi.CThostFtdcCFMMCBrokerKeyField_BrokerID_get, _thosttraderapi.CThostFtdcCFMMCBrokerKeyField_BrokerID_set)
    ParticipantID = property(_thosttraderapi.CThostFtdcCFMMCBrokerKeyField_ParticipantID_get, _thosttraderapi.CThostFtdcCFMMCBrokerKeyField_ParticipantID_set)
    CreateDate = property(_thosttraderapi.CThostFtdcCFMMCBrokerKeyField_CreateDate_get, _thosttraderapi.CThostFtdcCFMMCBrokerKeyField_CreateDate_set)
    CreateTime = property(_thosttraderapi.CThostFtdcCFMMCBrokerKeyField_CreateTime_get, _thosttraderapi.CThostFtdcCFMMCBrokerKeyField_CreateTime_set)
    KeyID = property(_thosttraderapi.CThostFtdcCFMMCBrokerKeyField_KeyID_get, _thosttraderapi.CThostFtdcCFMMCBrokerKeyField_KeyID_set)
    CurrentKey = property(_thosttraderapi.CThostFtdcCFMMCBrokerKeyField_CurrentKey_get, _thosttraderapi.CThostFtdcCFMMCBrokerKeyField_CurrentKey_set)
    KeyKind = property(_thosttraderapi.CThostFtdcCFMMCBrokerKeyField_KeyKind_get, _thosttraderapi.CThostFtdcCFMMCBrokerKeyField_KeyKind_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcCFMMCBrokerKeyField_swiginit(self, _thosttraderapi.new_CThostFtdcCFMMCBrokerKeyField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcCFMMCBrokerKeyField

# Register CThostFtdcCFMMCBrokerKeyField in _thosttraderapi:
_thosttraderapi.CThostFtdcCFMMCBrokerKeyField_swigregister(CThostFtdcCFMMCBrokerKeyField)

class CThostFtdcCFMMCTradingAccountKeyField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thosttraderapi.CThostFtdcCFMMCTradingAccountKeyField_BrokerID_get, _thosttraderapi.CThostFtdcCFMMCTradingAccountKeyField_BrokerID_set)
    ParticipantID = property(_thosttraderapi.CThostFtdcCFMMCTradingAccountKeyField_ParticipantID_get, _thosttraderapi.CThostFtdcCFMMCTradingAccountKeyField_ParticipantID_set)
    AccountID = property(_thosttraderapi.CThostFtdcCFMMCTradingAccountKeyField_AccountID_get, _thosttraderapi.CThostFtdcCFMMCTradingAccountKeyField_AccountID_set)
    KeyID = property(_thosttraderapi.CThostFtdcCFMMCTradingAccountKeyField_KeyID_get, _thosttraderapi.CThostFtdcCFMMCTradingAccountKeyField_KeyID_set)
    CurrentKey = property(_thosttraderapi.CThostFtdcCFMMCTradingAccountKeyField_CurrentKey_get, _thosttraderapi.CThostFtdcCFMMCTradingAccountKeyField_CurrentKey_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcCFMMCTradingAccountKeyField_swiginit(self, _thosttraderapi.new_CThostFtdcCFMMCTradingAccountKeyField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcCFMMCTradingAccountKeyField

# Register CThostFtdcCFMMCTradingAccountKeyField in _thosttraderapi:
_thosttraderapi.CThostFtdcCFMMCTradingAccountKeyField_swigregister(CThostFtdcCFMMCTradingAccountKeyField)

class CThostFtdcQryCFMMCTradingAccountKeyField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thosttraderapi.CThostFtdcQryCFMMCTradingAccountKeyField_BrokerID_get, _thosttraderapi.CThostFtdcQryCFMMCTradingAccountKeyField_BrokerID_set)
    InvestorID = property(_thosttraderapi.CThostFtdcQryCFMMCTradingAccountKeyField_InvestorID_get, _thosttraderapi.CThostFtdcQryCFMMCTradingAccountKeyField_InvestorID_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcQryCFMMCTradingAccountKeyField_swiginit(self, _thosttraderapi.new_CThostFtdcQryCFMMCTradingAccountKeyField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcQryCFMMCTradingAccountKeyField

# Register CThostFtdcQryCFMMCTradingAccountKeyField in _thosttraderapi:
_thosttraderapi.CThostFtdcQryCFMMCTradingAccountKeyField_swigregister(CThostFtdcQryCFMMCTradingAccountKeyField)

class CThostFtdcBrokerUserOTPParamField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thosttraderapi.CThostFtdcBrokerUserOTPParamField_BrokerID_get, _thosttraderapi.CThostFtdcBrokerUserOTPParamField_BrokerID_set)
    UserID = property(_thosttraderapi.CThostFtdcBrokerUserOTPParamField_UserID_get, _thosttraderapi.CThostFtdcBrokerUserOTPParamField_UserID_set)
    OTPVendorsID = property(_thosttraderapi.CThostFtdcBrokerUserOTPParamField_OTPVendorsID_get, _thosttraderapi.CThostFtdcBrokerUserOTPParamField_OTPVendorsID_set)
    SerialNumber = property(_thosttraderapi.CThostFtdcBrokerUserOTPParamField_SerialNumber_get, _thosttraderapi.CThostFtdcBrokerUserOTPParamField_SerialNumber_set)
    AuthKey = property(_thosttraderapi.CThostFtdcBrokerUserOTPParamField_AuthKey_get, _thosttraderapi.CThostFtdcBrokerUserOTPParamField_AuthKey_set)
    LastDrift = property(_thosttraderapi.CThostFtdcBrokerUserOTPParamField_LastDrift_get, _thosttraderapi.CThostFtdcBrokerUserOTPParamField_LastDrift_set)
    LastSuccess = property(_thosttraderapi.CThostFtdcBrokerUserOTPParamField_LastSuccess_get, _thosttraderapi.CThostFtdcBrokerUserOTPParamField_LastSuccess_set)
    OTPType = property(_thosttraderapi.CThostFtdcBrokerUserOTPParamField_OTPType_get, _thosttraderapi.CThostFtdcBrokerUserOTPParamField_OTPType_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcBrokerUserOTPParamField_swiginit(self, _thosttraderapi.new_CThostFtdcBrokerUserOTPParamField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcBrokerUserOTPParamField

# Register CThostFtdcBrokerUserOTPParamField in _thosttraderapi:
_thosttraderapi.CThostFtdcBrokerUserOTPParamField_swigregister(CThostFtdcBrokerUserOTPParamField)

class CThostFtdcManualSyncBrokerUserOTPField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thosttraderapi.CThostFtdcManualSyncBrokerUserOTPField_BrokerID_get, _thosttraderapi.CThostFtdcManualSyncBrokerUserOTPField_BrokerID_set)
    UserID = property(_thosttraderapi.CThostFtdcManualSyncBrokerUserOTPField_UserID_get, _thosttraderapi.CThostFtdcManualSyncBrokerUserOTPField_UserID_set)
    OTPType = property(_thosttraderapi.CThostFtdcManualSyncBrokerUserOTPField_OTPType_get, _thosttraderapi.CThostFtdcManualSyncBrokerUserOTPField_OTPType_set)
    FirstOTP = property(_thosttraderapi.CThostFtdcManualSyncBrokerUserOTPField_FirstOTP_get, _thosttraderapi.CThostFtdcManualSyncBrokerUserOTPField_FirstOTP_set)
    SecondOTP = property(_thosttraderapi.CThostFtdcManualSyncBrokerUserOTPField_SecondOTP_get, _thosttraderapi.CThostFtdcManualSyncBrokerUserOTPField_SecondOTP_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcManualSyncBrokerUserOTPField_swiginit(self, _thosttraderapi.new_CThostFtdcManualSyncBrokerUserOTPField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcManualSyncBrokerUserOTPField

# Register CThostFtdcManualSyncBrokerUserOTPField in _thosttraderapi:
_thosttraderapi.CThostFtdcManualSyncBrokerUserOTPField_swigregister(CThostFtdcManualSyncBrokerUserOTPField)

class CThostFtdcCommRateModelField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thosttraderapi.CThostFtdcCommRateModelField_BrokerID_get, _thosttraderapi.CThostFtdcCommRateModelField_BrokerID_set)
    CommModelID = property(_thosttraderapi.CThostFtdcCommRateModelField_CommModelID_get, _thosttraderapi.CThostFtdcCommRateModelField_CommModelID_set)
    CommModelName = property(_thosttraderapi.CThostFtdcCommRateModelField_CommModelName_get, _thosttraderapi.CThostFtdcCommRateModelField_CommModelName_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcCommRateModelField_swiginit(self, _thosttraderapi.new_CThostFtdcCommRateModelField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcCommRateModelField

# Register CThostFtdcCommRateModelField in _thosttraderapi:
_thosttraderapi.CThostFtdcCommRateModelField_swigregister(CThostFtdcCommRateModelField)

class CThostFtdcQryCommRateModelField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thosttraderapi.CThostFtdcQryCommRateModelField_BrokerID_get, _thosttraderapi.CThostFtdcQryCommRateModelField_BrokerID_set)
    CommModelID = property(_thosttraderapi.CThostFtdcQryCommRateModelField_CommModelID_get, _thosttraderapi.CThostFtdcQryCommRateModelField_CommModelID_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcQryCommRateModelField_swiginit(self, _thosttraderapi.new_CThostFtdcQryCommRateModelField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcQryCommRateModelField

# Register CThostFtdcQryCommRateModelField in _thosttraderapi:
_thosttraderapi.CThostFtdcQryCommRateModelField_swigregister(CThostFtdcQryCommRateModelField)

class CThostFtdcMarginModelField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thosttraderapi.CThostFtdcMarginModelField_BrokerID_get, _thosttraderapi.CThostFtdcMarginModelField_BrokerID_set)
    MarginModelID = property(_thosttraderapi.CThostFtdcMarginModelField_MarginModelID_get, _thosttraderapi.CThostFtdcMarginModelField_MarginModelID_set)
    MarginModelName = property(_thosttraderapi.CThostFtdcMarginModelField_MarginModelName_get, _thosttraderapi.CThostFtdcMarginModelField_MarginModelName_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcMarginModelField_swiginit(self, _thosttraderapi.new_CThostFtdcMarginModelField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcMarginModelField

# Register CThostFtdcMarginModelField in _thosttraderapi:
_thosttraderapi.CThostFtdcMarginModelField_swigregister(CThostFtdcMarginModelField)

class CThostFtdcQryMarginModelField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thosttraderapi.CThostFtdcQryMarginModelField_BrokerID_get, _thosttraderapi.CThostFtdcQryMarginModelField_BrokerID_set)
    MarginModelID = property(_thosttraderapi.CThostFtdcQryMarginModelField_MarginModelID_get, _thosttraderapi.CThostFtdcQryMarginModelField_MarginModelID_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcQryMarginModelField_swiginit(self, _thosttraderapi.new_CThostFtdcQryMarginModelField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcQryMarginModelField

# Register CThostFtdcQryMarginModelField in _thosttraderapi:
_thosttraderapi.CThostFtdcQryMarginModelField_swigregister(CThostFtdcQryMarginModelField)

class CThostFtdcEWarrantOffsetField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    TradingDay = property(_thosttraderapi.CThostFtdcEWarrantOffsetField_TradingDay_get, _thosttraderapi.CThostFtdcEWarrantOffsetField_TradingDay_set)
    BrokerID = property(_thosttraderapi.CThostFtdcEWarrantOffsetField_BrokerID_get, _thosttraderapi.CThostFtdcEWarrantOffsetField_BrokerID_set)
    InvestorID = property(_thosttraderapi.CThostFtdcEWarrantOffsetField_InvestorID_get, _thosttraderapi.CThostFtdcEWarrantOffsetField_InvestorID_set)
    ExchangeID = property(_thosttraderapi.CThostFtdcEWarrantOffsetField_ExchangeID_get, _thosttraderapi.CThostFtdcEWarrantOffsetField_ExchangeID_set)
    InstrumentID = property(_thosttraderapi.CThostFtdcEWarrantOffsetField_InstrumentID_get, _thosttraderapi.CThostFtdcEWarrantOffsetField_InstrumentID_set)
    Direction = property(_thosttraderapi.CThostFtdcEWarrantOffsetField_Direction_get, _thosttraderapi.CThostFtdcEWarrantOffsetField_Direction_set)
    HedgeFlag = property(_thosttraderapi.CThostFtdcEWarrantOffsetField_HedgeFlag_get, _thosttraderapi.CThostFtdcEWarrantOffsetField_HedgeFlag_set)
    Volume = property(_thosttraderapi.CThostFtdcEWarrantOffsetField_Volume_get, _thosttraderapi.CThostFtdcEWarrantOffsetField_Volume_set)
    InvestUnitID = property(_thosttraderapi.CThostFtdcEWarrantOffsetField_InvestUnitID_get, _thosttraderapi.CThostFtdcEWarrantOffsetField_InvestUnitID_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcEWarrantOffsetField_swiginit(self, _thosttraderapi.new_CThostFtdcEWarrantOffsetField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcEWarrantOffsetField

# Register CThostFtdcEWarrantOffsetField in _thosttraderapi:
_thosttraderapi.CThostFtdcEWarrantOffsetField_swigregister(CThostFtdcEWarrantOffsetField)

class CThostFtdcQryEWarrantOffsetField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thosttraderapi.CThostFtdcQryEWarrantOffsetField_BrokerID_get, _thosttraderapi.CThostFtdcQryEWarrantOffsetField_BrokerID_set)
    InvestorID = property(_thosttraderapi.CThostFtdcQryEWarrantOffsetField_InvestorID_get, _thosttraderapi.CThostFtdcQryEWarrantOffsetField_InvestorID_set)
    ExchangeID = property(_thosttraderapi.CThostFtdcQryEWarrantOffsetField_ExchangeID_get, _thosttraderapi.CThostFtdcQryEWarrantOffsetField_ExchangeID_set)
    InstrumentID = property(_thosttraderapi.CThostFtdcQryEWarrantOffsetField_InstrumentID_get, _thosttraderapi.CThostFtdcQryEWarrantOffsetField_InstrumentID_set)
    InvestUnitID = property(_thosttraderapi.CThostFtdcQryEWarrantOffsetField_InvestUnitID_get, _thosttraderapi.CThostFtdcQryEWarrantOffsetField_InvestUnitID_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcQryEWarrantOffsetField_swiginit(self, _thosttraderapi.new_CThostFtdcQryEWarrantOffsetField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcQryEWarrantOffsetField

# Register CThostFtdcQryEWarrantOffsetField in _thosttraderapi:
_thosttraderapi.CThostFtdcQryEWarrantOffsetField_swigregister(CThostFtdcQryEWarrantOffsetField)

class CThostFtdcQryInvestorProductGroupMarginField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thosttraderapi.CThostFtdcQryInvestorProductGroupMarginField_BrokerID_get, _thosttraderapi.CThostFtdcQryInvestorProductGroupMarginField_BrokerID_set)
    InvestorID = property(_thosttraderapi.CThostFtdcQryInvestorProductGroupMarginField_InvestorID_get, _thosttraderapi.CThostFtdcQryInvestorProductGroupMarginField_InvestorID_set)
    ProductGroupID = property(_thosttraderapi.CThostFtdcQryInvestorProductGroupMarginField_ProductGroupID_get, _thosttraderapi.CThostFtdcQryInvestorProductGroupMarginField_ProductGroupID_set)
    HedgeFlag = property(_thosttraderapi.CThostFtdcQryInvestorProductGroupMarginField_HedgeFlag_get, _thosttraderapi.CThostFtdcQryInvestorProductGroupMarginField_HedgeFlag_set)
    ExchangeID = property(_thosttraderapi.CThostFtdcQryInvestorProductGroupMarginField_ExchangeID_get, _thosttraderapi.CThostFtdcQryInvestorProductGroupMarginField_ExchangeID_set)
    InvestUnitID = property(_thosttraderapi.CThostFtdcQryInvestorProductGroupMarginField_InvestUnitID_get, _thosttraderapi.CThostFtdcQryInvestorProductGroupMarginField_InvestUnitID_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcQryInvestorProductGroupMarginField_swiginit(self, _thosttraderapi.new_CThostFtdcQryInvestorProductGroupMarginField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcQryInvestorProductGroupMarginField

# Register CThostFtdcQryInvestorProductGroupMarginField in _thosttraderapi:
_thosttraderapi.CThostFtdcQryInvestorProductGroupMarginField_swigregister(CThostFtdcQryInvestorProductGroupMarginField)

class CThostFtdcInvestorProductGroupMarginField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    ProductGroupID = property(_thosttraderapi.CThostFtdcInvestorProductGroupMarginField_ProductGroupID_get, _thosttraderapi.CThostFtdcInvestorProductGroupMarginField_ProductGroupID_set)
    BrokerID = property(_thosttraderapi.CThostFtdcInvestorProductGroupMarginField_BrokerID_get, _thosttraderapi.CThostFtdcInvestorProductGroupMarginField_BrokerID_set)
    InvestorID = property(_thosttraderapi.CThostFtdcInvestorProductGroupMarginField_InvestorID_get, _thosttraderapi.CThostFtdcInvestorProductGroupMarginField_InvestorID_set)
    TradingDay = property(_thosttraderapi.CThostFtdcInvestorProductGroupMarginField_TradingDay_get, _thosttraderapi.CThostFtdcInvestorProductGroupMarginField_TradingDay_set)
    SettlementID = property(_thosttraderapi.CThostFtdcInvestorProductGroupMarginField_SettlementID_get, _thosttraderapi.CThostFtdcInvestorProductGroupMarginField_SettlementID_set)
    FrozenMargin = property(_thosttraderapi.CThostFtdcInvestorProductGroupMarginField_FrozenMargin_get, _thosttraderapi.CThostFtdcInvestorProductGroupMarginField_FrozenMargin_set)
    LongFrozenMargin = property(_thosttraderapi.CThostFtdcInvestorProductGroupMarginField_LongFrozenMargin_get, _thosttraderapi.CThostFtdcInvestorProductGroupMarginField_LongFrozenMargin_set)
    ShortFrozenMargin = property(_thosttraderapi.CThostFtdcInvestorProductGroupMarginField_ShortFrozenMargin_get, _thosttraderapi.CThostFtdcInvestorProductGroupMarginField_ShortFrozenMargin_set)
    UseMargin = property(_thosttraderapi.CThostFtdcInvestorProductGroupMarginField_UseMargin_get, _thosttraderapi.CThostFtdcInvestorProductGroupMarginField_UseMargin_set)
    LongUseMargin = property(_thosttraderapi.CThostFtdcInvestorProductGroupMarginField_LongUseMargin_get, _thosttraderapi.CThostFtdcInvestorProductGroupMarginField_LongUseMargin_set)
    ShortUseMargin = property(_thosttraderapi.CThostFtdcInvestorProductGroupMarginField_ShortUseMargin_get, _thosttraderapi.CThostFtdcInvestorProductGroupMarginField_ShortUseMargin_set)
    ExchMargin = property(_thosttraderapi.CThostFtdcInvestorProductGroupMarginField_ExchMargin_get, _thosttraderapi.CThostFtdcInvestorProductGroupMarginField_ExchMargin_set)
    LongExchMargin = property(_thosttraderapi.CThostFtdcInvestorProductGroupMarginField_LongExchMargin_get, _thosttraderapi.CThostFtdcInvestorProductGroupMarginField_LongExchMargin_set)
    ShortExchMargin = property(_thosttraderapi.CThostFtdcInvestorProductGroupMarginField_ShortExchMargin_get, _thosttraderapi.CThostFtdcInvestorProductGroupMarginField_ShortExchMargin_set)
    CloseProfit = property(_thosttraderapi.CThostFtdcInvestorProductGroupMarginField_CloseProfit_get, _thosttraderapi.CThostFtdcInvestorProductGroupMarginField_CloseProfit_set)
    FrozenCommission = property(_thosttraderapi.CThostFtdcInvestorProductGroupMarginField_FrozenCommission_get, _thosttraderapi.CThostFtdcInvestorProductGroupMarginField_FrozenCommission_set)
    Commission = property(_thosttraderapi.CThostFtdcInvestorProductGroupMarginField_Commission_get, _thosttraderapi.CThostFtdcInvestorProductGroupMarginField_Commission_set)
    FrozenCash = property(_thosttraderapi.CThostFtdcInvestorProductGroupMarginField_FrozenCash_get, _thosttraderapi.CThostFtdcInvestorProductGroupMarginField_FrozenCash_set)
    CashIn = property(_thosttraderapi.CThostFtdcInvestorProductGroupMarginField_CashIn_get, _thosttraderapi.CThostFtdcInvestorProductGroupMarginField_CashIn_set)
    PositionProfit = property(_thosttraderapi.CThostFtdcInvestorProductGroupMarginField_PositionProfit_get, _thosttraderapi.CThostFtdcInvestorProductGroupMarginField_PositionProfit_set)
    OffsetAmount = property(_thosttraderapi.CThostFtdcInvestorProductGroupMarginField_OffsetAmount_get, _thosttraderapi.CThostFtdcInvestorProductGroupMarginField_OffsetAmount_set)
    LongOffsetAmount = property(_thosttraderapi.CThostFtdcInvestorProductGroupMarginField_LongOffsetAmount_get, _thosttraderapi.CThostFtdcInvestorProductGroupMarginField_LongOffsetAmount_set)
    ShortOffsetAmount = property(_thosttraderapi.CThostFtdcInvestorProductGroupMarginField_ShortOffsetAmount_get, _thosttraderapi.CThostFtdcInvestorProductGroupMarginField_ShortOffsetAmount_set)
    ExchOffsetAmount = property(_thosttraderapi.CThostFtdcInvestorProductGroupMarginField_ExchOffsetAmount_get, _thosttraderapi.CThostFtdcInvestorProductGroupMarginField_ExchOffsetAmount_set)
    LongExchOffsetAmount = property(_thosttraderapi.CThostFtdcInvestorProductGroupMarginField_LongExchOffsetAmount_get, _thosttraderapi.CThostFtdcInvestorProductGroupMarginField_LongExchOffsetAmount_set)
    ShortExchOffsetAmount = property(_thosttraderapi.CThostFtdcInvestorProductGroupMarginField_ShortExchOffsetAmount_get, _thosttraderapi.CThostFtdcInvestorProductGroupMarginField_ShortExchOffsetAmount_set)
    HedgeFlag = property(_thosttraderapi.CThostFtdcInvestorProductGroupMarginField_HedgeFlag_get, _thosttraderapi.CThostFtdcInvestorProductGroupMarginField_HedgeFlag_set)
    ExchangeID = property(_thosttraderapi.CThostFtdcInvestorProductGroupMarginField_ExchangeID_get, _thosttraderapi.CThostFtdcInvestorProductGroupMarginField_ExchangeID_set)
    InvestUnitID = property(_thosttraderapi.CThostFtdcInvestorProductGroupMarginField_InvestUnitID_get, _thosttraderapi.CThostFtdcInvestorProductGroupMarginField_InvestUnitID_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcInvestorProductGroupMarginField_swiginit(self, _thosttraderapi.new_CThostFtdcInvestorProductGroupMarginField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcInvestorProductGroupMarginField

# Register CThostFtdcInvestorProductGroupMarginField in _thosttraderapi:
_thosttraderapi.CThostFtdcInvestorProductGroupMarginField_swigregister(CThostFtdcInvestorProductGroupMarginField)

class CThostFtdcQueryCFMMCTradingAccountTokenField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thosttraderapi.CThostFtdcQueryCFMMCTradingAccountTokenField_BrokerID_get, _thosttraderapi.CThostFtdcQueryCFMMCTradingAccountTokenField_BrokerID_set)
    InvestorID = property(_thosttraderapi.CThostFtdcQueryCFMMCTradingAccountTokenField_InvestorID_get, _thosttraderapi.CThostFtdcQueryCFMMCTradingAccountTokenField_InvestorID_set)
    InvestUnitID = property(_thosttraderapi.CThostFtdcQueryCFMMCTradingAccountTokenField_InvestUnitID_get, _thosttraderapi.CThostFtdcQueryCFMMCTradingAccountTokenField_InvestUnitID_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcQueryCFMMCTradingAccountTokenField_swiginit(self, _thosttraderapi.new_CThostFtdcQueryCFMMCTradingAccountTokenField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcQueryCFMMCTradingAccountTokenField

# Register CThostFtdcQueryCFMMCTradingAccountTokenField in _thosttraderapi:
_thosttraderapi.CThostFtdcQueryCFMMCTradingAccountTokenField_swigregister(CThostFtdcQueryCFMMCTradingAccountTokenField)

class CThostFtdcCFMMCTradingAccountTokenField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thosttraderapi.CThostFtdcCFMMCTradingAccountTokenField_BrokerID_get, _thosttraderapi.CThostFtdcCFMMCTradingAccountTokenField_BrokerID_set)
    ParticipantID = property(_thosttraderapi.CThostFtdcCFMMCTradingAccountTokenField_ParticipantID_get, _thosttraderapi.CThostFtdcCFMMCTradingAccountTokenField_ParticipantID_set)
    AccountID = property(_thosttraderapi.CThostFtdcCFMMCTradingAccountTokenField_AccountID_get, _thosttraderapi.CThostFtdcCFMMCTradingAccountTokenField_AccountID_set)
    KeyID = property(_thosttraderapi.CThostFtdcCFMMCTradingAccountTokenField_KeyID_get, _thosttraderapi.CThostFtdcCFMMCTradingAccountTokenField_KeyID_set)
    Token = property(_thosttraderapi.CThostFtdcCFMMCTradingAccountTokenField_Token_get, _thosttraderapi.CThostFtdcCFMMCTradingAccountTokenField_Token_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcCFMMCTradingAccountTokenField_swiginit(self, _thosttraderapi.new_CThostFtdcCFMMCTradingAccountTokenField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcCFMMCTradingAccountTokenField

# Register CThostFtdcCFMMCTradingAccountTokenField in _thosttraderapi:
_thosttraderapi.CThostFtdcCFMMCTradingAccountTokenField_swigregister(CThostFtdcCFMMCTradingAccountTokenField)

class CThostFtdcQryProductGroupField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    ProductID = property(_thosttraderapi.CThostFtdcQryProductGroupField_ProductID_get, _thosttraderapi.CThostFtdcQryProductGroupField_ProductID_set)
    ExchangeID = property(_thosttraderapi.CThostFtdcQryProductGroupField_ExchangeID_get, _thosttraderapi.CThostFtdcQryProductGroupField_ExchangeID_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcQryProductGroupField_swiginit(self, _thosttraderapi.new_CThostFtdcQryProductGroupField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcQryProductGroupField

# Register CThostFtdcQryProductGroupField in _thosttraderapi:
_thosttraderapi.CThostFtdcQryProductGroupField_swigregister(CThostFtdcQryProductGroupField)

class CThostFtdcProductGroupField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    ProductID = property(_thosttraderapi.CThostFtdcProductGroupField_ProductID_get, _thosttraderapi.CThostFtdcProductGroupField_ProductID_set)
    ExchangeID = property(_thosttraderapi.CThostFtdcProductGroupField_ExchangeID_get, _thosttraderapi.CThostFtdcProductGroupField_ExchangeID_set)
    ProductGroupID = property(_thosttraderapi.CThostFtdcProductGroupField_ProductGroupID_get, _thosttraderapi.CThostFtdcProductGroupField_ProductGroupID_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcProductGroupField_swiginit(self, _thosttraderapi.new_CThostFtdcProductGroupField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcProductGroupField

# Register CThostFtdcProductGroupField in _thosttraderapi:
_thosttraderapi.CThostFtdcProductGroupField_swigregister(CThostFtdcProductGroupField)

class CThostFtdcBulletinField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    ExchangeID = property(_thosttraderapi.CThostFtdcBulletinField_ExchangeID_get, _thosttraderapi.CThostFtdcBulletinField_ExchangeID_set)
    TradingDay = property(_thosttraderapi.CThostFtdcBulletinField_TradingDay_get, _thosttraderapi.CThostFtdcBulletinField_TradingDay_set)
    BulletinID = property(_thosttraderapi.CThostFtdcBulletinField_BulletinID_get, _thosttraderapi.CThostFtdcBulletinField_BulletinID_set)
    SequenceNo = property(_thosttraderapi.CThostFtdcBulletinField_SequenceNo_get, _thosttraderapi.CThostFtdcBulletinField_SequenceNo_set)
    NewsType = property(_thosttraderapi.CThostFtdcBulletinField_NewsType_get, _thosttraderapi.CThostFtdcBulletinField_NewsType_set)
    NewsUrgency = property(_thosttraderapi.CThostFtdcBulletinField_NewsUrgency_get, _thosttraderapi.CThostFtdcBulletinField_NewsUrgency_set)
    SendTime = property(_thosttraderapi.CThostFtdcBulletinField_SendTime_get, _thosttraderapi.CThostFtdcBulletinField_SendTime_set)
    Abstract = property(_thosttraderapi.CThostFtdcBulletinField_Abstract_get, _thosttraderapi.CThostFtdcBulletinField_Abstract_set)
    ComeFrom = property(_thosttraderapi.CThostFtdcBulletinField_ComeFrom_get, _thosttraderapi.CThostFtdcBulletinField_ComeFrom_set)
    Content = property(_thosttraderapi.CThostFtdcBulletinField_Content_get, _thosttraderapi.CThostFtdcBulletinField_Content_set)
    URLLink = property(_thosttraderapi.CThostFtdcBulletinField_URLLink_get, _thosttraderapi.CThostFtdcBulletinField_URLLink_set)
    MarketID = property(_thosttraderapi.CThostFtdcBulletinField_MarketID_get, _thosttraderapi.CThostFtdcBulletinField_MarketID_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcBulletinField_swiginit(self, _thosttraderapi.new_CThostFtdcBulletinField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcBulletinField

# Register CThostFtdcBulletinField in _thosttraderapi:
_thosttraderapi.CThostFtdcBulletinField_swigregister(CThostFtdcBulletinField)

class CThostFtdcQryBulletinField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    ExchangeID = property(_thosttraderapi.CThostFtdcQryBulletinField_ExchangeID_get, _thosttraderapi.CThostFtdcQryBulletinField_ExchangeID_set)
    BulletinID = property(_thosttraderapi.CThostFtdcQryBulletinField_BulletinID_get, _thosttraderapi.CThostFtdcQryBulletinField_BulletinID_set)
    SequenceNo = property(_thosttraderapi.CThostFtdcQryBulletinField_SequenceNo_get, _thosttraderapi.CThostFtdcQryBulletinField_SequenceNo_set)
    NewsType = property(_thosttraderapi.CThostFtdcQryBulletinField_NewsType_get, _thosttraderapi.CThostFtdcQryBulletinField_NewsType_set)
    NewsUrgency = property(_thosttraderapi.CThostFtdcQryBulletinField_NewsUrgency_get, _thosttraderapi.CThostFtdcQryBulletinField_NewsUrgency_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcQryBulletinField_swiginit(self, _thosttraderapi.new_CThostFtdcQryBulletinField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcQryBulletinField

# Register CThostFtdcQryBulletinField in _thosttraderapi:
_thosttraderapi.CThostFtdcQryBulletinField_swigregister(CThostFtdcQryBulletinField)

class CThostFtdcReqOpenAccountField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    TradeCode = property(_thosttraderapi.CThostFtdcReqOpenAccountField_TradeCode_get, _thosttraderapi.CThostFtdcReqOpenAccountField_TradeCode_set)
    BankID = property(_thosttraderapi.CThostFtdcReqOpenAccountField_BankID_get, _thosttraderapi.CThostFtdcReqOpenAccountField_BankID_set)
    BankBranchID = property(_thosttraderapi.CThostFtdcReqOpenAccountField_BankBranchID_get, _thosttraderapi.CThostFtdcReqOpenAccountField_BankBranchID_set)
    BrokerID = property(_thosttraderapi.CThostFtdcReqOpenAccountField_BrokerID_get, _thosttraderapi.CThostFtdcReqOpenAccountField_BrokerID_set)
    BrokerBranchID = property(_thosttraderapi.CThostFtdcReqOpenAccountField_BrokerBranchID_get, _thosttraderapi.CThostFtdcReqOpenAccountField_BrokerBranchID_set)
    TradeDate = property(_thosttraderapi.CThostFtdcReqOpenAccountField_TradeDate_get, _thosttraderapi.CThostFtdcReqOpenAccountField_TradeDate_set)
    TradeTime = property(_thosttraderapi.CThostFtdcReqOpenAccountField_TradeTime_get, _thosttraderapi.CThostFtdcReqOpenAccountField_TradeTime_set)
    BankSerial = property(_thosttraderapi.CThostFtdcReqOpenAccountField_BankSerial_get, _thosttraderapi.CThostFtdcReqOpenAccountField_BankSerial_set)
    TradingDay = property(_thosttraderapi.CThostFtdcReqOpenAccountField_TradingDay_get, _thosttraderapi.CThostFtdcReqOpenAccountField_TradingDay_set)
    PlateSerial = property(_thosttraderapi.CThostFtdcReqOpenAccountField_PlateSerial_get, _thosttraderapi.CThostFtdcReqOpenAccountField_PlateSerial_set)
    LastFragment = property(_thosttraderapi.CThostFtdcReqOpenAccountField_LastFragment_get, _thosttraderapi.CThostFtdcReqOpenAccountField_LastFragment_set)
    SessionID = property(_thosttraderapi.CThostFtdcReqOpenAccountField_SessionID_get, _thosttraderapi.CThostFtdcReqOpenAccountField_SessionID_set)
    CustomerName = property(_thosttraderapi.CThostFtdcReqOpenAccountField_CustomerName_get, _thosttraderapi.CThostFtdcReqOpenAccountField_CustomerName_set)
    IdCardType = property(_thosttraderapi.CThostFtdcReqOpenAccountField_IdCardType_get, _thosttraderapi.CThostFtdcReqOpenAccountField_IdCardType_set)
    IdentifiedCardNo = property(_thosttraderapi.CThostFtdcReqOpenAccountField_IdentifiedCardNo_get, _thosttraderapi.CThostFtdcReqOpenAccountField_IdentifiedCardNo_set)
    Gender = property(_thosttraderapi.CThostFtdcReqOpenAccountField_Gender_get, _thosttraderapi.CThostFtdcReqOpenAccountField_Gender_set)
    CountryCode = property(_thosttraderapi.CThostFtdcReqOpenAccountField_CountryCode_get, _thosttraderapi.CThostFtdcReqOpenAccountField_CountryCode_set)
    CustType = property(_thosttraderapi.CThostFtdcReqOpenAccountField_CustType_get, _thosttraderapi.CThostFtdcReqOpenAccountField_CustType_set)
    Address = property(_thosttraderapi.CThostFtdcReqOpenAccountField_Address_get, _thosttraderapi.CThostFtdcReqOpenAccountField_Address_set)
    ZipCode = property(_thosttraderapi.CThostFtdcReqOpenAccountField_ZipCode_get, _thosttraderapi.CThostFtdcReqOpenAccountField_ZipCode_set)
    Telephone = property(_thosttraderapi.CThostFtdcReqOpenAccountField_Telephone_get, _thosttraderapi.CThostFtdcReqOpenAccountField_Telephone_set)
    MobilePhone = property(_thosttraderapi.CThostFtdcReqOpenAccountField_MobilePhone_get, _thosttraderapi.CThostFtdcReqOpenAccountField_MobilePhone_set)
    Fax = property(_thosttraderapi.CThostFtdcReqOpenAccountField_Fax_get, _thosttraderapi.CThostFtdcReqOpenAccountField_Fax_set)
    EMail = property(_thosttraderapi.CThostFtdcReqOpenAccountField_EMail_get, _thosttraderapi.CThostFtdcReqOpenAccountField_EMail_set)
    MoneyAccountStatus = property(_thosttraderapi.CThostFtdcReqOpenAccountField_MoneyAccountStatus_get, _thosttraderapi.CThostFtdcReqOpenAccountField_MoneyAccountStatus_set)
    BankAccount = property(_thosttraderapi.CThostFtdcReqOpenAccountField_BankAccount_get, _thosttraderapi.CThostFtdcReqOpenAccountField_BankAccount_set)
    BankPassWord = property(_thosttraderapi.CThostFtdcReqOpenAccountField_BankPassWord_get, _thosttraderapi.CThostFtdcReqOpenAccountField_BankPassWord_set)
    AccountID = property(_thosttraderapi.CThostFtdcReqOpenAccountField_AccountID_get, _thosttraderapi.CThostFtdcReqOpenAccountField_AccountID_set)
    Password = property(_thosttraderapi.CThostFtdcReqOpenAccountField_Password_get, _thosttraderapi.CThostFtdcReqOpenAccountField_Password_set)
    InstallID = property(_thosttraderapi.CThostFtdcReqOpenAccountField_InstallID_get, _thosttraderapi.CThostFtdcReqOpenAccountField_InstallID_set)
    VerifyCertNoFlag = property(_thosttraderapi.CThostFtdcReqOpenAccountField_VerifyCertNoFlag_get, _thosttraderapi.CThostFtdcReqOpenAccountField_VerifyCertNoFlag_set)
    CurrencyID = property(_thosttraderapi.CThostFtdcReqOpenAccountField_CurrencyID_get, _thosttraderapi.CThostFtdcReqOpenAccountField_CurrencyID_set)
    CashExchangeCode = property(_thosttraderapi.CThostFtdcReqOpenAccountField_CashExchangeCode_get, _thosttraderapi.CThostFtdcReqOpenAccountField_CashExchangeCode_set)
    Digest = property(_thosttraderapi.CThostFtdcReqOpenAccountField_Digest_get, _thosttraderapi.CThostFtdcReqOpenAccountField_Digest_set)
    BankAccType = property(_thosttraderapi.CThostFtdcReqOpenAccountField_BankAccType_get, _thosttraderapi.CThostFtdcReqOpenAccountField_BankAccType_set)
    DeviceID = property(_thosttraderapi.CThostFtdcReqOpenAccountField_DeviceID_get, _thosttraderapi.CThostFtdcReqOpenAccountField_DeviceID_set)
    BankSecuAccType = property(_thosttraderapi.CThostFtdcReqOpenAccountField_BankSecuAccType_get, _thosttraderapi.CThostFtdcReqOpenAccountField_BankSecuAccType_set)
    BrokerIDByBank = property(_thosttraderapi.CThostFtdcReqOpenAccountField_BrokerIDByBank_get, _thosttraderapi.CThostFtdcReqOpenAccountField_BrokerIDByBank_set)
    BankSecuAcc = property(_thosttraderapi.CThostFtdcReqOpenAccountField_BankSecuAcc_get, _thosttraderapi.CThostFtdcReqOpenAccountField_BankSecuAcc_set)
    BankPwdFlag = property(_thosttraderapi.CThostFtdcReqOpenAccountField_BankPwdFlag_get, _thosttraderapi.CThostFtdcReqOpenAccountField_BankPwdFlag_set)
    SecuPwdFlag = property(_thosttraderapi.CThostFtdcReqOpenAccountField_SecuPwdFlag_get, _thosttraderapi.CThostFtdcReqOpenAccountField_SecuPwdFlag_set)
    OperNo = property(_thosttraderapi.CThostFtdcReqOpenAccountField_OperNo_get, _thosttraderapi.CThostFtdcReqOpenAccountField_OperNo_set)
    TID = property(_thosttraderapi.CThostFtdcReqOpenAccountField_TID_get, _thosttraderapi.CThostFtdcReqOpenAccountField_TID_set)
    UserID = property(_thosttraderapi.CThostFtdcReqOpenAccountField_UserID_get, _thosttraderapi.CThostFtdcReqOpenAccountField_UserID_set)
    LongCustomerName = property(_thosttraderapi.CThostFtdcReqOpenAccountField_LongCustomerName_get, _thosttraderapi.CThostFtdcReqOpenAccountField_LongCustomerName_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcReqOpenAccountField_swiginit(self, _thosttraderapi.new_CThostFtdcReqOpenAccountField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcReqOpenAccountField

# Register CThostFtdcReqOpenAccountField in _thosttraderapi:
_thosttraderapi.CThostFtdcReqOpenAccountField_swigregister(CThostFtdcReqOpenAccountField)

class CThostFtdcReqCancelAccountField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    TradeCode = property(_thosttraderapi.CThostFtdcReqCancelAccountField_TradeCode_get, _thosttraderapi.CThostFtdcReqCancelAccountField_TradeCode_set)
    BankID = property(_thosttraderapi.CThostFtdcReqCancelAccountField_BankID_get, _thosttraderapi.CThostFtdcReqCancelAccountField_BankID_set)
    BankBranchID = property(_thosttraderapi.CThostFtdcReqCancelAccountField_BankBranchID_get, _thosttraderapi.CThostFtdcReqCancelAccountField_BankBranchID_set)
    BrokerID = property(_thosttraderapi.CThostFtdcReqCancelAccountField_BrokerID_get, _thosttraderapi.CThostFtdcReqCancelAccountField_BrokerID_set)
    BrokerBranchID = property(_thosttraderapi.CThostFtdcReqCancelAccountField_BrokerBranchID_get, _thosttraderapi.CThostFtdcReqCancelAccountField_BrokerBranchID_set)
    TradeDate = property(_thosttraderapi.CThostFtdcReqCancelAccountField_TradeDate_get, _thosttraderapi.CThostFtdcReqCancelAccountField_TradeDate_set)
    TradeTime = property(_thosttraderapi.CThostFtdcReqCancelAccountField_TradeTime_get, _thosttraderapi.CThostFtdcReqCancelAccountField_TradeTime_set)
    BankSerial = property(_thosttraderapi.CThostFtdcReqCancelAccountField_BankSerial_get, _thosttraderapi.CThostFtdcReqCancelAccountField_BankSerial_set)
    TradingDay = property(_thosttraderapi.CThostFtdcReqCancelAccountField_TradingDay_get, _thosttraderapi.CThostFtdcReqCancelAccountField_TradingDay_set)
    PlateSerial = property(_thosttraderapi.CThostFtdcReqCancelAccountField_PlateSerial_get, _thosttraderapi.CThostFtdcReqCancelAccountField_PlateSerial_set)
    LastFragment = property(_thosttraderapi.CThostFtdcReqCancelAccountField_LastFragment_get, _thosttraderapi.CThostFtdcReqCancelAccountField_LastFragment_set)
    SessionID = property(_thosttraderapi.CThostFtdcReqCancelAccountField_SessionID_get, _thosttraderapi.CThostFtdcReqCancelAccountField_SessionID_set)
    CustomerName = property(_thosttraderapi.CThostFtdcReqCancelAccountField_CustomerName_get, _thosttraderapi.CThostFtdcReqCancelAccountField_CustomerName_set)
    IdCardType = property(_thosttraderapi.CThostFtdcReqCancelAccountField_IdCardType_get, _thosttraderapi.CThostFtdcReqCancelAccountField_IdCardType_set)
    IdentifiedCardNo = property(_thosttraderapi.CThostFtdcReqCancelAccountField_IdentifiedCardNo_get, _thosttraderapi.CThostFtdcReqCancelAccountField_IdentifiedCardNo_set)
    Gender = property(_thosttraderapi.CThostFtdcReqCancelAccountField_Gender_get, _thosttraderapi.CThostFtdcReqCancelAccountField_Gender_set)
    CountryCode = property(_thosttraderapi.CThostFtdcReqCancelAccountField_CountryCode_get, _thosttraderapi.CThostFtdcReqCancelAccountField_CountryCode_set)
    CustType = property(_thosttraderapi.CThostFtdcReqCancelAccountField_CustType_get, _thosttraderapi.CThostFtdcReqCancelAccountField_CustType_set)
    Address = property(_thosttraderapi.CThostFtdcReqCancelAccountField_Address_get, _thosttraderapi.CThostFtdcReqCancelAccountField_Address_set)
    ZipCode = property(_thosttraderapi.CThostFtdcReqCancelAccountField_ZipCode_get, _thosttraderapi.CThostFtdcReqCancelAccountField_ZipCode_set)
    Telephone = property(_thosttraderapi.CThostFtdcReqCancelAccountField_Telephone_get, _thosttraderapi.CThostFtdcReqCancelAccountField_Telephone_set)
    MobilePhone = property(_thosttraderapi.CThostFtdcReqCancelAccountField_MobilePhone_get, _thosttraderapi.CThostFtdcReqCancelAccountField_MobilePhone_set)
    Fax = property(_thosttraderapi.CThostFtdcReqCancelAccountField_Fax_get, _thosttraderapi.CThostFtdcReqCancelAccountField_Fax_set)
    EMail = property(_thosttraderapi.CThostFtdcReqCancelAccountField_EMail_get, _thosttraderapi.CThostFtdcReqCancelAccountField_EMail_set)
    MoneyAccountStatus = property(_thosttraderapi.CThostFtdcReqCancelAccountField_MoneyAccountStatus_get, _thosttraderapi.CThostFtdcReqCancelAccountField_MoneyAccountStatus_set)
    BankAccount = property(_thosttraderapi.CThostFtdcReqCancelAccountField_BankAccount_get, _thosttraderapi.CThostFtdcReqCancelAccountField_BankAccount_set)
    BankPassWord = property(_thosttraderapi.CThostFtdcReqCancelAccountField_BankPassWord_get, _thosttraderapi.CThostFtdcReqCancelAccountField_BankPassWord_set)
    AccountID = property(_thosttraderapi.CThostFtdcReqCancelAccountField_AccountID_get, _thosttraderapi.CThostFtdcReqCancelAccountField_AccountID_set)
    Password = property(_thosttraderapi.CThostFtdcReqCancelAccountField_Password_get, _thosttraderapi.CThostFtdcReqCancelAccountField_Password_set)
    InstallID = property(_thosttraderapi.CThostFtdcReqCancelAccountField_InstallID_get, _thosttraderapi.CThostFtdcReqCancelAccountField_InstallID_set)
    VerifyCertNoFlag = property(_thosttraderapi.CThostFtdcReqCancelAccountField_VerifyCertNoFlag_get, _thosttraderapi.CThostFtdcReqCancelAccountField_VerifyCertNoFlag_set)
    CurrencyID = property(_thosttraderapi.CThostFtdcReqCancelAccountField_CurrencyID_get, _thosttraderapi.CThostFtdcReqCancelAccountField_CurrencyID_set)
    CashExchangeCode = property(_thosttraderapi.CThostFtdcReqCancelAccountField_CashExchangeCode_get, _thosttraderapi.CThostFtdcReqCancelAccountField_CashExchangeCode_set)
    Digest = property(_thosttraderapi.CThostFtdcReqCancelAccountField_Digest_get, _thosttraderapi.CThostFtdcReqCancelAccountField_Digest_set)
    BankAccType = property(_thosttraderapi.CThostFtdcReqCancelAccountField_BankAccType_get, _thosttraderapi.CThostFtdcReqCancelAccountField_BankAccType_set)
    DeviceID = property(_thosttraderapi.CThostFtdcReqCancelAccountField_DeviceID_get, _thosttraderapi.CThostFtdcReqCancelAccountField_DeviceID_set)
    BankSecuAccType = property(_thosttraderapi.CThostFtdcReqCancelAccountField_BankSecuAccType_get, _thosttraderapi.CThostFtdcReqCancelAccountField_BankSecuAccType_set)
    BrokerIDByBank = property(_thosttraderapi.CThostFtdcReqCancelAccountField_BrokerIDByBank_get, _thosttraderapi.CThostFtdcReqCancelAccountField_BrokerIDByBank_set)
    BankSecuAcc = property(_thosttraderapi.CThostFtdcReqCancelAccountField_BankSecuAcc_get, _thosttraderapi.CThostFtdcReqCancelAccountField_BankSecuAcc_set)
    BankPwdFlag = property(_thosttraderapi.CThostFtdcReqCancelAccountField_BankPwdFlag_get, _thosttraderapi.CThostFtdcReqCancelAccountField_BankPwdFlag_set)
    SecuPwdFlag = property(_thosttraderapi.CThostFtdcReqCancelAccountField_SecuPwdFlag_get, _thosttraderapi.CThostFtdcReqCancelAccountField_SecuPwdFlag_set)
    OperNo = property(_thosttraderapi.CThostFtdcReqCancelAccountField_OperNo_get, _thosttraderapi.CThostFtdcReqCancelAccountField_OperNo_set)
    TID = property(_thosttraderapi.CThostFtdcReqCancelAccountField_TID_get, _thosttraderapi.CThostFtdcReqCancelAccountField_TID_set)
    UserID = property(_thosttraderapi.CThostFtdcReqCancelAccountField_UserID_get, _thosttraderapi.CThostFtdcReqCancelAccountField_UserID_set)
    LongCustomerName = property(_thosttraderapi.CThostFtdcReqCancelAccountField_LongCustomerName_get, _thosttraderapi.CThostFtdcReqCancelAccountField_LongCustomerName_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcReqCancelAccountField_swiginit(self, _thosttraderapi.new_CThostFtdcReqCancelAccountField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcReqCancelAccountField

# Register CThostFtdcReqCancelAccountField in _thosttraderapi:
_thosttraderapi.CThostFtdcReqCancelAccountField_swigregister(CThostFtdcReqCancelAccountField)

class CThostFtdcReqChangeAccountField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    TradeCode = property(_thosttraderapi.CThostFtdcReqChangeAccountField_TradeCode_get, _thosttraderapi.CThostFtdcReqChangeAccountField_TradeCode_set)
    BankID = property(_thosttraderapi.CThostFtdcReqChangeAccountField_BankID_get, _thosttraderapi.CThostFtdcReqChangeAccountField_BankID_set)
    BankBranchID = property(_thosttraderapi.CThostFtdcReqChangeAccountField_BankBranchID_get, _thosttraderapi.CThostFtdcReqChangeAccountField_BankBranchID_set)
    BrokerID = property(_thosttraderapi.CThostFtdcReqChangeAccountField_BrokerID_get, _thosttraderapi.CThostFtdcReqChangeAccountField_BrokerID_set)
    BrokerBranchID = property(_thosttraderapi.CThostFtdcReqChangeAccountField_BrokerBranchID_get, _thosttraderapi.CThostFtdcReqChangeAccountField_BrokerBranchID_set)
    TradeDate = property(_thosttraderapi.CThostFtdcReqChangeAccountField_TradeDate_get, _thosttraderapi.CThostFtdcReqChangeAccountField_TradeDate_set)
    TradeTime = property(_thosttraderapi.CThostFtdcReqChangeAccountField_TradeTime_get, _thosttraderapi.CThostFtdcReqChangeAccountField_TradeTime_set)
    BankSerial = property(_thosttraderapi.CThostFtdcReqChangeAccountField_BankSerial_get, _thosttraderapi.CThostFtdcReqChangeAccountField_BankSerial_set)
    TradingDay = property(_thosttraderapi.CThostFtdcReqChangeAccountField_TradingDay_get, _thosttraderapi.CThostFtdcReqChangeAccountField_TradingDay_set)
    PlateSerial = property(_thosttraderapi.CThostFtdcReqChangeAccountField_PlateSerial_get, _thosttraderapi.CThostFtdcReqChangeAccountField_PlateSerial_set)
    LastFragment = property(_thosttraderapi.CThostFtdcReqChangeAccountField_LastFragment_get, _thosttraderapi.CThostFtdcReqChangeAccountField_LastFragment_set)
    SessionID = property(_thosttraderapi.CThostFtdcReqChangeAccountField_SessionID_get, _thosttraderapi.CThostFtdcReqChangeAccountField_SessionID_set)
    CustomerName = property(_thosttraderapi.CThostFtdcReqChangeAccountField_CustomerName_get, _thosttraderapi.CThostFtdcReqChangeAccountField_CustomerName_set)
    IdCardType = property(_thosttraderapi.CThostFtdcReqChangeAccountField_IdCardType_get, _thosttraderapi.CThostFtdcReqChangeAccountField_IdCardType_set)
    IdentifiedCardNo = property(_thosttraderapi.CThostFtdcReqChangeAccountField_IdentifiedCardNo_get, _thosttraderapi.CThostFtdcReqChangeAccountField_IdentifiedCardNo_set)
    Gender = property(_thosttraderapi.CThostFtdcReqChangeAccountField_Gender_get, _thosttraderapi.CThostFtdcReqChangeAccountField_Gender_set)
    CountryCode = property(_thosttraderapi.CThostFtdcReqChangeAccountField_CountryCode_get, _thosttraderapi.CThostFtdcReqChangeAccountField_CountryCode_set)
    CustType = property(_thosttraderapi.CThostFtdcReqChangeAccountField_CustType_get, _thosttraderapi.CThostFtdcReqChangeAccountField_CustType_set)
    Address = property(_thosttraderapi.CThostFtdcReqChangeAccountField_Address_get, _thosttraderapi.CThostFtdcReqChangeAccountField_Address_set)
    ZipCode = property(_thosttraderapi.CThostFtdcReqChangeAccountField_ZipCode_get, _thosttraderapi.CThostFtdcReqChangeAccountField_ZipCode_set)
    Telephone = property(_thosttraderapi.CThostFtdcReqChangeAccountField_Telephone_get, _thosttraderapi.CThostFtdcReqChangeAccountField_Telephone_set)
    MobilePhone = property(_thosttraderapi.CThostFtdcReqChangeAccountField_MobilePhone_get, _thosttraderapi.CThostFtdcReqChangeAccountField_MobilePhone_set)
    Fax = property(_thosttraderapi.CThostFtdcReqChangeAccountField_Fax_get, _thosttraderapi.CThostFtdcReqChangeAccountField_Fax_set)
    EMail = property(_thosttraderapi.CThostFtdcReqChangeAccountField_EMail_get, _thosttraderapi.CThostFtdcReqChangeAccountField_EMail_set)
    MoneyAccountStatus = property(_thosttraderapi.CThostFtdcReqChangeAccountField_MoneyAccountStatus_get, _thosttraderapi.CThostFtdcReqChangeAccountField_MoneyAccountStatus_set)
    BankAccount = property(_thosttraderapi.CThostFtdcReqChangeAccountField_BankAccount_get, _thosttraderapi.CThostFtdcReqChangeAccountField_BankAccount_set)
    BankPassWord = property(_thosttraderapi.CThostFtdcReqChangeAccountField_BankPassWord_get, _thosttraderapi.CThostFtdcReqChangeAccountField_BankPassWord_set)
    NewBankAccount = property(_thosttraderapi.CThostFtdcReqChangeAccountField_NewBankAccount_get, _thosttraderapi.CThostFtdcReqChangeAccountField_NewBankAccount_set)
    NewBankPassWord = property(_thosttraderapi.CThostFtdcReqChangeAccountField_NewBankPassWord_get, _thosttraderapi.CThostFtdcReqChangeAccountField_NewBankPassWord_set)
    AccountID = property(_thosttraderapi.CThostFtdcReqChangeAccountField_AccountID_get, _thosttraderapi.CThostFtdcReqChangeAccountField_AccountID_set)
    Password = property(_thosttraderapi.CThostFtdcReqChangeAccountField_Password_get, _thosttraderapi.CThostFtdcReqChangeAccountField_Password_set)
    BankAccType = property(_thosttraderapi.CThostFtdcReqChangeAccountField_BankAccType_get, _thosttraderapi.CThostFtdcReqChangeAccountField_BankAccType_set)
    InstallID = property(_thosttraderapi.CThostFtdcReqChangeAccountField_InstallID_get, _thosttraderapi.CThostFtdcReqChangeAccountField_InstallID_set)
    VerifyCertNoFlag = property(_thosttraderapi.CThostFtdcReqChangeAccountField_VerifyCertNoFlag_get, _thosttraderapi.CThostFtdcReqChangeAccountField_VerifyCertNoFlag_set)
    CurrencyID = property(_thosttraderapi.CThostFtdcReqChangeAccountField_CurrencyID_get, _thosttraderapi.CThostFtdcReqChangeAccountField_CurrencyID_set)
    BrokerIDByBank = property(_thosttraderapi.CThostFtdcReqChangeAccountField_BrokerIDByBank_get, _thosttraderapi.CThostFtdcReqChangeAccountField_BrokerIDByBank_set)
    BankPwdFlag = property(_thosttraderapi.CThostFtdcReqChangeAccountField_BankPwdFlag_get, _thosttraderapi.CThostFtdcReqChangeAccountField_BankPwdFlag_set)
    SecuPwdFlag = property(_thosttraderapi.CThostFtdcReqChangeAccountField_SecuPwdFlag_get, _thosttraderapi.CThostFtdcReqChangeAccountField_SecuPwdFlag_set)
    TID = property(_thosttraderapi.CThostFtdcReqChangeAccountField_TID_get, _thosttraderapi.CThostFtdcReqChangeAccountField_TID_set)
    Digest = property(_thosttraderapi.CThostFtdcReqChangeAccountField_Digest_get, _thosttraderapi.CThostFtdcReqChangeAccountField_Digest_set)
    LongCustomerName = property(_thosttraderapi.CThostFtdcReqChangeAccountField_LongCustomerName_get, _thosttraderapi.CThostFtdcReqChangeAccountField_LongCustomerName_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcReqChangeAccountField_swiginit(self, _thosttraderapi.new_CThostFtdcReqChangeAccountField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcReqChangeAccountField

# Register CThostFtdcReqChangeAccountField in _thosttraderapi:
_thosttraderapi.CThostFtdcReqChangeAccountField_swigregister(CThostFtdcReqChangeAccountField)

class CThostFtdcReqTransferField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    TradeCode = property(_thosttraderapi.CThostFtdcReqTransferField_TradeCode_get, _thosttraderapi.CThostFtdcReqTransferField_TradeCode_set)
    BankID = property(_thosttraderapi.CThostFtdcReqTransferField_BankID_get, _thosttraderapi.CThostFtdcReqTransferField_BankID_set)
    BankBranchID = property(_thosttraderapi.CThostFtdcReqTransferField_BankBranchID_get, _thosttraderapi.CThostFtdcReqTransferField_BankBranchID_set)
    BrokerID = property(_thosttraderapi.CThostFtdcReqTransferField_BrokerID_get, _thosttraderapi.CThostFtdcReqTransferField_BrokerID_set)
    BrokerBranchID = property(_thosttraderapi.CThostFtdcReqTransferField_BrokerBranchID_get, _thosttraderapi.CThostFtdcReqTransferField_BrokerBranchID_set)
    TradeDate = property(_thosttraderapi.CThostFtdcReqTransferField_TradeDate_get, _thosttraderapi.CThostFtdcReqTransferField_TradeDate_set)
    TradeTime = property(_thosttraderapi.CThostFtdcReqTransferField_TradeTime_get, _thosttraderapi.CThostFtdcReqTransferField_TradeTime_set)
    BankSerial = property(_thosttraderapi.CThostFtdcReqTransferField_BankSerial_get, _thosttraderapi.CThostFtdcReqTransferField_BankSerial_set)
    TradingDay = property(_thosttraderapi.CThostFtdcReqTransferField_TradingDay_get, _thosttraderapi.CThostFtdcReqTransferField_TradingDay_set)
    PlateSerial = property(_thosttraderapi.CThostFtdcReqTransferField_PlateSerial_get, _thosttraderapi.CThostFtdcReqTransferField_PlateSerial_set)
    LastFragment = property(_thosttraderapi.CThostFtdcReqTransferField_LastFragment_get, _thosttraderapi.CThostFtdcReqTransferField_LastFragment_set)
    SessionID = property(_thosttraderapi.CThostFtdcReqTransferField_SessionID_get, _thosttraderapi.CThostFtdcReqTransferField_SessionID_set)
    CustomerName = property(_thosttraderapi.CThostFtdcReqTransferField_CustomerName_get, _thosttraderapi.CThostFtdcReqTransferField_CustomerName_set)
    IdCardType = property(_thosttraderapi.CThostFtdcReqTransferField_IdCardType_get, _thosttraderapi.CThostFtdcReqTransferField_IdCardType_set)
    IdentifiedCardNo = property(_thosttraderapi.CThostFtdcReqTransferField_IdentifiedCardNo_get, _thosttraderapi.CThostFtdcReqTransferField_IdentifiedCardNo_set)
    CustType = property(_thosttraderapi.CThostFtdcReqTransferField_CustType_get, _thosttraderapi.CThostFtdcReqTransferField_CustType_set)
    BankAccount = property(_thosttraderapi.CThostFtdcReqTransferField_BankAccount_get, _thosttraderapi.CThostFtdcReqTransferField_BankAccount_set)
    BankPassWord = property(_thosttraderapi.CThostFtdcReqTransferField_BankPassWord_get, _thosttraderapi.CThostFtdcReqTransferField_BankPassWord_set)
    AccountID = property(_thosttraderapi.CThostFtdcReqTransferField_AccountID_get, _thosttraderapi.CThostFtdcReqTransferField_AccountID_set)
    Password = property(_thosttraderapi.CThostFtdcReqTransferField_Password_get, _thosttraderapi.CThostFtdcReqTransferField_Password_set)
    InstallID = property(_thosttraderapi.CThostFtdcReqTransferField_InstallID_get, _thosttraderapi.CThostFtdcReqTransferField_InstallID_set)
    FutureSerial = property(_thosttraderapi.CThostFtdcReqTransferField_FutureSerial_get, _thosttraderapi.CThostFtdcReqTransferField_FutureSerial_set)
    UserID = property(_thosttraderapi.CThostFtdcReqTransferField_UserID_get, _thosttraderapi.CThostFtdcReqTransferField_UserID_set)
    VerifyCertNoFlag = property(_thosttraderapi.CThostFtdcReqTransferField_VerifyCertNoFlag_get, _thosttraderapi.CThostFtdcReqTransferField_VerifyCertNoFlag_set)
    CurrencyID = property(_thosttraderapi.CThostFtdcReqTransferField_CurrencyID_get, _thosttraderapi.CThostFtdcReqTransferField_CurrencyID_set)
    TradeAmount = property(_thosttraderapi.CThostFtdcReqTransferField_TradeAmount_get, _thosttraderapi.CThostFtdcReqTransferField_TradeAmount_set)
    FutureFetchAmount = property(_thosttraderapi.CThostFtdcReqTransferField_FutureFetchAmount_get, _thosttraderapi.CThostFtdcReqTransferField_FutureFetchAmount_set)
    FeePayFlag = property(_thosttraderapi.CThostFtdcReqTransferField_FeePayFlag_get, _thosttraderapi.CThostFtdcReqTransferField_FeePayFlag_set)
    CustFee = property(_thosttraderapi.CThostFtdcReqTransferField_CustFee_get, _thosttraderapi.CThostFtdcReqTransferField_CustFee_set)
    BrokerFee = property(_thosttraderapi.CThostFtdcReqTransferField_BrokerFee_get, _thosttraderapi.CThostFtdcReqTransferField_BrokerFee_set)
    Message = property(_thosttraderapi.CThostFtdcReqTransferField_Message_get, _thosttraderapi.CThostFtdcReqTransferField_Message_set)
    Digest = property(_thosttraderapi.CThostFtdcReqTransferField_Digest_get, _thosttraderapi.CThostFtdcReqTransferField_Digest_set)
    BankAccType = property(_thosttraderapi.CThostFtdcReqTransferField_BankAccType_get, _thosttraderapi.CThostFtdcReqTransferField_BankAccType_set)
    DeviceID = property(_thosttraderapi.CThostFtdcReqTransferField_DeviceID_get, _thosttraderapi.CThostFtdcReqTransferField_DeviceID_set)
    BankSecuAccType = property(_thosttraderapi.CThostFtdcReqTransferField_BankSecuAccType_get, _thosttraderapi.CThostFtdcReqTransferField_BankSecuAccType_set)
    BrokerIDByBank = property(_thosttraderapi.CThostFtdcReqTransferField_BrokerIDByBank_get, _thosttraderapi.CThostFtdcReqTransferField_BrokerIDByBank_set)
    BankSecuAcc = property(_thosttraderapi.CThostFtdcReqTransferField_BankSecuAcc_get, _thosttraderapi.CThostFtdcReqTransferField_BankSecuAcc_set)
    BankPwdFlag = property(_thosttraderapi.CThostFtdcReqTransferField_BankPwdFlag_get, _thosttraderapi.CThostFtdcReqTransferField_BankPwdFlag_set)
    SecuPwdFlag = property(_thosttraderapi.CThostFtdcReqTransferField_SecuPwdFlag_get, _thosttraderapi.CThostFtdcReqTransferField_SecuPwdFlag_set)
    OperNo = property(_thosttraderapi.CThostFtdcReqTransferField_OperNo_get, _thosttraderapi.CThostFtdcReqTransferField_OperNo_set)
    RequestID = property(_thosttraderapi.CThostFtdcReqTransferField_RequestID_get, _thosttraderapi.CThostFtdcReqTransferField_RequestID_set)
    TID = property(_thosttraderapi.CThostFtdcReqTransferField_TID_get, _thosttraderapi.CThostFtdcReqTransferField_TID_set)
    TransferStatus = property(_thosttraderapi.CThostFtdcReqTransferField_TransferStatus_get, _thosttraderapi.CThostFtdcReqTransferField_TransferStatus_set)
    LongCustomerName = property(_thosttraderapi.CThostFtdcReqTransferField_LongCustomerName_get, _thosttraderapi.CThostFtdcReqTransferField_LongCustomerName_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcReqTransferField_swiginit(self, _thosttraderapi.new_CThostFtdcReqTransferField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcReqTransferField

# Register CThostFtdcReqTransferField in _thosttraderapi:
_thosttraderapi.CThostFtdcReqTransferField_swigregister(CThostFtdcReqTransferField)

class CThostFtdcRspTransferField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    TradeCode = property(_thosttraderapi.CThostFtdcRspTransferField_TradeCode_get, _thosttraderapi.CThostFtdcRspTransferField_TradeCode_set)
    BankID = property(_thosttraderapi.CThostFtdcRspTransferField_BankID_get, _thosttraderapi.CThostFtdcRspTransferField_BankID_set)
    BankBranchID = property(_thosttraderapi.CThostFtdcRspTransferField_BankBranchID_get, _thosttraderapi.CThostFtdcRspTransferField_BankBranchID_set)
    BrokerID = property(_thosttraderapi.CThostFtdcRspTransferField_BrokerID_get, _thosttraderapi.CThostFtdcRspTransferField_BrokerID_set)
    BrokerBranchID = property(_thosttraderapi.CThostFtdcRspTransferField_BrokerBranchID_get, _thosttraderapi.CThostFtdcRspTransferField_BrokerBranchID_set)
    TradeDate = property(_thosttraderapi.CThostFtdcRspTransferField_TradeDate_get, _thosttraderapi.CThostFtdcRspTransferField_TradeDate_set)
    TradeTime = property(_thosttraderapi.CThostFtdcRspTransferField_TradeTime_get, _thosttraderapi.CThostFtdcRspTransferField_TradeTime_set)
    BankSerial = property(_thosttraderapi.CThostFtdcRspTransferField_BankSerial_get, _thosttraderapi.CThostFtdcRspTransferField_BankSerial_set)
    TradingDay = property(_thosttraderapi.CThostFtdcRspTransferField_TradingDay_get, _thosttraderapi.CThostFtdcRspTransferField_TradingDay_set)
    PlateSerial = property(_thosttraderapi.CThostFtdcRspTransferField_PlateSerial_get, _thosttraderapi.CThostFtdcRspTransferField_PlateSerial_set)
    LastFragment = property(_thosttraderapi.CThostFtdcRspTransferField_LastFragment_get, _thosttraderapi.CThostFtdcRspTransferField_LastFragment_set)
    SessionID = property(_thosttraderapi.CThostFtdcRspTransferField_SessionID_get, _thosttraderapi.CThostFtdcRspTransferField_SessionID_set)
    CustomerName = property(_thosttraderapi.CThostFtdcRspTransferField_CustomerName_get, _thosttraderapi.CThostFtdcRspTransferField_CustomerName_set)
    IdCardType = property(_thosttraderapi.CThostFtdcRspTransferField_IdCardType_get, _thosttraderapi.CThostFtdcRspTransferField_IdCardType_set)
    IdentifiedCardNo = property(_thosttraderapi.CThostFtdcRspTransferField_IdentifiedCardNo_get, _thosttraderapi.CThostFtdcRspTransferField_IdentifiedCardNo_set)
    CustType = property(_thosttraderapi.CThostFtdcRspTransferField_CustType_get, _thosttraderapi.CThostFtdcRspTransferField_CustType_set)
    BankAccount = property(_thosttraderapi.CThostFtdcRspTransferField_BankAccount_get, _thosttraderapi.CThostFtdcRspTransferField_BankAccount_set)
    BankPassWord = property(_thosttraderapi.CThostFtdcRspTransferField_BankPassWord_get, _thosttraderapi.CThostFtdcRspTransferField_BankPassWord_set)
    AccountID = property(_thosttraderapi.CThostFtdcRspTransferField_AccountID_get, _thosttraderapi.CThostFtdcRspTransferField_AccountID_set)
    Password = property(_thosttraderapi.CThostFtdcRspTransferField_Password_get, _thosttraderapi.CThostFtdcRspTransferField_Password_set)
    InstallID = property(_thosttraderapi.CThostFtdcRspTransferField_InstallID_get, _thosttraderapi.CThostFtdcRspTransferField_InstallID_set)
    FutureSerial = property(_thosttraderapi.CThostFtdcRspTransferField_FutureSerial_get, _thosttraderapi.CThostFtdcRspTransferField_FutureSerial_set)
    UserID = property(_thosttraderapi.CThostFtdcRspTransferField_UserID_get, _thosttraderapi.CThostFtdcRspTransferField_UserID_set)
    VerifyCertNoFlag = property(_thosttraderapi.CThostFtdcRspTransferField_VerifyCertNoFlag_get, _thosttraderapi.CThostFtdcRspTransferField_VerifyCertNoFlag_set)
    CurrencyID = property(_thosttraderapi.CThostFtdcRspTransferField_CurrencyID_get, _thosttraderapi.CThostFtdcRspTransferField_CurrencyID_set)
    TradeAmount = property(_thosttraderapi.CThostFtdcRspTransferField_TradeAmount_get, _thosttraderapi.CThostFtdcRspTransferField_TradeAmount_set)
    FutureFetchAmount = property(_thosttraderapi.CThostFtdcRspTransferField_FutureFetchAmount_get, _thosttraderapi.CThostFtdcRspTransferField_FutureFetchAmount_set)
    FeePayFlag = property(_thosttraderapi.CThostFtdcRspTransferField_FeePayFlag_get, _thosttraderapi.CThostFtdcRspTransferField_FeePayFlag_set)
    CustFee = property(_thosttraderapi.CThostFtdcRspTransferField_CustFee_get, _thosttraderapi.CThostFtdcRspTransferField_CustFee_set)
    BrokerFee = property(_thosttraderapi.CThostFtdcRspTransferField_BrokerFee_get, _thosttraderapi.CThostFtdcRspTransferField_BrokerFee_set)
    Message = property(_thosttraderapi.CThostFtdcRspTransferField_Message_get, _thosttraderapi.CThostFtdcRspTransferField_Message_set)
    Digest = property(_thosttraderapi.CThostFtdcRspTransferField_Digest_get, _thosttraderapi.CThostFtdcRspTransferField_Digest_set)
    BankAccType = property(_thosttraderapi.CThostFtdcRspTransferField_BankAccType_get, _thosttraderapi.CThostFtdcRspTransferField_BankAccType_set)
    DeviceID = property(_thosttraderapi.CThostFtdcRspTransferField_DeviceID_get, _thosttraderapi.CThostFtdcRspTransferField_DeviceID_set)
    BankSecuAccType = property(_thosttraderapi.CThostFtdcRspTransferField_BankSecuAccType_get, _thosttraderapi.CThostFtdcRspTransferField_BankSecuAccType_set)
    BrokerIDByBank = property(_thosttraderapi.CThostFtdcRspTransferField_BrokerIDByBank_get, _thosttraderapi.CThostFtdcRspTransferField_BrokerIDByBank_set)
    BankSecuAcc = property(_thosttraderapi.CThostFtdcRspTransferField_BankSecuAcc_get, _thosttraderapi.CThostFtdcRspTransferField_BankSecuAcc_set)
    BankPwdFlag = property(_thosttraderapi.CThostFtdcRspTransferField_BankPwdFlag_get, _thosttraderapi.CThostFtdcRspTransferField_BankPwdFlag_set)
    SecuPwdFlag = property(_thosttraderapi.CThostFtdcRspTransferField_SecuPwdFlag_get, _thosttraderapi.CThostFtdcRspTransferField_SecuPwdFlag_set)
    OperNo = property(_thosttraderapi.CThostFtdcRspTransferField_OperNo_get, _thosttraderapi.CThostFtdcRspTransferField_OperNo_set)
    RequestID = property(_thosttraderapi.CThostFtdcRspTransferField_RequestID_get, _thosttraderapi.CThostFtdcRspTransferField_RequestID_set)
    TID = property(_thosttraderapi.CThostFtdcRspTransferField_TID_get, _thosttraderapi.CThostFtdcRspTransferField_TID_set)
    TransferStatus = property(_thosttraderapi.CThostFtdcRspTransferField_TransferStatus_get, _thosttraderapi.CThostFtdcRspTransferField_TransferStatus_set)
    ErrorID = property(_thosttraderapi.CThostFtdcRspTransferField_ErrorID_get, _thosttraderapi.CThostFtdcRspTransferField_ErrorID_set)
    ErrorMsg = property(_thosttraderapi.CThostFtdcRspTransferField_ErrorMsg_get, _thosttraderapi.CThostFtdcRspTransferField_ErrorMsg_set)
    LongCustomerName = property(_thosttraderapi.CThostFtdcRspTransferField_LongCustomerName_get, _thosttraderapi.CThostFtdcRspTransferField_LongCustomerName_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcRspTransferField_swiginit(self, _thosttraderapi.new_CThostFtdcRspTransferField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcRspTransferField

# Register CThostFtdcRspTransferField in _thosttraderapi:
_thosttraderapi.CThostFtdcRspTransferField_swigregister(CThostFtdcRspTransferField)

class CThostFtdcReqRepealField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    RepealTimeInterval = property(_thosttraderapi.CThostFtdcReqRepealField_RepealTimeInterval_get, _thosttraderapi.CThostFtdcReqRepealField_RepealTimeInterval_set)
    RepealedTimes = property(_thosttraderapi.CThostFtdcReqRepealField_RepealedTimes_get, _thosttraderapi.CThostFtdcReqRepealField_RepealedTimes_set)
    BankRepealFlag = property(_thosttraderapi.CThostFtdcReqRepealField_BankRepealFlag_get, _thosttraderapi.CThostFtdcReqRepealField_BankRepealFlag_set)
    BrokerRepealFlag = property(_thosttraderapi.CThostFtdcReqRepealField_BrokerRepealFlag_get, _thosttraderapi.CThostFtdcReqRepealField_BrokerRepealFlag_set)
    PlateRepealSerial = property(_thosttraderapi.CThostFtdcReqRepealField_PlateRepealSerial_get, _thosttraderapi.CThostFtdcReqRepealField_PlateRepealSerial_set)
    BankRepealSerial = property(_thosttraderapi.CThostFtdcReqRepealField_BankRepealSerial_get, _thosttraderapi.CThostFtdcReqRepealField_BankRepealSerial_set)
    FutureRepealSerial = property(_thosttraderapi.CThostFtdcReqRepealField_FutureRepealSerial_get, _thosttraderapi.CThostFtdcReqRepealField_FutureRepealSerial_set)
    TradeCode = property(_thosttraderapi.CThostFtdcReqRepealField_TradeCode_get, _thosttraderapi.CThostFtdcReqRepealField_TradeCode_set)
    BankID = property(_thosttraderapi.CThostFtdcReqRepealField_BankID_get, _thosttraderapi.CThostFtdcReqRepealField_BankID_set)
    BankBranchID = property(_thosttraderapi.CThostFtdcReqRepealField_BankBranchID_get, _thosttraderapi.CThostFtdcReqRepealField_BankBranchID_set)
    BrokerID = property(_thosttraderapi.CThostFtdcReqRepealField_BrokerID_get, _thosttraderapi.CThostFtdcReqRepealField_BrokerID_set)
    BrokerBranchID = property(_thosttraderapi.CThostFtdcReqRepealField_BrokerBranchID_get, _thosttraderapi.CThostFtdcReqRepealField_BrokerBranchID_set)
    TradeDate = property(_thosttraderapi.CThostFtdcReqRepealField_TradeDate_get, _thosttraderapi.CThostFtdcReqRepealField_TradeDate_set)
    TradeTime = property(_thosttraderapi.CThostFtdcReqRepealField_TradeTime_get, _thosttraderapi.CThostFtdcReqRepealField_TradeTime_set)
    BankSerial = property(_thosttraderapi.CThostFtdcReqRepealField_BankSerial_get, _thosttraderapi.CThostFtdcReqRepealField_BankSerial_set)
    TradingDay = property(_thosttraderapi.CThostFtdcReqRepealField_TradingDay_get, _thosttraderapi.CThostFtdcReqRepealField_TradingDay_set)
    PlateSerial = property(_thosttraderapi.CThostFtdcReqRepealField_PlateSerial_get, _thosttraderapi.CThostFtdcReqRepealField_PlateSerial_set)
    LastFragment = property(_thosttraderapi.CThostFtdcReqRepealField_LastFragment_get, _thosttraderapi.CThostFtdcReqRepealField_LastFragment_set)
    SessionID = property(_thosttraderapi.CThostFtdcReqRepealField_SessionID_get, _thosttraderapi.CThostFtdcReqRepealField_SessionID_set)
    CustomerName = property(_thosttraderapi.CThostFtdcReqRepealField_CustomerName_get, _thosttraderapi.CThostFtdcReqRepealField_CustomerName_set)
    IdCardType = property(_thosttraderapi.CThostFtdcReqRepealField_IdCardType_get, _thosttraderapi.CThostFtdcReqRepealField_IdCardType_set)
    IdentifiedCardNo = property(_thosttraderapi.CThostFtdcReqRepealField_IdentifiedCardNo_get, _thosttraderapi.CThostFtdcReqRepealField_IdentifiedCardNo_set)
    CustType = property(_thosttraderapi.CThostFtdcReqRepealField_CustType_get, _thosttraderapi.CThostFtdcReqRepealField_CustType_set)
    BankAccount = property(_thosttraderapi.CThostFtdcReqRepealField_BankAccount_get, _thosttraderapi.CThostFtdcReqRepealField_BankAccount_set)
    BankPassWord = property(_thosttraderapi.CThostFtdcReqRepealField_BankPassWord_get, _thosttraderapi.CThostFtdcReqRepealField_BankPassWord_set)
    AccountID = property(_thosttraderapi.CThostFtdcReqRepealField_AccountID_get, _thosttraderapi.CThostFtdcReqRepealField_AccountID_set)
    Password = property(_thosttraderapi.CThostFtdcReqRepealField_Password_get, _thosttraderapi.CThostFtdcReqRepealField_Password_set)
    InstallID = property(_thosttraderapi.CThostFtdcReqRepealField_InstallID_get, _thosttraderapi.CThostFtdcReqRepealField_InstallID_set)
    FutureSerial = property(_thosttraderapi.CThostFtdcReqRepealField_FutureSerial_get, _thosttraderapi.CThostFtdcReqRepealField_FutureSerial_set)
    UserID = property(_thosttraderapi.CThostFtdcReqRepealField_UserID_get, _thosttraderapi.CThostFtdcReqRepealField_UserID_set)
    VerifyCertNoFlag = property(_thosttraderapi.CThostFtdcReqRepealField_VerifyCertNoFlag_get, _thosttraderapi.CThostFtdcReqRepealField_VerifyCertNoFlag_set)
    CurrencyID = property(_thosttraderapi.CThostFtdcReqRepealField_CurrencyID_get, _thosttraderapi.CThostFtdcReqRepealField_CurrencyID_set)
    TradeAmount = property(_thosttraderapi.CThostFtdcReqRepealField_TradeAmount_get, _thosttraderapi.CThostFtdcReqRepealField_TradeAmount_set)
    FutureFetchAmount = property(_thosttraderapi.CThostFtdcReqRepealField_FutureFetchAmount_get, _thosttraderapi.CThostFtdcReqRepealField_FutureFetchAmount_set)
    FeePayFlag = property(_thosttraderapi.CThostFtdcReqRepealField_FeePayFlag_get, _thosttraderapi.CThostFtdcReqRepealField_FeePayFlag_set)
    CustFee = property(_thosttraderapi.CThostFtdcReqRepealField_CustFee_get, _thosttraderapi.CThostFtdcReqRepealField_CustFee_set)
    BrokerFee = property(_thosttraderapi.CThostFtdcReqRepealField_BrokerFee_get, _thosttraderapi.CThostFtdcReqRepealField_BrokerFee_set)
    Message = property(_thosttraderapi.CThostFtdcReqRepealField_Message_get, _thosttraderapi.CThostFtdcReqRepealField_Message_set)
    Digest = property(_thosttraderapi.CThostFtdcReqRepealField_Digest_get, _thosttraderapi.CThostFtdcReqRepealField_Digest_set)
    BankAccType = property(_thosttraderapi.CThostFtdcReqRepealField_BankAccType_get, _thosttraderapi.CThostFtdcReqRepealField_BankAccType_set)
    DeviceID = property(_thosttraderapi.CThostFtdcReqRepealField_DeviceID_get, _thosttraderapi.CThostFtdcReqRepealField_DeviceID_set)
    BankSecuAccType = property(_thosttraderapi.CThostFtdcReqRepealField_BankSecuAccType_get, _thosttraderapi.CThostFtdcReqRepealField_BankSecuAccType_set)
    BrokerIDByBank = property(_thosttraderapi.CThostFtdcReqRepealField_BrokerIDByBank_get, _thosttraderapi.CThostFtdcReqRepealField_BrokerIDByBank_set)
    BankSecuAcc = property(_thosttraderapi.CThostFtdcReqRepealField_BankSecuAcc_get, _thosttraderapi.CThostFtdcReqRepealField_BankSecuAcc_set)
    BankPwdFlag = property(_thosttraderapi.CThostFtdcReqRepealField_BankPwdFlag_get, _thosttraderapi.CThostFtdcReqRepealField_BankPwdFlag_set)
    SecuPwdFlag = property(_thosttraderapi.CThostFtdcReqRepealField_SecuPwdFlag_get, _thosttraderapi.CThostFtdcReqRepealField_SecuPwdFlag_set)
    OperNo = property(_thosttraderapi.CThostFtdcReqRepealField_OperNo_get, _thosttraderapi.CThostFtdcReqRepealField_OperNo_set)
    RequestID = property(_thosttraderapi.CThostFtdcReqRepealField_RequestID_get, _thosttraderapi.CThostFtdcReqRepealField_RequestID_set)
    TID = property(_thosttraderapi.CThostFtdcReqRepealField_TID_get, _thosttraderapi.CThostFtdcReqRepealField_TID_set)
    TransferStatus = property(_thosttraderapi.CThostFtdcReqRepealField_TransferStatus_get, _thosttraderapi.CThostFtdcReqRepealField_TransferStatus_set)
    LongCustomerName = property(_thosttraderapi.CThostFtdcReqRepealField_LongCustomerName_get, _thosttraderapi.CThostFtdcReqRepealField_LongCustomerName_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcReqRepealField_swiginit(self, _thosttraderapi.new_CThostFtdcReqRepealField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcReqRepealField

# Register CThostFtdcReqRepealField in _thosttraderapi:
_thosttraderapi.CThostFtdcReqRepealField_swigregister(CThostFtdcReqRepealField)

class CThostFtdcRspRepealField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    RepealTimeInterval = property(_thosttraderapi.CThostFtdcRspRepealField_RepealTimeInterval_get, _thosttraderapi.CThostFtdcRspRepealField_RepealTimeInterval_set)
    RepealedTimes = property(_thosttraderapi.CThostFtdcRspRepealField_RepealedTimes_get, _thosttraderapi.CThostFtdcRspRepealField_RepealedTimes_set)
    BankRepealFlag = property(_thosttraderapi.CThostFtdcRspRepealField_BankRepealFlag_get, _thosttraderapi.CThostFtdcRspRepealField_BankRepealFlag_set)
    BrokerRepealFlag = property(_thosttraderapi.CThostFtdcRspRepealField_BrokerRepealFlag_get, _thosttraderapi.CThostFtdcRspRepealField_BrokerRepealFlag_set)
    PlateRepealSerial = property(_thosttraderapi.CThostFtdcRspRepealField_PlateRepealSerial_get, _thosttraderapi.CThostFtdcRspRepealField_PlateRepealSerial_set)
    BankRepealSerial = property(_thosttraderapi.CThostFtdcRspRepealField_BankRepealSerial_get, _thosttraderapi.CThostFtdcRspRepealField_BankRepealSerial_set)
    FutureRepealSerial = property(_thosttraderapi.CThostFtdcRspRepealField_FutureRepealSerial_get, _thosttraderapi.CThostFtdcRspRepealField_FutureRepealSerial_set)
    TradeCode = property(_thosttraderapi.CThostFtdcRspRepealField_TradeCode_get, _thosttraderapi.CThostFtdcRspRepealField_TradeCode_set)
    BankID = property(_thosttraderapi.CThostFtdcRspRepealField_BankID_get, _thosttraderapi.CThostFtdcRspRepealField_BankID_set)
    BankBranchID = property(_thosttraderapi.CThostFtdcRspRepealField_BankBranchID_get, _thosttraderapi.CThostFtdcRspRepealField_BankBranchID_set)
    BrokerID = property(_thosttraderapi.CThostFtdcRspRepealField_BrokerID_get, _thosttraderapi.CThostFtdcRspRepealField_BrokerID_set)
    BrokerBranchID = property(_thosttraderapi.CThostFtdcRspRepealField_BrokerBranchID_get, _thosttraderapi.CThostFtdcRspRepealField_BrokerBranchID_set)
    TradeDate = property(_thosttraderapi.CThostFtdcRspRepealField_TradeDate_get, _thosttraderapi.CThostFtdcRspRepealField_TradeDate_set)
    TradeTime = property(_thosttraderapi.CThostFtdcRspRepealField_TradeTime_get, _thosttraderapi.CThostFtdcRspRepealField_TradeTime_set)
    BankSerial = property(_thosttraderapi.CThostFtdcRspRepealField_BankSerial_get, _thosttraderapi.CThostFtdcRspRepealField_BankSerial_set)
    TradingDay = property(_thosttraderapi.CThostFtdcRspRepealField_TradingDay_get, _thosttraderapi.CThostFtdcRspRepealField_TradingDay_set)
    PlateSerial = property(_thosttraderapi.CThostFtdcRspRepealField_PlateSerial_get, _thosttraderapi.CThostFtdcRspRepealField_PlateSerial_set)
    LastFragment = property(_thosttraderapi.CThostFtdcRspRepealField_LastFragment_get, _thosttraderapi.CThostFtdcRspRepealField_LastFragment_set)
    SessionID = property(_thosttraderapi.CThostFtdcRspRepealField_SessionID_get, _thosttraderapi.CThostFtdcRspRepealField_SessionID_set)
    CustomerName = property(_thosttraderapi.CThostFtdcRspRepealField_CustomerName_get, _thosttraderapi.CThostFtdcRspRepealField_CustomerName_set)
    IdCardType = property(_thosttraderapi.CThostFtdcRspRepealField_IdCardType_get, _thosttraderapi.CThostFtdcRspRepealField_IdCardType_set)
    IdentifiedCardNo = property(_thosttraderapi.CThostFtdcRspRepealField_IdentifiedCardNo_get, _thosttraderapi.CThostFtdcRspRepealField_IdentifiedCardNo_set)
    CustType = property(_thosttraderapi.CThostFtdcRspRepealField_CustType_get, _thosttraderapi.CThostFtdcRspRepealField_CustType_set)
    BankAccount = property(_thosttraderapi.CThostFtdcRspRepealField_BankAccount_get, _thosttraderapi.CThostFtdcRspRepealField_BankAccount_set)
    BankPassWord = property(_thosttraderapi.CThostFtdcRspRepealField_BankPassWord_get, _thosttraderapi.CThostFtdcRspRepealField_BankPassWord_set)
    AccountID = property(_thosttraderapi.CThostFtdcRspRepealField_AccountID_get, _thosttraderapi.CThostFtdcRspRepealField_AccountID_set)
    Password = property(_thosttraderapi.CThostFtdcRspRepealField_Password_get, _thosttraderapi.CThostFtdcRspRepealField_Password_set)
    InstallID = property(_thosttraderapi.CThostFtdcRspRepealField_InstallID_get, _thosttraderapi.CThostFtdcRspRepealField_InstallID_set)
    FutureSerial = property(_thosttraderapi.CThostFtdcRspRepealField_FutureSerial_get, _thosttraderapi.CThostFtdcRspRepealField_FutureSerial_set)
    UserID = property(_thosttraderapi.CThostFtdcRspRepealField_UserID_get, _thosttraderapi.CThostFtdcRspRepealField_UserID_set)
    VerifyCertNoFlag = property(_thosttraderapi.CThostFtdcRspRepealField_VerifyCertNoFlag_get, _thosttraderapi.CThostFtdcRspRepealField_VerifyCertNoFlag_set)
    CurrencyID = property(_thosttraderapi.CThostFtdcRspRepealField_CurrencyID_get, _thosttraderapi.CThostFtdcRspRepealField_CurrencyID_set)
    TradeAmount = property(_thosttraderapi.CThostFtdcRspRepealField_TradeAmount_get, _thosttraderapi.CThostFtdcRspRepealField_TradeAmount_set)
    FutureFetchAmount = property(_thosttraderapi.CThostFtdcRspRepealField_FutureFetchAmount_get, _thosttraderapi.CThostFtdcRspRepealField_FutureFetchAmount_set)
    FeePayFlag = property(_thosttraderapi.CThostFtdcRspRepealField_FeePayFlag_get, _thosttraderapi.CThostFtdcRspRepealField_FeePayFlag_set)
    CustFee = property(_thosttraderapi.CThostFtdcRspRepealField_CustFee_get, _thosttraderapi.CThostFtdcRspRepealField_CustFee_set)
    BrokerFee = property(_thosttraderapi.CThostFtdcRspRepealField_BrokerFee_get, _thosttraderapi.CThostFtdcRspRepealField_BrokerFee_set)
    Message = property(_thosttraderapi.CThostFtdcRspRepealField_Message_get, _thosttraderapi.CThostFtdcRspRepealField_Message_set)
    Digest = property(_thosttraderapi.CThostFtdcRspRepealField_Digest_get, _thosttraderapi.CThostFtdcRspRepealField_Digest_set)
    BankAccType = property(_thosttraderapi.CThostFtdcRspRepealField_BankAccType_get, _thosttraderapi.CThostFtdcRspRepealField_BankAccType_set)
    DeviceID = property(_thosttraderapi.CThostFtdcRspRepealField_DeviceID_get, _thosttraderapi.CThostFtdcRspRepealField_DeviceID_set)
    BankSecuAccType = property(_thosttraderapi.CThostFtdcRspRepealField_BankSecuAccType_get, _thosttraderapi.CThostFtdcRspRepealField_BankSecuAccType_set)
    BrokerIDByBank = property(_thosttraderapi.CThostFtdcRspRepealField_BrokerIDByBank_get, _thosttraderapi.CThostFtdcRspRepealField_BrokerIDByBank_set)
    BankSecuAcc = property(_thosttraderapi.CThostFtdcRspRepealField_BankSecuAcc_get, _thosttraderapi.CThostFtdcRspRepealField_BankSecuAcc_set)
    BankPwdFlag = property(_thosttraderapi.CThostFtdcRspRepealField_BankPwdFlag_get, _thosttraderapi.CThostFtdcRspRepealField_BankPwdFlag_set)
    SecuPwdFlag = property(_thosttraderapi.CThostFtdcRspRepealField_SecuPwdFlag_get, _thosttraderapi.CThostFtdcRspRepealField_SecuPwdFlag_set)
    OperNo = property(_thosttraderapi.CThostFtdcRspRepealField_OperNo_get, _thosttraderapi.CThostFtdcRspRepealField_OperNo_set)
    RequestID = property(_thosttraderapi.CThostFtdcRspRepealField_RequestID_get, _thosttraderapi.CThostFtdcRspRepealField_RequestID_set)
    TID = property(_thosttraderapi.CThostFtdcRspRepealField_TID_get, _thosttraderapi.CThostFtdcRspRepealField_TID_set)
    TransferStatus = property(_thosttraderapi.CThostFtdcRspRepealField_TransferStatus_get, _thosttraderapi.CThostFtdcRspRepealField_TransferStatus_set)
    ErrorID = property(_thosttraderapi.CThostFtdcRspRepealField_ErrorID_get, _thosttraderapi.CThostFtdcRspRepealField_ErrorID_set)
    ErrorMsg = property(_thosttraderapi.CThostFtdcRspRepealField_ErrorMsg_get, _thosttraderapi.CThostFtdcRspRepealField_ErrorMsg_set)
    LongCustomerName = property(_thosttraderapi.CThostFtdcRspRepealField_LongCustomerName_get, _thosttraderapi.CThostFtdcRspRepealField_LongCustomerName_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcRspRepealField_swiginit(self, _thosttraderapi.new_CThostFtdcRspRepealField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcRspRepealField

# Register CThostFtdcRspRepealField in _thosttraderapi:
_thosttraderapi.CThostFtdcRspRepealField_swigregister(CThostFtdcRspRepealField)

class CThostFtdcReqQueryAccountField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    TradeCode = property(_thosttraderapi.CThostFtdcReqQueryAccountField_TradeCode_get, _thosttraderapi.CThostFtdcReqQueryAccountField_TradeCode_set)
    BankID = property(_thosttraderapi.CThostFtdcReqQueryAccountField_BankID_get, _thosttraderapi.CThostFtdcReqQueryAccountField_BankID_set)
    BankBranchID = property(_thosttraderapi.CThostFtdcReqQueryAccountField_BankBranchID_get, _thosttraderapi.CThostFtdcReqQueryAccountField_BankBranchID_set)
    BrokerID = property(_thosttraderapi.CThostFtdcReqQueryAccountField_BrokerID_get, _thosttraderapi.CThostFtdcReqQueryAccountField_BrokerID_set)
    BrokerBranchID = property(_thosttraderapi.CThostFtdcReqQueryAccountField_BrokerBranchID_get, _thosttraderapi.CThostFtdcReqQueryAccountField_BrokerBranchID_set)
    TradeDate = property(_thosttraderapi.CThostFtdcReqQueryAccountField_TradeDate_get, _thosttraderapi.CThostFtdcReqQueryAccountField_TradeDate_set)
    TradeTime = property(_thosttraderapi.CThostFtdcReqQueryAccountField_TradeTime_get, _thosttraderapi.CThostFtdcReqQueryAccountField_TradeTime_set)
    BankSerial = property(_thosttraderapi.CThostFtdcReqQueryAccountField_BankSerial_get, _thosttraderapi.CThostFtdcReqQueryAccountField_BankSerial_set)
    TradingDay = property(_thosttraderapi.CThostFtdcReqQueryAccountField_TradingDay_get, _thosttraderapi.CThostFtdcReqQueryAccountField_TradingDay_set)
    PlateSerial = property(_thosttraderapi.CThostFtdcReqQueryAccountField_PlateSerial_get, _thosttraderapi.CThostFtdcReqQueryAccountField_PlateSerial_set)
    LastFragment = property(_thosttraderapi.CThostFtdcReqQueryAccountField_LastFragment_get, _thosttraderapi.CThostFtdcReqQueryAccountField_LastFragment_set)
    SessionID = property(_thosttraderapi.CThostFtdcReqQueryAccountField_SessionID_get, _thosttraderapi.CThostFtdcReqQueryAccountField_SessionID_set)
    CustomerName = property(_thosttraderapi.CThostFtdcReqQueryAccountField_CustomerName_get, _thosttraderapi.CThostFtdcReqQueryAccountField_CustomerName_set)
    IdCardType = property(_thosttraderapi.CThostFtdcReqQueryAccountField_IdCardType_get, _thosttraderapi.CThostFtdcReqQueryAccountField_IdCardType_set)
    IdentifiedCardNo = property(_thosttraderapi.CThostFtdcReqQueryAccountField_IdentifiedCardNo_get, _thosttraderapi.CThostFtdcReqQueryAccountField_IdentifiedCardNo_set)
    CustType = property(_thosttraderapi.CThostFtdcReqQueryAccountField_CustType_get, _thosttraderapi.CThostFtdcReqQueryAccountField_CustType_set)
    BankAccount = property(_thosttraderapi.CThostFtdcReqQueryAccountField_BankAccount_get, _thosttraderapi.CThostFtdcReqQueryAccountField_BankAccount_set)
    BankPassWord = property(_thosttraderapi.CThostFtdcReqQueryAccountField_BankPassWord_get, _thosttraderapi.CThostFtdcReqQueryAccountField_BankPassWord_set)
    AccountID = property(_thosttraderapi.CThostFtdcReqQueryAccountField_AccountID_get, _thosttraderapi.CThostFtdcReqQueryAccountField_AccountID_set)
    Password = property(_thosttraderapi.CThostFtdcReqQueryAccountField_Password_get, _thosttraderapi.CThostFtdcReqQueryAccountField_Password_set)
    FutureSerial = property(_thosttraderapi.CThostFtdcReqQueryAccountField_FutureSerial_get, _thosttraderapi.CThostFtdcReqQueryAccountField_FutureSerial_set)
    InstallID = property(_thosttraderapi.CThostFtdcReqQueryAccountField_InstallID_get, _thosttraderapi.CThostFtdcReqQueryAccountField_InstallID_set)
    UserID = property(_thosttraderapi.CThostFtdcReqQueryAccountField_UserID_get, _thosttraderapi.CThostFtdcReqQueryAccountField_UserID_set)
    VerifyCertNoFlag = property(_thosttraderapi.CThostFtdcReqQueryAccountField_VerifyCertNoFlag_get, _thosttraderapi.CThostFtdcReqQueryAccountField_VerifyCertNoFlag_set)
    CurrencyID = property(_thosttraderapi.CThostFtdcReqQueryAccountField_CurrencyID_get, _thosttraderapi.CThostFtdcReqQueryAccountField_CurrencyID_set)
    Digest = property(_thosttraderapi.CThostFtdcReqQueryAccountField_Digest_get, _thosttraderapi.CThostFtdcReqQueryAccountField_Digest_set)
    BankAccType = property(_thosttraderapi.CThostFtdcReqQueryAccountField_BankAccType_get, _thosttraderapi.CThostFtdcReqQueryAccountField_BankAccType_set)
    DeviceID = property(_thosttraderapi.CThostFtdcReqQueryAccountField_DeviceID_get, _thosttraderapi.CThostFtdcReqQueryAccountField_DeviceID_set)
    BankSecuAccType = property(_thosttraderapi.CThostFtdcReqQueryAccountField_BankSecuAccType_get, _thosttraderapi.CThostFtdcReqQueryAccountField_BankSecuAccType_set)
    BrokerIDByBank = property(_thosttraderapi.CThostFtdcReqQueryAccountField_BrokerIDByBank_get, _thosttraderapi.CThostFtdcReqQueryAccountField_BrokerIDByBank_set)
    BankSecuAcc = property(_thosttraderapi.CThostFtdcReqQueryAccountField_BankSecuAcc_get, _thosttraderapi.CThostFtdcReqQueryAccountField_BankSecuAcc_set)
    BankPwdFlag = property(_thosttraderapi.CThostFtdcReqQueryAccountField_BankPwdFlag_get, _thosttraderapi.CThostFtdcReqQueryAccountField_BankPwdFlag_set)
    SecuPwdFlag = property(_thosttraderapi.CThostFtdcReqQueryAccountField_SecuPwdFlag_get, _thosttraderapi.CThostFtdcReqQueryAccountField_SecuPwdFlag_set)
    OperNo = property(_thosttraderapi.CThostFtdcReqQueryAccountField_OperNo_get, _thosttraderapi.CThostFtdcReqQueryAccountField_OperNo_set)
    RequestID = property(_thosttraderapi.CThostFtdcReqQueryAccountField_RequestID_get, _thosttraderapi.CThostFtdcReqQueryAccountField_RequestID_set)
    TID = property(_thosttraderapi.CThostFtdcReqQueryAccountField_TID_get, _thosttraderapi.CThostFtdcReqQueryAccountField_TID_set)
    LongCustomerName = property(_thosttraderapi.CThostFtdcReqQueryAccountField_LongCustomerName_get, _thosttraderapi.CThostFtdcReqQueryAccountField_LongCustomerName_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcReqQueryAccountField_swiginit(self, _thosttraderapi.new_CThostFtdcReqQueryAccountField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcReqQueryAccountField

# Register CThostFtdcReqQueryAccountField in _thosttraderapi:
_thosttraderapi.CThostFtdcReqQueryAccountField_swigregister(CThostFtdcReqQueryAccountField)

class CThostFtdcRspQueryAccountField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    TradeCode = property(_thosttraderapi.CThostFtdcRspQueryAccountField_TradeCode_get, _thosttraderapi.CThostFtdcRspQueryAccountField_TradeCode_set)
    BankID = property(_thosttraderapi.CThostFtdcRspQueryAccountField_BankID_get, _thosttraderapi.CThostFtdcRspQueryAccountField_BankID_set)
    BankBranchID = property(_thosttraderapi.CThostFtdcRspQueryAccountField_BankBranchID_get, _thosttraderapi.CThostFtdcRspQueryAccountField_BankBranchID_set)
    BrokerID = property(_thosttraderapi.CThostFtdcRspQueryAccountField_BrokerID_get, _thosttraderapi.CThostFtdcRspQueryAccountField_BrokerID_set)
    BrokerBranchID = property(_thosttraderapi.CThostFtdcRspQueryAccountField_BrokerBranchID_get, _thosttraderapi.CThostFtdcRspQueryAccountField_BrokerBranchID_set)
    TradeDate = property(_thosttraderapi.CThostFtdcRspQueryAccountField_TradeDate_get, _thosttraderapi.CThostFtdcRspQueryAccountField_TradeDate_set)
    TradeTime = property(_thosttraderapi.CThostFtdcRspQueryAccountField_TradeTime_get, _thosttraderapi.CThostFtdcRspQueryAccountField_TradeTime_set)
    BankSerial = property(_thosttraderapi.CThostFtdcRspQueryAccountField_BankSerial_get, _thosttraderapi.CThostFtdcRspQueryAccountField_BankSerial_set)
    TradingDay = property(_thosttraderapi.CThostFtdcRspQueryAccountField_TradingDay_get, _thosttraderapi.CThostFtdcRspQueryAccountField_TradingDay_set)
    PlateSerial = property(_thosttraderapi.CThostFtdcRspQueryAccountField_PlateSerial_get, _thosttraderapi.CThostFtdcRspQueryAccountField_PlateSerial_set)
    LastFragment = property(_thosttraderapi.CThostFtdcRspQueryAccountField_LastFragment_get, _thosttraderapi.CThostFtdcRspQueryAccountField_LastFragment_set)
    SessionID = property(_thosttraderapi.CThostFtdcRspQueryAccountField_SessionID_get, _thosttraderapi.CThostFtdcRspQueryAccountField_SessionID_set)
    CustomerName = property(_thosttraderapi.CThostFtdcRspQueryAccountField_CustomerName_get, _thosttraderapi.CThostFtdcRspQueryAccountField_CustomerName_set)
    IdCardType = property(_thosttraderapi.CThostFtdcRspQueryAccountField_IdCardType_get, _thosttraderapi.CThostFtdcRspQueryAccountField_IdCardType_set)
    IdentifiedCardNo = property(_thosttraderapi.CThostFtdcRspQueryAccountField_IdentifiedCardNo_get, _thosttraderapi.CThostFtdcRspQueryAccountField_IdentifiedCardNo_set)
    CustType = property(_thosttraderapi.CThostFtdcRspQueryAccountField_CustType_get, _thosttraderapi.CThostFtdcRspQueryAccountField_CustType_set)
    BankAccount = property(_thosttraderapi.CThostFtdcRspQueryAccountField_BankAccount_get, _thosttraderapi.CThostFtdcRspQueryAccountField_BankAccount_set)
    BankPassWord = property(_thosttraderapi.CThostFtdcRspQueryAccountField_BankPassWord_get, _thosttraderapi.CThostFtdcRspQueryAccountField_BankPassWord_set)
    AccountID = property(_thosttraderapi.CThostFtdcRspQueryAccountField_AccountID_get, _thosttraderapi.CThostFtdcRspQueryAccountField_AccountID_set)
    Password = property(_thosttraderapi.CThostFtdcRspQueryAccountField_Password_get, _thosttraderapi.CThostFtdcRspQueryAccountField_Password_set)
    FutureSerial = property(_thosttraderapi.CThostFtdcRspQueryAccountField_FutureSerial_get, _thosttraderapi.CThostFtdcRspQueryAccountField_FutureSerial_set)
    InstallID = property(_thosttraderapi.CThostFtdcRspQueryAccountField_InstallID_get, _thosttraderapi.CThostFtdcRspQueryAccountField_InstallID_set)
    UserID = property(_thosttraderapi.CThostFtdcRspQueryAccountField_UserID_get, _thosttraderapi.CThostFtdcRspQueryAccountField_UserID_set)
    VerifyCertNoFlag = property(_thosttraderapi.CThostFtdcRspQueryAccountField_VerifyCertNoFlag_get, _thosttraderapi.CThostFtdcRspQueryAccountField_VerifyCertNoFlag_set)
    CurrencyID = property(_thosttraderapi.CThostFtdcRspQueryAccountField_CurrencyID_get, _thosttraderapi.CThostFtdcRspQueryAccountField_CurrencyID_set)
    Digest = property(_thosttraderapi.CThostFtdcRspQueryAccountField_Digest_get, _thosttraderapi.CThostFtdcRspQueryAccountField_Digest_set)
    BankAccType = property(_thosttraderapi.CThostFtdcRspQueryAccountField_BankAccType_get, _thosttraderapi.CThostFtdcRspQueryAccountField_BankAccType_set)
    DeviceID = property(_thosttraderapi.CThostFtdcRspQueryAccountField_DeviceID_get, _thosttraderapi.CThostFtdcRspQueryAccountField_DeviceID_set)
    BankSecuAccType = property(_thosttraderapi.CThostFtdcRspQueryAccountField_BankSecuAccType_get, _thosttraderapi.CThostFtdcRspQueryAccountField_BankSecuAccType_set)
    BrokerIDByBank = property(_thosttraderapi.CThostFtdcRspQueryAccountField_BrokerIDByBank_get, _thosttraderapi.CThostFtdcRspQueryAccountField_BrokerIDByBank_set)
    BankSecuAcc = property(_thosttraderapi.CThostFtdcRspQueryAccountField_BankSecuAcc_get, _thosttraderapi.CThostFtdcRspQueryAccountField_BankSecuAcc_set)
    BankPwdFlag = property(_thosttraderapi.CThostFtdcRspQueryAccountField_BankPwdFlag_get, _thosttraderapi.CThostFtdcRspQueryAccountField_BankPwdFlag_set)
    SecuPwdFlag = property(_thosttraderapi.CThostFtdcRspQueryAccountField_SecuPwdFlag_get, _thosttraderapi.CThostFtdcRspQueryAccountField_SecuPwdFlag_set)
    OperNo = property(_thosttraderapi.CThostFtdcRspQueryAccountField_OperNo_get, _thosttraderapi.CThostFtdcRspQueryAccountField_OperNo_set)
    RequestID = property(_thosttraderapi.CThostFtdcRspQueryAccountField_RequestID_get, _thosttraderapi.CThostFtdcRspQueryAccountField_RequestID_set)
    TID = property(_thosttraderapi.CThostFtdcRspQueryAccountField_TID_get, _thosttraderapi.CThostFtdcRspQueryAccountField_TID_set)
    BankUseAmount = property(_thosttraderapi.CThostFtdcRspQueryAccountField_BankUseAmount_get, _thosttraderapi.CThostFtdcRspQueryAccountField_BankUseAmount_set)
    BankFetchAmount = property(_thosttraderapi.CThostFtdcRspQueryAccountField_BankFetchAmount_get, _thosttraderapi.CThostFtdcRspQueryAccountField_BankFetchAmount_set)
    LongCustomerName = property(_thosttraderapi.CThostFtdcRspQueryAccountField_LongCustomerName_get, _thosttraderapi.CThostFtdcRspQueryAccountField_LongCustomerName_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcRspQueryAccountField_swiginit(self, _thosttraderapi.new_CThostFtdcRspQueryAccountField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcRspQueryAccountField

# Register CThostFtdcRspQueryAccountField in _thosttraderapi:
_thosttraderapi.CThostFtdcRspQueryAccountField_swigregister(CThostFtdcRspQueryAccountField)

class CThostFtdcFutureSignIOField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    TradeCode = property(_thosttraderapi.CThostFtdcFutureSignIOField_TradeCode_get, _thosttraderapi.CThostFtdcFutureSignIOField_TradeCode_set)
    BankID = property(_thosttraderapi.CThostFtdcFutureSignIOField_BankID_get, _thosttraderapi.CThostFtdcFutureSignIOField_BankID_set)
    BankBranchID = property(_thosttraderapi.CThostFtdcFutureSignIOField_BankBranchID_get, _thosttraderapi.CThostFtdcFutureSignIOField_BankBranchID_set)
    BrokerID = property(_thosttraderapi.CThostFtdcFutureSignIOField_BrokerID_get, _thosttraderapi.CThostFtdcFutureSignIOField_BrokerID_set)
    BrokerBranchID = property(_thosttraderapi.CThostFtdcFutureSignIOField_BrokerBranchID_get, _thosttraderapi.CThostFtdcFutureSignIOField_BrokerBranchID_set)
    TradeDate = property(_thosttraderapi.CThostFtdcFutureSignIOField_TradeDate_get, _thosttraderapi.CThostFtdcFutureSignIOField_TradeDate_set)
    TradeTime = property(_thosttraderapi.CThostFtdcFutureSignIOField_TradeTime_get, _thosttraderapi.CThostFtdcFutureSignIOField_TradeTime_set)
    BankSerial = property(_thosttraderapi.CThostFtdcFutureSignIOField_BankSerial_get, _thosttraderapi.CThostFtdcFutureSignIOField_BankSerial_set)
    TradingDay = property(_thosttraderapi.CThostFtdcFutureSignIOField_TradingDay_get, _thosttraderapi.CThostFtdcFutureSignIOField_TradingDay_set)
    PlateSerial = property(_thosttraderapi.CThostFtdcFutureSignIOField_PlateSerial_get, _thosttraderapi.CThostFtdcFutureSignIOField_PlateSerial_set)
    LastFragment = property(_thosttraderapi.CThostFtdcFutureSignIOField_LastFragment_get, _thosttraderapi.CThostFtdcFutureSignIOField_LastFragment_set)
    SessionID = property(_thosttraderapi.CThostFtdcFutureSignIOField_SessionID_get, _thosttraderapi.CThostFtdcFutureSignIOField_SessionID_set)
    InstallID = property(_thosttraderapi.CThostFtdcFutureSignIOField_InstallID_get, _thosttraderapi.CThostFtdcFutureSignIOField_InstallID_set)
    UserID = property(_thosttraderapi.CThostFtdcFutureSignIOField_UserID_get, _thosttraderapi.CThostFtdcFutureSignIOField_UserID_set)
    Digest = property(_thosttraderapi.CThostFtdcFutureSignIOField_Digest_get, _thosttraderapi.CThostFtdcFutureSignIOField_Digest_set)
    CurrencyID = property(_thosttraderapi.CThostFtdcFutureSignIOField_CurrencyID_get, _thosttraderapi.CThostFtdcFutureSignIOField_CurrencyID_set)
    DeviceID = property(_thosttraderapi.CThostFtdcFutureSignIOField_DeviceID_get, _thosttraderapi.CThostFtdcFutureSignIOField_DeviceID_set)
    BrokerIDByBank = property(_thosttraderapi.CThostFtdcFutureSignIOField_BrokerIDByBank_get, _thosttraderapi.CThostFtdcFutureSignIOField_BrokerIDByBank_set)
    OperNo = property(_thosttraderapi.CThostFtdcFutureSignIOField_OperNo_get, _thosttraderapi.CThostFtdcFutureSignIOField_OperNo_set)
    RequestID = property(_thosttraderapi.CThostFtdcFutureSignIOField_RequestID_get, _thosttraderapi.CThostFtdcFutureSignIOField_RequestID_set)
    TID = property(_thosttraderapi.CThostFtdcFutureSignIOField_TID_get, _thosttraderapi.CThostFtdcFutureSignIOField_TID_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcFutureSignIOField_swiginit(self, _thosttraderapi.new_CThostFtdcFutureSignIOField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcFutureSignIOField

# Register CThostFtdcFutureSignIOField in _thosttraderapi:
_thosttraderapi.CThostFtdcFutureSignIOField_swigregister(CThostFtdcFutureSignIOField)

class CThostFtdcRspFutureSignInField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    TradeCode = property(_thosttraderapi.CThostFtdcRspFutureSignInField_TradeCode_get, _thosttraderapi.CThostFtdcRspFutureSignInField_TradeCode_set)
    BankID = property(_thosttraderapi.CThostFtdcRspFutureSignInField_BankID_get, _thosttraderapi.CThostFtdcRspFutureSignInField_BankID_set)
    BankBranchID = property(_thosttraderapi.CThostFtdcRspFutureSignInField_BankBranchID_get, _thosttraderapi.CThostFtdcRspFutureSignInField_BankBranchID_set)
    BrokerID = property(_thosttraderapi.CThostFtdcRspFutureSignInField_BrokerID_get, _thosttraderapi.CThostFtdcRspFutureSignInField_BrokerID_set)
    BrokerBranchID = property(_thosttraderapi.CThostFtdcRspFutureSignInField_BrokerBranchID_get, _thosttraderapi.CThostFtdcRspFutureSignInField_BrokerBranchID_set)
    TradeDate = property(_thosttraderapi.CThostFtdcRspFutureSignInField_TradeDate_get, _thosttraderapi.CThostFtdcRspFutureSignInField_TradeDate_set)
    TradeTime = property(_thosttraderapi.CThostFtdcRspFutureSignInField_TradeTime_get, _thosttraderapi.CThostFtdcRspFutureSignInField_TradeTime_set)
    BankSerial = property(_thosttraderapi.CThostFtdcRspFutureSignInField_BankSerial_get, _thosttraderapi.CThostFtdcRspFutureSignInField_BankSerial_set)
    TradingDay = property(_thosttraderapi.CThostFtdcRspFutureSignInField_TradingDay_get, _thosttraderapi.CThostFtdcRspFutureSignInField_TradingDay_set)
    PlateSerial = property(_thosttraderapi.CThostFtdcRspFutureSignInField_PlateSerial_get, _thosttraderapi.CThostFtdcRspFutureSignInField_PlateSerial_set)
    LastFragment = property(_thosttraderapi.CThostFtdcRspFutureSignInField_LastFragment_get, _thosttraderapi.CThostFtdcRspFutureSignInField_LastFragment_set)
    SessionID = property(_thosttraderapi.CThostFtdcRspFutureSignInField_SessionID_get, _thosttraderapi.CThostFtdcRspFutureSignInField_SessionID_set)
    InstallID = property(_thosttraderapi.CThostFtdcRspFutureSignInField_InstallID_get, _thosttraderapi.CThostFtdcRspFutureSignInField_InstallID_set)
    UserID = property(_thosttraderapi.CThostFtdcRspFutureSignInField_UserID_get, _thosttraderapi.CThostFtdcRspFutureSignInField_UserID_set)
    Digest = property(_thosttraderapi.CThostFtdcRspFutureSignInField_Digest_get, _thosttraderapi.CThostFtdcRspFutureSignInField_Digest_set)
    CurrencyID = property(_thosttraderapi.CThostFtdcRspFutureSignInField_CurrencyID_get, _thosttraderapi.CThostFtdcRspFutureSignInField_CurrencyID_set)
    DeviceID = property(_thosttraderapi.CThostFtdcRspFutureSignInField_DeviceID_get, _thosttraderapi.CThostFtdcRspFutureSignInField_DeviceID_set)
    BrokerIDByBank = property(_thosttraderapi.CThostFtdcRspFutureSignInField_BrokerIDByBank_get, _thosttraderapi.CThostFtdcRspFutureSignInField_BrokerIDByBank_set)
    OperNo = property(_thosttraderapi.CThostFtdcRspFutureSignInField_OperNo_get, _thosttraderapi.CThostFtdcRspFutureSignInField_OperNo_set)
    RequestID = property(_thosttraderapi.CThostFtdcRspFutureSignInField_RequestID_get, _thosttraderapi.CThostFtdcRspFutureSignInField_RequestID_set)
    TID = property(_thosttraderapi.CThostFtdcRspFutureSignInField_TID_get, _thosttraderapi.CThostFtdcRspFutureSignInField_TID_set)
    ErrorID = property(_thosttraderapi.CThostFtdcRspFutureSignInField_ErrorID_get, _thosttraderapi.CThostFtdcRspFutureSignInField_ErrorID_set)
    ErrorMsg = property(_thosttraderapi.CThostFtdcRspFutureSignInField_ErrorMsg_get, _thosttraderapi.CThostFtdcRspFutureSignInField_ErrorMsg_set)
    PinKey = property(_thosttraderapi.CThostFtdcRspFutureSignInField_PinKey_get, _thosttraderapi.CThostFtdcRspFutureSignInField_PinKey_set)
    MacKey = property(_thosttraderapi.CThostFtdcRspFutureSignInField_MacKey_get, _thosttraderapi.CThostFtdcRspFutureSignInField_MacKey_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcRspFutureSignInField_swiginit(self, _thosttraderapi.new_CThostFtdcRspFutureSignInField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcRspFutureSignInField

# Register CThostFtdcRspFutureSignInField in _thosttraderapi:
_thosttraderapi.CThostFtdcRspFutureSignInField_swigregister(CThostFtdcRspFutureSignInField)

class CThostFtdcReqFutureSignOutField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    TradeCode = property(_thosttraderapi.CThostFtdcReqFutureSignOutField_TradeCode_get, _thosttraderapi.CThostFtdcReqFutureSignOutField_TradeCode_set)
    BankID = property(_thosttraderapi.CThostFtdcReqFutureSignOutField_BankID_get, _thosttraderapi.CThostFtdcReqFutureSignOutField_BankID_set)
    BankBranchID = property(_thosttraderapi.CThostFtdcReqFutureSignOutField_BankBranchID_get, _thosttraderapi.CThostFtdcReqFutureSignOutField_BankBranchID_set)
    BrokerID = property(_thosttraderapi.CThostFtdcReqFutureSignOutField_BrokerID_get, _thosttraderapi.CThostFtdcReqFutureSignOutField_BrokerID_set)
    BrokerBranchID = property(_thosttraderapi.CThostFtdcReqFutureSignOutField_BrokerBranchID_get, _thosttraderapi.CThostFtdcReqFutureSignOutField_BrokerBranchID_set)
    TradeDate = property(_thosttraderapi.CThostFtdcReqFutureSignOutField_TradeDate_get, _thosttraderapi.CThostFtdcReqFutureSignOutField_TradeDate_set)
    TradeTime = property(_thosttraderapi.CThostFtdcReqFutureSignOutField_TradeTime_get, _thosttraderapi.CThostFtdcReqFutureSignOutField_TradeTime_set)
    BankSerial = property(_thosttraderapi.CThostFtdcReqFutureSignOutField_BankSerial_get, _thosttraderapi.CThostFtdcReqFutureSignOutField_BankSerial_set)
    TradingDay = property(_thosttraderapi.CThostFtdcReqFutureSignOutField_TradingDay_get, _thosttraderapi.CThostFtdcReqFutureSignOutField_TradingDay_set)
    PlateSerial = property(_thosttraderapi.CThostFtdcReqFutureSignOutField_PlateSerial_get, _thosttraderapi.CThostFtdcReqFutureSignOutField_PlateSerial_set)
    LastFragment = property(_thosttraderapi.CThostFtdcReqFutureSignOutField_LastFragment_get, _thosttraderapi.CThostFtdcReqFutureSignOutField_LastFragment_set)
    SessionID = property(_thosttraderapi.CThostFtdcReqFutureSignOutField_SessionID_get, _thosttraderapi.CThostFtdcReqFutureSignOutField_SessionID_set)
    InstallID = property(_thosttraderapi.CThostFtdcReqFutureSignOutField_InstallID_get, _thosttraderapi.CThostFtdcReqFutureSignOutField_InstallID_set)
    UserID = property(_thosttraderapi.CThostFtdcReqFutureSignOutField_UserID_get, _thosttraderapi.CThostFtdcReqFutureSignOutField_UserID_set)
    Digest = property(_thosttraderapi.CThostFtdcReqFutureSignOutField_Digest_get, _thosttraderapi.CThostFtdcReqFutureSignOutField_Digest_set)
    CurrencyID = property(_thosttraderapi.CThostFtdcReqFutureSignOutField_CurrencyID_get, _thosttraderapi.CThostFtdcReqFutureSignOutField_CurrencyID_set)
    DeviceID = property(_thosttraderapi.CThostFtdcReqFutureSignOutField_DeviceID_get, _thosttraderapi.CThostFtdcReqFutureSignOutField_DeviceID_set)
    BrokerIDByBank = property(_thosttraderapi.CThostFtdcReqFutureSignOutField_BrokerIDByBank_get, _thosttraderapi.CThostFtdcReqFutureSignOutField_BrokerIDByBank_set)
    OperNo = property(_thosttraderapi.CThostFtdcReqFutureSignOutField_OperNo_get, _thosttraderapi.CThostFtdcReqFutureSignOutField_OperNo_set)
    RequestID = property(_thosttraderapi.CThostFtdcReqFutureSignOutField_RequestID_get, _thosttraderapi.CThostFtdcReqFutureSignOutField_RequestID_set)
    TID = property(_thosttraderapi.CThostFtdcReqFutureSignOutField_TID_get, _thosttraderapi.CThostFtdcReqFutureSignOutField_TID_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcReqFutureSignOutField_swiginit(self, _thosttraderapi.new_CThostFtdcReqFutureSignOutField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcReqFutureSignOutField

# Register CThostFtdcReqFutureSignOutField in _thosttraderapi:
_thosttraderapi.CThostFtdcReqFutureSignOutField_swigregister(CThostFtdcReqFutureSignOutField)

class CThostFtdcRspFutureSignOutField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    TradeCode = property(_thosttraderapi.CThostFtdcRspFutureSignOutField_TradeCode_get, _thosttraderapi.CThostFtdcRspFutureSignOutField_TradeCode_set)
    BankID = property(_thosttraderapi.CThostFtdcRspFutureSignOutField_BankID_get, _thosttraderapi.CThostFtdcRspFutureSignOutField_BankID_set)
    BankBranchID = property(_thosttraderapi.CThostFtdcRspFutureSignOutField_BankBranchID_get, _thosttraderapi.CThostFtdcRspFutureSignOutField_BankBranchID_set)
    BrokerID = property(_thosttraderapi.CThostFtdcRspFutureSignOutField_BrokerID_get, _thosttraderapi.CThostFtdcRspFutureSignOutField_BrokerID_set)
    BrokerBranchID = property(_thosttraderapi.CThostFtdcRspFutureSignOutField_BrokerBranchID_get, _thosttraderapi.CThostFtdcRspFutureSignOutField_BrokerBranchID_set)
    TradeDate = property(_thosttraderapi.CThostFtdcRspFutureSignOutField_TradeDate_get, _thosttraderapi.CThostFtdcRspFutureSignOutField_TradeDate_set)
    TradeTime = property(_thosttraderapi.CThostFtdcRspFutureSignOutField_TradeTime_get, _thosttraderapi.CThostFtdcRspFutureSignOutField_TradeTime_set)
    BankSerial = property(_thosttraderapi.CThostFtdcRspFutureSignOutField_BankSerial_get, _thosttraderapi.CThostFtdcRspFutureSignOutField_BankSerial_set)
    TradingDay = property(_thosttraderapi.CThostFtdcRspFutureSignOutField_TradingDay_get, _thosttraderapi.CThostFtdcRspFutureSignOutField_TradingDay_set)
    PlateSerial = property(_thosttraderapi.CThostFtdcRspFutureSignOutField_PlateSerial_get, _thosttraderapi.CThostFtdcRspFutureSignOutField_PlateSerial_set)
    LastFragment = property(_thosttraderapi.CThostFtdcRspFutureSignOutField_LastFragment_get, _thosttraderapi.CThostFtdcRspFutureSignOutField_LastFragment_set)
    SessionID = property(_thosttraderapi.CThostFtdcRspFutureSignOutField_SessionID_get, _thosttraderapi.CThostFtdcRspFutureSignOutField_SessionID_set)
    InstallID = property(_thosttraderapi.CThostFtdcRspFutureSignOutField_InstallID_get, _thosttraderapi.CThostFtdcRspFutureSignOutField_InstallID_set)
    UserID = property(_thosttraderapi.CThostFtdcRspFutureSignOutField_UserID_get, _thosttraderapi.CThostFtdcRspFutureSignOutField_UserID_set)
    Digest = property(_thosttraderapi.CThostFtdcRspFutureSignOutField_Digest_get, _thosttraderapi.CThostFtdcRspFutureSignOutField_Digest_set)
    CurrencyID = property(_thosttraderapi.CThostFtdcRspFutureSignOutField_CurrencyID_get, _thosttraderapi.CThostFtdcRspFutureSignOutField_CurrencyID_set)
    DeviceID = property(_thosttraderapi.CThostFtdcRspFutureSignOutField_DeviceID_get, _thosttraderapi.CThostFtdcRspFutureSignOutField_DeviceID_set)
    BrokerIDByBank = property(_thosttraderapi.CThostFtdcRspFutureSignOutField_BrokerIDByBank_get, _thosttraderapi.CThostFtdcRspFutureSignOutField_BrokerIDByBank_set)
    OperNo = property(_thosttraderapi.CThostFtdcRspFutureSignOutField_OperNo_get, _thosttraderapi.CThostFtdcRspFutureSignOutField_OperNo_set)
    RequestID = property(_thosttraderapi.CThostFtdcRspFutureSignOutField_RequestID_get, _thosttraderapi.CThostFtdcRspFutureSignOutField_RequestID_set)
    TID = property(_thosttraderapi.CThostFtdcRspFutureSignOutField_TID_get, _thosttraderapi.CThostFtdcRspFutureSignOutField_TID_set)
    ErrorID = property(_thosttraderapi.CThostFtdcRspFutureSignOutField_ErrorID_get, _thosttraderapi.CThostFtdcRspFutureSignOutField_ErrorID_set)
    ErrorMsg = property(_thosttraderapi.CThostFtdcRspFutureSignOutField_ErrorMsg_get, _thosttraderapi.CThostFtdcRspFutureSignOutField_ErrorMsg_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcRspFutureSignOutField_swiginit(self, _thosttraderapi.new_CThostFtdcRspFutureSignOutField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcRspFutureSignOutField

# Register CThostFtdcRspFutureSignOutField in _thosttraderapi:
_thosttraderapi.CThostFtdcRspFutureSignOutField_swigregister(CThostFtdcRspFutureSignOutField)

class CThostFtdcReqQueryTradeResultBySerialField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    TradeCode = property(_thosttraderapi.CThostFtdcReqQueryTradeResultBySerialField_TradeCode_get, _thosttraderapi.CThostFtdcReqQueryTradeResultBySerialField_TradeCode_set)
    BankID = property(_thosttraderapi.CThostFtdcReqQueryTradeResultBySerialField_BankID_get, _thosttraderapi.CThostFtdcReqQueryTradeResultBySerialField_BankID_set)
    BankBranchID = property(_thosttraderapi.CThostFtdcReqQueryTradeResultBySerialField_BankBranchID_get, _thosttraderapi.CThostFtdcReqQueryTradeResultBySerialField_BankBranchID_set)
    BrokerID = property(_thosttraderapi.CThostFtdcReqQueryTradeResultBySerialField_BrokerID_get, _thosttraderapi.CThostFtdcReqQueryTradeResultBySerialField_BrokerID_set)
    BrokerBranchID = property(_thosttraderapi.CThostFtdcReqQueryTradeResultBySerialField_BrokerBranchID_get, _thosttraderapi.CThostFtdcReqQueryTradeResultBySerialField_BrokerBranchID_set)
    TradeDate = property(_thosttraderapi.CThostFtdcReqQueryTradeResultBySerialField_TradeDate_get, _thosttraderapi.CThostFtdcReqQueryTradeResultBySerialField_TradeDate_set)
    TradeTime = property(_thosttraderapi.CThostFtdcReqQueryTradeResultBySerialField_TradeTime_get, _thosttraderapi.CThostFtdcReqQueryTradeResultBySerialField_TradeTime_set)
    BankSerial = property(_thosttraderapi.CThostFtdcReqQueryTradeResultBySerialField_BankSerial_get, _thosttraderapi.CThostFtdcReqQueryTradeResultBySerialField_BankSerial_set)
    TradingDay = property(_thosttraderapi.CThostFtdcReqQueryTradeResultBySerialField_TradingDay_get, _thosttraderapi.CThostFtdcReqQueryTradeResultBySerialField_TradingDay_set)
    PlateSerial = property(_thosttraderapi.CThostFtdcReqQueryTradeResultBySerialField_PlateSerial_get, _thosttraderapi.CThostFtdcReqQueryTradeResultBySerialField_PlateSerial_set)
    LastFragment = property(_thosttraderapi.CThostFtdcReqQueryTradeResultBySerialField_LastFragment_get, _thosttraderapi.CThostFtdcReqQueryTradeResultBySerialField_LastFragment_set)
    SessionID = property(_thosttraderapi.CThostFtdcReqQueryTradeResultBySerialField_SessionID_get, _thosttraderapi.CThostFtdcReqQueryTradeResultBySerialField_SessionID_set)
    Reference = property(_thosttraderapi.CThostFtdcReqQueryTradeResultBySerialField_Reference_get, _thosttraderapi.CThostFtdcReqQueryTradeResultBySerialField_Reference_set)
    RefrenceIssureType = property(_thosttraderapi.CThostFtdcReqQueryTradeResultBySerialField_RefrenceIssureType_get, _thosttraderapi.CThostFtdcReqQueryTradeResultBySerialField_RefrenceIssureType_set)
    RefrenceIssure = property(_thosttraderapi.CThostFtdcReqQueryTradeResultBySerialField_RefrenceIssure_get, _thosttraderapi.CThostFtdcReqQueryTradeResultBySerialField_RefrenceIssure_set)
    CustomerName = property(_thosttraderapi.CThostFtdcReqQueryTradeResultBySerialField_CustomerName_get, _thosttraderapi.CThostFtdcReqQueryTradeResultBySerialField_CustomerName_set)
    IdCardType = property(_thosttraderapi.CThostFtdcReqQueryTradeResultBySerialField_IdCardType_get, _thosttraderapi.CThostFtdcReqQueryTradeResultBySerialField_IdCardType_set)
    IdentifiedCardNo = property(_thosttraderapi.CThostFtdcReqQueryTradeResultBySerialField_IdentifiedCardNo_get, _thosttraderapi.CThostFtdcReqQueryTradeResultBySerialField_IdentifiedCardNo_set)
    CustType = property(_thosttraderapi.CThostFtdcReqQueryTradeResultBySerialField_CustType_get, _thosttraderapi.CThostFtdcReqQueryTradeResultBySerialField_CustType_set)
    BankAccount = property(_thosttraderapi.CThostFtdcReqQueryTradeResultBySerialField_BankAccount_get, _thosttraderapi.CThostFtdcReqQueryTradeResultBySerialField_BankAccount_set)
    BankPassWord = property(_thosttraderapi.CThostFtdcReqQueryTradeResultBySerialField_BankPassWord_get, _thosttraderapi.CThostFtdcReqQueryTradeResultBySerialField_BankPassWord_set)
    AccountID = property(_thosttraderapi.CThostFtdcReqQueryTradeResultBySerialField_AccountID_get, _thosttraderapi.CThostFtdcReqQueryTradeResultBySerialField_AccountID_set)
    Password = property(_thosttraderapi.CThostFtdcReqQueryTradeResultBySerialField_Password_get, _thosttraderapi.CThostFtdcReqQueryTradeResultBySerialField_Password_set)
    CurrencyID = property(_thosttraderapi.CThostFtdcReqQueryTradeResultBySerialField_CurrencyID_get, _thosttraderapi.CThostFtdcReqQueryTradeResultBySerialField_CurrencyID_set)
    TradeAmount = property(_thosttraderapi.CThostFtdcReqQueryTradeResultBySerialField_TradeAmount_get, _thosttraderapi.CThostFtdcReqQueryTradeResultBySerialField_TradeAmount_set)
    Digest = property(_thosttraderapi.CThostFtdcReqQueryTradeResultBySerialField_Digest_get, _thosttraderapi.CThostFtdcReqQueryTradeResultBySerialField_Digest_set)
    LongCustomerName = property(_thosttraderapi.CThostFtdcReqQueryTradeResultBySerialField_LongCustomerName_get, _thosttraderapi.CThostFtdcReqQueryTradeResultBySerialField_LongCustomerName_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcReqQueryTradeResultBySerialField_swiginit(self, _thosttraderapi.new_CThostFtdcReqQueryTradeResultBySerialField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcReqQueryTradeResultBySerialField

# Register CThostFtdcReqQueryTradeResultBySerialField in _thosttraderapi:
_thosttraderapi.CThostFtdcReqQueryTradeResultBySerialField_swigregister(CThostFtdcReqQueryTradeResultBySerialField)

class CThostFtdcRspQueryTradeResultBySerialField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    TradeCode = property(_thosttraderapi.CThostFtdcRspQueryTradeResultBySerialField_TradeCode_get, _thosttraderapi.CThostFtdcRspQueryTradeResultBySerialField_TradeCode_set)
    BankID = property(_thosttraderapi.CThostFtdcRspQueryTradeResultBySerialField_BankID_get, _thosttraderapi.CThostFtdcRspQueryTradeResultBySerialField_BankID_set)
    BankBranchID = property(_thosttraderapi.CThostFtdcRspQueryTradeResultBySerialField_BankBranchID_get, _thosttraderapi.CThostFtdcRspQueryTradeResultBySerialField_BankBranchID_set)
    BrokerID = property(_thosttraderapi.CThostFtdcRspQueryTradeResultBySerialField_BrokerID_get, _thosttraderapi.CThostFtdcRspQueryTradeResultBySerialField_BrokerID_set)
    BrokerBranchID = property(_thosttraderapi.CThostFtdcRspQueryTradeResultBySerialField_BrokerBranchID_get, _thosttraderapi.CThostFtdcRspQueryTradeResultBySerialField_BrokerBranchID_set)
    TradeDate = property(_thosttraderapi.CThostFtdcRspQueryTradeResultBySerialField_TradeDate_get, _thosttraderapi.CThostFtdcRspQueryTradeResultBySerialField_TradeDate_set)
    TradeTime = property(_thosttraderapi.CThostFtdcRspQueryTradeResultBySerialField_TradeTime_get, _thosttraderapi.CThostFtdcRspQueryTradeResultBySerialField_TradeTime_set)
    BankSerial = property(_thosttraderapi.CThostFtdcRspQueryTradeResultBySerialField_BankSerial_get, _thosttraderapi.CThostFtdcRspQueryTradeResultBySerialField_BankSerial_set)
    TradingDay = property(_thosttraderapi.CThostFtdcRspQueryTradeResultBySerialField_TradingDay_get, _thosttraderapi.CThostFtdcRspQueryTradeResultBySerialField_TradingDay_set)
    PlateSerial = property(_thosttraderapi.CThostFtdcRspQueryTradeResultBySerialField_PlateSerial_get, _thosttraderapi.CThostFtdcRspQueryTradeResultBySerialField_PlateSerial_set)
    LastFragment = property(_thosttraderapi.CThostFtdcRspQueryTradeResultBySerialField_LastFragment_get, _thosttraderapi.CThostFtdcRspQueryTradeResultBySerialField_LastFragment_set)
    SessionID = property(_thosttraderapi.CThostFtdcRspQueryTradeResultBySerialField_SessionID_get, _thosttraderapi.CThostFtdcRspQueryTradeResultBySerialField_SessionID_set)
    ErrorID = property(_thosttraderapi.CThostFtdcRspQueryTradeResultBySerialField_ErrorID_get, _thosttraderapi.CThostFtdcRspQueryTradeResultBySerialField_ErrorID_set)
    ErrorMsg = property(_thosttraderapi.CThostFtdcRspQueryTradeResultBySerialField_ErrorMsg_get, _thosttraderapi.CThostFtdcRspQueryTradeResultBySerialField_ErrorMsg_set)
    Reference = property(_thosttraderapi.CThostFtdcRspQueryTradeResultBySerialField_Reference_get, _thosttraderapi.CThostFtdcRspQueryTradeResultBySerialField_Reference_set)
    RefrenceIssureType = property(_thosttraderapi.CThostFtdcRspQueryTradeResultBySerialField_RefrenceIssureType_get, _thosttraderapi.CThostFtdcRspQueryTradeResultBySerialField_RefrenceIssureType_set)
    RefrenceIssure = property(_thosttraderapi.CThostFtdcRspQueryTradeResultBySerialField_RefrenceIssure_get, _thosttraderapi.CThostFtdcRspQueryTradeResultBySerialField_RefrenceIssure_set)
    OriginReturnCode = property(_thosttraderapi.CThostFtdcRspQueryTradeResultBySerialField_OriginReturnCode_get, _thosttraderapi.CThostFtdcRspQueryTradeResultBySerialField_OriginReturnCode_set)
    OriginDescrInfoForReturnCode = property(_thosttraderapi.CThostFtdcRspQueryTradeResultBySerialField_OriginDescrInfoForReturnCode_get, _thosttraderapi.CThostFtdcRspQueryTradeResultBySerialField_OriginDescrInfoForReturnCode_set)
    BankAccount = property(_thosttraderapi.CThostFtdcRspQueryTradeResultBySerialField_BankAccount_get, _thosttraderapi.CThostFtdcRspQueryTradeResultBySerialField_BankAccount_set)
    BankPassWord = property(_thosttraderapi.CThostFtdcRspQueryTradeResultBySerialField_BankPassWord_get, _thosttraderapi.CThostFtdcRspQueryTradeResultBySerialField_BankPassWord_set)
    AccountID = property(_thosttraderapi.CThostFtdcRspQueryTradeResultBySerialField_AccountID_get, _thosttraderapi.CThostFtdcRspQueryTradeResultBySerialField_AccountID_set)
    Password = property(_thosttraderapi.CThostFtdcRspQueryTradeResultBySerialField_Password_get, _thosttraderapi.CThostFtdcRspQueryTradeResultBySerialField_Password_set)
    CurrencyID = property(_thosttraderapi.CThostFtdcRspQueryTradeResultBySerialField_CurrencyID_get, _thosttraderapi.CThostFtdcRspQueryTradeResultBySerialField_CurrencyID_set)
    TradeAmount = property(_thosttraderapi.CThostFtdcRspQueryTradeResultBySerialField_TradeAmount_get, _thosttraderapi.CThostFtdcRspQueryTradeResultBySerialField_TradeAmount_set)
    Digest = property(_thosttraderapi.CThostFtdcRspQueryTradeResultBySerialField_Digest_get, _thosttraderapi.CThostFtdcRspQueryTradeResultBySerialField_Digest_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcRspQueryTradeResultBySerialField_swiginit(self, _thosttraderapi.new_CThostFtdcRspQueryTradeResultBySerialField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcRspQueryTradeResultBySerialField

# Register CThostFtdcRspQueryTradeResultBySerialField in _thosttraderapi:
_thosttraderapi.CThostFtdcRspQueryTradeResultBySerialField_swigregister(CThostFtdcRspQueryTradeResultBySerialField)

class CThostFtdcReqDayEndFileReadyField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    TradeCode = property(_thosttraderapi.CThostFtdcReqDayEndFileReadyField_TradeCode_get, _thosttraderapi.CThostFtdcReqDayEndFileReadyField_TradeCode_set)
    BankID = property(_thosttraderapi.CThostFtdcReqDayEndFileReadyField_BankID_get, _thosttraderapi.CThostFtdcReqDayEndFileReadyField_BankID_set)
    BankBranchID = property(_thosttraderapi.CThostFtdcReqDayEndFileReadyField_BankBranchID_get, _thosttraderapi.CThostFtdcReqDayEndFileReadyField_BankBranchID_set)
    BrokerID = property(_thosttraderapi.CThostFtdcReqDayEndFileReadyField_BrokerID_get, _thosttraderapi.CThostFtdcReqDayEndFileReadyField_BrokerID_set)
    BrokerBranchID = property(_thosttraderapi.CThostFtdcReqDayEndFileReadyField_BrokerBranchID_get, _thosttraderapi.CThostFtdcReqDayEndFileReadyField_BrokerBranchID_set)
    TradeDate = property(_thosttraderapi.CThostFtdcReqDayEndFileReadyField_TradeDate_get, _thosttraderapi.CThostFtdcReqDayEndFileReadyField_TradeDate_set)
    TradeTime = property(_thosttraderapi.CThostFtdcReqDayEndFileReadyField_TradeTime_get, _thosttraderapi.CThostFtdcReqDayEndFileReadyField_TradeTime_set)
    BankSerial = property(_thosttraderapi.CThostFtdcReqDayEndFileReadyField_BankSerial_get, _thosttraderapi.CThostFtdcReqDayEndFileReadyField_BankSerial_set)
    TradingDay = property(_thosttraderapi.CThostFtdcReqDayEndFileReadyField_TradingDay_get, _thosttraderapi.CThostFtdcReqDayEndFileReadyField_TradingDay_set)
    PlateSerial = property(_thosttraderapi.CThostFtdcReqDayEndFileReadyField_PlateSerial_get, _thosttraderapi.CThostFtdcReqDayEndFileReadyField_PlateSerial_set)
    LastFragment = property(_thosttraderapi.CThostFtdcReqDayEndFileReadyField_LastFragment_get, _thosttraderapi.CThostFtdcReqDayEndFileReadyField_LastFragment_set)
    SessionID = property(_thosttraderapi.CThostFtdcReqDayEndFileReadyField_SessionID_get, _thosttraderapi.CThostFtdcReqDayEndFileReadyField_SessionID_set)
    FileBusinessCode = property(_thosttraderapi.CThostFtdcReqDayEndFileReadyField_FileBusinessCode_get, _thosttraderapi.CThostFtdcReqDayEndFileReadyField_FileBusinessCode_set)
    Digest = property(_thosttraderapi.CThostFtdcReqDayEndFileReadyField_Digest_get, _thosttraderapi.CThostFtdcReqDayEndFileReadyField_Digest_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcReqDayEndFileReadyField_swiginit(self, _thosttraderapi.new_CThostFtdcReqDayEndFileReadyField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcReqDayEndFileReadyField

# Register CThostFtdcReqDayEndFileReadyField in _thosttraderapi:
_thosttraderapi.CThostFtdcReqDayEndFileReadyField_swigregister(CThostFtdcReqDayEndFileReadyField)

class CThostFtdcReturnResultField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    ReturnCode = property(_thosttraderapi.CThostFtdcReturnResultField_ReturnCode_get, _thosttraderapi.CThostFtdcReturnResultField_ReturnCode_set)
    DescrInfoForReturnCode = property(_thosttraderapi.CThostFtdcReturnResultField_DescrInfoForReturnCode_get, _thosttraderapi.CThostFtdcReturnResultField_DescrInfoForReturnCode_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcReturnResultField_swiginit(self, _thosttraderapi.new_CThostFtdcReturnResultField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcReturnResultField

# Register CThostFtdcReturnResultField in _thosttraderapi:
_thosttraderapi.CThostFtdcReturnResultField_swigregister(CThostFtdcReturnResultField)

class CThostFtdcVerifyFuturePasswordField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    TradeCode = property(_thosttraderapi.CThostFtdcVerifyFuturePasswordField_TradeCode_get, _thosttraderapi.CThostFtdcVerifyFuturePasswordField_TradeCode_set)
    BankID = property(_thosttraderapi.CThostFtdcVerifyFuturePasswordField_BankID_get, _thosttraderapi.CThostFtdcVerifyFuturePasswordField_BankID_set)
    BankBranchID = property(_thosttraderapi.CThostFtdcVerifyFuturePasswordField_BankBranchID_get, _thosttraderapi.CThostFtdcVerifyFuturePasswordField_BankBranchID_set)
    BrokerID = property(_thosttraderapi.CThostFtdcVerifyFuturePasswordField_BrokerID_get, _thosttraderapi.CThostFtdcVerifyFuturePasswordField_BrokerID_set)
    BrokerBranchID = property(_thosttraderapi.CThostFtdcVerifyFuturePasswordField_BrokerBranchID_get, _thosttraderapi.CThostFtdcVerifyFuturePasswordField_BrokerBranchID_set)
    TradeDate = property(_thosttraderapi.CThostFtdcVerifyFuturePasswordField_TradeDate_get, _thosttraderapi.CThostFtdcVerifyFuturePasswordField_TradeDate_set)
    TradeTime = property(_thosttraderapi.CThostFtdcVerifyFuturePasswordField_TradeTime_get, _thosttraderapi.CThostFtdcVerifyFuturePasswordField_TradeTime_set)
    BankSerial = property(_thosttraderapi.CThostFtdcVerifyFuturePasswordField_BankSerial_get, _thosttraderapi.CThostFtdcVerifyFuturePasswordField_BankSerial_set)
    TradingDay = property(_thosttraderapi.CThostFtdcVerifyFuturePasswordField_TradingDay_get, _thosttraderapi.CThostFtdcVerifyFuturePasswordField_TradingDay_set)
    PlateSerial = property(_thosttraderapi.CThostFtdcVerifyFuturePasswordField_PlateSerial_get, _thosttraderapi.CThostFtdcVerifyFuturePasswordField_PlateSerial_set)
    LastFragment = property(_thosttraderapi.CThostFtdcVerifyFuturePasswordField_LastFragment_get, _thosttraderapi.CThostFtdcVerifyFuturePasswordField_LastFragment_set)
    SessionID = property(_thosttraderapi.CThostFtdcVerifyFuturePasswordField_SessionID_get, _thosttraderapi.CThostFtdcVerifyFuturePasswordField_SessionID_set)
    AccountID = property(_thosttraderapi.CThostFtdcVerifyFuturePasswordField_AccountID_get, _thosttraderapi.CThostFtdcVerifyFuturePasswordField_AccountID_set)
    Password = property(_thosttraderapi.CThostFtdcVerifyFuturePasswordField_Password_get, _thosttraderapi.CThostFtdcVerifyFuturePasswordField_Password_set)
    BankAccount = property(_thosttraderapi.CThostFtdcVerifyFuturePasswordField_BankAccount_get, _thosttraderapi.CThostFtdcVerifyFuturePasswordField_BankAccount_set)
    BankPassWord = property(_thosttraderapi.CThostFtdcVerifyFuturePasswordField_BankPassWord_get, _thosttraderapi.CThostFtdcVerifyFuturePasswordField_BankPassWord_set)
    InstallID = property(_thosttraderapi.CThostFtdcVerifyFuturePasswordField_InstallID_get, _thosttraderapi.CThostFtdcVerifyFuturePasswordField_InstallID_set)
    TID = property(_thosttraderapi.CThostFtdcVerifyFuturePasswordField_TID_get, _thosttraderapi.CThostFtdcVerifyFuturePasswordField_TID_set)
    CurrencyID = property(_thosttraderapi.CThostFtdcVerifyFuturePasswordField_CurrencyID_get, _thosttraderapi.CThostFtdcVerifyFuturePasswordField_CurrencyID_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcVerifyFuturePasswordField_swiginit(self, _thosttraderapi.new_CThostFtdcVerifyFuturePasswordField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcVerifyFuturePasswordField

# Register CThostFtdcVerifyFuturePasswordField in _thosttraderapi:
_thosttraderapi.CThostFtdcVerifyFuturePasswordField_swigregister(CThostFtdcVerifyFuturePasswordField)

class CThostFtdcVerifyCustInfoField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    CustomerName = property(_thosttraderapi.CThostFtdcVerifyCustInfoField_CustomerName_get, _thosttraderapi.CThostFtdcVerifyCustInfoField_CustomerName_set)
    IdCardType = property(_thosttraderapi.CThostFtdcVerifyCustInfoField_IdCardType_get, _thosttraderapi.CThostFtdcVerifyCustInfoField_IdCardType_set)
    IdentifiedCardNo = property(_thosttraderapi.CThostFtdcVerifyCustInfoField_IdentifiedCardNo_get, _thosttraderapi.CThostFtdcVerifyCustInfoField_IdentifiedCardNo_set)
    CustType = property(_thosttraderapi.CThostFtdcVerifyCustInfoField_CustType_get, _thosttraderapi.CThostFtdcVerifyCustInfoField_CustType_set)
    LongCustomerName = property(_thosttraderapi.CThostFtdcVerifyCustInfoField_LongCustomerName_get, _thosttraderapi.CThostFtdcVerifyCustInfoField_LongCustomerName_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcVerifyCustInfoField_swiginit(self, _thosttraderapi.new_CThostFtdcVerifyCustInfoField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcVerifyCustInfoField

# Register CThostFtdcVerifyCustInfoField in _thosttraderapi:
_thosttraderapi.CThostFtdcVerifyCustInfoField_swigregister(CThostFtdcVerifyCustInfoField)

class CThostFtdcVerifyFuturePasswordAndCustInfoField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    CustomerName = property(_thosttraderapi.CThostFtdcVerifyFuturePasswordAndCustInfoField_CustomerName_get, _thosttraderapi.CThostFtdcVerifyFuturePasswordAndCustInfoField_CustomerName_set)
    IdCardType = property(_thosttraderapi.CThostFtdcVerifyFuturePasswordAndCustInfoField_IdCardType_get, _thosttraderapi.CThostFtdcVerifyFuturePasswordAndCustInfoField_IdCardType_set)
    IdentifiedCardNo = property(_thosttraderapi.CThostFtdcVerifyFuturePasswordAndCustInfoField_IdentifiedCardNo_get, _thosttraderapi.CThostFtdcVerifyFuturePasswordAndCustInfoField_IdentifiedCardNo_set)
    CustType = property(_thosttraderapi.CThostFtdcVerifyFuturePasswordAndCustInfoField_CustType_get, _thosttraderapi.CThostFtdcVerifyFuturePasswordAndCustInfoField_CustType_set)
    AccountID = property(_thosttraderapi.CThostFtdcVerifyFuturePasswordAndCustInfoField_AccountID_get, _thosttraderapi.CThostFtdcVerifyFuturePasswordAndCustInfoField_AccountID_set)
    Password = property(_thosttraderapi.CThostFtdcVerifyFuturePasswordAndCustInfoField_Password_get, _thosttraderapi.CThostFtdcVerifyFuturePasswordAndCustInfoField_Password_set)
    CurrencyID = property(_thosttraderapi.CThostFtdcVerifyFuturePasswordAndCustInfoField_CurrencyID_get, _thosttraderapi.CThostFtdcVerifyFuturePasswordAndCustInfoField_CurrencyID_set)
    LongCustomerName = property(_thosttraderapi.CThostFtdcVerifyFuturePasswordAndCustInfoField_LongCustomerName_get, _thosttraderapi.CThostFtdcVerifyFuturePasswordAndCustInfoField_LongCustomerName_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcVerifyFuturePasswordAndCustInfoField_swiginit(self, _thosttraderapi.new_CThostFtdcVerifyFuturePasswordAndCustInfoField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcVerifyFuturePasswordAndCustInfoField

# Register CThostFtdcVerifyFuturePasswordAndCustInfoField in _thosttraderapi:
_thosttraderapi.CThostFtdcVerifyFuturePasswordAndCustInfoField_swigregister(CThostFtdcVerifyFuturePasswordAndCustInfoField)

class CThostFtdcDepositResultInformField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    DepositSeqNo = property(_thosttraderapi.CThostFtdcDepositResultInformField_DepositSeqNo_get, _thosttraderapi.CThostFtdcDepositResultInformField_DepositSeqNo_set)
    BrokerID = property(_thosttraderapi.CThostFtdcDepositResultInformField_BrokerID_get, _thosttraderapi.CThostFtdcDepositResultInformField_BrokerID_set)
    InvestorID = property(_thosttraderapi.CThostFtdcDepositResultInformField_InvestorID_get, _thosttraderapi.CThostFtdcDepositResultInformField_InvestorID_set)
    Deposit = property(_thosttraderapi.CThostFtdcDepositResultInformField_Deposit_get, _thosttraderapi.CThostFtdcDepositResultInformField_Deposit_set)
    RequestID = property(_thosttraderapi.CThostFtdcDepositResultInformField_RequestID_get, _thosttraderapi.CThostFtdcDepositResultInformField_RequestID_set)
    ReturnCode = property(_thosttraderapi.CThostFtdcDepositResultInformField_ReturnCode_get, _thosttraderapi.CThostFtdcDepositResultInformField_ReturnCode_set)
    DescrInfoForReturnCode = property(_thosttraderapi.CThostFtdcDepositResultInformField_DescrInfoForReturnCode_get, _thosttraderapi.CThostFtdcDepositResultInformField_DescrInfoForReturnCode_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcDepositResultInformField_swiginit(self, _thosttraderapi.new_CThostFtdcDepositResultInformField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcDepositResultInformField

# Register CThostFtdcDepositResultInformField in _thosttraderapi:
_thosttraderapi.CThostFtdcDepositResultInformField_swigregister(CThostFtdcDepositResultInformField)

class CThostFtdcReqSyncKeyField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    TradeCode = property(_thosttraderapi.CThostFtdcReqSyncKeyField_TradeCode_get, _thosttraderapi.CThostFtdcReqSyncKeyField_TradeCode_set)
    BankID = property(_thosttraderapi.CThostFtdcReqSyncKeyField_BankID_get, _thosttraderapi.CThostFtdcReqSyncKeyField_BankID_set)
    BankBranchID = property(_thosttraderapi.CThostFtdcReqSyncKeyField_BankBranchID_get, _thosttraderapi.CThostFtdcReqSyncKeyField_BankBranchID_set)
    BrokerID = property(_thosttraderapi.CThostFtdcReqSyncKeyField_BrokerID_get, _thosttraderapi.CThostFtdcReqSyncKeyField_BrokerID_set)
    BrokerBranchID = property(_thosttraderapi.CThostFtdcReqSyncKeyField_BrokerBranchID_get, _thosttraderapi.CThostFtdcReqSyncKeyField_BrokerBranchID_set)
    TradeDate = property(_thosttraderapi.CThostFtdcReqSyncKeyField_TradeDate_get, _thosttraderapi.CThostFtdcReqSyncKeyField_TradeDate_set)
    TradeTime = property(_thosttraderapi.CThostFtdcReqSyncKeyField_TradeTime_get, _thosttraderapi.CThostFtdcReqSyncKeyField_TradeTime_set)
    BankSerial = property(_thosttraderapi.CThostFtdcReqSyncKeyField_BankSerial_get, _thosttraderapi.CThostFtdcReqSyncKeyField_BankSerial_set)
    TradingDay = property(_thosttraderapi.CThostFtdcReqSyncKeyField_TradingDay_get, _thosttraderapi.CThostFtdcReqSyncKeyField_TradingDay_set)
    PlateSerial = property(_thosttraderapi.CThostFtdcReqSyncKeyField_PlateSerial_get, _thosttraderapi.CThostFtdcReqSyncKeyField_PlateSerial_set)
    LastFragment = property(_thosttraderapi.CThostFtdcReqSyncKeyField_LastFragment_get, _thosttraderapi.CThostFtdcReqSyncKeyField_LastFragment_set)
    SessionID = property(_thosttraderapi.CThostFtdcReqSyncKeyField_SessionID_get, _thosttraderapi.CThostFtdcReqSyncKeyField_SessionID_set)
    InstallID = property(_thosttraderapi.CThostFtdcReqSyncKeyField_InstallID_get, _thosttraderapi.CThostFtdcReqSyncKeyField_InstallID_set)
    UserID = property(_thosttraderapi.CThostFtdcReqSyncKeyField_UserID_get, _thosttraderapi.CThostFtdcReqSyncKeyField_UserID_set)
    Message = property(_thosttraderapi.CThostFtdcReqSyncKeyField_Message_get, _thosttraderapi.CThostFtdcReqSyncKeyField_Message_set)
    DeviceID = property(_thosttraderapi.CThostFtdcReqSyncKeyField_DeviceID_get, _thosttraderapi.CThostFtdcReqSyncKeyField_DeviceID_set)
    BrokerIDByBank = property(_thosttraderapi.CThostFtdcReqSyncKeyField_BrokerIDByBank_get, _thosttraderapi.CThostFtdcReqSyncKeyField_BrokerIDByBank_set)
    OperNo = property(_thosttraderapi.CThostFtdcReqSyncKeyField_OperNo_get, _thosttraderapi.CThostFtdcReqSyncKeyField_OperNo_set)
    RequestID = property(_thosttraderapi.CThostFtdcReqSyncKeyField_RequestID_get, _thosttraderapi.CThostFtdcReqSyncKeyField_RequestID_set)
    TID = property(_thosttraderapi.CThostFtdcReqSyncKeyField_TID_get, _thosttraderapi.CThostFtdcReqSyncKeyField_TID_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcReqSyncKeyField_swiginit(self, _thosttraderapi.new_CThostFtdcReqSyncKeyField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcReqSyncKeyField

# Register CThostFtdcReqSyncKeyField in _thosttraderapi:
_thosttraderapi.CThostFtdcReqSyncKeyField_swigregister(CThostFtdcReqSyncKeyField)

class CThostFtdcRspSyncKeyField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    TradeCode = property(_thosttraderapi.CThostFtdcRspSyncKeyField_TradeCode_get, _thosttraderapi.CThostFtdcRspSyncKeyField_TradeCode_set)
    BankID = property(_thosttraderapi.CThostFtdcRspSyncKeyField_BankID_get, _thosttraderapi.CThostFtdcRspSyncKeyField_BankID_set)
    BankBranchID = property(_thosttraderapi.CThostFtdcRspSyncKeyField_BankBranchID_get, _thosttraderapi.CThostFtdcRspSyncKeyField_BankBranchID_set)
    BrokerID = property(_thosttraderapi.CThostFtdcRspSyncKeyField_BrokerID_get, _thosttraderapi.CThostFtdcRspSyncKeyField_BrokerID_set)
    BrokerBranchID = property(_thosttraderapi.CThostFtdcRspSyncKeyField_BrokerBranchID_get, _thosttraderapi.CThostFtdcRspSyncKeyField_BrokerBranchID_set)
    TradeDate = property(_thosttraderapi.CThostFtdcRspSyncKeyField_TradeDate_get, _thosttraderapi.CThostFtdcRspSyncKeyField_TradeDate_set)
    TradeTime = property(_thosttraderapi.CThostFtdcRspSyncKeyField_TradeTime_get, _thosttraderapi.CThostFtdcRspSyncKeyField_TradeTime_set)
    BankSerial = property(_thosttraderapi.CThostFtdcRspSyncKeyField_BankSerial_get, _thosttraderapi.CThostFtdcRspSyncKeyField_BankSerial_set)
    TradingDay = property(_thosttraderapi.CThostFtdcRspSyncKeyField_TradingDay_get, _thosttraderapi.CThostFtdcRspSyncKeyField_TradingDay_set)
    PlateSerial = property(_thosttraderapi.CThostFtdcRspSyncKeyField_PlateSerial_get, _thosttraderapi.CThostFtdcRspSyncKeyField_PlateSerial_set)
    LastFragment = property(_thosttraderapi.CThostFtdcRspSyncKeyField_LastFragment_get, _thosttraderapi.CThostFtdcRspSyncKeyField_LastFragment_set)
    SessionID = property(_thosttraderapi.CThostFtdcRspSyncKeyField_SessionID_get, _thosttraderapi.CThostFtdcRspSyncKeyField_SessionID_set)
    InstallID = property(_thosttraderapi.CThostFtdcRspSyncKeyField_InstallID_get, _thosttraderapi.CThostFtdcRspSyncKeyField_InstallID_set)
    UserID = property(_thosttraderapi.CThostFtdcRspSyncKeyField_UserID_get, _thosttraderapi.CThostFtdcRspSyncKeyField_UserID_set)
    Message = property(_thosttraderapi.CThostFtdcRspSyncKeyField_Message_get, _thosttraderapi.CThostFtdcRspSyncKeyField_Message_set)
    DeviceID = property(_thosttraderapi.CThostFtdcRspSyncKeyField_DeviceID_get, _thosttraderapi.CThostFtdcRspSyncKeyField_DeviceID_set)
    BrokerIDByBank = property(_thosttraderapi.CThostFtdcRspSyncKeyField_BrokerIDByBank_get, _thosttraderapi.CThostFtdcRspSyncKeyField_BrokerIDByBank_set)
    OperNo = property(_thosttraderapi.CThostFtdcRspSyncKeyField_OperNo_get, _thosttraderapi.CThostFtdcRspSyncKeyField_OperNo_set)
    RequestID = property(_thosttraderapi.CThostFtdcRspSyncKeyField_RequestID_get, _thosttraderapi.CThostFtdcRspSyncKeyField_RequestID_set)
    TID = property(_thosttraderapi.CThostFtdcRspSyncKeyField_TID_get, _thosttraderapi.CThostFtdcRspSyncKeyField_TID_set)
    ErrorID = property(_thosttraderapi.CThostFtdcRspSyncKeyField_ErrorID_get, _thosttraderapi.CThostFtdcRspSyncKeyField_ErrorID_set)
    ErrorMsg = property(_thosttraderapi.CThostFtdcRspSyncKeyField_ErrorMsg_get, _thosttraderapi.CThostFtdcRspSyncKeyField_ErrorMsg_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcRspSyncKeyField_swiginit(self, _thosttraderapi.new_CThostFtdcRspSyncKeyField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcRspSyncKeyField

# Register CThostFtdcRspSyncKeyField in _thosttraderapi:
_thosttraderapi.CThostFtdcRspSyncKeyField_swigregister(CThostFtdcRspSyncKeyField)

class CThostFtdcNotifyQueryAccountField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    TradeCode = property(_thosttraderapi.CThostFtdcNotifyQueryAccountField_TradeCode_get, _thosttraderapi.CThostFtdcNotifyQueryAccountField_TradeCode_set)
    BankID = property(_thosttraderapi.CThostFtdcNotifyQueryAccountField_BankID_get, _thosttraderapi.CThostFtdcNotifyQueryAccountField_BankID_set)
    BankBranchID = property(_thosttraderapi.CThostFtdcNotifyQueryAccountField_BankBranchID_get, _thosttraderapi.CThostFtdcNotifyQueryAccountField_BankBranchID_set)
    BrokerID = property(_thosttraderapi.CThostFtdcNotifyQueryAccountField_BrokerID_get, _thosttraderapi.CThostFtdcNotifyQueryAccountField_BrokerID_set)
    BrokerBranchID = property(_thosttraderapi.CThostFtdcNotifyQueryAccountField_BrokerBranchID_get, _thosttraderapi.CThostFtdcNotifyQueryAccountField_BrokerBranchID_set)
    TradeDate = property(_thosttraderapi.CThostFtdcNotifyQueryAccountField_TradeDate_get, _thosttraderapi.CThostFtdcNotifyQueryAccountField_TradeDate_set)
    TradeTime = property(_thosttraderapi.CThostFtdcNotifyQueryAccountField_TradeTime_get, _thosttraderapi.CThostFtdcNotifyQueryAccountField_TradeTime_set)
    BankSerial = property(_thosttraderapi.CThostFtdcNotifyQueryAccountField_BankSerial_get, _thosttraderapi.CThostFtdcNotifyQueryAccountField_BankSerial_set)
    TradingDay = property(_thosttraderapi.CThostFtdcNotifyQueryAccountField_TradingDay_get, _thosttraderapi.CThostFtdcNotifyQueryAccountField_TradingDay_set)
    PlateSerial = property(_thosttraderapi.CThostFtdcNotifyQueryAccountField_PlateSerial_get, _thosttraderapi.CThostFtdcNotifyQueryAccountField_PlateSerial_set)
    LastFragment = property(_thosttraderapi.CThostFtdcNotifyQueryAccountField_LastFragment_get, _thosttraderapi.CThostFtdcNotifyQueryAccountField_LastFragment_set)
    SessionID = property(_thosttraderapi.CThostFtdcNotifyQueryAccountField_SessionID_get, _thosttraderapi.CThostFtdcNotifyQueryAccountField_SessionID_set)
    CustomerName = property(_thosttraderapi.CThostFtdcNotifyQueryAccountField_CustomerName_get, _thosttraderapi.CThostFtdcNotifyQueryAccountField_CustomerName_set)
    IdCardType = property(_thosttraderapi.CThostFtdcNotifyQueryAccountField_IdCardType_get, _thosttraderapi.CThostFtdcNotifyQueryAccountField_IdCardType_set)
    IdentifiedCardNo = property(_thosttraderapi.CThostFtdcNotifyQueryAccountField_IdentifiedCardNo_get, _thosttraderapi.CThostFtdcNotifyQueryAccountField_IdentifiedCardNo_set)
    CustType = property(_thosttraderapi.CThostFtdcNotifyQueryAccountField_CustType_get, _thosttraderapi.CThostFtdcNotifyQueryAccountField_CustType_set)
    BankAccount = property(_thosttraderapi.CThostFtdcNotifyQueryAccountField_BankAccount_get, _thosttraderapi.CThostFtdcNotifyQueryAccountField_BankAccount_set)
    BankPassWord = property(_thosttraderapi.CThostFtdcNotifyQueryAccountField_BankPassWord_get, _thosttraderapi.CThostFtdcNotifyQueryAccountField_BankPassWord_set)
    AccountID = property(_thosttraderapi.CThostFtdcNotifyQueryAccountField_AccountID_get, _thosttraderapi.CThostFtdcNotifyQueryAccountField_AccountID_set)
    Password = property(_thosttraderapi.CThostFtdcNotifyQueryAccountField_Password_get, _thosttraderapi.CThostFtdcNotifyQueryAccountField_Password_set)
    FutureSerial = property(_thosttraderapi.CThostFtdcNotifyQueryAccountField_FutureSerial_get, _thosttraderapi.CThostFtdcNotifyQueryAccountField_FutureSerial_set)
    InstallID = property(_thosttraderapi.CThostFtdcNotifyQueryAccountField_InstallID_get, _thosttraderapi.CThostFtdcNotifyQueryAccountField_InstallID_set)
    UserID = property(_thosttraderapi.CThostFtdcNotifyQueryAccountField_UserID_get, _thosttraderapi.CThostFtdcNotifyQueryAccountField_UserID_set)
    VerifyCertNoFlag = property(_thosttraderapi.CThostFtdcNotifyQueryAccountField_VerifyCertNoFlag_get, _thosttraderapi.CThostFtdcNotifyQueryAccountField_VerifyCertNoFlag_set)
    CurrencyID = property(_thosttraderapi.CThostFtdcNotifyQueryAccountField_CurrencyID_get, _thosttraderapi.CThostFtdcNotifyQueryAccountField_CurrencyID_set)
    Digest = property(_thosttraderapi.CThostFtdcNotifyQueryAccountField_Digest_get, _thosttraderapi.CThostFtdcNotifyQueryAccountField_Digest_set)
    BankAccType = property(_thosttraderapi.CThostFtdcNotifyQueryAccountField_BankAccType_get, _thosttraderapi.CThostFtdcNotifyQueryAccountField_BankAccType_set)
    DeviceID = property(_thosttraderapi.CThostFtdcNotifyQueryAccountField_DeviceID_get, _thosttraderapi.CThostFtdcNotifyQueryAccountField_DeviceID_set)
    BankSecuAccType = property(_thosttraderapi.CThostFtdcNotifyQueryAccountField_BankSecuAccType_get, _thosttraderapi.CThostFtdcNotifyQueryAccountField_BankSecuAccType_set)
    BrokerIDByBank = property(_thosttraderapi.CThostFtdcNotifyQueryAccountField_BrokerIDByBank_get, _thosttraderapi.CThostFtdcNotifyQueryAccountField_BrokerIDByBank_set)
    BankSecuAcc = property(_thosttraderapi.CThostFtdcNotifyQueryAccountField_BankSecuAcc_get, _thosttraderapi.CThostFtdcNotifyQueryAccountField_BankSecuAcc_set)
    BankPwdFlag = property(_thosttraderapi.CThostFtdcNotifyQueryAccountField_BankPwdFlag_get, _thosttraderapi.CThostFtdcNotifyQueryAccountField_BankPwdFlag_set)
    SecuPwdFlag = property(_thosttraderapi.CThostFtdcNotifyQueryAccountField_SecuPwdFlag_get, _thosttraderapi.CThostFtdcNotifyQueryAccountField_SecuPwdFlag_set)
    OperNo = property(_thosttraderapi.CThostFtdcNotifyQueryAccountField_OperNo_get, _thosttraderapi.CThostFtdcNotifyQueryAccountField_OperNo_set)
    RequestID = property(_thosttraderapi.CThostFtdcNotifyQueryAccountField_RequestID_get, _thosttraderapi.CThostFtdcNotifyQueryAccountField_RequestID_set)
    TID = property(_thosttraderapi.CThostFtdcNotifyQueryAccountField_TID_get, _thosttraderapi.CThostFtdcNotifyQueryAccountField_TID_set)
    BankUseAmount = property(_thosttraderapi.CThostFtdcNotifyQueryAccountField_BankUseAmount_get, _thosttraderapi.CThostFtdcNotifyQueryAccountField_BankUseAmount_set)
    BankFetchAmount = property(_thosttraderapi.CThostFtdcNotifyQueryAccountField_BankFetchAmount_get, _thosttraderapi.CThostFtdcNotifyQueryAccountField_BankFetchAmount_set)
    ErrorID = property(_thosttraderapi.CThostFtdcNotifyQueryAccountField_ErrorID_get, _thosttraderapi.CThostFtdcNotifyQueryAccountField_ErrorID_set)
    ErrorMsg = property(_thosttraderapi.CThostFtdcNotifyQueryAccountField_ErrorMsg_get, _thosttraderapi.CThostFtdcNotifyQueryAccountField_ErrorMsg_set)
    LongCustomerName = property(_thosttraderapi.CThostFtdcNotifyQueryAccountField_LongCustomerName_get, _thosttraderapi.CThostFtdcNotifyQueryAccountField_LongCustomerName_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcNotifyQueryAccountField_swiginit(self, _thosttraderapi.new_CThostFtdcNotifyQueryAccountField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcNotifyQueryAccountField

# Register CThostFtdcNotifyQueryAccountField in _thosttraderapi:
_thosttraderapi.CThostFtdcNotifyQueryAccountField_swigregister(CThostFtdcNotifyQueryAccountField)

class CThostFtdcTransferSerialField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    PlateSerial = property(_thosttraderapi.CThostFtdcTransferSerialField_PlateSerial_get, _thosttraderapi.CThostFtdcTransferSerialField_PlateSerial_set)
    TradeDate = property(_thosttraderapi.CThostFtdcTransferSerialField_TradeDate_get, _thosttraderapi.CThostFtdcTransferSerialField_TradeDate_set)
    TradingDay = property(_thosttraderapi.CThostFtdcTransferSerialField_TradingDay_get, _thosttraderapi.CThostFtdcTransferSerialField_TradingDay_set)
    TradeTime = property(_thosttraderapi.CThostFtdcTransferSerialField_TradeTime_get, _thosttraderapi.CThostFtdcTransferSerialField_TradeTime_set)
    TradeCode = property(_thosttraderapi.CThostFtdcTransferSerialField_TradeCode_get, _thosttraderapi.CThostFtdcTransferSerialField_TradeCode_set)
    SessionID = property(_thosttraderapi.CThostFtdcTransferSerialField_SessionID_get, _thosttraderapi.CThostFtdcTransferSerialField_SessionID_set)
    BankID = property(_thosttraderapi.CThostFtdcTransferSerialField_BankID_get, _thosttraderapi.CThostFtdcTransferSerialField_BankID_set)
    BankBranchID = property(_thosttraderapi.CThostFtdcTransferSerialField_BankBranchID_get, _thosttraderapi.CThostFtdcTransferSerialField_BankBranchID_set)
    BankAccType = property(_thosttraderapi.CThostFtdcTransferSerialField_BankAccType_get, _thosttraderapi.CThostFtdcTransferSerialField_BankAccType_set)
    BankAccount = property(_thosttraderapi.CThostFtdcTransferSerialField_BankAccount_get, _thosttraderapi.CThostFtdcTransferSerialField_BankAccount_set)
    BankSerial = property(_thosttraderapi.CThostFtdcTransferSerialField_BankSerial_get, _thosttraderapi.CThostFtdcTransferSerialField_BankSerial_set)
    BrokerID = property(_thosttraderapi.CThostFtdcTransferSerialField_BrokerID_get, _thosttraderapi.CThostFtdcTransferSerialField_BrokerID_set)
    BrokerBranchID = property(_thosttraderapi.CThostFtdcTransferSerialField_BrokerBranchID_get, _thosttraderapi.CThostFtdcTransferSerialField_BrokerBranchID_set)
    FutureAccType = property(_thosttraderapi.CThostFtdcTransferSerialField_FutureAccType_get, _thosttraderapi.CThostFtdcTransferSerialField_FutureAccType_set)
    AccountID = property(_thosttraderapi.CThostFtdcTransferSerialField_AccountID_get, _thosttraderapi.CThostFtdcTransferSerialField_AccountID_set)
    InvestorID = property(_thosttraderapi.CThostFtdcTransferSerialField_InvestorID_get, _thosttraderapi.CThostFtdcTransferSerialField_InvestorID_set)
    FutureSerial = property(_thosttraderapi.CThostFtdcTransferSerialField_FutureSerial_get, _thosttraderapi.CThostFtdcTransferSerialField_FutureSerial_set)
    IdCardType = property(_thosttraderapi.CThostFtdcTransferSerialField_IdCardType_get, _thosttraderapi.CThostFtdcTransferSerialField_IdCardType_set)
    IdentifiedCardNo = property(_thosttraderapi.CThostFtdcTransferSerialField_IdentifiedCardNo_get, _thosttraderapi.CThostFtdcTransferSerialField_IdentifiedCardNo_set)
    CurrencyID = property(_thosttraderapi.CThostFtdcTransferSerialField_CurrencyID_get, _thosttraderapi.CThostFtdcTransferSerialField_CurrencyID_set)
    TradeAmount = property(_thosttraderapi.CThostFtdcTransferSerialField_TradeAmount_get, _thosttraderapi.CThostFtdcTransferSerialField_TradeAmount_set)
    CustFee = property(_thosttraderapi.CThostFtdcTransferSerialField_CustFee_get, _thosttraderapi.CThostFtdcTransferSerialField_CustFee_set)
    BrokerFee = property(_thosttraderapi.CThostFtdcTransferSerialField_BrokerFee_get, _thosttraderapi.CThostFtdcTransferSerialField_BrokerFee_set)
    AvailabilityFlag = property(_thosttraderapi.CThostFtdcTransferSerialField_AvailabilityFlag_get, _thosttraderapi.CThostFtdcTransferSerialField_AvailabilityFlag_set)
    OperatorCode = property(_thosttraderapi.CThostFtdcTransferSerialField_OperatorCode_get, _thosttraderapi.CThostFtdcTransferSerialField_OperatorCode_set)
    BankNewAccount = property(_thosttraderapi.CThostFtdcTransferSerialField_BankNewAccount_get, _thosttraderapi.CThostFtdcTransferSerialField_BankNewAccount_set)
    ErrorID = property(_thosttraderapi.CThostFtdcTransferSerialField_ErrorID_get, _thosttraderapi.CThostFtdcTransferSerialField_ErrorID_set)
    ErrorMsg = property(_thosttraderapi.CThostFtdcTransferSerialField_ErrorMsg_get, _thosttraderapi.CThostFtdcTransferSerialField_ErrorMsg_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcTransferSerialField_swiginit(self, _thosttraderapi.new_CThostFtdcTransferSerialField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcTransferSerialField

# Register CThostFtdcTransferSerialField in _thosttraderapi:
_thosttraderapi.CThostFtdcTransferSerialField_swigregister(CThostFtdcTransferSerialField)

class CThostFtdcQryTransferSerialField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thosttraderapi.CThostFtdcQryTransferSerialField_BrokerID_get, _thosttraderapi.CThostFtdcQryTransferSerialField_BrokerID_set)
    AccountID = property(_thosttraderapi.CThostFtdcQryTransferSerialField_AccountID_get, _thosttraderapi.CThostFtdcQryTransferSerialField_AccountID_set)
    BankID = property(_thosttraderapi.CThostFtdcQryTransferSerialField_BankID_get, _thosttraderapi.CThostFtdcQryTransferSerialField_BankID_set)
    CurrencyID = property(_thosttraderapi.CThostFtdcQryTransferSerialField_CurrencyID_get, _thosttraderapi.CThostFtdcQryTransferSerialField_CurrencyID_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcQryTransferSerialField_swiginit(self, _thosttraderapi.new_CThostFtdcQryTransferSerialField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcQryTransferSerialField

# Register CThostFtdcQryTransferSerialField in _thosttraderapi:
_thosttraderapi.CThostFtdcQryTransferSerialField_swigregister(CThostFtdcQryTransferSerialField)

class CThostFtdcNotifyFutureSignInField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    TradeCode = property(_thosttraderapi.CThostFtdcNotifyFutureSignInField_TradeCode_get, _thosttraderapi.CThostFtdcNotifyFutureSignInField_TradeCode_set)
    BankID = property(_thosttraderapi.CThostFtdcNotifyFutureSignInField_BankID_get, _thosttraderapi.CThostFtdcNotifyFutureSignInField_BankID_set)
    BankBranchID = property(_thosttraderapi.CThostFtdcNotifyFutureSignInField_BankBranchID_get, _thosttraderapi.CThostFtdcNotifyFutureSignInField_BankBranchID_set)
    BrokerID = property(_thosttraderapi.CThostFtdcNotifyFutureSignInField_BrokerID_get, _thosttraderapi.CThostFtdcNotifyFutureSignInField_BrokerID_set)
    BrokerBranchID = property(_thosttraderapi.CThostFtdcNotifyFutureSignInField_BrokerBranchID_get, _thosttraderapi.CThostFtdcNotifyFutureSignInField_BrokerBranchID_set)
    TradeDate = property(_thosttraderapi.CThostFtdcNotifyFutureSignInField_TradeDate_get, _thosttraderapi.CThostFtdcNotifyFutureSignInField_TradeDate_set)
    TradeTime = property(_thosttraderapi.CThostFtdcNotifyFutureSignInField_TradeTime_get, _thosttraderapi.CThostFtdcNotifyFutureSignInField_TradeTime_set)
    BankSerial = property(_thosttraderapi.CThostFtdcNotifyFutureSignInField_BankSerial_get, _thosttraderapi.CThostFtdcNotifyFutureSignInField_BankSerial_set)
    TradingDay = property(_thosttraderapi.CThostFtdcNotifyFutureSignInField_TradingDay_get, _thosttraderapi.CThostFtdcNotifyFutureSignInField_TradingDay_set)
    PlateSerial = property(_thosttraderapi.CThostFtdcNotifyFutureSignInField_PlateSerial_get, _thosttraderapi.CThostFtdcNotifyFutureSignInField_PlateSerial_set)
    LastFragment = property(_thosttraderapi.CThostFtdcNotifyFutureSignInField_LastFragment_get, _thosttraderapi.CThostFtdcNotifyFutureSignInField_LastFragment_set)
    SessionID = property(_thosttraderapi.CThostFtdcNotifyFutureSignInField_SessionID_get, _thosttraderapi.CThostFtdcNotifyFutureSignInField_SessionID_set)
    InstallID = property(_thosttraderapi.CThostFtdcNotifyFutureSignInField_InstallID_get, _thosttraderapi.CThostFtdcNotifyFutureSignInField_InstallID_set)
    UserID = property(_thosttraderapi.CThostFtdcNotifyFutureSignInField_UserID_get, _thosttraderapi.CThostFtdcNotifyFutureSignInField_UserID_set)
    Digest = property(_thosttraderapi.CThostFtdcNotifyFutureSignInField_Digest_get, _thosttraderapi.CThostFtdcNotifyFutureSignInField_Digest_set)
    CurrencyID = property(_thosttraderapi.CThostFtdcNotifyFutureSignInField_CurrencyID_get, _thosttraderapi.CThostFtdcNotifyFutureSignInField_CurrencyID_set)
    DeviceID = property(_thosttraderapi.CThostFtdcNotifyFutureSignInField_DeviceID_get, _thosttraderapi.CThostFtdcNotifyFutureSignInField_DeviceID_set)
    BrokerIDByBank = property(_thosttraderapi.CThostFtdcNotifyFutureSignInField_BrokerIDByBank_get, _thosttraderapi.CThostFtdcNotifyFutureSignInField_BrokerIDByBank_set)
    OperNo = property(_thosttraderapi.CThostFtdcNotifyFutureSignInField_OperNo_get, _thosttraderapi.CThostFtdcNotifyFutureSignInField_OperNo_set)
    RequestID = property(_thosttraderapi.CThostFtdcNotifyFutureSignInField_RequestID_get, _thosttraderapi.CThostFtdcNotifyFutureSignInField_RequestID_set)
    TID = property(_thosttraderapi.CThostFtdcNotifyFutureSignInField_TID_get, _thosttraderapi.CThostFtdcNotifyFutureSignInField_TID_set)
    ErrorID = property(_thosttraderapi.CThostFtdcNotifyFutureSignInField_ErrorID_get, _thosttraderapi.CThostFtdcNotifyFutureSignInField_ErrorID_set)
    ErrorMsg = property(_thosttraderapi.CThostFtdcNotifyFutureSignInField_ErrorMsg_get, _thosttraderapi.CThostFtdcNotifyFutureSignInField_ErrorMsg_set)
    PinKey = property(_thosttraderapi.CThostFtdcNotifyFutureSignInField_PinKey_get, _thosttraderapi.CThostFtdcNotifyFutureSignInField_PinKey_set)
    MacKey = property(_thosttraderapi.CThostFtdcNotifyFutureSignInField_MacKey_get, _thosttraderapi.CThostFtdcNotifyFutureSignInField_MacKey_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcNotifyFutureSignInField_swiginit(self, _thosttraderapi.new_CThostFtdcNotifyFutureSignInField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcNotifyFutureSignInField

# Register CThostFtdcNotifyFutureSignInField in _thosttraderapi:
_thosttraderapi.CThostFtdcNotifyFutureSignInField_swigregister(CThostFtdcNotifyFutureSignInField)

class CThostFtdcNotifyFutureSignOutField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    TradeCode = property(_thosttraderapi.CThostFtdcNotifyFutureSignOutField_TradeCode_get, _thosttraderapi.CThostFtdcNotifyFutureSignOutField_TradeCode_set)
    BankID = property(_thosttraderapi.CThostFtdcNotifyFutureSignOutField_BankID_get, _thosttraderapi.CThostFtdcNotifyFutureSignOutField_BankID_set)
    BankBranchID = property(_thosttraderapi.CThostFtdcNotifyFutureSignOutField_BankBranchID_get, _thosttraderapi.CThostFtdcNotifyFutureSignOutField_BankBranchID_set)
    BrokerID = property(_thosttraderapi.CThostFtdcNotifyFutureSignOutField_BrokerID_get, _thosttraderapi.CThostFtdcNotifyFutureSignOutField_BrokerID_set)
    BrokerBranchID = property(_thosttraderapi.CThostFtdcNotifyFutureSignOutField_BrokerBranchID_get, _thosttraderapi.CThostFtdcNotifyFutureSignOutField_BrokerBranchID_set)
    TradeDate = property(_thosttraderapi.CThostFtdcNotifyFutureSignOutField_TradeDate_get, _thosttraderapi.CThostFtdcNotifyFutureSignOutField_TradeDate_set)
    TradeTime = property(_thosttraderapi.CThostFtdcNotifyFutureSignOutField_TradeTime_get, _thosttraderapi.CThostFtdcNotifyFutureSignOutField_TradeTime_set)
    BankSerial = property(_thosttraderapi.CThostFtdcNotifyFutureSignOutField_BankSerial_get, _thosttraderapi.CThostFtdcNotifyFutureSignOutField_BankSerial_set)
    TradingDay = property(_thosttraderapi.CThostFtdcNotifyFutureSignOutField_TradingDay_get, _thosttraderapi.CThostFtdcNotifyFutureSignOutField_TradingDay_set)
    PlateSerial = property(_thosttraderapi.CThostFtdcNotifyFutureSignOutField_PlateSerial_get, _thosttraderapi.CThostFtdcNotifyFutureSignOutField_PlateSerial_set)
    LastFragment = property(_thosttraderapi.CThostFtdcNotifyFutureSignOutField_LastFragment_get, _thosttraderapi.CThostFtdcNotifyFutureSignOutField_LastFragment_set)
    SessionID = property(_thosttraderapi.CThostFtdcNotifyFutureSignOutField_SessionID_get, _thosttraderapi.CThostFtdcNotifyFutureSignOutField_SessionID_set)
    InstallID = property(_thosttraderapi.CThostFtdcNotifyFutureSignOutField_InstallID_get, _thosttraderapi.CThostFtdcNotifyFutureSignOutField_InstallID_set)
    UserID = property(_thosttraderapi.CThostFtdcNotifyFutureSignOutField_UserID_get, _thosttraderapi.CThostFtdcNotifyFutureSignOutField_UserID_set)
    Digest = property(_thosttraderapi.CThostFtdcNotifyFutureSignOutField_Digest_get, _thosttraderapi.CThostFtdcNotifyFutureSignOutField_Digest_set)
    CurrencyID = property(_thosttraderapi.CThostFtdcNotifyFutureSignOutField_CurrencyID_get, _thosttraderapi.CThostFtdcNotifyFutureSignOutField_CurrencyID_set)
    DeviceID = property(_thosttraderapi.CThostFtdcNotifyFutureSignOutField_DeviceID_get, _thosttraderapi.CThostFtdcNotifyFutureSignOutField_DeviceID_set)
    BrokerIDByBank = property(_thosttraderapi.CThostFtdcNotifyFutureSignOutField_BrokerIDByBank_get, _thosttraderapi.CThostFtdcNotifyFutureSignOutField_BrokerIDByBank_set)
    OperNo = property(_thosttraderapi.CThostFtdcNotifyFutureSignOutField_OperNo_get, _thosttraderapi.CThostFtdcNotifyFutureSignOutField_OperNo_set)
    RequestID = property(_thosttraderapi.CThostFtdcNotifyFutureSignOutField_RequestID_get, _thosttraderapi.CThostFtdcNotifyFutureSignOutField_RequestID_set)
    TID = property(_thosttraderapi.CThostFtdcNotifyFutureSignOutField_TID_get, _thosttraderapi.CThostFtdcNotifyFutureSignOutField_TID_set)
    ErrorID = property(_thosttraderapi.CThostFtdcNotifyFutureSignOutField_ErrorID_get, _thosttraderapi.CThostFtdcNotifyFutureSignOutField_ErrorID_set)
    ErrorMsg = property(_thosttraderapi.CThostFtdcNotifyFutureSignOutField_ErrorMsg_get, _thosttraderapi.CThostFtdcNotifyFutureSignOutField_ErrorMsg_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcNotifyFutureSignOutField_swiginit(self, _thosttraderapi.new_CThostFtdcNotifyFutureSignOutField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcNotifyFutureSignOutField

# Register CThostFtdcNotifyFutureSignOutField in _thosttraderapi:
_thosttraderapi.CThostFtdcNotifyFutureSignOutField_swigregister(CThostFtdcNotifyFutureSignOutField)

class CThostFtdcNotifySyncKeyField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    TradeCode = property(_thosttraderapi.CThostFtdcNotifySyncKeyField_TradeCode_get, _thosttraderapi.CThostFtdcNotifySyncKeyField_TradeCode_set)
    BankID = property(_thosttraderapi.CThostFtdcNotifySyncKeyField_BankID_get, _thosttraderapi.CThostFtdcNotifySyncKeyField_BankID_set)
    BankBranchID = property(_thosttraderapi.CThostFtdcNotifySyncKeyField_BankBranchID_get, _thosttraderapi.CThostFtdcNotifySyncKeyField_BankBranchID_set)
    BrokerID = property(_thosttraderapi.CThostFtdcNotifySyncKeyField_BrokerID_get, _thosttraderapi.CThostFtdcNotifySyncKeyField_BrokerID_set)
    BrokerBranchID = property(_thosttraderapi.CThostFtdcNotifySyncKeyField_BrokerBranchID_get, _thosttraderapi.CThostFtdcNotifySyncKeyField_BrokerBranchID_set)
    TradeDate = property(_thosttraderapi.CThostFtdcNotifySyncKeyField_TradeDate_get, _thosttraderapi.CThostFtdcNotifySyncKeyField_TradeDate_set)
    TradeTime = property(_thosttraderapi.CThostFtdcNotifySyncKeyField_TradeTime_get, _thosttraderapi.CThostFtdcNotifySyncKeyField_TradeTime_set)
    BankSerial = property(_thosttraderapi.CThostFtdcNotifySyncKeyField_BankSerial_get, _thosttraderapi.CThostFtdcNotifySyncKeyField_BankSerial_set)
    TradingDay = property(_thosttraderapi.CThostFtdcNotifySyncKeyField_TradingDay_get, _thosttraderapi.CThostFtdcNotifySyncKeyField_TradingDay_set)
    PlateSerial = property(_thosttraderapi.CThostFtdcNotifySyncKeyField_PlateSerial_get, _thosttraderapi.CThostFtdcNotifySyncKeyField_PlateSerial_set)
    LastFragment = property(_thosttraderapi.CThostFtdcNotifySyncKeyField_LastFragment_get, _thosttraderapi.CThostFtdcNotifySyncKeyField_LastFragment_set)
    SessionID = property(_thosttraderapi.CThostFtdcNotifySyncKeyField_SessionID_get, _thosttraderapi.CThostFtdcNotifySyncKeyField_SessionID_set)
    InstallID = property(_thosttraderapi.CThostFtdcNotifySyncKeyField_InstallID_get, _thosttraderapi.CThostFtdcNotifySyncKeyField_InstallID_set)
    UserID = property(_thosttraderapi.CThostFtdcNotifySyncKeyField_UserID_get, _thosttraderapi.CThostFtdcNotifySyncKeyField_UserID_set)
    Message = property(_thosttraderapi.CThostFtdcNotifySyncKeyField_Message_get, _thosttraderapi.CThostFtdcNotifySyncKeyField_Message_set)
    DeviceID = property(_thosttraderapi.CThostFtdcNotifySyncKeyField_DeviceID_get, _thosttraderapi.CThostFtdcNotifySyncKeyField_DeviceID_set)
    BrokerIDByBank = property(_thosttraderapi.CThostFtdcNotifySyncKeyField_BrokerIDByBank_get, _thosttraderapi.CThostFtdcNotifySyncKeyField_BrokerIDByBank_set)
    OperNo = property(_thosttraderapi.CThostFtdcNotifySyncKeyField_OperNo_get, _thosttraderapi.CThostFtdcNotifySyncKeyField_OperNo_set)
    RequestID = property(_thosttraderapi.CThostFtdcNotifySyncKeyField_RequestID_get, _thosttraderapi.CThostFtdcNotifySyncKeyField_RequestID_set)
    TID = property(_thosttraderapi.CThostFtdcNotifySyncKeyField_TID_get, _thosttraderapi.CThostFtdcNotifySyncKeyField_TID_set)
    ErrorID = property(_thosttraderapi.CThostFtdcNotifySyncKeyField_ErrorID_get, _thosttraderapi.CThostFtdcNotifySyncKeyField_ErrorID_set)
    ErrorMsg = property(_thosttraderapi.CThostFtdcNotifySyncKeyField_ErrorMsg_get, _thosttraderapi.CThostFtdcNotifySyncKeyField_ErrorMsg_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcNotifySyncKeyField_swiginit(self, _thosttraderapi.new_CThostFtdcNotifySyncKeyField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcNotifySyncKeyField

# Register CThostFtdcNotifySyncKeyField in _thosttraderapi:
_thosttraderapi.CThostFtdcNotifySyncKeyField_swigregister(CThostFtdcNotifySyncKeyField)

class CThostFtdcQryAccountregisterField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thosttraderapi.CThostFtdcQryAccountregisterField_BrokerID_get, _thosttraderapi.CThostFtdcQryAccountregisterField_BrokerID_set)
    AccountID = property(_thosttraderapi.CThostFtdcQryAccountregisterField_AccountID_get, _thosttraderapi.CThostFtdcQryAccountregisterField_AccountID_set)
    BankID = property(_thosttraderapi.CThostFtdcQryAccountregisterField_BankID_get, _thosttraderapi.CThostFtdcQryAccountregisterField_BankID_set)
    BankBranchID = property(_thosttraderapi.CThostFtdcQryAccountregisterField_BankBranchID_get, _thosttraderapi.CThostFtdcQryAccountregisterField_BankBranchID_set)
    CurrencyID = property(_thosttraderapi.CThostFtdcQryAccountregisterField_CurrencyID_get, _thosttraderapi.CThostFtdcQryAccountregisterField_CurrencyID_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcQryAccountregisterField_swiginit(self, _thosttraderapi.new_CThostFtdcQryAccountregisterField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcQryAccountregisterField

# Register CThostFtdcQryAccountregisterField in _thosttraderapi:
_thosttraderapi.CThostFtdcQryAccountregisterField_swigregister(CThostFtdcQryAccountregisterField)

class CThostFtdcAccountregisterField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    TradeDay = property(_thosttraderapi.CThostFtdcAccountregisterField_TradeDay_get, _thosttraderapi.CThostFtdcAccountregisterField_TradeDay_set)
    BankID = property(_thosttraderapi.CThostFtdcAccountregisterField_BankID_get, _thosttraderapi.CThostFtdcAccountregisterField_BankID_set)
    BankBranchID = property(_thosttraderapi.CThostFtdcAccountregisterField_BankBranchID_get, _thosttraderapi.CThostFtdcAccountregisterField_BankBranchID_set)
    BankAccount = property(_thosttraderapi.CThostFtdcAccountregisterField_BankAccount_get, _thosttraderapi.CThostFtdcAccountregisterField_BankAccount_set)
    BrokerID = property(_thosttraderapi.CThostFtdcAccountregisterField_BrokerID_get, _thosttraderapi.CThostFtdcAccountregisterField_BrokerID_set)
    BrokerBranchID = property(_thosttraderapi.CThostFtdcAccountregisterField_BrokerBranchID_get, _thosttraderapi.CThostFtdcAccountregisterField_BrokerBranchID_set)
    AccountID = property(_thosttraderapi.CThostFtdcAccountregisterField_AccountID_get, _thosttraderapi.CThostFtdcAccountregisterField_AccountID_set)
    IdCardType = property(_thosttraderapi.CThostFtdcAccountregisterField_IdCardType_get, _thosttraderapi.CThostFtdcAccountregisterField_IdCardType_set)
    IdentifiedCardNo = property(_thosttraderapi.CThostFtdcAccountregisterField_IdentifiedCardNo_get, _thosttraderapi.CThostFtdcAccountregisterField_IdentifiedCardNo_set)
    CustomerName = property(_thosttraderapi.CThostFtdcAccountregisterField_CustomerName_get, _thosttraderapi.CThostFtdcAccountregisterField_CustomerName_set)
    CurrencyID = property(_thosttraderapi.CThostFtdcAccountregisterField_CurrencyID_get, _thosttraderapi.CThostFtdcAccountregisterField_CurrencyID_set)
    OpenOrDestroy = property(_thosttraderapi.CThostFtdcAccountregisterField_OpenOrDestroy_get, _thosttraderapi.CThostFtdcAccountregisterField_OpenOrDestroy_set)
    RegDate = property(_thosttraderapi.CThostFtdcAccountregisterField_RegDate_get, _thosttraderapi.CThostFtdcAccountregisterField_RegDate_set)
    OutDate = property(_thosttraderapi.CThostFtdcAccountregisterField_OutDate_get, _thosttraderapi.CThostFtdcAccountregisterField_OutDate_set)
    TID = property(_thosttraderapi.CThostFtdcAccountregisterField_TID_get, _thosttraderapi.CThostFtdcAccountregisterField_TID_set)
    CustType = property(_thosttraderapi.CThostFtdcAccountregisterField_CustType_get, _thosttraderapi.CThostFtdcAccountregisterField_CustType_set)
    BankAccType = property(_thosttraderapi.CThostFtdcAccountregisterField_BankAccType_get, _thosttraderapi.CThostFtdcAccountregisterField_BankAccType_set)
    LongCustomerName = property(_thosttraderapi.CThostFtdcAccountregisterField_LongCustomerName_get, _thosttraderapi.CThostFtdcAccountregisterField_LongCustomerName_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcAccountregisterField_swiginit(self, _thosttraderapi.new_CThostFtdcAccountregisterField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcAccountregisterField

# Register CThostFtdcAccountregisterField in _thosttraderapi:
_thosttraderapi.CThostFtdcAccountregisterField_swigregister(CThostFtdcAccountregisterField)

class CThostFtdcOpenAccountField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    TradeCode = property(_thosttraderapi.CThostFtdcOpenAccountField_TradeCode_get, _thosttraderapi.CThostFtdcOpenAccountField_TradeCode_set)
    BankID = property(_thosttraderapi.CThostFtdcOpenAccountField_BankID_get, _thosttraderapi.CThostFtdcOpenAccountField_BankID_set)
    BankBranchID = property(_thosttraderapi.CThostFtdcOpenAccountField_BankBranchID_get, _thosttraderapi.CThostFtdcOpenAccountField_BankBranchID_set)
    BrokerID = property(_thosttraderapi.CThostFtdcOpenAccountField_BrokerID_get, _thosttraderapi.CThostFtdcOpenAccountField_BrokerID_set)
    BrokerBranchID = property(_thosttraderapi.CThostFtdcOpenAccountField_BrokerBranchID_get, _thosttraderapi.CThostFtdcOpenAccountField_BrokerBranchID_set)
    TradeDate = property(_thosttraderapi.CThostFtdcOpenAccountField_TradeDate_get, _thosttraderapi.CThostFtdcOpenAccountField_TradeDate_set)
    TradeTime = property(_thosttraderapi.CThostFtdcOpenAccountField_TradeTime_get, _thosttraderapi.CThostFtdcOpenAccountField_TradeTime_set)
    BankSerial = property(_thosttraderapi.CThostFtdcOpenAccountField_BankSerial_get, _thosttraderapi.CThostFtdcOpenAccountField_BankSerial_set)
    TradingDay = property(_thosttraderapi.CThostFtdcOpenAccountField_TradingDay_get, _thosttraderapi.CThostFtdcOpenAccountField_TradingDay_set)
    PlateSerial = property(_thosttraderapi.CThostFtdcOpenAccountField_PlateSerial_get, _thosttraderapi.CThostFtdcOpenAccountField_PlateSerial_set)
    LastFragment = property(_thosttraderapi.CThostFtdcOpenAccountField_LastFragment_get, _thosttraderapi.CThostFtdcOpenAccountField_LastFragment_set)
    SessionID = property(_thosttraderapi.CThostFtdcOpenAccountField_SessionID_get, _thosttraderapi.CThostFtdcOpenAccountField_SessionID_set)
    CustomerName = property(_thosttraderapi.CThostFtdcOpenAccountField_CustomerName_get, _thosttraderapi.CThostFtdcOpenAccountField_CustomerName_set)
    IdCardType = property(_thosttraderapi.CThostFtdcOpenAccountField_IdCardType_get, _thosttraderapi.CThostFtdcOpenAccountField_IdCardType_set)
    IdentifiedCardNo = property(_thosttraderapi.CThostFtdcOpenAccountField_IdentifiedCardNo_get, _thosttraderapi.CThostFtdcOpenAccountField_IdentifiedCardNo_set)
    Gender = property(_thosttraderapi.CThostFtdcOpenAccountField_Gender_get, _thosttraderapi.CThostFtdcOpenAccountField_Gender_set)
    CountryCode = property(_thosttraderapi.CThostFtdcOpenAccountField_CountryCode_get, _thosttraderapi.CThostFtdcOpenAccountField_CountryCode_set)
    CustType = property(_thosttraderapi.CThostFtdcOpenAccountField_CustType_get, _thosttraderapi.CThostFtdcOpenAccountField_CustType_set)
    Address = property(_thosttraderapi.CThostFtdcOpenAccountField_Address_get, _thosttraderapi.CThostFtdcOpenAccountField_Address_set)
    ZipCode = property(_thosttraderapi.CThostFtdcOpenAccountField_ZipCode_get, _thosttraderapi.CThostFtdcOpenAccountField_ZipCode_set)
    Telephone = property(_thosttraderapi.CThostFtdcOpenAccountField_Telephone_get, _thosttraderapi.CThostFtdcOpenAccountField_Telephone_set)
    MobilePhone = property(_thosttraderapi.CThostFtdcOpenAccountField_MobilePhone_get, _thosttraderapi.CThostFtdcOpenAccountField_MobilePhone_set)
    Fax = property(_thosttraderapi.CThostFtdcOpenAccountField_Fax_get, _thosttraderapi.CThostFtdcOpenAccountField_Fax_set)
    EMail = property(_thosttraderapi.CThostFtdcOpenAccountField_EMail_get, _thosttraderapi.CThostFtdcOpenAccountField_EMail_set)
    MoneyAccountStatus = property(_thosttraderapi.CThostFtdcOpenAccountField_MoneyAccountStatus_get, _thosttraderapi.CThostFtdcOpenAccountField_MoneyAccountStatus_set)
    BankAccount = property(_thosttraderapi.CThostFtdcOpenAccountField_BankAccount_get, _thosttraderapi.CThostFtdcOpenAccountField_BankAccount_set)
    BankPassWord = property(_thosttraderapi.CThostFtdcOpenAccountField_BankPassWord_get, _thosttraderapi.CThostFtdcOpenAccountField_BankPassWord_set)
    AccountID = property(_thosttraderapi.CThostFtdcOpenAccountField_AccountID_get, _thosttraderapi.CThostFtdcOpenAccountField_AccountID_set)
    Password = property(_thosttraderapi.CThostFtdcOpenAccountField_Password_get, _thosttraderapi.CThostFtdcOpenAccountField_Password_set)
    InstallID = property(_thosttraderapi.CThostFtdcOpenAccountField_InstallID_get, _thosttraderapi.CThostFtdcOpenAccountField_InstallID_set)
    VerifyCertNoFlag = property(_thosttraderapi.CThostFtdcOpenAccountField_VerifyCertNoFlag_get, _thosttraderapi.CThostFtdcOpenAccountField_VerifyCertNoFlag_set)
    CurrencyID = property(_thosttraderapi.CThostFtdcOpenAccountField_CurrencyID_get, _thosttraderapi.CThostFtdcOpenAccountField_CurrencyID_set)
    CashExchangeCode = property(_thosttraderapi.CThostFtdcOpenAccountField_CashExchangeCode_get, _thosttraderapi.CThostFtdcOpenAccountField_CashExchangeCode_set)
    Digest = property(_thosttraderapi.CThostFtdcOpenAccountField_Digest_get, _thosttraderapi.CThostFtdcOpenAccountField_Digest_set)
    BankAccType = property(_thosttraderapi.CThostFtdcOpenAccountField_BankAccType_get, _thosttraderapi.CThostFtdcOpenAccountField_BankAccType_set)
    DeviceID = property(_thosttraderapi.CThostFtdcOpenAccountField_DeviceID_get, _thosttraderapi.CThostFtdcOpenAccountField_DeviceID_set)
    BankSecuAccType = property(_thosttraderapi.CThostFtdcOpenAccountField_BankSecuAccType_get, _thosttraderapi.CThostFtdcOpenAccountField_BankSecuAccType_set)
    BrokerIDByBank = property(_thosttraderapi.CThostFtdcOpenAccountField_BrokerIDByBank_get, _thosttraderapi.CThostFtdcOpenAccountField_BrokerIDByBank_set)
    BankSecuAcc = property(_thosttraderapi.CThostFtdcOpenAccountField_BankSecuAcc_get, _thosttraderapi.CThostFtdcOpenAccountField_BankSecuAcc_set)
    BankPwdFlag = property(_thosttraderapi.CThostFtdcOpenAccountField_BankPwdFlag_get, _thosttraderapi.CThostFtdcOpenAccountField_BankPwdFlag_set)
    SecuPwdFlag = property(_thosttraderapi.CThostFtdcOpenAccountField_SecuPwdFlag_get, _thosttraderapi.CThostFtdcOpenAccountField_SecuPwdFlag_set)
    OperNo = property(_thosttraderapi.CThostFtdcOpenAccountField_OperNo_get, _thosttraderapi.CThostFtdcOpenAccountField_OperNo_set)
    TID = property(_thosttraderapi.CThostFtdcOpenAccountField_TID_get, _thosttraderapi.CThostFtdcOpenAccountField_TID_set)
    UserID = property(_thosttraderapi.CThostFtdcOpenAccountField_UserID_get, _thosttraderapi.CThostFtdcOpenAccountField_UserID_set)
    ErrorID = property(_thosttraderapi.CThostFtdcOpenAccountField_ErrorID_get, _thosttraderapi.CThostFtdcOpenAccountField_ErrorID_set)
    ErrorMsg = property(_thosttraderapi.CThostFtdcOpenAccountField_ErrorMsg_get, _thosttraderapi.CThostFtdcOpenAccountField_ErrorMsg_set)
    LongCustomerName = property(_thosttraderapi.CThostFtdcOpenAccountField_LongCustomerName_get, _thosttraderapi.CThostFtdcOpenAccountField_LongCustomerName_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcOpenAccountField_swiginit(self, _thosttraderapi.new_CThostFtdcOpenAccountField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcOpenAccountField

# Register CThostFtdcOpenAccountField in _thosttraderapi:
_thosttraderapi.CThostFtdcOpenAccountField_swigregister(CThostFtdcOpenAccountField)

class CThostFtdcCancelAccountField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    TradeCode = property(_thosttraderapi.CThostFtdcCancelAccountField_TradeCode_get, _thosttraderapi.CThostFtdcCancelAccountField_TradeCode_set)
    BankID = property(_thosttraderapi.CThostFtdcCancelAccountField_BankID_get, _thosttraderapi.CThostFtdcCancelAccountField_BankID_set)
    BankBranchID = property(_thosttraderapi.CThostFtdcCancelAccountField_BankBranchID_get, _thosttraderapi.CThostFtdcCancelAccountField_BankBranchID_set)
    BrokerID = property(_thosttraderapi.CThostFtdcCancelAccountField_BrokerID_get, _thosttraderapi.CThostFtdcCancelAccountField_BrokerID_set)
    BrokerBranchID = property(_thosttraderapi.CThostFtdcCancelAccountField_BrokerBranchID_get, _thosttraderapi.CThostFtdcCancelAccountField_BrokerBranchID_set)
    TradeDate = property(_thosttraderapi.CThostFtdcCancelAccountField_TradeDate_get, _thosttraderapi.CThostFtdcCancelAccountField_TradeDate_set)
    TradeTime = property(_thosttraderapi.CThostFtdcCancelAccountField_TradeTime_get, _thosttraderapi.CThostFtdcCancelAccountField_TradeTime_set)
    BankSerial = property(_thosttraderapi.CThostFtdcCancelAccountField_BankSerial_get, _thosttraderapi.CThostFtdcCancelAccountField_BankSerial_set)
    TradingDay = property(_thosttraderapi.CThostFtdcCancelAccountField_TradingDay_get, _thosttraderapi.CThostFtdcCancelAccountField_TradingDay_set)
    PlateSerial = property(_thosttraderapi.CThostFtdcCancelAccountField_PlateSerial_get, _thosttraderapi.CThostFtdcCancelAccountField_PlateSerial_set)
    LastFragment = property(_thosttraderapi.CThostFtdcCancelAccountField_LastFragment_get, _thosttraderapi.CThostFtdcCancelAccountField_LastFragment_set)
    SessionID = property(_thosttraderapi.CThostFtdcCancelAccountField_SessionID_get, _thosttraderapi.CThostFtdcCancelAccountField_SessionID_set)
    CustomerName = property(_thosttraderapi.CThostFtdcCancelAccountField_CustomerName_get, _thosttraderapi.CThostFtdcCancelAccountField_CustomerName_set)
    IdCardType = property(_thosttraderapi.CThostFtdcCancelAccountField_IdCardType_get, _thosttraderapi.CThostFtdcCancelAccountField_IdCardType_set)
    IdentifiedCardNo = property(_thosttraderapi.CThostFtdcCancelAccountField_IdentifiedCardNo_get, _thosttraderapi.CThostFtdcCancelAccountField_IdentifiedCardNo_set)
    Gender = property(_thosttraderapi.CThostFtdcCancelAccountField_Gender_get, _thosttraderapi.CThostFtdcCancelAccountField_Gender_set)
    CountryCode = property(_thosttraderapi.CThostFtdcCancelAccountField_CountryCode_get, _thosttraderapi.CThostFtdcCancelAccountField_CountryCode_set)
    CustType = property(_thosttraderapi.CThostFtdcCancelAccountField_CustType_get, _thosttraderapi.CThostFtdcCancelAccountField_CustType_set)
    Address = property(_thosttraderapi.CThostFtdcCancelAccountField_Address_get, _thosttraderapi.CThostFtdcCancelAccountField_Address_set)
    ZipCode = property(_thosttraderapi.CThostFtdcCancelAccountField_ZipCode_get, _thosttraderapi.CThostFtdcCancelAccountField_ZipCode_set)
    Telephone = property(_thosttraderapi.CThostFtdcCancelAccountField_Telephone_get, _thosttraderapi.CThostFtdcCancelAccountField_Telephone_set)
    MobilePhone = property(_thosttraderapi.CThostFtdcCancelAccountField_MobilePhone_get, _thosttraderapi.CThostFtdcCancelAccountField_MobilePhone_set)
    Fax = property(_thosttraderapi.CThostFtdcCancelAccountField_Fax_get, _thosttraderapi.CThostFtdcCancelAccountField_Fax_set)
    EMail = property(_thosttraderapi.CThostFtdcCancelAccountField_EMail_get, _thosttraderapi.CThostFtdcCancelAccountField_EMail_set)
    MoneyAccountStatus = property(_thosttraderapi.CThostFtdcCancelAccountField_MoneyAccountStatus_get, _thosttraderapi.CThostFtdcCancelAccountField_MoneyAccountStatus_set)
    BankAccount = property(_thosttraderapi.CThostFtdcCancelAccountField_BankAccount_get, _thosttraderapi.CThostFtdcCancelAccountField_BankAccount_set)
    BankPassWord = property(_thosttraderapi.CThostFtdcCancelAccountField_BankPassWord_get, _thosttraderapi.CThostFtdcCancelAccountField_BankPassWord_set)
    AccountID = property(_thosttraderapi.CThostFtdcCancelAccountField_AccountID_get, _thosttraderapi.CThostFtdcCancelAccountField_AccountID_set)
    Password = property(_thosttraderapi.CThostFtdcCancelAccountField_Password_get, _thosttraderapi.CThostFtdcCancelAccountField_Password_set)
    InstallID = property(_thosttraderapi.CThostFtdcCancelAccountField_InstallID_get, _thosttraderapi.CThostFtdcCancelAccountField_InstallID_set)
    VerifyCertNoFlag = property(_thosttraderapi.CThostFtdcCancelAccountField_VerifyCertNoFlag_get, _thosttraderapi.CThostFtdcCancelAccountField_VerifyCertNoFlag_set)
    CurrencyID = property(_thosttraderapi.CThostFtdcCancelAccountField_CurrencyID_get, _thosttraderapi.CThostFtdcCancelAccountField_CurrencyID_set)
    CashExchangeCode = property(_thosttraderapi.CThostFtdcCancelAccountField_CashExchangeCode_get, _thosttraderapi.CThostFtdcCancelAccountField_CashExchangeCode_set)
    Digest = property(_thosttraderapi.CThostFtdcCancelAccountField_Digest_get, _thosttraderapi.CThostFtdcCancelAccountField_Digest_set)
    BankAccType = property(_thosttraderapi.CThostFtdcCancelAccountField_BankAccType_get, _thosttraderapi.CThostFtdcCancelAccountField_BankAccType_set)
    DeviceID = property(_thosttraderapi.CThostFtdcCancelAccountField_DeviceID_get, _thosttraderapi.CThostFtdcCancelAccountField_DeviceID_set)
    BankSecuAccType = property(_thosttraderapi.CThostFtdcCancelAccountField_BankSecuAccType_get, _thosttraderapi.CThostFtdcCancelAccountField_BankSecuAccType_set)
    BrokerIDByBank = property(_thosttraderapi.CThostFtdcCancelAccountField_BrokerIDByBank_get, _thosttraderapi.CThostFtdcCancelAccountField_BrokerIDByBank_set)
    BankSecuAcc = property(_thosttraderapi.CThostFtdcCancelAccountField_BankSecuAcc_get, _thosttraderapi.CThostFtdcCancelAccountField_BankSecuAcc_set)
    BankPwdFlag = property(_thosttraderapi.CThostFtdcCancelAccountField_BankPwdFlag_get, _thosttraderapi.CThostFtdcCancelAccountField_BankPwdFlag_set)
    SecuPwdFlag = property(_thosttraderapi.CThostFtdcCancelAccountField_SecuPwdFlag_get, _thosttraderapi.CThostFtdcCancelAccountField_SecuPwdFlag_set)
    OperNo = property(_thosttraderapi.CThostFtdcCancelAccountField_OperNo_get, _thosttraderapi.CThostFtdcCancelAccountField_OperNo_set)
    TID = property(_thosttraderapi.CThostFtdcCancelAccountField_TID_get, _thosttraderapi.CThostFtdcCancelAccountField_TID_set)
    UserID = property(_thosttraderapi.CThostFtdcCancelAccountField_UserID_get, _thosttraderapi.CThostFtdcCancelAccountField_UserID_set)
    ErrorID = property(_thosttraderapi.CThostFtdcCancelAccountField_ErrorID_get, _thosttraderapi.CThostFtdcCancelAccountField_ErrorID_set)
    ErrorMsg = property(_thosttraderapi.CThostFtdcCancelAccountField_ErrorMsg_get, _thosttraderapi.CThostFtdcCancelAccountField_ErrorMsg_set)
    LongCustomerName = property(_thosttraderapi.CThostFtdcCancelAccountField_LongCustomerName_get, _thosttraderapi.CThostFtdcCancelAccountField_LongCustomerName_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcCancelAccountField_swiginit(self, _thosttraderapi.new_CThostFtdcCancelAccountField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcCancelAccountField

# Register CThostFtdcCancelAccountField in _thosttraderapi:
_thosttraderapi.CThostFtdcCancelAccountField_swigregister(CThostFtdcCancelAccountField)

class CThostFtdcChangeAccountField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    TradeCode = property(_thosttraderapi.CThostFtdcChangeAccountField_TradeCode_get, _thosttraderapi.CThostFtdcChangeAccountField_TradeCode_set)
    BankID = property(_thosttraderapi.CThostFtdcChangeAccountField_BankID_get, _thosttraderapi.CThostFtdcChangeAccountField_BankID_set)
    BankBranchID = property(_thosttraderapi.CThostFtdcChangeAccountField_BankBranchID_get, _thosttraderapi.CThostFtdcChangeAccountField_BankBranchID_set)
    BrokerID = property(_thosttraderapi.CThostFtdcChangeAccountField_BrokerID_get, _thosttraderapi.CThostFtdcChangeAccountField_BrokerID_set)
    BrokerBranchID = property(_thosttraderapi.CThostFtdcChangeAccountField_BrokerBranchID_get, _thosttraderapi.CThostFtdcChangeAccountField_BrokerBranchID_set)
    TradeDate = property(_thosttraderapi.CThostFtdcChangeAccountField_TradeDate_get, _thosttraderapi.CThostFtdcChangeAccountField_TradeDate_set)
    TradeTime = property(_thosttraderapi.CThostFtdcChangeAccountField_TradeTime_get, _thosttraderapi.CThostFtdcChangeAccountField_TradeTime_set)
    BankSerial = property(_thosttraderapi.CThostFtdcChangeAccountField_BankSerial_get, _thosttraderapi.CThostFtdcChangeAccountField_BankSerial_set)
    TradingDay = property(_thosttraderapi.CThostFtdcChangeAccountField_TradingDay_get, _thosttraderapi.CThostFtdcChangeAccountField_TradingDay_set)
    PlateSerial = property(_thosttraderapi.CThostFtdcChangeAccountField_PlateSerial_get, _thosttraderapi.CThostFtdcChangeAccountField_PlateSerial_set)
    LastFragment = property(_thosttraderapi.CThostFtdcChangeAccountField_LastFragment_get, _thosttraderapi.CThostFtdcChangeAccountField_LastFragment_set)
    SessionID = property(_thosttraderapi.CThostFtdcChangeAccountField_SessionID_get, _thosttraderapi.CThostFtdcChangeAccountField_SessionID_set)
    CustomerName = property(_thosttraderapi.CThostFtdcChangeAccountField_CustomerName_get, _thosttraderapi.CThostFtdcChangeAccountField_CustomerName_set)
    IdCardType = property(_thosttraderapi.CThostFtdcChangeAccountField_IdCardType_get, _thosttraderapi.CThostFtdcChangeAccountField_IdCardType_set)
    IdentifiedCardNo = property(_thosttraderapi.CThostFtdcChangeAccountField_IdentifiedCardNo_get, _thosttraderapi.CThostFtdcChangeAccountField_IdentifiedCardNo_set)
    Gender = property(_thosttraderapi.CThostFtdcChangeAccountField_Gender_get, _thosttraderapi.CThostFtdcChangeAccountField_Gender_set)
    CountryCode = property(_thosttraderapi.CThostFtdcChangeAccountField_CountryCode_get, _thosttraderapi.CThostFtdcChangeAccountField_CountryCode_set)
    CustType = property(_thosttraderapi.CThostFtdcChangeAccountField_CustType_get, _thosttraderapi.CThostFtdcChangeAccountField_CustType_set)
    Address = property(_thosttraderapi.CThostFtdcChangeAccountField_Address_get, _thosttraderapi.CThostFtdcChangeAccountField_Address_set)
    ZipCode = property(_thosttraderapi.CThostFtdcChangeAccountField_ZipCode_get, _thosttraderapi.CThostFtdcChangeAccountField_ZipCode_set)
    Telephone = property(_thosttraderapi.CThostFtdcChangeAccountField_Telephone_get, _thosttraderapi.CThostFtdcChangeAccountField_Telephone_set)
    MobilePhone = property(_thosttraderapi.CThostFtdcChangeAccountField_MobilePhone_get, _thosttraderapi.CThostFtdcChangeAccountField_MobilePhone_set)
    Fax = property(_thosttraderapi.CThostFtdcChangeAccountField_Fax_get, _thosttraderapi.CThostFtdcChangeAccountField_Fax_set)
    EMail = property(_thosttraderapi.CThostFtdcChangeAccountField_EMail_get, _thosttraderapi.CThostFtdcChangeAccountField_EMail_set)
    MoneyAccountStatus = property(_thosttraderapi.CThostFtdcChangeAccountField_MoneyAccountStatus_get, _thosttraderapi.CThostFtdcChangeAccountField_MoneyAccountStatus_set)
    BankAccount = property(_thosttraderapi.CThostFtdcChangeAccountField_BankAccount_get, _thosttraderapi.CThostFtdcChangeAccountField_BankAccount_set)
    BankPassWord = property(_thosttraderapi.CThostFtdcChangeAccountField_BankPassWord_get, _thosttraderapi.CThostFtdcChangeAccountField_BankPassWord_set)
    NewBankAccount = property(_thosttraderapi.CThostFtdcChangeAccountField_NewBankAccount_get, _thosttraderapi.CThostFtdcChangeAccountField_NewBankAccount_set)
    NewBankPassWord = property(_thosttraderapi.CThostFtdcChangeAccountField_NewBankPassWord_get, _thosttraderapi.CThostFtdcChangeAccountField_NewBankPassWord_set)
    AccountID = property(_thosttraderapi.CThostFtdcChangeAccountField_AccountID_get, _thosttraderapi.CThostFtdcChangeAccountField_AccountID_set)
    Password = property(_thosttraderapi.CThostFtdcChangeAccountField_Password_get, _thosttraderapi.CThostFtdcChangeAccountField_Password_set)
    BankAccType = property(_thosttraderapi.CThostFtdcChangeAccountField_BankAccType_get, _thosttraderapi.CThostFtdcChangeAccountField_BankAccType_set)
    InstallID = property(_thosttraderapi.CThostFtdcChangeAccountField_InstallID_get, _thosttraderapi.CThostFtdcChangeAccountField_InstallID_set)
    VerifyCertNoFlag = property(_thosttraderapi.CThostFtdcChangeAccountField_VerifyCertNoFlag_get, _thosttraderapi.CThostFtdcChangeAccountField_VerifyCertNoFlag_set)
    CurrencyID = property(_thosttraderapi.CThostFtdcChangeAccountField_CurrencyID_get, _thosttraderapi.CThostFtdcChangeAccountField_CurrencyID_set)
    BrokerIDByBank = property(_thosttraderapi.CThostFtdcChangeAccountField_BrokerIDByBank_get, _thosttraderapi.CThostFtdcChangeAccountField_BrokerIDByBank_set)
    BankPwdFlag = property(_thosttraderapi.CThostFtdcChangeAccountField_BankPwdFlag_get, _thosttraderapi.CThostFtdcChangeAccountField_BankPwdFlag_set)
    SecuPwdFlag = property(_thosttraderapi.CThostFtdcChangeAccountField_SecuPwdFlag_get, _thosttraderapi.CThostFtdcChangeAccountField_SecuPwdFlag_set)
    TID = property(_thosttraderapi.CThostFtdcChangeAccountField_TID_get, _thosttraderapi.CThostFtdcChangeAccountField_TID_set)
    Digest = property(_thosttraderapi.CThostFtdcChangeAccountField_Digest_get, _thosttraderapi.CThostFtdcChangeAccountField_Digest_set)
    ErrorID = property(_thosttraderapi.CThostFtdcChangeAccountField_ErrorID_get, _thosttraderapi.CThostFtdcChangeAccountField_ErrorID_set)
    ErrorMsg = property(_thosttraderapi.CThostFtdcChangeAccountField_ErrorMsg_get, _thosttraderapi.CThostFtdcChangeAccountField_ErrorMsg_set)
    LongCustomerName = property(_thosttraderapi.CThostFtdcChangeAccountField_LongCustomerName_get, _thosttraderapi.CThostFtdcChangeAccountField_LongCustomerName_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcChangeAccountField_swiginit(self, _thosttraderapi.new_CThostFtdcChangeAccountField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcChangeAccountField

# Register CThostFtdcChangeAccountField in _thosttraderapi:
_thosttraderapi.CThostFtdcChangeAccountField_swigregister(CThostFtdcChangeAccountField)

class CThostFtdcSecAgentACIDMapField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thosttraderapi.CThostFtdcSecAgentACIDMapField_BrokerID_get, _thosttraderapi.CThostFtdcSecAgentACIDMapField_BrokerID_set)
    UserID = property(_thosttraderapi.CThostFtdcSecAgentACIDMapField_UserID_get, _thosttraderapi.CThostFtdcSecAgentACIDMapField_UserID_set)
    AccountID = property(_thosttraderapi.CThostFtdcSecAgentACIDMapField_AccountID_get, _thosttraderapi.CThostFtdcSecAgentACIDMapField_AccountID_set)
    CurrencyID = property(_thosttraderapi.CThostFtdcSecAgentACIDMapField_CurrencyID_get, _thosttraderapi.CThostFtdcSecAgentACIDMapField_CurrencyID_set)
    BrokerSecAgentID = property(_thosttraderapi.CThostFtdcSecAgentACIDMapField_BrokerSecAgentID_get, _thosttraderapi.CThostFtdcSecAgentACIDMapField_BrokerSecAgentID_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcSecAgentACIDMapField_swiginit(self, _thosttraderapi.new_CThostFtdcSecAgentACIDMapField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcSecAgentACIDMapField

# Register CThostFtdcSecAgentACIDMapField in _thosttraderapi:
_thosttraderapi.CThostFtdcSecAgentACIDMapField_swigregister(CThostFtdcSecAgentACIDMapField)

class CThostFtdcQrySecAgentACIDMapField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thosttraderapi.CThostFtdcQrySecAgentACIDMapField_BrokerID_get, _thosttraderapi.CThostFtdcQrySecAgentACIDMapField_BrokerID_set)
    UserID = property(_thosttraderapi.CThostFtdcQrySecAgentACIDMapField_UserID_get, _thosttraderapi.CThostFtdcQrySecAgentACIDMapField_UserID_set)
    AccountID = property(_thosttraderapi.CThostFtdcQrySecAgentACIDMapField_AccountID_get, _thosttraderapi.CThostFtdcQrySecAgentACIDMapField_AccountID_set)
    CurrencyID = property(_thosttraderapi.CThostFtdcQrySecAgentACIDMapField_CurrencyID_get, _thosttraderapi.CThostFtdcQrySecAgentACIDMapField_CurrencyID_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcQrySecAgentACIDMapField_swiginit(self, _thosttraderapi.new_CThostFtdcQrySecAgentACIDMapField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcQrySecAgentACIDMapField

# Register CThostFtdcQrySecAgentACIDMapField in _thosttraderapi:
_thosttraderapi.CThostFtdcQrySecAgentACIDMapField_swigregister(CThostFtdcQrySecAgentACIDMapField)

class CThostFtdcUserRightsAssignField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thosttraderapi.CThostFtdcUserRightsAssignField_BrokerID_get, _thosttraderapi.CThostFtdcUserRightsAssignField_BrokerID_set)
    UserID = property(_thosttraderapi.CThostFtdcUserRightsAssignField_UserID_get, _thosttraderapi.CThostFtdcUserRightsAssignField_UserID_set)
    DRIdentityID = property(_thosttraderapi.CThostFtdcUserRightsAssignField_DRIdentityID_get, _thosttraderapi.CThostFtdcUserRightsAssignField_DRIdentityID_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcUserRightsAssignField_swiginit(self, _thosttraderapi.new_CThostFtdcUserRightsAssignField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcUserRightsAssignField

# Register CThostFtdcUserRightsAssignField in _thosttraderapi:
_thosttraderapi.CThostFtdcUserRightsAssignField_swigregister(CThostFtdcUserRightsAssignField)

class CThostFtdcBrokerUserRightAssignField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thosttraderapi.CThostFtdcBrokerUserRightAssignField_BrokerID_get, _thosttraderapi.CThostFtdcBrokerUserRightAssignField_BrokerID_set)
    DRIdentityID = property(_thosttraderapi.CThostFtdcBrokerUserRightAssignField_DRIdentityID_get, _thosttraderapi.CThostFtdcBrokerUserRightAssignField_DRIdentityID_set)
    Tradeable = property(_thosttraderapi.CThostFtdcBrokerUserRightAssignField_Tradeable_get, _thosttraderapi.CThostFtdcBrokerUserRightAssignField_Tradeable_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcBrokerUserRightAssignField_swiginit(self, _thosttraderapi.new_CThostFtdcBrokerUserRightAssignField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcBrokerUserRightAssignField

# Register CThostFtdcBrokerUserRightAssignField in _thosttraderapi:
_thosttraderapi.CThostFtdcBrokerUserRightAssignField_swigregister(CThostFtdcBrokerUserRightAssignField)

class CThostFtdcDRTransferField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    OrigDRIdentityID = property(_thosttraderapi.CThostFtdcDRTransferField_OrigDRIdentityID_get, _thosttraderapi.CThostFtdcDRTransferField_OrigDRIdentityID_set)
    DestDRIdentityID = property(_thosttraderapi.CThostFtdcDRTransferField_DestDRIdentityID_get, _thosttraderapi.CThostFtdcDRTransferField_DestDRIdentityID_set)
    OrigBrokerID = property(_thosttraderapi.CThostFtdcDRTransferField_OrigBrokerID_get, _thosttraderapi.CThostFtdcDRTransferField_OrigBrokerID_set)
    DestBrokerID = property(_thosttraderapi.CThostFtdcDRTransferField_DestBrokerID_get, _thosttraderapi.CThostFtdcDRTransferField_DestBrokerID_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcDRTransferField_swiginit(self, _thosttraderapi.new_CThostFtdcDRTransferField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcDRTransferField

# Register CThostFtdcDRTransferField in _thosttraderapi:
_thosttraderapi.CThostFtdcDRTransferField_swigregister(CThostFtdcDRTransferField)

class CThostFtdcFensUserInfoField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thosttraderapi.CThostFtdcFensUserInfoField_BrokerID_get, _thosttraderapi.CThostFtdcFensUserInfoField_BrokerID_set)
    UserID = property(_thosttraderapi.CThostFtdcFensUserInfoField_UserID_get, _thosttraderapi.CThostFtdcFensUserInfoField_UserID_set)
    LoginMode = property(_thosttraderapi.CThostFtdcFensUserInfoField_LoginMode_get, _thosttraderapi.CThostFtdcFensUserInfoField_LoginMode_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcFensUserInfoField_swiginit(self, _thosttraderapi.new_CThostFtdcFensUserInfoField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcFensUserInfoField

# Register CThostFtdcFensUserInfoField in _thosttraderapi:
_thosttraderapi.CThostFtdcFensUserInfoField_swigregister(CThostFtdcFensUserInfoField)

class CThostFtdcCurrTransferIdentityField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    IdentityID = property(_thosttraderapi.CThostFtdcCurrTransferIdentityField_IdentityID_get, _thosttraderapi.CThostFtdcCurrTransferIdentityField_IdentityID_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcCurrTransferIdentityField_swiginit(self, _thosttraderapi.new_CThostFtdcCurrTransferIdentityField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcCurrTransferIdentityField

# Register CThostFtdcCurrTransferIdentityField in _thosttraderapi:
_thosttraderapi.CThostFtdcCurrTransferIdentityField_swigregister(CThostFtdcCurrTransferIdentityField)

class CThostFtdcLoginForbiddenUserField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thosttraderapi.CThostFtdcLoginForbiddenUserField_BrokerID_get, _thosttraderapi.CThostFtdcLoginForbiddenUserField_BrokerID_set)
    UserID = property(_thosttraderapi.CThostFtdcLoginForbiddenUserField_UserID_get, _thosttraderapi.CThostFtdcLoginForbiddenUserField_UserID_set)
    IPAddress = property(_thosttraderapi.CThostFtdcLoginForbiddenUserField_IPAddress_get, _thosttraderapi.CThostFtdcLoginForbiddenUserField_IPAddress_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcLoginForbiddenUserField_swiginit(self, _thosttraderapi.new_CThostFtdcLoginForbiddenUserField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcLoginForbiddenUserField

# Register CThostFtdcLoginForbiddenUserField in _thosttraderapi:
_thosttraderapi.CThostFtdcLoginForbiddenUserField_swigregister(CThostFtdcLoginForbiddenUserField)

class CThostFtdcQryLoginForbiddenUserField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thosttraderapi.CThostFtdcQryLoginForbiddenUserField_BrokerID_get, _thosttraderapi.CThostFtdcQryLoginForbiddenUserField_BrokerID_set)
    UserID = property(_thosttraderapi.CThostFtdcQryLoginForbiddenUserField_UserID_get, _thosttraderapi.CThostFtdcQryLoginForbiddenUserField_UserID_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcQryLoginForbiddenUserField_swiginit(self, _thosttraderapi.new_CThostFtdcQryLoginForbiddenUserField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcQryLoginForbiddenUserField

# Register CThostFtdcQryLoginForbiddenUserField in _thosttraderapi:
_thosttraderapi.CThostFtdcQryLoginForbiddenUserField_swigregister(CThostFtdcQryLoginForbiddenUserField)

class CThostFtdcMulticastGroupInfoField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    GroupIP = property(_thosttraderapi.CThostFtdcMulticastGroupInfoField_GroupIP_get, _thosttraderapi.CThostFtdcMulticastGroupInfoField_GroupIP_set)
    GroupPort = property(_thosttraderapi.CThostFtdcMulticastGroupInfoField_GroupPort_get, _thosttraderapi.CThostFtdcMulticastGroupInfoField_GroupPort_set)
    SourceIP = property(_thosttraderapi.CThostFtdcMulticastGroupInfoField_SourceIP_get, _thosttraderapi.CThostFtdcMulticastGroupInfoField_SourceIP_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcMulticastGroupInfoField_swiginit(self, _thosttraderapi.new_CThostFtdcMulticastGroupInfoField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcMulticastGroupInfoField

# Register CThostFtdcMulticastGroupInfoField in _thosttraderapi:
_thosttraderapi.CThostFtdcMulticastGroupInfoField_swigregister(CThostFtdcMulticastGroupInfoField)

class CThostFtdcTradingAccountReserveField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thosttraderapi.CThostFtdcTradingAccountReserveField_BrokerID_get, _thosttraderapi.CThostFtdcTradingAccountReserveField_BrokerID_set)
    AccountID = property(_thosttraderapi.CThostFtdcTradingAccountReserveField_AccountID_get, _thosttraderapi.CThostFtdcTradingAccountReserveField_AccountID_set)
    Reserve = property(_thosttraderapi.CThostFtdcTradingAccountReserveField_Reserve_get, _thosttraderapi.CThostFtdcTradingAccountReserveField_Reserve_set)
    CurrencyID = property(_thosttraderapi.CThostFtdcTradingAccountReserveField_CurrencyID_get, _thosttraderapi.CThostFtdcTradingAccountReserveField_CurrencyID_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcTradingAccountReserveField_swiginit(self, _thosttraderapi.new_CThostFtdcTradingAccountReserveField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcTradingAccountReserveField

# Register CThostFtdcTradingAccountReserveField in _thosttraderapi:
_thosttraderapi.CThostFtdcTradingAccountReserveField_swigregister(CThostFtdcTradingAccountReserveField)

class CThostFtdcQryLoginForbiddenIPField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    IPAddress = property(_thosttraderapi.CThostFtdcQryLoginForbiddenIPField_IPAddress_get, _thosttraderapi.CThostFtdcQryLoginForbiddenIPField_IPAddress_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcQryLoginForbiddenIPField_swiginit(self, _thosttraderapi.new_CThostFtdcQryLoginForbiddenIPField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcQryLoginForbiddenIPField

# Register CThostFtdcQryLoginForbiddenIPField in _thosttraderapi:
_thosttraderapi.CThostFtdcQryLoginForbiddenIPField_swigregister(CThostFtdcQryLoginForbiddenIPField)

class CThostFtdcQryIPListField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    IPAddress = property(_thosttraderapi.CThostFtdcQryIPListField_IPAddress_get, _thosttraderapi.CThostFtdcQryIPListField_IPAddress_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcQryIPListField_swiginit(self, _thosttraderapi.new_CThostFtdcQryIPListField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcQryIPListField

# Register CThostFtdcQryIPListField in _thosttraderapi:
_thosttraderapi.CThostFtdcQryIPListField_swigregister(CThostFtdcQryIPListField)

class CThostFtdcQryUserRightsAssignField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thosttraderapi.CThostFtdcQryUserRightsAssignField_BrokerID_get, _thosttraderapi.CThostFtdcQryUserRightsAssignField_BrokerID_set)
    UserID = property(_thosttraderapi.CThostFtdcQryUserRightsAssignField_UserID_get, _thosttraderapi.CThostFtdcQryUserRightsAssignField_UserID_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcQryUserRightsAssignField_swiginit(self, _thosttraderapi.new_CThostFtdcQryUserRightsAssignField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcQryUserRightsAssignField

# Register CThostFtdcQryUserRightsAssignField in _thosttraderapi:
_thosttraderapi.CThostFtdcQryUserRightsAssignField_swigregister(CThostFtdcQryUserRightsAssignField)

class CThostFtdcReserveOpenAccountConfirmField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    TradeCode = property(_thosttraderapi.CThostFtdcReserveOpenAccountConfirmField_TradeCode_get, _thosttraderapi.CThostFtdcReserveOpenAccountConfirmField_TradeCode_set)
    BankID = property(_thosttraderapi.CThostFtdcReserveOpenAccountConfirmField_BankID_get, _thosttraderapi.CThostFtdcReserveOpenAccountConfirmField_BankID_set)
    BankBranchID = property(_thosttraderapi.CThostFtdcReserveOpenAccountConfirmField_BankBranchID_get, _thosttraderapi.CThostFtdcReserveOpenAccountConfirmField_BankBranchID_set)
    BrokerID = property(_thosttraderapi.CThostFtdcReserveOpenAccountConfirmField_BrokerID_get, _thosttraderapi.CThostFtdcReserveOpenAccountConfirmField_BrokerID_set)
    BrokerBranchID = property(_thosttraderapi.CThostFtdcReserveOpenAccountConfirmField_BrokerBranchID_get, _thosttraderapi.CThostFtdcReserveOpenAccountConfirmField_BrokerBranchID_set)
    TradeDate = property(_thosttraderapi.CThostFtdcReserveOpenAccountConfirmField_TradeDate_get, _thosttraderapi.CThostFtdcReserveOpenAccountConfirmField_TradeDate_set)
    TradeTime = property(_thosttraderapi.CThostFtdcReserveOpenAccountConfirmField_TradeTime_get, _thosttraderapi.CThostFtdcReserveOpenAccountConfirmField_TradeTime_set)
    BankSerial = property(_thosttraderapi.CThostFtdcReserveOpenAccountConfirmField_BankSerial_get, _thosttraderapi.CThostFtdcReserveOpenAccountConfirmField_BankSerial_set)
    TradingDay = property(_thosttraderapi.CThostFtdcReserveOpenAccountConfirmField_TradingDay_get, _thosttraderapi.CThostFtdcReserveOpenAccountConfirmField_TradingDay_set)
    PlateSerial = property(_thosttraderapi.CThostFtdcReserveOpenAccountConfirmField_PlateSerial_get, _thosttraderapi.CThostFtdcReserveOpenAccountConfirmField_PlateSerial_set)
    LastFragment = property(_thosttraderapi.CThostFtdcReserveOpenAccountConfirmField_LastFragment_get, _thosttraderapi.CThostFtdcReserveOpenAccountConfirmField_LastFragment_set)
    SessionID = property(_thosttraderapi.CThostFtdcReserveOpenAccountConfirmField_SessionID_get, _thosttraderapi.CThostFtdcReserveOpenAccountConfirmField_SessionID_set)
    CustomerName = property(_thosttraderapi.CThostFtdcReserveOpenAccountConfirmField_CustomerName_get, _thosttraderapi.CThostFtdcReserveOpenAccountConfirmField_CustomerName_set)
    IdCardType = property(_thosttraderapi.CThostFtdcReserveOpenAccountConfirmField_IdCardType_get, _thosttraderapi.CThostFtdcReserveOpenAccountConfirmField_IdCardType_set)
    IdentifiedCardNo = property(_thosttraderapi.CThostFtdcReserveOpenAccountConfirmField_IdentifiedCardNo_get, _thosttraderapi.CThostFtdcReserveOpenAccountConfirmField_IdentifiedCardNo_set)
    Gender = property(_thosttraderapi.CThostFtdcReserveOpenAccountConfirmField_Gender_get, _thosttraderapi.CThostFtdcReserveOpenAccountConfirmField_Gender_set)
    CountryCode = property(_thosttraderapi.CThostFtdcReserveOpenAccountConfirmField_CountryCode_get, _thosttraderapi.CThostFtdcReserveOpenAccountConfirmField_CountryCode_set)
    CustType = property(_thosttraderapi.CThostFtdcReserveOpenAccountConfirmField_CustType_get, _thosttraderapi.CThostFtdcReserveOpenAccountConfirmField_CustType_set)
    Address = property(_thosttraderapi.CThostFtdcReserveOpenAccountConfirmField_Address_get, _thosttraderapi.CThostFtdcReserveOpenAccountConfirmField_Address_set)
    ZipCode = property(_thosttraderapi.CThostFtdcReserveOpenAccountConfirmField_ZipCode_get, _thosttraderapi.CThostFtdcReserveOpenAccountConfirmField_ZipCode_set)
    Telephone = property(_thosttraderapi.CThostFtdcReserveOpenAccountConfirmField_Telephone_get, _thosttraderapi.CThostFtdcReserveOpenAccountConfirmField_Telephone_set)
    MobilePhone = property(_thosttraderapi.CThostFtdcReserveOpenAccountConfirmField_MobilePhone_get, _thosttraderapi.CThostFtdcReserveOpenAccountConfirmField_MobilePhone_set)
    Fax = property(_thosttraderapi.CThostFtdcReserveOpenAccountConfirmField_Fax_get, _thosttraderapi.CThostFtdcReserveOpenAccountConfirmField_Fax_set)
    EMail = property(_thosttraderapi.CThostFtdcReserveOpenAccountConfirmField_EMail_get, _thosttraderapi.CThostFtdcReserveOpenAccountConfirmField_EMail_set)
    MoneyAccountStatus = property(_thosttraderapi.CThostFtdcReserveOpenAccountConfirmField_MoneyAccountStatus_get, _thosttraderapi.CThostFtdcReserveOpenAccountConfirmField_MoneyAccountStatus_set)
    BankAccount = property(_thosttraderapi.CThostFtdcReserveOpenAccountConfirmField_BankAccount_get, _thosttraderapi.CThostFtdcReserveOpenAccountConfirmField_BankAccount_set)
    BankPassWord = property(_thosttraderapi.CThostFtdcReserveOpenAccountConfirmField_BankPassWord_get, _thosttraderapi.CThostFtdcReserveOpenAccountConfirmField_BankPassWord_set)
    InstallID = property(_thosttraderapi.CThostFtdcReserveOpenAccountConfirmField_InstallID_get, _thosttraderapi.CThostFtdcReserveOpenAccountConfirmField_InstallID_set)
    VerifyCertNoFlag = property(_thosttraderapi.CThostFtdcReserveOpenAccountConfirmField_VerifyCertNoFlag_get, _thosttraderapi.CThostFtdcReserveOpenAccountConfirmField_VerifyCertNoFlag_set)
    CurrencyID = property(_thosttraderapi.CThostFtdcReserveOpenAccountConfirmField_CurrencyID_get, _thosttraderapi.CThostFtdcReserveOpenAccountConfirmField_CurrencyID_set)
    Digest = property(_thosttraderapi.CThostFtdcReserveOpenAccountConfirmField_Digest_get, _thosttraderapi.CThostFtdcReserveOpenAccountConfirmField_Digest_set)
    BankAccType = property(_thosttraderapi.CThostFtdcReserveOpenAccountConfirmField_BankAccType_get, _thosttraderapi.CThostFtdcReserveOpenAccountConfirmField_BankAccType_set)
    BrokerIDByBank = property(_thosttraderapi.CThostFtdcReserveOpenAccountConfirmField_BrokerIDByBank_get, _thosttraderapi.CThostFtdcReserveOpenAccountConfirmField_BrokerIDByBank_set)
    TID = property(_thosttraderapi.CThostFtdcReserveOpenAccountConfirmField_TID_get, _thosttraderapi.CThostFtdcReserveOpenAccountConfirmField_TID_set)
    AccountID = property(_thosttraderapi.CThostFtdcReserveOpenAccountConfirmField_AccountID_get, _thosttraderapi.CThostFtdcReserveOpenAccountConfirmField_AccountID_set)
    Password = property(_thosttraderapi.CThostFtdcReserveOpenAccountConfirmField_Password_get, _thosttraderapi.CThostFtdcReserveOpenAccountConfirmField_Password_set)
    BankReserveOpenSeq = property(_thosttraderapi.CThostFtdcReserveOpenAccountConfirmField_BankReserveOpenSeq_get, _thosttraderapi.CThostFtdcReserveOpenAccountConfirmField_BankReserveOpenSeq_set)
    BookDate = property(_thosttraderapi.CThostFtdcReserveOpenAccountConfirmField_BookDate_get, _thosttraderapi.CThostFtdcReserveOpenAccountConfirmField_BookDate_set)
    BookPsw = property(_thosttraderapi.CThostFtdcReserveOpenAccountConfirmField_BookPsw_get, _thosttraderapi.CThostFtdcReserveOpenAccountConfirmField_BookPsw_set)
    ErrorID = property(_thosttraderapi.CThostFtdcReserveOpenAccountConfirmField_ErrorID_get, _thosttraderapi.CThostFtdcReserveOpenAccountConfirmField_ErrorID_set)
    ErrorMsg = property(_thosttraderapi.CThostFtdcReserveOpenAccountConfirmField_ErrorMsg_get, _thosttraderapi.CThostFtdcReserveOpenAccountConfirmField_ErrorMsg_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcReserveOpenAccountConfirmField_swiginit(self, _thosttraderapi.new_CThostFtdcReserveOpenAccountConfirmField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcReserveOpenAccountConfirmField

# Register CThostFtdcReserveOpenAccountConfirmField in _thosttraderapi:
_thosttraderapi.CThostFtdcReserveOpenAccountConfirmField_swigregister(CThostFtdcReserveOpenAccountConfirmField)

class CThostFtdcReserveOpenAccountField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    TradeCode = property(_thosttraderapi.CThostFtdcReserveOpenAccountField_TradeCode_get, _thosttraderapi.CThostFtdcReserveOpenAccountField_TradeCode_set)
    BankID = property(_thosttraderapi.CThostFtdcReserveOpenAccountField_BankID_get, _thosttraderapi.CThostFtdcReserveOpenAccountField_BankID_set)
    BankBranchID = property(_thosttraderapi.CThostFtdcReserveOpenAccountField_BankBranchID_get, _thosttraderapi.CThostFtdcReserveOpenAccountField_BankBranchID_set)
    BrokerID = property(_thosttraderapi.CThostFtdcReserveOpenAccountField_BrokerID_get, _thosttraderapi.CThostFtdcReserveOpenAccountField_BrokerID_set)
    BrokerBranchID = property(_thosttraderapi.CThostFtdcReserveOpenAccountField_BrokerBranchID_get, _thosttraderapi.CThostFtdcReserveOpenAccountField_BrokerBranchID_set)
    TradeDate = property(_thosttraderapi.CThostFtdcReserveOpenAccountField_TradeDate_get, _thosttraderapi.CThostFtdcReserveOpenAccountField_TradeDate_set)
    TradeTime = property(_thosttraderapi.CThostFtdcReserveOpenAccountField_TradeTime_get, _thosttraderapi.CThostFtdcReserveOpenAccountField_TradeTime_set)
    BankSerial = property(_thosttraderapi.CThostFtdcReserveOpenAccountField_BankSerial_get, _thosttraderapi.CThostFtdcReserveOpenAccountField_BankSerial_set)
    TradingDay = property(_thosttraderapi.CThostFtdcReserveOpenAccountField_TradingDay_get, _thosttraderapi.CThostFtdcReserveOpenAccountField_TradingDay_set)
    PlateSerial = property(_thosttraderapi.CThostFtdcReserveOpenAccountField_PlateSerial_get, _thosttraderapi.CThostFtdcReserveOpenAccountField_PlateSerial_set)
    LastFragment = property(_thosttraderapi.CThostFtdcReserveOpenAccountField_LastFragment_get, _thosttraderapi.CThostFtdcReserveOpenAccountField_LastFragment_set)
    SessionID = property(_thosttraderapi.CThostFtdcReserveOpenAccountField_SessionID_get, _thosttraderapi.CThostFtdcReserveOpenAccountField_SessionID_set)
    CustomerName = property(_thosttraderapi.CThostFtdcReserveOpenAccountField_CustomerName_get, _thosttraderapi.CThostFtdcReserveOpenAccountField_CustomerName_set)
    IdCardType = property(_thosttraderapi.CThostFtdcReserveOpenAccountField_IdCardType_get, _thosttraderapi.CThostFtdcReserveOpenAccountField_IdCardType_set)
    IdentifiedCardNo = property(_thosttraderapi.CThostFtdcReserveOpenAccountField_IdentifiedCardNo_get, _thosttraderapi.CThostFtdcReserveOpenAccountField_IdentifiedCardNo_set)
    Gender = property(_thosttraderapi.CThostFtdcReserveOpenAccountField_Gender_get, _thosttraderapi.CThostFtdcReserveOpenAccountField_Gender_set)
    CountryCode = property(_thosttraderapi.CThostFtdcReserveOpenAccountField_CountryCode_get, _thosttraderapi.CThostFtdcReserveOpenAccountField_CountryCode_set)
    CustType = property(_thosttraderapi.CThostFtdcReserveOpenAccountField_CustType_get, _thosttraderapi.CThostFtdcReserveOpenAccountField_CustType_set)
    Address = property(_thosttraderapi.CThostFtdcReserveOpenAccountField_Address_get, _thosttraderapi.CThostFtdcReserveOpenAccountField_Address_set)
    ZipCode = property(_thosttraderapi.CThostFtdcReserveOpenAccountField_ZipCode_get, _thosttraderapi.CThostFtdcReserveOpenAccountField_ZipCode_set)
    Telephone = property(_thosttraderapi.CThostFtdcReserveOpenAccountField_Telephone_get, _thosttraderapi.CThostFtdcReserveOpenAccountField_Telephone_set)
    MobilePhone = property(_thosttraderapi.CThostFtdcReserveOpenAccountField_MobilePhone_get, _thosttraderapi.CThostFtdcReserveOpenAccountField_MobilePhone_set)
    Fax = property(_thosttraderapi.CThostFtdcReserveOpenAccountField_Fax_get, _thosttraderapi.CThostFtdcReserveOpenAccountField_Fax_set)
    EMail = property(_thosttraderapi.CThostFtdcReserveOpenAccountField_EMail_get, _thosttraderapi.CThostFtdcReserveOpenAccountField_EMail_set)
    MoneyAccountStatus = property(_thosttraderapi.CThostFtdcReserveOpenAccountField_MoneyAccountStatus_get, _thosttraderapi.CThostFtdcReserveOpenAccountField_MoneyAccountStatus_set)
    BankAccount = property(_thosttraderapi.CThostFtdcReserveOpenAccountField_BankAccount_get, _thosttraderapi.CThostFtdcReserveOpenAccountField_BankAccount_set)
    BankPassWord = property(_thosttraderapi.CThostFtdcReserveOpenAccountField_BankPassWord_get, _thosttraderapi.CThostFtdcReserveOpenAccountField_BankPassWord_set)
    InstallID = property(_thosttraderapi.CThostFtdcReserveOpenAccountField_InstallID_get, _thosttraderapi.CThostFtdcReserveOpenAccountField_InstallID_set)
    VerifyCertNoFlag = property(_thosttraderapi.CThostFtdcReserveOpenAccountField_VerifyCertNoFlag_get, _thosttraderapi.CThostFtdcReserveOpenAccountField_VerifyCertNoFlag_set)
    CurrencyID = property(_thosttraderapi.CThostFtdcReserveOpenAccountField_CurrencyID_get, _thosttraderapi.CThostFtdcReserveOpenAccountField_CurrencyID_set)
    Digest = property(_thosttraderapi.CThostFtdcReserveOpenAccountField_Digest_get, _thosttraderapi.CThostFtdcReserveOpenAccountField_Digest_set)
    BankAccType = property(_thosttraderapi.CThostFtdcReserveOpenAccountField_BankAccType_get, _thosttraderapi.CThostFtdcReserveOpenAccountField_BankAccType_set)
    BrokerIDByBank = property(_thosttraderapi.CThostFtdcReserveOpenAccountField_BrokerIDByBank_get, _thosttraderapi.CThostFtdcReserveOpenAccountField_BrokerIDByBank_set)
    TID = property(_thosttraderapi.CThostFtdcReserveOpenAccountField_TID_get, _thosttraderapi.CThostFtdcReserveOpenAccountField_TID_set)
    ReserveOpenAccStas = property(_thosttraderapi.CThostFtdcReserveOpenAccountField_ReserveOpenAccStas_get, _thosttraderapi.CThostFtdcReserveOpenAccountField_ReserveOpenAccStas_set)
    ErrorID = property(_thosttraderapi.CThostFtdcReserveOpenAccountField_ErrorID_get, _thosttraderapi.CThostFtdcReserveOpenAccountField_ErrorID_set)
    ErrorMsg = property(_thosttraderapi.CThostFtdcReserveOpenAccountField_ErrorMsg_get, _thosttraderapi.CThostFtdcReserveOpenAccountField_ErrorMsg_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcReserveOpenAccountField_swiginit(self, _thosttraderapi.new_CThostFtdcReserveOpenAccountField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcReserveOpenAccountField

# Register CThostFtdcReserveOpenAccountField in _thosttraderapi:
_thosttraderapi.CThostFtdcReserveOpenAccountField_swigregister(CThostFtdcReserveOpenAccountField)

class CThostFtdcAccountPropertyField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thosttraderapi.CThostFtdcAccountPropertyField_BrokerID_get, _thosttraderapi.CThostFtdcAccountPropertyField_BrokerID_set)
    AccountID = property(_thosttraderapi.CThostFtdcAccountPropertyField_AccountID_get, _thosttraderapi.CThostFtdcAccountPropertyField_AccountID_set)
    BankID = property(_thosttraderapi.CThostFtdcAccountPropertyField_BankID_get, _thosttraderapi.CThostFtdcAccountPropertyField_BankID_set)
    BankAccount = property(_thosttraderapi.CThostFtdcAccountPropertyField_BankAccount_get, _thosttraderapi.CThostFtdcAccountPropertyField_BankAccount_set)
    OpenName = property(_thosttraderapi.CThostFtdcAccountPropertyField_OpenName_get, _thosttraderapi.CThostFtdcAccountPropertyField_OpenName_set)
    OpenBank = property(_thosttraderapi.CThostFtdcAccountPropertyField_OpenBank_get, _thosttraderapi.CThostFtdcAccountPropertyField_OpenBank_set)
    IsActive = property(_thosttraderapi.CThostFtdcAccountPropertyField_IsActive_get, _thosttraderapi.CThostFtdcAccountPropertyField_IsActive_set)
    AccountSourceType = property(_thosttraderapi.CThostFtdcAccountPropertyField_AccountSourceType_get, _thosttraderapi.CThostFtdcAccountPropertyField_AccountSourceType_set)
    OpenDate = property(_thosttraderapi.CThostFtdcAccountPropertyField_OpenDate_get, _thosttraderapi.CThostFtdcAccountPropertyField_OpenDate_set)
    CancelDate = property(_thosttraderapi.CThostFtdcAccountPropertyField_CancelDate_get, _thosttraderapi.CThostFtdcAccountPropertyField_CancelDate_set)
    OperatorID = property(_thosttraderapi.CThostFtdcAccountPropertyField_OperatorID_get, _thosttraderapi.CThostFtdcAccountPropertyField_OperatorID_set)
    OperateDate = property(_thosttraderapi.CThostFtdcAccountPropertyField_OperateDate_get, _thosttraderapi.CThostFtdcAccountPropertyField_OperateDate_set)
    OperateTime = property(_thosttraderapi.CThostFtdcAccountPropertyField_OperateTime_get, _thosttraderapi.CThostFtdcAccountPropertyField_OperateTime_set)
    CurrencyID = property(_thosttraderapi.CThostFtdcAccountPropertyField_CurrencyID_get, _thosttraderapi.CThostFtdcAccountPropertyField_CurrencyID_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcAccountPropertyField_swiginit(self, _thosttraderapi.new_CThostFtdcAccountPropertyField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcAccountPropertyField

# Register CThostFtdcAccountPropertyField in _thosttraderapi:
_thosttraderapi.CThostFtdcAccountPropertyField_swigregister(CThostFtdcAccountPropertyField)

class CThostFtdcQryCurrDRIdentityField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    DRIdentityID = property(_thosttraderapi.CThostFtdcQryCurrDRIdentityField_DRIdentityID_get, _thosttraderapi.CThostFtdcQryCurrDRIdentityField_DRIdentityID_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcQryCurrDRIdentityField_swiginit(self, _thosttraderapi.new_CThostFtdcQryCurrDRIdentityField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcQryCurrDRIdentityField

# Register CThostFtdcQryCurrDRIdentityField in _thosttraderapi:
_thosttraderapi.CThostFtdcQryCurrDRIdentityField_swigregister(CThostFtdcQryCurrDRIdentityField)

class CThostFtdcCurrDRIdentityField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    DRIdentityID = property(_thosttraderapi.CThostFtdcCurrDRIdentityField_DRIdentityID_get, _thosttraderapi.CThostFtdcCurrDRIdentityField_DRIdentityID_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcCurrDRIdentityField_swiginit(self, _thosttraderapi.new_CThostFtdcCurrDRIdentityField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcCurrDRIdentityField

# Register CThostFtdcCurrDRIdentityField in _thosttraderapi:
_thosttraderapi.CThostFtdcCurrDRIdentityField_swigregister(CThostFtdcCurrDRIdentityField)

class CThostFtdcQrySecAgentCheckModeField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thosttraderapi.CThostFtdcQrySecAgentCheckModeField_BrokerID_get, _thosttraderapi.CThostFtdcQrySecAgentCheckModeField_BrokerID_set)
    InvestorID = property(_thosttraderapi.CThostFtdcQrySecAgentCheckModeField_InvestorID_get, _thosttraderapi.CThostFtdcQrySecAgentCheckModeField_InvestorID_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcQrySecAgentCheckModeField_swiginit(self, _thosttraderapi.new_CThostFtdcQrySecAgentCheckModeField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcQrySecAgentCheckModeField

# Register CThostFtdcQrySecAgentCheckModeField in _thosttraderapi:
_thosttraderapi.CThostFtdcQrySecAgentCheckModeField_swigregister(CThostFtdcQrySecAgentCheckModeField)

class CThostFtdcQrySecAgentTradeInfoField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thosttraderapi.CThostFtdcQrySecAgentTradeInfoField_BrokerID_get, _thosttraderapi.CThostFtdcQrySecAgentTradeInfoField_BrokerID_set)
    BrokerSecAgentID = property(_thosttraderapi.CThostFtdcQrySecAgentTradeInfoField_BrokerSecAgentID_get, _thosttraderapi.CThostFtdcQrySecAgentTradeInfoField_BrokerSecAgentID_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcQrySecAgentTradeInfoField_swiginit(self, _thosttraderapi.new_CThostFtdcQrySecAgentTradeInfoField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcQrySecAgentTradeInfoField

# Register CThostFtdcQrySecAgentTradeInfoField in _thosttraderapi:
_thosttraderapi.CThostFtdcQrySecAgentTradeInfoField_swigregister(CThostFtdcQrySecAgentTradeInfoField)

class CThostFtdcUserSystemInfoField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thosttraderapi.CThostFtdcUserSystemInfoField_BrokerID_get, _thosttraderapi.CThostFtdcUserSystemInfoField_BrokerID_set)
    UserID = property(_thosttraderapi.CThostFtdcUserSystemInfoField_UserID_get, _thosttraderapi.CThostFtdcUserSystemInfoField_UserID_set)
    ClientSystemInfoLen = property(_thosttraderapi.CThostFtdcUserSystemInfoField_ClientSystemInfoLen_get, _thosttraderapi.CThostFtdcUserSystemInfoField_ClientSystemInfoLen_set)
    ClientSystemInfo = property(_thosttraderapi.CThostFtdcUserSystemInfoField_ClientSystemInfo_get, _thosttraderapi.CThostFtdcUserSystemInfoField_ClientSystemInfo_set)
    ClientPublicIP = property(_thosttraderapi.CThostFtdcUserSystemInfoField_ClientPublicIP_get, _thosttraderapi.CThostFtdcUserSystemInfoField_ClientPublicIP_set)
    ClientIPPort = property(_thosttraderapi.CThostFtdcUserSystemInfoField_ClientIPPort_get, _thosttraderapi.CThostFtdcUserSystemInfoField_ClientIPPort_set)
    ClientLoginTime = property(_thosttraderapi.CThostFtdcUserSystemInfoField_ClientLoginTime_get, _thosttraderapi.CThostFtdcUserSystemInfoField_ClientLoginTime_set)
    ClientAppID = property(_thosttraderapi.CThostFtdcUserSystemInfoField_ClientAppID_get, _thosttraderapi.CThostFtdcUserSystemInfoField_ClientAppID_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcUserSystemInfoField_swiginit(self, _thosttraderapi.new_CThostFtdcUserSystemInfoField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcUserSystemInfoField

# Register CThostFtdcUserSystemInfoField in _thosttraderapi:
_thosttraderapi.CThostFtdcUserSystemInfoField_swigregister(CThostFtdcUserSystemInfoField)

class CThostFtdcReqUserAuthMethodField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    TradingDay = property(_thosttraderapi.CThostFtdcReqUserAuthMethodField_TradingDay_get, _thosttraderapi.CThostFtdcReqUserAuthMethodField_TradingDay_set)
    BrokerID = property(_thosttraderapi.CThostFtdcReqUserAuthMethodField_BrokerID_get, _thosttraderapi.CThostFtdcReqUserAuthMethodField_BrokerID_set)
    UserID = property(_thosttraderapi.CThostFtdcReqUserAuthMethodField_UserID_get, _thosttraderapi.CThostFtdcReqUserAuthMethodField_UserID_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcReqUserAuthMethodField_swiginit(self, _thosttraderapi.new_CThostFtdcReqUserAuthMethodField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcReqUserAuthMethodField

# Register CThostFtdcReqUserAuthMethodField in _thosttraderapi:
_thosttraderapi.CThostFtdcReqUserAuthMethodField_swigregister(CThostFtdcReqUserAuthMethodField)

class CThostFtdcRspUserAuthMethodField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    UsableAuthMethod = property(_thosttraderapi.CThostFtdcRspUserAuthMethodField_UsableAuthMethod_get, _thosttraderapi.CThostFtdcRspUserAuthMethodField_UsableAuthMethod_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcRspUserAuthMethodField_swiginit(self, _thosttraderapi.new_CThostFtdcRspUserAuthMethodField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcRspUserAuthMethodField

# Register CThostFtdcRspUserAuthMethodField in _thosttraderapi:
_thosttraderapi.CThostFtdcRspUserAuthMethodField_swigregister(CThostFtdcRspUserAuthMethodField)

class CThostFtdcReqGenUserCaptchaField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    TradingDay = property(_thosttraderapi.CThostFtdcReqGenUserCaptchaField_TradingDay_get, _thosttraderapi.CThostFtdcReqGenUserCaptchaField_TradingDay_set)
    BrokerID = property(_thosttraderapi.CThostFtdcReqGenUserCaptchaField_BrokerID_get, _thosttraderapi.CThostFtdcReqGenUserCaptchaField_BrokerID_set)
    UserID = property(_thosttraderapi.CThostFtdcReqGenUserCaptchaField_UserID_get, _thosttraderapi.CThostFtdcReqGenUserCaptchaField_UserID_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcReqGenUserCaptchaField_swiginit(self, _thosttraderapi.new_CThostFtdcReqGenUserCaptchaField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcReqGenUserCaptchaField

# Register CThostFtdcReqGenUserCaptchaField in _thosttraderapi:
_thosttraderapi.CThostFtdcReqGenUserCaptchaField_swigregister(CThostFtdcReqGenUserCaptchaField)

class CThostFtdcRspGenUserCaptchaField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thosttraderapi.CThostFtdcRspGenUserCaptchaField_BrokerID_get, _thosttraderapi.CThostFtdcRspGenUserCaptchaField_BrokerID_set)
    UserID = property(_thosttraderapi.CThostFtdcRspGenUserCaptchaField_UserID_get, _thosttraderapi.CThostFtdcRspGenUserCaptchaField_UserID_set)
    CaptchaInfoLen = property(_thosttraderapi.CThostFtdcRspGenUserCaptchaField_CaptchaInfoLen_get, _thosttraderapi.CThostFtdcRspGenUserCaptchaField_CaptchaInfoLen_set)
    CaptchaInfo = property(_thosttraderapi.CThostFtdcRspGenUserCaptchaField_CaptchaInfo_get, _thosttraderapi.CThostFtdcRspGenUserCaptchaField_CaptchaInfo_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcRspGenUserCaptchaField_swiginit(self, _thosttraderapi.new_CThostFtdcRspGenUserCaptchaField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcRspGenUserCaptchaField

# Register CThostFtdcRspGenUserCaptchaField in _thosttraderapi:
_thosttraderapi.CThostFtdcRspGenUserCaptchaField_swigregister(CThostFtdcRspGenUserCaptchaField)

class CThostFtdcReqGenUserTextField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    TradingDay = property(_thosttraderapi.CThostFtdcReqGenUserTextField_TradingDay_get, _thosttraderapi.CThostFtdcReqGenUserTextField_TradingDay_set)
    BrokerID = property(_thosttraderapi.CThostFtdcReqGenUserTextField_BrokerID_get, _thosttraderapi.CThostFtdcReqGenUserTextField_BrokerID_set)
    UserID = property(_thosttraderapi.CThostFtdcReqGenUserTextField_UserID_get, _thosttraderapi.CThostFtdcReqGenUserTextField_UserID_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcReqGenUserTextField_swiginit(self, _thosttraderapi.new_CThostFtdcReqGenUserTextField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcReqGenUserTextField

# Register CThostFtdcReqGenUserTextField in _thosttraderapi:
_thosttraderapi.CThostFtdcReqGenUserTextField_swigregister(CThostFtdcReqGenUserTextField)

class CThostFtdcRspGenUserTextField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    UserTextSeq = property(_thosttraderapi.CThostFtdcRspGenUserTextField_UserTextSeq_get, _thosttraderapi.CThostFtdcRspGenUserTextField_UserTextSeq_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcRspGenUserTextField_swiginit(self, _thosttraderapi.new_CThostFtdcRspGenUserTextField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcRspGenUserTextField

# Register CThostFtdcRspGenUserTextField in _thosttraderapi:
_thosttraderapi.CThostFtdcRspGenUserTextField_swigregister(CThostFtdcRspGenUserTextField)

class CThostFtdcReqUserLoginWithCaptchaField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    TradingDay = property(_thosttraderapi.CThostFtdcReqUserLoginWithCaptchaField_TradingDay_get, _thosttraderapi.CThostFtdcReqUserLoginWithCaptchaField_TradingDay_set)
    BrokerID = property(_thosttraderapi.CThostFtdcReqUserLoginWithCaptchaField_BrokerID_get, _thosttraderapi.CThostFtdcReqUserLoginWithCaptchaField_BrokerID_set)
    UserID = property(_thosttraderapi.CThostFtdcReqUserLoginWithCaptchaField_UserID_get, _thosttraderapi.CThostFtdcReqUserLoginWithCaptchaField_UserID_set)
    Password = property(_thosttraderapi.CThostFtdcReqUserLoginWithCaptchaField_Password_get, _thosttraderapi.CThostFtdcReqUserLoginWithCaptchaField_Password_set)
    UserProductInfo = property(_thosttraderapi.CThostFtdcReqUserLoginWithCaptchaField_UserProductInfo_get, _thosttraderapi.CThostFtdcReqUserLoginWithCaptchaField_UserProductInfo_set)
    InterfaceProductInfo = property(_thosttraderapi.CThostFtdcReqUserLoginWithCaptchaField_InterfaceProductInfo_get, _thosttraderapi.CThostFtdcReqUserLoginWithCaptchaField_InterfaceProductInfo_set)
    ProtocolInfo = property(_thosttraderapi.CThostFtdcReqUserLoginWithCaptchaField_ProtocolInfo_get, _thosttraderapi.CThostFtdcReqUserLoginWithCaptchaField_ProtocolInfo_set)
    MacAddress = property(_thosttraderapi.CThostFtdcReqUserLoginWithCaptchaField_MacAddress_get, _thosttraderapi.CThostFtdcReqUserLoginWithCaptchaField_MacAddress_set)
    ClientIPAddress = property(_thosttraderapi.CThostFtdcReqUserLoginWithCaptchaField_ClientIPAddress_get, _thosttraderapi.CThostFtdcReqUserLoginWithCaptchaField_ClientIPAddress_set)
    LoginRemark = property(_thosttraderapi.CThostFtdcReqUserLoginWithCaptchaField_LoginRemark_get, _thosttraderapi.CThostFtdcReqUserLoginWithCaptchaField_LoginRemark_set)
    Captcha = property(_thosttraderapi.CThostFtdcReqUserLoginWithCaptchaField_Captcha_get, _thosttraderapi.CThostFtdcReqUserLoginWithCaptchaField_Captcha_set)
    ClientIPPort = property(_thosttraderapi.CThostFtdcReqUserLoginWithCaptchaField_ClientIPPort_get, _thosttraderapi.CThostFtdcReqUserLoginWithCaptchaField_ClientIPPort_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcReqUserLoginWithCaptchaField_swiginit(self, _thosttraderapi.new_CThostFtdcReqUserLoginWithCaptchaField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcReqUserLoginWithCaptchaField

# Register CThostFtdcReqUserLoginWithCaptchaField in _thosttraderapi:
_thosttraderapi.CThostFtdcReqUserLoginWithCaptchaField_swigregister(CThostFtdcReqUserLoginWithCaptchaField)

class CThostFtdcReqUserLoginWithTextField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    TradingDay = property(_thosttraderapi.CThostFtdcReqUserLoginWithTextField_TradingDay_get, _thosttraderapi.CThostFtdcReqUserLoginWithTextField_TradingDay_set)
    BrokerID = property(_thosttraderapi.CThostFtdcReqUserLoginWithTextField_BrokerID_get, _thosttraderapi.CThostFtdcReqUserLoginWithTextField_BrokerID_set)
    UserID = property(_thosttraderapi.CThostFtdcReqUserLoginWithTextField_UserID_get, _thosttraderapi.CThostFtdcReqUserLoginWithTextField_UserID_set)
    Password = property(_thosttraderapi.CThostFtdcReqUserLoginWithTextField_Password_get, _thosttraderapi.CThostFtdcReqUserLoginWithTextField_Password_set)
    UserProductInfo = property(_thosttraderapi.CThostFtdcReqUserLoginWithTextField_UserProductInfo_get, _thosttraderapi.CThostFtdcReqUserLoginWithTextField_UserProductInfo_set)
    InterfaceProductInfo = property(_thosttraderapi.CThostFtdcReqUserLoginWithTextField_InterfaceProductInfo_get, _thosttraderapi.CThostFtdcReqUserLoginWithTextField_InterfaceProductInfo_set)
    ProtocolInfo = property(_thosttraderapi.CThostFtdcReqUserLoginWithTextField_ProtocolInfo_get, _thosttraderapi.CThostFtdcReqUserLoginWithTextField_ProtocolInfo_set)
    MacAddress = property(_thosttraderapi.CThostFtdcReqUserLoginWithTextField_MacAddress_get, _thosttraderapi.CThostFtdcReqUserLoginWithTextField_MacAddress_set)
    ClientIPAddress = property(_thosttraderapi.CThostFtdcReqUserLoginWithTextField_ClientIPAddress_get, _thosttraderapi.CThostFtdcReqUserLoginWithTextField_ClientIPAddress_set)
    LoginRemark = property(_thosttraderapi.CThostFtdcReqUserLoginWithTextField_LoginRemark_get, _thosttraderapi.CThostFtdcReqUserLoginWithTextField_LoginRemark_set)
    Text = property(_thosttraderapi.CThostFtdcReqUserLoginWithTextField_Text_get, _thosttraderapi.CThostFtdcReqUserLoginWithTextField_Text_set)
    ClientIPPort = property(_thosttraderapi.CThostFtdcReqUserLoginWithTextField_ClientIPPort_get, _thosttraderapi.CThostFtdcReqUserLoginWithTextField_ClientIPPort_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcReqUserLoginWithTextField_swiginit(self, _thosttraderapi.new_CThostFtdcReqUserLoginWithTextField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcReqUserLoginWithTextField

# Register CThostFtdcReqUserLoginWithTextField in _thosttraderapi:
_thosttraderapi.CThostFtdcReqUserLoginWithTextField_swigregister(CThostFtdcReqUserLoginWithTextField)

class CThostFtdcReqUserLoginWithOTPField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    TradingDay = property(_thosttraderapi.CThostFtdcReqUserLoginWithOTPField_TradingDay_get, _thosttraderapi.CThostFtdcReqUserLoginWithOTPField_TradingDay_set)
    BrokerID = property(_thosttraderapi.CThostFtdcReqUserLoginWithOTPField_BrokerID_get, _thosttraderapi.CThostFtdcReqUserLoginWithOTPField_BrokerID_set)
    UserID = property(_thosttraderapi.CThostFtdcReqUserLoginWithOTPField_UserID_get, _thosttraderapi.CThostFtdcReqUserLoginWithOTPField_UserID_set)
    Password = property(_thosttraderapi.CThostFtdcReqUserLoginWithOTPField_Password_get, _thosttraderapi.CThostFtdcReqUserLoginWithOTPField_Password_set)
    UserProductInfo = property(_thosttraderapi.CThostFtdcReqUserLoginWithOTPField_UserProductInfo_get, _thosttraderapi.CThostFtdcReqUserLoginWithOTPField_UserProductInfo_set)
    InterfaceProductInfo = property(_thosttraderapi.CThostFtdcReqUserLoginWithOTPField_InterfaceProductInfo_get, _thosttraderapi.CThostFtdcReqUserLoginWithOTPField_InterfaceProductInfo_set)
    ProtocolInfo = property(_thosttraderapi.CThostFtdcReqUserLoginWithOTPField_ProtocolInfo_get, _thosttraderapi.CThostFtdcReqUserLoginWithOTPField_ProtocolInfo_set)
    MacAddress = property(_thosttraderapi.CThostFtdcReqUserLoginWithOTPField_MacAddress_get, _thosttraderapi.CThostFtdcReqUserLoginWithOTPField_MacAddress_set)
    ClientIPAddress = property(_thosttraderapi.CThostFtdcReqUserLoginWithOTPField_ClientIPAddress_get, _thosttraderapi.CThostFtdcReqUserLoginWithOTPField_ClientIPAddress_set)
    LoginRemark = property(_thosttraderapi.CThostFtdcReqUserLoginWithOTPField_LoginRemark_get, _thosttraderapi.CThostFtdcReqUserLoginWithOTPField_LoginRemark_set)
    OTPPassword = property(_thosttraderapi.CThostFtdcReqUserLoginWithOTPField_OTPPassword_get, _thosttraderapi.CThostFtdcReqUserLoginWithOTPField_OTPPassword_set)
    ClientIPPort = property(_thosttraderapi.CThostFtdcReqUserLoginWithOTPField_ClientIPPort_get, _thosttraderapi.CThostFtdcReqUserLoginWithOTPField_ClientIPPort_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcReqUserLoginWithOTPField_swiginit(self, _thosttraderapi.new_CThostFtdcReqUserLoginWithOTPField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcReqUserLoginWithOTPField

# Register CThostFtdcReqUserLoginWithOTPField in _thosttraderapi:
_thosttraderapi.CThostFtdcReqUserLoginWithOTPField_swigregister(CThostFtdcReqUserLoginWithOTPField)

class CThostFtdcReqApiHandshakeField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    CryptoKeyVersion = property(_thosttraderapi.CThostFtdcReqApiHandshakeField_CryptoKeyVersion_get, _thosttraderapi.CThostFtdcReqApiHandshakeField_CryptoKeyVersion_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcReqApiHandshakeField_swiginit(self, _thosttraderapi.new_CThostFtdcReqApiHandshakeField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcReqApiHandshakeField

# Register CThostFtdcReqApiHandshakeField in _thosttraderapi:
_thosttraderapi.CThostFtdcReqApiHandshakeField_swigregister(CThostFtdcReqApiHandshakeField)

class CThostFtdcRspApiHandshakeField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    FrontHandshakeDataLen = property(_thosttraderapi.CThostFtdcRspApiHandshakeField_FrontHandshakeDataLen_get, _thosttraderapi.CThostFtdcRspApiHandshakeField_FrontHandshakeDataLen_set)
    FrontHandshakeData = property(_thosttraderapi.CThostFtdcRspApiHandshakeField_FrontHandshakeData_get, _thosttraderapi.CThostFtdcRspApiHandshakeField_FrontHandshakeData_set)
    IsApiAuthEnabled = property(_thosttraderapi.CThostFtdcRspApiHandshakeField_IsApiAuthEnabled_get, _thosttraderapi.CThostFtdcRspApiHandshakeField_IsApiAuthEnabled_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcRspApiHandshakeField_swiginit(self, _thosttraderapi.new_CThostFtdcRspApiHandshakeField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcRspApiHandshakeField

# Register CThostFtdcRspApiHandshakeField in _thosttraderapi:
_thosttraderapi.CThostFtdcRspApiHandshakeField_swigregister(CThostFtdcRspApiHandshakeField)

class CThostFtdcReqVerifyApiKeyField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    ApiHandshakeDataLen = property(_thosttraderapi.CThostFtdcReqVerifyApiKeyField_ApiHandshakeDataLen_get, _thosttraderapi.CThostFtdcReqVerifyApiKeyField_ApiHandshakeDataLen_set)
    ApiHandshakeData = property(_thosttraderapi.CThostFtdcReqVerifyApiKeyField_ApiHandshakeData_get, _thosttraderapi.CThostFtdcReqVerifyApiKeyField_ApiHandshakeData_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcReqVerifyApiKeyField_swiginit(self, _thosttraderapi.new_CThostFtdcReqVerifyApiKeyField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcReqVerifyApiKeyField

# Register CThostFtdcReqVerifyApiKeyField in _thosttraderapi:
_thosttraderapi.CThostFtdcReqVerifyApiKeyField_swigregister(CThostFtdcReqVerifyApiKeyField)

class CThostFtdcDepartmentUserField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thosttraderapi.CThostFtdcDepartmentUserField_BrokerID_get, _thosttraderapi.CThostFtdcDepartmentUserField_BrokerID_set)
    UserID = property(_thosttraderapi.CThostFtdcDepartmentUserField_UserID_get, _thosttraderapi.CThostFtdcDepartmentUserField_UserID_set)
    InvestorRange = property(_thosttraderapi.CThostFtdcDepartmentUserField_InvestorRange_get, _thosttraderapi.CThostFtdcDepartmentUserField_InvestorRange_set)
    InvestorID = property(_thosttraderapi.CThostFtdcDepartmentUserField_InvestorID_get, _thosttraderapi.CThostFtdcDepartmentUserField_InvestorID_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcDepartmentUserField_swiginit(self, _thosttraderapi.new_CThostFtdcDepartmentUserField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcDepartmentUserField

# Register CThostFtdcDepartmentUserField in _thosttraderapi:
_thosttraderapi.CThostFtdcDepartmentUserField_swigregister(CThostFtdcDepartmentUserField)

class CThostFtdcQueryFreqField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    QueryFreq = property(_thosttraderapi.CThostFtdcQueryFreqField_QueryFreq_get, _thosttraderapi.CThostFtdcQueryFreqField_QueryFreq_set)

    def __init__(self):
        _thosttraderapi.CThostFtdcQueryFreqField_swiginit(self, _thosttraderapi.new_CThostFtdcQueryFreqField())
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcQueryFreqField

# Register CThostFtdcQueryFreqField in _thosttraderapi:
_thosttraderapi.CThostFtdcQueryFreqField_swigregister(CThostFtdcQueryFreqField)

class CThostFtdcTraderSpi(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def OnFrontConnected(self) -> "void":
        return _thosttraderapi.CThostFtdcTraderSpi_OnFrontConnected(self)

    def OnFrontDisconnected(self, nReason: "int") -> "void":
        return _thosttraderapi.CThostFtdcTraderSpi_OnFrontDisconnected(self, nReason)

    def OnHeartBeatWarning(self, nTimeLapse: "int") -> "void":
        return _thosttraderapi.CThostFtdcTraderSpi_OnHeartBeatWarning(self, nTimeLapse)

    def OnRspAuthenticate(self, pRspAuthenticateField: "CThostFtdcRspAuthenticateField", pRspInfo: "CThostFtdcRspInfoField", nRequestID: "int", bIsLast: "bool") -> "void":
        return _thosttraderapi.CThostFtdcTraderSpi_OnRspAuthenticate(self, pRspAuthenticateField, pRspInfo, nRequestID, bIsLast)

    def OnRspUserLogin(self, pRspUserLogin: "CThostFtdcRspUserLoginField", pRspInfo: "CThostFtdcRspInfoField", nRequestID: "int", bIsLast: "bool") -> "void":
        return _thosttraderapi.CThostFtdcTraderSpi_OnRspUserLogin(self, pRspUserLogin, pRspInfo, nRequestID, bIsLast)

    def OnRspUserLogout(self, pUserLogout: "CThostFtdcUserLogoutField", pRspInfo: "CThostFtdcRspInfoField", nRequestID: "int", bIsLast: "bool") -> "void":
        return _thosttraderapi.CThostFtdcTraderSpi_OnRspUserLogout(self, pUserLogout, pRspInfo, nRequestID, bIsLast)

    def OnRspUserPasswordUpdate(self, pUserPasswordUpdate: "CThostFtdcUserPasswordUpdateField", pRspInfo: "CThostFtdcRspInfoField", nRequestID: "int", bIsLast: "bool") -> "void":
        return _thosttraderapi.CThostFtdcTraderSpi_OnRspUserPasswordUpdate(self, pUserPasswordUpdate, pRspInfo, nRequestID, bIsLast)

    def OnRspTradingAccountPasswordUpdate(self, pTradingAccountPasswordUpdate: "CThostFtdcTradingAccountPasswordUpdateField", pRspInfo: "CThostFtdcRspInfoField", nRequestID: "int", bIsLast: "bool") -> "void":
        return _thosttraderapi.CThostFtdcTraderSpi_OnRspTradingAccountPasswordUpdate(self, pTradingAccountPasswordUpdate, pRspInfo, nRequestID, bIsLast)

    def OnRspUserAuthMethod(self, pRspUserAuthMethod: "CThostFtdcRspUserAuthMethodField", pRspInfo: "CThostFtdcRspInfoField", nRequestID: "int", bIsLast: "bool") -> "void":
        return _thosttraderapi.CThostFtdcTraderSpi_OnRspUserAuthMethod(self, pRspUserAuthMethod, pRspInfo, nRequestID, bIsLast)

    def OnRspGenUserCaptcha(self, pRspGenUserCaptcha: "CThostFtdcRspGenUserCaptchaField", pRspInfo: "CThostFtdcRspInfoField", nRequestID: "int", bIsLast: "bool") -> "void":
        return _thosttraderapi.CThostFtdcTraderSpi_OnRspGenUserCaptcha(self, pRspGenUserCaptcha, pRspInfo, nRequestID, bIsLast)

    def OnRspGenUserText(self, pRspGenUserText: "CThostFtdcRspGenUserTextField", pRspInfo: "CThostFtdcRspInfoField", nRequestID: "int", bIsLast: "bool") -> "void":
        return _thosttraderapi.CThostFtdcTraderSpi_OnRspGenUserText(self, pRspGenUserText, pRspInfo, nRequestID, bIsLast)

    def OnRspOrderInsert(self, pInputOrder: "CThostFtdcInputOrderField", pRspInfo: "CThostFtdcRspInfoField", nRequestID: "int", bIsLast: "bool") -> "void":
        return _thosttraderapi.CThostFtdcTraderSpi_OnRspOrderInsert(self, pInputOrder, pRspInfo, nRequestID, bIsLast)

    def OnRspParkedOrderInsert(self, pParkedOrder: "CThostFtdcParkedOrderField", pRspInfo: "CThostFtdcRspInfoField", nRequestID: "int", bIsLast: "bool") -> "void":
        return _thosttraderapi.CThostFtdcTraderSpi_OnRspParkedOrderInsert(self, pParkedOrder, pRspInfo, nRequestID, bIsLast)

    def OnRspParkedOrderAction(self, pParkedOrderAction: "CThostFtdcParkedOrderActionField", pRspInfo: "CThostFtdcRspInfoField", nRequestID: "int", bIsLast: "bool") -> "void":
        return _thosttraderapi.CThostFtdcTraderSpi_OnRspParkedOrderAction(self, pParkedOrderAction, pRspInfo, nRequestID, bIsLast)

    def OnRspOrderAction(self, pInputOrderAction: "CThostFtdcInputOrderActionField", pRspInfo: "CThostFtdcRspInfoField", nRequestID: "int", bIsLast: "bool") -> "void":
        return _thosttraderapi.CThostFtdcTraderSpi_OnRspOrderAction(self, pInputOrderAction, pRspInfo, nRequestID, bIsLast)

    def OnRspQueryMaxOrderVolume(self, pQueryMaxOrderVolume: "CThostFtdcQueryMaxOrderVolumeField", pRspInfo: "CThostFtdcRspInfoField", nRequestID: "int", bIsLast: "bool") -> "void":
        return _thosttraderapi.CThostFtdcTraderSpi_OnRspQueryMaxOrderVolume(self, pQueryMaxOrderVolume, pRspInfo, nRequestID, bIsLast)

    def OnRspSettlementInfoConfirm(self, pSettlementInfoConfirm: "CThostFtdcSettlementInfoConfirmField", pRspInfo: "CThostFtdcRspInfoField", nRequestID: "int", bIsLast: "bool") -> "void":
        return _thosttraderapi.CThostFtdcTraderSpi_OnRspSettlementInfoConfirm(self, pSettlementInfoConfirm, pRspInfo, nRequestID, bIsLast)

    def OnRspRemoveParkedOrder(self, pRemoveParkedOrder: "CThostFtdcRemoveParkedOrderField", pRspInfo: "CThostFtdcRspInfoField", nRequestID: "int", bIsLast: "bool") -> "void":
        return _thosttraderapi.CThostFtdcTraderSpi_OnRspRemoveParkedOrder(self, pRemoveParkedOrder, pRspInfo, nRequestID, bIsLast)

    def OnRspRemoveParkedOrderAction(self, pRemoveParkedOrderAction: "CThostFtdcRemoveParkedOrderActionField", pRspInfo: "CThostFtdcRspInfoField", nRequestID: "int", bIsLast: "bool") -> "void":
        return _thosttraderapi.CThostFtdcTraderSpi_OnRspRemoveParkedOrderAction(self, pRemoveParkedOrderAction, pRspInfo, nRequestID, bIsLast)

    def OnRspExecOrderInsert(self, pInputExecOrder: "CThostFtdcInputExecOrderField", pRspInfo: "CThostFtdcRspInfoField", nRequestID: "int", bIsLast: "bool") -> "void":
        return _thosttraderapi.CThostFtdcTraderSpi_OnRspExecOrderInsert(self, pInputExecOrder, pRspInfo, nRequestID, bIsLast)

    def OnRspExecOrderAction(self, pInputExecOrderAction: "CThostFtdcInputExecOrderActionField", pRspInfo: "CThostFtdcRspInfoField", nRequestID: "int", bIsLast: "bool") -> "void":
        return _thosttraderapi.CThostFtdcTraderSpi_OnRspExecOrderAction(self, pInputExecOrderAction, pRspInfo, nRequestID, bIsLast)

    def OnRspForQuoteInsert(self, pInputForQuote: "CThostFtdcInputForQuoteField", pRspInfo: "CThostFtdcRspInfoField", nRequestID: "int", bIsLast: "bool") -> "void":
        return _thosttraderapi.CThostFtdcTraderSpi_OnRspForQuoteInsert(self, pInputForQuote, pRspInfo, nRequestID, bIsLast)

    def OnRspQuoteInsert(self, pInputQuote: "CThostFtdcInputQuoteField", pRspInfo: "CThostFtdcRspInfoField", nRequestID: "int", bIsLast: "bool") -> "void":
        return _thosttraderapi.CThostFtdcTraderSpi_OnRspQuoteInsert(self, pInputQuote, pRspInfo, nRequestID, bIsLast)

    def OnRspQuoteAction(self, pInputQuoteAction: "CThostFtdcInputQuoteActionField", pRspInfo: "CThostFtdcRspInfoField", nRequestID: "int", bIsLast: "bool") -> "void":
        return _thosttraderapi.CThostFtdcTraderSpi_OnRspQuoteAction(self, pInputQuoteAction, pRspInfo, nRequestID, bIsLast)

    def OnRspBatchOrderAction(self, pInputBatchOrderAction: "CThostFtdcInputBatchOrderActionField", pRspInfo: "CThostFtdcRspInfoField", nRequestID: "int", bIsLast: "bool") -> "void":
        return _thosttraderapi.CThostFtdcTraderSpi_OnRspBatchOrderAction(self, pInputBatchOrderAction, pRspInfo, nRequestID, bIsLast)

    def OnRspOptionSelfCloseInsert(self, pInputOptionSelfClose: "CThostFtdcInputOptionSelfCloseField", pRspInfo: "CThostFtdcRspInfoField", nRequestID: "int", bIsLast: "bool") -> "void":
        return _thosttraderapi.CThostFtdcTraderSpi_OnRspOptionSelfCloseInsert(self, pInputOptionSelfClose, pRspInfo, nRequestID, bIsLast)

    def OnRspOptionSelfCloseAction(self, pInputOptionSelfCloseAction: "CThostFtdcInputOptionSelfCloseActionField", pRspInfo: "CThostFtdcRspInfoField", nRequestID: "int", bIsLast: "bool") -> "void":
        return _thosttraderapi.CThostFtdcTraderSpi_OnRspOptionSelfCloseAction(self, pInputOptionSelfCloseAction, pRspInfo, nRequestID, bIsLast)

    def OnRspCombActionInsert(self, pInputCombAction: "CThostFtdcInputCombActionField", pRspInfo: "CThostFtdcRspInfoField", nRequestID: "int", bIsLast: "bool") -> "void":
        return _thosttraderapi.CThostFtdcTraderSpi_OnRspCombActionInsert(self, pInputCombAction, pRspInfo, nRequestID, bIsLast)

    def OnRspQryOrder(self, pOrder: "CThostFtdcOrderField", pRspInfo: "CThostFtdcRspInfoField", nRequestID: "int", bIsLast: "bool") -> "void":
        return _thosttraderapi.CThostFtdcTraderSpi_OnRspQryOrder(self, pOrder, pRspInfo, nRequestID, bIsLast)

    def OnRspQryTrade(self, pTrade: "CThostFtdcTradeField", pRspInfo: "CThostFtdcRspInfoField", nRequestID: "int", bIsLast: "bool") -> "void":
        return _thosttraderapi.CThostFtdcTraderSpi_OnRspQryTrade(self, pTrade, pRspInfo, nRequestID, bIsLast)

    def OnRspQryInvestorPosition(self, pInvestorPosition: "CThostFtdcInvestorPositionField", pRspInfo: "CThostFtdcRspInfoField", nRequestID: "int", bIsLast: "bool") -> "void":
        return _thosttraderapi.CThostFtdcTraderSpi_OnRspQryInvestorPosition(self, pInvestorPosition, pRspInfo, nRequestID, bIsLast)

    def OnRspQryTradingAccount(self, pTradingAccount: "CThostFtdcTradingAccountField", pRspInfo: "CThostFtdcRspInfoField", nRequestID: "int", bIsLast: "bool") -> "void":
        return _thosttraderapi.CThostFtdcTraderSpi_OnRspQryTradingAccount(self, pTradingAccount, pRspInfo, nRequestID, bIsLast)

    def OnRspQryInvestor(self, pInvestor: "CThostFtdcInvestorField", pRspInfo: "CThostFtdcRspInfoField", nRequestID: "int", bIsLast: "bool") -> "void":
        return _thosttraderapi.CThostFtdcTraderSpi_OnRspQryInvestor(self, pInvestor, pRspInfo, nRequestID, bIsLast)

    def OnRspQryTradingCode(self, pTradingCode: "CThostFtdcTradingCodeField", pRspInfo: "CThostFtdcRspInfoField", nRequestID: "int", bIsLast: "bool") -> "void":
        return _thosttraderapi.CThostFtdcTraderSpi_OnRspQryTradingCode(self, pTradingCode, pRspInfo, nRequestID, bIsLast)

    def OnRspQryInstrumentMarginRate(self, pInstrumentMarginRate: "CThostFtdcInstrumentMarginRateField", pRspInfo: "CThostFtdcRspInfoField", nRequestID: "int", bIsLast: "bool") -> "void":
        return _thosttraderapi.CThostFtdcTraderSpi_OnRspQryInstrumentMarginRate(self, pInstrumentMarginRate, pRspInfo, nRequestID, bIsLast)

    def OnRspQryInstrumentCommissionRate(self, pInstrumentCommissionRate: "CThostFtdcInstrumentCommissionRateField", pRspInfo: "CThostFtdcRspInfoField", nRequestID: "int", bIsLast: "bool") -> "void":
        return _thosttraderapi.CThostFtdcTraderSpi_OnRspQryInstrumentCommissionRate(self, pInstrumentCommissionRate, pRspInfo, nRequestID, bIsLast)

    def OnRspQryExchange(self, pExchange: "CThostFtdcExchangeField", pRspInfo: "CThostFtdcRspInfoField", nRequestID: "int", bIsLast: "bool") -> "void":
        return _thosttraderapi.CThostFtdcTraderSpi_OnRspQryExchange(self, pExchange, pRspInfo, nRequestID, bIsLast)

    def OnRspQryProduct(self, pProduct: "CThostFtdcProductField", pRspInfo: "CThostFtdcRspInfoField", nRequestID: "int", bIsLast: "bool") -> "void":
        return _thosttraderapi.CThostFtdcTraderSpi_OnRspQryProduct(self, pProduct, pRspInfo, nRequestID, bIsLast)

    def OnRspQryInstrument(self, pInstrument: "CThostFtdcInstrumentField", pRspInfo: "CThostFtdcRspInfoField", nRequestID: "int", bIsLast: "bool") -> "void":
        return _thosttraderapi.CThostFtdcTraderSpi_OnRspQryInstrument(self, pInstrument, pRspInfo, nRequestID, bIsLast)

    def OnRspQryDepthMarketData(self, pDepthMarketData: "CThostFtdcDepthMarketDataField", pRspInfo: "CThostFtdcRspInfoField", nRequestID: "int", bIsLast: "bool") -> "void":
        return _thosttraderapi.CThostFtdcTraderSpi_OnRspQryDepthMarketData(self, pDepthMarketData, pRspInfo, nRequestID, bIsLast)

    def OnRspQrySettlementInfo(self, pSettlementInfo: "CThostFtdcSettlementInfoField", pRspInfo: "CThostFtdcRspInfoField", nRequestID: "int", bIsLast: "bool") -> "void":
        return _thosttraderapi.CThostFtdcTraderSpi_OnRspQrySettlementInfo(self, pSettlementInfo, pRspInfo, nRequestID, bIsLast)

    def OnRspQryTransferBank(self, pTransferBank: "CThostFtdcTransferBankField", pRspInfo: "CThostFtdcRspInfoField", nRequestID: "int", bIsLast: "bool") -> "void":
        return _thosttraderapi.CThostFtdcTraderSpi_OnRspQryTransferBank(self, pTransferBank, pRspInfo, nRequestID, bIsLast)

    def OnRspQryInvestorPositionDetail(self, pInvestorPositionDetail: "CThostFtdcInvestorPositionDetailField", pRspInfo: "CThostFtdcRspInfoField", nRequestID: "int", bIsLast: "bool") -> "void":
        return _thosttraderapi.CThostFtdcTraderSpi_OnRspQryInvestorPositionDetail(self, pInvestorPositionDetail, pRspInfo, nRequestID, bIsLast)

    def OnRspQryNotice(self, pNotice: "CThostFtdcNoticeField", pRspInfo: "CThostFtdcRspInfoField", nRequestID: "int", bIsLast: "bool") -> "void":
        return _thosttraderapi.CThostFtdcTraderSpi_OnRspQryNotice(self, pNotice, pRspInfo, nRequestID, bIsLast)

    def OnRspQrySettlementInfoConfirm(self, pSettlementInfoConfirm: "CThostFtdcSettlementInfoConfirmField", pRspInfo: "CThostFtdcRspInfoField", nRequestID: "int", bIsLast: "bool") -> "void":
        return _thosttraderapi.CThostFtdcTraderSpi_OnRspQrySettlementInfoConfirm(self, pSettlementInfoConfirm, pRspInfo, nRequestID, bIsLast)

    def OnRspQryInvestorPositionCombineDetail(self, pInvestorPositionCombineDetail: "CThostFtdcInvestorPositionCombineDetailField", pRspInfo: "CThostFtdcRspInfoField", nRequestID: "int", bIsLast: "bool") -> "void":
        return _thosttraderapi.CThostFtdcTraderSpi_OnRspQryInvestorPositionCombineDetail(self, pInvestorPositionCombineDetail, pRspInfo, nRequestID, bIsLast)

    def OnRspQryCFMMCTradingAccountKey(self, pCFMMCTradingAccountKey: "CThostFtdcCFMMCTradingAccountKeyField", pRspInfo: "CThostFtdcRspInfoField", nRequestID: "int", bIsLast: "bool") -> "void":
        return _thosttraderapi.CThostFtdcTraderSpi_OnRspQryCFMMCTradingAccountKey(self, pCFMMCTradingAccountKey, pRspInfo, nRequestID, bIsLast)

    def OnRspQryEWarrantOffset(self, pEWarrantOffset: "CThostFtdcEWarrantOffsetField", pRspInfo: "CThostFtdcRspInfoField", nRequestID: "int", bIsLast: "bool") -> "void":
        return _thosttraderapi.CThostFtdcTraderSpi_OnRspQryEWarrantOffset(self, pEWarrantOffset, pRspInfo, nRequestID, bIsLast)

    def OnRspQryInvestorProductGroupMargin(self, pInvestorProductGroupMargin: "CThostFtdcInvestorProductGroupMarginField", pRspInfo: "CThostFtdcRspInfoField", nRequestID: "int", bIsLast: "bool") -> "void":
        return _thosttraderapi.CThostFtdcTraderSpi_OnRspQryInvestorProductGroupMargin(self, pInvestorProductGroupMargin, pRspInfo, nRequestID, bIsLast)

    def OnRspQryExchangeMarginRate(self, pExchangeMarginRate: "CThostFtdcExchangeMarginRateField", pRspInfo: "CThostFtdcRspInfoField", nRequestID: "int", bIsLast: "bool") -> "void":
        return _thosttraderapi.CThostFtdcTraderSpi_OnRspQryExchangeMarginRate(self, pExchangeMarginRate, pRspInfo, nRequestID, bIsLast)

    def OnRspQryExchangeMarginRateAdjust(self, pExchangeMarginRateAdjust: "CThostFtdcExchangeMarginRateAdjustField", pRspInfo: "CThostFtdcRspInfoField", nRequestID: "int", bIsLast: "bool") -> "void":
        return _thosttraderapi.CThostFtdcTraderSpi_OnRspQryExchangeMarginRateAdjust(self, pExchangeMarginRateAdjust, pRspInfo, nRequestID, bIsLast)

    def OnRspQryExchangeRate(self, pExchangeRate: "CThostFtdcExchangeRateField", pRspInfo: "CThostFtdcRspInfoField", nRequestID: "int", bIsLast: "bool") -> "void":
        return _thosttraderapi.CThostFtdcTraderSpi_OnRspQryExchangeRate(self, pExchangeRate, pRspInfo, nRequestID, bIsLast)

    def OnRspQrySecAgentACIDMap(self, pSecAgentACIDMap: "CThostFtdcSecAgentACIDMapField", pRspInfo: "CThostFtdcRspInfoField", nRequestID: "int", bIsLast: "bool") -> "void":
        return _thosttraderapi.CThostFtdcTraderSpi_OnRspQrySecAgentACIDMap(self, pSecAgentACIDMap, pRspInfo, nRequestID, bIsLast)

    def OnRspQryProductExchRate(self, pProductExchRate: "CThostFtdcProductExchRateField", pRspInfo: "CThostFtdcRspInfoField", nRequestID: "int", bIsLast: "bool") -> "void":
        return _thosttraderapi.CThostFtdcTraderSpi_OnRspQryProductExchRate(self, pProductExchRate, pRspInfo, nRequestID, bIsLast)

    def OnRspQryProductGroup(self, pProductGroup: "CThostFtdcProductGroupField", pRspInfo: "CThostFtdcRspInfoField", nRequestID: "int", bIsLast: "bool") -> "void":
        return _thosttraderapi.CThostFtdcTraderSpi_OnRspQryProductGroup(self, pProductGroup, pRspInfo, nRequestID, bIsLast)

    def OnRspQryMMInstrumentCommissionRate(self, pMMInstrumentCommissionRate: "CThostFtdcMMInstrumentCommissionRateField", pRspInfo: "CThostFtdcRspInfoField", nRequestID: "int", bIsLast: "bool") -> "void":
        return _thosttraderapi.CThostFtdcTraderSpi_OnRspQryMMInstrumentCommissionRate(self, pMMInstrumentCommissionRate, pRspInfo, nRequestID, bIsLast)

    def OnRspQryMMOptionInstrCommRate(self, pMMOptionInstrCommRate: "CThostFtdcMMOptionInstrCommRateField", pRspInfo: "CThostFtdcRspInfoField", nRequestID: "int", bIsLast: "bool") -> "void":
        return _thosttraderapi.CThostFtdcTraderSpi_OnRspQryMMOptionInstrCommRate(self, pMMOptionInstrCommRate, pRspInfo, nRequestID, bIsLast)

    def OnRspQryInstrumentOrderCommRate(self, pInstrumentOrderCommRate: "CThostFtdcInstrumentOrderCommRateField", pRspInfo: "CThostFtdcRspInfoField", nRequestID: "int", bIsLast: "bool") -> "void":
        return _thosttraderapi.CThostFtdcTraderSpi_OnRspQryInstrumentOrderCommRate(self, pInstrumentOrderCommRate, pRspInfo, nRequestID, bIsLast)

    def OnRspQrySecAgentTradingAccount(self, pTradingAccount: "CThostFtdcTradingAccountField", pRspInfo: "CThostFtdcRspInfoField", nRequestID: "int", bIsLast: "bool") -> "void":
        return _thosttraderapi.CThostFtdcTraderSpi_OnRspQrySecAgentTradingAccount(self, pTradingAccount, pRspInfo, nRequestID, bIsLast)

    def OnRspQrySecAgentCheckMode(self, pSecAgentCheckMode: "CThostFtdcSecAgentCheckModeField", pRspInfo: "CThostFtdcRspInfoField", nRequestID: "int", bIsLast: "bool") -> "void":
        return _thosttraderapi.CThostFtdcTraderSpi_OnRspQrySecAgentCheckMode(self, pSecAgentCheckMode, pRspInfo, nRequestID, bIsLast)

    def OnRspQrySecAgentTradeInfo(self, pSecAgentTradeInfo: "CThostFtdcSecAgentTradeInfoField", pRspInfo: "CThostFtdcRspInfoField", nRequestID: "int", bIsLast: "bool") -> "void":
        return _thosttraderapi.CThostFtdcTraderSpi_OnRspQrySecAgentTradeInfo(self, pSecAgentTradeInfo, pRspInfo, nRequestID, bIsLast)

    def OnRspQryOptionInstrTradeCost(self, pOptionInstrTradeCost: "CThostFtdcOptionInstrTradeCostField", pRspInfo: "CThostFtdcRspInfoField", nRequestID: "int", bIsLast: "bool") -> "void":
        return _thosttraderapi.CThostFtdcTraderSpi_OnRspQryOptionInstrTradeCost(self, pOptionInstrTradeCost, pRspInfo, nRequestID, bIsLast)

    def OnRspQryOptionInstrCommRate(self, pOptionInstrCommRate: "CThostFtdcOptionInstrCommRateField", pRspInfo: "CThostFtdcRspInfoField", nRequestID: "int", bIsLast: "bool") -> "void":
        return _thosttraderapi.CThostFtdcTraderSpi_OnRspQryOptionInstrCommRate(self, pOptionInstrCommRate, pRspInfo, nRequestID, bIsLast)

    def OnRspQryExecOrder(self, pExecOrder: "CThostFtdcExecOrderField", pRspInfo: "CThostFtdcRspInfoField", nRequestID: "int", bIsLast: "bool") -> "void":
        return _thosttraderapi.CThostFtdcTraderSpi_OnRspQryExecOrder(self, pExecOrder, pRspInfo, nRequestID, bIsLast)

    def OnRspQryForQuote(self, pForQuote: "CThostFtdcForQuoteField", pRspInfo: "CThostFtdcRspInfoField", nRequestID: "int", bIsLast: "bool") -> "void":
        return _thosttraderapi.CThostFtdcTraderSpi_OnRspQryForQuote(self, pForQuote, pRspInfo, nRequestID, bIsLast)

    def OnRspQryQuote(self, pQuote: "CThostFtdcQuoteField", pRspInfo: "CThostFtdcRspInfoField", nRequestID: "int", bIsLast: "bool") -> "void":
        return _thosttraderapi.CThostFtdcTraderSpi_OnRspQryQuote(self, pQuote, pRspInfo, nRequestID, bIsLast)

    def OnRspQryOptionSelfClose(self, pOptionSelfClose: "CThostFtdcOptionSelfCloseField", pRspInfo: "CThostFtdcRspInfoField", nRequestID: "int", bIsLast: "bool") -> "void":
        return _thosttraderapi.CThostFtdcTraderSpi_OnRspQryOptionSelfClose(self, pOptionSelfClose, pRspInfo, nRequestID, bIsLast)

    def OnRspQryInvestUnit(self, pInvestUnit: "CThostFtdcInvestUnitField", pRspInfo: "CThostFtdcRspInfoField", nRequestID: "int", bIsLast: "bool") -> "void":
        return _thosttraderapi.CThostFtdcTraderSpi_OnRspQryInvestUnit(self, pInvestUnit, pRspInfo, nRequestID, bIsLast)

    def OnRspQryCombInstrumentGuard(self, pCombInstrumentGuard: "CThostFtdcCombInstrumentGuardField", pRspInfo: "CThostFtdcRspInfoField", nRequestID: "int", bIsLast: "bool") -> "void":
        return _thosttraderapi.CThostFtdcTraderSpi_OnRspQryCombInstrumentGuard(self, pCombInstrumentGuard, pRspInfo, nRequestID, bIsLast)

    def OnRspQryCombAction(self, pCombAction: "CThostFtdcCombActionField", pRspInfo: "CThostFtdcRspInfoField", nRequestID: "int", bIsLast: "bool") -> "void":
        return _thosttraderapi.CThostFtdcTraderSpi_OnRspQryCombAction(self, pCombAction, pRspInfo, nRequestID, bIsLast)

    def OnRspQryTransferSerial(self, pTransferSerial: "CThostFtdcTransferSerialField", pRspInfo: "CThostFtdcRspInfoField", nRequestID: "int", bIsLast: "bool") -> "void":
        return _thosttraderapi.CThostFtdcTraderSpi_OnRspQryTransferSerial(self, pTransferSerial, pRspInfo, nRequestID, bIsLast)

    def OnRspQryAccountregister(self, pAccountregister: "CThostFtdcAccountregisterField", pRspInfo: "CThostFtdcRspInfoField", nRequestID: "int", bIsLast: "bool") -> "void":
        return _thosttraderapi.CThostFtdcTraderSpi_OnRspQryAccountregister(self, pAccountregister, pRspInfo, nRequestID, bIsLast)

    def OnRspError(self, pRspInfo: "CThostFtdcRspInfoField", nRequestID: "int", bIsLast: "bool") -> "void":
        return _thosttraderapi.CThostFtdcTraderSpi_OnRspError(self, pRspInfo, nRequestID, bIsLast)

    def OnRtnOrder(self, pOrder: "CThostFtdcOrderField") -> "void":
        return _thosttraderapi.CThostFtdcTraderSpi_OnRtnOrder(self, pOrder)

    def OnRtnTrade(self, pTrade: "CThostFtdcTradeField") -> "void":
        return _thosttraderapi.CThostFtdcTraderSpi_OnRtnTrade(self, pTrade)

    def OnErrRtnOrderInsert(self, pInputOrder: "CThostFtdcInputOrderField", pRspInfo: "CThostFtdcRspInfoField") -> "void":
        return _thosttraderapi.CThostFtdcTraderSpi_OnErrRtnOrderInsert(self, pInputOrder, pRspInfo)

    def OnErrRtnOrderAction(self, pOrderAction: "CThostFtdcOrderActionField", pRspInfo: "CThostFtdcRspInfoField") -> "void":
        return _thosttraderapi.CThostFtdcTraderSpi_OnErrRtnOrderAction(self, pOrderAction, pRspInfo)

    def OnRtnInstrumentStatus(self, pInstrumentStatus: "CThostFtdcInstrumentStatusField") -> "void":
        return _thosttraderapi.CThostFtdcTraderSpi_OnRtnInstrumentStatus(self, pInstrumentStatus)

    def OnRtnBulletin(self, pBulletin: "CThostFtdcBulletinField") -> "void":
        return _thosttraderapi.CThostFtdcTraderSpi_OnRtnBulletin(self, pBulletin)

    def OnRtnTradingNotice(self, pTradingNoticeInfo: "CThostFtdcTradingNoticeInfoField") -> "void":
        return _thosttraderapi.CThostFtdcTraderSpi_OnRtnTradingNotice(self, pTradingNoticeInfo)

    def OnRtnErrorConditionalOrder(self, pErrorConditionalOrder: "CThostFtdcErrorConditionalOrderField") -> "void":
        return _thosttraderapi.CThostFtdcTraderSpi_OnRtnErrorConditionalOrder(self, pErrorConditionalOrder)

    def OnRtnExecOrder(self, pExecOrder: "CThostFtdcExecOrderField") -> "void":
        return _thosttraderapi.CThostFtdcTraderSpi_OnRtnExecOrder(self, pExecOrder)

    def OnErrRtnExecOrderInsert(self, pInputExecOrder: "CThostFtdcInputExecOrderField", pRspInfo: "CThostFtdcRspInfoField") -> "void":
        return _thosttraderapi.CThostFtdcTraderSpi_OnErrRtnExecOrderInsert(self, pInputExecOrder, pRspInfo)

    def OnErrRtnExecOrderAction(self, pExecOrderAction: "CThostFtdcExecOrderActionField", pRspInfo: "CThostFtdcRspInfoField") -> "void":
        return _thosttraderapi.CThostFtdcTraderSpi_OnErrRtnExecOrderAction(self, pExecOrderAction, pRspInfo)

    def OnErrRtnForQuoteInsert(self, pInputForQuote: "CThostFtdcInputForQuoteField", pRspInfo: "CThostFtdcRspInfoField") -> "void":
        return _thosttraderapi.CThostFtdcTraderSpi_OnErrRtnForQuoteInsert(self, pInputForQuote, pRspInfo)

    def OnRtnQuote(self, pQuote: "CThostFtdcQuoteField") -> "void":
        return _thosttraderapi.CThostFtdcTraderSpi_OnRtnQuote(self, pQuote)

    def OnErrRtnQuoteInsert(self, pInputQuote: "CThostFtdcInputQuoteField", pRspInfo: "CThostFtdcRspInfoField") -> "void":
        return _thosttraderapi.CThostFtdcTraderSpi_OnErrRtnQuoteInsert(self, pInputQuote, pRspInfo)

    def OnErrRtnQuoteAction(self, pQuoteAction: "CThostFtdcQuoteActionField", pRspInfo: "CThostFtdcRspInfoField") -> "void":
        return _thosttraderapi.CThostFtdcTraderSpi_OnErrRtnQuoteAction(self, pQuoteAction, pRspInfo)

    def OnRtnForQuoteRsp(self, pForQuoteRsp: "CThostFtdcForQuoteRspField") -> "void":
        return _thosttraderapi.CThostFtdcTraderSpi_OnRtnForQuoteRsp(self, pForQuoteRsp)

    def OnRtnCFMMCTradingAccountToken(self, pCFMMCTradingAccountToken: "CThostFtdcCFMMCTradingAccountTokenField") -> "void":
        return _thosttraderapi.CThostFtdcTraderSpi_OnRtnCFMMCTradingAccountToken(self, pCFMMCTradingAccountToken)

    def OnErrRtnBatchOrderAction(self, pBatchOrderAction: "CThostFtdcBatchOrderActionField", pRspInfo: "CThostFtdcRspInfoField") -> "void":
        return _thosttraderapi.CThostFtdcTraderSpi_OnErrRtnBatchOrderAction(self, pBatchOrderAction, pRspInfo)

    def OnRtnOptionSelfClose(self, pOptionSelfClose: "CThostFtdcOptionSelfCloseField") -> "void":
        return _thosttraderapi.CThostFtdcTraderSpi_OnRtnOptionSelfClose(self, pOptionSelfClose)

    def OnErrRtnOptionSelfCloseInsert(self, pInputOptionSelfClose: "CThostFtdcInputOptionSelfCloseField", pRspInfo: "CThostFtdcRspInfoField") -> "void":
        return _thosttraderapi.CThostFtdcTraderSpi_OnErrRtnOptionSelfCloseInsert(self, pInputOptionSelfClose, pRspInfo)

    def OnErrRtnOptionSelfCloseAction(self, pOptionSelfCloseAction: "CThostFtdcOptionSelfCloseActionField", pRspInfo: "CThostFtdcRspInfoField") -> "void":
        return _thosttraderapi.CThostFtdcTraderSpi_OnErrRtnOptionSelfCloseAction(self, pOptionSelfCloseAction, pRspInfo)

    def OnRtnCombAction(self, pCombAction: "CThostFtdcCombActionField") -> "void":
        return _thosttraderapi.CThostFtdcTraderSpi_OnRtnCombAction(self, pCombAction)

    def OnErrRtnCombActionInsert(self, pInputCombAction: "CThostFtdcInputCombActionField", pRspInfo: "CThostFtdcRspInfoField") -> "void":
        return _thosttraderapi.CThostFtdcTraderSpi_OnErrRtnCombActionInsert(self, pInputCombAction, pRspInfo)

    def OnRspQryContractBank(self, pContractBank: "CThostFtdcContractBankField", pRspInfo: "CThostFtdcRspInfoField", nRequestID: "int", bIsLast: "bool") -> "void":
        return _thosttraderapi.CThostFtdcTraderSpi_OnRspQryContractBank(self, pContractBank, pRspInfo, nRequestID, bIsLast)

    def OnRspQryParkedOrder(self, pParkedOrder: "CThostFtdcParkedOrderField", pRspInfo: "CThostFtdcRspInfoField", nRequestID: "int", bIsLast: "bool") -> "void":
        return _thosttraderapi.CThostFtdcTraderSpi_OnRspQryParkedOrder(self, pParkedOrder, pRspInfo, nRequestID, bIsLast)

    def OnRspQryParkedOrderAction(self, pParkedOrderAction: "CThostFtdcParkedOrderActionField", pRspInfo: "CThostFtdcRspInfoField", nRequestID: "int", bIsLast: "bool") -> "void":
        return _thosttraderapi.CThostFtdcTraderSpi_OnRspQryParkedOrderAction(self, pParkedOrderAction, pRspInfo, nRequestID, bIsLast)

    def OnRspQryTradingNotice(self, pTradingNotice: "CThostFtdcTradingNoticeField", pRspInfo: "CThostFtdcRspInfoField", nRequestID: "int", bIsLast: "bool") -> "void":
        return _thosttraderapi.CThostFtdcTraderSpi_OnRspQryTradingNotice(self, pTradingNotice, pRspInfo, nRequestID, bIsLast)

    def OnRspQryBrokerTradingParams(self, pBrokerTradingParams: "CThostFtdcBrokerTradingParamsField", pRspInfo: "CThostFtdcRspInfoField", nRequestID: "int", bIsLast: "bool") -> "void":
        return _thosttraderapi.CThostFtdcTraderSpi_OnRspQryBrokerTradingParams(self, pBrokerTradingParams, pRspInfo, nRequestID, bIsLast)

    def OnRspQryBrokerTradingAlgos(self, pBrokerTradingAlgos: "CThostFtdcBrokerTradingAlgosField", pRspInfo: "CThostFtdcRspInfoField", nRequestID: "int", bIsLast: "bool") -> "void":
        return _thosttraderapi.CThostFtdcTraderSpi_OnRspQryBrokerTradingAlgos(self, pBrokerTradingAlgos, pRspInfo, nRequestID, bIsLast)

    def OnRspQueryCFMMCTradingAccountToken(self, pQueryCFMMCTradingAccountToken: "CThostFtdcQueryCFMMCTradingAccountTokenField", pRspInfo: "CThostFtdcRspInfoField", nRequestID: "int", bIsLast: "bool") -> "void":
        return _thosttraderapi.CThostFtdcTraderSpi_OnRspQueryCFMMCTradingAccountToken(self, pQueryCFMMCTradingAccountToken, pRspInfo, nRequestID, bIsLast)

    def OnRtnFromBankToFutureByBank(self, pRspTransfer: "CThostFtdcRspTransferField") -> "void":
        return _thosttraderapi.CThostFtdcTraderSpi_OnRtnFromBankToFutureByBank(self, pRspTransfer)

    def OnRtnFromFutureToBankByBank(self, pRspTransfer: "CThostFtdcRspTransferField") -> "void":
        return _thosttraderapi.CThostFtdcTraderSpi_OnRtnFromFutureToBankByBank(self, pRspTransfer)

    def OnRtnRepealFromBankToFutureByBank(self, pRspRepeal: "CThostFtdcRspRepealField") -> "void":
        return _thosttraderapi.CThostFtdcTraderSpi_OnRtnRepealFromBankToFutureByBank(self, pRspRepeal)

    def OnRtnRepealFromFutureToBankByBank(self, pRspRepeal: "CThostFtdcRspRepealField") -> "void":
        return _thosttraderapi.CThostFtdcTraderSpi_OnRtnRepealFromFutureToBankByBank(self, pRspRepeal)

    def OnRtnFromBankToFutureByFuture(self, pRspTransfer: "CThostFtdcRspTransferField") -> "void":
        return _thosttraderapi.CThostFtdcTraderSpi_OnRtnFromBankToFutureByFuture(self, pRspTransfer)

    def OnRtnFromFutureToBankByFuture(self, pRspTransfer: "CThostFtdcRspTransferField") -> "void":
        return _thosttraderapi.CThostFtdcTraderSpi_OnRtnFromFutureToBankByFuture(self, pRspTransfer)

    def OnRtnRepealFromBankToFutureByFutureManual(self, pRspRepeal: "CThostFtdcRspRepealField") -> "void":
        return _thosttraderapi.CThostFtdcTraderSpi_OnRtnRepealFromBankToFutureByFutureManual(self, pRspRepeal)

    def OnRtnRepealFromFutureToBankByFutureManual(self, pRspRepeal: "CThostFtdcRspRepealField") -> "void":
        return _thosttraderapi.CThostFtdcTraderSpi_OnRtnRepealFromFutureToBankByFutureManual(self, pRspRepeal)

    def OnRtnQueryBankBalanceByFuture(self, pNotifyQueryAccount: "CThostFtdcNotifyQueryAccountField") -> "void":
        return _thosttraderapi.CThostFtdcTraderSpi_OnRtnQueryBankBalanceByFuture(self, pNotifyQueryAccount)

    def OnErrRtnBankToFutureByFuture(self, pReqTransfer: "CThostFtdcReqTransferField", pRspInfo: "CThostFtdcRspInfoField") -> "void":
        return _thosttraderapi.CThostFtdcTraderSpi_OnErrRtnBankToFutureByFuture(self, pReqTransfer, pRspInfo)

    def OnErrRtnFutureToBankByFuture(self, pReqTransfer: "CThostFtdcReqTransferField", pRspInfo: "CThostFtdcRspInfoField") -> "void":
        return _thosttraderapi.CThostFtdcTraderSpi_OnErrRtnFutureToBankByFuture(self, pReqTransfer, pRspInfo)

    def OnErrRtnRepealBankToFutureByFutureManual(self, pReqRepeal: "CThostFtdcReqRepealField", pRspInfo: "CThostFtdcRspInfoField") -> "void":
        return _thosttraderapi.CThostFtdcTraderSpi_OnErrRtnRepealBankToFutureByFutureManual(self, pReqRepeal, pRspInfo)

    def OnErrRtnRepealFutureToBankByFutureManual(self, pReqRepeal: "CThostFtdcReqRepealField", pRspInfo: "CThostFtdcRspInfoField") -> "void":
        return _thosttraderapi.CThostFtdcTraderSpi_OnErrRtnRepealFutureToBankByFutureManual(self, pReqRepeal, pRspInfo)

    def OnErrRtnQueryBankBalanceByFuture(self, pReqQueryAccount: "CThostFtdcReqQueryAccountField", pRspInfo: "CThostFtdcRspInfoField") -> "void":
        return _thosttraderapi.CThostFtdcTraderSpi_OnErrRtnQueryBankBalanceByFuture(self, pReqQueryAccount, pRspInfo)

    def OnRtnRepealFromBankToFutureByFuture(self, pRspRepeal: "CThostFtdcRspRepealField") -> "void":
        return _thosttraderapi.CThostFtdcTraderSpi_OnRtnRepealFromBankToFutureByFuture(self, pRspRepeal)

    def OnRtnRepealFromFutureToBankByFuture(self, pRspRepeal: "CThostFtdcRspRepealField") -> "void":
        return _thosttraderapi.CThostFtdcTraderSpi_OnRtnRepealFromFutureToBankByFuture(self, pRspRepeal)

    def OnRspFromBankToFutureByFuture(self, pReqTransfer: "CThostFtdcReqTransferField", pRspInfo: "CThostFtdcRspInfoField", nRequestID: "int", bIsLast: "bool") -> "void":
        return _thosttraderapi.CThostFtdcTraderSpi_OnRspFromBankToFutureByFuture(self, pReqTransfer, pRspInfo, nRequestID, bIsLast)

    def OnRspFromFutureToBankByFuture(self, pReqTransfer: "CThostFtdcReqTransferField", pRspInfo: "CThostFtdcRspInfoField", nRequestID: "int", bIsLast: "bool") -> "void":
        return _thosttraderapi.CThostFtdcTraderSpi_OnRspFromFutureToBankByFuture(self, pReqTransfer, pRspInfo, nRequestID, bIsLast)

    def OnRspQueryBankAccountMoneyByFuture(self, pReqQueryAccount: "CThostFtdcReqQueryAccountField", pRspInfo: "CThostFtdcRspInfoField", nRequestID: "int", bIsLast: "bool") -> "void":
        return _thosttraderapi.CThostFtdcTraderSpi_OnRspQueryBankAccountMoneyByFuture(self, pReqQueryAccount, pRspInfo, nRequestID, bIsLast)

    def OnRtnOpenAccountByBank(self, pOpenAccount: "CThostFtdcOpenAccountField") -> "void":
        return _thosttraderapi.CThostFtdcTraderSpi_OnRtnOpenAccountByBank(self, pOpenAccount)

    def OnRtnCancelAccountByBank(self, pCancelAccount: "CThostFtdcCancelAccountField") -> "void":
        return _thosttraderapi.CThostFtdcTraderSpi_OnRtnCancelAccountByBank(self, pCancelAccount)

    def OnRtnChangeAccountByBank(self, pChangeAccount: "CThostFtdcChangeAccountField") -> "void":
        return _thosttraderapi.CThostFtdcTraderSpi_OnRtnChangeAccountByBank(self, pChangeAccount)

    def __init__(self):
        if self.__class__ == CThostFtdcTraderSpi:
            _self = None
        else:
            _self = self
        _thosttraderapi.CThostFtdcTraderSpi_swiginit(self, _thosttraderapi.new_CThostFtdcTraderSpi(_self, ))
    __swig_destroy__ = _thosttraderapi.delete_CThostFtdcTraderSpi
    def __disown__(self):
        self.this.disown()
        _thosttraderapi.disown_CThostFtdcTraderSpi(self)
        return weakref.proxy(self)

# Register CThostFtdcTraderSpi in _thosttraderapi:
_thosttraderapi.CThostFtdcTraderSpi_swigregister(CThostFtdcTraderSpi)

class CThostFtdcTraderApi(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args, **kwargs):
        raise AttributeError("No constructor defined - class is abstract")
    __repr__ = _swig_repr

    @staticmethod
    def CreateFtdcTraderApi(*args) -> "CThostFtdcTraderApi *":
        return _thosttraderapi.CThostFtdcTraderApi_CreateFtdcTraderApi(*args)

    @staticmethod
    def GetApiVersion() -> "char const *":
        return _thosttraderapi.CThostFtdcTraderApi_GetApiVersion()

    def Release(self) -> "void":
        return _thosttraderapi.CThostFtdcTraderApi_Release(self)

    def Init(self) -> "void":
        return _thosttraderapi.CThostFtdcTraderApi_Init(self)

    def Join(self) -> "int":
        return _thosttraderapi.CThostFtdcTraderApi_Join(self)

    def GetTradingDay(self) -> "char const *":
        return _thosttraderapi.CThostFtdcTraderApi_GetTradingDay(self)

    def RegisterFront(self, pszFrontAddress: "char *") -> "void":
        return _thosttraderapi.CThostFtdcTraderApi_RegisterFront(self, pszFrontAddress)

    def RegisterNameServer(self, pszNsAddress: "char *") -> "void":
        return _thosttraderapi.CThostFtdcTraderApi_RegisterNameServer(self, pszNsAddress)

    def RegisterFensUserInfo(self, pFensUserInfo: "CThostFtdcFensUserInfoField") -> "void":
        return _thosttraderapi.CThostFtdcTraderApi_RegisterFensUserInfo(self, pFensUserInfo)

    def RegisterSpi(self, pSpi: "CThostFtdcTraderSpi") -> "void":
        return _thosttraderapi.CThostFtdcTraderApi_RegisterSpi(self, pSpi)

    def SubscribePrivateTopic(self, nResumeType: "THOST_TE_RESUME_TYPE") -> "void":
        return _thosttraderapi.CThostFtdcTraderApi_SubscribePrivateTopic(self, nResumeType)

    def SubscribePublicTopic(self, nResumeType: "THOST_TE_RESUME_TYPE") -> "void":
        return _thosttraderapi.CThostFtdcTraderApi_SubscribePublicTopic(self, nResumeType)

    def ReqAuthenticate(self, pReqAuthenticateField: "CThostFtdcReqAuthenticateField", nRequestID: "int") -> "int":
        return _thosttraderapi.CThostFtdcTraderApi_ReqAuthenticate(self, pReqAuthenticateField, nRequestID)

    def RegisterUserSystemInfo(self, pUserSystemInfo: "CThostFtdcUserSystemInfoField") -> "int":
        return _thosttraderapi.CThostFtdcTraderApi_RegisterUserSystemInfo(self, pUserSystemInfo)

    def SubmitUserSystemInfo(self, pUserSystemInfo: "CThostFtdcUserSystemInfoField") -> "int":
        return _thosttraderapi.CThostFtdcTraderApi_SubmitUserSystemInfo(self, pUserSystemInfo)

    def ReqUserLogin(self, pReqUserLoginField: "CThostFtdcReqUserLoginField", nRequestID: "int") -> "int":
        return _thosttraderapi.CThostFtdcTraderApi_ReqUserLogin(self, pReqUserLoginField, nRequestID)

    def ReqUserLogout(self, pUserLogout: "CThostFtdcUserLogoutField", nRequestID: "int") -> "int":
        return _thosttraderapi.CThostFtdcTraderApi_ReqUserLogout(self, pUserLogout, nRequestID)

    def ReqUserPasswordUpdate(self, pUserPasswordUpdate: "CThostFtdcUserPasswordUpdateField", nRequestID: "int") -> "int":
        return _thosttraderapi.CThostFtdcTraderApi_ReqUserPasswordUpdate(self, pUserPasswordUpdate, nRequestID)

    def ReqTradingAccountPasswordUpdate(self, pTradingAccountPasswordUpdate: "CThostFtdcTradingAccountPasswordUpdateField", nRequestID: "int") -> "int":
        return _thosttraderapi.CThostFtdcTraderApi_ReqTradingAccountPasswordUpdate(self, pTradingAccountPasswordUpdate, nRequestID)

    def ReqUserAuthMethod(self, pReqUserAuthMethod: "CThostFtdcReqUserAuthMethodField", nRequestID: "int") -> "int":
        return _thosttraderapi.CThostFtdcTraderApi_ReqUserAuthMethod(self, pReqUserAuthMethod, nRequestID)

    def ReqGenUserCaptcha(self, pReqGenUserCaptcha: "CThostFtdcReqGenUserCaptchaField", nRequestID: "int") -> "int":
        return _thosttraderapi.CThostFtdcTraderApi_ReqGenUserCaptcha(self, pReqGenUserCaptcha, nRequestID)

    def ReqGenUserText(self, pReqGenUserText: "CThostFtdcReqGenUserTextField", nRequestID: "int") -> "int":
        return _thosttraderapi.CThostFtdcTraderApi_ReqGenUserText(self, pReqGenUserText, nRequestID)

    def ReqUserLoginWithCaptcha(self, pReqUserLoginWithCaptcha: "CThostFtdcReqUserLoginWithCaptchaField", nRequestID: "int") -> "int":
        return _thosttraderapi.CThostFtdcTraderApi_ReqUserLoginWithCaptcha(self, pReqUserLoginWithCaptcha, nRequestID)

    def ReqUserLoginWithText(self, pReqUserLoginWithText: "CThostFtdcReqUserLoginWithTextField", nRequestID: "int") -> "int":
        return _thosttraderapi.CThostFtdcTraderApi_ReqUserLoginWithText(self, pReqUserLoginWithText, nRequestID)

    def ReqUserLoginWithOTP(self, pReqUserLoginWithOTP: "CThostFtdcReqUserLoginWithOTPField", nRequestID: "int") -> "int":
        return _thosttraderapi.CThostFtdcTraderApi_ReqUserLoginWithOTP(self, pReqUserLoginWithOTP, nRequestID)

    def ReqOrderInsert(self, pInputOrder: "CThostFtdcInputOrderField", nRequestID: "int") -> "int":
        return _thosttraderapi.CThostFtdcTraderApi_ReqOrderInsert(self, pInputOrder, nRequestID)

    def ReqParkedOrderInsert(self, pParkedOrder: "CThostFtdcParkedOrderField", nRequestID: "int") -> "int":
        return _thosttraderapi.CThostFtdcTraderApi_ReqParkedOrderInsert(self, pParkedOrder, nRequestID)

    def ReqParkedOrderAction(self, pParkedOrderAction: "CThostFtdcParkedOrderActionField", nRequestID: "int") -> "int":
        return _thosttraderapi.CThostFtdcTraderApi_ReqParkedOrderAction(self, pParkedOrderAction, nRequestID)

    def ReqOrderAction(self, pInputOrderAction: "CThostFtdcInputOrderActionField", nRequestID: "int") -> "int":
        return _thosttraderapi.CThostFtdcTraderApi_ReqOrderAction(self, pInputOrderAction, nRequestID)

    def ReqQueryMaxOrderVolume(self, pQueryMaxOrderVolume: "CThostFtdcQueryMaxOrderVolumeField", nRequestID: "int") -> "int":
        return _thosttraderapi.CThostFtdcTraderApi_ReqQueryMaxOrderVolume(self, pQueryMaxOrderVolume, nRequestID)

    def ReqSettlementInfoConfirm(self, pSettlementInfoConfirm: "CThostFtdcSettlementInfoConfirmField", nRequestID: "int") -> "int":
        return _thosttraderapi.CThostFtdcTraderApi_ReqSettlementInfoConfirm(self, pSettlementInfoConfirm, nRequestID)

    def ReqRemoveParkedOrder(self, pRemoveParkedOrder: "CThostFtdcRemoveParkedOrderField", nRequestID: "int") -> "int":
        return _thosttraderapi.CThostFtdcTraderApi_ReqRemoveParkedOrder(self, pRemoveParkedOrder, nRequestID)

    def ReqRemoveParkedOrderAction(self, pRemoveParkedOrderAction: "CThostFtdcRemoveParkedOrderActionField", nRequestID: "int") -> "int":
        return _thosttraderapi.CThostFtdcTraderApi_ReqRemoveParkedOrderAction(self, pRemoveParkedOrderAction, nRequestID)

    def ReqExecOrderInsert(self, pInputExecOrder: "CThostFtdcInputExecOrderField", nRequestID: "int") -> "int":
        return _thosttraderapi.CThostFtdcTraderApi_ReqExecOrderInsert(self, pInputExecOrder, nRequestID)

    def ReqExecOrderAction(self, pInputExecOrderAction: "CThostFtdcInputExecOrderActionField", nRequestID: "int") -> "int":
        return _thosttraderapi.CThostFtdcTraderApi_ReqExecOrderAction(self, pInputExecOrderAction, nRequestID)

    def ReqForQuoteInsert(self, pInputForQuote: "CThostFtdcInputForQuoteField", nRequestID: "int") -> "int":
        return _thosttraderapi.CThostFtdcTraderApi_ReqForQuoteInsert(self, pInputForQuote, nRequestID)

    def ReqQuoteInsert(self, pInputQuote: "CThostFtdcInputQuoteField", nRequestID: "int") -> "int":
        return _thosttraderapi.CThostFtdcTraderApi_ReqQuoteInsert(self, pInputQuote, nRequestID)

    def ReqQuoteAction(self, pInputQuoteAction: "CThostFtdcInputQuoteActionField", nRequestID: "int") -> "int":
        return _thosttraderapi.CThostFtdcTraderApi_ReqQuoteAction(self, pInputQuoteAction, nRequestID)

    def ReqBatchOrderAction(self, pInputBatchOrderAction: "CThostFtdcInputBatchOrderActionField", nRequestID: "int") -> "int":
        return _thosttraderapi.CThostFtdcTraderApi_ReqBatchOrderAction(self, pInputBatchOrderAction, nRequestID)

    def ReqOptionSelfCloseInsert(self, pInputOptionSelfClose: "CThostFtdcInputOptionSelfCloseField", nRequestID: "int") -> "int":
        return _thosttraderapi.CThostFtdcTraderApi_ReqOptionSelfCloseInsert(self, pInputOptionSelfClose, nRequestID)

    def ReqOptionSelfCloseAction(self, pInputOptionSelfCloseAction: "CThostFtdcInputOptionSelfCloseActionField", nRequestID: "int") -> "int":
        return _thosttraderapi.CThostFtdcTraderApi_ReqOptionSelfCloseAction(self, pInputOptionSelfCloseAction, nRequestID)

    def ReqCombActionInsert(self, pInputCombAction: "CThostFtdcInputCombActionField", nRequestID: "int") -> "int":
        return _thosttraderapi.CThostFtdcTraderApi_ReqCombActionInsert(self, pInputCombAction, nRequestID)

    def ReqQryOrder(self, pQryOrder: "CThostFtdcQryOrderField", nRequestID: "int") -> "int":
        return _thosttraderapi.CThostFtdcTraderApi_ReqQryOrder(self, pQryOrder, nRequestID)

    def ReqQryTrade(self, pQryTrade: "CThostFtdcQryTradeField", nRequestID: "int") -> "int":
        return _thosttraderapi.CThostFtdcTraderApi_ReqQryTrade(self, pQryTrade, nRequestID)

    def ReqQryInvestorPosition(self, pQryInvestorPosition: "CThostFtdcQryInvestorPositionField", nRequestID: "int") -> "int":
        return _thosttraderapi.CThostFtdcTraderApi_ReqQryInvestorPosition(self, pQryInvestorPosition, nRequestID)

    def ReqQryTradingAccount(self, pQryTradingAccount: "CThostFtdcQryTradingAccountField", nRequestID: "int") -> "int":
        return _thosttraderapi.CThostFtdcTraderApi_ReqQryTradingAccount(self, pQryTradingAccount, nRequestID)

    def ReqQryInvestor(self, pQryInvestor: "CThostFtdcQryInvestorField", nRequestID: "int") -> "int":
        return _thosttraderapi.CThostFtdcTraderApi_ReqQryInvestor(self, pQryInvestor, nRequestID)

    def ReqQryTradingCode(self, pQryTradingCode: "CThostFtdcQryTradingCodeField", nRequestID: "int") -> "int":
        return _thosttraderapi.CThostFtdcTraderApi_ReqQryTradingCode(self, pQryTradingCode, nRequestID)

    def ReqQryInstrumentMarginRate(self, pQryInstrumentMarginRate: "CThostFtdcQryInstrumentMarginRateField", nRequestID: "int") -> "int":
        return _thosttraderapi.CThostFtdcTraderApi_ReqQryInstrumentMarginRate(self, pQryInstrumentMarginRate, nRequestID)

    def ReqQryInstrumentCommissionRate(self, pQryInstrumentCommissionRate: "CThostFtdcQryInstrumentCommissionRateField", nRequestID: "int") -> "int":
        return _thosttraderapi.CThostFtdcTraderApi_ReqQryInstrumentCommissionRate(self, pQryInstrumentCommissionRate, nRequestID)

    def ReqQryExchange(self, pQryExchange: "CThostFtdcQryExchangeField", nRequestID: "int") -> "int":
        return _thosttraderapi.CThostFtdcTraderApi_ReqQryExchange(self, pQryExchange, nRequestID)

    def ReqQryProduct(self, pQryProduct: "CThostFtdcQryProductField", nRequestID: "int") -> "int":
        return _thosttraderapi.CThostFtdcTraderApi_ReqQryProduct(self, pQryProduct, nRequestID)

    def ReqQryInstrument(self, pQryInstrument: "CThostFtdcQryInstrumentField", nRequestID: "int") -> "int":
        return _thosttraderapi.CThostFtdcTraderApi_ReqQryInstrument(self, pQryInstrument, nRequestID)

    def ReqQryDepthMarketData(self, pQryDepthMarketData: "CThostFtdcQryDepthMarketDataField", nRequestID: "int") -> "int":
        return _thosttraderapi.CThostFtdcTraderApi_ReqQryDepthMarketData(self, pQryDepthMarketData, nRequestID)

    def ReqQrySettlementInfo(self, pQrySettlementInfo: "CThostFtdcQrySettlementInfoField", nRequestID: "int") -> "int":
        return _thosttraderapi.CThostFtdcTraderApi_ReqQrySettlementInfo(self, pQrySettlementInfo, nRequestID)

    def ReqQryTransferBank(self, pQryTransferBank: "CThostFtdcQryTransferBankField", nRequestID: "int") -> "int":
        return _thosttraderapi.CThostFtdcTraderApi_ReqQryTransferBank(self, pQryTransferBank, nRequestID)

    def ReqQryInvestorPositionDetail(self, pQryInvestorPositionDetail: "CThostFtdcQryInvestorPositionDetailField", nRequestID: "int") -> "int":
        return _thosttraderapi.CThostFtdcTraderApi_ReqQryInvestorPositionDetail(self, pQryInvestorPositionDetail, nRequestID)

    def ReqQryNotice(self, pQryNotice: "CThostFtdcQryNoticeField", nRequestID: "int") -> "int":
        return _thosttraderapi.CThostFtdcTraderApi_ReqQryNotice(self, pQryNotice, nRequestID)

    def ReqQrySettlementInfoConfirm(self, pQrySettlementInfoConfirm: "CThostFtdcQrySettlementInfoConfirmField", nRequestID: "int") -> "int":
        return _thosttraderapi.CThostFtdcTraderApi_ReqQrySettlementInfoConfirm(self, pQrySettlementInfoConfirm, nRequestID)

    def ReqQryInvestorPositionCombineDetail(self, pQryInvestorPositionCombineDetail: "CThostFtdcQryInvestorPositionCombineDetailField", nRequestID: "int") -> "int":
        return _thosttraderapi.CThostFtdcTraderApi_ReqQryInvestorPositionCombineDetail(self, pQryInvestorPositionCombineDetail, nRequestID)

    def ReqQryCFMMCTradingAccountKey(self, pQryCFMMCTradingAccountKey: "CThostFtdcQryCFMMCTradingAccountKeyField", nRequestID: "int") -> "int":
        return _thosttraderapi.CThostFtdcTraderApi_ReqQryCFMMCTradingAccountKey(self, pQryCFMMCTradingAccountKey, nRequestID)

    def ReqQryEWarrantOffset(self, pQryEWarrantOffset: "CThostFtdcQryEWarrantOffsetField", nRequestID: "int") -> "int":
        return _thosttraderapi.CThostFtdcTraderApi_ReqQryEWarrantOffset(self, pQryEWarrantOffset, nRequestID)

    def ReqQryInvestorProductGroupMargin(self, pQryInvestorProductGroupMargin: "CThostFtdcQryInvestorProductGroupMarginField", nRequestID: "int") -> "int":
        return _thosttraderapi.CThostFtdcTraderApi_ReqQryInvestorProductGroupMargin(self, pQryInvestorProductGroupMargin, nRequestID)

    def ReqQryExchangeMarginRate(self, pQryExchangeMarginRate: "CThostFtdcQryExchangeMarginRateField", nRequestID: "int") -> "int":
        return _thosttraderapi.CThostFtdcTraderApi_ReqQryExchangeMarginRate(self, pQryExchangeMarginRate, nRequestID)

    def ReqQryExchangeMarginRateAdjust(self, pQryExchangeMarginRateAdjust: "CThostFtdcQryExchangeMarginRateAdjustField", nRequestID: "int") -> "int":
        return _thosttraderapi.CThostFtdcTraderApi_ReqQryExchangeMarginRateAdjust(self, pQryExchangeMarginRateAdjust, nRequestID)

    def ReqQryExchangeRate(self, pQryExchangeRate: "CThostFtdcQryExchangeRateField", nRequestID: "int") -> "int":
        return _thosttraderapi.CThostFtdcTraderApi_ReqQryExchangeRate(self, pQryExchangeRate, nRequestID)

    def ReqQrySecAgentACIDMap(self, pQrySecAgentACIDMap: "CThostFtdcQrySecAgentACIDMapField", nRequestID: "int") -> "int":
        return _thosttraderapi.CThostFtdcTraderApi_ReqQrySecAgentACIDMap(self, pQrySecAgentACIDMap, nRequestID)

    def ReqQryProductExchRate(self, pQryProductExchRate: "CThostFtdcQryProductExchRateField", nRequestID: "int") -> "int":
        return _thosttraderapi.CThostFtdcTraderApi_ReqQryProductExchRate(self, pQryProductExchRate, nRequestID)

    def ReqQryProductGroup(self, pQryProductGroup: "CThostFtdcQryProductGroupField", nRequestID: "int") -> "int":
        return _thosttraderapi.CThostFtdcTraderApi_ReqQryProductGroup(self, pQryProductGroup, nRequestID)

    def ReqQryMMInstrumentCommissionRate(self, pQryMMInstrumentCommissionRate: "CThostFtdcQryMMInstrumentCommissionRateField", nRequestID: "int") -> "int":
        return _thosttraderapi.CThostFtdcTraderApi_ReqQryMMInstrumentCommissionRate(self, pQryMMInstrumentCommissionRate, nRequestID)

    def ReqQryMMOptionInstrCommRate(self, pQryMMOptionInstrCommRate: "CThostFtdcQryMMOptionInstrCommRateField", nRequestID: "int") -> "int":
        return _thosttraderapi.CThostFtdcTraderApi_ReqQryMMOptionInstrCommRate(self, pQryMMOptionInstrCommRate, nRequestID)

    def ReqQryInstrumentOrderCommRate(self, pQryInstrumentOrderCommRate: "CThostFtdcQryInstrumentOrderCommRateField", nRequestID: "int") -> "int":
        return _thosttraderapi.CThostFtdcTraderApi_ReqQryInstrumentOrderCommRate(self, pQryInstrumentOrderCommRate, nRequestID)

    def ReqQrySecAgentTradingAccount(self, pQryTradingAccount: "CThostFtdcQryTradingAccountField", nRequestID: "int") -> "int":
        return _thosttraderapi.CThostFtdcTraderApi_ReqQrySecAgentTradingAccount(self, pQryTradingAccount, nRequestID)

    def ReqQrySecAgentCheckMode(self, pQrySecAgentCheckMode: "CThostFtdcQrySecAgentCheckModeField", nRequestID: "int") -> "int":
        return _thosttraderapi.CThostFtdcTraderApi_ReqQrySecAgentCheckMode(self, pQrySecAgentCheckMode, nRequestID)

    def ReqQrySecAgentTradeInfo(self, pQrySecAgentTradeInfo: "CThostFtdcQrySecAgentTradeInfoField", nRequestID: "int") -> "int":
        return _thosttraderapi.CThostFtdcTraderApi_ReqQrySecAgentTradeInfo(self, pQrySecAgentTradeInfo, nRequestID)

    def ReqQryOptionInstrTradeCost(self, pQryOptionInstrTradeCost: "CThostFtdcQryOptionInstrTradeCostField", nRequestID: "int") -> "int":
        return _thosttraderapi.CThostFtdcTraderApi_ReqQryOptionInstrTradeCost(self, pQryOptionInstrTradeCost, nRequestID)

    def ReqQryOptionInstrCommRate(self, pQryOptionInstrCommRate: "CThostFtdcQryOptionInstrCommRateField", nRequestID: "int") -> "int":
        return _thosttraderapi.CThostFtdcTraderApi_ReqQryOptionInstrCommRate(self, pQryOptionInstrCommRate, nRequestID)

    def ReqQryExecOrder(self, pQryExecOrder: "CThostFtdcQryExecOrderField", nRequestID: "int") -> "int":
        return _thosttraderapi.CThostFtdcTraderApi_ReqQryExecOrder(self, pQryExecOrder, nRequestID)

    def ReqQryForQuote(self, pQryForQuote: "CThostFtdcQryForQuoteField", nRequestID: "int") -> "int":
        return _thosttraderapi.CThostFtdcTraderApi_ReqQryForQuote(self, pQryForQuote, nRequestID)

    def ReqQryQuote(self, pQryQuote: "CThostFtdcQryQuoteField", nRequestID: "int") -> "int":
        return _thosttraderapi.CThostFtdcTraderApi_ReqQryQuote(self, pQryQuote, nRequestID)

    def ReqQryOptionSelfClose(self, pQryOptionSelfClose: "CThostFtdcQryOptionSelfCloseField", nRequestID: "int") -> "int":
        return _thosttraderapi.CThostFtdcTraderApi_ReqQryOptionSelfClose(self, pQryOptionSelfClose, nRequestID)

    def ReqQryInvestUnit(self, pQryInvestUnit: "CThostFtdcQryInvestUnitField", nRequestID: "int") -> "int":
        return _thosttraderapi.CThostFtdcTraderApi_ReqQryInvestUnit(self, pQryInvestUnit, nRequestID)

    def ReqQryCombInstrumentGuard(self, pQryCombInstrumentGuard: "CThostFtdcQryCombInstrumentGuardField", nRequestID: "int") -> "int":
        return _thosttraderapi.CThostFtdcTraderApi_ReqQryCombInstrumentGuard(self, pQryCombInstrumentGuard, nRequestID)

    def ReqQryCombAction(self, pQryCombAction: "CThostFtdcQryCombActionField", nRequestID: "int") -> "int":
        return _thosttraderapi.CThostFtdcTraderApi_ReqQryCombAction(self, pQryCombAction, nRequestID)

    def ReqQryTransferSerial(self, pQryTransferSerial: "CThostFtdcQryTransferSerialField", nRequestID: "int") -> "int":
        return _thosttraderapi.CThostFtdcTraderApi_ReqQryTransferSerial(self, pQryTransferSerial, nRequestID)

    def ReqQryAccountregister(self, pQryAccountregister: "CThostFtdcQryAccountregisterField", nRequestID: "int") -> "int":
        return _thosttraderapi.CThostFtdcTraderApi_ReqQryAccountregister(self, pQryAccountregister, nRequestID)

    def ReqQryContractBank(self, pQryContractBank: "CThostFtdcQryContractBankField", nRequestID: "int") -> "int":
        return _thosttraderapi.CThostFtdcTraderApi_ReqQryContractBank(self, pQryContractBank, nRequestID)

    def ReqQryParkedOrder(self, pQryParkedOrder: "CThostFtdcQryParkedOrderField", nRequestID: "int") -> "int":
        return _thosttraderapi.CThostFtdcTraderApi_ReqQryParkedOrder(self, pQryParkedOrder, nRequestID)

    def ReqQryParkedOrderAction(self, pQryParkedOrderAction: "CThostFtdcQryParkedOrderActionField", nRequestID: "int") -> "int":
        return _thosttraderapi.CThostFtdcTraderApi_ReqQryParkedOrderAction(self, pQryParkedOrderAction, nRequestID)

    def ReqQryTradingNotice(self, pQryTradingNotice: "CThostFtdcQryTradingNoticeField", nRequestID: "int") -> "int":
        return _thosttraderapi.CThostFtdcTraderApi_ReqQryTradingNotice(self, pQryTradingNotice, nRequestID)

    def ReqQryBrokerTradingParams(self, pQryBrokerTradingParams: "CThostFtdcQryBrokerTradingParamsField", nRequestID: "int") -> "int":
        return _thosttraderapi.CThostFtdcTraderApi_ReqQryBrokerTradingParams(self, pQryBrokerTradingParams, nRequestID)

    def ReqQryBrokerTradingAlgos(self, pQryBrokerTradingAlgos: "CThostFtdcQryBrokerTradingAlgosField", nRequestID: "int") -> "int":
        return _thosttraderapi.CThostFtdcTraderApi_ReqQryBrokerTradingAlgos(self, pQryBrokerTradingAlgos, nRequestID)

    def ReqQueryCFMMCTradingAccountToken(self, pQueryCFMMCTradingAccountToken: "CThostFtdcQueryCFMMCTradingAccountTokenField", nRequestID: "int") -> "int":
        return _thosttraderapi.CThostFtdcTraderApi_ReqQueryCFMMCTradingAccountToken(self, pQueryCFMMCTradingAccountToken, nRequestID)

    def ReqFromBankToFutureByFuture(self, pReqTransfer: "CThostFtdcReqTransferField", nRequestID: "int") -> "int":
        return _thosttraderapi.CThostFtdcTraderApi_ReqFromBankToFutureByFuture(self, pReqTransfer, nRequestID)

    def ReqFromFutureToBankByFuture(self, pReqTransfer: "CThostFtdcReqTransferField", nRequestID: "int") -> "int":
        return _thosttraderapi.CThostFtdcTraderApi_ReqFromFutureToBankByFuture(self, pReqTransfer, nRequestID)

    def ReqQueryBankAccountMoneyByFuture(self, pReqQueryAccount: "CThostFtdcReqQueryAccountField", nRequestID: "int") -> "int":
        return _thosttraderapi.CThostFtdcTraderApi_ReqQueryBankAccountMoneyByFuture(self, pReqQueryAccount, nRequestID)

# Register CThostFtdcTraderApi in _thosttraderapi:
_thosttraderapi.CThostFtdcTraderApi_swigregister(CThostFtdcTraderApi)

def CThostFtdcTraderApi_CreateFtdcTraderApi(*args) -> "CThostFtdcTraderApi *":
    return _thosttraderapi.CThostFtdcTraderApi_CreateFtdcTraderApi(*args)

def CThostFtdcTraderApi_GetApiVersion() -> "char const *":
    return _thosttraderapi.CThostFtdcTraderApi_GetApiVersion()



